// PAQUETE
package API.Correos;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Tablas.Departamento;
import API.Tablas.Personal;
import java.rmi.RemoteException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;



/** CLASE CARTERO:  Clase que representa un cartero con todas las funcionalidades para enviar correos electronicos (E-Mails)
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Cartero
{
	// ATRIBUTOS -----------------------------------------------------------
        /* Propiedades:                     Propiedades del servidor de correo */
        private Properties                  Propiedades;
        /* Usuario:                         Nombre de usuario de la cuenta de correo */
        private String                      Usuario;
        /* Pass:                            Contraseña de la cuenta de correo */
        private String                      Pass;



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
	/** Constructor: Constructor de clase que crea el objeto Aplicacion con los valores pasados parametricamente
         *
         * @param   String      DirSMTP         Direccion del host de correo
         * @param   String      TLS             TLS, si esta disponible
         * @para    String      Puerto          Puerto del servidor de correo para el envio
         * @param   String      DirEmail        Direccion de E-Mail
         * @param   String      Autent          Si requiere o no de usuario y contraseña para conectarse
         * @param   String      Usuario         Nombre de usuario de la cuenta
         * @param   String      Pass            Contraseña de la cuenta
         * @return  JFrame      Aplicacion      Objeto de tipo Aplicacion
         */
        public Cartero (String DirSMTP, String TLS, String Puerto, String DirEmail, String Autent, String Usuario, String Pass) throws RemoteException
        {
            // Creamos el objeto Properties
            Propiedades = new Properties();

            // Le asignamos valor a sus atributos
            Propiedades.setProperty("mail.smtp.host", DirSMTP);
            Propiedades.setProperty("mail.smtp.starttls.enable", TLS);
            Propiedades.setProperty("mail.smtp.port", Puerto);
            Propiedades.setProperty("mail.smtp.user", DirEmail);
            Propiedades.setProperty("mail.smtp.auth", Autent);

            // Establecemos el usuario y contraseña
            this.Usuario = Usuario;
            this.Pass = Pass;
        }


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getPropiedades: Metodo que obtiene el valor del atributo Propiedades
	 *
	 * @param   void
	 * @return  Properties      Propiedades         Propiedades del servidor de correo
	 */
	public Properties getPropiedades()                      { return this.Propiedades; }
        /** setPropiedades: Metodo que establece el valor del atributo Propiedades
	 *
	 * @param   Properties      Propiedades         Propiedades del servidor de correo
	 * @return  void
	 */
	public void setPropiedades(Properties Propiedades) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Propiedades == null)                            throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Propiedades = Propiedades;
        }

        /** getUsuario: Metodo que obtiene el valor del atributo Usuario
	 *
	 * @param   void
	 * @return  String          Usuario             Nombre de usuario de la cuenta de correo
	 */
	public String getUsuario()                              { return this.Usuario; }
        /** setUsuario: Metodo que establece el valor del atributo Usuario
	 *
	 * @param   String          Usuario             Nombre de usuario de la cuenta de correo
	 * @return  void
	 */
	public void setUsuario(String Usuario) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Usuario == null)                                throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Usuario = Usuario;
        }

        /** getPass: Metodo que obtiene el valor del atributo Pass
	 *
	 * @param   void
	 * @return  String          Pass                Contraseña de la cuenta de correo
	 */
	public String getPass()                                 { return this.Pass; }
        /** setPass: Metodo que establece el valor del atributo Pass
	 *
	 * @param   String          Pass                Contraseña de la cuenta de correo
	 * @return  void
	 */
	public void setPass(String Pass) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Pass == null)                                   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Pass = Pass;
        }


        // ENVIO DE CORREOS
        /* enviarCorreo: Metodo que envia un correo con los datos y destinatarios proporcionados
         *
         * @param   ArrayList       Receptores      Receptores del E-Mail
         * @param   String          Tema            Tema del E-Mail
         * @param   String          Texto           Texto del E-Mail
         * @return  void
         */
        public void enviarCorreo(ArrayList Receptores, String Tema, String Texto) throws MessagingException
        {
            // Atributos
            String Emisor = "informaticanavarrasa@gmail.com";
            String Codificacion = "ISO-8859-1";
            String Tipo = "plain";
            
            // Enviamos el mensaje
            enviarCorreo(Emisor,Receptores,Tema,Texto,Codificacion,Tipo);
        }

        /* enviarCorreo: Metodo que envia un correo con los datos y destinatarios proporcionados
         *
         * @param   String          Emisor          Emisor del E-Mail
         * @param   ArrayList       Receptores      Receptores del E-Mail
         * @param   String          Tema            Tema del E-Mail
         * @param   String          Texto           Texto del E-Mail
         * @param   String          Codificacion    Codificacion del texto del mensaje
         * @param   String          Tipo            Tipo de mensaje
         * @return  void
         */
        public void enviarCorreo(String Emisor, ArrayList Receptores, String Tema, String Texto, String Codificacion,String Tipo) throws MessagingException
        {
            // Obtenemos la instancia de sesion
            Session session = Session.getDefaultInstance(Propiedades);

            // Creamos el mensaje
            MimeMessage message = new MimeMessage(session);
            // Emisor
            message.setFrom(new InternetAddress(Emisor));
            // Añadimos cada receptor
            Iterator it = Receptores.iterator();
            while (it.hasNext())    message.addRecipient(Message.RecipientType.TO, new InternetAddress((String) it.next()));
            // Rellenamos los campos del mensaje
            message.setSubject(Tema);
            message.setSentDate(new Date());
            message.setText(Texto,Codificacion,Tipo);

            // Enviamos el mensaje
            Transport t = session.getTransport("smtp");
            t.connect(Usuario,Pass);
            t.sendMessage(message,message.getAllRecipients());

            // Cerramos la conexion
            t.close();
        }

        /* enviarCorreosRenovacionPersonal: Metodo que envia los correos de peticion de renovacion de personal
         * que expiran en Dias dias
         *
         * @param   int             Dias            Dias restantes para que caduque el personal
         * @return  void
         */
        public void enviarCorreosRenovacionPersonal(int Dias) throws DataException, SQLException, MessagingException, RemoteException
        {
            // Obtenemos los datos de todos los correos a enviar
            Departamento dep = new Departamento();
            ResultSet rs = dep.obtenerDatosRenovacionPersonal(Dias);

            // Enviamos a cada RP el E-Mail
            ArrayList Receptores;
            String Tema = "Actualización de personal para el centro de 'Informática Navarra S.A.'";
            String Texto;
            while (rs.next())
            {
                // Rellenamos los campos del E-Mail
                Receptores = new ArrayList();
                Receptores.add(rs.getString(1));

                String F_Caducidad = new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss").format(rs.getObject(6));

                Texto = rs.getString(2) + "\n" + rs.getString(3) + "\n\n\n" +
                        "Estimado/a " + rs.getString(4) + " " + rs.getString(5) + ", nos ponemos en contacto con usted" +
                        " para informarle que el plazo para renovar el personal perteneciente a su departamento y que" +
                        " trabaja en nuestro centro expira el próximo día " + F_Caducidad + ". Le avisamos" +
                        " que a partir de tal fecha, si usted no procede con la actualización pertinente, nos veremos obligados a" +
                        " dar de baja en nuestro sistema a todo su personal.\n\n" +
                        " Reciba un cordial saludo,\n" +
                        " Informática Navarra S.A.";

                // Enviamos el correo
                enviarCorreo(Receptores,Tema,Texto);
            }

            // Cerramos la conexion
            dep.UBD.cerrarConexion();
        }

        /* enviarCorreosPrestamosPendientes: Metodo que envia a la persona un E-Mail con todo el material pendiente de devolucion
         *
         * @param   String          DNI             DNI de la persona a la que se le quiere recordar sus prestamos pendientes
         * @return  void
         */
        public void enviarCorreosPrestamosPendientes(String DNI) throws DataException, SQLException, MessagingException, RemoteException
        {
            // Obtenemos los datos del correo a enviar
            Personal pers = new Personal(DNI);
            ResultSet rs = pers.obtenerDatosPrestamosPendientes();

            // Enviamos el E-Mail con los datos de todo el material que la persona tiene pendiente su devolucion
            ArrayList Receptores = pers.getEmails();
            String Tema = "Prestamos pendientes de devolución con 'Informática Navarra S.A.'";
            String Texto = "Estimado/a " + pers.getNombre() + " " + pers.getApellido1() + ", nos ponemos en contacto con usted" +
                        " para hacerle llegar la lista de préstamos pendientes de devolución que posee con nosotros. Le rogamos" +
                        " nos haga llegar el siguiente material lo antes posible.\n\n" +
                        "Reciba un cordial saludo,\n" +
                        "Informática Navarra S.A.\n\n\n" +
                        "-------------------------------------------\n" +
                        "TIPO - CÓDIGO - FECHA DE PRÉSTAMO\n" +
                        "-------------------------------------------\n";

            while (rs.next())   Texto = Texto + rs.getString(1) + " - " + rs.getString(2) + " - " + new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss").format(rs.getObject(3)) + "\n";

            Texto = Texto + "-------------------------------------------\n";

            // Enviamos el correo
            enviarCorreo(Receptores,Tema,Texto);

            // Cerramos la conexion
            pers.UBD.cerrarConexion();
        }

        /* enviarCorreosPrestamosPendientes: Metodo que envia correos a todas las personas que poseen prestamos desde hace Dias dias
         *
         * @param   int             Dias            Dias transcurridos desde la realizacion del primer prestamo pendiente
         * @return  void
         */
        public void enviarCorreosPrestamosPendientes(int Dias) throws DataException, SQLException, MessagingException, RemoteException
        {
            // Obtenemos todas las personas accesoras al centro (que puedan disponer de prestamos pendientes de devolucion)
            Personal pers = new Personal();
            ArrayList personal = pers.obtenerPersonalCentro();

            // Recorremos el array de personas comprobando si alguna de ellas posee prestamos por devolver desde hace Dias dias
            Iterator it = personal.iterator();
            ResultSet rs;
            while(it.hasNext())
            {
                // Creamos la persona
                pers = new Personal((String) it.next());

                // Obtenemos los datos del correo a enviar
                rs = pers.obtenerDatosPrestamosPendientes(Dias);

                if (rs.next())
                {
                    // Enviamos el E-Mail con los datos de todo el material que la persona tiene pendiente su devolucion
                    ArrayList Receptores = pers.getEmails();
                    String Tema = "Prestamos pendientes de devolución con 'Informática Navarra S.A.'";
                    String Texto = "Estimado/a " + pers.getNombre() + " " + pers.getApellido1() + ", nos ponemos en contacto con usted" +
                                " para hacerle llegar la lista de préstamos pendientes de devolución que posee con nosotros. Le rogamos" +
                                " nos haga llegar el siguiente material lo antes posible.\n\n" +
                                "Reciba un cordial saludo,\n" +
                                "Informática Navarra S.A.\n\n\n" +
                                "-------------------------------------------\n" +
                                "TIPO - CÓDIGO - FECHA DE PRÉSTAMO\n" +
                                "-------------------------------------------\n";

                    Texto = Texto + rs.getString(1) + " - " + rs.getString(2) + " - " + new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss").format(rs.getObject(3)) + "\n";
                    while (rs.next())   Texto = Texto + rs.getString(1) + " - " + rs.getString(2) + " - " + new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss").format(rs.getObject(3)) + "\n";

                    Texto = Texto + "-------------------------------------------\n";

                    // Enviamos el correo
                    enviarCorreo(Receptores,Tema,Texto);
                }

                // Cerramos la conexion
                pers.UBD.cerrarConexion();
            }
        }
}