// PAQUETE
package API.Interfaces_RMI;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Tablas.*;
import java.rmi.*;
import java.sql.*;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;



/** INTERFAZ ACCESO_I:  Interfaz que establece los metodos a implementar en el uso de la tabla Acceso
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public interface Acceso_I extends Remote
{
	// METODOS -------------------------------------------------------------
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        // getPersona: Metodo que obtiene el valor del atributo Persona
	public Personal getPersona()                                            throws RemoteException;
        // setPersona: Metodo que establece el valor del atributo Persona
	public void setPersona(Personal Persona)                                        throws DataException, RemoteException;
        // poseeSalida: Metodo que denota si el acceso posee salida (es null)
	public boolean poseeSalida();
        // getMotivo: Metodo que obtiene el valor del atributo Motivo
	public String getMotivo()                                                       throws RemoteException;
        // setMotivo: Metodo que establece el valor del atributo Motivo
	public void setMotivo(String Motivo)                                            throws DataException, RemoteException;
        // getTarjeta: Metodo que obtiene el valor del atributo Tarjeta
	public Tarjeta getTarjeta()                                                     throws RemoteException;
        // setTarjeta: Metodo que establece el valor del atributo Tarjeta
	public void setTarjeta(Tarjeta Tarjeta)                                         throws DataException, RemoteException;
        // getGLLDs: Metodo que obtiene el valor del atributo Llaves_Llaveros
	public HashMap<String,GLL_Dev> getGLLDs()                                       throws RemoteException;
        // setGLLDs: Metodo que establece el valor del atributo Llaves_Llaveros
	public void setGLLDs(HashMap<String,GLL_Dev> GLLDs)                             throws RemoteException;
        // addGLLD: Metodo que añade una llave o llavero a los ya prestados
        public void addGLLD(GLL_Dev GLLD)                                               throws DataException, RemoteException;
        // updateGLLD: Metodo que modifica una llave o llavero prestado por otro
        public void updateGLLD(GLL_Dev LLl_old, GLL_Dev LLl_new)                        throws DataException, RemoteException;
        // removeGLLD: Metodo que elimina una llave o llavero prestado
        public void removeGLLD(GLL_Dev GLLD)                                            throws DataException, RemoteException;
        // getGLLD: Metodo que obtiene un prestamo de los existentes
        public GLL_Dev getGLLD(String Key_GLLD)                                         throws DataException, RemoteException;
        // devGLLD: Metodo que devuelve un prestamo de los existentes
        public void devGLLD(String Key_GLLD)                                            throws DataException, RemoteException;
        // getOtrosDatos: Metodo que obtiene el valor del atributo Otros_Datos
	public String getOtrosDatos()                                                   throws RemoteException;
        // setOtrosDatos: Metodo que establece el valor del atributo Otros_Datos
	public void setOtrosDatos(String Otros_Datos)                                   throws DataException, RemoteException;
        // getInsertador: Metodo que obtiene el valor del atributo Insertador
	public Personal getInsertador()                                         throws RemoteException;
        // setInsertador: Metodo que establece el valor del atributo Insertador
	public void setInsertador(Personal Insertador)                          throws DataException, RemoteException;

        // DML
        // insertGLLDBD: Metodo que inserta un prestamo en la BD
        public int insertGLLDBD(GLL_Dev GLLD)                                           throws DataException, SQLException, RemoteException;
        // updateGLLDBD: Metodo que actualiza un prestamo de la BD
        public int updateGLLDBD(GLL_Dev LLl_old, GLL_Dev LLl_new)                       throws DataException, SQLException, RemoteException;
        // devGLLDBD: Metodo que devuelve un prestamo (GLL_Dev) de la BD
        public int devGLLDBD(GLL_Dev GLLD)                                              throws DataException, SQLException, RemoteException;
        // devGLLDBD: Metodo que devuelve un prestamo (GLL_Dev) de la BD
        public int devGLLDBD(String Tipo, int Codigo)                                   throws DataException, SQLException, RemoteException;
        // removeGLLDBD: Metodo que borra un prestamo de la BD
        public int removeGLLDBD(GLL_Dev GLLD)                                           throws DataException, SQLException, RemoteException;
        // getGLLDBD: Metodo que obtiene un GLL_Dev de la BD
	public GLL_Dev getGLLDBD(String GLLD_Key)                                       throws DataException, SQLException, RemoteException;
        // getGLLDBD: Metodo que obtiene un GLL_Dev de la BD
	public GLL_Dev getGLLDBD(String Tipo, int Codigo)                               throws DataException, SQLException, RemoteException;
        // insertGLLDsBD: Metodo que inserta los GLL_Devs de este acceso en la BD
        public int insertGLLDsBD()                                                      throws DataException, SQLException, RemoteException;
        // insertGLLDsBD: Metodo que inserta los GLL_Devs de este acceso en la BD
        public int insertGLLDsBD(HashMap<String,GLL_Dev> GLLDs)                         throws DataException, SQLException, RemoteException;
        // updateGLLDsBD: Metodo que actualiza los GLL_Devs de un Acceso
        public int updateGLLDsBD(HashMap<String,GLL_Dev> GLLDs)                         throws DataException, SQLException, RemoteException;
        // removeGLLDsBD: Metodo que borra los GLL_Devs de una Acceso en la BD
        public int removeGLLDsBD()                                                      throws SQLException, RemoteException;
        // getGLLDsBD: Metodo que obtiene el conjunto de GLL_Dev del acceso de la BD
	public HashMap<String,GLL_Dev> getGLLDsBD()                                     throws DataException, SQLException, RemoteException;
        // insertAccesoBD: Metodo que inserta el Acceso en la BD
        public int insertAccesoBD()                                                     throws DataException, SQLException, RemoteException;
        // updateAccesoBD: Metodo que actualiza el Acceso en la BD
        public int updateAccesoBD(Acceso Acceso)                                        throws DataException, SQLException, RemoteException;
        // updateDatosAccesoBD: Metodo que actualiza el Acceso en la BD
        public int updateDatosAccesoBD(Acceso Acceso)                                   throws DataException, SQLException, RemoteException;
        // removeAccesoBD: Metodo que borra un Acceso de la BD
        public int removeAccesoBD()                                                     throws DataException, SQLException, RemoteException;
        // removeAccesoBD: Metodo que borra un Acceso de la BD
        public int removeAccesoBD(Acceso Acceso)                                        throws DataException, SQLException, RemoteException;

        // FUNCIONALIDADES
        // obtenerAccesosSinSalida: Metodo que obtiene todos los accesos del dia anterior que no disponen de salida
        public ArrayList<Acceso> obtenerAccesosSinSalida()                              throws DataException, SQLException, RemoteException;
        // darSalidaAutomatica: Metodo que dota al acceso de salida segun el tipo de acceso de la persona
        public void darSalidaAutomatica()                                               throws DataException, SQLException, RemoteException;
        // obtenerFAltaL: Metodo que obtiene la fecha de alta de una llave prestada en este acceso
        public String obtenerFAltaL(int Codigo)                                         throws SQLException, RemoteException;
        // obtenerFAltaLl: Metodo que obtiene la fecha de alta de un llavero prestado en este acceso
        public String obtenerFAltaLl(int Codigo)                                        throws SQLException, RemoteException;
        // obtenerAccesos: Metodo que devuelve los accesos realizados entre Fecha_Inic y Fecha_Fin, entre
        // las horas Hora_Inic y Hora_Fin, sobre la entrada o sobre la salida y pasando un filtro dado
        public ResultSet obtenerAccesos(boolean Entrada, String Fecha_Inic, String Fecha_Fin, String Hora_Inic, String Hora_Fin, String Filtro)      throws SQLException, RemoteException;
        // obtenerTarjetaAcceso: Metodo que obtiene la tarjeta empleada en un acceso
        public ResultSet obtenerTarjetaAcceso(String Entrada)                           throws SQLException, RemoteException;
        // obtenerLsLlsAcceso: Metodo que obtiene las llaves y llaveros que han sido prestadas en un acceso
        public ResultSet obtenerLsLlsAcceso()                                           throws SQLException, RemoteException;
        // darSalida: Metodo que da salida a un acceso
        public void darSalida()                                                         throws SQLException, RemoteException;
        // devolverLlavesLlaveros: Metodo que marca como devueltas las llaves y llaveros prestadas en un acceso
        public void devolverLlavesLlaveros()                                            throws SQLException, RemoteException;
        // devolverLlaves: Metodo que marca como devueltas las llaves prestadas en un acceso
        public void devolverLlaves()                                                    throws SQLException, RemoteException;
        // devolverLlaveros: Metodo que marca como devueltos los llaveros prestados en un acceso
        public void devolverLlaveros()                                                  throws SQLException, RemoteException;
        // eliminarAccesos: Metodo que borra todos los accesos mas antiguos de Dias dias
        public int eliminarAccesos(int Dias)                                            throws SQLException, RemoteException;
}