// PAQUETE
package API.Interfaces_RMI;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Tablas.Almacen;
import API.Tablas.GLL_Copias;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;



/** INTERFAZ ALMACEN_I:  Interfaz que establece los metodos a implementar en el uso de la tabla Almacen
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public interface Almacen_I extends Remote
{
	// METODOS -------------------------------------------------------------
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        // getCodAlm: Metodo que obtiene el valor del atributo Cod_Alm
	public String getCodAlm()                                                   throws RemoteException;
        // setCodAlm: Metodo que establece el valor del atributo Cod_Alm
	public void setCodAlm(String Cod_Alm)                                       throws DataException, RemoteException;
        // getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	public String getDescripcion()                                              throws RemoteException;
        // setDescripcion: Metodo que establece el valor del atributo Descripcion
	public void setDescripcion(String Descripcion)                              throws DataException, RemoteException;
        // getGLLCs: Metodo que obtiene el valor del atributo Llaves_Llaveros
	public HashMap<String,GLL_Copias> getGLLCs()                                throws RemoteException;
        // setGLLCs: Metodo que establece el valor del atributo Llaves_Llaveros
	public void setGLLCs(HashMap<String,GLL_Copias> GLLCs)                      throws DataException, RemoteException;
        // addGLLC: Metodo que añade una llave o llavero a los ya existentes en el almacen
        public void addGLLC(GLL_Copias GLLC)                                        throws DataException, RemoteException;
        // subsGLLC: Metodo que elimina parte de las copias de una llave o llavero del almacen
        public void subsGLLC(GLL_Copias GLLC)                                       throws DataException, RemoteException;
        // updateGLLC: Metodo que elimina una llave o llavero del almacen y lo reemplaza por otro
        public void updateGLLC(GLL_Copias GLLC_old, GLL_Copias GLLC_new)            throws DataException, RemoteException;
        // removeGLLC: Metodo que elimina todas las copias de una llave o llavero del almacen
        public void removeGLLC(GLL_Copias GLLC)                                     throws DataException, RemoteException;
        // getGLLC: Metodo que obtiene un GLL_Copias de los existentes
        public GLL_Copias getGLLC(String Key_GLLC)                                  throws DataException, RemoteException;

        // DML
        // insertGLLCBD: Metodo que inserta un GLL_Copias de este almacen en la BD
        public int insertGLLCBD(GLL_Copias GLLC)                                    throws DataException, SQLException, RemoteException;
        // updateGLLCBD: Metodo que actualiza un GLL_Copias de este almacen en la BD
        public int updateGLLCBD(GLL_Copias GLLC_old, GLL_Copias GLLC_new)           throws DataException, SQLException, RemoteException;
        // removeGLLCBD: Metodo que borra un GLL_Copias de un almacen de la BD
        public int removeGLLCBD(GLL_Copias GLLC)                                    throws DataException, SQLException, RemoteException;
        // getGLLCBD: Metodo que obtiene un GLL_Copias del almacen de la BD
	public GLL_Copias getGLLCBD(String GLLC_Key)                                throws DataException, SQLException, RemoteException;
        // insertGLLCsBD: Metodo que inserta los GLL_Copias de este almacen en la BD
        public int insertGLLCsBD()                                                  throws DataException, SQLException, RemoteException;
        // insertGLLCsBD: Metodo que inserta los GLL_Copias de este almacen en la BD
        public int insertGLLCsBD(HashMap<String,GLL_Copias> GLLCs)                  throws DataException, SQLException, RemoteException;
        // updateGLLCsBD: Metodo que actualiza los GLL_Copias de un Almacen
        public int updateGLLCsBD(HashMap<String,GLL_Copias> GLLCs)                  throws DataException, SQLException, RemoteException;
        // removeGLLCsBD: Metodo que borra los GLL_Copias de un Almacen en la BD
        public int removeGLLCsBD()                                                  throws SQLException, RemoteException;
        // getGLLCsBD: Metodo que obtiene el conjunto de GLL_Copias del almacen de la BD
	public HashMap<String,GLL_Copias> getGLLCsBD()                              throws DataException, SQLException, RemoteException;
        // insertAlmacenBD: Metodo que inserta el Almacen en la BD
        public int insertAlmacenBD()                                                throws DataException, SQLException, RemoteException;
        // updateAlmacenBD: Metodo que actualiza el Almacen en la BD
        public int updateAlmacenBD(Almacen Almacen)                                 throws DataException, SQLException, RemoteException;
        // removeAlmacenBD: Metodo que borra un Almacen de la BD
        public int removeAlmacenBD()                                                throws DataException, SQLException, RemoteException;
        // removeAlmacenBD: Metodo que borra un Almacen de la BD
        public int removeAlmacenBD(Almacen Almacen)                                 throws DataException, SQLException, RemoteException;

        // FUNCIONALIDADES
        // obtenerAlmacenes: Metodo que devuelve los almacenes de llaves existentes en el centro y que pasan un filtro dado
        public ResultSet obtenerAlmacenes(String Filtro)                            throws SQLException, RemoteException;
        // obtenerLlaves: Metodo que devuelve un listado de las llaves existentes en el almacen, que pasan un filtro dado, e informacion adicional
        public ResultSet obtenerLlaves(String Filtro)                               throws SQLException, RemoteException;
        // obtenerLlaveros: Metodo que devuelve un listado de los llaveros existentes en el almacen, que pasan un filtro dado, e informacion adicional
        public ResultSet obtenerLlaveros(String Filtro)                             throws SQLException, RemoteException;
        // obtenerLlavesTotales: Metodo que devuelve un listado de las llaves existentes en los almacenes, que pasan un filtro dado, e informacion adicional
        public ResultSet obtenerLlavesTotales(String Filtro)                        throws SQLException, RemoteException;
        // obtenerLlaverosTotales: Metodo que devuelve un listado de los llaveros existentes en los almacenes, que pasan un filtro dado, e informacion adicional
        public ResultSet obtenerLlaverosTotales(String Filtro)                      throws SQLException, RemoteException;
        // obtenerCodLls: Metodo que devuelve los codigos de todos los llaveros existentes y que no estan en el almacen
        public ResultSet obtenerCodLls()                                            throws SQLException, RemoteException;
        // disparadorAlmL: Metodo que ejecuta el disparador sobre AFTER DELETE ON Almacen_Llave
        public int disparadorAlmL()                                                 throws SQLException, RemoteException;
        // disparadorAlmLl: Metodo que ejecuta el disparador sobre AFTER DELETE ON Almacen_Llavero
        public int disparadorAlmLl()                                                throws SQLException, RemoteException;
}