// PAQUETE
package API.Interfaces_RMI;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Tablas.*;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;



/** INTERFAZ EMPRESA_I:  Interfaz que establece los metodos a implementar en el uso de la tabla Empresa
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public interface Empresa_I extends Remote
{
	// METODOS -------------------------------------------------------------
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        // getCodEmp: Metodo que obtiene el valor del atributo Cod_Emp
	public String getCodEmp()                                                                                   throws RemoteException;
        // setCodEmp: Metodo que establece el valor del atributo Cod_Emp
	public void setCodEmp(String Cod_Emp)                                                                       throws DataException, RemoteException;
        // getCIF: Metodo que obtiene el valor del atributo CIF
	public String getCIF()                                                                                      throws RemoteException;
        // setCIF: Metodo que establece el valor del atributo CIF
	public void setCIF(String CIF)                                                                              throws DataException, RemoteException;
        // getNombre: Metodo que obtiene el valor del atributo Nombre
	public String getNombre()                                                                                   throws RemoteException;
        // setNombre: Metodo que establece el valor del atributo Nombre
	public void setNombre(String Nombre)                                                                        throws DataException, RemoteException;
        // getDireccion: Metodo que obtiene el valor del atributo Direccion
	public String getDireccion()                                                                                throws RemoteException;
        // setDireccion: Metodo que establece el valor del atributo Direccion
	public void setDireccion(String Direccion)                                                                  throws DataException, RemoteException;
        // getDepartamentos: Metodo que obtiene el valor del atributo Departamentos
	public HashMap<String,Departamento> getDepartamentos()                                                      throws RemoteException;
        // setDepartamentos: Metodo que establece el valor del atributo Departamentos
	public void setDepartamentos(HashMap<String,Departamento> Departamentos)                                    throws DataException, RemoteException;
        // addDepartamento: Metodo que añade un departamento a los ya existentes
        public void addDepartamento(Departamento Departamento)                                                      throws DataException, RemoteException;
        // updateDepartamento: Metodo que modifica un departamento existente
        public void updateDepartamento(Departamento Dep_old, Departamento Dep_new)                                  throws DataException, RemoteException;
        // removeDepartamento: Metodo que elimina un departamento de los existentes
        public void removeDepartamento(Departamento Departamento)                                                   throws DataException, RemoteException;
        // getDepartamento: Metodo que obtiene un departamento de los existentes
        public Departamento getDepartamento(String Cod_Dep)                                                         throws DataException, RemoteException;

        // DML
        // insertLlaveBD: Metodo que inserta una Llave entre las prestables a un Departamento de la Empresa
        public int insertLlaveBD(Departamento Departamento, Llave Llave)                                            throws DataException, SQLException, RemoteException;
        // updateLlaveBD: Metodo que actualiza una llave prestable a un departamento de la empresa
        public int updateLlaveBD(Departamento Departamento, Llave Llave_old, Llave Llave_new)                       throws DataException, SQLException, RemoteException;
        // removeLlaveBD: Metodo que borra una llave prestable a un Departamento de la Empresa
        public int removeLlaveBD(Departamento Departamento, Llave Llave)                                            throws DataException, SQLException, RemoteException;
        // insertLlavesBD: Metodo que inserta las Llaves prestables a un Departamento de la Empresa
        public int insertLlavesBD(Departamento Departamento)                                                        throws DataException, SQLException, RemoteException;
        // updateLlavesBD: Metodo que actualiza las Llaves prestables a un Departamento de la Empresa
        public int updateLlavesBD(Departamento Departamento)                                                        throws DataException, SQLException, RemoteException;
        // removeLlavesBD: Metodo que borra las Llaves prestables a un Departamento de la Empresa
        public int removeLlavesBD(Departamento Departamento)                                                        throws DataException, SQLException, RemoteException;
        // getLlavesBD: Metodo que obtiene todas las llaves prestables a un Departamento dado de la Empresa
        public HashMap<String,Llave> getLlavesBD (String Cod_Dep)                                                   throws DataException, SQLException, RemoteException;
        // insertPersonaCargoBD: Metodo que inserta una relacion Persona-Cargo en la BD
        public int insertPersonaCargoBD(Departamento Departamento, Periodico_Contacto Persona, Cargo Cargo)         throws DataException, SQLException, RemoteException;
        // updatePersonaCargoBD: Metodo que modifica una relacion Persona-Cargo en la BD
        public int updatePersonaCargoBD(Departamento Departamento, Periodico_Contacto Persona, Cargo Cargo_old, Cargo Cargo_new)    throws DataException, SQLException, RemoteException;
        // removePersonaCargoBD: Metodo que borra una relacion Persona-Cargo en la BD
        public int removePersonaCargoBD(Departamento Departamento, Periodico_Contacto Persona, Cargo Cargo)         throws DataException, SQLException, RemoteException;
        // insertPersonaCargosBD: Metodo que inserta todos los cargos de una persona en la Empresa en la BD
        public int insertPersonaCargosBD(Departamento Departamento, Periodico_Contacto Persona)                     throws DataException, SQLException, RemoteException;
        // updatePersonaCargosBD: Metodo que actualiza los Cargos desempeñados por una persona en el Departamento
        public int updatePersonaCargosBD(Departamento Departamento, Periodico_Contacto Persona, HashMap<String,Cargo> Cargos)   throws DataException, SQLException, RemoteException, RemoteException;
        // removePersonaCargosBD: Metodo que borra todos los Cargos desempeñados por una persona en un departamento en la BD
        public int removePersonaCargosBD(Departamento Departamento, Periodico_Contacto Persona)                     throws DataException, SQLException, RemoteException;
        // insertPeriodicoContactoBD: Metodo que inserta una relacion Departamento-Persona en la BD
        public int insertPeriodicoContactoBD(Departamento Departamento, Periodico_Contacto Persona)                 throws DataException, SQLException, RemoteException;
        // removePeriodicoContactoBD: Metodo que borra una relacion Departamento-Persona en la BD
        public int removePeriodicoContactoBD(Departamento Departamento, Periodico_Contacto Persona)                 throws DataException, SQLException, RemoteException;
        // getPeridicoContactosBD: Metodo que obtiene todo el personal periodico y contactos de un departamento
        public HashMap<String,Personal> getPeriodicoContactosBD (String Cod_Dep)                                    throws DataException, SQLException, RemoteException;
        // insertVigilanteBD: Metodo que inserta una Vigilante en la BD
        public int insertVigilanteBD(Departamento Departamento, Vigilante Vigilante)                                throws DataException, SQLException, RemoteException;
        // removeVigilanteBD: Metodo que borra un vigilante de la BD
        public int removeVigilanteBD(Vigilante Vigilante)                                                           throws DataException, SQLException, RemoteException;
        // getVigilantesBD: Metodo que obtiene todos los vigilantes del departamento de la BD
        public HashMap<String,Personal> getVigilantesBD (String Cod_Dep)                                            throws DataException, SQLException, RemoteException;
        // insertPersonaBD: Metodo que inserta una Persona en la BD
        public int insertPersonaBD(Departamento Departamento, Personal Persona)                                     throws DataException, SQLException, RemoteException;
        // updatePersonaBD: Metodo que modifica una Persona en la BD
        public int updatePersonaBD(Departamento Departamento, Personal Persona_old, Personal Persona_new)           throws DataException, SQLException, RemoteException;
        // removePersonaBD: Metodo que borra una Persona de la BD
        public int removePersonaBD(Departamento Departamento, Personal Persona)                                     throws DataException, SQLException, RemoteException;
        // insertPersonalBD: Metodo que inserta todo el personal de un Departamento en la BD
        public int insertPersonalBD(Departamento Departamento)                                                      throws DataException, SQLException, RemoteException;
        // updatePersonalBD: Metodo que modifica el personal de un departamento en la BD
        public int updatePersonalBD(Departamento Departamento)                                                      throws DataException, SQLException, RemoteException;
        // removePersonalBD: Metodo que borra todo el personal de un Departamento de la BD
        public int removePersonalBD(Departamento Departamento)                                                      throws DataException, SQLException, RemoteException;
        // getPersonalBD: Metodo que obtiene todo el personal de un departamento de la BD
        public HashMap<String,Personal> getPersonalBD (String Cod_Dep)                                              throws DataException, SQLException, RemoteException;
        // insertDepartamentoBD: Metodo que inserta el Departamento en la BD
        public int insertDepartamentoBD(Departamento Departamento)                                                  throws DataException, SQLException, RemoteException;
        // updateDepartamentoBD: Metodo que actualiza un Departamento en la BD
        public int updateDepartamentoBD(Departamento Dep_old, Departamento Dep_new)                                 throws DataException, SQLException, RemoteException;
        // removeDepartamentoBD: Metodo que borra un Departamento de la BD
        public int removeDepartamentoBD(Departamento Departamento)                                                  throws DataException, SQLException, RemoteException;
        // getDepartamentoBD: Metodo que obtiene un departamento de la BD
        public Departamento getDepartamentoBD(String Cod_Dep)                                                       throws DataException, SQLException, RemoteException;
        // insertDepartamentosBD: Metodo que inserta todos los departamentos de esta Empresa en la BD
        public int insertDepartamentosBD()                                                                          throws DataException, SQLException, RemoteException;
        // updateDepartamentosBD: Metodo que modifica los departamentos de esta Empresa en la BD
        public int updateDepartamentosBD(HashMap<String,Departamento> Departamentos)                                throws DataException, SQLException, RemoteException;
        // removeDepartamentosBD: Metodo que borra todos los Departamento de esta Empresa de la BD
        public int removeDepartamentosBD()                                                                          throws SQLException, RemoteException;
        // getDepartamentosBD: Metodo que obtiene los departamentos de la BD
        public HashMap<String,Departamento> getDepartamentosBD()                                                    throws DataException, SQLException, RemoteException;
        // insertEmpresaBD: Metodo que inserta la Empresa en la BD
        public int insertEmpresaBD()                                                                                throws DataException, SQLException, RemoteException;
        // updateDatosBD: Metodo que actualiza los datos de una empresa en la BD
        public int updateDatosBD(Empresa Empresa)                                                                   throws DataException, SQLException, RemoteException;
        // updateEmpresaBD: Metodo que actualiza la Empresa en la BD
        public int updateEmpresaBD(Empresa Empresa)                                                                 throws DataException, SQLException, RemoteException;
        // removeEmpresaBD: Metodo que borra una Empresa de la BD
        public int removeEmpresaBD()                                                                                throws DataException, SQLException, RemoteException;
        // removeEmpresaBD: Metodo que borra una Empresa de la BD
        public int removeEmpresaBD(Empresa Empresa)                                                                 throws DataException, SQLException, RemoteException;

        // FUNCIONALIDADES
        // datosEmpresas: Metodo que devuelve los datos de las empresas que cumplen con los criterios establecidos
        public ResultSet datosEmpresas(String CIF, String Nom_Emp, String Nom_Dep, String DNI, String DNI_Usu, String Tipo_Usu)      throws SQLException, RemoteException;
        // datosDepartamentos: Metodo que devuelve los datos de los departamentos que cumplen con los criterios establecidos
        public ResultSet datosDepartamentos(String Nom_Emp, String Nom_Dep, String Actividad, String DNI, String DNI_Usu, String Tipo_Usu)           throws SQLException, RemoteException;
        // datosEmpleados: Metodo que devuelve los datos generales de los empleados que cumplen los criterios establecidos
        public ResultSet datosEmpleados(String Nom_Emp, String Nom_Dep, String Filtro, String DNI_Usu, String Tipo_Usu)                              throws SQLException, RemoteException;
        // datosContactos: Metodo que devuelve los datos de los contactos que cumplen con los criterios establecidos
        public ResultSet datosContactos(String Nom_Emp, String Nom_Dep, String Filtro, String DNI_Usu, String Tipo_Usu)                              throws SQLException, RemoteException;
        // obtenerCodEmp: Metodo que devuelve el codigo de empresa dado su nombre
        public String obtenerCodEmp()                                                                               throws SQLException, RemoteException;
        // obtenerCodDep: Metodo que devuelve el codigo de departamento dado su nombre
        public String obtenerCodDep(String Nom_Dep)                                                                 throws SQLException, RemoteException;
        // obtenerNomEmps: Metodo que devuelve los nombres de empresas existentes
        public ResultSet obtenerNomEmps()                                                                           throws SQLException, RemoteException;
        // obtenerNomDeps: Metodo que devuelve los nombres de departamentos existentes en la empresa
        public ResultSet obtenerNomDeps()                                                                           throws SQLException, RemoteException;
        // restablecerDatos: Metodo que asigna al objeto que lo llama sus datos
        public void restablecerDatos(String nombre)                                                                 throws SQLException, RemoteException;
        // obtenerLlaves: Metodo que obtiene todas las llaves prestables a un departamento
        public ResultSet obtenerLlaves(String Cod_Dep)                                                              throws SQLException, RemoteException;
        // obtenerLsPorPrestarDep: Metodo que obtiene todas las llaves que no son prestables a un departamento y pasan el filtro proporcionado
        public ResultSet obtenerLsPorPrestarDep(String Cod_Dep, String Filtro)                                      throws SQLException, RemoteException;
}