// PAQUETE
package API.Interfaces_RMI;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Tablas.Llave;
import API.Tablas.Llavero;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.HashMap;



/** INTERFAZ LLAVERO_I:  Interfaz que establece los metodos a implementar en el uso de la tabla Llavero
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public interface Llavero_I extends Remote
{
	// METODOS -------------------------------------------------------------
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        // getLlaves: Metodo que obtiene el valor del atributo Llaves
	public HashMap<String,Llave> getLlaves()                                    throws RemoteException;
        // setLlaves: Metodo que establece el valor del atributo Llaves
	public void setLlaves(HashMap<String,Llave> Llaves)                         throws DataException, RemoteException;
        // addLlave: Metodo que añade una llave a las ya existentes en el llavero
        public void addLlave(Llave Llave)                                           throws DataException, RemoteException;
        // updateLlave: Metodo que actualiza una llave del llavero
        public void updateLlave(Llave Llave_old, Llave Llave_new)                   throws DataException, RemoteException;
        // removeLlave: Metodo que elimina una llave de las existentes en el llavero
        public void removeLlave(Llave Llave)                                        throws DataException, RemoteException;
        // getLlave: Metodo que obtiene una Llave de las existentes
        public Llave getLlave(String Key_Llave)                                     throws DataException, RemoteException;

        // DML
        // insertLlaveBD: Metodo que inserta una Llave en el llavero en la BD
        public int insertLlaveBD(Llave Llave)                                       throws DataException, SQLException, RemoteException;
        // updateLlaveBD: Metodo que actualiza una llave del Llavero en la BD
        public int updateLlaveBD(Llave Llave_old, Llave Llave_new)                  throws DataException, SQLException, RemoteException;
        // removeLlaveBD: Metodo que borra una llave de un llavero de la BD
        public int removeLlaveBD(Llave Llave)                                       throws DataException, SQLException, RemoteException;
        // updateLlavesBD: Metodo que modifica las llaves de este llavero en la BD
        public int updateLlavesBD(HashMap<String,Llave> Llaves)                     throws DataException, SQLException, RemoteException;
        // getLlavesBD: Metodo que obtiene las llaves de un llavero de la BD
        public HashMap<String,Llave> getLlavesBD(int Cod_Llavero, String F_Alta_Ll) throws DataException, SQLException, RemoteException;
        // insertLlaveroBD: Metodo que inserta el Llavero en la BD
        public int insertLlaveroBD()                                                throws DataException, SQLException, RemoteException;
        // updateDatosBD: Metodo que actualiza los datos de un llavero en la BD
        public int updateDatosBD(Llavero Llavero)                                   throws DataException, SQLException, RemoteException;
        // updateLlaveroBD: Metodo que actualiza el Llavero en la BD
        public int updateLlaveroBD(Llavero Llavero)                                 throws DataException, SQLException, RemoteException;
        // darBajaLlaveroBD: Metodo que da de baja el Llavero en la BD
        public int darBajaLlaveroBD()                                               throws SQLException, RemoteException;
        // removeLlaveroBD: Metodo que borra un Llavero de la BD
        public int removeLlaveroBD()                                                throws DataException, SQLException, RemoteException;
        // removeLlaveroBD: Metodo que borra un Llavero de la BD
        public int removeLlaveroBD(Llavero Llavero)                                 throws DataException, SQLException, RemoteException;

        // FUNCIONALIDADES
        // obtenerLlaveros: Metodo que devuelve los llaveros prestables a una persona y que pasan un filtro dado
        public ResultSet obtenerLlaveros(String DNI, String Entrada, String Filtro) throws SQLException, RemoteException;
        // obtenerPrestamos: Metodo que devuelve los datos de todas los llaveros que permanecen en prestamo
        public ResultSet obtenerPrestamos()                                         throws SQLException, RemoteException;
        // obtenerLlaves: Metodo que devuelve los datos de todas las llaves que conforman el llavero y pasan un filtro dado
        public ResultSet obtenerLlaves(String Filtro)                               throws SQLException, RemoteException;
        // codigoPorDefecto: Metodo que devuelve el codigo de llavero por defecto
        public int codigoPorDefecto()                                               throws SQLException, RemoteException;
        // eliminarLlaveros: Metodo que borra todos los llaveros mas antiguos de Dias dias
        public int eliminarLlaveros(int Dias)                                       throws SQLException, RemoteException;
        // disparadorInsert: Metodo que ejecuta el disparador sobre BEFORE INSERT ON Llavero
        public int disparadorInsert()                                               throws SQLException, RemoteException;
        // disparadorRemove: Metodo que ejecuta el disparador sobre AFTER DELETE ON Llavero
        public int disparadorRemove()                                               throws SQLException, RemoteException;
}