// PAQUETE
package API.Interfaces_RMI;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Tablas.Personal;
import API.Tablas.Nota;
import API.Tablas.Destinatario;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;



/** INTERFAZ NOTA_I:  Interfaz que establece los metodos a implementar en el uso de la tabla Nota
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public interface Nota_I extends Remote
{
	// METODOS -------------------------------------------------------------
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        // getRedactor: Metodo que obtiene el valor del atributo Redactor
	public Personal getRedactor()                                                   throws RemoteException;
        // setRedactor: Metodo que establece el valor del atributo Redactor
	public void setRedactor(Personal Redactor)                                      throws DataException, RemoteException;
        // getTema: Metodo que obtiene el valor del atributo Tema
	public String getTema()                                                         throws RemoteException;
        // setTema: Metodo que establece el valor del atributo Tema
	public void setTema(String Tema)                                                throws DataException, RemoteException;
        // getTexto: Metodo que obtiene el valor del atributo Texto
	public String getTexto()                                                        throws RemoteException;
        // setTexto: Metodo que establece el valor del atributo Texto
	public void setTexto(String Texto)                                              throws DataException, RemoteException;
        // getDescartada: Metodo que obtiene el valor del atributo Descartada
	public boolean getDescartada()                                                  throws RemoteException;
        // setDescartada: Metodo que establece el valor del atributo Descartada
	public void setDescartada(boolean Descartada)                                   throws RemoteException;
        // getDestinatarios: Metodo que obtiene el valor del atributo Destinatarios
	public HashMap<String,Destinatario> getDestinatarios()                          throws RemoteException;
        // setDestinatarios: Metodo que establece el valor del atributo Destinatarios
	public void setDestinatarios(HashMap<String,Destinatario> Destinatarios)        throws DataException, RemoteException;
        // addDestinatario: Metodo que añade un destinatario a los ya existentes
        public void addDestinatario(Destinatario Destinatario)                          throws DataException, RemoteException;
        // updateDestinatario: Metodo que modifica un destinatario existente
        public void updateDestinatario(Destinatario Dest_old,Destinatario Dest_new)     throws DataException, RemoteException;
        // removeDestinatario: Metodo que elimina un destinatario de los existentes
        public void removeDestinatario(Destinatario Destinatario)                       throws DataException, RemoteException;
        // getDestinatario: Metodo que obtiene un destinatario de los existentes
        public Destinatario getDestinatario(String DNI)                                 throws DataException, RemoteException;

        // DML
        // insertDestinatarioBD: Metodo que inserta un Destinatario en la BD
        public int insertDestinatarioBD(Destinatario Destinatario)                      throws DataException, SQLException, RemoteException;
        // updateDestinatarioBD: Metodo que actualiza un Destinatario de la BD
        public int updateDestinatarioBD(Destinatario Dest_old, Destinatario Dest_new)   throws DataException, SQLException, RemoteException;
        // removeDestinatarioBD: Metodo que borra un Destinatario de la BD
        public int removeDestinatarioBD(Destinatario Destinatario)                      throws DataException, SQLException, RemoteException;
        // getDestinatarioBD: Metodo que crea un Destinatario (receptor de esta nota) con los datos existentes en la BD
        public Destinatario getDestinatarioBD(String DNI)                               throws DataException, SQLException, RemoteException;
        // insertDestinatariosBD: Metodo que inserta los destinatarios de esta nota en la BD
        public int insertDestinatariosBD()                                              throws DataException, SQLException, RemoteException;
        // insertDestinatariosBD: Metodo que inserta los destinatarios suministrados en la BD
        public int insertDestinatariosBD(HashMap<String,Destinatario> Destinatarios)    throws DataException, SQLException, RemoteException;
        // updateDestinatariosBD: Metodo que actualiza los destinatarios de una nota
        public int updateDestinatariosBD(HashMap<String,Destinatario> Destinatarios)    throws DataException, SQLException;
        // removeDestinatariosBD: Metodo que borra los destinatarios de una Nota en la BD
        public int removeDestinatariosBD()                                              throws SQLException, RemoteException;
        // getDestinatariosBD: Metodo que obtiene todos los destinatarios de una Nota en la BD
	public HashMap<String,Destinatario> getDestinatariosBD()                        throws DataException, SQLException, RemoteException;


        // insertNotaBD: Metodo que inserta la Nota en la BD
        public int insertNotaBD()                                                       throws DataException, SQLException, RemoteException;
        // updateNotaBD: Metodo que actualiza la Nota en la BD
        public int updateNotaBD(Nota Nota)                                              throws DataException, SQLException, RemoteException;
        // removeNotaBD: Metodo que borra una Nota de la BD
        public int removeNotaBD()                                                       throws DataException, SQLException, RemoteException;
        // removeNotaBD: Metodo que borra una Nota de la BD
        public int removeNotaBD(Nota Nota)                                              throws DataException, SQLException, RemoteException;


        // FUNCIONALIDADES
        // obtenerDestinatarios: Metodo que devuelve todos los usuarios del sistema que pueden usar el sistema de mensajeria
        public ResultSet obtenerDestinatarios(String Filtro)                            throws SQLException, RemoteException;
        // obtenerBandejaEntrada: Metodo que devuelve las notas enviadas a un usuario
        public ResultSet obtenerBandejaEntrada(String DNI)                              throws SQLException, RemoteException;
        // obtenerBandejaSalida: Metodo que devuelve las notas enviadas por un usuario
        public ResultSet obtenerBandejaSalida(String DNI)                               throws SQLException, RemoteException;
        // leida: Metodo que establece la nota como leida por el usuario
        public void leida(String DNI)                                                   throws DataException, SQLException;
        // descartada: Metodo que establece la nota como descartada por el destinatario
        public void descartada(String DNI)                                              throws DataException, SQLException, RemoteException;
        // descartada: Metodo que establece la nota como descartada por el redactor
        public void descartada()                                                        throws DataException, SQLException, RemoteException;
        // disparadorAutor: Metodo que ejecuta el disparador sobre AFTER UPDATE ON Nota
        public int disparadorAutor()                                                    throws SQLException, RemoteException;
        // disparadorDestinatario: Metodo que ejecuta el disparador sobre AFTER UPDATE ON Destinatario
        public int disparadorDestinatario()                                             throws SQLException, RemoteException;
}