// PAQUETE
package API.Interfaces_RMI;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Tablas.Nivel;
import API.Tablas.Llave;
import API.Tablas.Zona;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;



/** INTERFAZ ZONA_I:  Interfaz que establece los metodos a implementar en el uso de la tabla Zona
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public interface Zona_I extends Remote
{
	// METODOS -------------------------------------------------------------
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        // getCodZona: Metodo que obtiene el valor del atributo Cod_Zona
	public String getCodZona()                                              throws RemoteException;
        // setCodZona: Metodo que establece el valor del atributo Cod_Zona
	public void setCodZona(String Cod_Zona)                                 throws DataException, RemoteException;
        // getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	public String getDescripcion()                                          throws RemoteException;
        // setDescripcion: Metodo que establece el valor del atributo Descripcion
	public void setDescripcion(String Descripcion)                          throws DataException, RemoteException;
        // getNivel: Metodo que obtiene el valor del atributo Nivel
	public Nivel getNivel()                                                 throws RemoteException;
        // setNivel: Metodo que establece el valor del atributo Nivel
	public void setNivel(Nivel Nivel)                                       throws RemoteException;
        // getLlaves: Metodo que obtiene el valor del atributo Llaves
	public HashMap<String,Llave> getLlaves()                                throws RemoteException;
        // setLlaves: Metodo que establece el valor del atributo Zonas
	public void setLlaves(HashMap<String,Llave> Llaves)                     throws DataException, RemoteException;
        // addLlave: Metodo que añade una llave a las ya existentes
        public void addLlave(Llave Llave)                                       throws DataException, RemoteException;
        // updateLlave: Metodo que actualiza una llave existente
        public void updateLlave(Llave Llave_old, Llave Llave_new)               throws DataException, RemoteException;
        // removeLlave: Metodo que elimina una llave de las existentes
        public void removeLlave(Llave Llave)                                    throws DataException, RemoteException;
        // getLlave: Metodo que obtiene una Llave de las existentes
        public Llave getLlave(String Key_Llave)                                 throws DataException, RemoteException;

        // DML
        // insertLlaveBD: Metodo que inserta una Llave en la zona en la BD
        public int insertLlaveBD(Llave Llave)                                   throws DataException, SQLException, RemoteException;
        // updateLlaveBD: Metodo que actualiza una llave de la zona en la BD
        public int updateLlaveBD(Llave Llave_old, Llave Llave_new)              throws DataException, SQLException, RemoteException;
        // darBajaLlaveBD: Metodo que da de baja la Llave en la BD
        public int darBajaLlaveBD(Llave llave)                                  throws SQLException, RemoteException;
        // removeLlaveBD: Metodo que borra una llave de la zona en la BD
        public int removeLlaveBD(Llave Llave)                                   throws DataException, SQLException, RemoteException;
        // insertLlavesBD: Metodo que inserta las Llaves de la Zona en la BD
        public int insertLlavesBD()                                             throws DataException, SQLException, RemoteException;
        // updateLlavesBD: Metodo que actualiza las Llaves de la Zona en la BD
        public int updateLlavesBD()                                             throws DataException, SQLException, RemoteException;
        // removeLlavesBD: Metodo que borra las Llaves de la Zona en la BD
        public int removeLlavesBD()                                             throws DataException, SQLException, RemoteException;
        // getLlavesBD: Metodo que obtiene todas las llaves de la Zona
        public HashMap<String,Llave> getLlavesBD ()                             throws DataException, SQLException, RemoteException;
        // insertZonaBD: Metodo que inserta la Zona en la BD
        public int insertZonaBD()                                               throws DataException, SQLException, RemoteException;
        // updateZonaBD: Metodo que actualiza la Zona en la BD
        public int updateZonaBD(Zona Zona)                                      throws DataException, SQLException, RemoteException;
        // removeZonaBD: Metodo que borra la de la BD
        public int removeZonaBD()                                               throws DataException, SQLException, RemoteException;

        // FUNCIONALIDADES
        // obtenerZonas: Metodo que devuelve las zonas existentes en el centro
        public ResultSet obtenerZonas()                                         throws SQLException, RemoteException;
        // obtenerLlaves: Metodo que devuelve las llaves pertenecientes a la zona
        public ResultSet obtenerLlaves()                                        throws SQLException, RemoteException;
        // disparador: Metodo que ejecuta el disparador sobre BEFORE INSERT ON Llave
        public int disparador(Llave Llave)                                      throws SQLException, RemoteException;
        // disparador: Metodo que ejecuta el disparador sobre AFTER DELETE ON Llave
        public int disparador()                                                 throws SQLException, RemoteException;
}