// PAQUETE
package API.PDFs;


// INPORTACIONES
import API.Interfaces_RMI.GeneradorInformes_I;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.Font;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Image;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfPageEventHelper;
import com.itextpdf.text.pdf.PdfWriter;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.FileOutputStream;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.plaf.PanelUI;
import javax.swing.table.DefaultTableModel;



/** CLASE GENERADORINFORMES:  Clase que se encarga de generar los informes solicitados
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class GeneradorInformes extends PdfPageEventHelper implements GeneradorInformes_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /* Ruta:                        Ruta en la que depositar los informes */
        private String                  Ruta;



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
	/** Constructor: Constructor de clase que crea el objeto GeneradorInformes con los valores pasados parametricamente
         *
         * @param   String              Ruta        Ruta en la que depositar los informes
         * @return  GeneradorInformes               Objeto de tipo GeneradorInformes
         */
        public GeneradorInformes (String Ruta) throws RemoteException
        {
            // Asignamos valor a los atributos
            this.Ruta = Ruta;
        }


        // GENERACION DE INFORMES
        /** generarInforme: Metodo que genera un informe dados un titulo, texto y tabla cuyo contenido mostrar
	 *
	 * @param   String          Titulo              Titulo del informe
	 * @param   String          Texto               Texto del informe
         * @param   JTable          Tabla               Tabla cuya informacion debe aparecer en el informe
         * @return  void
	 */
        public void generarInforme(String Titulo, String Texto, JTable Tabla) throws Exception
        {
            // Se crea el documento
            Document documento = new Document();
            // Se crea el OutputStream para el fichero donde queremos crear el pdf
            FileOutputStream ficheroPdf = new FileOutputStream(Ruta);
            // Se asocia el documento al OutputStream y se indica que el espaciado entre
            // lineas sera de 20. Esta llamada debe hacerse antes de abrir el documento
            PdfWriter.getInstance(documento,ficheroPdf).setInitialLeading(20);
            // Se abre el documento.
            documento.open();

            // Se añade el contenido al documento
            // -> Logo
            Image logo = Image.getInstance("C:/Users/Héctor/Desktop/accessManager/Código/Java/API/accessManager/src/GUI/Imgs/in_logo.gif");
            //logo.scaleToFit(100, 100);
            logo.setAlignment(Chunk.ALIGN_LEFT);
            documento.add(logo);
            // -> Espacio
            documento.add(new Paragraph("\n\n\n"));
            // -> Titulo
            documento.add(new Paragraph(Titulo,FontFactory.getFont("Garamond",22,Font.BOLD,BaseColor.BLACK)));             // color
            // -> Espacio
            documento.add(new Paragraph("\n\n"));
            // -> Texto
            documento.add(new Paragraph(Texto));
            // -> Espacio
            documento.add(new Paragraph("\n\n"));
            // -> Tabla
            PdfPTable tabla = JTable2PdfPTable(Tabla);
            documento.add(tabla);

            // Cerramos el documento
            documento.close();
        }

        /** generarInformeAvanzado: Metodo que genera un informe dados un titulo, explicacion, tres subtitulos,
         * tres tablas y el subconjunto de ellas que debe aprecer en el informe
	 *
	 * @param   String          Titulo              Titulo del informe
	 * @param   String          Explicacion         Explicacion del informe
         * @param   String          Subtitulo1          Primer subtitulo
         * @param   String          Subtitulo2          Segundo subtitulo
         * @param   String          Subtitulo3          Tercero subtitulo
         * @param   JTable          Tabla1              Primera tabla
         * @param   JTable          Tabla2              Segunda tabla
         * @param   JTable          Tabla3              Tercera tabla
         * @param   boolean         Todas               Denota si deben aparecer todas las tablas en el informe
         * @param   boolean         T1                  Denota si debe aparecer la primera tabla en el informe
         * @param   boolean         T2                  Denota si debe aparecer la segunda tabla en el informe
         * @param   boolean         T3                  Denota si debe aparecer la tercera tabla en el informe
         * @return  void
	 */
        public void generarInformeAvanzado(String Titulo, String Explicacion, String Subtitulo1, String Subtitulo2, String Subtitulo3, JTable Tabla1, JTable Tabla2, JTable Tabla3, boolean Todas, boolean T1, boolean T2, boolean T3) throws Exception
        {
            // Se crea el documento
            Document documento = new Document();
            // Se crea el OutputStream para el fichero donde queremos crear el pdf
            FileOutputStream ficheroPdf = new FileOutputStream(Ruta);
            // Se asocia el documento al OutputStream y se indica que el espaciado entre
            // lineas sera de 20. Esta llamada debe hacerse antes de abrir el documento
            PdfWriter.getInstance(documento,ficheroPdf).setInitialLeading(20);
            // Se abre el documento.
            documento.open();

            // Se añade el contenido al documento
            // -> Logo
            Image logo = Image.getInstance("C:/Users/Héctor/Desktop/accessManager/Código/Java/API/accessManager/src/GUI/Imgs/in_logo.gif");
            //logo.scaleToFit(100, 100);
            logo.setAlignment(Chunk.ALIGN_LEFT);
            documento.add(logo);
            // -> Espacio
            documento.add(new Paragraph("\n\n\n"));
            // -> Titulo
            documento.add(new Paragraph(Titulo,FontFactory.getFont("Garamond",22,Font.BOLD,BaseColor.BLACK)));             // color
            // -> Espacio
            documento.add(new Paragraph("\n\n"));
            // -> Texto
            documento.add(new Paragraph(Explicacion));
            // -> Espacio
            documento.add(new Paragraph("\n\n"));

            // Si se exigen todas las tablas
            if (Todas)
            {
                // -> Tabla
                documento.add(new Paragraph(Subtitulo1 + "\n\n",FontFactory.getFont("Garamond",18,Font.BOLDITALIC,BaseColor.BLACK)));
                PdfPTable tabla1 = JTable2PdfPTable(Tabla1);
                documento.add(tabla1);
                // -> Espacio
                documento.add(new Paragraph("\n\n"));
                // -> Tabla
                documento.add(new Paragraph(Subtitulo2 + "\n\n",FontFactory.getFont("Garamond",18,Font.BOLDITALIC,BaseColor.BLACK)));
                PdfPTable tabla2 = JTable2PdfPTable(Tabla2);
                documento.add(tabla2);
                // -> Espacio
                documento.add(new Paragraph("\n\n"));
                // -> Tabla
                documento.add(new Paragraph(Subtitulo3 + "\n\n",FontFactory.getFont("Garamond",18,Font.BOLDITALIC,BaseColor.BLACK)));
                PdfPTable tabla3 = JTable2PdfPTable(Tabla3);
                documento.add(tabla3);
            }
            // Si no se exigen todas las tablas
            else
            {
                // Si se exige la primera tabla
                if (T1)
                {
                    // -> Tabla
                    documento.add(new Paragraph(Subtitulo1 + "\n\n",FontFactory.getFont("Garamond",18,Font.BOLDITALIC,BaseColor.BLACK)));
                    PdfPTable tabla1 = JTable2PdfPTable(Tabla1);
                    documento.add(tabla1);
                    // -> Espacio
                    documento.add(new Paragraph("\n\n"));
                }
                // Si se exige la segunda tabla
                if (T2)
                {
                    // -> Tabla
                    documento.add(new Paragraph(Subtitulo2 + "\n\n",FontFactory.getFont("Garamond",18,Font.BOLDITALIC,BaseColor.BLACK)));
                    PdfPTable tabla2 = JTable2PdfPTable(Tabla2);
                    documento.add(tabla2);
                    // -> Espacio
                    documento.add(new Paragraph("\n\n"));
                }
                // Si se exige la tercera tabla
                if (T3)
                {
                    // -> Tabla
                    documento.add(new Paragraph(Subtitulo3 + "\n\n",FontFactory.getFont("Garamond",18,Font.BOLDITALIC,BaseColor.BLACK)));
                    PdfPTable tabla3 = JTable2PdfPTable(Tabla3);
                    documento.add(tabla3);
                }
            }

            // Cerramos el documento
            documento.close();
        }

        /** JTable2PdfPTable: Metodo que convierte una JTabla a una PdfPTable
	 *
         * @param   JTable          Tabla               Tabla a convertir a PdfPTable
         * @return  PdfPTable       tabla               PdfPTable resultante de la conversion
	 */
        public PdfPTable JTable2PdfPTable (JTable Tabla)
        {
            // Creamos la tabla
            PdfPTable tabla = new PdfPTable(Tabla.getColumnCount());

            // Creamos la cabecera
            PdfPCell c;
            Paragraph p;
            for (int i=0; i<Tabla.getColumnCount(); i++)
            {
                p = new Paragraph(Tabla.getColumnName(i),FontFactory.getFont("Garamond",12,Font.BOLD,BaseColor.WHITE));
                p.setAlignment(p.ALIGN_CENTER);
                c = new PdfPCell(p);
                c.setBackgroundColor(new BaseColor(166,0,24));
                c.setHorizontalAlignment(c.ALIGN_CENTER);
                c.setVerticalAlignment(c.ALIGN_CENTER);
                tabla.addCell(c);
            }

            // Rellenamos la tabla
            DefaultTableModel modelo = (DefaultTableModel) Tabla.getModel();
            Vector vector = modelo.getDataVector();
            Iterator it = vector.iterator();
            Vector tupla;
            if (!it.hasNext())  for (int i=0; i<Tabla.getColumnCount(); i++)    tabla.addCell("Sin datos");

            while (it.hasNext())
            {
                tupla = (Vector) it.next();
                for (int i=0; i<tupla.size(); i++)
                {
                    Object o = tupla.get(i);
                    tabla.addCell(o == null ? "" : o.toString());
                }
            }

            // Devolvemos la tabla
            return tabla;
        }
        
        /** generarParte: Metodo que genera un informe con el parte de seguridad proporcionado
	 *
	 * @param   JPanel          Panel              Panel en el que va incluido el parte de seguridad a reportar
         * @return  void
	 */
        public void generarParte(JPanel Panel) throws Exception
        {

            // Se crea el documento
            Document documento = new Document();
            // Se crea el OutputStream para el fichero donde queremos crear el pdf
            FileOutputStream ficheroPdf = new FileOutputStream(Ruta);
            // Se asocia el documento al OutputStream y se indica que el espaciado entre
            // lineas sera de 20. Esta llamada debe hacerse antes de abrir el documento
            PdfWriter out = PdfWriter.getInstance(documento,ficheroPdf);
            out.setInitialLeading(20);
            // Se abre el documento.
            documento.open();

            // Se añade el contenido al documento
            // -> Logo
            Image logo = Image.getInstance("C:/Users/Héctor/Desktop/accessManager/Código/Java/API/accessManager/src/GUI/Imgs/in_logo.gif");
            //logo.scaleToFit(100, 100);
            logo.setAlignment(Chunk.ALIGN_LEFT);
            documento.add(logo);
            // -> Espacio
            documento.add(new Paragraph("\n\n\n\n"));
            PanelUI old = Panel.getUI();
            Panel.setUI((PanelUI)javax.swing.plaf.basic.BasicPanelUI.createUI(Panel));
            // -> Imagen del parte de seguridad
            BufferedImage img = new BufferedImage(Panel.getWidth(), Panel.getHeight(),BufferedImage.TYPE_INT_ARGB);
            Graphics2D img2D = img.createGraphics();
            Panel.print(img2D);
            float viewWidth = out.getPageSize().getWidth() - documento.leftMargin() - documento.rightMargin();
            double scaleX = viewWidth / Panel.getWidth();
            img2D.scale(scaleX, scaleX);
            Paragraph p = new Paragraph();
            Image itextImg = Image.getInstance(img, Color.BLUE, false);
            itextImg.scalePercent(75);
            itextImg.setAlignment(itextImg.ALIGN_CENTER);
            p.add(itextImg );
            documento.add(p);
            Panel.setUI(old);

            // Cerramos el documento
            documento.close();
        }
}