// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Acceso_I;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;



/** CLASE ACCESO:  Clase que representa un acceso al centro
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Acceso extends DosFechas implements Acceso_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Persona:                        Persona que realiza el acceso */
	private Personal                    Persona;
        /** Motivo:                         String con el motivo del acceso */
        private String                      Motivo;
        /** Tarjeta:                        Tarjeta empleada para realizar el acceso */
	private Tarjeta                     Tarjeta;
        /** GLLDs:                          HashMap con las llaves y llaveros prestados */
        private HashMap<String,GLL_Dev>     GLLDs;
        /** Otros_Datos:                    String con otros datos que puedan ser relevantes */
        private String                      Otros_Datos;
        /** Insertador:                     Persona que realiza la insercion del acceso en la BD */
        private Personal                    Insertador;


        /** UBD:                            Objeto de tipo UtilidadesBD */
        public static UtilidadesBD          UBD = new UtilidadesBD();
        /** UCAL:                           Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar    UCAL = new UtilidadesCalendar();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Acceso      Objeto de tipo Acceso
         */
        public Acceso () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Acceso con los valores pasados parametricamente
	 *
	 * @param   Persona     Persona que realiza el acceso
         * @param   Entrada     Fecha y hora en la que se realiza la entrada
         * @param   Motivo      String con el motivo del acceso
         * @param   Tarjeta     Tarjeta empleada para realizar el acceso
         * @param   Insertador  Persona que realiza la insercion del acceso en la BD
	 * @return  Acceso	Objeto de tipo Acceso
	 */
	public Acceso (Personal Persona, Calendar Entrada, String Motivo, Tarjeta Tarjeta, Personal Insertador) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(Entrada);

            // Comprobamos que los parametros pasados son correctos
            if ((Persona == null) || (Motivo == null) || (Tarjeta == null) || (Insertador == null) ||
                    (Motivo.length()>100) || (Tarjeta.getFechaI().after(Entrada)))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Persona = Persona;
            this.Motivo = Motivo;
            this.Tarjeta = Tarjeta;
            this.GLLDs = null;
            this.Otros_Datos = null;
            this.Insertador = Insertador;
	}
        /** Constructor: Constructor de clase que crea el objeto Acceso con los valores pasados parametricamente
	 *
	 * @param   Persona         Persona que realiza el acceso
         * @param   Entrada         Fecha y hora en la que se realiza la entrada
         * @param   Motivo          String con el motivo del acceso
         * @param   Tarjeta         Tarjeta empleada para realizar el acceso
         * @param   GLLDs           HashMap con las llaves y llaveros prestados
         * @param   Insertador      Persona que realiza la insercion del acceso en la BD
	 * @return  Acceso          Objeto de tipo Acceso
	 */
	public Acceso (Personal Persona, Calendar Entrada, String Motivo, Tarjeta Tarjeta, HashMap<String,GLL_Dev> GLLDs, Personal Insertador) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(Entrada);
            // Comprobamos que los parametros pasados son correctos
            if ((Persona == null) || (Motivo == null) || (Tarjeta == null) || (Insertador == null) ||
                    (Motivo.length()>100) || (Tarjeta.getFechaI().after(Entrada)))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Persona = Persona;
            this.Motivo = Motivo;
            this.Tarjeta = Tarjeta;
            this.GLLDs = GLLDs;
            this.Otros_Datos = null;
            this.Insertador = Insertador;
	}
	/** Constructor: Constructor de clase que crea el objeto Acceso con los valores pasados parametricamente
	 *
	 * @param   Persona     Persona que realiza el acceso
         * @param   Entrada     Fecha y hora en la que se realiza la entrada
         * @param   Salida      Fecha y hora en la que se realiza la salida
         * @param   Motivo      String con el motivo del acceso
         * @param   Tarjeta     Tarjeta empleada para realizar el acceso
         * @param   GLLDs       HashMap con las llaves y llaveros prestados
         * @param   Otros_Datos String con otros datos que puedan ser relevantes
         * @param   Insertador  Persona que realiza la insercion del acceso en la BD
	 * @return  Acceso	Objeto de tipo Acceso
	 */
	public Acceso (Personal Persona, Calendar Entrada, Calendar Salida, String Motivo, Tarjeta Tarjeta, HashMap<String,GLL_Dev> GLLDs, String Otros_Datos, Personal Insertador) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(Entrada,Salida);

            // Comprobamos que los parametros pasados son correctos
            if ((Persona == null) || (Motivo == null) || (Tarjeta == null) || (Insertador == null) ||
                    (Motivo.length()>100) || ((Otros_Datos != null) && (Otros_Datos.length()>150)) ||
                    (Tarjeta.getFechaI().after(Entrada)) ||
                    (Salida != null) && (Tarjeta.getFechaJ() != null) && (Salida.after(Tarjeta.getFechaJ())))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Persona = Persona;
            this.Motivo = Motivo;
            this.Tarjeta = Tarjeta;
            this.GLLDs = GLLDs;
            this.Otros_Datos = Otros_Datos;
            this.Insertador = Insertador;
	}
	/** Constructor: Constructor de clase que crea el objeto Acceso con los valores pasados parametricamente
	 *
         * @param   Entrada     Fecha y hora en la que se realiza la entrada
	 * @return  Acceso	Objeto de tipo Acceso
	 */
	public Acceso (String Entrada) throws DataException, SQLException, RemoteException
        {
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if (Entrada == null)                                        throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Acceso WHERE Entrada = \"" + Entrada + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Persona = new Personal(rs.getString(1));
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())  setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            this.Motivo = rs.getString(4);
            this.Tarjeta = new Tarjeta(rs.getInt(5),rs.getString(6).substring(0,19));
            this.GLLDs = getGLLDsBD();
            rs.getObject(7);
            if (rs.wasNull())   this.Otros_Datos = null;
            else                this.Otros_Datos = rs.getString(7);
            this.Insertador = new Personal(rs.getString(8));

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getPersona: Metodo que obtiene el valor del atributo Persona
	 *
	 * @param   void
	 * @return  Persona	Persona con la Persona que realiza el acceso
	 */
	public Personal getPersona()                  { return this.Persona; }
        /** setPersona: Metodo que establece el valor del atributo Persona
	 *
	 * @param   Persona     Persona     Persona que realiza el acceso
	 * @return  void
	 */
	public void setPersona(Personal Persona) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona == null)                                throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Persona = Persona;
        }

        /** poseeSalida: Metodo que denota si el acceso posee salida (es null)
	 *
	 * @param   void
	 * @return  boolean	Denota si el acceso posee salida (es null)
	 */
	public boolean poseeSalida()                            { return (getFechaJ() == null ? true : false); }

        /** getMotivo: Metodo que obtiene el valor del atributo Motivo
	 *
	 * @param   void
	 * @return  String	String con el motivo del acceso
	 */
	public String getMotivo()			{ return this.Motivo; }
        /** setMotivo: Metodo que establece el valor del atributo Motivo
	 *
	 * @param   Motivo      String     String con el motivo del acceso
	 * @return  void
	 */
	public void setMotivo(String Motivo) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Motivo == null) || (Motivo.length()>100))  throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Motivo = Motivo;
        }
        
        /** getTarjeta: Metodo que obtiene el valor del atributo Tarjeta
	 *
	 * @param   void
	 * @return  Tarjeta	Tarjeta empleada para realizar el acceso
	 */
	public Tarjeta getTarjeta()			{ return this.Tarjeta; }
        /** setTarjeta: Metodo que establece el valor del atributo Tarjeta
	 *
	 * @param   Tarjeta     Tarjeta     Tarjeta empleada para realizar el acceso
	 * @return  void
	 */
	public void setTarjeta(Tarjeta Tarjeta) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Tarjeta == null) || (Tarjeta.getFechaI().after(getFechaI())) ||
                    ((getFechaJ() != null) && (Tarjeta.getFechaJ()!=null) && (getFechaJ().after(Tarjeta.getFechaJ()))))
                    throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Tarjeta = Tarjeta;
        }

        /** getGLLDs: Metodo que obtiene el valor del atributo GLLDs
	 *
	 * @param   void
	 * @return  HashMap<String,GLL_Dev>	HashMap de llaves y llaveros prestados
	 */
	public HashMap<String,GLL_Dev> getGLLDs()                   { return this.GLLDs; }
        /** setGLLDs: Metodo que establece el valor del atributo GLLDs
	 *
	 * @param   GLLDs     HashMap<String,GLL_Dev>     HashMap de llaves y llaveros prestados
	 * @return  void
	 */
	public void setGLLDs(HashMap<String,GLL_Dev> GLLDs)         { this.GLLDs = GLLDs; }
        /** addGLLD: Metodo que añade una llave o llavero a los ya prestados
         *
         * @param   GLLD_Dev       GLLD     Llave o llavero prestado
         * @return  void
         */
        public void addGLLD(GLL_Dev GLLD) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLD == null)                       throw new DataException();

            // Obtenemos el tipo que queremos prestar: Llave o llavero
            String tipo = "L";
            if (GLLD.getLlaveLlavero() instanceof Llavero)         tipo = tipo + "l";

            // Si la llave o llavero no ha sido aun prestado, lo prestamos
            String key = tipo + " - " + GLLD.getLlaveLlavero().getCodigo() + " - " + calendarToString(GLLD.getLlaveLlavero().getFechaI());
            if (!getGLLDs().containsKey(key))               getGLLDs().put(key, GLLD);
        }
        /** updateGLLD: Metodo que modifica una llave o llavero prestado por otro
         *
         * @param   LLl_old             GLL_Dev  Llave o llavero prestado a reemplazar
         * @param   LLl_old             GLL_Dev  Llave o llavero por el que reemplazar
         * @return  void
         */
        public void updateGLLD(GLL_Dev LLl_old, GLL_Dev LLl_new) throws DataException
        {
            // Obtenemos los tipos de los argumentos pasados
            String tipo_LLl_old = "L";
            String tipo_LLl_new = "L";
            if (LLl_old.getLlaveLlavero() instanceof Llavero)   tipo_LLl_old = tipo_LLl_old + "l";
            if (LLl_new.getLlaveLlavero() instanceof Llavero)   tipo_LLl_new = tipo_LLl_new + "l";

            // Comprobamos que los parametros pasados son correctos
            if ((LLl_old == null) || (LLl_new == null) || (!tipo_LLl_old.equals(tipo_LLl_new)))     throw new DataException();

            // Realizamos la actualizacion de los atributos
            String key_old = tipo_LLl_old + " - " + LLl_old.getLlaveLlavero().getCodigo() + " - " + calendarToString(LLl_old.getLlaveLlavero().getFechaI());
            if (getGLLDs().containsKey(key_old))
            {
                removeGLLD(LLl_old);
                addGLLD(LLl_new);
            }
        }
        /** removeGLLD: Metodo que elimina una llave o llavero prestado
         *
         * @param   Llave_Llavero        GLL_Dev  Llave o llavero a eliminar
         * @return  void
         */
        public void removeGLLD(GLL_Dev GLLD) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLD == null)                       throw new DataException();

            // Obtenemos el tipo que queremos prestar: Llave o llavero
            String tipo = "L";
            if (GLLD.getLlaveLlavero() instanceof Llavero)         tipo = tipo + "l";

            getGLLDs().remove(tipo + " - " + GLLD.getLlaveLlavero().getCodigo() + " - " + calendarToString(GLLD.getLlaveLlavero().getFechaI()));
        }
        /** getGLLD: Metodo que obtiene un prestamo de los existentes
         *
         * @param   String        Key_GLLD      Key del prestamo
         * @return  GLL_Dev                     GLL_Dev solicitado
         */
        public GLL_Dev getGLLD(String Key_GLLD) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Key_GLLD == null) || (Key_GLLD.length()>28))       throw new DataException();
            // Devolvemos el resultado
            return this.getGLLDs().get(Key_GLLD);
        }
        /** devGLLD: Metodo que devuelve un prestamo de los existentes
         *
         * @param   String        Key_GLLD      Key del prestamo
         * @return  void
         */
        public void devGLLD(String Key_GLLD) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Key_GLLD == null) || (Key_GLLD.length()>29))       throw new DataException();
            // Devolvemos el prestamo
            getGLLD(Key_GLLD).setDev(true);
        }

        /** getOtrosDatos: Metodo que obtiene el valor del atributo Otros_Datos
	 *
	 * @param   void
	 * @return  String	String con otros datos que puedan ser relevantes
	 */
	public String getOtrosDatos()			{ return this.Otros_Datos; }
        /** setOtrosDatos: Metodo que establece el valor del atributo Otros_Datos
	 *
	 * @param   Otros_Datos      String     String con otros datos que puedan ser relevantes
	 * @return  void
	 */
	public void setOtrosDatos(String Otros_Datos) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Otros_Datos != null) && (Otros_Datos.length()>150))    throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Otros_Datos = Otros_Datos;
        }

        /** getInsertador: Metodo que obtiene el valor del atributo Insertador
	 *
	 * @param   void
	 * @return  Personal	Persona que realiza la insercion del acceso en la BD
	 */
	public Personal getInsertador()         { return this.Insertador; }
        /** setInsertador: Metodo que establece el valor del atributo Insertador
	 *
	 * @param   Insertador      Personal     Persona que realiza la insercion del acceso en la BD
	 * @return  void
	 */
	public void setInsertador(Personal Insertador) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Insertador == null)                     throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Insertador = Insertador;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Strings con las llaves y llaveros prestados
            String lls = "";

            // Añadimos las Strings con todas las llaves y llaveros prestados
            Iterator it = getGLLDs().entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKGLLD = (Map.Entry) it.next();
                GLL_Dev GLLD = (GLL_Dev) VKGLLD.getValue();
                lls = lls + GLLD.toString();
            }

            // Devolvemos el resultado
            return "ACCESO:\nPersona:\n" + getPersona().toString() + "\n" +
            super.toString() + "\n" +
            "Tarjeta:\n" + getTarjeta().toString() + "\n" +
            "Llaves y llaveros prestados:\n" + lls + "\n" +
            "Motivo: " + getMotivo() + "\n" +
            "Otros datos: " + (getOtrosDatos() == null ? "" : getOtrosDatos()) + "\n" +
            "Insertador: " + getInsertador().toString() + "\n";
	}


        // DML
        /** insertGLLDBD: Metodo que inserta un GLLD en la BD
         *
         * @param   GLL_Dev GLLD    Llave o llavero prestado
         * @return  int             Numero de filas (Prestamos) insertadas
         */
        public int insertGLLDBD(GLL_Dev GLLD) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLD == null)                               throw new DataException();

            // Creamos la String con la sentencia a realizar
            String tabla = "Llave_Prestada";

            if (GLLD.getLlaveLlavero() instanceof Llavero)       tabla = "Llavero_Prestado";

            String query = "INSERT INTO " + tabla + " VALUES (\""  +
                calendarToString (getFechaI()) + "\"," +
                GLLD.getLlaveLlavero().getCodigo() + ",\"" +
                calendarToString(GLLD.getLlaveLlavero().getFechaI()) + "\", false);";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateGLLDBD: Metodo que actualiza un GLLD de la BD
         *
         * @param   LLl_old     Llave (o llavero) a modificar
         * @param   LLl_new     Llave (o llavero) de reemplazo
         * @return  int         Resultado
         */
        public int updateGLLDBD(GLL_Dev LLl_old, GLL_Dev LLl_new) throws DataException, SQLException
        {
            // Obtenemos los tipos de los argumentos pasados
            String tipo_LLl_old = "L";
            String tipo_LLl_new = "L";
            if (LLl_old.getLlaveLlavero() instanceof Llavero)   tipo_LLl_old = tipo_LLl_old + "l";
            if (LLl_new.getLlaveLlavero() instanceof Llavero)   tipo_LLl_new = tipo_LLl_new + "l";

            // Comprobamos que los parametros pasados son correctos
            if ((LLl_old == null) || (LLl_new == null) || (!tipo_LLl_old.equals(tipo_LLl_new)))     throw new DataException();

            // Obtenemos la tabla sobre la que realizar la consulta
            String tabla = "Llave_Prestada";
            String cod_elem = "";
            String cod_f = "";
            String cod_d = "a";
            if (tipo_LLl_old.equals("Ll"))
            {
                tabla = "Llavero_Prestado";
                cod_elem = "ro";
                cod_f = "l";
                cod_d = "o";
            }

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE " + tabla + " SET " +
                    "Cod_Llave" + cod_elem + " = " + LLl_new.getLlaveLlavero().getCodigo() + "," +
                    "F_Alta_L" + cod_f + " = \"" + calendarToString(LLl_new.getLlaveLlavero().getFechaI()) + "\"," +
                    "Devuelt" + cod_d + " = " + (LLl_new.getDev() ? "true" : "false") +
                    " WHERE Entrada = \"" +
                    calendarToString(getFechaI()) + "\" AND Cod_Llave" + cod_elem + " = " +
                    LLl_old.getLlaveLlavero().getCodigo() + " AND F_Alta_L" + cod_f + " = \"" +
                    calendarToString(LLl_old.getLlaveLlavero().getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** devGLLDBD: Metodo que devuelve un prestamo (GLL_Dev) de la BD
         *
         * @param   GLLD        Llave (o llavero) a devolver
         * @return  int         Resultado
         */
        public int devGLLDBD(GLL_Dev GLLD) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLD == null)                               throw new DataException();

            // Creamos el GLL_Dev devuelto
            GLL_Dev GLL_new = new GLL_Dev(GLLD.getLlaveLlavero(),true);

            // Devolvemos la llave (llavero)
            return updateGLLDBD(GLLD, GLL_new);
        }
        /** devGLLDBD: Metodo que devuelve un prestamo (GLL_Dev) de la BD
         *
         * @param   Tipo        Si el prestamo se trata de una llave o un llavero
         * @param   Codigo      Codigo de la llave o llavero
         * @return  int         Resultado
         */
        public int devGLLDBD(String Tipo, int Codigo) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (((!Tipo.equals("Llave")) && (!Tipo.equals("Llavero"))) || (Codigo<0) || (Codigo>999))   throw new DataException();

            // Creamos el GLL_Dev sin devolver
            GLL_Dev GLLD_old = getGLLDBD(Tipo,Codigo);
            UBD.cerrarConexion();

            // Creamos el GLL_Dev devuelto
            GLL_Dev GLLD_new = new GLL_Dev(GLLD_old.getLlaveLlavero(),true);

            // Devolvemos la llave (llavero)
            return updateGLLDBD(GLLD_old, GLLD_new);
        }
        /** removeGLLDBD: Metodo que borra un prestamo de la BD
         *
         * @param   GLL_Dev    GLLD         Llave o llavero a eliminar
         * @return  int                     Numero de filas (Prestamos) eliminadas
         */
        public int removeGLLDBD(GLL_Dev GLLD) throws DataException, SQLException
        {

            // Comprobamos que los parametros pasados son correctos
            if (GLLD == null)                               throw new DataException();

            // Query, tabla y elementos sobre los que realizar la consulta
            String query = "";
            String tabla = "Llave_Prestada";
            String elem_cod = "Llave";
            String elem_f = "L";
            // Obtenemos la tabla sobre la que realizar la consulta
            if (GLLD.getLlaveLlavero() instanceof Llavero)
            {
                tabla = "Llavero_Prestado";
                elem_cod = "Llavero";
                elem_f = "Ll";
            }

            // Creamos la String con la sentencia a realizar
            query = "DELETE FROM " + tabla + "  WHERE Entrada = \"" + calendarToString(getFechaI()) +
                    "\" AND Cod_" + elem_cod + " = " + GLLD.getLlaveLlavero().getCodigo() +
                    " AND F_Alta_" + elem_f + " = \"" + calendarToString(GLLD.getLlaveLlavero().getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getGLLDBD: Metodo que obtiene un GLL_Dev de la BD
         *
	 * @param   GLLD_Key        Key de la GLL_Dev
	 * @return  GLL_Dev         Objeto de tipo GLL_Dev
	 */
	public GLL_Dev getGLLDBD(String GLLD_Key) throws DataException, SQLException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((GLLD_Key == null) || (GLLD_Key.length()>29))           throw new DataException();

            // Tabla y elementos sobre los que realizar la consulta
            String tabla = "Llave_Prestada";
            String elem_cod = "Llave";
            String elem_d = "a";
            // Obtenemos la tabla sobre la que realizar la consulta
            if (GLLD_Key.startsWith("Ll"))
            {
                tabla = "Llavero_Prestado";
                elem_cod = "Llavero";
                elem_d = "o";
            }

            // Tokens con los datos de la llave/llavero
            StringTokenizer st = new StringTokenizer(GLLD_Key);
            st.nextToken(" - ");
            int Codigo = Integer.parseInt(st.nextToken());

            // Obtenemos los datos para formar el GLL_Dev
            String query = "SELECT * FROM " + tabla + " WHERE Entrada = \"" + calendarToString(getFechaI()) + "\" AND" +
                    " Cod_" + elem_cod + " = " + Codigo;

            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            Gen_LLl Llave_Llavero;
            if (elem_cod.equals("Llave"))   Llave_Llavero = new Llave(rs.getInt(2),rs.getString(3).substring(0,19));
            else                            Llave_Llavero = new Llavero(rs.getInt(2),rs.getString(3).substring(0,19));
            boolean Dev = rs.getBoolean(4);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return new GLL_Dev(Llave_Llavero,Dev);
	}
        /** getGLLDBD: Metodo que obtiene un GLL_Dev de la BD
         *
	 * @param   Tipo            Si se trata del prestamo de una llave o de un llavero
         * @param   Codigo          Codigo de la llave o llavero
	 * @return  GLL_Dev         Objeto de tipo GLL_Dev
	 */
	public GLL_Dev getGLLDBD(String Tipo, int Codigo) throws DataException, SQLException
	{
            // Comprobamos que los parametros pasados son correctos
            if (((!Tipo.equals("Llave")) && (!Tipo.equals("Llavero"))) || (Codigo<0) || (Codigo>999))   throw new DataException();

            // Creamos la Key
            String key = "L";
            String f_alta = "";

            // Obtenemos la fecha de alta
            // En caso de tratarse de una llave
            if (Tipo.equals("Llave"))           f_alta = obtenerFAltaL(Codigo);
            // En caso de tratarse de un llavero
            else
            {
                // Añadimos la "l" al comienzo de la key
                key = key + "l";

                // Obtenemos la fecha de alta
                f_alta = obtenerFAltaLl(Codigo);
            }

            // Terminamos de construir la key
            key = key + " - " + Codigo + " - " + f_alta.substring(0,19);

            // Devolvemos el GLL_Dev solicitado
            return getGLLDs().get(key);
	}

        /** insertGLLDsBD: Metodo que inserta los GLL_Devs de este acceso en la BD
         *
         * @param   void
         * @return  int                                             Numero de filas insertadas
         */
        public int insertGLLDsBD() throws DataException, SQLException
        {
            // Insertamos los GLL_Devs de este acceso en la BD
            return insertGLLDsBD(getGLLDs());
        }
        /** insertGLLDsBD: Metodo que inserta los GLL_Devs de este acceso en la BD
         *
         * @param   HashMap<String,GLL_Dev>    GLL_Devs     GLL_Devs del acceso
         * @return  int                                     Numero de filas (GLL_Devs) insertadas
         */
        public int insertGLLDsBD(HashMap<String,GLL_Dev> GLLDs) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLDs == null)                              throw new DataException();

            // Insertamos cada GLL_Dev del HashMap
            Iterator it = GLLDs.entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos el destinatario
                Map.Entry VKGLLD = (Map.Entry) it.next();
                GLL_Dev GLLD = (GLL_Dev) VKGLLD.getValue();

                // Insertamos el GLL_Dev
                insertGLLDBD (GLLD);
            }

            return GLLDs.size();
        }
        /** updateGLLDsBD: Metodo que actualiza los GLL_Devs de un Acceso
         *
         * @param   HashMap<String,GLL_Dev>     GLLDs       GLL_Devs del acceso
         * @return  int                                     Numero de filas (GLL_Devs) actualizadas
         */
        public int updateGLLDsBD(HashMap<String,GLL_Dev> GLLDs) throws DataException, SQLException
        {
            // Filas actualizadas
            int res = 0;

            // Recorremos el HashMap nuevo, actualizando los GLL_Devs existentes e insertando los nuevos
            Iterator it_new = GLLDs.entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKGLLD = (Map.Entry) it_new.next();
                GLL_Dev glld_new = (GLL_Dev) VKGLLD.getValue();
                String glld_key = (String) VKGLLD.getKey();

                // Si existe el GLLD -> Lo actualizamos
                if (getGLLDs().containsKey(glld_key))   updateGLLDBD(getGLLDs().get(glld_key),glld_new);
                // Si no existe -> Lo insertamos
                else                                    insertGLLDBD(glld_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellos GLL_Devs que no consten en el nuevo
            Iterator it_old = getGLLDs().entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKGLLD = (Map.Entry) it_old.next();
                GLL_Dev glld_old = (GLL_Dev) VKGLLD.getValue();
                String glld_key = (String) VKGLLD.getKey();

                // Si el GLL_Dev viejo no existe como nuevo -> Lo eliminamos y actualizamos el contador de filas actualizadas
                if (! GLLDs.containsKey(glld_key))
                {
                    removeGLLDBD(glld_old);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removeGLLDsBD: Metodo que borra los GLL_Devs de un Acceso en la BD
         *
         * @param   void
         * @return  int                             Numero de filas (GLL_Devs) eliminadas
         */
        public int removeGLLDsBD() throws SQLException
        {
            // BORRAMOS LAS LLAVES PRESTADAS
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Llave_Prestada WHERE DNI = \"" + getPersona().getDNI() +
                    "\" AND Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // BORRAMOS LOS LLAVEROS PRESTADOS
            // Creamos la String con la sentencia a realizar
            query = "DELETE FROM Llavero_Prestado WHERE DNI = \"" + getPersona().getDNI() +
                    "\" AND Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query) + res;
        }
        /** getGLLDsBD: Metodo que obtiene el conjunto de GLL_Dev del acceso de la BD
         *
	 * @param   void
	 * @return  HashMap<String,GLL_Dev>      HashMap con los GLL_Dev del acceso
	 */
	public HashMap<String,GLL_Dev> getGLLDsBD() throws DataException, SQLException, RemoteException
	{
            // Creamos el HashMap de GLL_Dev
            HashMap<String,GLL_Dev> Llaves_Llaveros = new HashMap();

            // Obtenemos las llaves prestadas
            String query = "SELECT Cod_Llave,F_Alta_L FROM Llave_Prestada WHERE" +
                    " ENTRADA = \"" + calendarToString(getFechaI()) + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Añadimos todos los GLL_Dev correspondientes a llaves
            while (rs.next())
            {
                String key = "L - " + rs.getInt(1) + " - " + rs.getString(2).substring(0,19);
                Llaves_Llaveros.put(key, getGLLDBD(key));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Obtenemos los llaveros prestados
            query = "SELECT Cod_Llavero,F_Alta_Ll FROM Llavero_Prestado WHERE" +
                    " ENTRADA = \"" + calendarToString(getFechaI()) + "\";";
            rs = UBD.ejecutarQuery(query);

            // Añadimos todos los GLL_Copias correspondientes a llaveros
            while (rs.next())
            {
                String key = "Ll - " + rs.getInt(1) + " - " + rs.getString(2).substring(0,19);
                Llaves_Llaveros.put(key, getGLLDBD(key));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Llaves_Llaveros;
	}

        /** insertAccesoBD: Metodo que inserta el Acceso en la BD
         *
         * @param   void
         * @return  int     Numero de filas (Accesos) insertadas
         */
        public int insertAccesoBD() throws DataException, SQLException
        {
            // INSERTAMOS EL ACCESO
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Acceso VALUES (\""  +
                    getPersona().getDNI() + "\",\"" +
                    calendarToString(getFechaI()) + "\"," +
                    (getFechaJ() == null ? "null" : "\"" + calendarToString(getFechaJ()) + "\"") + ",\"" +
                    getMotivo() + "\"," +
                    getTarjeta().getCodTarj() + ",\"" +
                    calendarToString(getTarjeta().getFechaI()) + "\"," +
                    (getOtrosDatos() == null ? "null" : "\"" + getOtrosDatos() + "\"") + ",\"" +
                    getInsertador().getDNI() + "\");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // Si el acceso ha sido insertado correctamente -> INSERTAMOS LOS PRESTAMOS
            if (res == 1)   insertGLLDsBD();

            // Establecemos la tarjeta como prestada
            Tarjeta tarj = getTarjeta();
            tarj.setPersona(getPersona());
            getTarjeta().updateTarjetaBD(tarj);

            // Devolvemos el resultado
            return res;
        }

        /** updateAccesoBD: Metodo que actualiza el Acceso en la BD
         *
         * @param   Acceso          Acceso con el que modificar
         * @return  int             Resultado
         */
        public int updateAccesoBD(Acceso Acceso) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Acceso == null)                                     throw new DataException();

            // ACTUALIZAMOS EL ACCESO
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Acceso SET " +
                    "DNI = \"" + Acceso.getPersona().getDNI() + "\"," +
                    "Entrada = \"" + calendarToString(Acceso.getFechaI()) + "\"," +
                    "Salida = " + (Acceso.getFechaJ() == null ? "null" : "\"" + calendarToString(Acceso.getFechaJ()) + "\"") + "," +
                    "Motivo = \"" + Acceso.getMotivo() + "\"," +
                    "Cod_Tarj = " + Acceso.getTarjeta().getCodTarj() + "," +
                    "F_Alta_T = \"" + calendarToString(Acceso.getTarjeta().getFechaI()) + "\"," +
                    "Otros_Datos = " + (Acceso.getOtrosDatos() == null ? "null" : "\"" + Acceso.getOtrosDatos() + "\"") + "," +
                    "DNI_Ins = \"" + Acceso.getInsertador().getDNI() +
                    "\" WHERE Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la sentencia
            int res = UBD.ejecutarUpdate(query);

            // ACTUALIZAMOS LOS GLL_DEVS
            // Actualizamos la PK del Acceso para una correcta actualizacion de los GLL_Devs
            setPersona(Acceso.getPersona());
            setFechaI(Acceso.getFechaI());
            // Actualizamos los GLL_Devs y devolvemos el resultado
            return updateGLLDsBD(Acceso.getGLLDs()) + res;
        }
        /** updateDatosAccesoBD: Metodo que actualiza el Acceso en la BD
         *
         * @param   Acceso          Acceso con el que modificar
         * @return  int             Resultado
         */
        public int updateDatosAccesoBD(Acceso Acceso) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Acceso == null)                                     throw new DataException();

            // ACTUALIZAMOS EL ACCESO
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Acceso SET " +
                    "Motivo = \"" + Acceso.getMotivo() + "\"," +
                    "Otros_Datos = " + ((Acceso.getOtrosDatos() == null) ? "null" : ("\"" + Acceso.getOtrosDatos() + "\"")) +
                    " WHERE Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la sentencia y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** removeAccesoBD: Metodo que borra un Acceso de la BD
         *
         * @param   void
         * @return  int             Numero de filas (Accesos) eliminadas
         */
        public int removeAccesoBD() throws DataException, SQLException
        {
            // Borramos el acceso y devolvemos el resultado
            return removeAccesoBD(this);
        }
        /** removeAccesoBD: Metodo que borra un Acceso de la BD
         *
         * @param   Acceso          Acceso a borrar
         * @return  int             Numero de filas (Accesos) eliminadas
         */
        public int removeAccesoBD(Acceso Acceso) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Acceso == null)                                     throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Acceso WHERE Entrada = \"" + calendarToString(Acceso.getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* obtenerAccesosSinSalida: Metodo que obtiene todos los accesos del dia anterior que no disponen de salida
         *
         * @param   void
         * @return  ArrayList<Acceso>       ArrayList con los accesos que no disponen de salida
         */
        public ArrayList<Acceso> obtenerAccesosSinSalida() throws DataException, SQLException, RemoteException
        {
            // Resultado
            ArrayList<Acceso> al = new ArrayList();

            // Creamos la String con la sentencia a ejecutar
            String query = "SELECT Entrada FROM Acceso WHERE Salida IS NULL AND DATE(Entrada) = (SELECT SUBDATE(CURDATE(), INTERVAL 1 DAY));";

            // Ejecutamos la query
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            while (rs.next())       al.add(new Acceso(rs.getString(1).substring(0,19)));

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return al;
        }

        /* darSalidaAutomatica: Metodo que dota al acceso de salida segun el tipo de acceso de la persona
         *
         * @param   void
         * @return  void
         */
        public void darSalidaAutomatica() throws DataException, SQLException, RemoteException
        {
            // Creamos la String con la sentencia a ejecutar
            String query = "SELECT Hora_Fin" +
                            " FROM	(SELECT P.DNI, Hora_Fin" +
                                        " FROM Acceso JOIN Periodico P USING (DNI) JOIN Departamento USING (Cod_Emp,Cod_Dep) JOIN T_Acceso USING (Cod_Acc)" +
                                            " UNION" +
                                        " SELECT DNI, Hora_Fin" +
                                        " FROM Acceso JOIN Ocasional USING (DNI) JOIN T_Acceso USING (Cod_Acc)" +
                                            " UNION" +
                                        " SELECT DNI, '23:59:59'" +
                                        " FROM Acceso JOIN Residente USING (DNI)) Horas_Maximas" +
                            " WHERE DNI = \"" + getPersona().getDNI() + "\";";


            // Ejecutamos la query
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos la hora maxima de salida
            rs.next();
            String hora_fin = rs.getString(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Creamos la siguiente query a ejecutar
            query = "UPDATE Acceso SET Salida = CONCAT(SUBDATE(CURDATE(), INTERVAL 1 DAY),' ','" + hora_fin + "') WHERE Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos el update
            UBD.ejecutarUpdate(query);
        }

        /* obtenerFAltaL: Metodo que obtiene la fecha de alta de una llave prestada en este acceso
         *
         * @param   int     Codigo de la llave prestada
         * @return  String  Fecha de alta de la llave prestada
         */
        public String obtenerFAltaL(int Codigo) throws SQLException
        {
            // Resultado
            String F_Alta = "";

            // Creamos la String con la sentencia a ejecutar
            String query = "SELECT F_Alta_L FROM Llave_Prestada" +
                    " WHERE Entrada = \"" + calendarToString(getFechaI()) + "\" AND Cod_Llave = " + Codigo + ";";

            // Ejecutamos la query
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            if (rs.next())      F_Alta = rs.getString(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return F_Alta;
        }

        /* obtenerFAltaLl: Metodo que obtiene la fecha de alta de un llavero prestado en este acceso
         *
         * @param   int     Codigo de la llavero prestado
         * @return  String  Fecha de alta del llavero prestada
         */
        public String obtenerFAltaLl(int Codigo) throws SQLException
        {
            // Resultado
            String F_Alta = "";

            // Creamos la String con la sentencia a ejecutar
            String query = "SELECT F_Alta_Ll FROM Llavero_Prestado" +
                    " WHERE Entrada = \"" + calendarToString(getFechaI()) + "\" AND Cod_Llavero = " + Codigo + ";";

            // Ejecutamos la query
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            if (rs.next())      F_Alta = rs.getString(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return F_Alta;
        }

        /* obtenerAccesos: Metodo que devuelve los accesos realizados entre Fecha_Inic y Fecha_Fin, entre
         * las horas Hora_Inic y Hora_Fin, sobre la entrada o sobre la salida y pasando un filtro dado
         *
         * @param   boolean     Entrada     Denota si la busqueda es sobre la fecha-hora de entrada (si no lo sera sobre la salida)
         * @param   String      Fecha_Inic  Fecha de inicio
         * @param   String      Fecha_Fin   Fecha de fin
         * @param   String      Hora_Inic   Hora de inicio
         * @param   String      Hora_Fin    Hora de fin
         * @param   String      Filtro      Filtro que debe pasar el acceso
         * @return  ResultSet   rs          Accesos realizados al centro
         */
        public ResultSet obtenerAccesos(boolean Entrada, String Fecha_Inic, String Fecha_Fin, String Hora_Inic, String Hora_Fin, String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String where;
            if ((Hora_Inic == null) && (Hora_Fin == null))          where = "BETWEEN \"" + Fecha_Inic + " - 00:00:00\" AND \"" + Fecha_Fin + " - 23:59:59";
            else                                                    where = "BETWEEN \"" + Fecha_Inic + " - " + Hora_Inic + ":00\" AND \"" + Fecha_Fin + " - " + Hora_Fin + ":00";

            String query = "SELECT DNI,Entrada,Salida,Motivo" +
                    " FROM Acceso JOIN Personal USING (DNI)" +
                    " WHERE " + (Entrada ? "Entrada " : "Salida ") + where + "\" AND" +
                        " (DNI LIKE \"%" + Filtro + "%\" OR Apellido1 LIKE \"%" + Filtro + "%\" OR Apellido2 LIKE \"%" + Filtro + "%\"" +
                        " OR Nombre LIKE \"%" + Filtro + "%\" OR Motivo LIKE \"%" + Filtro + "%\" OR Cod_Tarj LIKE \"%" + Filtro + "%\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerDatosTarjetaAcceso: Metodo que obtiene la tarjeta empleada en un acceso
         *
         * @param   String      Entrada     Fecha y hora de entrada
         * @return  ResultSet   rs          Datos y tarjeta empleada en el acceso
         */
        public ResultSet obtenerTarjetaAcceso(String Entrada) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Tarj, Cod_Color" +
                    " FROM Acceso JOIN Tarjeta USING (Cod_Tarj,F_Alta_T)" +
                    " WHERE Entrada = \"" + Entrada + "\"";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLsLlsAcceso: Metodo que obtiene las llaves y llaveros que han sido prestadas en un acceso
         *
         * @param   void
         * @return  ResultSet   rs          Llaves y llaveros prestados en el acceso
         */
        public ResultSet obtenerLsLlsAcceso() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Llave Tipo, Cod_Llave Código, Devuelta" +
                    " FROM ((SELECT 'Llave', Entrada, Cod_Llave, Descripcion, Devuelta" +
                            " FROM Llave_Prestada JOIN Llave USING (Cod_Llave))" +
                                " UNION" +
                            " (SELECT 'Llavero', Entrada, Cod_Llavero, Descripcion, Devuelto" +
                            " FROM Llavero_Prestado JOIN Llavero USING (Cod_Llavero))) Prestamos" +
                    " WHERE Entrada = \"" + calendarToString(getFechaI()) + "\"";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }


        /* darSalida: Metodo que da salida a un acceso
         *
         * @return                          El acceso dispone de salida tras la ejecucion de este metodo
         */
        public void darSalida() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Acceso SET Salida = \"" + UCAL.calendarToString(Calendar.getInstance()) +
                    "\" WHERE Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta
            UBD.ejecutarUpdate(query);
        }

        /* devolverLlavesLlaveros: Metodo que marca como devueltas las llaves y llaveros prestadas en un acceso
         *
         * @return                          El acceso consta con todos sus prestamos de llaves y llaveros como devueltos
         */
        public void devolverLlavesLlaveros() throws SQLException
        {
            // Devolvemos las llaves
            devolverLlaves();
            // Devolvemos los llaveros
            devolverLlaveros();
        }

        /* devolverLlaves: Metodo que marca como devueltas las llaves prestadas en un acceso
         *
         * @return                          El acceso consta con todos sus prestamos de llaves como devueltos
         */
        public void devolverLlaves() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llave_Prestada SET Devuelta = true" +
                    " WHERE Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta
            UBD.ejecutarUpdate(query);
        }

        /* devolverLlaveros: Metodo que marca como devueltos los llaveros prestados en un acceso
         *
         * @return                          El acceso consta con todos sus prestamos de llaveros como devueltos
         */
        public void devolverLlaveros() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llavero_Prestado SET Devuelto = true" +
                    " WHERE Entrada = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta
            UBD.ejecutarUpdate(query);
        }

        /* eliminarAccesos: Metodo que borra todos los accesos mas antiguos de Dias dias
         *
         * @param   Dias    Numero de dias transcurridos desde la salida del acceso
         * @return  int     Numero de accesos eliminados
         */
        public int eliminarAccesos(int Dias) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Acceso WHERE ADDDATE(DATE(Salida),INTERVAL " + Dias + " DAY)<=CURDATE();";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
}