// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Interfaces_RMI.Acceso_Sistema_I;
import API.Excepciones.DataException;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;
import java.util.Calendar;



/** CLASE ACCESO_SISTEMA:  Clase que representa un  acceso al sistema
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Acceso_Sistema extends UnaFecha implements Acceso_Sistema_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cuenta:                 Cuenta que accede al sistema */
	private Cuenta              Cuenta;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Acceso_Sistema      Objeto de tipo Acceso_Sistema
         */
        public Acceso_Sistema () throws DataException, RemoteException {}
        /** Constructor: Constructor de clase que crea el objeto Cuenta con los valores pasados parametricamente
	 *
	 * @param   Cuenta              Cuenta que accede al sistema
         * @param   Fecha               Fecha y hora de entrada en el sistema
	 * @return  Acceso_Sistema	Objeto de tipo Acceso_Sistema
	 */
	public Acceso_Sistema(Cuenta Cuenta, Calendar Fecha) throws DataException, RemoteException
	{
            // Llamamos al constructor de la clase base
            super(Fecha);

            // Comprobamos que los parametros pasados son correctos
            if (Cuenta == null)         throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cuenta = Cuenta;
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCuenta: Metodo que obtiene el valor del atributo Cuenta
	 *
	 * @param   void
	 * @return  Cuenta     Cuenta que accede al sistema
	 */
	public Cuenta getCuenta()                      { return this.Cuenta; }
        /** setCuenta: Metodo que establece el valor del atributo Cuenta
	 *
	 * @param   Cuenta     Cuenta     Cuenta que accede al sistema
	 * @return  void
	 */
	public void setCuenta(Cuenta Cuenta) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Cuenta == null)                         throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cuenta = Cuenta;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "ACCESO_SISTEMA:\n" + getCuenta().toString() + "\n" +
            super.toString() + "\n";
	}


        // DML
        /** insertAccesoSistemaBD: Metodo que inserta el Acceso_Sistema en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertAccesoSistemaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Acceso_Sistema VALUES (\""  +
                    getCuenta().getUsuario() + "\",\"" +
                    calendarToString(getFechaI()) + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateAccesoSistemaBD: Metodo que actualiza el Acceso_Sistema en la BD
         *
         * @param   Acceso_Sistema  Acceso_Sistema con el que modificar
         * @return  int             Numero de filas actualizadas
         */
        public int updateAccesoSistemaBD(Acceso_Sistema Acceso_Sistema) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Acceso_Sistema == null)                         throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Acceso_Sistema SET " +
                    "Usuario = \"" + Acceso_Sistema.getCuenta().getUsuario() + "\"," +
                    "Fecha = \"" + calendarToString(Acceso_Sistema.getFechaI()) +
                    " WHERE Usuario = \"" + getCuenta().getUsuario() + "\" AND Fecha = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeAccesiSistemaBD: Metodo que borra un Acceso_Sistema de la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeAccesoSistemaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Acceso_Sistema WHERE Usuario = \"" + getCuenta().getUsuario() + "\" AND Fecha = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /** primerVigilante: Metodo que denota si solo se dispone de un vigilante accedido en el sistema en el dia de hoy
         *
         * @param   void
         * @return  boolean     Verdadero si solo ha accedido un vigilante al sistema en el dia de hoy, en caso contrario falso
         */
        public boolean primerVigilante() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT COUNT(*)" +
                    " FROM Acceso_Sistema JOIN Cuenta USING (Usuario)" +
                    " WHERE Tipo = \"VIGILANTE\"";

            // Ejecutamos la consulta y recogemos el resultado
            ResultSet rs = UBD.ejecutarQuery(query);
            rs.next();
            int numVigs = rs.getInt(1);
            UBD.cerrarConexion();

            // Devolvemos el resultado
            if (numVigs==1) return true;
            else            return false;
        }

        /** obtenerAccesosSistema: Metodo que devuelve los datos de los accesos realizados en el sistema bajo unas condiciones dadas
         *
         * @param   String      Fecha_Inic  Fecha de inicio
         * @param   String      Fecha_Fin   Fecha de fin
         * @param   String      Hora_Inic   Hora de inicio
         * @param   String      Hora_Fin    Hora de fin
         * @param   String      Filtro      Filtro que deben cumplir los accesos realizados en el sistema
         * @return  ResultSet   rs          Datos de los accesos realizados en el sistema
         */
        public ResultSet obtenerAccesosSistema(String Fecha_Inic, String Fecha_Fin, String Hora_Inic, String Hora_Fin, String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String where;
            if ((Hora_Inic == null) && (Hora_Fin == null))          where = "BETWEEN \"" + Fecha_Inic + " - 00:00:00\" AND \"" + Fecha_Fin + " - 23:59:59";
            else                                                    where = "BETWEEN \"" + Fecha_Inic + " - " + Hora_Inic + ":00\" AND \"" + Fecha_Fin + " - " + Hora_Fin + ":00";

            String query = "SELECT Usuario, Tipo, DNI, Fecha" +
                    " FROM Acceso_Sistema JOIN Cuenta USING (Usuario) JOIN T_Usuario USING (Tipo)" +
                    " WHERE Fecha " + where + "\" AND (DNI LIKE \"%" + Filtro + "%\" OR Usuario LIKE \"%" + Filtro + "%\" OR Descripcion LIKE \"%" + Filtro +
                    "%\") ORDER BY Fecha;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
}