// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Almacen_I;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;



/** CLASE ALMACEN:  Clase que representa un lugar donde almacenar llaves y llaveros
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Almacen implements Almacen_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cod_Alm:                            String con el codigo de almacen */
	private String                          Cod_Alm;
        /** Descripcion:                        String con la descripcion del almacen */
        private String                          Descripcion;
        /** GLLCs:                              HashMap de llaves y llaveros almacenados, junto con sus numeros de copias */
        private HashMap<String,GLL_Copias>      GLLCs;

        /** UBD:                                Objeto de tipo UtilidadesBD */
        public static UtilidadesBD              UBD = new UtilidadesBD();
        /** UCAL:                               Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar        UCAL = new UtilidadesCalendar();


	
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Almacen      Objeto de tipo Almacen
         */
        public Almacen () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Almacen con los valores pasados parametricamente
	 *
	 * @param   Cod_Alm         String con el codigo de almacen
         * @param   Descripcion     String con la descripcion del almacen
         * @param   GLLCs           HashMap de llaves y llaveros almacenados, junto con sus numeros de copias
	 * @return  Almacen         Objeto de tipo Almacen
	 */
	public Almacen (String Cod_Alm, String Descripcion, HashMap<String,GLL_Copias> GLLCs) throws DataException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Alm == null) || (Descripcion == null) || (Cod_Alm.length()>4) || (Descripcion.length()>30))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Alm = Cod_Alm.toUpperCase();
            this.Descripcion = Descripcion;
            this.GLLCs = GLLCs;
	}
	/** Constructor: Constructor de clase que crea el objeto Almacen con los valores pasados parametricamente
	 *
	 * @param   Descripcion     String con la descripcion de almacen
	 * @return  Almacen         Objeto de tipo Almacen
	 */
	public Almacen (String Descripcion) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || ((Descripcion.length()>30)))           throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Almacen WHERE Descripcion = \"" + Descripcion + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Alm = rs.getString(1);
            this.Descripcion = rs.getString(2);
            this.GLLCs = getGLLCsBD();

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodAlm: Metodo que obtiene el valor del atributo Cod_Alm
	 *
	 * @param   void
	 * @return  String	String con el codigo de almacen
	 */
	public String getCodAlm()			{ return this.Cod_Alm; }
        /** setCodAlm: Metodo que establece el valor del atributo Cod_Alm
	 *
	 * @param   Cod_Alm     String     String con el codigo de almacen
	 * @return  void
	 */
	public void setCodAlm(String Cod_Alm) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Alm == null) || (Cod_Alm.length()>4))  throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Alm = Cod_Alm.toUpperCase();
        }
	
        /** getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	 *
	 * @param   void
	 * @return  String	String con la descripcion del almacen
	 */
	public String getDescripcion()			{ return this.Descripcion; }
        /** setDescripcion: Metodo que establece el valor del atributo Descripcion
	 *
	 * @param   Descripcion     String     String con la descripcion del almacen
	 * @return  void
	 */
	public void setDescripcion(String Descripcion) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Descripcion.length()>30)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Descripcion = Descripcion;
        }

        /** getGLLCs: Metodo que obtiene el valor del atributo Llaves_Llaveros
	 *
	 * @param   void
	 * @return  HashMap<String,GLL_Copias>	HashMap de llaves y llaveros almacenados, junto con sus numeros de copias
	 */
	public HashMap<String,GLL_Copias> getGLLCs()                { return this.GLLCs; }
        /** setGLLCs: Metodo que establece el valor del atributo Llaves_Llaveros
	 *
	 * @param   GLLCs     HashMap<String,GLL_Copias>     HashMap de llaves y llaveros almacenados, junto con sus numeros de copias
	 * @return  void
	 */
	public void setGLLCs(HashMap<String,GLL_Copias> GLLCs) throws DataException     { this.GLLCs = GLLCs; }
        /** addGLLC: Metodo que añade una llave o llavero a los ya existentes en el almacen
         *
         * @param   GLLC       GLL_Copias      Llave o llavero a almacenar
         * @return  void
         */
        public void addGLLC(GLL_Copias GLLC) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLC == null)                           throw new DataException();

            // Obtenemos el tipo que queremos insertar: Llave o llavero
            String tipo = "L";
            if (GLLC.getLlaveLlavero() instanceof Llavero)         tipo = "LL";

            // Comprobamos si existe en el almacen
            String key = tipo + " - " + GLLC.getLlaveLlavero().getCodigo() + " - " + UCAL.calendarToString(GLLC.getLlaveLlavero().getFechaI());
            if (getGLLCs().containsKey(key))
            {
                // Obtenemos el GLL_Copias
                GLL_Copias gllc = getGLLC(key);
                // Le sumamos las copias que queremos insertar
                if ((gllc.getNumCopias() + GLLC.getNumCopias())>99)     gllc.setNumCopias(99);
                else                                                    gllc.setNumCopias(gllc.getNumCopias() + GLLC.getNumCopias());
            }
        }
        /** subsGLLC: Metodo que elimina parte de las copias de una llave o llavero del almacen
         *
         * @param   GLLC                GLL_Copias      Llave o llavero en el que eliminar
         * @return  void
         */
        public void subsGLLC(GLL_Copias GLLC) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLC == null)                           throw new DataException();

            // Obtenemos el tipo que queremos insertar: Llave o llavero
            String tipo = "L";
            if (GLLC.getLlaveLlavero() instanceof Llavero)         tipo = "LL";

            // Comprobamos si existe en el almacen
            String key =tipo + " - " + GLLC.getLlaveLlavero().getCodigo() + " - " + UCAL.calendarToString(GLLC.getLlaveLlavero().getFechaI());
            if (getGLLCs().containsKey(key))
            {
                // Obtenemos el GLL_Copias
                GLL_Copias gllc = getGLLC(key);
                // Le restamos las copias deseadas
                if ((gllc.getNumCopias() - GLLC.getNumCopias())<0)      removeGLLC(gllc);
                else                                                    gllc.setNumCopias(gllc.getNumCopias() - GLLC.getNumCopias());
            }
        }
        /** updateGLLC: Metodo que elimina una llave o llavero del almacen y lo reemplaza por otro
         *
         * @param   GLLC        GLL_Copias      GLLC a eliminar
         * @param   GLLC        GLL_Copias      GLL por el que reemplazar
         * @return  void
         */
        public void updateGLLC(GLL_Copias GLLC_old, GLL_Copias GLLC_new) throws DataException
        {
            // Obtenemos los tipos de los argumentos pasados
            String tipo_GLLC_old = "L";
            String tipo_GLLC_new = "L";
            if (GLLC_old.getLlaveLlavero() instanceof Llavero)   tipo_GLLC_old = "LL";
            if (GLLC_new.getLlaveLlavero() instanceof Llavero)   tipo_GLLC_new = "LL";

            // Comprobamos que los parametros pasados son correctos
            if ((GLLC_old == null) || (GLLC_new == null) || (!tipo_GLLC_old.equals(tipo_GLLC_new)))     throw new DataException();

            // Realizamos la actualizacion de los atributos
            String key_old = tipo_GLLC_old + " - " + GLLC_old.getLlaveLlavero().getCodigo() + " - " + UCAL.calendarToString(GLLC_old.getLlaveLlavero().getFechaI());
            if (getGLLCs().containsKey(key_old))
            {
                removeGLLC(GLLC_old);
                addGLLC(GLLC_new);
            }
        }
        /** removeGLLC: Metodo que elimina todas las copias de una llave o llavero del almacen
         *
         * @param   GLLC        GLL_Copias  Llave o llavero a eliminar
         * @return  void
         */
        public void removeGLLC(GLL_Copias GLLC) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLC == null)                  throw new DataException();

            // Obtenemos el tipo que queremos insertar: Llave o llavero
            String tipo = "L";
            if (GLLC.getLlaveLlavero() instanceof Llavero)         tipo = "LL";

            getGLLCs().remove(tipo + " - " + GLLC.getLlaveLlavero().getCodigo() + " - " + UCAL.calendarToString(GLLC.getLlaveLlavero().getFechaI()));
        }
        /** getGLLC: Metodo que obtiene un GLL_Copias de los existentes
         *
         * @param   String        Key_GLLC  Key del GLLC a obtener
         * @return  void
         */
        public GLL_Copias getGLLC(String Key_GLLC) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Key_GLLC == null) || (Key_GLLC.length()>30))       throw new DataException();
            // Devolvemos el resultado
            return this.getGLLCs().get(Key_GLLC);
        }

        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Strings con las llaves y llaveros almacenados
            String lls = "";

            // Añadimos las Strings con todas las llaves y llaveros
            Iterator it = getGLLCs().entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKGLLC = (Map.Entry) it.next();
                GLL_Copias gllc = (GLL_Copias) VKGLLC.getValue();
                lls = lls + gllc.toString();
            }

            // Devolvemos el resultado
            return "ALMACEN:\nCod_Alm: " + getCodAlm() + "\n" +
            "Descripcion: " + getDescripcion() + "\n\n" +
            lls + "\n";
	}


        // DML
        /** insertGLLCBD: Metodo que inserta un GLL_Copias de este almacen en la BD
         *
         * @param   GLL_Copias  GLLC            GLL_Copias a insertar en el almacen
         * @return  int                         Numero de filas (Llaves o Llaveros) insertadas
         */
        public int insertGLLCBD(GLL_Copias GLLC) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLC == null)                          throw new DataException();

            // Creamos la String con la sentencia a realizar
            String tabla = "Almacen_Llave";

            if (GLLC.getLlaveLlavero() instanceof Llavero) tabla = "Almacen_Llavero";

            String query = "INSERT INTO " + tabla + " VALUES (\"" +
                getCodAlm() + "\"," +
                GLLC.getLlaveLlavero().getCodigo() + ",\"" +
                UCAL.calendarToString(GLLC.getLlaveLlavero().getFechaI()) + "\"," +
                GLLC.getNumCopias() + ");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateGLLCBD: Metodo que actualiza un GLL_Copias de este almacen en la BD
         *
         * @param   GLLC_old    GLL_Copias a modificar
         * @param   GLLC_new    GLL_Copias de reemplazo
         * @return  int         Resultado
         */
        public int updateGLLCBD(GLL_Copias GLLC_old, GLL_Copias GLLC_new) throws DataException, SQLException
        {
            // Obtenemos los tipos de los argumentos pasados
            String tipo_GLLC_old = "L";
            String tipo_GLLC_new = "L";
            if (GLLC_old.getLlaveLlavero() instanceof Llavero)   tipo_GLLC_old = "LL";
            if (GLLC_new.getLlaveLlavero() instanceof Llavero)   tipo_GLLC_new = "LL";

            // Comprobamos que los parametros pasados son correctos
            if ((GLLC_old == null) || (GLLC_new == null) || (!tipo_GLLC_old.equals(tipo_GLLC_new)))     throw new DataException();

            // Obtenemos la tabla sobre la que realizar la consulta
            String tabla = "Almacen_Llave";
            String cod_elem = "";
            String cod_f = "";
            if (tipo_GLLC_old.equals("LL"))
            {
                tabla = "Almacen_Llavero";
                cod_elem = "ro";
                cod_f = "l";
            }

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE " + tabla + " SET " +
                    "Cod_Llave" + cod_elem + " = " + GLLC_new.getLlaveLlavero().getCodigo() + "," +
                    "F_Alta_L" + cod_f + " = \"" + UCAL.calendarToString(GLLC_new.getLlaveLlavero().getFechaI()).substring(0,19) + "\"," +
                    "Num_Copias = " + GLLC_new.getNumCopias() +
                    " WHERE Cod_Alm = \"" + getCodAlm()+ "\" AND Cod_Llave" + cod_elem + " = " +
                    GLLC_old.getLlaveLlavero().getCodigo() + " AND F_Alta_L" + cod_f + " = \"" +
                    UCAL.calendarToString(GLLC_old.getLlaveLlavero().getFechaI()).substring(0,19) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeGLLCBD: Metodo que borra un GLL_Copias de un almacen de la BD
         *
         * @param   GLL_Copias    GLLC      GLL_Copias a eliminar
         * @return  int                     Numero de filas (Llaves o Llaveros) eliminadas
         */
        public int removeGLLCBD(GLL_Copias GLLC) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLC == null)                          throw new DataException();

            // Tabla y elementos sobre los que realizar la consulta
            String tabla = "Almacen_Llave";
            String elem_cod = "Llave";
            String elem_f = "L";

            if (GLLC.getLlaveLlavero() instanceof Llavero)
            {
                tabla = "Almacen_Llavero";
                elem_cod = "Llavero";
                elem_f = "Ll";
            }

            String query = "DELETE FROM " + tabla + " WHERE Cod_Alm = \"" +
                getCodAlm() + "\" AND Cod_" + elem_cod + " = " + GLLC.getLlaveLlavero().getCodigo() +
                " AND F_Alta_" + elem_f + " = \"" + UCAL.calendarToString(GLLC.getLlaveLlavero().getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getGLLCBD: Metodo que obtiene un GLL_Copias del almacen de la BD
         *
	 * @param   GLLC_Key        Key de la GLL_Copias
	 * @return  GLL_Copias      Objeto de tipo GLL_Copias
	 */
	public GLL_Copias getGLLCBD(String GLLC_Key) throws DataException, SQLException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((GLLC_Key == null) || (GLLC_Key.length()>29))           throw new DataException();

            // Tabla y elementos sobre los que realizar la consulta
            String tabla = "Almacen_Llave";
            String elem_cod = "Llave";
            String elem_f;
            // Obtenemos la tabla sobre la que realizar la consulta
            if (GLLC_Key.startsWith("LL"))
            {
                tabla = "Almacen_Llavero";
                elem_cod = "Llavero";
            }

            // Tokens con los datos de la llave/llavero
            StringTokenizer st = new StringTokenizer(GLLC_Key);
            elem_f = st.nextToken(" - ");
            int Codigo = Integer.parseInt(st.nextToken());
            String FAlta = st.nextToken("").substring(3);

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Num_Copias FROM " + tabla +
                    " WHERE Cod_Alm = \"" + getCodAlm() + "\" AND Cod_" + elem_cod + " = " + Codigo +
                    " AND F_Alta_" + elem_f + " = \"" + FAlta + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            Gen_LLl Llave_Llavero;
            if (elem_cod.equals("Llave"))   Llave_Llavero = new Llave(Codigo,FAlta);
            else                            Llave_Llavero = new Llavero(Codigo,FAlta);
            int Num_Copias = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return new GLL_Copias(Llave_Llavero,Num_Copias);
	}

        /** insertGLLCsBD: Metodo que inserta los GLL_Copias de este almacen en la BD
         *
         * @param   void
         * @return  int                                             Numero de filas insertadas
         */
        public int insertGLLCsBD() throws DataException, SQLException
        {
            // Insertamos los GLL_Devs de este acceso en la BD
            return insertGLLCsBD(getGLLCs());
        }
        /** insertGLLCsBD: Metodo que inserta los GLL_Copias de este almacen en la BD
         *
         * @param   HashMap<String,GLL_Copias>    GLL_Copias    GLL_Copias del almacen
         * @return  int                                         Numero de filas (GLL_Devs) insertadas
         */
        public int insertGLLCsBD(HashMap<String,GLL_Copias> GLLCs) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (GLLCs == null)                              throw new DataException();

            // Insertamos cada GLL_Copias del HashMap
            Iterator it = GLLCs.entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos el destinatario
                Map.Entry VKGLLC = (Map.Entry) it.next();
                GLL_Copias GLLC = (GLL_Copias) VKGLLC.getValue();

                // Insertamos el GLL_Copias
                insertGLLCBD (GLLC);
            }

            return GLLCs.size();
        }
        /** updateGLLCsBD: Metodo que actualiza los GLL_Copias de un Almacen
         *
         * @param   HashMap<String,GLL_Copias>     GLLCs    GLL_Copias del Almacen
         * @return  int                                     Numero de filas (GLL_Copias) actualizadas
         */
        public int updateGLLCsBD(HashMap<String,GLL_Copias> GLLCs) throws DataException, SQLException
        {
            // Filas actualizadas
            int res = 0;

            // Recorremos el HashMap nuevo, actualizando los GLL_Copias existentes e insertando los nuevos
            Iterator it_new = GLLCs.entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKGLLC = (Map.Entry) it_new.next();
                GLL_Copias gllc_new = (GLL_Copias) VKGLLC.getValue();
                String gllc_key = (String) VKGLLC.getKey();

                // Si existe el GLLC -> Lo actualizamos
                if (getGLLCs().containsKey(gllc_key))   updateGLLCBD(getGLLCs().get(gllc_key),gllc_new);
                // Si no existe -> Lo insertamos
                else                                    insertGLLCBD(gllc_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellos GLL_Copias que no consten en el nuevo
            Iterator it_old = getGLLCs().entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKGLLC = (Map.Entry) it_old.next();
                GLL_Copias gllc_old = (GLL_Copias) VKGLLC.getValue();
                String gllc_key = (String) VKGLLC.getKey();

                // Si el GLL_Copias viejo no existe como nuevo -> Lo eliminamos y actualizamos el contador de filas actualizadas
                if (! GLLCs.containsKey(gllc_key))
                {
                    removeGLLCBD(gllc_old);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removeGLLCsBD: Metodo que borra los GLL_Copias de un Almacen en la BD
         *
         * @param   void
         * @return  int                             Numero de filas (GLL_Copias) eliminadas
         */
        public int removeGLLCsBD() throws SQLException
        {
            // BORRAMOS LAS LLAVES ALMACENADAS
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Almacen_Llave WHERE Cod_Alm = \"" + getCodAlm() + "\";";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // BORRAMOS LOS LLAVEROS ALMACENADOS
            // Creamos la String con la sentencia a realizar
            query = "DELETE FROM Almacen_Llavero WHERE Cod_Alm = \"" + getCodAlm() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query) + res;
        }

        /** getGLLCsBD: Metodo que obtiene el conjunto de GLL_Copias del almacen de la BD
         *
	 * @param   void
	 * @return  HashMap<String,GLL_Copias>      HashMap con los GLL_Copias del almacen
	 */
	public HashMap<String,GLL_Copias> getGLLCsBD() throws DataException, SQLException, RemoteException
	{
            // Creamos el HashMap de GLL_Copias
            HashMap<String,GLL_Copias> Llaves_Llaveros = new HashMap();

            // Obtenemos las llaves del almacen
            String query = "SELECT Cod_Llave,F_Alta_L FROM Almacen_Llave WHERE Cod_Alm = \"" + getCodAlm() + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Añadimos todos los GLL_Copias correspondientes a llaves
            while (rs.next())
            {
                String key = "L - " + rs.getInt(1) + " - " + rs.getString(2).substring(0,19);
                Llaves_Llaveros.put(key, getGLLCBD(key));
            }

            // Obtenemos las llaveros del almacen
            query = "SELECT Cod_Llavero,F_Alta_LL FROM Almacen_Llavero WHERE Cod_Alm = \"" + getCodAlm() + "\";";
            rs = UBD.ejecutarQuery(query);

            // Añadimos todos los GLL_Copias correspondientes a llaveros
            while (rs.next())
            {
                String key = "LL - " + rs.getInt(1) + " - " + rs.getString(2).substring(0,19);
                Llaves_Llaveros.put(key, getGLLCBD(key));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Llaves_Llaveros;
	}

        /** insertAlmacenBD: Metodo que inserta el Almacen en la BD
         *
         * @param   void
         * @return  int     Numero de filas (Almacenes) insertadas
         */
        public int insertAlmacenBD() throws DataException, SQLException
        {
            // INSERTAMOS EL ALMACEN
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Almacen VALUES (\""  +
                    getCodAlm() + "\",\"" +
                    getDescripcion() + "\");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);


            // Si el almacen ha sido insertado correctamente -> INSERTAMOS LAS LLAVES Y LLAVEROS QUE ALMACENA
            if (res == 1)   insertGLLCsBD();

            // Devolvemos el resultado
            return res;
        }
        /** updateAlmacenBD: Metodo que actualiza el Almacen en la BD
         *
         * @param   Almacen         Almacen a modificar
         * @return  int             Resultado
         */
        public int updateAlmacenBD(Almacen Almacen) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Almacen == null)                                     throw new DataException();

            // ACTUALIZAMOS EL ALMACEN
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Almacen SET " +
                    "Cod_Alm = \"" + Almacen.getCodAlm() + "\"," +
                    "Descripcion = \"" + Almacen.getDescripcion() + "\"" +
                    " WHERE Cod_Alm = \"" + getCodAlm() + "\";";

            // Ejecutamos la sentencia
            int res = UBD.ejecutarUpdate(query);

            // ACTUALIZAMOS LOS GLL_COPIAS
            // Actualizamos la PK del Almacen para una correcta actualizacion de los GLL_Copias
            setCodAlm(Almacen.getCodAlm());
            // Actualizamos los GLL_Devs y devolvemos el resultado
            return updateGLLCsBD(Almacen.getGLLCs()) + res;
        }
        /** removeAlmacenBD: Metodo que borra un Almacen de la BD
         *
         * @param   void
         * @return  int             Numero de filas eliminadas
         */
        public int removeAlmacenBD() throws DataException, SQLException
        {
            // Borramos el almacen y devolvemos el resultado
            return removeAlmacenBD(this);
        }
        /** removeAlmacenBD: Metodo que borra un Almacen de la BD
         *
         * @param   Almacen         Almacen a borrar
         * @return  int             Numero de filas (Almacenes) eliminadas
         */
        public int removeAlmacenBD(Almacen Almacen) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Almacen == null)                                     throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Almacen WHERE Cod_Alm = \"" + Almacen.getCodAlm() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* obtenerAlmacenes: Metodo que devuelve los almacenes de llaves existentes en el centro y que pasan un filtro dado
         *
         * @param   String      Filtro      Filtro que debe pasar la descripcion del almacen
         * @return  ResultSet   rs          Almacenes existentes en el centro
         */
        public ResultSet obtenerAlmacenes(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT * FROM (SELECT'Todos' Almacenes UNION SELECT Descripcion FROM Almacen) Alm_Tot" +
                    " WHERE Almacenes LIKE \"%" + Filtro + "%\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLlaves: Metodo que devuelve un listado de las llaves existentes en el almacen, que pasan un filtro dado, e informacion adicional
         *
         * @param   String      Filtro      Filtro que debe pasar la descripcion de las llaves
         * @return  ResultSet   rs          Llaves existentes en el almacen e informacion adicional
         */
        public ResultSet obtenerLlaves(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT AL.Cod_Llave Código, Z.Descripcion Zona, L.Descripcion Descripción, Num_Copias Total" +
                            " FROM Almacen_Llave AL JOIN Llave L USING (Cod_Llave,F_Alta_L) JOIN Zona Z USING (Cod_Zona)" +
                            " WHERE Cod_Alm = \"" + getCodAlm() +
                                "\" AND L.F_Baja_L IS NULL AND (Z.Descripcion LIKE \"%" + Filtro + "%\" OR L.Descripcion LIKE \"%" + Filtro +
                            "%\") GROUP BY AL.Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLlaveros: Metodo que devuelve un listado de los llaveros existentes en el almacen, que pasan un filtro dado, e informacion adicional
         *
         * @param   String      Filtro      Filtro que debe pasar la descripcion de los llaveros
         * @return  ResultSet   rs          Llaveros existentes en el almacen e informacion adicional
         */
        public ResultSet obtenerLlaveros(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT AL.Cod_Llavero Código, Ll.Descripcion Descripción, Num_Copias Total" +
                            " FROM Almacen_Llavero AL JOIN Llavero Ll USING (Cod_Llavero,F_Alta_Ll) " +
                            " WHERE Cod_Alm = \"" + getCodAlm() + "\" AND Ll.F_Baja_Ll IS NULL AND Ll.Descripcion LIKE \"%" + Filtro +
                            "%\" GROUP BY AL.Cod_Llavero;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLlavesTotales: Metodo que devuelve un listado de las llaves existentes en los almacenes, que pasan un filtro dado, e informacion adicional
         *
         * @param   String      Filtro      Filtro que debe pasar la descripcion de las llaves
         * @return  ResultSet   rs          Llaves existentes en los almacenes e informacion adicional
         */
        public ResultSet obtenerLlavesTotales(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT AL.Cod_Llave Código, Z.Descripcion Zona, L.Descripcion Descripción, En_Prestamo.Prestadas 'En préstamo', SUM(Num_Copias) Total" +
                            " FROM Almacen_Llave AL JOIN Llave L USING (Cod_Llave,F_Alta_L) JOIN Zona Z USING (Cod_Zona) JOIN" +
                                " (SELECT Cod_Llave,F_Alta_L, IFNULL(COUNT(Devuelta),0)-IFNULL(SUM(Devuelta),0) Prestadas" +
                                " FROM Llave LEFT JOIN Llave_Prestada USING (Cod_Llave,F_Alta_L)" +
                                " GROUP BY Cod_Llave,F_Alta_L) En_Prestamo USING (Cod_Llave,F_Alta_L)" +
                            " WHERE L.F_Baja_L IS NULL AND (Z.Descripcion LIKE \"%" + Filtro + "%\" OR L.Descripcion LIKE \"%" + Filtro +
                            "%\") GROUP BY AL.Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLlaverosTotales: Metodo que devuelve un listado de los llaveros existentes en los almacenes, que pasan un filtro dado, e informacion adicional
         *
         * @param   String      Filtro      Filtro que debe pasar la descripcion de los llaveros
         * @return  ResultSet   rs          Llaveros existentes en los almacenes e informacion adicional
         */
        public ResultSet obtenerLlaverosTotales(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT AL.Cod_Llavero Código, L.Descripcion Descripción, En_Prestamo.Prestados 'En préstamo', SUM(Num_Copias) Total" +
                            " FROM Almacen_Llavero AL LEFT JOIN Llavero L USING (Cod_Llavero,F_Alta_Ll) JOIN" +
                                " (SELECT Cod_Llavero,F_Alta_Ll, IFNULL(COUNT(Devuelto),0)-IFNULL(SUM(Devuelto),0) Prestados" +
                                " FROM Llavero LEFT JOIN Llavero_Prestado USING (Cod_Llavero,F_Alta_Ll)" +
                                " GROUP BY Cod_Llavero,F_Alta_Ll) En_Prestamo USING (Cod_Llavero,F_Alta_Ll)" +
                            " WHERE L.F_Baja_Ll IS NULL AND L.Descripcion LIKE \"%" + Filtro +
                            "%\" GROUP BY AL.Cod_Llavero;";

            // Creamos la String con la sentencia a realizar
            // String query = "SELECT Cod_Llavero,Descripcion FROM Llavero WHERE F_Baja_Ll IS NULL;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerCodLls: Metodo que devuelve los codigos de todos los llaveros existentes y que no estan en el almacen
         *
         * @param   void
         * @return  ResultSet   rs          Codigos de todos los llaveros existentes no almacenados en este almacen
         */
        public ResultSet obtenerCodLls() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llavero FROM Llavero WHERE F_Baja_Ll IS NULL AND Cod_Llavero NOT IN (SELECT Cod_Llavero FROM Almacen_Llavero JOIN Llavero USING (Cod_Llavero,F_Alta_Ll) WHERE Cod_Alm = \"" + getCodAlm() + "\" AND F_Baja_Ll IS NULL);";

            // Ejecutamos y devolvemos el resultado de la consulta
            return UBD.ejecutarQuery(query);
        }

        /* disparadorAlmL: Metodo que ejecuta el disparador sobre AFTER DELETE ON Almacen_Llave
         *
         * @param   void
         * @return  int
         */
        public int disparadorAlmL() throws SQLException
        {
            // Eliminamos los almacenes que no contengan elementos
            String query = "DELETE FROM Almacen WHERE Cod_Alm NOT IN (SELECT DISTINCT Cod_Alm FROM Almacen_Llave JOIN Llave USING (Cod_Llave,F_Alta_L) WHERE F_Baja_L IS NULL UNION SELECT DISTINCT Cod_Alm FROM Almacen_Llavero JOIN Llavero USING (Cod_Llavero,F_Alta_Ll) WHERE F_Baja_Ll IS NULL);";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }

        /* disparadorAlmLl: Metodo que ejecuta el disparador sobre AFTER DELETE ON Almacen_Llavero
         *
         * @param   void
         * @return  int
         */
        public int disparadorAlmLl() throws SQLException
        {
            // Eliminamos los llaveros que no dispongan de copias
            String query = "DELETE FROM Llavero WHERE (Cod_Llavero,F_Alta_Ll) NOT IN (SELECT DISTINCT Cod_Llavero, F_Alta_Ll FROM Almacen_Llavero);";
            UBD.ejecutarUpdate(query);

            // Eliminamos todas aquellas llaves que no formen parte de algun llavero o tengan existencia propia
            query = "DELETE FROM Llave WHERE (Cod_Llave,F_Alta_L) NOT IN (SELECT DISTINCT Cod_Llave,F_Alta_L FROM Composicion_Llavero UNION SELECT DISTINCT Cod_Llave,F_Alta_L FROM Almacen_Llave);";
            UBD.ejecutarUpdate(query);

            // Eliminamos todos los almacenes que no contengan ningun elemento
            query = "DELETE FROM Almacen WHERE Cod_Alm NOT IN (SELECT DISTINCT Cod_Alm FROM Almacen_Llave JOIN Llave USING (Cod_Llave,F_Alta_L) WHERE F_Baja_L IS NULL UNION SELECT DISTINCT Cod_Alm FROM Almacen_Llavero JOIN Llavero USING (Cod_Llavero,F_Alta_Ll) WHERE F_Baja_Ll IS NULL);";            UBD.ejecutarUpdate(query);
            
            // Devolvemos el resultado
            return 0;
        }
}