// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Cargo_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;



/** CLASE CARGO:  Clase que representa un cargo posible para los contactos
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Cargo implements Cargo_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cod_Cargo:              String con el codigo de cargo */
	private String              Cod_Cargo;
        /** Descripcion:            String con la Descripcion del cargo */
	private String              Descripcion;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Cargo      Objeto de tipo Cargo
         */
        public Cargo () throws DataException, RemoteException {}
        /** Constructor: Constructor de clase que crea el objeto Cargo con los valores pasados parametricamente
	 *
	 * @param   Cod_Cargo   String con el codigo de cargo
         * @param   Descripcion String con la descripcion del cargo
	 * @return  Cargo	Objeto de tipo Cargo
	 */
	public Cargo(String Cod_Cargo, String Descripcion) throws DataException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Cargo == null) || (Descripcion == null) || (Cod_Cargo.length()>5) || (Descripcion.length()>50))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Cargo = Cod_Cargo.toUpperCase();
            this.Descripcion = Descripcion;
	}

	/** Constructor: Constructor de clase que crea el objeto Cargo con los valores pasados parametricamente
	 *
	 * @param   Cod_Cargo   String con el codigo de cargo
	 * @return  Cargo	Objeto de tipo Cargo
	 */
	public Cargo(String Cod_Cargo) throws DataException, SQLException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Cargo == null) || (Cod_Cargo.length()>5))      throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Cargo WHERE Cod_Cargo = \"" + Cod_Cargo + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Cargo = rs.getString(1);
            this.Descripcion = rs.getString(2);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}

	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodCargo: Metodo que obtiene el valor del atributo Cod_Cargo
	 *
	 * @param   void
	 * @return  String	String con el codigo de cargo
	 */
	public String getCodCargo()			{ return this.Cod_Cargo; }
        /** setCodCargo: Metodo que establece el valor del atributo Cod_Cargo
	 *
	 * @param   Cod_Cargo     String     String con el codigo de cargo
	 * @return  void
	 */
	public void setCodCargo(String Cod_Cargo) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Cargo == null) || (Cod_Cargo.length()>5))  throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Cargo = Cod_Cargo.toUpperCase();
        }
	
        /** getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	 *
	 * @param   void
	 * @return  String	String con la descripcion del cargo
	 */
	public String getDescripcion()			{ return this.Descripcion; }
        /** setDescripcion: Metodo que establece el valor del atributo Descripcion
	 *
	 * @param   Descripcion     String     String con la descripcion del cargo
	 * @return  void
	 */
	public void setDescripcion(String Descripcion) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Descripcion.length()>50)                throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Descripcion = Descripcion;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "CARGO:\nCod_Cargo: " + getCodCargo() + "\n" +
            "Descripcion: " + getDescripcion() + "\n";
	}


        // DML
        /** insertCargoBD: Metodo que inserta el Cargo en la BD
         *
         * @param   void
         * @return  int     Resultado
         */
        public int insertCargoBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Cargo VALUES (\""  +
                    getCodCargo() + "\",\"" +
                    getDescripcion() + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateCargoBD: Metodo que actualiza el Cargo en la BD
         *
         * @param   Cargo   Cargo   Cargo con el que modificar
         * @return  int             Resultado
         */
        public int updateCargoBD(Cargo Cargo) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Cargo == null)                                   throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Cargo SET " +
                    "Cod_Cargo = \"" + Cargo.getCodCargo() + "\"," +
                    "Descripcion = \"" + Cargo.getDescripcion() +
                    "\" WHERE Cod_Cargo = \"" + getCodCargo() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeCargo: Metodo que borra un Cargo de la BD
         *
         * @param   void
         * @return  int     Resultado
         */
        public int removeCargoBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Cargo WHERE Cod_Cargo = \"" + getCodCargo() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDAD
        /* obtenerCargos: Metodo que devuelve los datos de los cargos existentes en el sistema
         *
         * @return  ResultSet   rs          Datos de los cargos existentes en el sistema
         */
        public ResultSet obtenerCargos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Cargo Código, Descripcion Descripción FROM Cargo;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
        /* obtenerCodCargo: Metodo que obtiene el codigo del cargo dada su descripcion
         *
         * @param   String  Descripcion         Descripcion del cargo
         * @return  String  Cod_Cargo           Codigo del cargo
         */
        public String obtenerCodCargo(String Descripcion) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Cargo FROM Cargo WHERE Descripcion = \"" + Descripcion + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            String Cod_Cargo = "";
            if (rs.next())  Cod_Cargo = rs.getString(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Cod_Cargo;
        }
        /* obtenerDescripCargos: Metodo que devuelve las descripciones de los cargos existentes en el sistema
         *
         * @return  ResultSet   rs          Descripciones de los cargos existentes en el sistema
         */
        public ResultSet obtenerDescripCargos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Descripcion Cargos FROM Cargo;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
}