// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Color_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.Calendar;



/** CLASE COLOR:  Clase que representa un color de tarjeta
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Color extends DosFechas implements Color_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Color:                  Codigo de color */
	private int                 Cod_Color;
        /** Nivel:                  Nivel de acceso que otorga el color */
	private Nivel               Nivel;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();


	
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Color      Objeto de tipo Color
         */
        public Color () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Color con los valores pasados parametricamente
	 *
	 * @param   Cod_Color   Codigo de color
         * @param   F_Alta_C    Fecha en la que se realiza el alta del color
         * @param   Nivel       Nivel de acceso que otorga el color
	 * @return  Color	Objeto de tipo Color
	 */
	public Color(int Cod_Color, Calendar F_Alta_C, Nivel Nivel) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta_C);

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Color<-16777216) || (Cod_Color>16777216))         throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Color = Cod_Color;
            this.Nivel = Nivel;
	}
	/** Constructor: Constructor de clase que crea el objeto Color con los valores pasados parametricamente
	 *
	 * @param   Color       Codigo de color
         * @param   F_Alta_C    Fecha en la que se realiza el alta del color
         * @param   F_Baja_C    Fecha en la que se realiza la baja del color
         * @param   Nivel       Nivel de acceso que otorga el color
	 * @return  Color	Objeto de tipo color
	 */
	public Color(int Cod_Color, Calendar F_Alta_C, Calendar F_Baja_C, Nivel Nivel) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta_C,F_Baja_C);

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Color<-16777216) || (Cod_Color>16777216))              throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Color = Cod_Color;
            this.Nivel = Nivel;
	}
	/** Constructor: Constructor de clase que crea el objeto Color con los valores pasados parametricamente
	 *
	 * @param   Color       Codigo de color
         * @param   F_Alta_C    Fecha en la que se realiza el alta del color
	 * @return  Color	Objeto de tipo color
	 */
	public Color(int Cod_Color, String F_Alta_C) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Color<-16777216) || (Cod_Color>16777216) || (F_Alta_C == null) || (F_Alta_C.length()>19))
                throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Color WHERE Cod_Color = " + Cod_Color +
                " AND F_Alta_C = \"" + F_Alta_C + "\";";

            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            boolean tiene_nivel = rs.next();
            this.Cod_Color = rs.getInt(1);
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            this.Nivel = (tiene_nivel ? null : new Nivel(rs.getInt(4),rs.getString(5).substring(0,19)));

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
	/** Constructor: Constructor de clase que crea el objeto Color con los valores pasados parametricamente
	 *
	 * @param   Color       Codigo de color
	 * @return  Color	Objeto de tipo color
	 */
	public Color(int Cod_Color) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Color<-16777216) || (Cod_Color>16777216))                      throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Color WHERE Cod_Color = " + Cod_Color +
                " AND F_Baja_C IS NULL" +
                " ORDER BY F_Alta_C DESC" +
                " LIMIT 0,1;";

            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Color = rs.getInt(1);
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            this.Nivel = new Nivel(rs.getInt(4),rs.getString(5).substring(0,19));

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodColor: Metodo que obtiene el valor del atributo CodColor
	 *
	 * @param   void
	 * @return  int         Codigo de color
	 */
	public int getCodColor()                        { return this.Cod_Color; }
        /** setColor: Metodo que establece el valor del atributo Color
	 *
	 * @param   Color     int     Codigo de color
	 * @return  void
	 */
	public void setCodColor(int Cod_Color) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Color<0) || (Cod_Color>16777216))          throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Color = Cod_Color;
        }

        /** getNivel: Metodo que obtiene el valor del atributo Nivel
	 *
	 * @param   void
	 * @return  Nivel	Nivel de acceso que otorga el color
	 */
	public Nivel getNivel()                         { return this.Nivel; }
        /** setNivel: Metodo que establece el valor del atributo Nivel
	 *
	 * @param   Nivel     Nivel     Nivel de acceso que otorga el color
	 * @return  void
	 */
	public void setNivel(Nivel Nivel)               { this.Nivel = Nivel; }


        // TOSTRING        
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "COLOR:\nCod_Color: " + getCodColor() + "\n" +
            super.toString() + "\n" +
            (getNivel() == null ? "" : getNivel().toString()) + "\n";
        }


        // DML
        /** insertColorBD: Metodo que inserta el Color en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertColorBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Color VALUES (\""  +
                    getCodColor() + "\",\"" +
                    calendarToString(getFechaI()) + "\"," +
                    (getFechaJ() == null ? "null" : "\"" + calendarToString(getFechaJ()) + "\"") + "," +
                    getNivel().getCodNiv() + ",\"" +
                    calendarToString(getNivel().getFechaI()) + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateColorBD: Metodo que actualiza el Color en la BD
         *
         * @param   Color       Color con el que modificar
         * @return  int         Numero de filas actualizadas
         */
        public int updateColorBD(Color Color) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Color == null)                                      throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Color SET " +
                    "Cod_Color = \"" + Color.getCodColor() + "\"," +
                    "F_Alta_C = \"" + calendarToString(Color.getFechaI()) + "\"," +
                    "F_Baja_C = " + (Color.getFechaJ() == null ? "null" : "\"" + calendarToString(Color.getFechaJ()) + "\"") + "," +
                    "Cod_Niv = " + Color.getNivel().getCodNiv() + "," +
                    "F_Alta_N = \"" + calendarToString(Color.getNivel().getFechaI()) +
                    "\" WHERE Cod_Color = \"" + getCodColor() + "\" AND F_Alta_C = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** darBajaColorBD: Metodo que da de baja el Color en la BD
         *
         * @param   void
         * @return  int         Numero de filas actualizadas
         */
        public int darBajaColorBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Color" +
                    " SET F_Baja_C = \"" + calendarToString(Calendar.getInstance()) + "\"" +
                    " WHERE Cod_Color = " + getCodColor() + " AND F_Alta_C = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** removeColorBD: Metodo que borra un Color en la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeColorBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Color WHERE Cod_Color = \"" + getCodColor() + "\" AND F_Alta_C = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }



        // FUNCIONALIDADES
        /* obtenerCeldasColores: Metodo que devuelve las celdas de los colores
         *
         * @return  ResultSet   rs          Celdas para los colores
         */
        public ResultSet obtenerCeldasColores() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Niv Colores" +
                    " FROM Color JOIN Nivel USING (Cod_Niv,F_Alta_N)" +
                    " WHERE F_Baja_C IS NULL AND F_Baja_N IS NULL" +
                    " ORDER BY Cod_Niv, Cod_Color;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerColores: Metodo que devuelve los colores de las colores existentes
         *
         * @return  ArrayList   al          Colores de los colores existentes
         */
        public ArrayList obtenerColores() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Color,Cod_Niv" +
                    " FROM Color JOIN Nivel USING (Cod_Niv,F_Alta_N)" +
                    " WHERE F_Baja_C IS NULL AND F_Baja_N IS NULL" +
                    " ORDER BY Cod_Niv;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Creamos el resultado
            ArrayList al = new ArrayList();
            while(rs.next())    al.add(rs.getInt(1));

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return al;
        }

        /* colorPorDefecto: Metodo que devuelve el color por defecto
         *
         * @return  int   Color          Color por defecto
         */
        public int colorPorDefecto() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT MIN(Cod_Color)" +
                    " FROM Color" +
                    " WHERE Cod_Niv = (SELECT MIN(Cod_Niv) FROM Nivel WHERE F_Baja_N IS NULL) AND F_Baja_C IS NULL;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int Color = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Color;
        }

        /* colorPorDefecto: Metodo que devuelve el color por defecto que le corresponde a la persona proporcionada
         *
         * @param   String  DNI         DNI de la persona de la cual se quiere obtener su color por defecto
         * @return  int     Color        Color obtenido
         */
        public int colorPorDefecto(String DNI) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
                String query = "SELECT Cod_Color" +
                        " FROM	( SELECT DISTINCT PC.DNI, Cod_Color" +
                                    " FROM Periodico PC JOIN Departamento D USING (Cod_Emp,Cod_Dep)" +
                                        " JOIN T_Acceso USING (Cod_Acc)" +
                                        " JOIN Color USING (Cod_Niv)" +

                                            " UNION " +

                                    "SELECT DISTINCT DNI, Cod_Color" +
                                    " FROM Ocasional JOIN T_Acceso USING (Cod_Acc) JOIN Color USING (Cod_Niv)" +
                                    " WHERE F_Caducidad > CURDATE()" +

                                            " UNION " +

                                    " SELECT DNI, Cod_Color" +
                                    " FROM Residente JOIN Nivel USING (Cod_Niv) JOIN Color USING (Cod_Niv)) Personal_Color" +

                        " WHERE DNI = \"" + DNI + "\" ORDER BY Cod_Color DESC LIMIT 1;";

             // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int Color = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Color;
        }

        /* eliminarColores: Metodo que borra todos los colores mas antiguos de Dias dias
         *
         * @param   Dias    Numero de dias transcurridos desde la baja del color
         * @return  int     Numero de colores eliminados
         */
        public int eliminarColores(int Dias) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Color WHERE ADDDATE(DATE(F_Baja_C),INTERVAL " + Dias + " DAY)<=CURDATE();";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /* disparadorInsert: Metodo que ejecuta el disparador sobre BEFORE INSERT ON Color
         *
         * @param   void
         * @return  int
         */
        public int disparadorInsert() throws SQLException
        {
            // Damos de baja el color a insertar
            String query = "UPDATE Color SET F_Baja_C = CURDATE() WHERE Cod_Color = " + getCodColor() + " AND F_Baja_C IS NULL;";
            UBD.ejecutarUpdate(query);
            
            // Damos de baja las tarjetas cuyo color insertamos
            query = "UPDATE Tarjeta SET F_Baja_T = CURDATE() WHERE (Cod_Color,F_Alta_C) IN (SELECT Cod_Color,F_Alta_C FROM Color WHERE Cod_Color = " + getCodColor() + " AND F_Baja_C IS NULL);";
            UBD.ejecutarUpdate(query);
            
            // Devolvemos el resultado
            return 0;
        }

        /* disparadorUpdate: Metodo que ejecuta el disparador sobre AFTER UPDATE ON Color
         *
         * @param   void
         * @return  int
         */
        public int disparadorUpdate() throws SQLException
        {
            // Eliminamos los niveles que no dispongan de colores
            String query = "DELETE FROM Nivel WHERE Cod_Niv NOT IN (SELECT Cod_Niv FROM Color WHERE F_Baja_C IS NULL);";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }

        /* disparadorRemove: Metodo que ejecuta el disparador sobre AFTER REMOVE ON Color
         *
         * @param   void
         * @return  int
         */
        public int disparadorRemove() throws SQLException
        {
            // Eliminamos los niveles que no dispongan de colores
            String query = "DELETE FROM Nivel WHERE (Cod_Niv,F_Alta_N) NOT IN (SELECT DISTINCT Cod_Niv,F_Alta_N FROM Color WHERE F_Baja_C IS NULL AND Cod_Niv IS NOT NULL);";
 System.out.println(query);
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }
}