// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Cuenta_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;



/** CLASE CUENTA:  Clase que representa una cuenta de un usuario del sistema
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Cuenta implements Cuenta_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Usuario:                Nombre de usuario */
	private String              Usuario;
        /** Pass:                   Contraseña */
	private String              Pass;
        /** Tipo:                   Tipo de usuario */
        private T_Usuario           Tipo;
        /** Persona:                Persona a la que pertenece la cuenta */
        private Personal    Persona;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Cuenta      Objeto de tipo Cuenta
         */
        public Cuenta () throws DataException, RemoteException {}
        /** Constructor: Constructor de clase que crea el objeto Cuenta con los valores pasados parametricamente
	 *
	 * @param   Usuario     Nombre de usuario
         * @param   Tipo        Tipo de usuario
         * @param   Persona     Persona a la que pertenece la cuenta
	 * @return  Cuenta	Objeto de tipo Cuenta
	 */
	public Cuenta(String Usuario, T_Usuario Tipo, Personal Persona) throws DataException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((Usuario == null) || (Tipo == null) || (Persona == null) || (Usuario.length()>20))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Usuario = Usuario;
            this.Pass = "cambiar";
            this.Tipo = Tipo;
            this.Persona = Persona;
	}
	/** Constructor: Constructor de clase que crea el objeto Cuenta con los valores pasados parametricamente
	 *
	 * @param   Usuario     Nombre de usuario
         * @param   Pass        Contraseña
         * @param   Tipo        Tipo de usuario
         * @param   Persona     Persona a la que pertenece la cuenta
	 * @return  Cuenta	Objeto de tipo Cuenta
	 */
	public Cuenta(String Usuario, String Pass, T_Usuario Tipo, Personal Persona) throws DataException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((Usuario == null) || (Pass == null) || (Tipo == null) || (Persona == null) || (Usuario.length()>20) || (Pass.length()>20))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Usuario = Usuario;
            this.Pass = Pass;
            this.Tipo = Tipo;
            this.Persona = Persona;
	}
	/** Constructor: Constructor de clase que crea el objeto Cuenta con los valores pasados parametricamente
	 *
	 * @param   Usuario     Nombre de usuario
	 * @return  Cuenta	Objeto de tipo Cuenta
	 */
	public Cuenta(String Usuario) throws DataException, SQLException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((Usuario == null) || (Usuario.length()>20))
                throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Cuenta WHERE Usuario = \"" + Usuario + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            if (rs.next())
            {
                this.Usuario = rs.getString(1);
                this.Pass = rs.getString(2);
                this.Tipo = new T_Usuario(rs.getString(3));
                this.Persona = new Personal(rs.getString(4));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
	/** Constructor: Constructor de clase que crea el objeto Cuenta con los valores pasados parametricamente
	 *
	 * @param   DNI         DNI del usuario
         * @param   Tipo        Tipo de usuario
	 * @return  Cuenta	Objeto de tipo Cuenta
	 */
	public Cuenta(String DNI, String Tipo) throws DataException, SQLException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (DNI.length()>9) || (Tipo == null) || (Tipo.length()>20))
                throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Cuenta WHERE DNI = \"" + DNI + "\" AND Tipo = \"" + Tipo + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            if (rs.next())
            {
                this.Usuario = rs.getString(1);
                this.Pass = rs.getString(2);
                this.Tipo = new T_Usuario(rs.getString(3));
                this.Persona = new Personal(rs.getString(4));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();
	}

	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getUsuario: Metodo que obtiene el valor del atributo Usuario
	 *
	 * @param   void
	 * @return  String     Nombre de usuario
	 */
	public String getUsuario()                      { return this.Usuario; }
        /** setUsuario: Metodo que establece el valor del atributo Usuario
	 *
	 * @param   Usuario     String     Nombre de usuario
	 * @return  void
	 */
	public void setUsuario(String Usuario) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Usuario == null) || (Usuario.length()>20)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Usuario = Usuario;
        }
	
        /** getPass: Metodo que obtiene el valor del atributo Pass
	 *
	 * @param   void
	 * @return  String     Contraseña
	 */
	public String getPass()                         { return this.Pass; }
        /** setPass: Metodo que establece el valor del atributo Pass
	 *
	 * @param   Pass     String     Contraseña
	 * @return  void
	 */
	public void setPass(String Pass) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Pass == null) || (Pass.length()>20))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Pass = Pass;
        }

        /** getTipo: Metodo que obtiene el valor del atributo Tipo
	 *
	 * @param   void
	 * @return  T_Usuario     Tipo de usuario
	 */
	public T_Usuario getTipo()                      { return this.Tipo; }
        /** setTipo: Metodo que establece el valor del atributo Tipo
	 *
	 * @param   Tipo     T_Usuario     Tipo de usuario
	 * @return  void
	 */
	public void setTipo(T_Usuario Tipo) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Tipo == null)                           throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Tipo = Tipo;
        }

        /** getPersona: Metodo que obtiene el valor del atributo Persona
	 *
	 * @param   void
	 * @return  Persona     Persona a la que pertenece la cuenta
	 */
	public Personal getPersona()                     { return this.Persona; }
        /** setPersona: Metodo que establece el valor del atributo Persona
	 *
	 * @param   Persona     Persona     Persona a la que pertenece la cuenta
	 * @return  void
	 */
	public void setPersona(Personal Persona) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona == null)                        throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Persona = Persona;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "CUENTA:\nUsuario: " + getUsuario() + "\n" +
            "Pass: " + getPass() + "\n\n" +
            getTipo() + "\n" +
            "Persona: " + getPersona() + "\n";
	}


        // DML
        /** insertCuentaBD: Metodo que inserta la Cuenta en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertCuentaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Cuenta VALUES (\""  +
                    getUsuario() + "\",\"" +
                    getPass() + "\",\"" +
                    getTipo().getTipo() + "\",\"" +
                    getPersona().getDNI() + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateCuentaBD: Metodo que actualiza la Cuenta en la BD
         *
         * @param   Cuenta  Cuenta con la que modificar
         * @return  int     Numero de filas actualizadas
         */
        public int updateCuentaBD(Cuenta Cuenta) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Cuenta == null)                         throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Cuenta SET " +
                    "Usuario = \"" + Cuenta.getUsuario() + "\"," +
                    "Pass = \"" + Cuenta.getPass() + "\"," +
                    "Tipo = \"" + Cuenta.getTipo().getTipo() + "\"," +
                    "DNI = \"" + Cuenta.getPersona().getDNI() +
                    "\" WHERE Usuario = \"" + getUsuario() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeCuentaBD: Metodo que borra una Cuenta de la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeCuentaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Cuenta WHERE Usuario = \"" + getUsuario() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /** checkPass: Metodo que comprueba si el password pasado como parametro corresponde a la cuenta
         *
         * @param   String  Pass    Password a comprobar
         * @return  String          Tipo de cuenta al que corresponde el usuario-password o "Inexistente" en caso de no existir
         */
        public String checkPass(String Pass) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Tipo FROM Cuenta WHERE Usuario = \"" + getUsuario() + "\" AND Pass = \"" + Pass + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Si el usuario existe -> Devolvemos su tipo
            if (rs.next())          return rs.getString(1);
            // Si el usuario no existe -> Devolvemos null
            else                    return "Inexistente";
        }

        /* obtenerCuentas: Metodo que devuelve los datos de las cuentas existentes en el sistema
         *
         * @param   void
         * @return  ResultSet   rs          Datos de las cuentas existentes en el sistema
         */
        public ResultSet obtenerCuentas() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI, Tipo, Usuario, '******' Contraseña FROM Cuenta ORDER BY DNI;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerDNIsAdmAut: Metodo que devuelve los DNIs de todos los administradores y autorizadores
         *
         * @param   void
         * @return  ResultSet   rs          DNIs de todos los administradores y autorizadores
         */
        public ResultSet obtenerDNIsAdmAut() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT 'Ninguno' UNION SELECT DNI FROM Cuenta WHERE Tipo IN ('ADMINISTRADOR','AUTORIZADOR');";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

}