// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Departamento_I;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;



/** CLASE DEPARTAMENTO:  Clase que representa un departamento perteneciente a una empresa
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Departamento extends UnaFecha implements Departamento_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cod_Dep:                                String con el codigo de departamento */
	private String                              Cod_Dep;
        /** Nombre:                                 String con el nombre del departamento */
        private String                              Nombre;
        /** Acceso:                                 Tipo de acceso al centro que posee el personal del departamento */
        private T_Acceso                            Acceso;
        /** Periodicidad:                           Periodicidad con la que el departamento actualizar la lista de su personal accesor al centro */
        private int                                 Periodicidad;
        /** Autorizador:                            Persona autorizadora y responsable del departamento */
        private Residente                           Autorizador;
        /** Personal:                               Personal relacionado con el departamento */
        private HashMap<String,Personal>            Personal;
        /** Llaves:                                 Llaves que pueden solicitar */
        private HashMap<String,Llave>               Llaves;

        /** UBD:                                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD                  UBD = new UtilidadesBD();
        /** UCAL:                                   Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar            UCAL = new UtilidadesCalendar();

        
	
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Departamento      Objeto de tipo Departamento
         */
        public Departamento () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Departamento con los valores pasados parametricamente
	 *
         * @param   Empresa         Empresa a la que pertenece el departamento
         * @param   Cod_Dep         String con el codigo de departamento
         * @param   Nombre          String con el nombre del departamento
         * @param   Acceso          Tipo de acceso al centro que posee el personal del departamento
         * @param   Periodicidad    Periodicidad con la que el departamento debe actualizar la lista de su personal accesor al centro
         * @param   F_Ult_Actu      Fecha de la ultima actualización de personal realizada por el departamento
         * @param   Autorizador     Persona autorizadora y responsable del departamento
         * @param   Personal        Personal relacionado con el departamento
	 * @return  Departamento    Objeto de tipo Departamento
	 */
	public Departamento (String Cod_Dep, String Nombre, T_Acceso Acceso, int Periodicidad, Calendar F_Ult_Actu, Residente Autorizador, HashMap<String,Personal> Personal) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(F_Ult_Actu);

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Nombre == null) || (Acceso == null) ||
                    (Cod_Dep.length()>4) || (Cod_Dep.equals("")) || (Nombre.length()>50) || (Periodicidad<0) || (Periodicidad>999)) throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Dep = Cod_Dep.toUpperCase();
            this.Nombre = Nombre;
            this.Acceso = Acceso;
            this.Periodicidad = Periodicidad;
            this.Autorizador = Autorizador;
            this.Personal = Personal;
            this.Llaves = new HashMap();
	}
	/** Constructor: Constructor de clase que crea el objeto Departamento con los valores pasados parametricamente
	 *
         * @param   Empresa         Empresa a la que pertenece el departamento
         * @param   Cod_Dep         String con el codigo de departamento
         * @param   Nombre          String con el nombre del departamento
         * @param   Acceso          Tipo de acceso al centro que posee el personal del departamento
         * @param   Periodicidad    Periodicidad con la que el departamento debe actualizar la lista de su personal accesor al centro
         * @param   F_Ult_Actu      Fecha de la ultima actualización de personal realizada por el departamento
         * @param   Autorizador     Persona autorizadora y responsable del departamento
         * @param   Personal        Personal relacionado con el departamento
         * @param   Llaves          Llaves que pueden solicitar
	 * @return  Departamento    Objeto de tipo Departamento
	 */
	public Departamento (String Cod_Dep, String Nombre, T_Acceso Acceso, int Periodicidad, Calendar F_Ult_Actu, Residente Autorizador, HashMap<String,Personal> Personal, HashMap<String,Llave> Llaves) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(F_Ult_Actu);

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Nombre == null) || (Acceso == null) ||
                    (Cod_Dep.length()>4) ||
                    (Nombre.length()>50) || (Periodicidad<0) || (Periodicidad>999)) throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Dep = Cod_Dep.toUpperCase();
            this.Nombre = Nombre;
            this.Acceso = Acceso;
            this.Periodicidad = Periodicidad;
            this.Autorizador = Autorizador;
            this.Personal = Personal;
            this.Llaves = Llaves;
	}

	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodDep: Metodo que obtiene el valor del atributo Cod_Dep
	 *
	 * @param   void
	 * @return  String	String con el codigo de departamento
	 */
	public String getCodDep()                       { return this.Cod_Dep; }
        /** setCodDep: Metodo que establece el valor del atributo Cod_Dep
	 *
	 * @param   Cod_Dep     String     String con el codigo de departamento
	 * @return  void
	 */
	public void setCodDep(String Cod_Dep) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Cod_Dep.length()>4))  throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Dep = Cod_Dep.toUpperCase();
        }

        /** getNombre: Metodo que obtiene el valor del atributo Nombre
	 *
	 * @param   void
	 * @return  String	String con el nombre del departamento
	 */
	public String getNombre()			{ return this.Nombre; }
        /** setNombre: Metodo que establece el valor del atributo Nombre
	 *
	 * @param   Nombre     String     String con el nombre del departamento
	 * @return  void
	 */
	public void setNombre(String Nombre) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Nombre == null) || (Nombre.length()>50))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Nombre = Nombre;
        }

        /** getAcceso: Metodo que obtiene el valor del atributo Acceso
	 *
	 * @param   void
	 * @return  T_Acceso	Tipo de acceso al centro que posee el personal del departamento
	 */
	public T_Acceso getAcceso()			{ return this.Acceso; }
        /** setAcceso: Metodo que establece el valor del atributo Acceso
	 *
	 * @param   Acceso     T_Acceso     Tipo de acceso al centro que posee el personal del departamento
	 * @return  void
	 */
	public void setAcceso(T_Acceso Acceso)           { this.Acceso = Acceso; }

        /** getPeriodicidad: Metodo que obtiene el valor del atributo Periodicidad
	 *
	 * @param   void
	 * @return  int     Periodicidad con la que el departamento debe actualizar la lista de su personal accesor al centro
	 */
	public int getPeriodicidad()			{ return this.Periodicidad; }
        /** setPeriodicidad: Metodo que establece el valor del atributo Periodicidad
	 *
	 * @param   Periodicidad     int     Periodicidad con la que el departamento debe actualizar la lista de su personal accesor al centro
	 * @return  void
	 */
	public void setPeriodicidad(int Periodicidad) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Periodicidad<0) || (Periodicidad>999)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Periodicidad = Periodicidad;
        }

        /** getAutorizador: Metodo que obtiene el valor del atributo Autorizador
	 *
	 * @param   void
	 * @return  Residente	Persona autorizadora y responsable del departamento
	 */
	public Residente getAutorizador()               { return this.Autorizador; }
        /** setAutorizador: Metodo que establece el valor del atributo Autorizador
	 *
	 * @param   Autorizador     Residente     Persona autorizadora y responsable del departamento
	 * @return  void
	 */
	public void setAutorizador(Residente Autorizador)   { this.Autorizador = Autorizador; }

        /** getPersonal: Metodo que obtiene el valor del atributo Personal
	 *
	 * @param   void
	 * @return  HashMap<String,Personal>	Personal relacionado con el departamento
	 */
	public HashMap<String,Personal> getPersonal()     { return this.Personal; }
        /** setPersonal: Metodo que establece el valor del atributo Personal
	 *
	 * @param   Personal     HashMap<String,Personal>     Personal relacionado con el departamento
	 * @return  void
	 */
	public void setPersonal(HashMap<String,Personal> Personal) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Personal == null) || (Personal.isEmpty())) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Personal = Personal;
        }
        /** addPersona: Metodo que añade una persona a las ya existentes
         *
         * @param   Persona        Personal  Persona a insertar en el departamento
         * @return  void
         */
        public void addPersona(Personal Persona) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona == null)                            throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getPersonal().put(Persona.getDNI(),Persona);
        }
        /** updatePersona: Metodo que modifica una persona existente
         *
         * @param   Pers_old,Pers_new        Personal  Personas a modificar y por la que reemplazar
         * @return  void
         */
        public void updatePersona(Personal Pers_old, Personal Pers_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Pers_old == null) || (Pers_new == null))           throw new DataException();

            // Comprobamos si existe la persona, en cuyo caso la cambiamos por la nueva
            if (getPersonal().containsKey(Pers_old.getDNI()))
            {
                removePersona(Pers_old);
                addPersona(Pers_new);
            }
        }
        /** removePersona: Metodo que elimina una persona de las existentes
         *
         * @param   Persona        Personal  Persona a eliminar
         * @return  void
         */
        public void removePersona(Personal Persona) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona == null)                            throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getPersonal().remove(Persona.getDNI());
        }
        /** getPersona: Metodo que obtiene una persona de las existentes
         *
         * @param   String              DNI         DNI de la persona a obtener
         * @return  Personal                Persona solicitada
         */
        public Personal getPersona(String DNI) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (DNI.length()>9))                  throw new DataException();
            // Devolvemos el resultado
            return this.getPersonal().get(DNI);
        }

        /** getLlaves: Metodo que obtiene el valor del atributo Llaves
	 *
	 * @param   void
	 * @return  HashMapa	Llaves prestables al Departamento
	 */
	public HashMap<String,Llave> getLlaves()                      { return this.Llaves; }
        /** setLlaves: Metodo que establece el valor del atributo Llaves
	 *
	 * @param   Llaves     HashMap     Llaves prestables al Departamento
	 * @return  void
	 */
	public void setLlaves(HashMap<String,Llave> Llaves)
        {
            // Realizamos la asignacion de valores a los atributos
            this.Llaves = Llaves;
        }
        /** addLlave: Metodo que añade una llave a las ya existentes
         *
         * @param   Llave        Llave  Llave nueva a añadir
         * @return  void
         */
        public void addLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().put("L - " + Llave.getCodigo() + " - " + calendarToString(Llave.getFechaI()),Llave);
        }
        /** updateLlave: Metodo que actualiza una llave existente
         *
         * @param   Llave_old       Llave       Llave a modificar
         * @param   Llave_new       Llave       Llave por la que reemplazar
         * @return  void
         */
        public void updateLlave(Llave Llave_old, Llave Llave_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llave_old == null) || (Llave_new == null))   throw new DataException();
            // Reemplazamos la zona vieja por la nueva
            if (getLlaves().containsKey("L - " + Llave_old.getCodigo() + " - " + calendarToString(Llave_old.getFechaI())));
            {
                removeLlave(Llave_old);
                addLlave(Llave_new);
            }
        }
        /** removeLlave: Metodo que elimina una llave de las existentes
         *
         * @param   Llave        Llave  Llave a eliminar
         * @return  void
         */
        public void removeLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().remove("L - " + Llave.getCodigo() + " - " + calendarToString(Llave.getFechaI()));
        }
        /** getLlave: Metodo que obtiene una Llave de las existentes
         *
         * @param   String          Key_Llave   Key de la Llave a obtener
         * @return  Llave                       Llave solicitada
         */
        public Llave getLlave(String Key_Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Key_Llave == null) || (Key_Llave.length()>29))     throw new DataException();
            // Devolvemos el resultado
            return this.getLlaves().get(Key_Llave);
        }

        
        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // String con el personal que conforman la empresa
            String pers = "";

            // Añadimos las Strings con todo el personal
            Iterator it = getPersonal().entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKPersona = (Map.Entry)it.next();
                Object o = (Object) VKPersona.getValue();
                
                // En caso de tratarse de un trabajador periodico o contacto
                if (o instanceof Periodico_Contacto)
                {
                    Periodico_Contacto per_contac = (Periodico_Contacto) o;
                    pers = pers + per_contac.toString();
                }
                // En caso de tratarse de un vigilante
                else
                {
                    Vigilante vig = (Vigilante) o;
                    pers = pers + vig.toString();
                }
            }

            // Devolvemos el resultado
            return "DEPARTAMENTO:\nCod_Dep: " + getCodDep() + "\n" +
            "Nombre: " + getNombre() + "\n" +
            (getAcceso() == null ? "" : getAcceso().toString()) + "\n" +
            "Periodicidad: " + getPeriodicidad() + "\n" +
            super.toString() + "\n" +
            "Autorizador: " + (getAutorizador() == null ? "" : getAutorizador().toString()) + "\n" +
            pers + "\n";
	}


        // FUNCIONALIDADES
        /* obtenerDatosRenovacionPersonal: Metodo que obtiene los datos de todos los departamentos cuyo personal debe
         * ser actualizado en un periodo un dia dado
         *
         * @param   int         Dias        Dias en los que caduca el personal del departamento
         * @return  ResultSet   rs          Datos de los departamentos afectados por la actualizacion del personal
         */
        public ResultSet obtenerDatosRenovacionPersonal(int Dias) throws SQLException
        {
            // Creamos la String a ejecutar
            String query = "SELECT Email, Emp.Nombre, D.Nombre, P.Nombre, P.Apellido1," +
                                " ADDDATE(F_Ult_Actu,INTERVAL Periodicidad DAY) F_Caducidad" +
                            " FROM Empresa Emp JOIN Departamento D USING (Cod_Emp) JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                            " JOIN Cargo USING (Cod_Cargo) JOIN Personal P ON (C.DNI=P.DNI) JOIN Email Ema ON (C.DNI=Ema.DNI)" +
                            " WHERE SUBDATE(ADDDATE(DATE(F_Ult_Actu),INTERVAL Periodicidad DAY),INTERVAL " + Dias + " DAY)=CURDATE()" +
                            " AND (Descripcion = \"Responsable de personal\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* eliminarDepartamentos: Metodo que borra todos los departamentos que no hayan actualizado su personals egun lo establecido
         *
         * @return  int     Numero de departamentos eliminados
         */
        public int eliminarDepartamentos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Departamento WHERE ADDDATE(DATE(F_Ult_Actu),INTERVAL Periodicidad DAY)<CURDATE();";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
}