// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Destinatario_I;
import java.rmi.*;



/** CLASE DESTINATARIO:  Clase que representa una relacion de una nota con uno de sus destinatarios
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Destinatario implements Destinatario_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Receptor:               Receptor al que va dirigida la nota */
        private Personal            Receptor;
        /** Leida:                  Denota si la nota ha sido leida por el receptor */
        private boolean             Leida;
        /** Descartada:             Denota si la nota ha sido descartada por el receptor */
        private boolean             Descartada;



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor: Constructor de clase que crea el objeto Destinatario con los valores pasados parametricamente
	 *
         * @param   Receptor            Receptor al que va dirigida la nota
	 * @return  Destinatario	Objeto de tipo Destinatario
	 */
	public Destinatario (Personal Receptor) throws DataException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Receptor == null)                   throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Receptor = Receptor;
            this.Leida = false;
            this.Descartada = false;

	}
        /** Constructor: Constructor de clase que crea el objeto Destinatario con los valores pasados parametricamente
	 *
         * @param   Receptor            Receptor al que va dirigida la nota
         * @param   Leida               Denota si la nota ha sido leida por el receptor
         * @param   Descartada          Denota si la nota ha sido descartada por el receptor
	 * @return  Destinatario	Objeto de tipo Destinatario
	 */
	public Destinatario (Personal Receptor, boolean Leida, boolean Descartada) throws DataException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Receptor == null)                   throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Receptor = Receptor;
            this.Leida = Leida;
            this.Descartada = Descartada;
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getReceptor: Metodo que obtiene el valor del atributo Receptor
	 *
	 * @param   void
	 * @return  Persona	Receptor al que va dirigida la nota
	 */
	public Personal getReceptor()                    { return this.Receptor; }
        /** setReceptor: Metodo que establece el valor del atributo Receptor
	 *
	 * @param   Receptor     Persona     Receptor al que va dirigida la nota
	 * @return  void
	 */
	public void setReceptor(Personal Receptor) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Receptor == null)                       throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Receptor = Receptor;
        }

        /** getLeida: Metodo que obtiene el valor del atributo Leida
	 *
	 * @param   void
	 * @return  boolean	Denota si la nota ha sido leida por el destinatario
	 */
	public boolean getLeida()			{ return this.Leida; }
        /** setLeida: Metodo que establece el valor del atributo Leida
	 *
	 * @param   Leida     boolean     Denota si la nota ha sido leida por el destinatario
	 * @return  void
	 */
	public void setLeida(boolean Leida)             { this.Leida = Leida; }

        /** getDescartada: Metodo que obtiene el valor del atributo Descartada
	 *
	 * @param   void
	 * @return  boolean	Denota si la nota ha sido descartada por el destinatario
	 */
	public boolean getDescartada()			{ return this.Descartada; }
        /** setDescartada: Metodo que establece el valor del atributo Descartada
	 *
	 * @param   Descartada     boolean     Denota si la nota ha sido descartada por el destinatario
	 * @return  void
	 */
	public void setDescartada(boolean Descartada)   { this.Descartada = Descartada; }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return (getReceptor() == null ? "" : getReceptor().toString()) + "\n" +
            "Leida: " + (getLeida() ? "Sí" : "No") + "\n" +
            "Descartada: " + (getDescartada() ? "Sí" : "No") + "\n";
	}
}