// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Empresa_I;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;



/** CLASE EMPRESA:  Clase que representa una empresa
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Empresa implements Empresa_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cod_Emp:                            String con el codigo de empresa */
	private String                          Cod_Emp;
        /** CIF:                                String con el CIF de la empresa */
	private String                          CIF;
        /** Nombre:                             String con el nombre de la empresa */
        private String                          Nombre;
        /** Direccion:                          String con la direccion de la empresa */
        private String                          Direccion;
        /** Departamentos:                      HashMap con los departamentos que conforman la empresa */
        private HashMap<String,Departamento>    Departamentos;

        /** UBD:                                Objeto de tipo UtilidadesBD */
        public static UtilidadesBD              UBD = new UtilidadesBD();
        /** UCAL:                               Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar        UCAL = new UtilidadesCalendar();


	
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Acceso      Objeto de tipo Empresa
         */
        public Empresa () throws DataException, RemoteException {}
        /** Constructor: Constructor de clase que crea el objeto Empresa con los valores pasados parametricamente
	 *
	 * @param   Cod_Emp         String con el codigo de empresa
         * @param   CIF             String con el CIF de la empresa
         * @param   Nombre          String con el nombre de la empresa
         * @param   Direccion       String con la direccion de la empresa
         * @param   Departamentos   HashMap con los departamentos que conforman la empresa
	 * @return  Empresa         Objeto de tipo Empresa
	 */
	public Empresa (String Cod_Emp, String CIF, String Nombre, String Direccion, HashMap<String,Departamento> Departamentos) throws DataException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Emp == null) || (CIF == null) || (Nombre == null) || (Direccion == null) ||
                    (Departamentos == null) || (Cod_Emp.length()>4) || (Cod_Emp.equals("")) ||
                    (CIF.length()>9) || (Nombre.length()>50) || (Direccion.length()>100))    throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Emp = Cod_Emp.toUpperCase();
            this.CIF = CIF;
            this.Nombre = Nombre;
            this.Direccion = Direccion;
            this.Departamentos = Departamentos;
	}
	/** Constructor: Constructor de clase que crea el objeto Empresa con los valores pasados parametricamente
	 *
	 * @param   Cadena           String con el codigo o nombre de empresa
         * @param   Codigo          Denota si el valor pasado es el codigo, o por el contrario es el nombre de empresa
	 * @return  Empresa         Objeto de tipo Empresa
	 */
	public Empresa (String Cadena, boolean Codigo) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cadena == null) || ((Cadena.length()>4) && (Codigo)) || ((Cadena.length()>50) && (!Codigo)))   throw new DataException();

            String query;
            ResultSet rs;
            // Si se le ha pasado el codigo de empresa
            if (Codigo)
            {
                // Obtenemos los valores de la tupla existente
                query = "SELECT * FROM Empresa WHERE Cod_Emp = \"" + Cadena + "\";";
                rs = UBD.ejecutarQuery(query);

                // Realizamos la asignacion de valores a los atributos
                rs.next();
                this.Cod_Emp = rs.getString(1);
                this.CIF = rs.getString(2);
                this.Nombre = rs.getString(3);
                this.Direccion = rs.getString(4);
                this.Departamentos = getDepartamentosBD();
            }
            // Si lo que se le pasa es el nombre de empresa
            else
            {
                // Creamos la String con la sentencia a realizar
                query = "SELECT * FROM Empresa WHERE Nombre = \"" + Cadena + "\";";

                // Ejecutamos la consulta
                rs = UBD.ejecutarQuery(query);

                // Establecemos los valores de los atributos
                rs.next();
                this.Cod_Emp = rs.getString(1);
                this.CIF = rs.getString(2);
                this.Nombre = rs.getString(3);
                this.Direccion = rs.getString(4);
                this.Departamentos = getDepartamentosBD();
            }

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodEmp: Metodo que obtiene el valor del atributo Cod_Emp
	 *
	 * @param   void
	 * @return  String	String con el codigo de empresa
	 */
	public String getCodEmp()			{ return this.Cod_Emp; }
        /** setCodEmp: Metodo que establece el valor del atributo Cod_Emp
	 *
	 * @param   Cod_Emp     String     String con el codigo de empresa
	 * @return  void
	 */
	public void setCodEmp(String Cod_Emp) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Emp == null) || (Cod_Emp.length()>4))  throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Emp = Cod_Emp.toUpperCase();
        }

        /** getCIF: Metodo que obtiene el valor del atributo CIF
	 *
	 * @param   void
	 * @return  String	String con el CIF de la empresa
	 */
	public String getCIF()                          { return this.CIF; }
        /** setCIF: Metodo que establece el valor del atributo CIF
	 *
	 * @param   CIF     String     String con el CIF de la empresa
	 * @return  void
	 */
	public void setCIF(String CIF) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((CIF == null) || (CIF.length()>9))      throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.CIF = CIF;
        }

        /** getNombre: Metodo que obtiene el valor del atributo Nombre
	 *
	 * @param   void
	 * @return  String	String con el nombre de la empresa
	 */
	public String getNombre()			{ return this.Nombre; }
        /** setNombre: Metodo que establece el valor del atributo Nombre
	 *
	 * @param   Nombre     String     String con el nombre de la empresa
	 * @return  void
	 */
	public void setNombre(String Nombre) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Nombre == null) || (Nombre.length()>50))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Nombre = Nombre;
        }

        /** getDireccion: Metodo que obtiene el valor del atributo Direccion
	 *
	 * @param   void
	 * @return  String	String con la direccion de la empresa
	 */
	public String getDireccion()			{ return this.Direccion; }
        /** setDireccion: Metodo que establece el valor del atributo Direccion
	 *
	 * @param   Direccion     String     String con la direccion de la empresa
	 * @return  void
	 */
	public void setDireccion(String Direccion) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Direccion == null) || (Direccion.length()>100))    throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Direccion = Direccion;
        }

        /** getDepartamentos: Metodo que obtiene el valor del atributo Departamentos
	 *
	 * @param   void
	 * @return  HashMap<String,Departamento>	Departamentos que conforman la empresa
	 */
	public HashMap<String,Departamento> getDepartamentos()   { return this.Departamentos; }
        /** setDepartamentos: Metodo que establece el valor del atributo Departamentos
	 *
	 * @param   Departamentos     HashMap<String,Departamento>     Departamentos que conforman la empresa
	 * @return  void
	 */
	public void setDepartamentos(HashMap<String,Departamento> Departamentos) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamentos == null) || (Departamentos.isEmpty()))       throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Departamentos = Departamentos;
        }
        /** addDepartamento: Metodo que añade un departamento a los ya existentes
         *
         * @param   Departamento        Departamento  Departamento nuevo para el que trabaja
         * @return  void
         */
        public void addDepartamento(Departamento Departamento) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                       throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getDepartamentos().put(Departamento.getCodDep(),Departamento);
        }
        /** updateDepartamento: Metodo que modifica un departamento existente
         *
         * @param   Dep_old,Dep_new        Destinatario  Departamentos a modificar y por el que reemplazar
         * @return  void
         */
        public void updateDepartamento(Departamento Dep_old, Departamento Dep_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Dep_old == null) || (Dep_new == null))             throw new DataException();

            // Comprobamos si existe el departamento, en cuyo caso lo cambiamos por el nuevo
            if (getDepartamentos().containsKey(Dep_old.getCodDep()))
            {
                removeDepartamento(Dep_old);
                addDepartamento(Dep_new);
            }
        }
        /** removeDepartamento: Metodo que elimina un departamento de los existentes
         *
         * @param   Departamento        Departamento  Departamento a eliminar
         * @return  void
         */
        public void removeDepartamento(Departamento Departamento) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                       throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getDepartamentos().remove(Departamento.getCodDep());
        }
        /** getDepartamento: Metodo que obtiene un departamento de los existentes
         *
         * @param   String          Cod_Dep     Codigo del departamento a obtener
         * @return  Departammento               Departamento solicitado
         */
        public Departamento getDepartamento(String Cod_Dep) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Cod_Dep.length()>4))          throw new DataException();
            // Devolvemos el resultado
            return this.getDepartamentos().get(Cod_Dep);
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{   
            // String con los departamentos que conforman la empresa
            String deps = "";

            // Añadimos las Strings con todos los departamentos
            Iterator it = getDepartamentos().entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKDep = (Map.Entry)it.next();
                Departamento dep = (Departamento) VKDep.getValue();
                deps = deps + dep.toString();
            }

            // Devolvemos el resultado
            return "EMPRESA:\nCod_Emp: " + getCodEmp() + "\n" +
            "CIF: " + getCIF() + "\n" +
            "Nombre: " + getNombre() + "\n\n" +
            deps + "\n" +
            "Direccion: " + getDireccion() + "\n";
	}


        // DML
        /** insertLlaveBD: Metodo que inserta una Llave entre las prestables a un Departamento de la Empresa
         *
         * @param   Departamento    Departamento    Departamento al que se le puede prestar la llave
         * @param   Llave           Llave           Llave prestable
         * @return  int                             Numero de filas insertadas
         */
        public int insertLlaveBD(Departamento Departamento, Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Llave == null))                  throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Prestamo_Departamento VALUES (" +
                Llave.getCodigo() + ",\"" +
                UCAL.calendarToString(Llave.getFechaI()) + "\",\"" +
                getCodEmp() + "\",\"" +
                Departamento.getCodDep() + "\");";

                // Ejecutamos la consulta y devolvemos el resultado
                return UBD.ejecutarUpdate(query);
        }
        /** updateLlaveBD: Metodo que actualiza una llave prestable a un departamento de la empresa
         *
         * @param   Departamento    Departamento            Departamento al que se le puede prestar la llave
         * @param   Llave_old       Llave a modificar
         * @param   Llave_new       Llave de reemplazo
         * @return  int             Resultado
         */
        public int updateLlaveBD(Departamento Departamento, Llave Llave_old, Llave Llave_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Llave_old == null) || (Llave_new == null))         throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Prestamo_Departamento SET " +
                "Cod_Llave = " + Llave_new.getCodigo() + "," +
                "F_Alta_L = \"" + UCAL.calendarToString(Llave_new.getFechaI()) + "\"," +
                "Cod_Emp = \"" + getCodEmp() + "\"," +
                "Cod_Dep = \"" + Departamento.getCodDep() + "\"" +
                " WHERE Cod_Llave = " + Llave_old.getCodigo() + " AND F_Alta_L = \"" +
                UCAL.calendarToString(Llave_old.getFechaI()) + "\" AND Cod_Emp = \"" + getCodEmp() + "\" AND" +
                " Cod_Dep = \"" + Departamento.getCodDep() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeLlaveBD: Metodo que borra una llave prestable a un Departamento de la Empresa
         *
         * @param   Departamento    Departamento            Departamento al que se le puede prestar la llave
         * @param   Llave           Llave                   Llave a eliminar
         * @return  int                                     Numero de filas eliminadas
         */
        public int removeLlaveBD(Departamento Departamento, Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Llave == null))              throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Prestamo_Departamento  WHERE Cod_Llave = " + Llave.getCodigo() +
                        " AND F_Alta_L = \"" + UCAL.calendarToString(Llave.getFechaI()) + "\" AND" +
                        " Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** insertLlavesBD: Metodo que inserta las Llaves prestables a un Departamento de la Empresa
         *
         * @param   Departamento    Departamento    Departamento del que insertar sus llaves prestables
         * @return  int                             Numero de filas insertadas
         */
        public int insertLlavesBD(Departamento Departamento) throws DataException, SQLException
        {
            // Obtenemos las llaves
            HashMap<String,Llave> Llaves = Departamento.getLlaves();

            // Insertamos cada Llave
            Iterator it = Llaves.entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos la llave
                Map.Entry VKLlaves = (Map.Entry) it.next();
                Llave ll = (Llave) VKLlaves.getValue();

                // Insertamos la llave
                insertLlaveBD (Departamento,ll);
            }

            return Llaves.size();
        }
        /** updateLlavesBD: Metodo que actualiza las Llaves prestables a un Departamento de la Empresa
         *
         * @param   Departamento        Departamento        Departamento al que se le puede prestar las llaves
         * @return  int                                     Numero de filas (Llaves) actualizadas
         */
        public int updateLlavesBD(Departamento Departamento) throws DataException, SQLException, RemoteException
        {
            // Filas actualizadas
            int res = 0;

            // Recorremos el HashMap nuevo, actualizando las Llaves existentes e insertando las nuevas
            Iterator it_new = Departamento.getLlaves().entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_new.next();
                Llave llave_new = (Llave) VKLlaves.getValue();
                String llave_key = (String) VKLlaves.getKey();

                // Si existe la llave -> La actualizamos
                if (Departamento.getLlaves().containsKey(llave_key))        updateLlaveBD(Departamento,Departamento.getLlaves().get(llave_key),llave_new);
                // Si no existe -> La insertamos
                else                                                        insertLlaveBD(Departamento,llave_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellas Llaves que no consten en el nuevo
            HashMap<String,Llave> Llaves_actu = getLlavesBD(Departamento.getCodDep());
            Iterator it_old = Llaves_actu.entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_old.next();
                Llave llave_old = (Llave) VKLlaves.getValue();
                String llave_key = (String) VKLlaves.getKey();

                // Si la Llave vieja no existe como nueva -> La eliminamos y actualizamos el contador de filas actualizadas
                if (!Departamento.getLlaves().containsKey(llave_key))
                {
                    removeLlaveBD(Departamento,llave_old);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removeLlavesBD: Metodo que borra las Llaves prestables a un Departamento de la Empresa
         *
         * @param   Departamento    Departamento    Departamento del que eliminar todas sus llaves prestables
         * @return  int                             Numero de filas (Llaves) eliminadas
         */
        public int removeLlavesBD(Departamento Departamento) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                               throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Prestamo_Departamento WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getLlavesBD: Metodo que obtiene todas las llaves prestables a un Departamento dado de la Empresa
         *
         * @param   String                      Cod_Dep        Codigo de departamento al que pertenecen las llaves prestables
         * @return  HashMap<Integer,Llave>      Llaves         HashMap con todas las llaves prestables
         */
        public HashMap<String,Llave> getLlavesBD (String Cod_Dep) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Cod_Dep.length()>4))              throw new DataException();

           // Creamos el HashMap resultado
           HashMap<String,Llave> Llaves = new HashMap();

           // Introducimos las llaves
           String query = "SELECT Cod_Llave,F_Alta_L FROM Prestamo_Departamento WHERE Cod_Emp = \"" +
                   getCodEmp() + "\" AND Cod_Dep = \"" + Cod_Dep + "\";";
           ResultSet rs = UBD.ejecutarQuery(query);

            while (rs.next())
            {
                // Creamos la llave y la introduciomos
                Llave ll = new Llave(rs.getInt(1),rs.getString(2).substring(0,19));
                Llaves.put("L - " + ll.getCodigo() + " - " + UCAL.calendarToString(ll.getFechaI()),ll);
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Llaves;
        }

        /** insertPersonaCargoBD: Metodo que inserta una relacion Persona-Cargo en la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @param   Persona         Periodico_Contacto  Persona periodica
         * @param   Cargo           Cargo               Cargo desempeñado
         * @return  int                                 Numero de filas insertadas
         */
        public int insertPersonaCargoBD(Departamento Departamento, Periodico_Contacto Persona, Cargo Cargo) throws DataException, SQLException
        {
        // Comprobamos que los parametros pasados son correctos
        if ((Departamento == null) || (Persona == null) || (Cargo == null))     throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Contacto VALUES (\""  +
                Persona.getDNI() + "\",\"" +
                getCodEmp() + "\",\"" +
                Departamento.getCodDep() + "\",\"" +
                Cargo.getCodCargo() + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updatePersonaCargoBD: Metodo que modifica una relacion Persona-Cargo en la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @param   Persona         Periodico_Contacto  Persona periodica
         * @param   Cargo_old       Cargo               Cargo que desempeñaba
         * @param   Cargo_new       Cargo               Cargo de reemplazo
         * @return  int                                 Numero de filas actualizadas
         */
        public int updatePersonaCargoBD(Departamento Departamento, Periodico_Contacto Persona, Cargo Cargo_old, Cargo Cargo_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null) || (Cargo_old == null) || (Cargo_new == null))     throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Contacto SET "  +
                "Cod_Cargo = \"" + Cargo_new.getCodCargo() +
                "\" WHERE DNI = " + Persona.getDNI() + " AND Cod_Emp = " + getCodEmp() + " AND Cod_Dep = " + Departamento.getCodDep() + ";" ;

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removePersonaCargoBD: Metodo que borra una relacion Persona-Cargo en la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @param   Persona         Periodico_Contacto  Persona periodica
         * @param   Cargo           Cargo               Cargo a borrar
         * @return  int                                 Numero de filas eliminadas
         */
        public int removePersonaCargoBD(Departamento Departamento, Periodico_Contacto Persona, Cargo Cargo) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null) || (Cargo == null))     throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Contacto WHERE DNI = \"" + Persona.getDNI() + "\" AND Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\" AND Cod_Cargo = \"" + Cargo.getCodCargo() + "\";" ;

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** insertPersonaCargosBD: Metodo que inserta todos los cargos de una persona en la Empresa en la BD
         *
         * @param   Departamento        Departamento        Departamento en el que trabaja
         * @param   Periodico_Contacto  Persona             Persona
         * @return  int                                     Numero de filas insertadas
         */
        public int insertPersonaCargosBD(Departamento Departamento, Periodico_Contacto Persona) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null))                    throw new DataException();

            // Obtenemos los cargos que desempeña
            HashMap<String,Cargo> Cargos = Persona.getCargos();

            // Insertamos cada Cargo
            Iterator it = Cargos.entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos la llave
                Map.Entry VKCargos = (Map.Entry) it.next();
                Cargo Cargo = (Cargo) VKCargos.getValue();

                // Insertamos el Cargo
                insertPersonaCargoBD (Departamento,Persona,Cargo);
            }

            return Cargos.size();
        }
        /** updatePersonaCargosBD: Metodo que actualiza los Cargos desempeñados por una persona en el Departamento
         *
         * @param   Departamento            Departamento    Departamento en el que trabaja
         * @param   Periodico_Contacto      Persona         Persona a actualizar
         * @param   HashMap<String,Cargo>   Cargos          HashMap de cargos con los que actualizar
         * @return  int                                     Numero de filas (Cargos) actualizadas
         */
        public int updatePersonaCargosBD(Departamento Departamento, Periodico_Contacto Persona, HashMap<String,Cargo> Cargos) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null) || (Cargos == null))   throw new DataException();

            // Filas actualizadas
            int res = 0;

            // Recorremos el HashMap nuevo, dejando los cargos existentes e insertando los nuevos
            Iterator it_new = Cargos.entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKCargos = (Map.Entry) it_new.next();
                Cargo cargo_new = (Cargo) VKCargos.getValue();
                String cargo_key = (String) VKCargos.getKey();

                // Si no existe el Cargo -> Lo insertamos y lo añadimos al HashMap
                if (!Persona.getCargos().containsKey(cargo_key))
                {
                    insertPersonaCargoBD(Departamento,Persona,Cargos.get(cargo_key));
                    Persona.getCargos().put(cargo_key, cargo_new);
                }

                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellos Cargos que no consten en el nuevo
            Iterator it_old = Persona.getCargos().entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKCargos = (Map.Entry) it_old.next();
                Cargo cargo_old = (Cargo) VKCargos.getValue();
                String cargo_key = (String) VKCargos.getKey();

                // Si el Cargo viejo no existe como nuevo -> Lo eliminamos y actualizamos el contador de filas actualizadas
                if (!Cargos.containsKey(cargo_key))
                {
                    removePersonaCargoBD(Departamento,Persona,cargo_old);
                    res++;
                }
            }
            // Devolvemos el resultado
            return res;
        }
        /** removePersonaCargosBD: Metodo que borra todos los Cargos desempeñados por una persona en un departamento en la BD
         *
         * @param   Departamento        Departamento        Departamento en el que trabaja
         * @param   Periodico_Contacto  Persona             Persona de la que borrar todos sus cargos
         * @return  int                                     Numero de filas (Cargos) eliminadas
         */
        public int removePersonaCargosBD(Departamento Departamento, Periodico_Contacto Persona) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null))                    throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Contacto WHERE DNI = \"" + Persona.getDNI() +
                    "\" AND Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** insertPeriodicoContactoBD: Metodo que inserta una relacion Departamento-Persona en la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @param   Persona         Periodico_Contacto  Persona periodica a introducir en el departamento
         * @return  int                                 Numero de filas insertadas
         */
        public int insertPeriodicoContactoBD(Departamento Departamento, Periodico_Contacto Persona) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null))            throw new DataException();

            // INSERTAMOS LOS DATOS PERSONALES si no existen previamente
            Personal pers = new Personal();
            pers.setDNI(Persona.getDNI());
            int res = 0;
            if (!pers.existe())     res = Persona.insertPersonalBD();

            // INSERTAMOS LOS DATOS ESPECIFICOS
            // Si no posee cargos -> Persona periodica
            if (Persona.getCargos().isEmpty())
            {
                // Creamos la String con la sentencia a realizar
                String query = "INSERT INTO Periodico VALUES (\""  +
                Persona.getDNI() + "\",\"" +
                getCodEmp() + "\",\"" +
                Departamento.getCodDep() + "\");";

                // Ejecutamos la consulta
                UBD.ejecutarUpdate(query);
            }

            // Si se ha insertado correctamente -> INSERTAMOS LOS CARGOS QUE DESEMPEÑA
            insertPersonaCargosBD(Departamento,Persona);

            // Devolvemos el resultado
            return res;
        }

        /** removePeriodicoContactoBD: Metodo que borra una relacion Departamento-Persona en la BD
         *
         * @param   Departamento        Departamento
         * @param   Periodico_Contacto  Periodico_Contacto a eliminar
         * @return  int                 Numero de filas eliminadas
         */
        public int removePeriodicoContactoBD(Departamento Departamento, Periodico_Contacto Persona) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona == null)                                    throw new DataException();

            // Si la persona no posee cargos -> Persona periódica -> Borrar desde Personal
            if (Persona.getCargos().isEmpty())
            {
                // Creamos la String con la sentencia a realizar, ejecutamos y devolvemos el resultado
                Personal pers = (Personal) Persona;
                return pers.removePersonalBD();
            }
            // Si la persona posee cargos -> Contacto -> Borrar en Contacto
            else
            {
                // Creamos la String con la sentencia a realizar
                String query = "DELETE FROM Contacto WHERE DNI = \""  + Persona.getDNI() +
                        "\" AND Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\";";

                // Ejecutamos la consulta y devolvemos el resultado
                return UBD.ejecutarUpdate(query);
            }
        }
        /** getPeriodicoContactosBD: Metodo que obtiene todo el personal periodico y contactos de un departamento
         *
         * @param   String                              Cod_Dep             Codigo de departamento
         * @return  HashMap<String,Personal>            Personal            HashMap con todo el personal periodico y contactos relacionado con el departamento
         */
        public HashMap<String,Personal> getPeriodicoContactosBD (String Cod_Dep) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Cod_Dep.length()>4))                  throw new DataException();

           // Creamos el HashMap resultado
           HashMap<String,Personal> Personas = new HashMap();

           // Introducimos el TRABAJADOR PERIODICO
           String query = "SELECT DNI FROM Periodico WHERE Cod_Emp = \"" + getCodEmp() +
                   "\" AND Cod_Dep = \"" + Cod_Dep + "\" AND DNI NOT IN (SELECT DISTINCT DNI FROM" +
                   " Vigilante)" +
                   " UNION" +
                   " SELECT DISTINCT DNI FROM Contacto WHERE Cod_Emp = \"" + getCodEmp() + "\"" +
                   " AND Cod_Dep = \"" + Cod_Dep + "\";";
           ResultSet rs = UBD.ejecutarQuery(query);

            while (rs.next())
            {
                // Creamos el trabajador periodico
                Periodico_Contacto pc = new Periodico_Contacto(rs.getString(1));
                // Obtenemos y añadimos sus cargos
                HashMap<String,Cargo> Cargos = new HashMap();
                query = "SELECT Cod_Cargo FROM Contacto WHERE DNI = \"" + rs.getString(1) +
                        "\" AND Cod_Emp = \"" + getCodEmp() +
                       "\" AND Cod_Dep = \"" + Cod_Dep + "\";";
                ResultSet rsc = UBD.ejecutarQuery(query);
                while (rsc.next())  Cargos.put(rsc.getString(1),new Cargo(rsc.getString(1)));
                pc.setCargos(Cargos);

                // Introducimos el contacto
                Personas.put(pc.getDNI(),pc);
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Personas;
        }

        /** insertVigilanteBD: Metodo que inserta una Vigilante en la BD
         *
         * @param   Departamento    Departamento    Departamento
         * @param   Vigilante       Vigilante       Vigilante a introducir en el departamento
         * @return  int                             Numero de filas insertadas
         */
        public int insertVigilanteBD(Departamento Departamento, Vigilante Vigilante) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Vigilante == null))          throw new DataException();

            // INSERTAMOS LOS DATOS PERSONALES
            this.insertPeriodicoContactoBD(Departamento, Vigilante);

            // INSERTAMOS LOS DATOS ESPECIFICOS
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Vigilante VALUES (\""  +
                Vigilante.getDNI() + "\"," +
                Vigilante.getCodVig() + ");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        } 
        /** removeVigilanteBD: Metodo que borra un vigilante de la BD
         *
         * @param   Vigilante           Vigilante a eliminar
         * @return  int                 Numero de filas eliminadas
         */
        public int removeVigilanteBD(Vigilante Vigilante) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Vigilante == null)                                  throw new DataException();

            // Borramos el vigilante desde Personal, devolviendo el resultado
            Personal pers = (Personal) Vigilante;
            return pers.removePersonalBD();
        }
        /** getVigilantesBD: Metodo que obtiene todos los vigilantes del departamento de la BD
         *
         * @param   String                      Cod_Dep             Codigo de departamento
         * @return  HashMap<String,Personal>    Personas            HashMap con todos los vigilantes relacionados con el departamento
         */
        public HashMap<String,Personal> getVigilantesBD (String Cod_Dep) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Cod_Dep.length()>4))                  throw new DataException();

            // Creamos el HashMap resultado
            HashMap<String,Personal> Personas = new HashMap();

            // Introducimos los Vigilantes
            String query = "SELECT DNI,Cod_vig FROM Vigilante JOIN Periodico USING (DNI) WHERE Cod_Emp = \"" + getCodEmp() +
                    "\" AND Cod_Dep = \"" + Cod_Dep + "\";";

            ResultSet rs = UBD.ejecutarQuery(query);

            while (rs.next())
            {
                // Creamos el vigilante
                Vigilante vig = new Vigilante(rs.getString(1));
                // Introducimos el vigilante
                Personas.put(vig.getDNI(),vig);
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Personas;
        }

        /** insertPersonaBD: Metodo que inserta una Persona en la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @param   Persona         Personal            Persona a introducir en el departamento
         * @return  int                                 Numero de filas insertadas
         */
        public int insertPersonaBD(Departamento Departamento, Personal Persona) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null))            throw new DataException();

            // Si se trata de un vigilante
            if (Persona instanceof Vigilante)
            {
                Vigilante Vig = (Vigilante) Persona;
                return insertVigilanteBD(Departamento, Vig);
            }
            // Si se trata de un trabajador periodico
            else if (Persona instanceof Periodico_Contacto)
            {
                Periodico_Contacto Per_Contac = (Periodico_Contacto) Persona;
                return insertPeriodicoContactoBD(Departamento, Per_Contac);
            }
            else            return 0;
        }
        /** updatePersonaBD: Metodo que modifica una Persona en la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @param   Persona_old     Personal            Persona a modificar
         * @param   Persona_new     Personal            Persona de reemplazo
         * @return  int                                 Resultado
         */
        public int updatePersonaBD(Departamento Departamento, Personal Persona_old, Personal Persona_new) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona_old == null) || (Persona_new == null) ||
                    (Persona_old.getClass() != Persona_new.getClass()))           throw new DataException();
            Periodico_Contacto pc = (Periodico_Contacto) Persona_old;

            // Actualizamos los datos personales
            int res = Persona_old.updatePersonalBD(Persona_new);

            // Si la persona existe -> Actualizamos los cargos desempeñados
            if (res == 1)
            {
                // Si se trata de dos vigilantes
                if (Persona_old instanceof Vigilante)
                {
                    Vigilante vig = (Vigilante) Persona_new;
                    String query = "UPDATE Vigilante SET " +
                            "Cod_Vig = " + vig.getCodVig() +
                            " WHERE DNI = \"" + Persona_old.getDNI() + "\";";

                    // Ejecutamos la consulta
                    UBD.ejecutarUpdate(query);
                }
                // Si se trata de dos trabajadores periodicos -> Actualizamos los cargos
                else if (Persona_old instanceof Periodico_Contacto)
                {
                    Persona_old.setDNI(Persona_new.getDNI());
                    Periodico_Contacto per_contac_old = (Periodico_Contacto) Persona_old;
                    Periodico_Contacto per_contac_new = (Periodico_Contacto) Persona_new;
                    updatePersonaCargosBD(Departamento,per_contac_old,per_contac_new.getCargos());
                }
                return 1;
            }
            // En cualquier otro caso, no se actualiza nada
            else                return 0;
        }
        /** removePersonaBD: Metodo que borra una Persona de la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @param   Persona         Personal            Persona a borrar del departamento
         * @return  int                                 Numero de filas eliminadas
         */
        public int removePersonaBD(Departamento Departamento, Personal Persona) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Departamento == null) || (Persona == null))            throw new DataException();

            // En caso de tratarse de un vigilante
            if (Persona instanceof Vigilante)                   return removeVigilanteBD((Vigilante)Persona);
            // En caso de tratarse de un Periodico_Contacto
            else if (Persona instanceof Periodico_Contacto)     return removePeriodicoContactoBD(Departamento,(Periodico_Contacto) Persona);
            else                                                return 0;
        }

        /** insertPersonalBD: Metodo que inserta todo el personal de un Departamento en la BD
         *
         * @param   Departamento                        Departamento        Departamento
         * @return  int                                 Numero de filas insertadas
         */
        public int insertPersonalBD(Departamento Departamento) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                                   throw new DataException();

            // Recorremos el array de personas insertandolas
            Iterator it = Departamento.getPersonal().entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos la persona
                Map.Entry VKPersona = (Map.Entry) it.next();
                Personal Persona = (Personal) VKPersona.getValue();

                // Insertamos la persona
                insertPersonaBD(Departamento,Persona);
            }

            // Devolvemos el resultado
            return Departamento.getPersonal().size();
        }
        /** updatePersonalBD: Metodo que modifica el personal de un departamento en la BD
         *
         * @param   Departamento                        Departamento        Departamento a modificar
         * @return  int                                                     Numero de filas (Personas) actualizadas
         */
        public int updatePersonalBD(Departamento Departamento) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                                   throw new DataException();

            // Filas actualizadas
            int res = 0;

            // Personal actual
            HashMap<String,Personal> Personal_actu = getPersonalBD(Departamento.getCodDep());

            // Recorremos el HashMap nuevo, actualizando las personas existentes e insertando las nuevas
            Iterator it_new = Departamento.getPersonal().entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKPersonas = (Map.Entry) it_new.next();
                Personal pp_new = (Personal) VKPersonas.getValue();
                String pp_key = (String) VKPersonas.getKey();

                // Si existe la persona -> La actualizamos
                if (Personal_actu.containsKey(pp_key))                  updatePersonaBD(Departamento,getDepartamento(Departamento.getCodDep()).getPersona(pp_key),pp_new);
                // Si no existe -> La insertamos
                else                                                    insertPersonaBD(Departamento,pp_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellas Personas que no consten en el nuevo
            Iterator it_old = Personal_actu.entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKPersonas = (Map.Entry) it_old.next();
                Personal pp_old = (Personal) VKPersonas.getValue();
                String pp_key = (String) VKPersonas.getKey();

                // Si la persona vieja no existe como nueva -> La eliminamos y actualizamos el contador de filas actualizadas
                if (! Departamento.getPersonal().containsKey(pp_key))
                {
                    removePersonaBD(Departamento,pp_old);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removePersonalBD: Metodo que borra todo el personal de un Departamento de la BD
         *
         * @param   Departamento    Departamento        Departamento
         * @return  int                                 Numero de filas eliminadas
         */
        public int removePersonalBD(Departamento Departamento) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                           throw new DataException();

            // Creamos la String con la sentencia a ejecutar
             String query = "DELETE FROM Periodico WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\";";
            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // Creamos la String con la sentencia a ejecutar
            query = "DELETE FROM Contacto WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\";";
            // Ejecutamos la consulta
            return UBD.ejecutarUpdate(query) + res;
        }
        /** getPersonalBD: Metodo que obtiene todo el personal de un departamento de la BD
         *
         * @param   String                              Cod_Dep             Codigo de departamento
         * @return  HashMap<String,Personal>    Personal            HashMap con todo el personal relacionado con el departamento
         */
        public HashMap<String,Personal> getPersonalBD (String Cod_Dep) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Cod_Dep.length()>4))              throw new DataException();

           // Creamos el HashMap resultado
           HashMap<String,Personal> Personal = new HashMap();

           // Introducimos el TRABAJADOR PERIODICO y CONTACTOS
           Personal = getPeriodicoContactosBD(Cod_Dep);

           // Introducimos los VIGILANTES
           HashMap<String,Personal> Vigilantes = getVigilantesBD(Cod_Dep);

           Iterator it = Vigilantes.entrySet().iterator();
           while (it.hasNext())
           {
               Map.Entry VKVigilante = (Map.Entry) it.next();
               Vigilante Vigilante = (Vigilante) VKVigilante.getValue();
               Personal.put(Vigilante.getDNI(), Vigilante);
           }

            // Devolvemos el resultado
            return Personal;
        }

        /** insertDepartamentoBD: Metodo que inserta el Departamento en la BD
         *
         * @param   Departamento    Departamento    Departamento a insertar
         * @return  int                             Numero de filas insertadas
         */
        public int insertDepartamentoBD(Departamento Departamento) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                               throw new DataException();
            
            // INSERTAMOS EL DEPARTAMENTO
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Departamento VALUES (\""  +
                    getCodEmp() + "\",\"" +
                    Departamento.getCodDep() + "\",\"" +
                    Departamento.getNombre() + "\",\"" +
                    Departamento.getAcceso().getCodAcc() + "\"," +
                    Departamento.getPeriodicidad() + ",\"" +
                    UCAL.calendarToString(Departamento.getFechaI()) + "\"," +
                    (Departamento.getAutorizador() == null ? "null" : ("\"" + Departamento.getAutorizador().getDNI() + "\"")) + ");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // Si se ha insertado correctamente el departamento -> INSERTAMOS EL PERSONAL
            if (res == 1)       insertPersonalBD(Departamento);

            // Devolvemos el resultado
            return res;
        }
        /** updateDepartamentoBD: Metodo que actualiza un Departamento en la BD
         *
         * @param   Dep_old     Departamento    Departamento a modificar
         * @param   Dep_new     Departamento    Departamento por el que reemplazar
         * @return  int                         Resultado
         */
        public int updateDepartamentoBD(Departamento Dep_old, Departamento Dep_new) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Dep_old == null) || (Dep_new == null))             throw new DataException();

            // ACTUALIZAMOS LOS DATOS
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Departamento SET " +
                "Cod_Dep = \"" + Dep_new.getCodDep() + "\"," +
                "Nombre = \"" + Dep_new.getNombre() + "\"," +
                "Cod_Acc = \"" + Dep_new.getAcceso().getCodAcc() + "\"," +
                "Periodicidad = " + Dep_new.getPeriodicidad() + "," +
                "F_Ult_Actu = \"" + UCAL.calendarToString(Dep_new.getFechaI()) + "\"," +
                "DNI = " + (Dep_new.getAutorizador() == null ? "null" : ("\"" + Dep_new.getAutorizador().getDNI() + "\"")) +
                " WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Dep_old.getCodDep() + "\";";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // ACTUALIZAMOS EL PERSONAL y devolvemos el resultado
            return updatePersonalBD(Dep_new) + res;
        }
        /** removeDepartamentoBD: Metodo que borra un Departamento de la BD
         *
         * @param   Departamento    Departamento    Departamento a eliminar
         * @return  int                             Numero de filas eliminadas
         */
        public int removeDepartamentoBD(Departamento Departamento) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamento == null)                               throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Departamento WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Departamento.getCodDep() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getDepartamentoBD: Metodo que obtiene un departamento de la BD
         *
         * @param   String          Cod_Dep     Codigo de departamento
         * @return  Departamento                Departamento solicitado
         */
        public Departamento getDepartamentoBD(String Cod_Dep) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Dep == null) || (Cod_Dep.length()>4))              throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "SELECT * FROM Departamento WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Cod_Dep + "\";";

            // Obtenemos el departamento
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos su personal
            HashMap<String,Personal> Personal = getPersonalBD(Cod_Dep);

            // Obtenemos sus llaves
            HashMap<String,Llave> Llaves = getLlavesBD(Cod_Dep);

            // Creamos el Departamento
            rs.next();
            rs.getObject(7);
            Residente autoriz = null;
            if (!rs.wasNull())  autoriz = new Residente (rs.getString(7));

            Departamento Departamento = new Departamento(rs.getString(2),rs.getString(3),
                new T_Acceso(rs.getString(4)), rs.getInt(5), UCAL.stringToCalendar(rs.getString(6).substring(0,19)),
                autoriz, Personal, Llaves);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Departamento;
        }

        /** insertDepartamentosBD: Metodo que inserta todos los departamentos de esta Empresa en la BD
         *
         * @param   void
         * @return  int                                 Numero de filas insertadas
         */
        public int insertDepartamentosBD() throws DataException, SQLException, RemoteException
        {
            // Recorremos el array de personas insertandolas
            Iterator it = getDepartamentos().entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos el Departamento
                Map.Entry VKDepartamento = (Map.Entry) it.next();
                Departamento Departamento = (Departamento) VKDepartamento.getValue();

                // Insertamos el Departamento
                insertDepartamentoBD(Departamento);
            }

            // Devolvemos el resultado
            return getDepartamentos().size();
        }
        /** updateDepartamentosBD: Metodo que modifica los departamentos de esta Empresa en la BD
         *
         * @param   HashMap<String,Departamento>        Departamentos       Departamento con los que modificar
         * @return  int                                                     Numero de filas (Departamentos) actualizadas
         */
        public int updateDepartamentosBD(HashMap<String,Departamento> Departamentos) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Departamentos == null)                                  throw new DataException();

            // Filas actualizadas
            int res = 0;

            // Departamentos actuales
            HashMap<String,Departamento> Departamentos_actu = getDepartamentosBD();

            // Recorremos el HashMap nuevo, actualizando los Departamentos existentes e insertando los nuevos
            Iterator it_new = Departamentos.entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKDepartamentos = (Map.Entry) it_new.next();
                Departamento dep_new = (Departamento) VKDepartamentos.getValue();
                String dep_key = (String) VKDepartamentos.getKey();

                // Si existe el Departamento -> Lo actualizamos
                if (Departamentos_actu.containsKey(dep_key))            updateDepartamentoBD(Departamentos_actu.get(dep_key),dep_new);
                // Si no existe -> Lo insertamos
                else                                                    insertDepartamentoBD(dep_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellos Departamentos que no consten en el nuevo
            Iterator it_old = Departamentos_actu.entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKDepartamentos = (Map.Entry) it_old.next();
                Departamento Departamento = (Departamento) VKDepartamentos.getValue();
                String pp_key = (String) VKDepartamentos.getKey();

                // Si el Departamento viejo no existe como nuevo -> Lo eliminamos y actualizamos el contador de filas actualizadas
                if (! Departamentos.containsKey(pp_key))
                {
                    removeDepartamentoBD(Departamento);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removeDepartamentosBD: Metodo que borra todos los Departamento de esta Empresa de la BD
         *
         * @param   void
         * @return  int                                 Numero de filas eliminadas
         */
        public int removeDepartamentosBD() throws SQLException
        {
            // Creamos la String con la sentencia a ejecutar
             String query = "DELETE FROM Departamento WHERE Cod_Emp = \"" + getCodEmp() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getDepartamentosBD: Metodo que obtiene los departamentos de la BD
         *
         * @param   void
         * @return  HashMap<String,Departamento>     HashMap con los departamentos que conforman la empresa
         */
        public HashMap<String,Departamento> getDepartamentosBD() throws DataException, SQLException, RemoteException
        {
            // Creamos el HashMap con los departamentos
            HashMap<String,Departamento> Departamentos = new HashMap();
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Dep FROM Departamento WHERE Cod_Emp = \"" + getCodEmp() + "\";";

            // Obtenemos los departamentos
            ResultSet rs = UBD.ejecutarQuery(query);

            // Creamos el HashMap de Departamentos
            while (rs.next())
            {
                Departamentos.put(getDepartamentoBD((rs.getString(1))).getCodDep(),getDepartamentoBD((rs.getString(1))));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Departamentos;
        }

        /** insertEmpresaBD: Metodo que inserta la Empresa en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertEmpresaBD() throws DataException, SQLException, RemoteException
        {
            // INSERTAMOS LA EMPRESA
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Empresa VALUES (\""  +
                    getCodEmp() + "\",\"" +
                    getCIF() + "\",\"" +
                    getNombre() + "\",\"" +
                    getDireccion() + "\");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);
            
            // Si se ha introducido correctamente la empresa -> INSERTAMOS LOS DEPARTAMENTOS
            if (res == 1)               insertDepartamentosBD();

            // Devolvemos el resultado
            return res;
        }
        /** updateDatosBD: Metodo que actualiza los datos de una empresa en la BD
         *
         * @param   Empresa     Empresa con la que actualizar los datos
         * @return  int         Numero de filas actualizadas
         */
        public int updateDatosBD(Empresa Empresa) throws DataException, SQLException, RemoteException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Empresa SET " +
                "Cod_Emp = \"" + Empresa.getCodEmp() + "\"," +
                "CIF = \"" + Empresa.getCIF() + "\"," +
                "Nombre = \"" + Empresa.getNombre() + "\"," +
                "Direccion = \"" + Empresa.getDireccion() +
                "\" WHERE Cod_Emp = \"" + getCodEmp() + "\";";

            // Ejecutamos la consulta
            return UBD.ejecutarUpdate(query);
        }
        /** updateEmpresaBD: Metodo que actualiza la Empresa en la BD
         *
         * @param   Empresa         Empresa a modificar
         * @return  int             Resultado
         */
        public int updateEmpresaBD(Empresa Empresa) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Empresa == null)                                    throw new DataException();

            // ACTUALIZAMOS LOS DATOS
            int res = updateDatosBD(Empresa);

            // Actualizamos la PK para que las actualizaciones posteriores se realicen correctamente
            setCodEmp(Empresa.getCodEmp());

            // ACTUALIZAMOS LOS DEPARTAMENTOS y devolvemos el resultado
            return updateDepartamentosBD(Empresa.getDepartamentosBD()) + res;
        }
        /** removeEmpresaBD: Metodo que borra una Empresa de la BD
         *
         * @param   void
         * @return  int             Numero de filas eliminadas
         */
        public int removeEmpresaBD() throws DataException, SQLException
        {
            // Borramos la empresa
            return removeEmpresaBD(this);
        }
        /** removeEmpresaBD: Metodo que borra una Empresa de la BD
         *
         * @param   Empresa         Empresa a borrar
         * @return  int             Numero de filas eliminadas
         */
        public int removeEmpresaBD(Empresa Empresa) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Empresa == null)                                        throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Empresa WHERE Cod_Emp = \"" + getCodEmp() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* datosEmpresas: Metodo que devuelve los datos de las empresas que cumplen con los criterios establecidos
         *
         * @param   String      CIF         CIF de la empresa
         * @param   String      Nom_Emp     Nombre de la empresa
         * @param   String      Nom_Dep     Nombre del departamento
         * @param   String      DNI         DNI de la persona
         * @param   String      DNI_Usu     DNI de la persona que esta usando la aplicacion
         * @param   String      Tipo_Usu    Tipo de usuario que esta usando la aplicacion
         * @return  ResultSet   rs          Datos de las empresas que cumplen con los criterios establecidos
         */
        public ResultSet datosEmpresas(String CIF, String Nom_Emp, String Nom_Dep, String DNI, String DNI_Usu, String Tipo_Usu) throws SQLException
        {
            // Parte de la clausula WHERE a añadir
            String where = "";
            // Si se trata de un autorizador o un responsable de personal
            if (!Tipo_Usu.equals("ADMINISTRADOR"))
                where = " AND (Nombre IN (SELECT E.Nombre" +
                                    " FROM Empresa E JOIN Departamento D USING (Cod_Emp) LEFT JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                                    " WHERE (Cod_Cargo LIKE 'RP' OR Cod_Cargo IS NULL) AND ((D.DNI = \"" + DNI_Usu + "\" OR C.DNI = \"" + DNI_Usu + "\")))" +
                                        " OR (Nombre NOT IN (SELECT DISTINCT E.Nombre FROM Empresa E JOIN Departamento USING (Cod_Emp) UNION SELECT 'Informática Navarra S.A.' Nombre UNION SELECT 'Ocasional' Nombre)))";

            // Creamos la String con la sentencia a realizar
            String query =  " SELECT DISTINCT Nombre" +
                            " FROM (    SELECT CIF, Nombre, '_' Nom_Dep, '_' DNI" +
                                        " FROM Empresa" +
                                        " UNION " +
                                        " SELECT E.CIF, E.Nombre, D.Nombre , P.DNI" +
                                        " FROM Empresa E JOIN Departamento D USING (Cod_Emp) JOIN Periodico Per USING (Cod_Emp,Cod_Dep) JOIN Personal P ON (Per.DNI=P.DNI)" +
                                                " UNION" +
                                        " SELECT E.CIF, E.Nombre, D.Nombre, P.DNI" +
                                        " FROM Empresa E JOIN Departamento D USING (Cod_Emp) JOIN Contacto C USING (Cod_Emp,Cod_Dep) JOIN Personal P ON (C.DNI=P.DNI)" +
                                                " UNION" +
                                        " SELECT '_', 'Ocasional', '_', DNI" +
                                        " FROM Ocasional JOIN Personal USING (DNI)" +
                                                " UNION" +
                                        " SELECT '_', 'Informática Navarra S.A.', '_', DNI" +
                                        " FROM Residente JOIN Personal USING (DNI)) Empresa_Personal" +
                            " WHERE Nom_Dep LIKE \"%" + Nom_Dep + "%\" AND DNI LIKE \"%" + DNI +
                                "%\" AND CIF LIKE \"%" + CIF + "%\" AND Nombre LIKE \"%" + Nom_Emp + "%\"" +
                                where + " ORDER BY Nombre;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* datosDepartamentos: Metodo que devuelve los datos de los departamentos que cumplen con los criterios establecidos
         *
         * @param   String      Nom_Emp     Nombre de la empresa
         * @param   String      Nom_Dep     Nombre del departamento
         * @param   String      Actividad   Actividad del departamento
         * @param   String      DNI         DNI de la persona
         * @param   String      DNI_Usu     DNI de la persona que esta usando la aplicacion
         * @param   String      Tipo_Usu    Tipo de usuario que esta usando la aplicacion
         * @return  ResultSet   rs          Datos de los departamentos que cumplen con los criterios establecidos
         */
        public ResultSet datosDepartamentos(String Nom_Emp, String Nom_Dep, String Actividad, String DNI, String DNI_Usu, String Tipo_Usu) throws SQLException
        {
            // Parte de la clausula WHERE a añadir
            String where = "";
            // Si se trata de un autorizador o un responsable de personal
            if (!Tipo_Usu.equals("ADMINISTRADOR"))
                where = " AND D.Nombre IN (SELECT D.Nombre" +
                                    " FROM Empresa E JOIN Departamento D USING (Cod_Emp) LEFT JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                                    " WHERE (Cod_Cargo LIKE 'RP' OR Cod_Cargo IS NULL) AND (D.DNI = \"" + DNI_Usu + "\" OR C.DNI = \""+ DNI_Usu + "\"))";

            // Creamos la String con la sentencia a realizar
            String query =  "SELECT DISTINCT D.Nombre, TA.Actividad" +
                            " FROM Empresa E JOIN Departamento D USING (Cod_Emp) JOIN T_Acceso TA USING (Cod_Acc)" +
                            " WHERE E.Nombre LIKE \"%" + Nom_Emp + // "%\" AND P.DNI LIKE \"%" + DNI +
                                "%\" AND D.Nombre LIKE \"%" + Nom_Dep + "%\" AND TA.Actividad LIKE \"%" + Actividad + "%\"" +
                            where + " ORDER BY D.Nombre;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* datosEmpleados: Metodo que devuelve los datos generales y especificos de los empleados quen cumple los criterios establecidos
         *
         * @param   String      Nom_Emp     Nombre de la empresa
         * @param   String      Nom_Dep     Nombre del departamento
         * @param   String      Filtro      Criterio de busqueda en los datos personales
         * @param   String      DNI_Usu     DNI de la persona que esta usando la aplicacion
         * @param   String      Tipo_Usu    Tipo de usuario que esta usando la aplicacion
         * @return  ResultSet   rs          Datos de los empleados que cumplen los criterios establecidos
         */
        public ResultSet datosEmpleados(String Nom_Emp, String Nom_Dep, String Filtro, String DNI_Usu, String Tipo_Usu) throws SQLException
        {
            // Parte de la clausula WHERE a añadir
            String where = "";
            // Si se trata de un autorizador o un responsable de personal
            if (!Tipo_Usu.equals("ADMINISTRADOR"))
                where = " AND (Nom_Emp IN (SELECT E.Nombre" +
                                    " FROM Empresa E JOIN Departamento D USING (Cod_Emp) LEFT JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                                    " WHERE (Cod_Cargo LIKE 'RP' OR Cod_Cargo IS NULL) AND (D.DNI = \"" + DNI_Usu + "\" OR C.DNI = \""+ DNI_Usu + "\"))" +
                        " OR Nom_Dep IN (SELECT D.Nombre" +
                                    " FROM Empresa E JOIN Departamento D USING (Cod_Emp) LEFT JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                                    " WHERE (Cod_Cargo LIKE 'RP' OR Cod_Cargo IS NULL) AND (D.DNI = \"" + DNI_Usu + "\" OR C.DNI = \""+ DNI_Usu + "\")))";

            // Creamos la String con la sentencia a realizar
            String query =  "SELECT DISTINCT DNI, Apellido1 \"Primer apellido\", Apellido2 \"Segundo apellido\", Nombre" +
                            " FROM (SELECT E.Nombre 'Nom_Emp', D.Nombre 'Nom_Dep', P.DNI, Apellido1, Apellido2, P.Nombre" +
                                    " FROM Empresa E JOIN Departamento D USING (Cod_Emp) JOIN Periodico Per USING (Cod_Emp,Cod_Dep)" +
                                    " JOIN Personal P ON (Per.DNI=P.DNI)" +
                                        " UNION" +
                                    " SELECT 'Ocasional', '_', P.DNI, Apellido1, Apellido2, P.Nombre" +
                                    " FROM Ocasional O JOIN Personal P USING (DNI)" +
                                        " UNION" +
                                    " SELECT 'Informática Navarra S.A.', '_', P.DNI, Apellido1, Apellido2, P.Nombre" +
                                    " FROM Residente JOIN Personal P USING (DNI)) Personas" +
                            " WHERE DNI NOT IN (SELECT DNI FROM Contacto) " +
                            " AND Nom_Emp LIKE \"%" + Nom_Emp + "%\" AND Nom_Dep LIKE \"%" + Nom_Dep +
                                "%\" AND (DNI LIKE \"%" + Filtro + "%\" OR Apellido1 LIKE \"%" + Filtro +
                                "%\" OR Apellido2 LIKE \"%" + Filtro + "%\" OR Nombre LIKE \"%" + Filtro + "%\")" +
                            where + " ORDER BY DNI;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* datosContactos: Metodo que devuelve los datos de los contactos que cumplen con los criterios establecidos
         *
         * @param   String      Nom_Emp     Nombre de la empresa
         * @param   String      Nom_Dep     Nombre del departamento
         * @param   String      Filtro      Criterio de busqueda en los datos personales
         * @param   String      DNI_Usu     DNI de la persona que esta usando la aplicacion
         * @param   String      Tipo_Usu    Tipo de usuario que esta usando la aplicacion
         * @return  ResultSet   rs          Datos de los contactos que cumplen los criterios establecidos
         */
        public ResultSet datosContactos(String Nom_Emp, String Nom_Dep, String Filtro, String DNI_Usu, String Tipo_Usu) throws SQLException
        {
            // Parte de la clausula WHERE a añadir
            String where = "";
            // Si se trata de un autorizador o un responsable de personal
            if (!Tipo_Usu.equals("ADMINISTRADOR"))
                where = " AND (E.Nombre IN (SELECT E.Nombre" +
                                    " FROM Empresa E JOIN Departamento D USING (Cod_Emp) LEFT JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                                    " WHERE (Cod_Cargo LIKE 'RP' OR Cod_Cargo IS NULL) AND (D.DNI = \"" + DNI_Usu + "\" OR C.DNI = \""+ DNI_Usu + "\"))" +
                        " OR D.Nombre IN (SELECT D.Nombre" +
                                    " FROM Empresa E JOIN Departamento D USING (Cod_Emp) LEFT JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                                    " WHERE (Cod_Cargo LIKE 'RP' OR Cod_Cargo IS NULL) AND (D.DNI = \"" + DNI_Usu + "\" OR C.DNI = \""+ DNI_Usu + "\")))";

            // Creamos la String con la sentencia a realizar
            String query =  "SELECT DISTINCT P.DNI, Apellido1 \"Primer apellido\", Apellido2 \"Segundo apellido\", P.Nombre" +
                            " FROM Empresa E JOIN Departamento D USING (Cod_Emp) JOIN Contacto C USING (Cod_Emp,Cod_Dep)" +
                                    " JOIN Personal P ON (C.DNI=P.DNI)" +
                            " WHERE E.Nombre LIKE \"%" + Nom_Emp + "%\" AND (D.Nombre LIKE \"%" + Nom_Dep +
                                "%\" OR D.Nombre IS NULL) AND (P.DNI LIKE \"%" + Filtro + "%\" OR Apellido1 LIKE \"%" + Filtro +
                                "%\" OR Apellido2 LIKE \"%" + Filtro + "%\" OR P.Nombre LIKE \"%" + Filtro + "%\")" +
                            where + " ORDER BY P.DNI;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerCodEmp: Metodo que devuelve el codigo de empresa dado su nombre
         *
         * @param   String      void
         * @return  String      Cod_Emp             Codigo de empresa cuyo nombre se corresponde con el proporcionado
         */
        public String obtenerCodEmp() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Emp FROM Empresa WHERE Nombre = \"" + getNombre() + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            String Cod_Emp = rs.getString(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Cod_Emp;
        }

        /* obtenerCodDep: Metodo que devuelve el codigo de departamento dado su nombre
         *
         * @param   String      Nom_Dep             Nombre de departamento
         * @return  String      Cod_Dep             Codigo de departamento cuyo nombre se corresponde con el proporcionado
         */
        public String obtenerCodDep(String Nom_Dep) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Dep FROM Departamento WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Nombre = \"" + Nom_Dep + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            String Cod_Dep = rs.getString(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Cod_Dep;
        }

        /* obtenerNomEmps: Metodo que devuelve los nombres de empresas existentes
         *
         * @param   void
         * @return  ResultSet      rs             ResultSet con los nombres de empresas existentes
         */
        public ResultSet obtenerNomEmps() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Nombre FROM Empresa;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Devolvemos el resultado
            return rs;
        }

        /* obtenerNomDeps: Metodo que devuelve los nombres de departamentos existentes en la empresa
         *
         * @param   void
         * @return  ResultSet      rs             ResultSet con los nombres de departamentos existentes en la empresa
         */
        public ResultSet obtenerNomDeps() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Nombre FROM Departamento WHERE Cod_Emp = \"" + obtenerCodEmp() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
        
        /* restablecerDatos: Metodo que asigna al objeto que lo llama sus datos
         *
         * @param   void
         * @return  void
         */
        public void restablecerDatos(String nombre) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT * FROM Empresa WHERE Nombre = \"" + nombre + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Establecemos los valores de los atributos
            rs.next();
            this.Cod_Emp = rs.getString(1);
            this.CIF = rs.getString(2);
            this.Nombre = rs.getString(3);
            this.Direccion = rs.getString(4);
            this.Departamentos = new HashMap();
        }

        /* obtenerLlaves: Metodo que obtiene todas las llaves prestables a un departamento
         *
         * @param   String      Cod_Dep     Codigo del departamento a consultar
         * @return  ResultSet               Llaves prestables al departamento
         */
        public ResultSet obtenerLlaves(String Cod_Dep) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llave 'Código', Z.Descripcion 'Zona', L.Descripcion 'Descripción'" +
                            " FROM Zona Z JOIN Llave L USING (Cod_Zona,F_Alta_Z) JOIN Prestamo_Departamento USING (Cod_Llave,F_Alta_L)" +
                            " WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Cod_Dep + "\"" +
                            " ORDER BY Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }


        /* obtenerLsPorPrestarDep: Metodo que obtiene todas las llaves que no son prestables a un departamento y pasan el filtro proporcionado
         *
         * @param   String      Cod_Dep     Codigo del departamento a consultar
         * @param   String      Filtro      Filtro de datos
         * @return  ResultSet               Llaves prestables al departamento
         */
        public ResultSet obtenerLsPorPrestarDep(String Cod_Dep, String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llave 'Código', Z.Descripcion 'Zona', L.Descripcion 'Descripción'" +
                            " FROM Zona Z JOIN Llave L USING (Cod_Zona,F_Alta_Z)" +
                            " WHERE (Cod_Llave,F_Alta_L) NOT IN (SELECT Cod_Llave,F_Alta_L" +
                                                                    " FROM Prestamo_Departamento" +
                                                                    " WHERE Cod_Emp = \"" + getCodEmp() + "\" AND Cod_Dep = \"" + Cod_Dep + "\")" +
                                    " AND (Cod_Llave LIKE \"%" + Filtro + "%\" OR Z.Descripcion LIKE \"%" + Filtro + "%\" OR L.Descripcion LIKE \"%" + Filtro + "%\")" +
                            " ORDER BY Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
}