// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Gen_LLl_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.util.Calendar;



/** CLASE GEN_LLL:  Clase que recoge los datos comunes a LLave y Llavero
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Gen_LLl extends DosFechas implements Gen_LLl_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Codigo:                 Numero de llave o llavero */
	private int                 Codigo;
        /** Descripcion:            Descripcion de la(s) cerradura(s) que abre(n) */
        private String              Descripcion;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();

        
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor: Constructor de clase por defecto
	 *
	 * @return  Gen_LLl	Objeto de tipo Gen_LLl
	 */
	public Gen_LLl() throws DataException, RemoteException {}
        /** Constructor: Constructor de clase que crea el objeto Llave con los valores pasados parametricamente
	 *
         * @param   Codigo      Numero de llave o llavero
         * @param   F_Alta      Fecha y hora de alta
         * @param   Descripcion Descripcion de la(s) cerradura(s) que abre(n)
	 * @return  Gen_LLl	Objeto de tipo Gen_LLl
	 */
	public Gen_LLl(int Codigo, Calendar F_Alta, String Descripcion) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta);
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Codigo<0) || (Codigo>999) ||(Descripcion.length()>100))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Codigo = Codigo;
            this.Descripcion = Descripcion;
	}
        /** Constructor: Constructor de clase que crea el objeto Llave con los valores pasados parametricamente
	 *
         * @param   Codigo      Numero de llave o llavero
         * @param   F_Alta      Fecha y hora de alta
         * @param   F_Baja      Fecha y hora de baja
         * @param   Descripcion Descripcion de la(s) cerradura(s) que abre(n)
	 * @return  Gen_LLl	Objeto de tipo Gen_LLl
	 */
	public Gen_LLl(int Codigo, Calendar F_Alta, Calendar F_Baja, String Descripcion) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta,F_Baja);
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Codigo<0) || (Codigo>999) || (Descripcion.length()>100))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Codigo = Codigo;
            this.Descripcion = Descripcion;
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodigo: Metodo que obtiene el valor del atributo Codigo
	 *
	 * @param   void
	 * @return  int     Numero de llave o llavero
	 */
	public int getCodigo()                          { return this.Codigo; }
        /** setCodigo: Metodo que establece el valor del atributo Codigo
	 *
	 * @param   Codigo     int     Numero de llave o llavero
	 * @return  void
	 */
	public void setCodigo(int Codigo) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Codigo<0) || (Codigo>999))             throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Codigo = Codigo;
        }

        /** getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	 *
	 * @param   void
	 * @return  String	Descripcion de la(s) cerradura(s) que abre (n)
	 */
	public String getDescripcion()                  { return this.Descripcion; }
        /** setDescripcion: Metodo que establece el valor del atributo Descripcion
	 *
	 * @param   Descripcion     String     Descripcion de la(s) cerradura(s) que abre(n)
	 * @return  void
	 */
	public void setDescripcion(String Descripcion) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Descripcion.length()>100))    throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Descripcion = Descripcion;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "Codigo: " + getCodigo() + "\n" +
            super.toString() + "\n" +
            "Descripcion: " + getDescripcion() + "\n";
	}
}