// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Llave_I;
import java.rmi.*;
import java.sql.*;
import java.util.Calendar;
import java.util.HashMap;



/** CLASE LLAVE:  Clase que representa una llave
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Llave extends Gen_LLl implements Llave_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Notas:                              Otros datos de interes acerca de la llave */
        private String                          Notas;
        /** Departamentos:                      Llaves que pertenecen a la zona */
        private HashMap<String,Departamento>    Departamentos;
        /** Ocasionales:                        Llaves que pertenecen a la zona */
        private HashMap<String,Ocasional>       Ocasionales;

	
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Llave      Objeto de tipo Llave
         */
        public Llave () throws DataException, RemoteException {}
        /** Constructor: Constructor de clase que crea el objeto Llave con los valores pasados parametricamente
	 *
	 * @param   Cod_Llave       Numero de llave
         * @param   F_Alta          Fecha y hora de alta de la llave
         * @param   Descripcion     Descripcion de la cerradura que abre
	 * @return  Llave           Objeto de tipo Llave
	 */
	public Llave(int Cod_Llave, Calendar F_Alta_L, String Descripcion) throws DataException, RemoteException
	{
            // Llamamos al constructor de la clase base
            super (Cod_Llave, F_Alta_L, Descripcion);

            // Comprobamos que los parametros pasados son correctos
            if ((Departamentos != null) && (Ocasionales != null))               throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Notas = null;
	}
	/** Constructor: Constructor de clase que crea el objeto Llave con los valores pasados parametricamente
	 *
	 * @param   Cod_Llave   Numero de llave
         * @param   F_Alta      Fecha y hora de alta de la llave
         * @param   F_Baja      Fecha y hora de baja de la llave
         * @param   Descripcion Descripcion de la cerradura que abre
         * @param   Notas       Otros datos de interes acerca de la llave
	 * @return  Llave	Objeto de tipo Llave
	 */
	public Llave(int Cod_Llave, Calendar F_Alta_L, Calendar F_Baja_L, String Descripcion, String Notas) throws DataException, RemoteException
	{
            // Llamamos al constructor de la clase base
            super (Cod_Llave, F_Alta_L, F_Baja_L, Descripcion);

            // Comprobamos que los parametros pasados son correctos
            if ((Notas != null) && (Notas.length()>150) || (Departamentos != null) && (Ocasionales != null))        throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Notas = Notas;
	}
	/** Constructor: Constructor de clase que crea el objeto Llave con los valores pasados parametricamente
	 *
	 * @param   Cod_Llave   Numero de llave
         * @param   F_Alta      Fecha y hora de alta de la llave
	 * @return  Llave	Objeto de tipo Llave
	 */
	public Llave(int Cod_Llave, String F_Alta_L) throws DataException, SQLException, RemoteException
	{
            // Realizamos la asignacion de valores a los atributos bases
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((F_Alta_L == null) || (Cod_Llave<0) || (Cod_Llave>999) || (F_Alta_L.length()>19))
                throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Cod_Llave,F_Alta_L,F_Baja_L,Descripcion,Notas FROM Llave WHERE Cod_Llave = " + Cod_Llave +
                " AND F_Alta_L = \"" + F_Alta_L + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            setCodigo(rs.getInt(1));
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,10) + " 23:59:59"));
            setDescripcion(rs.getString(4));
            rs.getObject(5);
            if (rs.wasNull())       this.Notas = null;
            else                    this.Notas = rs.getString(5);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
	/** Constructor: Constructor de clase que crea el objeto Llave con los valores pasados parametricamente
	 *
	 * @param   Cod_Llave   Numero de llave
	 * @return  Llave	Objeto de tipo Llave
	 */
	public Llave(int Cod_Llave) throws DataException, SQLException, RemoteException
	{
            // Realizamos la asignacion de valores a los atributos bases
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Llave<0) || (Cod_Llave>999))                               throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Cod_Llave,F_Alta_L,F_Baja_L,Descripcion,Notas FROM Llave WHERE Cod_Llave = " + Cod_Llave +
                " AND F_Baja_L IS NULL;";

            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            setCodigo(rs.getInt(1));
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            setDescripcion(rs.getString(4));
            rs.getObject(5);
            if (rs.wasNull())       this.Notas = null;
            else                    this.Notas = rs.getString(5);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getNotas: Metodo que obtiene el valor del atributo Notas
	 *
	 * @param   void
	 * @return  String	Otros datos de interes acerca de la llave
	 */
	public String getNotas()                        { return this.Notas; }
        /** setNotas: Metodo que establece el valor del atributo Notas
	 *
	 * @param   Notas     String     Otros datos de interes acerca de la llave
	 * @return  void
	 */
	public void setNotas(String Notas) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Notas != null) && (Notas.length()>150))    throw new DataException ();
            // Realizamos la asignacion de valores a los atributos
            this.Notas = Notas;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "LLAVE:\n" + super.toString() + "\n" +
            "Notas: " + (getNotas() == null ? "" : getNotas()) + "\n";
	}


        // FUNCIONALIDADES
        /* obtenerLlaves: Metodo que devuelve las llaves existentes en la BD
         *
         * @param   void
         * @return  ResultSet   rs          Llaves existentes en la BD
         */
        public ResultSet obtenerLlaves() throws SQLException
        {
            // Creamos la String con la sentencia a ejecutar
            String query = "SELECT Cod_Llave 'Código', Descripcion 'Descripción'" +
                    " FROM Llave" +
                    " WHERE F_Baja_L IS NULL;";

            // Ejecutamos la query y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLlaves: Metodo que devuelve las llaves prestables a una persona y que pasan un filtro dado
         *
         * @param   String      DNI         DNI de la persona que solicita llaves
         * @param   String      Entrada     Fecha y hora en la que se realiza el acceso
         * @param   String      Filtro      Filtro que debe pasar las llaves
         * @return  ResultSet   rs          Llaves que pasan el filtro
         */
        public ResultSet obtenerLlaves(String DNI, String Entrada, String Filtro) throws SQLException
        {
            // Obtenemos la Empresa en caso de ser posible
            String Empresa = "";
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Empresa FROM Ocasional WHERE DNI = \"" + DNI + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);
            if (rs.next())
            {
                // Obtenemos la empresa
                Empresa = rs.getString(1);
            }
            // Cerramos la conexion
            UBD.cerrarConexion();


            // Obtenemos el ResultSet con los datos de las llaves prestables
            // = Llaves prestables que no han sido prestadas en el acceso, de las que se dispone de copias y que cumplen el filtro
            query = "SELECT Cod_Llave Código, Z.Descripcion Zona, L.Descripcion Descripción" +
                    " FROM Llave L JOIN Zona Z USING (Cod_Zona)" +
                    " WHERE (Cod_Llave, F_Alta_L) IN (SELECT Cod_Llave, F_Alta_L" +
                                                        " FROM Prestamo_Departamento JOIN Periodico USING (Cod_Emp,Cod_Dep)" +
                                                        " WHERE DNI = \"" + DNI + "\"" +
                                                            " UNION" +
                                                        " SELECT Cod_Llave, F_Alta_L" +
                                                        " FROM Prestamo_Ocasional JOIN Ocasional USING (DNI,Empresa)" +
                                                        " WHERE DNI = \"" + DNI + "\" AND Empresa = \"" + Empresa + "\")" +
                        " AND (Cod_Llave, F_Alta_L) NOT IN (SELECT Cod_Llave, F_Alta_L" +
                                                            " FROM Llave_Prestada" +
                                                            " WHERE Entrada = \"" + Entrada + "\"" +
                                                                " UNION" +
                                                            " SELECT Cod_Llave, F_Alta_L" +
                                                            " FROM Llave" +
                                                            " WHERE F_Baja_L IS NOT NULL)" +
                        " AND (Cod_Llave, F_Alta_L) IN (SELECT Cod_Llave,F_Alta_L" +
                                                        " FROM (SELECT Cod_Llave,F_Alta_L, SUM(Num_Copias)-Copias_Prestadas Copias_disponibles" +
                                                                " FROM Llave JOIN Almacen_Llave USING (Cod_Llave, F_Alta_L) LEFT JOIN" +
                                                                    " (SELECT Cod_Llave, F_Alta_L, COUNT(*) Copias_prestadas" +
                                                                    " FROM Llave JOIN Llave_Prestada USING (Cod_Llave, F_Alta_L)" +
                                                                    " WHERE Devuelta = False" +
                                                                    " GROUP BY Cod_Llave, F_Alta_L) Llaves_Prestadas USING (Cod_Llave, F_Alta_L)" +
                                                                " GROUP BY Cod_Llave,F_Alta_L) Llaves_Disponibles" +
                                                        " WHERE Copias_disponibles>0 OR Copias_Disponibles IS NULL)" +
                        " AND (Cod_Llave LIKE \"%" + Filtro + "%\" OR" +
                        " Z.Descripcion LIKE \"% " + Filtro + "%\"" +
                        " OR L.Descripcion LIKE \"%" + Filtro + "%\")" +
                    " ORDER BY Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerPrestamos: Metodo que devuelve los datos de todas las llaves que permanecen en prestamo
         *
         * @return  ResultSet   rs          Datos de las llaves en prestamo
         */
        public ResultSet obtenerPrestamos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT Cod_Llave 'Código', Entrada 'Fecha', DNI" +
                    " FROM Llave_Prestada JOIN Acceso USING (Entrada) JOIN Llave USING (Cod_Llave,F_Alta_L)" +
                    " WHERE F_Baja_L IS NULL AND Devuelta = false" +
                    " ORDER BY Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerEmpDeps: Metodo que devuelve todos los departamentos que pueden pedir la llave en prestamo
         *
         * @param   void
         * @return  ResultSet   rs          Departamentos que pueden pedir la llave en prestamo
         */
        public ResultSet obtenerEmpDeps() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT E.Nombre Empresa,D.Nombre Departamento" +
                            " FROM Prestamo_Departamento JOIN Departamento D USING (Cod_Emp,Cod_Dep) JOIN Empresa E USING (Cod_Emp)" +
                            " WHERE Cod_Llave = " + getCodigo() + " AND F_Alta_L = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerOcasional: Metodo que devuelve todo el personal ocasional que puede pedir la llave en prestamo
         *
         * @param   void
         * @return  ResultSet   rs          Personal ocasional que puede pedir la llave en prestamo
         */
        public ResultSet obtenerOcasional() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI,Empresa" +
                            " FROM Prestamo_Ocasional" +
                            " WHERE Cod_Llave = " + getCodigo() + " AND F_Alta_L = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerEmpDepsResto: Metodo que devuelve todos los departamentos que no pueden pedir la llave en prestamo
         *
         * @param   void
         * @return  ResultSet   rs          Departamentos que no pueden pedir la llave en prestamo
         */
        public ResultSet obtenerEmpDepsResto() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT E.Nombre Empresa,D.Nombre Departamento" +
                            " FROM Empresa E JOIN Departamento D USING (Cod_Emp)" +
                            " WHERE (Cod_Emp,Cod_Dep) NOT IN ( SELECT Cod_Emp,Cod_Dep" +
                                                                " FROM Prestamo_Departamento" +
                                                                " WHERE Cod_Llave = " + getCodigo() + " AND F_Alta_L = \"" + calendarToString(getFechaI()) + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerOcasResto: Metodo que devuelve todo el personal ocasional que no puede pedir la llave en prestamo
         *
         * @param   void
         * @return  ResultSet   rs          Personal ocasional que no puede pedir la llave en prestamo
         */
        public ResultSet obtenerOcasionalResto() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI,Empresa" +
                            " FROM Ocasional" +
                            " WHERE (DNI,Empresa) NOT IN (SELECT DNI,Empresa" +
                                                            " FROM Prestamo_Ocasional" +
                                                            " WHERE Cod_Llave = " + getCodigo() + " AND F_Alta_L = \"" + calendarToString(getFechaI()) + "\")" +
                            " ORDER BY DNI;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* eliminarPrestables: Metodo que elimina cualquier permiso de prestamo de esta llave
         *
         * @param   void
         * @return  int     res       Numero de filas (permisos) eliminadas
         */
        public int eliminarPrestables() throws SQLException
        {
            int res;

            // Creamos la String con la sentencia a realizar
            String query = " DELETE FROM Prestamo_Departamento WHERE Cod_Llave = " + getCodigo() + " AND F_Alta_L = \"" + calendarToString(getFechaI()) + "\";";
            // Ejecutamos la consulta
            res = UBD.ejecutarUpdate(query);

            // Creamos la String con la sentencia a realizar
            query = " DELETE FROM Prestamo_Ocasional WHERE Cod_Llave = " + getCodigo() + " AND F_Alta_L = \"" + calendarToString(getFechaI()) + "\";";
            // Ejecutamos la consulta
            return UBD.ejecutarUpdate(query) + res;
        }

        /* codigoPorDefecto: Metodo que devuelve el codigo de llave por defecto
         *
         * @param   void
         * @return  int   Codigo          Codigo de llave por defecto
         */
        public int codigoPorDefecto() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Codigo" +
                            " FROM Codigo" +
                            " WHERE codigo NOT IN (SELECT Cod_Llave" +
                                                    " FROM Llave" +
                                                    " WHERE F_Baja_L IS NULL)" +
                            " LIMIT 0,1;";
            
            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int Codigo = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Codigo;
        }

        /* eliminarLlaves: Metodo que borra todas las llaves mas antiguas de Dias dias
         *
         * @param   Dias    Numero de dias transcurridos desde la baja de la llave
         * @return  int     Numero de llaves eliminadas
         */
        public int eliminarLlaves(int Dias) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Llave WHERE ADDDATE(DATE(F_Baja_L),INTERVAL " + Dias + " DAY)<=CURDATE();";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
}