// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Llavero_I;
import java.rmi.*;
import java.sql.*;
import java.util.Calendar;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;



/** CLASE LLAVERO:  Clase que representa un llavero
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Llavero extends Gen_LLl implements Llavero_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Llaves:                     HashMap de llaves que conforman el llavero */
        private HashMap<String,Llave>   Llaves;


	
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Llavero      Objeto de tipo Llavero
         */
        public Llavero () throws DataException, RemoteException {}
        /** Constructor: Constructor de clase que crea el objeto Llavero con los valores pasados parametricamente
	 *
	 * @param   Cod_Llavero Numero de llavero
         * @param   F_Alta_Ll   Fecha y hora en la que el llavero ha sido dado de alta
         * @param   Descripcion String con la descripcion de las cerraduras que abren las llaves
         * @param   Llaves      HashMap de llaves que conforman el llavero
	 * @return  Llavero	Objeto de tipo Llavero
	 */
	public Llavero(int Cod_Llavero, Calendar F_Alta_Ll, String Descripcion, HashMap<String,Llave> Llaves) throws DataException, RemoteException
	{
            // Llamamos al constructor de la clase base
            super (Cod_Llavero, F_Alta_Ll, Descripcion);
            // Comprobamos que los parametros pasados son correctos
            if ((Llaves == null) || (Llaves.isEmpty()))     throw new DataException();
            this.Llaves = Llaves;   // Realizamos la asignacion de valores a los atributos
	}
	/** Constructor: Constructor de clase que crea el objeto Llavero con los valores pasados parametricamente
	 *
	 * @param   Cod_Llavero Numero de llavero
         * @param   F_Alta_Ll   Fecha y hora en la que el llavero ha sido dado de alta
         * @param   F_Baja_Ll   Fecha y hora en la que el llavero ha sido dado de baja
         * @param   Descripcion String con la descripcion de las cerraduras que abren las llaves
         * @param   Llaves      HashMap de llaves que conforman el llavero
	 * @return  Llavero	Objeto de tipo Llavero
	 */
	public Llavero(int Cod_Llavero, Calendar F_Alta_Ll, Calendar F_Baja_Ll, String Descripcion, HashMap<String,Llave> Llaves) throws DataException, RemoteException
	{
            // Llamamos al constructor de la clase base
            super (Cod_Llavero, F_Alta_Ll, F_Baja_Ll, Descripcion);
            this.Llaves = Llaves;   // Realizamos la asignacion de valores a los atributos
	}
	/** Constructor: Constructor de clase que crea el objeto Llavero con los valores pasados parametricamente
	 *
	 * @param   Cod_Llavero Numero de llavero
         * @param   F_Alta_L    Fecha de alta del Llavero
	 * @return  Llavero	Objeto de tipo Llavero
	 */
	public Llavero(int Cod_Llavero, String F_Alta_Ll) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor de la clase base
            super ();
            // Comprobamos que los parametros pasados son correctos
            if ((F_Alta_Ll == null) || (Cod_Llavero<0) || (Cod_Llavero>999))       throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Llavero WHERE Cod_Llavero = " + Cod_Llavero +
                " AND F_Alta_Ll = \"" + F_Alta_Ll + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion
            rs.next();
            setCodigo(rs.getInt(1));
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,10) + " 23:59:59"));
            setDescripcion(rs.getString(4));
            this.Llaves = getLlavesBD(Cod_Llavero,F_Alta_Ll);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
	/** Constructor: Constructor de clase que crea el objeto Llavero con los valores pasados parametricamente
	 *
	 * @param   Cod_Llavero Numero de llavero
	 * @return  Llavero	Objeto de tipo Llavero
	 */
	public Llavero(int Cod_Llavero) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor de la clase base
            super ();
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Llavero<0) || (Cod_Llavero>999))       throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Llavero WHERE Cod_Llavero = " + Cod_Llavero + " ORDER BY F_Alta_Ll DESC;";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion
            rs.next();
            setCodigo(rs.getInt(1));
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            setDescripcion(rs.getString(4));
            this.Llaves = getLlavesBD(Cod_Llavero,calendarToString(getFechaI()).substring(0,19));

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getLlaves: Metodo que obtiene el valor del atributo Llaves
	 *
	 * @param   void
	 * @return  HashMap	HashMap con las llaves que conforman el llavero
	 */
	public HashMap<String,Llave> getLlaves()                  { return this.Llaves; }
        /** setLlaves: Metodo que establece el valor del atributo Llaves
	 *
	 * @param   Llaves     HashMap     HashMap con las llaves que conforman el llavero
	 * @return  void
	 */
	public void setLlaves(HashMap<String,Llave> Llaves) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llaves == null) || (Llaves.isEmpty())) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Llaves = Llaves;
        }
        /** addLlave: Metodo que añade una llave a las ya existentes en el llavero
         *
         * @param   Llave        Llave  Llave nueva a insertar en el llavero
         * @return  void
         */
        public void addLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                          throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().put(Llave.getCodigo() + " - " + calendarToString(Llave.getFechaI()),Llave);
        }
        /** updateLlave: Metodo que actualiza una llave del llavero
         *
         * @param   Llave_old,Llave_new        Llave,Llave  Llaves a modificar y por la que reemplazar
         * @return  void
         */
        public void updateLlave(Llave Llave_old, Llave Llave_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llave_old == null) || (Llave_new == null))   throw new DataException();

            if (getLlaves().containsKey(Llave_old.getCodigo()))
            {
                removeLlave(Llave_old);
                addLlave(Llave_new);
            }
        }
        /** removeLlave: Metodo que elimina una llave de las existentes en el llavero
         *
         * @param   Llave        Llave  Llave a eliminar
         * @return  void
         */
        public void removeLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                          throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().remove(Llave.getCodigo() + " - " + calendarToString(Llave.getFechaI()));
        }
        /** getLlave: Metodo que obtiene una Llave de las existentes
         *
         * @param   String        Key_Llave  Key de la Llave a obtener
         * @return  void
         */
        public Llave getLlave(String Key_Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Key_Llave == null) || (Key_Llave.length()>29))     throw new DataException();
            // Devolvemos el resultado
            return this.getLlaves().get(Key_Llave);
        }

        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // String con las llaves que conforman el llavero
            String ll = "";

            // Añadimos las Strings con todas las llaves
            Iterator it = getLlaves().entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKLlave = (Map.Entry) it.next();
                Llave l = (Llave) VKLlave.getValue();
                ll = ll + l.toString();
            }
            // Devolvemos el resultado
            return "LLAVERO:\n" + super.toString() + "\n" +
            "Llaves:\n" + ll + "\n";
	}


        // DML
        /** insertLlaveBD: Metodo que inserta una Llave en el llavero en la BD
         *
         * @param   Llave  Llave    Llave a insertar en el llavero
         * @return  int             Numero de filas insertadas
         */
        public int insertLlaveBD(Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                                  throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Composicion_Llavero VALUES (" +
                getCodigo() + ",\"" +
                calendarToString(getFechaI()) + "\"," +
                Llave.getCodigo() + ",\"" +
                calendarToString(Llave.getFechaI()) + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateLlaveBD: Metodo que actualiza una llave del Llavero en la BD
         *
         * @param   Llave_old       Llave a modificar
         * @param   Llave_new       Llave de reemplazo
         * @return  int             Numero de filas (Llaves) actualizadas
         */
        public int updateLlaveBD(Llave Llave_old, Llave Llave_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llave_old == null) || (Llave_new == null))         throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Composicion_Llavero SET " +
                "Cod_Llave = " + Llave_new.getCodigo() + "," +
                "F_Alta_L = \"" + calendarToString(Llave_new.getFechaI()) + "\"" +
                " WHERE Cod_Llavero = " + getCodigo() + " AND F_Alta_Ll = \"" +
                calendarToString(Llave_old.getFechaI()) + "\" AND Cod_Llave = " +
                Llave_old.getCodigo() + " AND F_Alta_L = \"" +
                calendarToString(Llave_old.getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeLlaveBD: Metodo que borra una llave de un llavero de la BD
         *
         * @param   Llave    Llave     Llave a eliminar del llavero
         * @return  int                Numero de filas eliminadas
         */
        public int removeLlaveBD(Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                                  throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Composicion_Llavero WHERE Cod_Llavero = " + getCodigo() +
                        " AND F_Alta_Ll = \"" + calendarToString(getFechaI()).substring(0,19) + "\" AND Cod_Llave = " + Llave.getCodigo() +
                        " AND F_Alta_L = \"" + calendarToString(Llave.getFechaI()).substring(0,19) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateLlavesBD: Metodo que modifica las llaves de este llavero en la BD
         *
         * @param   HashMap<String,Llave>        Llaves                     Llaves con las que modificar
         * @return  int                                                     Numero de filas (Llaves) actualizadas
         */
        public int updateLlavesBD(HashMap<String,Llave> Llaves) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llaves == null)                                  throw new DataException();

            // Filas actualizadas
            int res = 0;

            // Llaves actuales
            HashMap<String,Llave> Llaves_actu = getLlavesBD(getCodigo(),calendarToString(getFechaI()).substring(0,19));

            // Recorremos el HashMap nuevo, actualizando las llaves existentes e insertando las nuevas
            Iterator it_new = Llaves.entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_new.next();
                Llave l_new = (Llave) VKLlaves.getValue();
                String l_key = (String) VKLlaves.getKey();

                // Si existe la llave -> La actualizamos
                if (Llaves_actu.containsKey(l_key))                     updateLlaveBD(Llaves_actu.get(l_key),l_new);
                // Si no existe -> La insertamos
                else                                                    insertLlaveBD(l_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellas llaves que no consten en el nuevo
            Iterator it_old = Llaves_actu.entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_old.next();
                Llave l = (Llave) VKLlaves.getValue();
                String l_key = (String) VKLlaves.getKey();

                // Si la llave vieja no existe como nueva -> La eliminamos y actualizamos el contador de filas actualizadas
                if (! Llaves.containsKey(l_key))
                {
                    removeLlaveBD(l);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** getLlavesBD: Metodo que obtiene las llaves de un llavero de la BD
         *
         * @param   void
         * @return  HashMap<String,Llave>     HashMap con las llaves que conforman el llavero
         */
        public HashMap<String,Llave> getLlavesBD(int Cod_Llavero, String F_Alta_Ll) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((F_Alta_Ll == null) || (Cod_Llavero<0) || (Cod_Llavero>999))        throw new DataException();

            // Creamos el HashMap con las Llaves
            HashMap<String,Llave> Llaves = new HashMap();
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llave,F_Alta_L FROM Composicion_Llavero WHERE Cod_Llavero = " + Cod_Llavero +
                    " AND F_Alta_Ll = \"" + F_Alta_Ll + "\";";

            // Obtenemos las zonas
            ResultSet rs = UBD.ejecutarQuery(query);

            // Creamos el HashMap de Zonas
            while (rs.next())       Llaves.put(rs.getInt(1) + " - " + rs.getString(2).substring(0,19), new Llave(rs.getInt(1),rs.getString(2).substring(0,19)));

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Llaves;
        }

        /** insertLlaveroBD: Metodo que inserta el Llavero en la BD
         *
         * @param   void
         * @return  int     Numero de filas (Llaveros) insertadas
         */
        public int insertLlaveroBD() throws DataException, SQLException
        {
            // INSERTAMOS EL LLAVERO
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Llavero VALUES ("  +
                    getCodigo() + ",\"" +
                    calendarToString(getFechaI()) + "\"," +
                    (getFechaJ() == null ? "null" : "\"" + calendarToString(getFechaJ()) + "\"") + ",\"" +
                    getDescripcion() + "\");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // Si el llavero ha sido insertado correctamente -> INSERTAMOS LAS LLAVES QUE LO COMPONEN
            if (res == 1)
            {
                Iterator it = getLlaves().entrySet().iterator();
                while (it.hasNext())
                {
                    // Obtenemos la llave
                    Map.Entry VKLlave = (Map.Entry) it.next();
                    Llave Llave = (Llave) VKLlave.getValue();

                    // Insertamos la llave en el llavero
                    insertLlaveBD (Llave);
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** updateDatosBD: Metodo que actualiza los datos de un llavero en la BD
         *
         * @param   Llavero     Llavero con la que actualizar los datos
         * @return  int         Numero de filas actualizadas
         */
        public int updateDatosBD(Llavero Llavero) throws DataException, SQLException, RemoteException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llavero SET " +
                "Cod_Llavero = " + Llavero.getCodigo() + "," +
                "Descripcion = \"" + Llavero.getDescripcion() + "\"" +
                " WHERE Cod_Llavero = " + getCodigo() + ";";

            // Ejecutamos la consulta
            return UBD.ejecutarUpdate(query);
        }
        /** updateLlaveroBD: Metodo que actualiza el Llavero en la BD
         *
         * @param   Llavero         Llavero con el que modificar
         * @return  int             Numero de filas (Llaveros) actualizados
         */
        public int updateLlaveroBD(Llavero Llavero) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llavero == null)                                    throw new DataException();

            // ACTUALIZAMOS LOS DATOS
            int res = updateDatosBD(Llavero);

            // Actualizamos la PK para que las actualizaciones posteriores se realicen correctamente
            setCodigo(Llavero.getCodigo());

            // ACTUALIZAMOS LAS LLAVES y devolvemos el resultado
            return updateLlavesBD(Llavero.getLlaves()) + res;
        }
        /** darBajaLlaveroBD: Metodo que da de baja el Llavero en la BD
         *
         * @param   void
         * @return  int         Numero de filas actualizadas
         */
        public int darBajaLlaveroBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llavero" +
                    " SET F_Baja_Ll = \"" + calendarToString(Calendar.getInstance()) + "\"" +
                    " WHERE Cod_Llavero = " + getCodigo() + " AND F_Alta_Ll = \"" + calendarToString(getFechaI()).substring(0,19) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeLlaveroBD: Metodo que borra un Llavero de la BD
         *
         * @param   void
         * @return  int             Numero de filas eliminadas
         */
        public int removeLlaveroBD() throws DataException, SQLException
        {
            // Borramos el llavero y devolvemos el resultado
            return removeLlaveroBD(this);
        }
        /** removeLlaveroBD: Metodo que borra un Llavero de la BD
         *
         * @param   Llavero         Llavero a borrar
         * @return  int             Numero de filas eliminadas
         */
        public int removeLlaveroBD(Llavero Llavero) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llavero == null)                                     throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Llavero WHERE Cod_Llavero = " + Llavero.getCodigo() + " AND F_Alta_Ll = \"" + calendarToString(Llavero.getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* obtenerLlaveros: Metodo que devuelve los llaveros prestables a una persona y que pasan un filtro dado
         *
         * @param   String      DNI         DNI de la persona a la que se le va a realizar el prestamo
         * @param   String      Entrada     Fecha y hora en la que se realiza el acceso
         * @param   String      Filtro      Filtro que debe pasar los llaveros
         * @return  ResultSet   rs          Llaveros que pasan el filtro
         */
        public ResultSet obtenerLlaveros(String DNI, String Entrada, String Filtro) throws SQLException
        {
            // Obtenemos la Empresa en caso de ser posible
            String Empresa = "";
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Empresa FROM Ocasional WHERE DNI = \"" + DNI + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);
            if (rs.next())
            {
                // Obtenemos la empresa
                Empresa = rs.getString(1);
            }
            // Cerramos la conexion
            UBD.cerrarConexion();


            // Obtenemos el ResultSet con los datos de los llaveros prestables
            // = Llaveros que contienen llaves prestables que no han sido prestados en el acceso, de los que se dispone copias y que cumplen el filtro
            query = "SELECT DISTINCT Cod_Llavero Código, Descripcion Descripción" +
                    " FROM Llavero JOIN Composicion_Llavero C USING (Cod_Llavero,F_alta_Ll)" +
                    " WHERE (Cod_Llave, F_Alta_L) IN (SELECT Cod_Llave, F_Alta_L" +
                                                        " FROM Prestamo_Departamento JOIN Periodico USING (Cod_Emp,Cod_Dep)" +
                                                        " WHERE DNI = \"" + DNI + "\"" +
                                                            " UNION" +
                                                        " SELECT Cod_Llave, F_Alta_L" +
                                                        " FROM Prestamo_Ocasional JOIN Ocasional USING (DNI,Empresa)" +
                                                        " WHERE DNI = \"" + DNI + "\" AND Empresa = \"" + Empresa + "\")" +
                        " AND (Cod_Llavero, F_Alta_Ll) NOT IN (SELECT Cod_Llavero, F_Alta_Ll" +
                                                            " FROM Llavero_Prestado" +
                                                            " WHERE Entrada = \"" + Entrada + "\"" +
                                                                " UNION" +
                                                            " SELECT Cod_Llavero, F_Alta_Ll" +
                                                            " FROM Llavero" +
                                                            " WHERE F_Baja_Ll IS NOT NULL)" +
                        " AND (Cod_Llavero, F_Alta_Ll) IN (SELECT Cod_Llavero,F_Alta_Ll" +
                                                            " FROM (SELECT Cod_Llavero,F_Alta_Ll, SUM(Num_Copias)-Copias_Prestadas Copias_disponibles" +
                                                                    " FROM Llavero JOIN Almacen_Llavero USING (Cod_Llavero, F_Alta_Ll) LEFT JOIN" +
                                                                        " (SELECT Cod_Llavero, F_Alta_Ll, COUNT(*) Copias_prestadas" +
                                                                        " FROM Llavero JOIN Llavero_Prestado USING (Cod_Llavero, F_Alta_Ll)" +
                                                                        " WHERE Devuelto = False" +
                                                                        " GROUP BY Cod_Llavero, F_Alta_Ll) Llaveros_Prestados USING (Cod_Llavero, F_Alta_Ll)" +
                                                                    " GROUP BY Cod_Llavero,F_Alta_Ll) Llaveros_Disponibles" +
                                                            " WHERE Copias_disponibles>0 OR Copias_Disponibles IS NULL)" +
                        " AND (Cod_Llavero LIKE \"%" + Filtro + "%\" OR" +
                        " Descripcion LIKE \"% " + Filtro + "%\")" +
                    " ORDER BY Cod_Llavero;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerPrestamos: Metodo que devuelve los datos de todas los llaveros que permanecen en prestamo
         *
         * @param   void
         * @return  ResultSet   rs          Datos de los llaveros en prestamo
         */
        public ResultSet obtenerPrestamos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT Cod_Llavero 'Código', Entrada 'Fecha', DNI" +
                    " FROM Llavero_Prestado JOIN Acceso USING (Entrada) JOIN Llavero USING (Cod_Llavero,F_Alta_Ll)" +
                    " WHERE F_Baja_Ll IS NULL AND Devuelto = false" +
                    " ORDER BY Cod_Llavero;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLlaves: Metodo que devuelve los datos de todas las llaves que conforman el llavero y pasan un filtro dado
         *
         * @param   String      Filtro      Filtro que debe pasar la descripcion de cada llave
         * @return  ResultSet   rs          Datos de todas las llaves que conforman el llavero
         */
        public ResultSet obtenerLlaves(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llave Código, Z.Descripcion Zona, L.Descripcion" +
                            " FROM Composicion_Llavero JOIN Llave L USING (Cod_Llave,F_Alta_L) JOIN Zona Z USING (Cod_Zona,F_Alta_Z) JOIN Llavero USING (Cod_Llavero,F_Alta_Ll)" +
                            " WHERE Cod_Llavero = " + getCodigo() +
                                " AND F_Baja_Ll IS NULL AND F_Baja_L IS NULL AND (Z.Descripcion LIKE \"%" + Filtro + "%\" OR L.Descripcion LIKE \"%" + Filtro + "%\")" +
                            " ORDER BY Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* codigoPorDefecto: Metodo que devuelve el codigo de llavero por defecto
         *
         * @param   void
         * @return  int   Codigo          Codigo de llavero por defecto
         */
        public int codigoPorDefecto() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Codigo" +
                            " FROM Codigo" +
                            " WHERE codigo NOT IN (SELECT Cod_Llavero" +
                                                    " FROM Llavero" +
                                                    " WHERE F_Baja_Ll IS NULL)" +
                            " LIMIT 0,1;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int Codigo = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Codigo;
        }

        /* eliminarLlaveros: Metodo que borra todos los llaveros mas antiguos de Dias dias
         *
         * @param   Dias    Numero de dias transcurridos desde la baja del llavero
         * @return  int     Numero de llaveros eliminados
         */
        public int eliminarLlaveros(int Dias) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Llavero WHERE ADDDATE(DATE(F_Baja_Ll),INTERVAL " + Dias + " DAY)<=CURDATE();";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /* disparadorInsert: Metodo que ejecuta el disparador sobre BEFORE INSERT ON Llavero
         *
         * @param   void
         * @return  int
         */
        public int disparadorInsert() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llavero SET F_Baja_Ll = CURDATE() WHERE Cod_Llavero = " + getCodigo() + " AND F_Baja_Ll IS NULL;";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }

        /* disparadorRemove: Metodo que ejecuta el disparador sobre AFTER DELETE ON Llavero
         *
         * @param   void
         * @return  int
         */
        public int disparadorRemove() throws SQLException
        {
            // Eliminamos todas aquellas llaves que no formen parte de algun llavero o tengan existencia propia
            String query = "DELETE FROM Llave WHERE (Cod_Llave,F_Alta_L) NOT IN (SELECT DISTINCT Cod_Llave,F_Alta_L FROM Composicion_Llavero UNION SELECT DISTINCT Cod_Llave,F_Alta_L FROM Almacen_Llave);";
            UBD.ejecutarUpdate(query);

            // Eliminamos todos los almacenes que no contengan ningun elemento
            query = "DELETE FROM Almacen WHERE Cod_Alm NOT IN (SELECT DISTINCT Cod_Alm FROM Almacen_Llave JOIN Llave USING (Cod_Llave,F_Alta_L) WHERE F_Baja_L IS NULL UNION SELECT DISTINCT Cod_Alm FROM Almacen_Llavero JOIN Llavero USING (Cod_Llavero,F_Alta_Ll) WHERE F_Baja_Ll IS NULL);";            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }
}