// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Utilidades.UtilidadesBD;
import API.Interfaces_RMI.Nivel_I;
import java.rmi.*;
import java.sql.*;
import java.util.Calendar;



/** CLASE NIVEL:  Clase que representa un nivel de acceso al centro
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Nivel extends DosFechas implements Nivel_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Nivel:                  Nivel de acceso */
	private int                 Cod_Niv;
        /** Descripcion:            String con la descripcion del nivel */
	private String              Descripcion;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Nivel      Objeto de tipo Nivel
         */
        public Nivel () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Nivel con los valores pasados parametricamente
	 *
	 * @param   Cod_Niv     Nivel de acceso
         * @param   F_Alta_N    Fecha en la que se realiza el alta del nivel
         * @param   Descripcion String con la descripcion del nivel
	 * @return  Nivel	Objeto de tipo Nivel
	 */
	public Nivel(int Cod_Niv, Calendar F_Alta_N, String Descripcion) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta_N);

            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Cod_Niv<0) || (Cod_Niv>9) || (Descripcion.length()>50))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Niv = Cod_Niv;
            this.Descripcion = Descripcion;
	}
        /** Constructor: Constructor de clase que crea el objeto Nivel con los valores pasados parametricamente
	 *
	 * @param   Cod_Niv     Nivel de acceso
         * @param   F_Alta_N    Fecha en la que se realiza el alta del nivel
         * @param   F_Baja_N    Fecha en la que se realiza la baja del nivel
         * @param   Descripcion String con la descripcion del nivel
	 * @return  Nivel	Objeto de tipo Nivel
	 */
	public Nivel(int Cod_Niv, Calendar F_Alta_N, Calendar F_Baja_N, String Descripcion) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta_N,F_Baja_N);

            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Cod_Niv<0) || (Cod_Niv>9) || (Descripcion.length()>50))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Niv = Cod_Niv;
            this.Descripcion = Descripcion;
	}

        /** Constructor: Constructor de clase que crea el objeto Nivel con los valores pasados parametricamente
	 *
	 * @param   Cod_Niv     Nivel de acceso
	 * @return  Nivel	Objeto de tipo Nivel
	 */
	public Nivel(int Cod_Niv) throws DataException, SQLException, RemoteException
        {
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Niv<0) || (Cod_Niv>9))    throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Nivel WHERE Cod_Niv = " + Cod_Niv +
                " AND F_Baja_N IS NULL;";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Niv = rs.getInt(1);
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            this.Descripcion = rs.getString(4);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
        /** Constructor: Constructor de clase que crea el objeto Nivel con los valores pasados parametricamente
	 *
	 * @param   Cod_Niv     Nivel de acceso
         * @param   F_Alta_N    Fecha en la que se realiza el alta del nivel
	 * @return  Nivel	Objeto de tipo Nivel
	 */
	public Nivel(int Cod_Niv, String F_Alta_N) throws DataException, SQLException, RemoteException
        {
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((F_Alta_N == null) || (Cod_Niv<0) || (Cod_Niv>9) || (F_Alta_N.length()>19))
                throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Nivel WHERE Cod_Niv = " + Cod_Niv +
                " AND F_Alta_N = \"" + F_Alta_N + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Niv = rs.getInt(1);
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            this.Descripcion = rs.getString(4);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getNCodNiv: Metodo que obtiene el valor del atributo Cod_Niv
	 *
	 * @param   void
	 * @return  int     Nivel de acceso
	 */
	public int getCodNiv()                              { return this.Cod_Niv; }
        /** setCodNiv: Metodo que establece el valor del atributo Cod_Niv
	 *
	 * @param   Cod_Niv     int     Nivel de acceso
	 * @return  void
	 */
	public void setCodNiv(int Cod_Niv) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Niv<0) || (Cod_Niv>9))             throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Niv = Cod_Niv;
        }

        /** getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	 *
	 * @param   void
	 * @return  String	String con la descripcion del nivel
	 */
	public String getDescripcion()                      { return this.Descripcion; }
        /** setDescripcion: Metodo que establece el valor del atributo Descripcion
	 *
	 * @param   Descripcion     String     String con la descripcion del nivel
	 * @return  void
	 */
	public void setDescripcion(String Descripcion) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Descripcion.length()>50))     throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Descripcion = Descripcion;
        }

         
        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
        {
            // Devolvemos el resultado
            return "NIVEL:\nCod_Niv: " + getCodNiv() + "\n" +
            super.toString() + "\n" +
            "Descripcion: " + getDescripcion() + "\n";
	}


        // DML
        /** insertNivelBD: Metodo que inserta el Nivel en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertNivelBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Nivel VALUES ("  +
                    getCodNiv() + ",\"" +
                    calendarToString(getFechaI()) + "\"," +
                    (getFechaJ() == null ? "null" : "\"" + calendarToString(getFechaJ())  + "\"") + ",\"" +
                    getDescripcion() + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateNivelBD: Metodo que actualiza el Nivel en la BD
         *
         * @param   Nivel       Nivel con el que modificar
         * @return  int         Numero de filas actualizadas
         */
        public int updateNivelBD(Nivel Nivel) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Nivel SET " +
                    "Cod_Niv = " + Nivel.getCodNiv() + "," +
                    "F_Alta_N = \"" + calendarToString(Nivel.getFechaI()) + "\"," +
                    "F_Baja_N = " + (Nivel.getFechaJ() == null ? "null" : "\"" + calendarToString(Nivel.getFechaJ()) + "\"") + "," +
                    "Descripcion = \"" + Nivel.getDescripcion() +
                    "\" WHERE Cod_Niv = " + getCodNiv() + " AND F_Alta_N = \"" + calendarToString(getFechaI()) + "\";";
            
            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** darBajaNivelBD: Metodo que da de baja el Nivel en la BD
         *
         * @param   void
         * @return  int         Numero de filas actualizadas
         */
        public int darBajaNivelBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Nivel" +
                    " SET F_Baja_N = \"" + calendarToString(Calendar.getInstance()) + "\"" +
                    " WHERE Cod_Niv = " + getCodNiv() + " AND F_Alta_N = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** removeNivelBD: Metodo que borra un Nivel en la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeNivelBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Nivel WHERE Cod_Niv = " + getCodNiv() + " AND F_Alta_N = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

    
        // FUNCIONALIDADES
        /** obtenerNiveles: Metodo que obtiene los niveles existentes
         *
         * @param   void
         * @return  ResultSet   rs      Niveles existentes
         */
        public ResultSet obtenerNiveles() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Niv 'Codigos' FROM Nivel WHERE F_Baja_N IS NULL;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /** obtenerNivelesJCB: Metodo que obtiene los niveles existentes
         *
         * @param   void
         * @return  ResultSet   rs      Niveles existentes
         */
        public ResultSet obtenerNivelesJCB() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT 'Ninguno', 'Ninguno' UNION SELECT Cod_Niv, Descripcion FROM Nivel WHERE F_Baja_N IS NULL;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
        
        /* nivelMinimo: Metodo que devuelve el nivel minimo existente
         *
         * @return  int   Nivel          Nivel minimo existente
         */
        public int nivelMinimo() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT MIN(Cod_Niv)" +
                    " FROM Nivel" +
                    " WHERE F_Baja_N IS NULL;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int Nivel = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Nivel;
        }

        /* nivelPorDefecto: Metodo que devuelve el nivel por defecto
         *
         * @param   void
         * @return  int   Nivel          Nivel por defecto
         */
        public int nivelPorDefecto() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Codigo" +
                    " FROM Codigo" +
                    " WHERE Codigo NOT IN (SELECT Cod_Niv" +
                                            " FROM Nivel" +
                                            " WHERE F_Baja_N IS NULL)" +
                    " LIMIT 0,1;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int Nivel = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Nivel;
        }

        /* eliminarNiveles: Metodo que borra todos los niveles mas antiguos de Dias dias
         *
         * @param   Dias    Numero de dias transcurridos desde la baja del nivel
         * @return  int     Numero de niveles eliminados
         */
        public int eliminarNiveles(int Dias) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Nivel WHERE ADDDATE(DATE(F_Baja_N),INTERVAL " + Dias + " DAY)<=CURDATE();";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /* disparadorInsert: Metodo que ejecuta el disparador sobre BEFORE INSERT ON Nivel
         *
         * @param   void
         * @return  int
         */
        public int disparadorInsert() throws SQLException
        {
            // Damos de baja el nivel a insertar
            String query = "UPDATE Nivel SET F_Baja_N = CURDATE() WHERE Cod_Niv = " + getCodNiv() + " AND F_Baja_N IS NULL;";
            UBD.ejecutarUpdate(query);

            // Damos de baja el color cuyo nivel insertamos
            query = "UPDATE Color SET F_Baja_C = CURDATE() WHERE Cod_Niv = " + getCodNiv() + " AND F_Baja_C IS NULL;";
            UBD.ejecutarUpdate(query);

            // Damos de baja las tarjetas cuyo color tiene el nivel que insertamos
            query = "UPDATE Tarjeta SET F_Baja_T = CURDATE() WHERE (Cod_Color,F_Alta_C) IN (SELECT Cod_Color,F_Alta_C FROM Color JOIN Nivel USING (Cod_Niv,F_Alta_N) WHERE Cod_Niv = " + getCodNiv() + " AND F_Baja_C IS NULL) AND F_Baja_T IS NULL;";
            UBD.ejecutarUpdate(query);
        
            // Devolvemos el resultado
            return 0;
        }

        /* disparadorUpdate: Metodo que ejecuta el disparador sobre BEFORE UPDATE ON Nivel
         *
         * @param   void
         * @return  int
         */
        public int disparadorUpdate() throws SQLException
        {
            // Damos de baja el nivel a insertar
            String query = "UPDATE Color SET F_Baja_C = " + (getFechaJ() == null ? "null" : ("\"" + calendarToString(getFechaJ()) + "\"")) + " WHERE Cod_Niv = " + getCodNiv() + " AND F_Alta_N = \"" + calendarToString(getFechaI()) + "\";";
            UBD.ejecutarUpdate(query);

            // Damos de baja las tarjetas cuyo color tiene el nivel que insertamos
            query = "UPDATE Tarjeta SET F_Baja_T = " + (getFechaJ() == null ? "null" : ("\"" + calendarToString(getFechaJ()) + "\"")) + " WHERE (Cod_Color,F_Alta_C) IN (SELECT Cod_Color,F_Alta_C FROM Color WHERE Cod_Niv = " + getCodNiv() + " AND F_Alta_N = \"" + calendarToString(getFechaI()) + "\");";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }
}