// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Nota_I;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Calendar;



/** CLASE NOTA:  Clase que representa una nota
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Nota extends UnaFecha implements Nota_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** DNI_A:                              Persona autora de la nota */
        private Personal                        Redactor;
        /** Tema:                               String con el tema de la nota */
        private String                          Tema;
        /** Texto:                              String con el texto de la nota */
        private String                          Texto;
        /** Descartada:                         Denota si la nota ha sido descartada por el redactor */
        private boolean                         Descartada;
        /** Destinatarios:                      Lista con los destinatarios de la nota */
        private HashMap<String,Destinatario>    Destinatarios;

        /** UBD:                                Objeto de tipo UtilidadesBD */
        public static UtilidadesBD              UBD = new UtilidadesBD();
        /** UCAL:                               Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar        UCAL = new UtilidadesCalendar();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Nota      Objeto de tipo Nota
         */
        public Nota () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Nota con los valores pasados parametricamente
	 *
	 * @param   Fecha           Fecha de redaccion de la nota
         * @param   DNI_A           String con el DNI del autor de la nota
         * @param   Tema            String con el tema de la nota
         * @param   Texto           String con el texto de la nota
         * @param   Destinatarios   Lista con los destinatarios de la nota
	 * @return  Nota            Objeto de tipo Nota
	 */
	public Nota (Calendar Fecha, Personal Redactor, String Tema, String Texto, HashMap<String,Destinatario> Destinatarios) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(Fecha);
            // Comprobamos que los parametros pasados son correctos
            if ((Redactor == null) || (Tema == null) || (Texto == null) || (Destinatarios == null) || (Destinatarios.isEmpty()) || (Tema.length()>50) || (Texto.length()>500))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Redactor = Redactor;
            this.Tema = Tema;
            this.Texto = Texto;
            this.Descartada = false;
            this.Destinatarios = Destinatarios;
	}
	/** Constructor: Constructor de clase que crea el objeto Nota con los valores pasados parametricamente
	 *
	 * @param   Fecha           Fecha de redaccion de la nota
         * @param   DNI_A           String con el DNI del autor de la nota
         * @param   Tema            String con el tema de la nota
         * @param   Texto           String con el texto de la nota
         * @param   Destinatarios   Lista con los destinatarios de la nota
	 * @return  Nota            Objeto de tipo Nota
	 */
	public Nota (String Fecha, String Redactor) throws DataException, SQLException, RemoteException
        {
            // Llamamos al constructor base
            super();
            // Comprobamos que los parametros pasados son correctos
            if ((Fecha == null) || (Redactor == null) || (Redactor.length()>9))     throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Nota WHERE Fecha = \"" + Fecha +
                "\" AND DNI_A = \"" + Redactor + "\";";

            ResultSet rs = UBD.ejecutarQuery(query);
            // Realizamos la asignacion de valores a atributos de la nota
            rs.next();
            setFechaI(stringToCalendar(rs.getString(1).substring(0,19)));
            this.Redactor = new Personal(rs.getString(2));
            this.Tema = rs.getString(3);
            this.Texto = rs.getString(4);
            this.Descartada = false;

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Realizamos la asignacion de los destinatarios
            this.Destinatarios = getDestinatariosBD();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getRedactor: Metodo que obtiene el valor del atributo Redactor
	 *
	 * @param   void
	 * @return  Persona	Persona autora de la nota
	 */
	public Personal getRedactor()                    { return this.Redactor; }
        /** setRedactor: Metodo que establece el valor del atributo Redactor
	 *
	 * @param   Redactor     Persona     Persona autora de la nota
	 * @return  void
	 */
	public void setRedactor(Personal Redactor) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
                if (Redactor == null)                   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            else                                        this.Redactor = Redactor;
        }

        /** getTema: Metodo que obtiene el valor del atributo Tema
	 *
	 * @param   void
	 * @return  String	String con el tema de la nota
	 */
	public String getTema()                         { return this.Tema; }
        /** setTema: Metodo que establece el valor del atributo Tema
	 *
	 * @param   Tema     String     String con el tema de la nota
	 * @return  void
	 */
	public void setTema(String Tema) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Tema == null) || (Tema.length()>50))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Tema = Tema;
        }

        /** getTexto: Metodo que obtiene el valor del atributo Texto
	 *
	 * @param   void
	 * @return  String	String con el texto de la nota
	 */
	public String getTexto()			{ return this.Texto; }
        /** setTexto: Metodo que establece el valor del atributo Texto
	 *
	 * @param   Texto     String     String con el texto de la nota
	 * @return  void
	 */
	public void setTexto(String Texto) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Texto == null) || (Texto.length()>500))    throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Texto = Texto;
        }

        /** getDescartada: Metodo que obtiene el valor del atributo Descartada
	 *
	 * @param   void
	 * @return  boolean	Denota si la nota ha sido descartada por el redactor de la misma
	 */
	public boolean getDescartada()                              { return this.Descartada; }
        /** setDescartada: Metodo que establece el valor del atributo Descartada
	 *
	 * @param   Descartada      boolean     Denota si la nota ha sido descartada por el redactor de la misma
	 * @return  void
	 */
	public void setDescartada(boolean Descartada)               { this.Descartada = Descartada; }

        /** getDestinatarios: Metodo que obtiene el valor del atributo Destinatarios
	 *
	 * @param   void
	 * @return  HashMap	Lista con los destinatarios de la nota
	 */
	public HashMap<String,Destinatario> getDestinatarios()        { return this.Destinatarios; }
        /** setDestinatarios: Metodo que establece el valor del atributo Destinatarios
	 *
	 * @param   Destinatarios     HashMap     Lista con los destinatarios de la nota
	 * @return  void
	 */
	public void setDestinatarios(HashMap<String,Destinatario> Destinatarios) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Destinatarios == null) || (Destinatarios.isEmpty()))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Destinatarios = Destinatarios;
        }

        /** addDestinatario: Metodo que añade un destinatario a los ya existentes
         *
         * @param   Destinatario        Destinatario  Nuevo destinatario de la nota
         * @return  void
         */
        public void addDestinatario(Destinatario Destinatario) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Destinatario == null)                       throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getDestinatarios().put(Destinatario.getReceptor().getDNI(),Destinatario);
        }
        /** updateDestinatario: Metodo que modifica un destinatario existente
         *
         * @param   Dest_old,Dest_new        Destinatario  Destinatarios a modificar y por el que reemplazar
         * @return  void
         */
        public void updateDestinatario(Destinatario Dest_old, Destinatario Dest_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Dest_new == null)                           throw new DataException();

            // Comprobamos si existe el destinatario, en cuyo caso lo cambiamos por el nuevo
            if (getDestinatarios().containsKey(Dest_old.getReceptor().getDNI()))
            {
                removeDestinatario(Dest_old);
                addDestinatario(Dest_new);
            }
        }
        /** removeDestinatario: Metodo que elimina un destinatario de los existentes
         *
         * @param   Destinatario        Destinatario  Destinatario a eliminar
         * @return  void
         */
        public void removeDestinatario(Destinatario Destinatario) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Destinatario == null)                       throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getDestinatarios().remove(Destinatario.getReceptor().getDNI());
        }
        /** getDestinatario: Metodo que obtiene un destinatario de los existentes
         *
         * @param   String          DNI         DNI del destinatario a obtener
         * @return  Destinatario                Destinatario solicitado
         */
        public Destinatario getDestinatario(String DNI) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (DNI.length()>9))                  throw new DataException();
            // Devolvemos el resultado
            return this.getDestinatarios().get(DNI);
        }

        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // String con los destinatarios de la nota
            String dests = "";

            // Añadimos las Strings con todos los destinatarios
            Iterator it = getDestinatarios().entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKDest = (Map.Entry) it.next();
                Destinatario dest = (Destinatario) VKDest.getValue();
                dests = dests + dest.toString() + "\n";
            }

            // Devolvemos el resultado
            return super.toString() + "\n" +
            getRedactor().toString() + "\n" +
            "Tema: " + getTema() + "\n" +
            "Texto: " + getTexto() + "\n" +
            "Descartada: " + (getDescartada() ? "Sí" : "No") + "\n\n" +
            dests + "\n";
	}


        // DML
        /** insertDestinatarioBD: Metodo que inserta un Destinatario en la BD
         *
         * @param   Destinatario    Destinatario    Destinatario de la nota
         * @return  int                             Numero de filas insertadas
         */
        public int insertDestinatarioBD(Destinatario Destinatario) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Destinatario == null)                           throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Destinatario VALUES (\""  +
                calendarToString(getFechaI()) + "\",\"" +
                getRedactor().getDNI() + "\",\"" +
                Destinatario.getReceptor().getDNI() + "\"," +
                Destinatario.getLeida() + "," +
                Destinatario.getDescartada() + ");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateDestinatarioBD: Metodo que actualiza un Destinatario de la BD
         *
         * @param   Dest_old    Destinatario a modificar
         * @param   Dest_new    Destinatario de reemplazo
         * @return  int         Numero de filas actualizadas
         */
        public int updateDestinatarioBD(Destinatario Dest_old, Destinatario Dest_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Dest_old == null) || (Dest_new == null))               throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Destinatario SET " +
                    "Fecha = \"" + calendarToString(getFechaI()) + "\"," +
                    "DNI_A = \"" + getRedactor().getDNI() + "\"," +
                    "DNI_D = \"" + Dest_new.getReceptor().getDNI() + "\"," +
                    "Leida = " + (Dest_new.getLeida() ? "true" : "false") + "," +
                    "Descartada = " + (Dest_new.getDescartada() ? "true" : "false") +
                    " WHERE Fecha = \"" + calendarToString(getFechaI()) + "\" AND DNI_A = \"" +
                    getRedactor().getDNI() + "\" AND DNI_D = \"" + Dest_old.getReceptor().getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeDestinatarioBD: Metodo que borra un destinatario de la Nota en la BD
         *
         * @param   Destinatario        Destinatario    Destinatario a eliminar de la nota
         * @return  int                                 Numero de filas eliminadas
         */
        public int removeDestinatarioBD(Destinatario Destinatario) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Destinatario == null)                               throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Destinatario WHERE Fecha = \"" + calendarToString(getFechaI()) +
                    "\" AND DNI_A = \"" + getRedactor().getDNI() + "\" AND DNI_D = \"" + Destinatario.getReceptor().getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getDestinatarioBD: Metodo que crea un Destinatario (receptor de esta nota) con los datos existentes en la BD
         *
         * @param   String          DNI                 DNI del destinatario a recuperar
         * @return  Destinatario    Destinatario        Destinatario creado con los datos recuperados
         */
        public Destinatario getDestinatarioBD(String DNI) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (DNI.length()>9))            throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Leida,Descartada FROM Destinatario WHERE Fecha = \"" + calendarToString(getFechaI()) +
                "\" AND DNI_A = \"" + getRedactor().getDNI() + "\" AND DNI_D = \"" + DNI + "\";";

            ResultSet rs = UBD.ejecutarQuery(query);


            // Realizamos la asignacion de valores a atributos de la nota
            rs.next();
            boolean Leida = (rs.getInt(1) == 0 ? false : true);
            boolean Descartada = (rs.getInt(2) == 0 ? false : true);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return new Destinatario(new Personal(DNI),Leida,Descartada);
        }

        /** insertDestinatariosBD: Metodo que inserta los destinatarios de esta nota en la BD
         *
         * @param   void
         * @return  int                                             Numero de filas insertadas
         */
        public int insertDestinatariosBD() throws DataException, SQLException
        {
            // Insertamos los destinatarios existentes en este objeto Nota
            return insertDestinatariosBD(getDestinatarios());
        }
        /** insertDestinatariosBD: Metodo que inserta los destinatarios suministrados en la BD
         *
         * @param   HashMap<String,Destinatario>     Destinatarios      Destinatarios de la nota
         * @return  int                                                 Numero de filas (Destinatarios) insertadas
         */
        public int insertDestinatariosBD(HashMap<String,Destinatario> Destinatarios) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Destinatarios == null)                              throw new DataException();

            // Insertamos cada destinatario del HashMap
            Iterator it = Destinatarios.entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos el destinatario
                Map.Entry VKDest = (Map.Entry) it.next();
                Destinatario Destinatario = (Destinatario) VKDest.getValue();

                // Insertamos el destinatario
                insertDestinatarioBD (Destinatario);
            }

            return Destinatarios.size();
        }
        /** updateDestinatariosBD: Metodo que actualiza los destinatarios de una nota
         *
         * @param   HashMap<String,Destinatario>     Destinatarios      Destinatarios de la nota
         * @return  int                                                 Numero de filas (Destinatarios) actualizadas
         */
        public int updateDestinatariosBD(HashMap<String,Destinatario> Destinatarios) throws DataException, SQLException
        {
            // Filas actualizadas
            int res = 0;

            // Recorremos el HashMap nuevo, actualizando los destinatarios existentes e insertando los nuevos
            Iterator it_new = Destinatarios.entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKDest = (Map.Entry) it_new.next();
                Destinatario dest_new = (Destinatario) VKDest.getValue();

                // Si existe el destinatario -> Lo actualizamos
                if (getDestinatarios().containsKey(dest_new.getReceptor().getDNI()))    updateDestinatarioBD(getDestinatarios().get(dest_new.getReceptor().getDNI()),dest_new);
                // Si no existe -> Lo insertamos
                else                                                                    insertDestinatarioBD(dest_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }
            
            // Finalmente, recorremos el HashMap viejo eliminando aquellos destinatarios que no consten en el nuevo
            Iterator it_old = getDestinatarios().entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKDest = (Map.Entry) it_old.next();
                Destinatario dest_old = (Destinatario) VKDest.getValue();

                // Si el destinatario viejo no existe como nuevo -> Lo eliminamos y actualizamos el contador de filas actualizadas
                if (! Destinatarios.containsKey(dest_old.getReceptor().getDNI()))
                {
                    removeDestinatarioBD(dest_old);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removeDestinatariosBD: Metodo que borra los destinatarios de una Nota en la BD
         *
         * @param   void
         * @return  int                             Numero de filas (Destinatarios) eliminadas
         */
        public int removeDestinatariosBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Destinatario WHERE Fecha = \"" + calendarToString(getFechaI()) +
                    "\" AND DNI_A = \"" + getRedactor().getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getDestinatariosBD: Metodo que obtiene todos los destinatarios de una Nota en la BD
         *
         * @param   void
         * @return  HashMap<String,Destinatario>        HashMap con los destinatarios de la Nota
         */
	public HashMap<String,Destinatario> getDestinatariosBD() throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de los destinatarios
            HashMap<String,Destinatario> Destinatarios = new HashMap();

            String query = "SELECT DNI_D,Leida,Descartada FROM Destinatario WHERE Fecha = \"" + UCAL.calendarToString(getFechaI()) +
                "\" AND DNI_A = \"" + getRedactor().getDNI() + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            while(rs.next())    Destinatarios.put(rs.getString(1),new Destinatario(new Personal(rs.getString(1)),rs.getBoolean(2),rs.getBoolean(3)));

            // Cerramos la conexion y devolvemos el resultado
            UBD.cerrarConexion();
            return Destinatarios;
	}

        /** insertNotaBD: Metodo que inserta la Nota en la BD
         *
         * @param   void
         * @return  int     Resultado
         */
        public int insertNotaBD() throws DataException, SQLException
        {
            // INSERTAMOS LA NOTA
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Nota VALUES (\""  +
                    calendarToString(getFechaI()) + "\",\"" +
                    getRedactor().getDNI() + "\",\"" +
                    getTema() + "\",\"" +
                    getTexto() + "\"," +
                    getDescartada() + ");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);


            // Si la nota ha sido insertada correctamente -> INSERTAMOS LOS DESTINATARIOS
            if (res == 1)
            {
                Iterator it = getDestinatarios().entrySet().iterator();
                while (it.hasNext())
                {
                    // Obtenemos el destinatario
                    Map.Entry VKDest = (Map.Entry) it.next();
                    Destinatario Destinatario = (Destinatario) VKDest.getValue();

                    // Insertamos el destinatario
                    insertDestinatarioBD (Destinatario);
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** updateNotaBD: Metodo que actualiza la Nota en la BD
         *
         * @param   Nota            Nota a modificar
         * @return  int             Numero de filas (notas) actualizadas
         */
        public int updateNotaBD(Nota Nota) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Nota == null)                                       throw new DataException();

            // ACTUALIZAMOS LA NOTA
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Nota SET " +
                    "Fecha = \"" + calendarToString(Nota.getFechaI()) + "\"," +
                    "DNI_A = \"" + Nota.getRedactor().getDNI() + "\"," +
                    "Tema = \"" + Nota.getTema() + "\"," +
                    "Texto = \"" + Nota.getTexto() + "\"," +
                    "Descartada = " + Nota.getDescartada() +
                    " WHERE Fecha = \"" + calendarToString(getFechaI()) + "\" AND DNI_A = \"" +
                    getRedactor().getDNI() + "\";";

            // Ejecutamos la sentencia
            int res = UBD.ejecutarUpdate(query);

            // ACTUALIZAMOS LOS DESTINATARIOS
            // Actualizamos la PK de la nota para una correcta actualizacion de los destinatarios
            setFechaI(Nota.getFechaI());
            setRedactor(Nota.getRedactor());
            // Actualizamos los destinatarios y devolvemos el resultado
            return updateDestinatariosBD(Nota.getDestinatarios()) + res;
        }
        /** removeNotaBD: Metodo que borra una Nota de la BD
         *
         * @param   void
         * @return  int             Numero de filas (notas) eliminadas
         */
        public int removeNotaBD() throws DataException, SQLException
        {
            // Borramos la nota y devolvemos el resultado
            return removeNotaBD(this);
        }
        /** removeNotaBD: Metodo que borra una Nota de la BD
         *
         * @param   Nota            Nota a borrar
         * @return  int             Numero de filas (notas) eliminadas
         */
        public int removeNotaBD(Nota Nota) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Nota == null)                                       throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Nota WHERE Fecha = \"" + calendarToString(Nota.getFechaI()) + "\" AND DNI_A = \"" + Nota.getRedactor().getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }



        // FUNCIONALIDADES
        /* obtenerDestinatarios: Metodo que devuelve todos los usuarios del sistema que pueden usar el sistema de mensajeria
         *
         * @param   String      Filtro      Filtro que debe pasar el usuario
         * @return  ResultSet   rs          Usuarios que pueden utilizar el sistema de mensajería
         */
        public ResultSet obtenerDestinatarios(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Apellido1 \"Primer apellido\",Apellido2 \"Segundo apellido\", Nombre" +
                    " FROM ( SELECT Apellido1, Apellido2, Nombre" +
                            " FROM Personal JOIN Vigilante USING (DNI) JOIN Cuenta USING (DNI)" +
                                " UNION" +
                            " SELECT Apellido1, Apellido2, Nombre" +
                            " FROM Personal JOIN Residente USING (DNI) JOIN Cuenta USING (DNI)) Destinatarios" +
                    " WHERE Apellido1 LIKE \"%" + Filtro + "%\" OR Apellido2 LIKE \"%" + Filtro + "%\" OR Nombre LIKE \"%" + Filtro + "%\"" +
                    " ORDER BY Apellido1, Apellido2, Nombre;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerBandejaEntrada: Metodo que devuelve las notas enviadas a un usuario
         *
         * @param   String      DNI         DNI del usuario receptor de las notas
         * @return  ResultSet   rs          Notas destinadas al usuario
         */
        public ResultSet obtenerBandejaEntrada(String DNI) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT CONCAT(Apellido1,' ',Apellido2,', ',Nombre) De,Tema,Fecha" +
                    " FROM Nota JOIN Destinatario USING (Fecha,DNI_A) JOIN Personal ON (Nota.DNI_A=Personal.DNI)" +
                    " WHERE DNI_D = \"" + DNI + "\" AND Destinatario.Descartada = false" +
                    " ORDER BY Fecha DESC;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerBandejaSalida: Metodo que devuelve las notas enviadas por un usuario
         *
         * @param   String      DNI         DNI del usuario redactor de las notas
         * @return  ResultSet   rs          Notas redactadas por el usuario
         */
        public ResultSet obtenerBandejaSalida(String DNI) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Tema,Fecha" +
                    " FROM Nota" +
                    " WHERE DNI_A = \"" + DNI + "\" AND Descartada = false" +
                    " ORDER BY Fecha DESC;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* leida: Metodo que establece la nota como leida por el usuario
         *
         * @param   String      DNI         DNI del usuario que la lee
         * @return  void
         */
        public void leida(String DNI) throws DataException, SQLException
        {
            // Obtenemos el destinatario viejo
            Destinatario dest_old = getDestinatario(DNI);
            // Obtenemos el destinatario nuevo
            Destinatario dest_new = getDestinatario(DNI);
            dest_new.setLeida(true);

            // Actualizamos el destinatario
            updateDestinatarioBD(dest_old, dest_new);
        }

        /* descartada: Metodo que establece la nota como descartada por el destinatario
         *
         * @param   String      DNI       DNI del destinatario
         * @return  void
         */
        public void descartada(String DNI) throws DataException, SQLException
        {
            // Obtenemos el destinatario viejo
            Destinatario dest_old = getDestinatario(DNI);

            // Obtenemos el destinatario nuevo
            Destinatario dest_new = getDestinatario(DNI);
            dest_new.setDescartada(true);

            // Actualizamos el destinatario
            updateDestinatarioBD(dest_old, dest_new);
        }

        /* descartada: Metodo que establece la nota como descartada por el redactor
         *
         * @param   void
         * @return  void
         */
        public void descartada() throws DataException, SQLException, RemoteException
        {
            // Establecemos la nota como descartada
            setDescartada(true);

            // Actualizamos la nota en la BD
            updateNotaBD(this);
        }

        /* disparadorAutor: Metodo que ejecuta el disparador sobre AFTER UPDATE ON Nota
         *
         * @param   void
         * @return  int
         */
        public int disparadorAutor() throws SQLException
        {
            // Eliminamos las notas cuyos lectores y autor la hayan descartadp
            String query = "DELETE FROM Nota WHERE Descartada = true AND (Fecha,DNI_A) NOT IN (SELECT DISTINCT Fecha,DNI_A FROM Destinatario WHERE Descartada = false);";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }

        /* disparadorDestinatario: Metodo que ejecuta el disparador sobre AFTER UPDATE ON Destinatario
         *
         * @param   void
         * @return  int
         */
        public int disparadorDestinatario() throws SQLException
        {
            // Eliminamos las notas cuyos lectores y autor la hayan descartadp
            String query = "DELETE FROM Nota WHERE Descartada = true AND (Fecha,DNI_A) NOT IN (SELECT DISTINCT Fecha,DNI_A FROM Destinatario WHERE Descartada = false);";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }
}