// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Ocasional_I;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;



/** CLASE OCASIONAL:  Clase que representa un trabajador ocasional
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Ocasional extends Personal implements Ocasional_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Empresa:                            String con la empresa para la que trabaja */
	String                                  Empresa;
        /** Acceso:                             Tipo de acceso otorgado */
	private T_Acceso                        Acceso;
        /** Fecha:                              Fecha de caducidad del permiso */
        private UnaFecha                        F_Caducidad;
        /** Notas:                              Otros datos de interes */
        private String                          Notas;
        /** Llaves:                             Llaves prestables */
        private HashMap<String,Llave>           Llaves;

        /** UBD:                                Objeto de tipo UtilidadesBD */
        public static UtilidadesBD              UBD = new UtilidadesBD();
        /** UCAL:                               Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar        UCAL = new UtilidadesCalendar();



	// METODOS -------------------------------------------------------------
        // CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Ocasional      Objeto de tipo Ocasional
         */
        public Ocasional () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
         * @param   Apellido1   String con el primer apellido de la persona
         * @param   Apellido2   String con el segundo apellido de la persona
         * @param   Nombre      String con el nombre de la persona
         * @param   Insertador  Insertador de la persona en la BD
         * @param   Empresa     String con la empresa para la que trabaja
         * @param   Acceso      Tipo de acceso otorgado
         * @param   F_Caducidad Fecha de caducidad del permiso
         * @param   Telefonos   Telefonos de contacto
         * @param   Emails      Emails de contacto
	 * @return  Ocasional   Objeto de tipo Ocasional
	 */
	public Ocasional (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, String Empresa, T_Acceso Acceso, Calendar F_Caducidad, ArrayList<String> Telefonos, ArrayList<String> Emails) throws DataException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI, Apellido1, Apellido2, Nombre, Insertador, Telefonos, Emails);
            // Comprobamos que los parametros pasados son correctos
            if ((Empresa == null) || (Empresa.length()>50) || (Acceso == null) || (F_Caducidad == null))  throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Empresa = Empresa;
            this.Acceso = Acceso;
            this.F_Caducidad = new UnaFecha(F_Caducidad);
            this.Notas = null;
            this.Llaves = new HashMap();
	}
        /** Constructor: Constructor de clase que crea el objeto Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
         * @param   Apellido1   String con el primer apellido de la persona
         * @param   Apellido2   String con el segundo apellido de la persona
         * @param   Nombre      String con el nombre de la persona
         * @param   Insertador  Insertador de la persona en la BD
         * @param   Empresa     String con la empresa para la que trabaja
         * @param   Acceso      Tipo de acceso otorgado
         * @param   F_Caducidad Fecha de caducidad del permiso
         * @param   Notas       Otros datos de interes
         * @param   Llaves      Llaves prestables
         * @param   Telefonos   Telefonos de contacto
         * @param   Emails      Emails de contacto
	 * @return  Ocasional   Objeto de tipo Ocasional
	 */
	public Ocasional (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, String Empresa, T_Acceso Acceso, Calendar F_Caducidad, String Notas, HashMap<String,Llave> Llaves, ArrayList<String> Telefonos, ArrayList<String> Emails) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI, Apellido1, Apellido2, Nombre, Insertador, Telefonos, Emails);
            // Comprobamos que los parametros pasados son correctos
            if ((Empresa == null) || (Empresa.length()>50) || (Acceso == null) || (F_Caducidad == null) || (Notas.length()>250))  throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Empresa = Empresa;
            this.Acceso = Acceso;
            this.F_Caducidad = new UnaFecha(F_Caducidad);
            this.Notas = Notas;
            this.Llaves = getLlavesBD();
	}
        /** Constructor: Constructor de clase que crea el objeto Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
         * @param   Empresa     String con la empresa para la que trabaja
	 * @return  Ocasional   Objeto de tipo Ocasional
	 */
	public Ocasional (String DNI, String Empresa) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos base
            super(DNI);
            // Comprobamos que los parametros pasados son correctos
            if ((Empresa == null) || (Empresa.length()>50))  throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Cod_Acc,F_Caducidad,Notas FROM Ocasional WHERE DNI = \"" + DNI +
                    "\" AND Empresa = \"" + Empresa + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Empresa = Empresa;
            this.Acceso = new T_Acceso(rs.getString(1));
            this.F_Caducidad = new UnaFecha(UCAL.stringToCalendar(rs.getString(2).substring(0,19)));
            this.Notas = rs.getString(3);
            this.Llaves = getLlavesBD();

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
        /** Constructor: Constructor de clase que crea el objeto Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
	 * @return  Ocasional   Objeto de tipo Ocasional
	 */
	public Ocasional (String DNI) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos base
            super(DNI);

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Empresa,Cod_Acc,F_Caducidad,Notas FROM Ocasional WHERE DNI = \"" + DNI +
                    "\" ORDER BY Empresa;";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Empresa = rs.getString(1);
            this.Acceso = new T_Acceso(rs.getString(2));
            this.F_Caducidad = new UnaFecha(UCAL.stringToCalendar(rs.getString(3).substring(0,19)));
            this.Notas = rs.getString(4);
            this.Llaves = getLlavesBD();

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getEmpresa: Metodo que obtiene el valor del atributo Empresa
	 *
	 * @param   void
	 * @return  String	String con la empresa para la que trabaja
	 */
	public String getEmpresa()                      { return this.Empresa; }
        /** setEmpresa: Metodo que establece el valor del atributo Empresa
	 *
	 * @param   Empresa     String     String con la empresa para la que trabaja
	 * @return  void
	 */
	public void setEmpresa(String Empresa) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Empresa == null) || (Empresa.length()>50)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Empresa = Empresa;
        }

        /** getAcceso: Metodo que obtiene el valor del atributo Acceso
	 *
	 * @param   void
	 * @return  T_Acceso     Tipo de acceso otorgado
	 */
	public T_Acceso getAcceso()                         { return this.Acceso; }
        /** setAcceso: Metodo que establece el valor del atributo Acceso
	 *
	 * @param   Acceso     Tipo de acceso otorgado
	 * @return  void
	 */
	public void setAcceso(T_Acceso Acceso) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Acceso == null)                             throw new DataException();
            // Realizamos la asignacion del atributo
            this.Acceso = Acceso;
        }

        /** getFCaducidad: Metodo que obtiene el valor del atributo F_Caducidad
	 *
	 * @param   void
	 * @return  Calendar     Fecha de caducidad del permiso
	 */
	public Calendar getFCaducidad()                     { return this.F_Caducidad.getFechaI(); }
        /** setFCaducidad: Metodo que establece el valor del atributo F_Caducidad
	 *
	 * @param   F_Caducidad     Fecha de caducidad del permiso
	 * @return  void
	 */
	public void setFCaducidad(Calendar F_Caducidad) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (F_Caducidad == null)                        throw new DataException();
            // Realizamos la asignacion del atributo
            this.F_Caducidad.setFechaI(F_Caducidad);
        }

        /** getNotas: Metodo que obtiene el valor del atributo Notas
	 *
	 * @param   void
	 * @return  String     Otros datos de interes
	 */
	public String getNotas()                            { return this.Notas; }
        /** setNotas: Metodo que establece el valor del atributo Notas
	 *
	 * @param   Notas     Otros datos de interes
	 * @return  void
	 */
	public void setNotas(String Notas) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Notas.length()>250)                         throw new DataException();
            // Realizamos la asignacion del atributo
            this.Notas = Notas;
        }

        /** getLlaves: Metodo que obtiene el valor del atributo Llaves
	 *
	 * @param   void
	 * @return  HashMapa	Llaves prestables al Ocasional
	 */
	public HashMap<String,Llave> getLlaves()                      { return this.Llaves; }
        /** setLlaves: Metodo que establece el valor del atributo Llaves
	 *
	 * @param   Llaves     HashMap     Llaves prestables al Ocasional
	 * @return  void
	 */
	public void setLlaves(HashMap<String,Llave> Llaves)
        {
            // Realizamos la asignacion de valores a los atributos
            this.Llaves = Llaves;
        }
        /** addLlave: Metodo que añade una llave a las ya existentes
         *
         * @param   Llave        Llave  Llave nueva a añadir
         * @return  void
         */
        public void addLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().put("L - " + Llave.getCodigo() + " - " + UCAL.calendarToString(Llave.getFechaI()),Llave);
        }
        /** updateLlave: Metodo que actualiza una llave existente
         *
         * @param   Llave_old       Llave       Llave a modificar
         * @param   Llave_new       Llave       Llave por la que reemplazar
         * @return  void
         */
        public void updateLlave(Llave Llave_old, Llave Llave_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llave_old == null) || (Llave_new == null))   throw new DataException();
            // Reemplazamos la zona vieja por la nueva
            if (getLlaves().containsKey("L - " + Llave_old.getCodigo() + " - " + UCAL.calendarToString(Llave_old.getFechaI())));
            {
                removeLlave(Llave_old);
                addLlave(Llave_new);
            }
        }
        /** removeLlave: Metodo que elimina una llave de las existentes
         *
         * @param   Llave        Llave  Llave a eliminar
         * @return  void
         */
        public void removeLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().remove("L - " + Llave.getCodigo() + " - " + UCAL.calendarToString(Llave.getFechaI()));
        }
        /** getLlave: Metodo que obtiene una Llave de las existentes
         *
         * @param   String          Key_Llave   Key de la Llave a obtener
         * @return  Llave                       Llave solicitada
         */
        public Llave getLlave(String Key_Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Key_Llave == null) || (Key_Llave.length()>29))     throw new DataException();
            // Devolvemos el resultado
            return this.getLlaves().get(Key_Llave);
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return super.toString() + "\n" +
            "Acceso: " + getAcceso() + "\n" +
            "F_Caducidad: " + UCAL.calendarToString(getFCaducidad()) + "\n" +
            "Notas: " + (getNotas() == null ? "" : getNotas().toString()) + "\n";
	}


        // DML
        /** insertLlaveBD: Metodo que inserta una Llave prestable a un Ocasional
         *
         * @param   Llave   Llave   Llave prestable al Ocasional
         * @return  int             Numero de filas insertadas
         */
        public int insertLlaveBD(Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                                      throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Prestamo_Ocasional VALUES (" +
                Llave.getCodigo() + ",\"" +
                UCAL.calendarToString(Llave.getFechaI()) + "\",\"" +
                getDNI() + "\",\"" +
                getEmpresa() + "\");";

                // Ejecutamos la consulta y devolvemos el resultado
                return UBD.ejecutarUpdate(query);
        }
        /** updateLlaveBD: Metodo que actualiza una llave prestable al Ocasional
         *
         * @param   Llave_old       Llave a modificar
         * @param   Llave_new       Llave de reemplazo
         * @return  int             Resultado
         */
        public int updateLlaveBD(Llave Llave_old, Llave Llave_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llave_old == null) || (Llave_new == null))         throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Prestamo_Ocasional SET " +
                "Cod_Llave = " + Llave_new.getCodigo() + "," +
                "F_Alta_L = \"" + UCAL.calendarToString(Llave_new.getFechaI()) + "\"," +
                "\" WHERE Cod_Llave = " + Llave_old.getCodigo() + " AND F_Alta_L = \"" +
                UCAL.calendarToString(Llave_old.getFechaI()) + "\" AND DNI = \"" + getDNI() + "\" AND" +
                " Empresa = \"" + getEmpresa() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** removeLlaveBD: Metodo que borra una llave de las prestables al Ocasional
         *
         * @param   Llave    Llave     Llave a eliminar
         * @return  int                Numero de filas eliminadas
         */
        public int removeLlaveBD(Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                                  throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Prestamo_Ocasional  WHERE Cod_Llave = " + Llave.getCodigo() +
                        " AND F_Alta_L = \"" + UCAL.calendarToString(Llave.getFechaI()) + "\" AND" +
                        " DNI = \"" + getDNI() + "\" AND Empresa = \"" + getEmpresa() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** insertLlavesBD: Metodo que inserta las Llaves prestables al Ocasional
         *
         * @param   void
         * @return  int                 Numero de filas insertadas
         */
        public int insertLlavesBD() throws DataException, SQLException
        {
            // Obtenemos las llaves
            HashMap<String,Llave> Llaves = getLlaves();

            // Insertamos cada Llave
            Iterator it = Llaves.entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos la llave
                Map.Entry VKLlaves = (Map.Entry) it.next();
                Llave ll = (Llave) VKLlaves.getValue();

                // Insertamos la llave
                insertLlaveBD (ll);
            }

            return Llaves.size();
        }
        /** updateLlavesBD: Metodo que actualiza las Llaves prestables al ocasional
         *
         * @param   void
         * @return  int                                     Numero de filas (Llaves) actualizadas
         */
        public int updateLlavesBD() throws DataException, SQLException, RemoteException
        {
            // Filas actualizadas
            int res = 0;

            // Recorremos el HashMap nuevo, actualizando las Llaves existentes e insertando las nuevas
            Iterator it_new = getLlaves().entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_new.next();
                Llave llave_new = (Llave) VKLlaves.getValue();
                String llave_key = (String) VKLlaves.getKey();

                // Si existe la llave -> La actualizamos
                if (getLlaves().containsKey(llave_key))         updateLlaveBD(getLlaves().get(llave_key),llave_new);
                // Si no existe -> La insertamos
                else                                            insertLlaveBD(llave_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellas Llaves que no consten en el nuevo
            HashMap<String,Llave> Llaves_actu = getLlavesBD();
            Iterator it_old = Llaves_actu.entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_old.next();
                Llave llave_old = (Llave) VKLlaves.getValue();
                String llave_key = (String) VKLlaves.getKey();

                // Si la Llave vieja no existe como nueva -> La eliminamos y actualizamos el contador de filas actualizadas
                if (!getLlaves().containsKey(llave_key))
                {
                    removeLlaveBD(llave_old);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removeLlavesBD: Metodo que borra las Llaves prestables al Ocasional
         *
         * @param   void
         * @return  int             Numero de filas (Llaves) eliminadas
         */
        public int removeLlavesBD() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Prestamo_Ocasional WHERE DNI = \"" + getDNI() + "\" AND Empresa = \"" + getEmpresa() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getLlavesBD: Metodo que obtiene todas las llaves prestables al personal
         *
         * @param   void
         * @return  HashMap<Integer,Llave>       Llaves               HashMap con todas las llaves
         */
        public HashMap<String,Llave> getLlavesBD () throws DataException, SQLException, RemoteException
        {
           // Creamos el HashMap resultado
           HashMap<String,Llave> Llaves = new HashMap();

           // Introducimos las llaves
           String query = "SELECT Cod_Llave,F_Alta_L FROM Prestamo_Ocasional WHERE DNI = \"" + getDNI() + "\" AND Empresa = \"" + getEmpresa() + "\";" ;
           ResultSet rs = UBD.ejecutarQuery(query);

            while (rs.next())
            {
                // Creamos la llave y la introduciomos
                Llave ll = new Llave(rs.getInt(1),rs.getString(2).substring(0,19));
                Llaves.put("L - " + ll.getCodigo() + " - " + UCAL.calendarToString(ll.getFechaI()),ll);
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Llaves;
        }

        /** insertOcasionalBD: Metodo que inserta el Ocasional en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertOcasionalBD() throws DataException, SQLException
        {
            // INSERTAMOS LOS DATOS PERSONALES si no existen previamente
            if (!existe())  super.insertPersonalBD();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Ocasional VALUES (\""  +
                    getDNI() + "\",\"" +
                    getEmpresa() + "\",\"" +
                    getAcceso().getCodAcc() + "\",\"" +
                    F_Caducidad.calendarToString(getFCaducidad()) + "\"," +
                    (getNotas()==null ? "null" : "\"" + getNotas() + "\"") + ");";

                // Ejecutamos la consulta y devolvemos el resultado
                UBD.ejecutarUpdate(query);

                insertLlavesBD();

            // Devolvemos el resultado
            return 1;
        }
        /** updateOcasionalBD: Metodo que actualiza el Ocasional en la BD
         *
         * @param   Ocasional    Ocasional                      Ocasional a actualizar
         * @return  int                                         Numero de filas actualizadas
         */
        public int updateOcasionalBD(Ocasional Ocasional) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Ocasional == null)                                  throw new DataException();

            // ACTUALIZAMOS LOS DATOS PERSONALES
            updatePersonalBD(Ocasional);

            // ACTUALIZAMOS LOS DATOS ESPECIFICOS
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Ocasional SET " +
                    "Empresa = \"" + Ocasional.getEmpresa() + "\"," +
                    "Cod_Acc = \"" + Ocasional.getAcceso().getCodAcc() + "\"," +
                    "F_Caducidad = \"" + F_Caducidad.calendarToString(Ocasional.getFCaducidad()) + "\"," +
                    "Notas = " + (Ocasional.getNotas() == null ? "null" : "\"" + Ocasional.getNotas() + "\"") +
                    " WHERE DNI = \"" + getDNI() + "\" AND Empresa = \"" + getEmpresa() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeOcasionalBD: Metodo que borra un Ocasional de la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeOcasional() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Ocasional WHERE DNI = \"" + getDNI() + "\" AND Empresa = \"" + getEmpresa() + "\";";
System.out.println(query);
            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /** obtenerEmpresas: Metodo que obtiene las empresas para las que trabaja una persona dada
         *
         * @param   String      DNI     DNI de la persona
         * @return  ResultSet   rs      Empresas para las que trabaja
         */
        public ResultSet obtenerEmpresas(String DNI) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Empresa FROM Ocasional WHERE DNI = \"" + DNI + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerOcasional: Metodo que devuelve todo el personal ocasional de la BD
         *
         * @return  ResultSet   rs          Personal ocasional
         */
        public ResultSet obtenerOcasional() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT DNI,Apellido1,Apellido2,Nombre" +
                    " FROM Personal JOIN Ocasional USING (DNI)" +
                    " ORDER BY DNI;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* eliminarOcasional: Metodo que borra todo el personal ocasional cuya fecha de caducidad haya expirado
         *
         * @return  int     Numero de personas ocasionales eliminadas
         */
        public int eliminarOcasional() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Personal WHERE DNI IN (SELECT DNI FROM Ocasional WHERE (F_Caducidad<CURDATE()));";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /* obtenerLlaves: Metodo que obtiene todas las llaves prestables a una persona ocasional
         *
         * @param   void
         * @return  ResultSet               Llaves prestables a la persona ocasional
         */
        public ResultSet obtenerLlaves() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llave 'Código', Z.Descripcion 'Zona', L.Descripcion 'Descripción'" +
                            " FROM Zona Z JOIN Llave L USING (Cod_Zona,F_Alta_Z) JOIN Prestamo_Ocasional USING (Cod_Llave,F_Alta_L)" +
                            " WHERE DNI = \"" + getDNI() + "\" AND Empresa = \"" + getEmpresa() + "\"" +
                            " ORDER BY Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLsPorPrestarOcas: Metodo que obtiene todas las llaves que no son prestables a una persona ocasional y que pasan el filtro proporcionado
         *
         * @param   String      Filtro      Filtro que deben pasar las llaves
         * @return  ResultSet               Llaves prestables a la persona
         */
        public ResultSet obtenerLsPorPrestarOcas(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llave 'Código', Z.Descripcion 'Zona', L.Descripcion 'Descripción'" +
                            " FROM Zona Z JOIN Llave L USING (Cod_Zona,F_Alta_Z)" +
                            " WHERE (Cod_Llave,F_Alta_L) NOT IN (SELECT Cod_Llave,F_Alta_L" +
                                                                    " FROM Prestamo_Ocasional" +
                                                                    " WHERE DNI = \"" + getDNI() + "\" AND Empresa = \"" + getEmpresa() + "\")" +
                            " AND (Cod_Llave LIKE \"%" + Filtro + "%\" OR Z.Descripcion LIKE \"%" + Filtro + "%\" OR L.Descripcion LIKE \"%" + Filtro + "%\")" +
                            " ORDER BY Cod_Llave;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
}