// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Parte_Seguridad_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.Calendar;



/** CLASE PARTE_SEGURIDAD:  Clase que representa un parte de un servicio de seguridad
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Parte_Seguridad extends UnaFecha implements Parte_Seguridad_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Vigilante:              Vigilante de seguridad que ha redactado el parte */
        private Vigilante           Vigilante;
        /** Incidencias:            String con las incidencias acontecidas durante el servicio */
        private String              Incidencias;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();


	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Parte_Seguridad      Objeto de tipo Parte_Seguridad
         */
        public Parte_Seguridad () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Ocasional con los valores pasados parametricamente
	 *
	 * @param   Fecha       Fecha en la que es redactado el parte
         * @param   Vigilante   Vigilante de seguridad que ha redactado el parte
         * @param   Incidencias String con las incidencias acontecidas durante el servicio
	 * @return  Parte	Objeto de tipo Parte_Seguridad
	 */
	public Parte_Seguridad (Calendar Fecha, Vigilante Vigilante, String Incidencias) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(Fecha);

            // Comprobamos que los parametros pasados son correctos
            if ((Vigilante == null) || (Incidencias == null) || (Incidencias.length()>1000))    throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Vigilante = Vigilante;
            this.Incidencias = Incidencias;
	}

	/** Constructor: Constructor de clase que crea el objeto Ocasional con los valores pasados parametricamente
	 *
	 * @param   Fecha       Fecha en la que es redactado el parte
	 * @return  Parte	Objeto de tipo Parte_Seguridad
	 */
	public Parte_Seguridad (String Fecha) throws DataException, SQLException, RemoteException
        {
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Fecha == null) || (Fecha.length()>19))             throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Fecha,Cod_Vig,Incidencias FROM Parte_Seguridad" +
                    " WHERE Fecha = \"" + Fecha + "\";";

            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            setFechaI(stringToCalendar(rs.getString(1).substring(0,19)));
            this.Vigilante = new Vigilante();
            this.Vigilante.setCodVig(rs.getInt(2));
            this.Incidencias = rs.getString(3);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getVigilante: Metodo que obtiene el valor del atributo Vigilante
	 *
	 * @param   void
	 * @return  Vigilante	Vigilante de seguridad que ha redactado el parte
	 */
	public Vigilante getVigilante()                 { return this.Vigilante; }
        /** setVigilante: Metodo que establece el valor del atributo Vigilante
	 *
	 * @param   Vigilante     Vigilante     Vigilante de seguridad que ha redactado el parte
	 * @return  void
	 */
	public void setVigilante(Vigilante Vigilante) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Vigilante == null)                      throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Vigilante = Vigilante;
        }

        /** getIncidencias: Metodo que obtiene el valor del atributo Incidencias
	 *
	 * @param   void
	 * @return  String	String con las incidencias acontecidas durante el servicio
	 */
	public String getIncidencias()                  { return this.Incidencias; }
        /** setIncidencias: Metodo que establece el valor del atributo Incidencias
	 *
	 * @param   Incidencias     String     String con las incidencias acontecidas durante el servicio
	 * @return  void
	 */
	public void setIncidencias(String Incidencias) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Incidencias == null) || (Incidencias.length()>1000))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Incidencias = Incidencias;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "PARTE_SEGURIDAD:\n" + super.toString() + "\n" +
            "Cod_Vig: " + getVigilante().getCodVig() + "\n" +
            "Incidencias: " + getIncidencias() + "\n";
	}


        // DML
        /** insertParteSeguridadBD: Metodo que inserta el Parte_Seguridad en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertParteSeguridadBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String Query = "INSERT INTO Parte_Seguridad VALUES (\""  +
                    calendarToString(getFechaI()) + "\",\"" +
                    getVigilante().getCodVig() + "\",\"" +
                    getIncidencias() + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(Query);
        }
        /** updateParteSeguridadBD: Metodo que actualiza el Parte_Seguridad en la BD
         *
         * @param   Parte_Seguridad     Parte de seguridad con el que modificar
         * @return  int                 Numero de filas actualizadas
         */
        public int updateParteSeguridadBD(Parte_Seguridad Parte_Seguridad) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Parte_Seguridad SET " +
                    "Fecha = \"" + calendarToString(Parte_Seguridad.getFechaI()) + "\"," +
                    "Cod_Vig = \"" + Parte_Seguridad.getVigilante().getCodVig() + "\"," +
                    "Incidencias = \"" + Parte_Seguridad.getIncidencias() +
                    "\" WHERE Fecha = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeParteSeguridadBD: Metodo que borra un Parte_Seguridad en la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeParteSeguridadBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Parte_Seguridad WHERE Fecha = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }



        // FUNCIONALIDADES
        /* obtenerPartes: Metodo que devuelve los partes redactados entre Fecha_Inic y Fecha_Fin, entre
         * las horas Hora_Inic y Hora_Fin por unos vigilantes dados
         *
         * @param   String      Fecha_Inic  Fecha de inicio
         * @param   String      Fecha_Fin   Fecha de fin
         * @param   String      Hora_Inic   Hora de inicio
         * @param   String      Hora_Fin    Hora de fin
         * @param   String      DNI         DNI del vigilante redactos
         * @return  ResultSet   rs          Partes redactados
         */
        public ResultSet obtenerPartes(String Fecha_Inic, String Fecha_Fin, String Hora_Inic, String Hora_Fin, String Cod_Vig) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String where_fecha;
            if ((Hora_Inic == null) && (Hora_Fin == null))          where_fecha = "BETWEEN \"" + Fecha_Inic + " - 00:00:00\" AND \"" + Fecha_Fin + " - 23:59:59";
            else                                                    where_fecha = "BETWEEN \"" + Fecha_Inic + " - " + Hora_Inic + ":00\" AND \"" + Fecha_Fin + " - " + Hora_Fin + ":00";

            String query = "SELECT Fecha" +
                    " FROM Parte_Seguridad" +
                    " WHERE Cod_Vig LIKE \"" + (Cod_Vig.equals("Todos") ? "%%" : Cod_Vig) + "\"" + (Fecha_Inic == null ? "" : "AND Fecha " + where_fecha + "\"") +
                    " ORDER BY Fecha DESC;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
        /* obtenerPartesEditables: Metodo que devuelve las fechas de todos aquellos partes que son editables por el usuario
         *
         * @param   String      DNI                 DNI del usuario
         * @return  ArrayList   partesEditables     Partes editables por el usuario
         */
        public ArrayList obtenerPartesEditables(String DNI) throws SQLException
        {
            // Obtenemos los partes editables
            String query = "SELECT Fecha" +
                            " FROM Parte_Seguridad JOIN Vigilante USING (Cod_Vig)" +
                            " WHERE DNI = \"" + DNI + "\"" +
                            " ORDER BY Fecha DESC" +
                            " LIMIT 0,2;";

            ResultSet rs = UBD.ejecutarQuery(query);

            // Generamos el resultado
            ArrayList partesEditables = new ArrayList();
            while (rs.next())   partesEditables.add(rs.getString(1).substring(0,19));

            // Devolvemos el resultado
            return partesEditables;
        }
}