// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Periodico_Contacto_I;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;



/** CLASE PERIODICO_CONTACTO:  Clase que representa un trabajador periodico o un contacto perteneciente a un departamento
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Periodico_Contacto extends Personal implements Periodico_Contacto_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cargos:                         HashMap con los cargos que desempeña */
	private HashMap<String,Cargo>       Cargos;

        /** UCAL:                           Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar    UCAL = new UtilidadesCalendar();





	// METODOS -------------------------------------------------------------
        // CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Periodico_Contacto      Objeto de tipo Periodico_Contacto
         */
        public Periodico_Contacto () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Permiso_Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI                 String con el DNI de la persona
         * @param   Apellido1           String con el primer apellido de la persona
         * @param   Apellido2           String con el segundo apellido de la persona
         * @param   Nombre              String con el nombre de la persona
         * @param   Insertador          Insertador de la persona en la BD
         * @param   Telefonos           Telefonos de contacto
         * @param   Emails              Emails de contacto
	 * @return  Periodico_Contacto  Objeto de tipo Periodico_Contacto
	 */
	public Periodico_Contacto (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, ArrayList Telefonos, ArrayList Emails) throws DataException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI, Apellido1, Apellido2, Nombre, Insertador, Telefonos, Emails);
            // Comprobamos que los parametros pasados son correctos
            this.Cargos = new HashMap();
	}
	/** Constructor: Constructor de clase que crea el objeto Permiso_Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI                 String con el DNI de la persona
         * @param   Apellido1           String con el primer apellido de la persona
         * @param   Apellido2           String con el segundo apellido de la persona
         * @param   Nombre              String con el nombre de la persona
         * @param   Insertador          Insertador de la persona en la BD
         * @param   Cargos              HashMap con cargos que desempeña
         * @param   Telefonos           Telefonos de contacto
         * @param   Emails              Emails de contacto
	 * @return  Periodico_Contacto  Objeto de tipo Periodico_Contacto
	 */
	public Periodico_Contacto (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, HashMap<String,Cargo> Cargos, ArrayList Telefonos, ArrayList Emails) throws DataException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI, Apellido1, Apellido2, Nombre, Insertador, Telefonos, Emails);
            // Establecemos los valores del resto de atributos
            this.Cargos = Cargos;
	}
	/** Constructor: Constructor de clase que crea el objeto Permiso_Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI                 String con el DNI de la persona
	 * @return  Periodico_Contacto  Objeto de tipo Periodico_Contacto
	 */
	public Periodico_Contacto (String DNI) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI);
            // Comprobamos que los parametros pasados son correctos
            this.Cargos = new HashMap();
	}
	/** Constructor: Constructor de clase que crea el objeto Permiso_Ocasional con los valores pasados parametricamente
	 *
	 * @param   DNI                 String con el DNI de la persona
         * @param   Cargos              HashMap con cargos que desempeña
	 * @return  Periodico_Contacto  Objeto de tipo Periodico_Contacto
	 */
	public Periodico_Contacto (String DNI, HashMap<String,Cargo> Cargos) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI);
            // Comprobamos que los parametros pasados son correctos
            this.Cargos = Cargos;
	}

        
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCargos: Metodo que obtiene el valor del atributo Cargos
	 *
	 * @param   void
	 * @return  HashMap<String,Cargo>     HashMap con los cargos que desempeña
	 */
	public HashMap<String,Cargo> getCargos()                      { return this.Cargos; }
        /** setCargos: Metodo que establece el valor del atributo Cargos
	 *
	 * @param   Cargos     HashMap con los cargos que desempeña
	 * @return  void
	 */
	public void setCargos(HashMap<String,Cargo> Cargos)           { this.Cargos = Cargos; }
        /** addCargo: Metodo que añade un Cargo a los ya existentes
         *
         * @param   Cargo        Cargo  Cargo nuevo que desempeña
         * @return  void
         */
        public void addCargo(Cargo Cargo) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Cargo == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getCargos().put(Cargo.getCodCargo(),Cargo);
        }
        /** updateCargo: Metodo que modifica un Cargo existente
         *
         * @param   Cargo_old,Cargo_new        Cargo  Cargos a modificar y por el que reemplazar
         * @return  void
         */
        public void updateCargo(Cargo Cargo_old, Cargo Cargo_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cargo_old == null) || (Cargo_new == null))         throw new DataException();

            // Comprobamos si existe el cargo, en cuyo caso lo cambiamos por el nuevo
            if (getCargos().containsKey(Cargo_old.getCodCargo()))
            {
                removeCargo(Cargo_old);
                addCargo(Cargo_new);
            }
        }
        /** removeCargo: Metodo que elimina un Cargo de los existentes
         *
         * @param   Cargo        Cargo  Cargo a eliminar
         * @return  void
         */
        public void removeCargo(Cargo Cargo) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Cargo == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getCargos().remove(Cargo.getCodCargo());
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // String resultado
            String s = super.toString() + "Cargos:\n";

            if (getCargos().size()>0)
            {
                // Añadimos las Strings con todos los Cargos
                Iterator it = getCargos().entrySet().iterator();
                while (it.hasNext())
                {
                    Map.Entry VKCargo = (Map.Entry)it.next();
                    Cargo Cargo = (Cargo) VKCargo.getValue();
                    s = s + Cargo.toString() + "\n";
                }
            }

            // Devolvemos el resultado
            return s;
	}

        // FUNCIONALIDADES
        /* obtenerContactos: Metodo que devuelve todos los contactos de la BD
         *
         * @param   void
         * @return  ResultSet   rs          Contactos
         */
        public ResultSet obtenerContactos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT DNI" +
                    " FROM Personal JOIN Contacto USING (DNI)" +
                    " ORDER BY DNI;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* actualizarFUA: Metodo que actualiza las fecha de ultima actualizacion de las que la persona
         * es responsable de personal
         *
         * @param   void
         * @return  void
         */
        public void actualizarFUA() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Departamento" +
                            " SET F_Ult_Actu = \"" + UCAL.calendarToString(Calendar.getInstance()) + "\"" +
                            " WHERE (Cod_Emp,Cod_Dep) IN (SELECT DISTINCT Cod_Emp,Cod_Dep" +
                                                            " FROM Contacto" +
                                                            " WHERE DNI = \"" + getDNI() + "\");";

            // Ejecutamos la consulta
            UBD.ejecutarUpdate(query);
        }
}