// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Personal_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.StringTokenizer;



/** CLASE PERSONAL:  Clase que representa una persona
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Personal implements Personal_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** DNI:                    String con el DNI de la persona */
	private String              DNI;
        /** Apellido1:              String con el primer apellido de la persona */
	private String              Apellido1;
        /** Apellido2:              String con el segundo apellido de la persona */
        private String              Apellido2;
        /** Nombre:                 String con el nombre de la persona */
        private String              Nombre;
        /** Insertador:             Insertador de la persona en la BD */
        private Personal            Insertador;
        /** Telefonos:              Lista con los telefonos de contacto */
        private ArrayList<String>   Telefonos;
        /** Emails:                 Lista con los e-mails de contacto */
        private ArrayList<String>   Emails;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Personal      Objeto de tipo Personal
         */
        public Personal () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto HPersona con los valores pasados parametricamente
	 *
	 * @param   DNI                 String con el DNI de la persona
         * @param   Apellido1           String con el primer apellido de la persona
         * @param   Apellido2           String con el segundo apellido de la persona
         * @param   Nombre              String con el nombre de la persona
         * @param   Insertador          Insertador de la persona en la BD
	 * @return  Personal    Objeto de tipo Personal
	 */
	public Personal (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador) throws DataException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (Apellido1 == null) || (Apellido2 == null) || (Nombre == null) ||
                    (DNI.length()>9) || (Apellido1.length()>25) || (Apellido2.length()>25) || (Nombre.length()>20))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.DNI = DNI.toUpperCase();
            this.Apellido1 = Apellido1;
            this.Apellido2 = Apellido2;
            this.Nombre = Nombre;
            this.Insertador = Insertador;
            this.Telefonos = new ArrayList<String>();
            this.Emails = new ArrayList<String>();
	}
	/** Constructor: Constructor de clase que crea el objeto HPersona con los valores pasados parametricamente
	 *
	 * @param   DNI                 String con el DNI de la persona
         * @param   Apellido1           String con el primer apellido de la persona
         * @param   Apellido2           String con el segundo apellido de la persona
         * @param   Nombre              String con el nombre de la persona
         * @param   Insertador          Insertador de la persona en la BD
         * @param   Telefonos           Lista con los telefonos de contacto
         * @param   Emails              Lista con los e-mails de contacto
	 * @return  Personal    Objeto de tipo Personal
	 */
	public Personal (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, ArrayList<String> Telefonos, ArrayList<String> Emails) throws DataException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (Apellido1 == null) || (Apellido2 == null) || (Nombre == null) || (Telefonos == null) || (Emails == null) ||
                    (DNI.length()>9) || (Apellido1.length()>25) || (Apellido2.length()>25) || (Nombre.length()>20))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.DNI = DNI.toUpperCase();
            this.Apellido1 = Apellido1;
            this.Apellido2 = Apellido2;
            this.Nombre = Nombre;
            this.Insertador = Insertador;
            this.Telefonos = Telefonos;
            this.Emails = Emails;
	}
	/** Constructor: Constructor de clase que crea el objeto HPersona con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
	 * @return  Personal    Objeto de tipo Personal
	 */
	public Personal (String DNI) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (DNI.length()>9))              throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Personal WHERE DNI = \"" + DNI + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            if (rs.next())
            {
                this.DNI = rs.getString(1);
                this.Apellido1 = rs.getString(2);
                this.Apellido2 = rs.getString(3);
                this.Nombre = rs.getString(4);
                this.Insertador = (((rs.getString(5) == null) || (rs.getString(1).equals(rs.getString(5)))) ? null : new Personal(rs.getString(5)));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();


            // Realizamos la asignacion de los telefonos
            ArrayList tfnos = new ArrayList();

            query = "SELECT Tfno FROM Telefono WHERE DNI = \"" + DNI + "\";";
            rs = UBD.ejecutarQuery(query);

            while (rs.next())       tfnos.add(rs.getString(1));
            Telefonos = tfnos;

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Realizamos la asignacion de los e-mails
            ArrayList emails = new ArrayList();

            query = "SELECT Email FROM Email WHERE DNI = \"" + DNI + "\";";
            rs = UBD.ejecutarQuery(query);

            while (rs.next())       emails.add(rs.getString(1));
            Emails = emails;

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getDNI: Metodo que obtiene el valor del atributo DNI
	 *
	 * @param   void
	 * @return  String	String con el DNI de la persona
	 */
	public String getDNI()				{ return this.DNI; }
        /** setDNI: Metodo que establece el valor del atributo DNI
	 *
	 * @param   DNI     String     String con el DNI de la persona
	 * @return  void
	 */
	public void setDNI(String DNI) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (DNI.length()>9))      throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.DNI = DNI.toUpperCase();
        }

        /** getApellido1: Metodo que obtiene el valor del atributo Apellido1
	 *
	 * @param   void
	 * @return  String	String con el primer apellido de la persona
	 */
	public String getApellido1()			{ return this.Apellido1; }
        /** setApellido1: Metodo que establece el valor del atributo Apellido1
	 *
	 * @param   Apellido1     String     String con el primer apellido de la persona
	 * @return  void
	 */
	public void setApellido1(String Apellido1) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Apellido1 == null) || (Apellido1.length()>25)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Apellido1 = Apellido1;
        }

        /** getApellido2: Metodo que obtiene el valor del atributo Apellido2
	 *
	 * @param   void
	 * @return  String	String con el segundo apellido de la persona
	 */
	public String getApellido2()			{ return this.Apellido2; }
        /** setApellido2: Metodo que establece el valor del atributo Apellido2
	 *
	 * @param   Apellido2     String     String con el segundo apellido de la persona
	 * @return  void
	 */
	public void setApellido2(String Apellido2) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Apellido2 == null) || (Apellido2.length()>25)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Apellido2 = Apellido2;
        }

        /** getNombre: Metodo que obtiene el valor del atributo Nombre
	 *
	 * @param   void
	 * @return  String	String con el nombre de la persona
	 */
	public String getNombre()			{ return this.Nombre; }
        /** setNombre: Metodo que establece el valor del atributo Nombre
	 *
	 * @param   Nombre     String     String con el nombre de la persona
	 * @return  void
	 */
	public void setNombre(String Nombre) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Nombre == null) || (Nombre.length()>20))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Nombre = Nombre;
        }

        /** getInsertador: Metodo que obtiene el valor del atributo Insertador
	 *
	 * @param   void
	 * @return  Personal	Insertador de la persona en la BD
	 */
	public Personal getInsertador()		{ return this.Insertador; }
        /** setInsertador: Metodo que establece el valor del atributo Insertador
	 *
	 * @param   Insertador     Personal     Insertador de la persona en la BD
	 * @return  void
	 */
	public void setInsertador(Personal Insertador) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Insertador == null)                 throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Insertador = Insertador;
        }

        /** getTelefonos: Metodo que obtiene el valor del atributo Telefonos
	 *
	 * @param   void
	 * @return  ArrayList<String>	Lista con los telefonos de contacto
	 */
	public ArrayList<String> getTelefonos()             { return this.Telefonos; }
        /** setTelefonos: Metodo que establece el valor del atributo Telefonos
	 *
	 * @param   Telefonos     ArrayList<String>     Lista con los telefonos de contacto
	 * @return  void
	 */
	public void setTelefonos(ArrayList<String> Telefonos)    { this.Telefonos = Telefonos; }
        /** addTelefono: Metodo que añade un telefono a los ya existentes
         * 
         * @param   Telefono        String  String con el nuevo numero de telefono
         * @return  void
         */
        public void addTelefono(String Telefono) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Telefono == null) || (Telefono.length()>12))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getTelefonos().add(Telefono);
        }
        /** updateTelefono: Metodo que modifica un Telefono existente
         *
         * @param   Telefono_old,Telefono_new        String  Telefonos a modificar y por el que reemplazar
         * @return  void
         */
        public void updateTelefono(String Telefono_old, String Telefono_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Telefono_new == null)                       throw new DataException();

            // Comprobamos si existe el Telefono, en cuyo caso lo cambiamos por el nuevo
            if (getTelefonos().contains(Telefono_old))
            {
                removeTelefono(Telefono_old);
                addTelefono(Telefono_new);
            }
        }
        /** removeTelefono: Metodo que elimina un telefono de los existentes
         *
         * @param   Telefono        String  String con el numero de telefono a eliminar
         * @return  void
         */
        public void removeTelefono(String Telefono) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Telefono == null) || (Telefono.length()>12))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getTelefonos().remove(Telefono);
        }

        /** getEmails: Metodo que obtiene el valor del atributo Emails
	 *
	 * @param   void
	 * @return  ArrayList<String>	Lista con los e-mails de contacto
	 */
	public ArrayList<String> getEmails()             { return this.Emails; }
        /** setEmails: Metodo que establece el valor del atributo Emails
	 *
	 * @param   Emails     ArrayList<String>     Lista con los e-mails de contacto
	 * @return  void
	 */
	public void setEmails(ArrayList<String> Emails)  { this.Emails = Emails; }
        /** addEmail: Metodo que añade un e-mail a los ya existentes
         *
         * @param   Email        String  String con el nuevo e-mail
         * @return  void
         */
        public void addEmail(String Email) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Email == null) || (Email.length()>50)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getEmails().add(Email);
        }
        /** updateEmail: Metodo que modifica un Email existente
         *
         * @param   Email_old,Email_new        String  Emails a modificar y por el que reemplazar
         * @return  void
         */
        public void updateEmail(String Email_old, String Email_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Email_new == null)                          throw new DataException();

            // Comprobamos si existe el e-mail, en cuyo caso lo cambiamos por el nuevo
            if (getEmails().contains(Email_old))
            {
                removeEmail(Email_old);
                addEmail(Email_new);
            }
        }
        /** removeEmail: Metodo que elimina un e-mail de los existentes
         *
         * @param   Telefono        String  String con el e-mail a eliminar
         * @return  void
         */
        public void removeEmail(String Email) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Email == null) || (Email.length()>50))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getEmails().remove(Email);
        }
        

        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // String para los telefonos
            String tels = "";
            // String para los e-mails
            String ems = "";

            // Creamos la String con todos los telefonos
            Iterator it = getTelefonos().iterator();
            while (it.hasNext())
            {
                String tfno = (String) it.next();
                tels = tels + tfno + "\n";
            }
            // Creamos la String con todos los e-mails
            it = getEmails().iterator();
            while (it.hasNext())
            {
                String mail = (String) it.next();
                ems = ems + mail + "\n";
            }

            // Devolvemos el resultado
            return "Personal:\nDNI: " + getDNI() + "\n" +
            "Apellido1: " + getApellido1() + "\n" +
            "Apellido2: " + getApellido2() + "\n" +
            "Nombre: " + getNombre() + "\n" +
            "Insertador:\n" + (getInsertador() == null ? "\n" : getInsertador().toString()) +
            "Telefonos:\n" + tels + "Emails:" + ems + "\n";
	}


        // DML
        /** insertTelefonoBD: Metodo que inserta un Telefono en la BD
         *
         * @param   Telefono    String      Telefono a insertar
         * @return  int                     Numero de filas  insertadas
         */
        public int insertTelefonoBD(String Telefono) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Telefono == null)                               throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Telefono VALUES (\""  +
                getDNI() + "\",\"" + Telefono + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateTelefonoBD: Metodo que actualiza un Telefono de la BD
         *
         * @param   Tfno_old    Telefono a modificar
         * @param   Tfno_new    Telefono de reemplazo
         * @return  int         Numero de filas actualizadas
         */
        public int updateTelefonoBD(String Tfno_old, String Tfno_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Tfno_old == null) || (Tfno_new == null))               throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Telefono SET " +
                    "DNI = \"" + getDNI() + "\"," +
                    "Tfno = \"" + Tfno_new +
                    "\" WHERE DNI = \"" + getDNI() + "\" AND Tfno = \"" + Tfno_old + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeTelefonoBD: Metodo que borra un Telefono de la BD
         *
         * @param   Telefono    String      Telefono a eliminar
         * @return  int                     Numero de filas eliminadas
         */
        public int removeTelefonoBD(String Telefono) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Telefono == null)                                   throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Telefono WHERE DNI = \"" + getDNI() + "\" AND Telefono = \"" + Telefono + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** insertTelefonosBD: Metodo que inserta los Telefonos en la BD
         *
         * @param   void
         * @return  int                 Numero de filas insertadas
         */
        public int insertTelefonosBD() throws DataException, SQLException
        {
            // Insertamos los telefonos
            return insertTelefonosBD(this);
        }
        /** insertTelefonosBD: Metodo que inserta los Telefonos en la BD
         *
         * @param   Personal   Persona      Persona con los telefonos a insertar
         * @return  int                             Numero de filas insertadas
         */
        public int insertTelefonosBD(Personal Persona) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona.getTelefonos() == null)                                  throw new DataException();
            // Comprobamos si se ha pasado algun telefono
            // Si los parametros son correctos y se ha pasado algun telefono, actuamos
            else if (!Persona.getTelefonos().isEmpty())
            {
                // String con los telefonos a insertar
                String tfnos = "";
                Iterator it = Persona.getTelefonos().iterator();
                while (it.hasNext())
                {
                    // Obtenemos el telefono
                    String Telefono = (String) it.next();

                    // Creamos la String con la sentencia a realizar
                    tfnos = tfnos + "(\"" + Persona.getDNI() + "\",\"" + Telefono + "\"),";
                }

                // Creamos la String con la sentencia a realizar
                String query = "INSERT INTO Telefono VALUES " + tfnos.substring(0,tfnos.length()-1) + ";";

                // Ejecutamos la consulta y devolvemos el resultado
                return UBD.ejecutarUpdate(query);
            }
            
            // Devolvemos el resultado en caso de no haber sido pasado ningun telefono
            return 0;
        }
        /** updateTelefonosBD: Metodo que actualiza los Telefonos de la BD
         *
         * @param   void
         * @return  int                                 Numero de filas actualizadas
         */
        public int updateTelefonosBD() throws DataException, SQLException
        {
            // Eliminamos los telefonos
            removeTelefonosBD();
            // Insertamos los nuevos telefonos
            return insertTelefonosBD();
        }
        /** removeTelefonosBD: Metodo que borra los Telefonos de la BD
         *
         * @param   void
         * @return  int                     Numero de filas eliminadas
         */
        public int removeTelefonosBD() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Telefono WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeTelefonosBD: Metodo que borra los Telefonos de una persona de la BD
         *
         * @param   Personal    Persona     Persona de la que eliminar todos sus telefonos
         * @return  int                             Numero de filas eliminadas
         */
        public int removeTelefonosBD(Personal Persona) throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Telefono WHERE DNI = \"" + Persona.getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** insertEmailBD: Metodo que inserta un Email en la BD
         *
         * @param   Email   String      Email a insertar
         * @return  int                 Numero de filas insertadas
         */
        public int insertEmailBD(String Email) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Email == null)                                  throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Email VALUES (\""  +
                getDNI() + "\",\"" + Email + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateEmailBD: Metodo que actualiza un Email de la BD
         *
         * @param   Email_old       Email a modificar
         * @param   Email_new       Email de reemplazo
         * @return  int             Numero de filas actualizadas
         */
        public int updateEmailBD(String Email_old, String Email_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Email_old == null) || (Email_new == null))             throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Email SET " +
                    "DNI = \"" + getDNI() + "\"," +
                    "Email = \"" + Email_new +
                    "\" WHERE DNI = \"" + getDNI() + "\" AND Email = \"" + Email_old + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeEmailBD: Metodo que borra un Email de la BD
         *
         * @param   Email    String         Email a eliminar
         * @return  int                     Numero de filas eliminadas
         */
        public int removeEmailBD(String Email) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Email == null)                                      throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Email WHERE DNI = \"" + getDNI() + "\" AND Email = \"" + Email + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** insertEmailsBD: Metodo que inserta los Emails en la BD
         *
         * @param   void
         * @return  int                 Numero de filas insertadas
         */
        public int insertEmailsBD() throws DataException, SQLException
        {
            // Insertamos los e-mails
            return insertEmailsBD(this);
        }
        /** insertEmailsBD: Metodo que inserta los Emails en la BD
         *
         * @param   Personal    Persona     Persona con los e-mails a insertar
         * @return  int                             Numero de filas insertadas
         */
        public int insertEmailsBD(Personal Persona) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona.getEmails() == null)                                 throw new DataException();
            // Si los parametros son correctos y se a pasado algun e-mail, actuamos
            else if (!Persona.getEmails().isEmpty())
            {
                // String con los emails a insertar
                String emails = "";
                Iterator it = Persona.getEmails().iterator();
                while (it.hasNext())
                {
                    // Obtenemos el telefono
                    String Email = (String) it.next();

                    // Creamos la String con la sentencia a realizar
                    emails = emails + "(\"" + Persona.getDNI() + "\",\"" + Email + "\"),";
                }

                // Creamos la String con la sentencia a realizar
                String query = "INSERT INTO Email VALUES " + emails.substring(0,emails.length()-1) + ";";

                // Ejecutamos la consulta y devolvemos el resultado
                return UBD.ejecutarUpdate(query);
            }

            // Devolvemos el resultado en caso de no haber sido pasado ningun e-mail
            return 0;
        }
        /** updateEmailsBD: Metodo que actualiza los Emails de la BD
         *
         * @param   void
         * @return  int                     Numero de filas eliminadas
         */
        public int updateEmailsBD() throws DataException, SQLException
        {
            // Eliminamos los e-mails
            removeEmailsBD();
            // Insertamos los nuevos e-mails
            return insertEmailsBD();
        }
        /** removeEmailsBD: Metodo que borra los Emails de la BD
         *
         * @param   void
         * @return  int                     Numero de filas eliminadas
         */
        public int removeEmailsBD() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Email WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeEmailsBD: Metodo que borra los Emails de una persona de la BD
         *
         * @param   Personal    Persona     Persona de la que eliminar todos sus e-mails
         * @return  int                             Numero de filas eliminadas
         */
        public int removeEmailsBD(Personal Persona) throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Email WHERE DNI = \"" + Persona.getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** insertPersonalBD: Metodo que inserta el Personal en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertPersonalBD() throws DataException, SQLException
        {
            // INSERTAMOS LA PERSONA
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Personal VALUES (\""  +
                    getDNI() + "\",\"" +
                    getApellido1() + "\",\"" +
                    getApellido2() + "\",\"" +
                    getNombre() + "\"," +
                    ((getInsertador() == null) ? "null" : "\"" + getInsertador().getDNI() + "\"") + ");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);


            // Si la persona ha sido insertada correctamente -> INSERTAMOS LOS TELEFONOS Y LOS EMAILS
            if (res == 1)
            {
                // Insertamos los telefonos
                insertTelefonosBD();

                // Insertamos los emails
                insertEmailsBD();
            }

            // Devolvemos el resultado (estado)
            return res;
        }
        /** updatePersonalBD: Metodo que actualiza el Personal en la BD
         *
         * @param   Persona         Persona a modificar
         * @return  int             Numero de filas actualizadas
         */
        public int updatePersonalBD(Personal Persona) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona == null)                                    throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Personal SET "  +
                    "DNI = \"" + Persona.getDNI() + "\"," +
                    "Apellido1 = \"" + Persona.getApellido1() + "\"," +
                    "Apellido2 = \"" + Persona.getApellido2() + "\"," +
                    "Nombre = \"" + Persona.getNombre() + "\"," +
                    "DNI_Ins = " + ((Persona.getInsertador() == null) ? "null" : "\"" + Persona.getInsertador().getDNI() + "\"") +
                    " WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // Actualizamos los telefonos e emails
            this.setDNI(Persona.getDNI());
            this.Telefonos = Persona.getTelefonos();
            this.Emails = Persona.getEmails();
            updateTelefonosBD();
            updateEmailsBD();

            // Devolvemos el resultado
            return res;
        }
        /** removePersonalBD: Metodo que borra un Personal de la BD
         *
         * @param   void
         * @return  int             Numero de filas eliminadas
         */
        public int removePersonalBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Personal WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* obtenerPersonalCuentas: Metodo que devuelve todo el personal que puede tener cuentas en el sistema, pasando previamente un filtro dado
         *
         * @param   String      Filtro      Filtro que debe pasar la persona
         * @return  ResultSet   rs          Personal que puede disponer de cuentas en el sistema
         */
        public ResultSet obtenerPersonalCuentas(String Filtro) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI" +
                    " FROM Personal" +
                    " WHERE DNI NOT IN (SELECT DNI FROM Ocasional)" +
                        " AND (DNI LIKE \"%" + Filtro +
                        "%\" OR Apellido1 LIKE \"%" + Filtro +
                        "%\" OR Apellido2 LIKE \"%" + Filtro +
                        "%\" OR Nombre LIKE \"%" + Filtro + "%\");";
            
            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerPersonalCentro: Metodo que devuelve todo el personal accesor al centro que se encuentra fuera de el, pasando un filtro dado
         *
         * @param   String      Filtro      Filtro que debe pasar la persona
         * @return  ResultSet   rs          Personal que puede acceder en este momento al centro
         */
        public ResultSet obtenerPersonalCentro(String Filtro) throws SQLException
        {
            String [] diasSemana = {"D","L","M","X","J","V","S","D"};

            Calendar cal = Calendar.getInstance();


            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI,Apellido1 \"Primer apellido\",Apellido2 \"Segundo apellido\", Nombre" +
                    " FROM Personal WHERE DNI NOT IN (SELECT DISTINCT DNI FROM Contacto" +
                                                        " UNION" +
                                                    " SELECT DNI FROM Acceso WHERE Salida IS NULL" +
                                                        " UNION" +
                                                    " (SELECT DISTINCT P.DNI" +
                                                    " FROM Departamento JOIN T_Acceso USING (Cod_Acc) JOIN Periodico P USING (Cod_Emp,Cod_Dep) JOIN Acceso A ON (P.DNI = A.DNI)" +
                                                    " WHERE Salida IS NOT NULL AND Dias NOT LIKE \"%"+ diasSemana[cal.get(cal.DAY_OF_WEEK)-1] + "%\" AND (CURTIME() BETWEEN Hora_Inic AND Hora_Fin))" +
                                                        " UNION" +
                                                    " SELECT DNI FROM Ocasional WHERE F_Caducidad<CURDATE())" +
                                        " AND (DNI LIKE \"%" + Filtro +
                                            "%\" OR Apellido1 LIKE \"%" + Filtro +
                                            "%\" OR Apellido2 LIKE \"%" + Filtro +
                                            "%\" OR Nombre LIKE \"%" + Filtro + "%\") ORDER BY DNI;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerPersonalCentro: Metodo que devuelve todo el personal accesor al centro
         *
         * @param   void
         * @return  ResultSet   rs          Personal que puede acceder al centro
         */
        public ArrayList<String> obtenerPersonalCentro() throws SQLException
        {
            // Obtenemos el personal
            ResultSet rs = obtenerPersonalCentro("");

            // Generamos el resultado
            ArrayList<String> personal = new ArrayList();
            while (rs.next())   personal.add(rs.getString(1));
            this.UBD.cerrarConexion();

            // Devolvemos el resultado
            return personal;
        }

        /* obtenerDNI: Metodo que devuelve el DNI de una persona
         *
         * @param   void
         * @return  String      DNI                 DNI de la persona
         */
        public String obtenerDNI() throws SQLException
        {           
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI FROM Personal WHERE Apellido1 = \"" + getApellido1() + "\" AND" +
                    " Apellido2 = \"" + getApellido2() + "\" AND Nombre = \"" + getNombre() + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);
            
            // Obtenemos el resultado
            rs.next();
            String DNI = rs.getString(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return DNI;
        }

        /* obtenerDNI: Metodo que devuelve el DNI de una persona
         *
         * @param   void
         * @return  String      DNI                 DNI de la persona
         */
        public String obtenerDNI(String apellidosNombre) throws DataException, SQLException, RemoteException
        {
            // Descomponemos la String apellidosNombre
            Personal pers = new Personal();
            StringTokenizer st = new StringTokenizer(apellidosNombre);
            pers.setApellido1(st.nextToken());
            pers.setApellido2(st.nextToken(",").substring(1));
            pers.setNombre(st.nextToken().substring(1));

            // Devolvemos el resultado
            return pers.obtenerDNI();
        }

        /* obtenerTfns: Metodo que devuelve los telefonos de esta persona
         *
         * @param   String      void
         * @return  ResultSet   rs       Telefonos de la persona
         */
        public ResultSet obtenerTfns() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Tfno 'Teléfonos' FROM Telefono WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Devolvemos el resultado
            return rs;
        }

        /* obtenerEmails: Metodo que devuelve los e-mails de esta persona
         *
         * @param   String      void
         * @return  ResultSet   rs       E-Mails de la persona
         */
        public ResultSet obtenerEmails() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Email 'E-Mails' FROM Email Emails WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Devolvemos el resultado
            return rs;
        }

        /* obtenerLsPrestables: Metodo que devuelve las llaves prestables a una persona dada
         *
         * @param   String      DNI      DNI de la persona
         * @return  ResultSet   rs       Llaves prestables a la persona
         */
        public ResultSet obtenerLsPrestables(String DNI) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Email FROM Email WHERE DNI = \"" + DNI + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Devolvemos el resultado
            return rs;
        }

        /* obtenerCargos: Metodo que devuelve los cargos de esta persona
         *
         * @param   void
         * @return  ResultSet   rs       Cargos de la persona
         */
        public ResultSet obtenerCargos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT Descripcion \"Cargos\" FROM Contacto JOIN Cargo USING (Cod_Cargo) WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerCargosEmp: Metodo que devuelve los cargos de esta persona en una empresa dada
         *
         * @param   Nom_Emp     String   Nombre de empresa
         * @return  ResultSet   rs       Cargos de la persona
         */
        public ResultSet obtenerCargosEmp(String Nom_Emp) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT Descripcion \"Cargos\"" +
                            " FROM Contacto C JOIN Cargo USING (Cod_Cargo) JOIN Departamento USING (Cod_Emp, Cod_Dep) JOIN Empresa E USING (Cod_Emp)" +
                            " WHERE C.DNI = \"" + getDNI() + "\" AND E.Nombre = \"" + Nom_Emp + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerCargosDep: Metodo que devuelve los cargos de esta persona en una empresa y departamento dados
         *
         * @param   Nom_Emp     String   Nombre de empresa
         * @param   Nom_Dep     String   Nombre de departamento
         * @return  ResultSet   rs       Cargos de la persona
         */
        public ResultSet obtenerCargosDep(String Nom_Emp, String Nom_Dep) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT Descripcion \"Cargos\"" +
                            " FROM Contacto C JOIN Cargo USING (Cod_Cargo) JOIN Departamento D USING (Cod_Emp, Cod_Dep) JOIN Empresa E USING (Cod_Emp)" +
                            " WHERE C.DNI = \"" + getDNI() + "\" AND E.Nombre = \"" + Nom_Emp + "\" AND D.Nombre = \"" + Nom_Dep + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerTipo: Metodo que devuelve el tipo de persona
         *
         * @param   void
         * @return  String      tipo     Tipo de persona
         */
        public String obtenerTipo() throws SQLException
        {
            // Vamos buscando en las tablas hasta encontrar la persona
            String query = "SELECT DNI FROM Contacto WHERE DNI = \"" + getDNI() + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);
            if (rs.next())      return "Contacto";

            query = "SELECT DNI FROM Ocasional WHERE DNI = \"" + getDNI() + "\";";
            rs = UBD.ejecutarQuery(query);
            if (rs.next())      return "Ocasional";

            query = "SELECT DNI FROM Residente WHERE DNI = \"" + getDNI() + "\";";
            rs = UBD.ejecutarQuery(query);
            if (rs.next())      return "Residente";

            query = "SELECT DNI FROM Vigilante WHERE DNI = \"" + getDNI() + "\";";
            rs = UBD.ejecutarQuery(query);
            if (rs.next())      return "Vigilante";

            // Devolvemos el resultado
            return "Periódico";
        }

        /* obtenerEmpresa: Metodo que devuelve la empresa para la que trabaja
         *
         * @param   void
         * @return  String      tipo     Empresa para la que trabaja
         */
        public String obtenerEmpresa() throws SQLException
        {
            // Vamos buscando en las tablas hasta encontrar la persona
            String query = "SELECT E.Nombre, D.Nombre" +
                            " FROM Empresa E JOIN Departamento D USING (Cod_Emp) JOIN Periodico P USING (Cod_emp,Cod_Dep)" +
                            " WHERE P.DNI = \"" + getDNI() + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);
            if (rs.next())      return rs.getString(1) + " - " + rs.getString(2);

            query = "SELECT Empresa FROM Ocasional WHERE DNI = \"" + getDNI() + "\";";
            rs = UBD.ejecutarQuery(query);
            if (rs.next())      return rs.getString(1);

            // Devolvemos el resultado
            return "Informática Navarra S.A.";
        }

        /* obtenerResponsable: Metodo que devuelve el DNI de la persona responsable del departamento donde trabaja o "" si
         * no tiene responsable o la persona no es de tipo Periodico
         *
         * @param   void
         * @return  String      DNI     DNI del responsable del departamento en el que trabaja o "" si no tiene responsable
         *                              o la persona no es de tipo Periodico
         */
        public String obtenerResponsable() throws SQLException
        {
            // Resultado
            String DNI = "";

            // Creamos la String a ejecutar
            String query = "SELECT D.DNI FROM Periodico P JOIN Departamento D USING (Cod_Emp,Cod_Dep)" +
                    " WHERE P.DNI = \"" + getDNI() + "\"" +
                        " UNION" +
                    " SELECT DNI_Ins FROM Personal JOIN Ocasional USING (DNI) WHERE DNI = \"" + getDNI() + "\";";

            ResultSet rs = UBD.ejecutarQuery(query);
            if (rs.next())
            {
                rs.getObject(1);
                // Si el DNI del autorizador del departamento no es null -> Es necesaria autorizacion telefonica
                if (!rs.wasNull())      DNI = rs.getString(1);
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return DNI;
        }

        /* existe: Metodo que denota si una persona existe en la BD
         *
         * @param   void
         * @return  boolean     Existe      Denota si la persona existe en la BD
         */
        public boolean existe() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT * FROM Personal WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            boolean Existe = rs.next();

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Existe;
        }

        /* obtenerInsertadores: Metodo que devuelve todo el personal que puede ser insertador en la BD
         *
         * @return  ResultSet   rs          Personal insertador
         */
        public ResultSet obtenerInsertadores() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI,Apellido1,Apellido2,Nombre" +
                    " FROM Personal JOIN Contacto USING (DNI)" +
                    " UNION" +
                    " SELECT DNI,Apellido1,Apellido2,Nombre" +
                    " FROM Personal JOIN Residente USING (DNI);";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerDatosPrestamosPendientes: Metodo que obtiene los datos de todos los prestamos pendientes
         *
         * @param   void
         * @return  ResultSet   rs          Datos de todos los prestamos pendientes
         */
        public ResultSet obtenerDatosPrestamosPendientes() throws SQLException
        {
            // Creamos la String a ejecutar
            String query = "SELECT Tipo, Codigo, MAX(Entrada)" +
                            " FROM (	SELECT DISTINCT T.DNI, 'Tarjeta' Tipo, Cod_Tarj Codigo, Entrada" +
                                        " FROM Tarjeta T JOIN Acceso USING (Cod_Tarj)" +
                                        " WHERE T.DNI IS NOT NULL" +
                                            " UNION" +
                                        " SELECT DNI, 'Llave' Tipo, Cod_Llave Codigo, Entrada" +
                                        " FROM Acceso JOIN Llave_Prestada USING (Entrada)" +
                                        " WHERE Devuelta = false" +
                                            " UNION" +
                                        " SELECT DNI, 'Llavero' Tipo, Cod_Llavero Codigo, Entrada" +
                                        " FROM Acceso JOIN Llavero_Prestado USING (Entrada)" +
                                        " WHERE Devuelto = false	) Prestamos_Pendientes" +
                            " WHERE DNI = \"" + getDNI() + "\"" +
                            " GROUP BY Codigo;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerDatosPrestamosPendientes: Metodo que obtiene los datos de todos los prestamos pendientes que estan pendientes de devolucion desde un numero de dias indicado
         *
         * @param   int         Dias        Dias desde los que esta pendiente el prestamo
         * @return  ResultSet   rs          Datos de los los prestamos pendientes de devolucion tras los dias indicados
         */
        public ResultSet obtenerDatosPrestamosPendientes(int Dias) throws SQLException
        {
            // Creamos la String a ejecutar
            String query = "SELECT Tipo, Codigo, Entrada" +
                            " FROM (	SELECT DISTINCT T.DNI, 'Tarjeta' Tipo, Cod_Tarj Codigo, Entrada" +
                                        " FROM Tarjeta T JOIN Acceso USING (Cod_Tarj)" +
                                        " WHERE T.DNI IS NOT NULL" +
                                            " UNION" +
                                        " SELECT DNI, 'Llave' Tipo, Cod_Llave Codigo, Entrada" +
                                        " FROM Acceso JOIN Llave_Prestada USING (Entrada)" +
                                        " WHERE Devuelta = false" +
                                            " UNION" +
                                        " SELECT DNI, 'Llavero' Tipo, Cod_Llavero Codigo, Entrada" +
                                        " FROM Acceso JOIN Llavero_Prestado USING (Entrada)" +
                                        " WHERE Devuelto = false	) Prestamos_Pendientes" +
                            " WHERE DNI = \"" + getDNI() + "\" AND ADDDATE(DATE(Entrada),INTERVAL " + Dias + " DAY)=CURDATE();";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* eliminarPersonal: Metodo que borra todas aquellas personas que constan en Personal y no disponen de ninguna especializacion en ninguna tabla derivada
         *
         * @return  int     Numero de personas eliminadas
         */
        public int eliminarPersonal() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Personal WHERE DNI NOT IN (SELECT DNI FROM Periodico" +
                                                                    " UNION" +
                                                                    " SELECT DISTINCT DNI FROM Contacto" +
                                                                    " UNION" +
                                                                    " SELECT DISTINCT DNI FROM Ocasional" +
                                                                    " UNION" +
                                                                    " SELECT DNI FROM Residente);";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
}