// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Residente_I;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;



/** CLASE RESIDENTE:  Clase que representa un trabajador residente
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Residente extends Personal implements Residente_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Nivel:      Nivel de acceso al centro que posee el trabajador residente */
	private Nivel   Nivel;

        /** UCAL:                               Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar        UCAL = new UtilidadesCalendar();



	// METODOS -------------------------------------------------------------
        // CONSTRUCTORES
	/** Constructor: Constructor de clase que crea el objeto Residente con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
         * @param   Apellido1   String con el primer apellido de la persona
         * @param   Apellido2   String con el segundo apellido de la persona
         * @param   Nombre      String con el nombre de la persona
         * @param   Insertador  Insertador de la persona en la BD
         * @param   Nivel       Nivel de acceso al centro que posee el trabajador residente
	 * @return  Residente	Objeto de tipo Residente
	 */
	public Residente (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, Nivel Nivel) throws DataException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI, Apellido1, Apellido2, Nombre, Insertador);
            this.Nivel = Nivel;
	}
	/** Constructor: Constructor de clase que crea el objeto Residente con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
         * @param   Apellido1   String con el primer apellido de la persona
         * @param   Apellido2   String con el segundo apellido de la persona
         * @param   Nombre      String con el nombre de la persona
         * @param   Insertador  Insertador de la persona en la BD
         * @param   Nivel       Nivel de acceso al centro que posee el trabajador residente
         * @param   Telefonos   Lista con los telefonos de contacto
         * @param   Emails      Lista con los e-mails de contacto
	 * @return  Residente	Objeto de tipo Residente
	 */
	public Residente (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, Nivel Nivel, ArrayList<String> Telefonos, ArrayList<String> Emails) throws DataException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI, Apellido1, Apellido2, Nombre, Insertador, Telefonos, Emails);
            // Comprobamos que los parametros pasados son correctos
            this.Nivel = Nivel;
	}
	/** Constructor: Constructor de clase que crea el objeto Residente con los valores pasados parametricamente
	 *
	 * @param   DNI         String con el DNI de la persona
	 * @return  Residente	Objeto de tipo Residente
	 */
	public Residente (String DNI) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI);
            // Comprobamos que los parametros pasados son correctos
            if ((DNI == null) || (DNI.length()>9))                  throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Cod_Niv,F_Alta_N FROM Residente WHERE DNI = \"" + DNI + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            rs.getObject(1);
            if (rs.wasNull())   this.Nivel = null;
            else                this.Nivel = new Nivel(rs.getInt(1),rs.getString(2).substring(0,19));

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getNivel: Metodo que obtiene el valor del atributo Nivel
	 *
	 * @param   void
	 * @return  Nivel     Nivel de acceso al centro que posee el trabajador residente
	 */
	public Nivel getNivel()                             { return this.Nivel; }
        /** setNivel: Metodo que establece el valor del atributo Nivel
	 *
	 * @param   Nivel     Nivel de acceso al centro que posee el trabajador residente
	 * @return  void
	 */
	public void setNivel(Nivel Nivel) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if(Nivel == null)                               throw new DataException();
            // Realizamos la asignacion del atributo
            this.Nivel = Nivel;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return super.toString() + "\n" +
            "Nivel:\n" + getNivel().toString() + "\n";
	}


        // DML
        /** insertResidenteBD: Metodo que inserta el Residente en la BD
         *
         * @param   void
         * @return  int     Numero de filas (residentes) insertadas
         */
        public int insertResidenteBD() throws DataException, SQLException
        {
            // INSERTAMOS LOS DATOS PERSONALES
            int res = super.insertPersonalBD();

            // Si los datos personales han sido correctamente insertados -> INSERTAMOS LOS DATOS ESPECIFICOS DEL PERSONAL RESIDENTE
            if (res == 1)
            {
                // Creamos la String con la sentencia a realizar
                String query = "INSERT INTO Residente VALUES (\""  +
                        getDNI() + "\"," +
                        (getNivel() == null ? ("null,null);") : (getNivel().getCodNiv() + ",\"" + UCAL.calendarToString(getNivel().getFechaI()) + "\");"));

                // Ejecutamos la consulta y devolvemos el resultado
                return UBD.ejecutarUpdate(query);
            }

            // Devolvemos el resultado
            return res;
        }
        /** updateResidenteBD: Metodo que actualiza el Residente en la BD
         *
         * @param   Residente       Residente a modificar
         * @return  int             Numero de filas (residentes) actualizadas
         */
        public int updateResidenteBD(Residente Residente) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Residente == null)                                  throw new DataException();
            // Realizamos la actualizacion del residente

            // ACTUALIZAMOS LOS DATOS PERSONALES
            updatePersonalBD(Residente);

            // ACTUALIZAMOS LOS DATOS ESPECIFICOS
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Residente SET "  +
                    "Cod_Niv = " + (Residente.getNivel() == null ? "null" : Residente.getNivel().getCodNiv()) + "," +
                    "F_Alta_N = " + (Residente.getNivel() == null ? "null" : ("\"" + UCAL.calendarToString(Residente.getNivel().getFechaI()) + "\"")) +
                    " WHERE DNI = \"" + getDNI() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeResidenteBD: Metodo que borra un Residente de la BD
         *
         * @param   void
         * @return  int             Numero de filas (residentes) eliminadas
         */
        public int removeResidenteBD() throws SQLException
        {
            // Borramos el residente y devolvemos el resultado (estado)
            return removePersonalBD();
        }
}