// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.T_Acceso_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;
import java.util.Calendar;
import java.util.StringTokenizer;
import java.text.SimpleDateFormat;



/** CLASE T_ACCESO:  Clase que representa un tipo de acceso al centro
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class T_Acceso extends DosFechas implements T_Acceso_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cod_Acc:                String con el codigo de acceso */
	private String              Cod_Acc;
        /** Actividad:              String con la actividad para la que se otorga el acceso */
	private String              Actividad;
        /** Dias:                   String con los dias en que esta permitido el acceso */
        private String              Dias;
        /** Aut_Tel:                Indica si es necesario avisar al autorizador del departamento cada vez que acceda algun empleado de esta */
        private boolean             Aut_Tel;
        /** Nivel:                  Nivel de acceso asignado al tipo de acceso */
        private Nivel               Nivel;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  T_Acceso      Objeto de tipo T_Acceso
         */
        public T_Acceso () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto T_Acceso con los valores pasados parametricamente
	 *
	 * @param   Cod_Acc     String con el codigo de acceso
         * @param   Actividad   String con la actividad para la que se otorga el acceso
         * @param   Dias        String con los dias en que esta permitido el acceso
         * @param   Hora_Inic   Hora a partir de la cual esta permitido acceder al centro
         * @param   Hora_Fin    Hora maxima de salida del centro
         * @param   Aut_Tel     Indica si es necesario avisar al autorizador del departamento cada vez que acceda algun empleado de esta
         * @param   Nivel       Nivel de acceso asignado al tipo de acceso
	 * @return  T_Acceso	Objeto de tipo T_Acceso
	 */
	public T_Acceso(String Cod_Acc, String Actividad, String Dias, Calendar Hora_Inic, Calendar Hora_Fin, boolean Aut_Tel, Nivel Nivel) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(Hora_Inic,Hora_Fin);

            // Comprobamos la correccion de los dias pasados
            boolean diasCorrectos = true;
            int i = 0;
            while ((diasCorrectos) && (i<Dias.length()))
            {
                if("LMXJVSD".indexOf(Dias.charAt(i)) == -1)     diasCorrectos = false;
                i++;
            }

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Acc == null) || (Actividad == null) || (Dias == null) || (Cod_Acc.length()>4) ||
                    (Actividad.length()>30) || (Dias.length() > 7) || (!diasCorrectos))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Acc = Cod_Acc.toUpperCase();
            this.Actividad = Actividad;
            this.Dias = Dias;
            this.Aut_Tel = Aut_Tel;
            this.Nivel = Nivel;
	}
	/** Constructor: Constructor de clase que crea el objeto T_Acceso con los valores pasados parametricamente
	 *
	 * @param   Cod_Acc     String con el codigo de acceso
	 * @return  T_Acceso	Objeto de tipo T_Acceso
	 */
	public T_Acceso(String Cod_Acc) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Acc == null) || (Cod_Acc.length()>4))          throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM T_Acceso WHERE Cod_Acc = \"" + Cod_Acc + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Acc = rs.getString(1);
            this.Actividad = rs.getString(2);
            this.Dias = rs.getString(3);
            setFechaI(stringToTime(rs.getString(4)));
            Calendar Hora_Fin = stringToTime(rs.getString(5));
            Hora_Fin.set(Hora_Fin.YEAR, Hora_Fin.get(Hora_Fin.YEAR)+1);
            setFechaJ(Hora_Fin);
            this.Aut_Tel = (rs.getInt(6) == 0 ? false : true);
            rs.getObject(7);
            if (rs.wasNull())   this.Nivel = null;
            else                this.Nivel = new Nivel(rs.getInt(7),rs.getString(8).substring(0,19));

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodAcc: Metodo que obtiene el valor del atributo Cod_Acc
	 *
	 * @param   void
	 * @return  String	String con el codigo de acceso
	 */
	public String getCodAcc()			{ return this.Cod_Acc; }
        /** setCodAcc: Metodo que establece el valor del atributo Cod_Acc
	 *
	 * @param   Cod_Acc     String     String con el codigo de acceso
	 * @return  void
	 */
	public void setCodAcc(String Cod_Acc) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Acc == null) || (Cod_Acc.length()>4))  throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Acc = Cod_Acc.toUpperCase();
        }
	
        /** getActividad: Metodo que obtiene el valor del atributo Actividad
	 *
	 * @param   void
	 * @return  String	String con la actividad para la que se otorga el acceso
	 */
	public String getActividad()			{ return this.Actividad; }
        /** setActividad: Metodo que establece el valor del atributo Actividad
	 *
	 * @param   Actividad      String     String con la actividad para la que se otorga el acceso
	 * @return  void
	 */
	public void setActividad(String Actividad) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Actividad == null) || (Actividad.length()>30)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Actividad = Actividad;
        }

        /** getDias: Metodo que obtiene el valor del atributo Dias
	 *
	 * @param   void
	 * @return  String      String con los dias en los que esta permitido el acceso
	 */
	public String getDias()                         { return this.Dias; }
        /** setDias: Metodo que establece el valor del atributo Dias
	 *
	 * @param   Dias      String         String con los dias en los que esta permitido el acceso
	 * @return  void
	 */
	public void setDias(String Dias) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Dias.length() != 7)                       throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Dias = Dias;
        }
        
        /** getAutTel: Metodo que obtiene el valor del atributo Aut_Tel
	 *
	 * @param   void
	 * @return  boolean	Indica si es necesario avisar al autorizador del departamento cada vez que acceda algun empleado de esta
	 */
	public boolean getAutTel()                      { return this.Aut_Tel; }
        /** setAutTel: Metodo que establece el valor del atributo Aut_Tel
	 *
	 * @param   Aut_Tel      boolean     Indica si es necesario avisar al autorizador del departamento cada vez que acceda algun empleado de esta
	 * @return  void
	 */
	public void setAutTel(boolean Aut_Tel)          { this.Aut_Tel = Aut_Tel; }

        /** getNivel: Metodo que obtiene el valor del atributo Nivel
	 *
	 * @param   void
	 * @return  Nivel	Nivel de acceso asignado al tipo de acceso
	 */
	public Nivel getNivel()                         { return this.Nivel; }
        /** setNivel: Metodo que establece el valor del atributo Nivel
	 *
	 * @param   Nivel      Nivel     Nivel de acceso asignado al tipo de acceso
	 * @return  void
	 */
	public void setNivel(Nivel Nivel)               { this.Nivel = Nivel; }


        // TOSTRING
        /** timeToString: Metodo que pasa una hora a una String
	 *
	 * @param   fecha      Calendar     Objeto Calendar a pasar a String
	 * @return  void
	 */
        public String timeToString (Calendar Hora)
        {
             SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");

             return dateFormat.format(Hora.getTime());
        }

        /** stringToTime: Metodo que pasa una String a objeto Calendar con solo hora
	 *
	 * @param   Fecha      String     Objeto String a pasar a Calendar (con solo hora)
	 * @return  void
	 */
        public Calendar stringToTime(String Hora)
        {
            // Creamos un StringTokenizer con el que recorrer la String
            StringTokenizer st = new StringTokenizer(Hora);
            // Creamos un objeto Calendar
            Calendar calendar = Calendar.getInstance();
            // Establecemos el estado del objeto Calendar con los datos obtenidos del String de entrada
            calendar.set(1,1,1,Integer.parseInt(st.nextToken(":")),Integer.parseInt(st.nextToken()),
                    Integer.parseInt(st.nextToken().substring(0,2)));

            // Devolvemos el resultado
            return calendar;
        }

        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "Cod_Acc: " + getCodAcc() + "\n" +
            "Actividad: " + getActividad() + "\n" +
            "Dias: " + getDias() + "\n" +
            super.toString() + "\n" +
            "Aut_Tel: " + (getAutTel() ? "Si" : "No") + "\n\n" +
            (getNivel() == null ? "" : getNivel().toString()) + "\n";
	}


        // DML
        /** insertTAccesoBD: Metodo que inserta el T_Acceso en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertTAccesoBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO T_Acceso VALUES (\""  +
                    getCodAcc() + "\",\"" +
                    getActividad() + "\",\"" +
                    getDias() + "\",\"" +
                    timeToString(getFechaI()) + "\",\"" +
                    timeToString(getFechaJ()) + "\"," +
                    (getAutTel() ? "true" : "false") + "," +
                    (getNivel() == null ? "null" : getNivel().getCodNiv()) + "," +
                    (getNivel() == null ? "null" : "\"" + calendarToString(getNivel().getFechaI()) + "\"") + ");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateTAccesoBD: Metodo que actualiza el T_Acceso en la BD
         *
         * @param   T_Acceso    T_Acceso    T_Acceso a actualizar
         * @return  int                     Numero de filas actualizadas
         */
        public int updateTAccesoBD(T_Acceso T_Acceso) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE T_Acceso SET " +
                    "Cod_Acc = \"" + T_Acceso.getCodAcc() + "\"," +
                    "Actividad = \"" + T_Acceso.getActividad() + "\"," +
                    "Dias = \"" + T_Acceso.getDias() + "\"," +
                    "Hora_Inic = \"" + timeToString(T_Acceso.getFechaI()) + "\"," +
                    "Hora_Fin = \"" + timeToString(T_Acceso.getFechaJ()) + "\"," +
                    "Aut_Tel = " + (T_Acceso.getAutTel() ? "true" : "false") + "," +
                    "Cod_Niv = " + (T_Acceso.getNivel() == null ? "null" : T_Acceso.getNivel().getCodNiv()) + "," +
                    "F_Alta_N = " + (T_Acceso.getNivel() == null ? "null" : ("\"" + calendarToString(T_Acceso.getNivel().getFechaI()) + "\"")) +
                    " WHERE Cod_Acc = \"" + getCodAcc() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeTAccesoBD: Metodo que borra un T_Acceso de la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeTAccesoBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM T_Acceso WHERE Cod_Acc = \"" + getCodAcc() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /** obtenerTAccesos: Metodo que obtiene los tipos de acceso existentes
         *
         * @param   void
         * @return  ResultSet   rs      Tipos de acceso existentes
         */
        public ResultSet obtenerTAccesos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Acc Código, Actividad FROM T_Acceso;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
}