// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.T_Usuario_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;



/** CLASE T_USUARIO:  Clase que representa un tipo de usuario existente en el sistema
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class T_Usuario implements T_Usuario_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Tipo:                   Tipo de usuario */
	private String              Tipo;
        /** Descripcion:            String con la descripcion del tipo de usuario */
	private String              Descripcion;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  T_Usuario      Objeto de tipo T_Usuario
         */
        public T_Usuario () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto T_Usuario con los valores pasados parametricamente
	 *
	 * @param   Tipo        Tipo de usuario
         * @param   Descripcion String con la descripcion del tipo de usuario
	 * @return  T_Usuario	Objeto de tipo T_Usuario
	 */
	public T_Usuario(String Tipo, String Descripcion) throws DataException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((Tipo == null) || (Descripcion == null) || (Tipo.length()>15) || (Descripcion.length()>50))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Tipo = Tipo.toUpperCase();
            this.Descripcion = Descripcion;
	}
        /** Constructor: Constructor de clase que crea el objeto T_Usuario con los valores pasados parametricamente
	 *
	 * @param   Tipo        Tipo de usuario
	 * @return  T_Usuario	Objeto de tipo T_Usuario
	 */
	public T_Usuario(String Tipo) throws DataException, SQLException, RemoteException
	{
            // Comprobamos que los parametros pasados son correctos
            if ((Tipo == null) || (Tipo.length()>15))           throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM T_Usuario WHERE Tipo = \"" + Tipo + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Tipo = rs.getString(1);
            this.Descripcion = rs.getString(2);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getTipo: Metodo que obtiene el valor del atributo Tipo
	 *
	 * @param   void
	 * @return  String     Tipo de usuario
	 */
	public String getTipo()                         { return this.Tipo; }
        /** setTipo: Metodo que establece el valor del atributo estado
	 *
	 * @param   Tipo     String     Tipo de usuario
	 * @return  void
	 */
	public void setTipo(String Tipo) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Tipo == null) || (Tipo.length()>15))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Tipo = Tipo.toUpperCase();
        }
	
        /** getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	 *
	 * @param   void
	 * @return  String     String con la descripcion del tipo de usuario
	 */
	public String getDescripcion()			{ return this.Descripcion; }
        /** setDescripcion: Metodo que establece el valor del atributo Descripcion
	 *
	 * @param   Descripcion     String     String con la descripcion del tipo de usuario
	 * @return  void
	 */
	public void setDescripcion(String Descripcion) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Descripcion.length()>50)) throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Descripcion = Descripcion;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "T_USUARIO:\nTipo: " + getTipo() + "\n" +
            "Descripcion: " + getDescripcion() + "\n";
	}

        
        // DML
        /** insertTUsuarioBD: Metodo que inserta el T_Usuario en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertTUsuarioBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO T_Usuario VALUES (\""  +
                    getTipo() + "\",\"" +
                    getDescripcion() + "\");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** updateTUsuarioBD: Metodo que actualiza el T_Usuario en la BD
         *
         * @param   T_Usuario       T_Usuario con el que modificar
         * @return  int             Numero de filas actualizadas
         */
        public int updateTUsuarioBD(T_Usuario T_Usuario) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE T_Usuario SET " +
                    "Tipo = \"" + T_Usuario.getTipo() + "\"," +
                    "Descripcion = \"" + T_Usuario.getDescripcion() +
                    "\" WHERE Tipo = \"" + getTipo() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** removeTUsuarioBD: Metodo que borra un T_Usuario de la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeTUsuarioBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM T_Usuario WHERE Tipo = \"" + getTipo() + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* obtenerCodsTUsuarios: Metodo que devuelve todos codigos de los tipos de usuarios existentes en el sistema
         *
         * @param   void
         * @return  ResultSet   rs          Codigos de los tipos de usuarios existentes en el sistema
         */
        public ResultSet obtenerCodsTUsuarios() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Tipo FROM T_Usuario;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
        /* obtenerTUsuarios: Metodo que devuelve todos tipos de usuarios existentes en el sistema
         *
         * @param   void
         * @return  ResultSet   rs          Tipos de usuarios existentes en el sistema
         */
        public ResultSet obtenerTUsuarios() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Tipo, Descripcion Descripción FROM T_Usuario;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
}