// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Tarjeta_I;
import API.Utilidades.UtilidadesBD;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.Calendar;



/** CLASE TARJETA:  Clase que representa la tarjeta empleada para acceder al centro
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Tarjeta extends DosFechas implements Tarjeta_I
{
	// ATRIBUTOS -----------------------------------------------------------
        //** Cod_Tarj:              Numero de tarjeta */
	private int                 Cod_Tarj;
        /** Color:                  Color de la tarjeta */
	private Color               Color;
        /** Persona:                Persona que posee la tarjeta en prestamo */
        private Personal            Persona;

        /** UBD:                    Objeto de tipo UtilidadesBD */
        public static UtilidadesBD  UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Tarjeta      Objeto de tipo Tarjeta
         */
        public Tarjeta () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Tarjeta con los valores pasados parametricamente
	 *
	 * @param   Cod_Tarj    Numero de tarjeta
         * @param   F_Alta_T    Fecha en la que ha sido dada de alta la tarjeta
         * @param   Color       Color de la tarjeta
	 * @return  Tarjeta	Objeto de tipo Tarjeta
	 */
	public Tarjeta(int Cod_Tarj, Calendar F_Alta_T, Color Color) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta_T);

            // Comprobamos que los parametros pasados son correctos
            if ((Color == null) || (Cod_Tarj<0) || (Cod_Tarj>999) || (Color.getFechaI().after(F_Alta_T)))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Tarj = Cod_Tarj;
            this.Color = Color;
            this.Persona = null;
	}
	/** Constructor: Constructor de clase que crea el objeto Tarjeta con los valores pasados parametricamente
	 *
	 * @param   Cod_Tarj    Numero de tarjeta
         * @param   F_Alta_T    Fecha en la que ha sido dada de alta la tarjeta
         * @param   F_Baja_T    Fecha en la que ha sido dada de baja la tarjeta
         * @param   Color       Color de la tarjeta
         * @param   DNI         DNI de la persona que posee la tarjeta en prestamo
	 * @return  Tarjeta	Objeto de tipo Tarjeta
	 */
	public Tarjeta(int Cod_Tarj, Calendar F_Alta_T, Calendar F_Baja_T, Color Color, Personal Persona) throws DataException, RemoteException
	{
            // Llamamos al constructor base
            super(F_Alta_T,F_Baja_T);

            // Comprobamos que los parametros pasados son correctos
            if ((Color == null) || (Cod_Tarj<0) || (Cod_Tarj>999) ||
                    (Color.getFechaI().after(F_Alta_T)) ||
                    (Color.getFechaJ() != null) && (F_Baja_T != null) && (F_Baja_T.after(Color.getFechaJ())))
                throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Tarj = Cod_Tarj;
            this.Color = Color;
            this.Persona = Persona;
	}
	/** Constructor: Constructor de clase que crea el objeto Tarjeta con los valores pasados parametricamente
	 *
	 * @param   Cod_Tarj    Numero de tarjeta
         * @param   F_Alta_T    Fecha en la que ha sido dada de alta la tarjeta
	 * @return  Tarjeta	Objeto de tipo Tarjeta
	 */
	public Tarjeta(int Cod_Tarj, String F_Alta_T) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((F_Alta_T == null) || (Cod_Tarj<0) || (Cod_Tarj>999) ||  (F_Alta_T.length()>19))
                throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Tarjeta WHERE Cod_Tarj = " + Cod_Tarj +
                " AND F_Alta_T = \"" + F_Alta_T + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            if (rs.next())
            {
                this.Cod_Tarj = rs.getInt(1);
                setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
                rs.getObject(3);
                if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3)));
                this.Color = new Color (rs.getInt(4), rs.getString(5).substring(0,19));
                rs.getObject(6);
                if (rs.wasNull())   this.Persona = null;
                else                this.Persona = new Personal(rs.getString(6));
            }

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
	/** Constructor: Constructor de clase que crea el objeto Tarjeta con los valores pasados parametricamente
	 *
	 * @param   Cod_Tarj    Numero de tarjeta
	 * @return  Tarjeta	Objeto de tipo Tarjeta
	 */
	public Tarjeta(int Cod_Tarj) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Tarj<0) || (Cod_Tarj>999))                     throw new DataException();

            // Obtenemos la ultima fecha de alta de la tarjeta
            String query = "SELECT MAX(F_Alta_T) FROM Tarjeta WHERE Cod_Tarj = " + Cod_Tarj + ";";
            ResultSet rs = UBD.ejecutarQuery(query);
            rs.next();
            String F_Alta_T = rs.getString(1).substring(0,19);
            UBD.cerrarConexion();
            
            // Obtenemos los valores de la tarjeta
            query = "SELECT * FROM Tarjeta WHERE Cod_Tarj = " + Cod_Tarj +
                " AND F_Alta_T = \"" + F_Alta_T + "\";";
            rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Tarj = rs.getInt(1);
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            rs.getObject(3);
            if (!rs.wasNull())      setFechaJ(stringToCalendar(rs.getString(3).substring(0,19)));
            this.Color = new Color (rs.getInt(4), rs.getString(5).substring(0,19));
            rs.getObject(6);
            if (rs.wasNull())   this.Persona = null;
            else                this.Persona = new Personal(rs.getString(6));

            // Cerramos la conexion
            UBD.cerrarConexion();
	}

	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodTarj: Metodo que obtiene el valor del atributo Cod_Tarj
	 *
	 * @param   void
	 * @return  Cod_Tarj	Numero de tarjeta
	 */
	public int getCodTarj()                         { return this.Cod_Tarj; }
        /** setCodTarj: Metodo que establece el valor del atributo Cod_Tarj
	 *
	 * @param   persona     Numero de tarjeta
	 * @return  void
	 */
	public void setCodTarj(int Cod_Tarj) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Tarj<0) || (Cod_Tarj>999))         throw new DataException();
            // Realizamos la asignacion del atributo
            this.Cod_Tarj = Cod_Tarj;
        }
	
        /** getColor: Metodo que obtiene el valor del atributo Color
	 *
	 * @param   void
	 * @return  Color   Objeto de tipo Color con el color de la tarjeta
	 */
	public Color getColor()                         { return this.Color; }
        /** setColor: Metodo que establece el valor del atributo Color
	 *
	 * @param   Color   Color de la tarjeta
	 * @return  void
	 */
	public void setColor(Color Color) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Color == null) || (Color.getFechaI().after(getFechaI())) ||
                (Color.getFechaJ() != null) && (getFechaJ() != null) && (getFechaJ().after(Color.getFechaJ())))   throw new DataException();
            this.Color = Color;
        }

        /** getPersona: Metodo que obtiene el valor del atributo Persona
         *
         * @param   void
         * @return  Persona Objeto de tipo Persona con la persona que posee la tarjeta en prestamo
         */
        public Personal getPersona()                    { return this.Persona; }
        /** setPersona: Metodo que establece el valor del atributo Persona
          * 
          * @param  Persona Persona que posee la tarjeta en prestamo
          * @return void
          */
        public void setPersona(Personal Persona) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Persona == null)                        throw new DataException();
            this.Persona = Persona;
        }

        // TOSTRING       
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return "TARJETA:\nCod_Tarj: " + getCodTarj() + "\n" +
            super.toString() + "\n" +
            getColor().toString() + "\n" +
            (getPersona() == null ? "" : getPersona().toString()) + "\n";
	}


        // DML
        /** insertTarjetaBD: Metodo que inserta la Tarjeta en la BD
         *
         * @param   void
         * @return  int     Numero de filas insertadas
         */
        public int insertTarjetaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Tarjeta VALUES ("  +
                    getCodTarj() + ",\"" +
                    calendarToString(getFechaI()) + "\"," +
                    (getFechaJ() == null ? "null" : "\"" + calendarToString(getFechaJ()) + "\"") + ",\"" +
                    getColor().getCodColor() + "\",\"" +
                    calendarToString(getColor().getFechaI()) + "\"," +
                    (getPersona() == null ? "null" : "\"" + getPersona().getDNI() + "\"") + ");";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** updateTarjetaBD: Metodo que actualiza la Tarjeta en la BD
         *
         * @param   Tarjeta     Tarjeta con la que modificar
         * @return  int         Numero de filas actualizadas
         */
        public int updateTarjetaBD(Tarjeta Tarjeta) throws SQLException
        {
            // COMPROBACION DE ARGS
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Tarjeta SET " +
                    "Cod_Tarj = " + Tarjeta.getCodTarj() + "," +
                    "F_Alta_T = \"" + calendarToString(Tarjeta.getFechaI()) + "\"," +
                    "F_Baja_T = " + (Tarjeta.getFechaJ() == null ? "null" : "\"" + calendarToString(getFechaJ()) + "\"") + "," +
                    "Cod_Color = \"" + Tarjeta.getColor().getCodColor() + "\"," +
                    "F_Alta_C = \"" + calendarToString(Tarjeta.getColor().getFechaI()) + "\"," +
                    "DNI = " + (Tarjeta.getPersona() == null ? "null" : "\"" + Tarjeta.getPersona().getDNI() + "\"") +
                    " WHERE Cod_Tarj = " + getCodTarj() + " AND F_Alta_T = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** devTarjetaBD: Metodo que devuelve la Tarjeta en la BD
         *
         * @param   void
         * @return  int         Numero de filas actualizadas
         */
        public int devTarjetaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Tarjeta SET " +
                    "DNI = null WHERE Cod_Tarj = " + getCodTarj() + " AND F_Alta_T = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** darBajaTarjetaBD: Metodo que da de baja la Tarjeta en la BD
         *
         * @param   void
         * @return  int         Numero de filas actualizadas
         */
        public int darBajaTarjetaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Tarjeta" +
                    " SET F_Baja_T = \"" + calendarToString(Calendar.getInstance()) + "\"" +
                    " WHERE Cod_Tarj = " + getCodTarj() + " AND F_Alta_T = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** removeTarjetaBD: Metodo que borra una Tarjeta en la BD
         *
         * @param   void
         * @return  int     Numero de filas eliminadas
         */
        public int removeTarjetaBD() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Tarjeta WHERE Cod_Tarj = " + getCodTarj() + " AND F_Alta_T = \"" + calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* obtenerTarjetas: Metodo que devuelve los codigos de tarjetas que estan en circulacion
         *
         * @return  ResultSet   rs          Codigos de las tarjetas en circulacion
         */
        public ResultSet obtenerTarjetas() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Tarj 'Código'" +
                    " FROM Tarjeta JOIN Color USING (Cod_Color,F_Alta_C) JOIN Nivel USING (Cod_Niv,F_Alta_N)" +
                    " WHERE F_Baja_T IS NULL AND F_Baja_C IS NULL AND F_Baja_N IS NULL ORDER BY Cod_Tarj;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerColoresTarjs: Metodo que devuelve los colores de las tarjetas que estan en circulacion
         *
         * @return  ArrayList   al          Colores de las tarjetas en circulacion
         */
        public ArrayList obtenerColoresTarjs() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Color" +
                    " FROM Tarjeta JOIN Color USING (Cod_Color,F_Alta_C) JOIN Nivel USING (Cod_Niv,F_Alta_N)" +
                    " WHERE F_Baja_T IS NULL AND F_Baja_C IS NULL AND F_Baja_N IS NULL ORDER BY Cod_Tarj;";
            
            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Creamos el resultado
            ArrayList al = new ArrayList();
            while(rs.next())    al.add(rs.getInt(1));

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return al;
        }

        /* obtenerColoresTarjsPrest: Metodo que devuelve los colores de las tarjetas que estan en prestamo
         *
         * @return  ArrayList   al          Colores de las tarjetas en prestamo
         */
        public ArrayList obtenerColoresTarjsPrest() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Color" +
                            " FROM (SELECT Cod_Tarj, Cod_Color" +
                                    " FROM Tarjeta JOIN Color USING (Cod_Color,F_Alta_C) JOIN Nivel USING (Cod_Niv,F_Alta_N)" +
                                    " WHERE F_Baja_T IS NULL AND F_Baja_C IS NULL AND F_Baja_N IS NULL AND DNI IS NOT NULL" +
                                    " ORDER BY Cod_Tarj) Colores;";
            
            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Creamos el resultado
            ArrayList al = new ArrayList();
            while(rs.next())    al.add(rs.getInt(1));

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return al;
        }

        /* obtenerPrestamos: Metodo que devuelve los datos de todas las tarjetas que permanecen en prestamo
         *
         * @return  ResultSet   rs          Datos de las tarjetas en prestamo
         */
        public ResultSet obtenerPrestamos() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT Cod_Tarj 'Código', MAX(Entrada) 'Fecha', T.DNI" +
                    " FROM Tarjeta T JOIN Acceso USING (Cod_Tarj,F_Alta_T) JOIN Color USING (Cod_Color,F_Alta_C) JOIN Nivel USING (Cod_Niv,F_Alta_N)" +
                    " WHERE F_Baja_T IS NULL AND F_Baja_C IS NULL AND F_Baja_N IS NULL AND T.DNI IS NOT NULL" +
                    " GROUP BY Cod_Tarj" +
                    " ORDER BY Cod_Tarj;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* numTarjPorDefecto: Metodo que devuelve el numero de tarjeta por defecto
         *
         * @return  int   NumTarj          Numero de tarjeta por defecto
         */
        public int numTarjPorDefecto() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Codigo" +
                            " FROM Codigo" +
                            " WHERE codigo NOT IN (SELECT Cod_Tarj" +
                                                    " FROM Tarjeta JOIN Color USING (Cod_Color,F_Alta_C) JOIN Nivel USING (Cod_Niv,F_Alta_N)" +
                                                    " WHERE F_Baja_T IS NULL AND F_Baja_C IS NULL AND F_Baja_N IS NULL)" +
                            " ORDER BY Codigo ASC " +
                            " LIMIT 0,1;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int NumTarj = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return NumTarj;
        }

        /* numTarjAcc: Metodo que devuelve el numero de tarjeta seleccionado para el acceso segun el color proporcionado
         *
         * @param   int     Color       Color del que extraer la tarjeta a otorgar en el acceso
         * @return  int     NumTarj     Numero de tarjeta seleccionado para el acceso
         */
        public int numTarjAcc(int Color) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT MIN(Cod_Tarj) FROM Tarjeta JOIN Color USING (Cod_Color,F_Alta_C) WHERE Cod_Color = " + Color + " AND F_Baja_C IS NULL AND DNI IS NULL;";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Obtenemos el resultado
            rs.next();
            int NumTarj = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return NumTarj;
        }

        /* eliminarTarjetas: Metodo que borra todas las tarjetas mas antiguas de Dias dias
         *
         * @param   Dias    Numero de dias transcurridos desde la baja de la tarjeta
         * @return  int     Numero de tarjetas eliminadas
         */
        public int eliminarTarjetas(int Dias) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Tarjeta WHERE ADDDATE(DATE(F_Baja_T),INTERVAL " + Dias + " DAY)<=CURDATE()" +
                    " AND (Cod_Tarj,F_Alta_T) NOT IN (SELECT DISTINCT Cod_Tarj,F_Alta_T FROM Acceso);";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /* disparador: Metodo que ejecuta el disparador sobre BEFORE INSERT ON Tarjeta
         *
         * @param   void
         * @return  int
         */
        public int disparador() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Tarjeta SET F_Baja_T = NOW() WHERE Cod_Tarj = " + getCodTarj() + " AND F_Baja_T IS NULL;";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }
}