// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Vigilante_I;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;



/** CLASE VIGILANTE:  Clase que representa un vigilante de seguridad
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Vigilante extends Periodico_Contacto implements Vigilante_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cod_Vig:            Codigo de vigilante de seguridad */
	private int             Cod_Vig;



	// METODOS -------------------------------------------------------------
        // CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Vigilante      Objeto de tipo Vigilante
         */
        public Vigilante () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto Vigilante con los valores pasados parametricamente
	 *
	 * @param   DNI             String con el DNI de la persona
         * @param   Apellido1       String con el primer apellido de la persona
         * @param   Apellido2       String con el segundo apellido de la persona
         * @param   Nombre          String con el nombre de la persona
         * @param   Insertador      Insertador de la persona en la BD
         * @param   Cod_Vig         Codigo del vigilante de seguridad
         * @param   Telefonos       Telefonos de contacto
         * @param   Emails          Emails de contacto
	 * @return  Vigilante       Objeto de tipo Vigilante
	 */
	public Vigilante (String DNI, String Apellido1, String Apellido2, String Nombre, Personal Insertador, int Cod_Vig, ArrayList Telefonos, ArrayList Emails) throws DataException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI, Apellido1, Apellido2, Nombre, Insertador, Telefonos, Emails);
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Vig<0) || (Cod_Vig>999999))                throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Vig = Cod_Vig;
	}
	/** Constructor: Constructor de clase que crea el objeto Vigilante con los valores pasados parametricamente
	 *
	 * @param   DNI             String con el DNI de la persona
         * @param   Cod_Vig         Codigo de vigilante
	 * @return  Vigilante       Objeto de tipo Vigilante
	 */
	public Vigilante (String DNI, int Cod_Vig) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI);
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Vig<0) || (Cod_Vig>999999))                throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Vig = Cod_Vig;
	}
	/** Constructor: Constructor de clase que crea el objeto Vigilante con los valores pasados parametricamente
	 *
	 * @param   DNI             String con el DNI de la persona
	 * @return  Vigilante       Objeto de tipo Vigilante
	 */
	public Vigilante (String DNI) throws DataException, SQLException, RemoteException
        {
            // Realizamos la asignacion de valores a los atributos bases
            super(DNI);

            // Obtenemos los valores de la tupla existente
            String query = "SELECT Cod_Vig FROM Vigilante WHERE DNI = \"" + DNI + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Vig = rs.getInt(1);

            // Cerramos la conexion
            UBD.cerrarConexion();
	}


	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodVig: Metodo que obtiene el valor del atributo Cod_Vig
	 *
	 * @param   void
	 * @return  Cod_Vig     Codigo de vigilante de seguridad
	 */
	public int getCodVig()                              { return this.Cod_Vig; }
        /** setCodVig: Metodo que establece el valor del atributo Cod_Vig
	 *
	 * @param   Cod_Vig     Codigo de vigilante de seguridad
	 * @return  void
	 */
	public void setCodVig(int Cod_Vig) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if((Cod_Vig<0) || (Cod_Vig>999999))             throw new DataException();
            // Realizamos la asignacion del atributo
            this.Cod_Vig = Cod_Vig;
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // Devolvemos el resultado
            return super.toString() + "\n" +
            "Cod_Vig: " + getCodVig() + "\n";
	}


        // FUNCIONALIDADES
        /** existeVigilante: Metodo que denota si existe un vigilante con el codigo de vigilante proporcionado
	 *
	 * @param   int     void
	 * @return  boolean             Denota si exsiste un vigilante con el codigo de vigilante proporcionado
	 */
	public boolean existeVigilante() throws SQLException
	{
            // Creamos la String con la query a ejecutar
            String query = "SELECT * FROM Vigilante WHERE Cod_Vig = " + getCodVig() + ";";

            // Ejecutamos la consulta
            ResultSet rs = UBD.ejecutarQuery(query);

            // Recogemos el resultado
            boolean resultado = false;
            if (rs.next())  resultado = true;

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return resultado;
	}
        /** obtenerCodsVigilantes: Metodo que obtiene todos los codigos de vigilante que han redactado partes
	 *
         * @param   void
         * @return  ResultSet   rs          Codigos de vigilante de los vigilantes que han redactado partes
         */
        public ResultSet obtenerCodsVigilantes() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DISTINCT Cod_Vig FROM Parte_Seguridad;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }
        /** obtenerDNI: Metodo que obtiene el DNI del vigilante a partir de su codigo de vigilante
	 *
         * @param   int         Cod_Vig     Codigo de vigilante
         * @return  String      DNI         DNI del vigilante
         */
        public String obtenerDNI() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT DNI FROM Vigilante WHERE Cod_Vig = " + Cod_Vig + ";";

            // Obtenemos el resultado
            ResultSet rs = UBD.ejecutarQuery(query);
            rs.next();
            String DNI = rs.getString(1);
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return DNI;
        }
}