// PAQUETE
package API.Tablas;

// IMPORTACIONES
import API.Excepciones.DataException;
import API.Interfaces_RMI.Zona_I;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesCalendar;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;



/** CLASE ZONA:  Clase que representa una zona del centro
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Zona extends UnaFecha implements Zona_I
{
	// ATRIBUTOS -----------------------------------------------------------
        /** Cod_Zona:                       String con el codigo de zona */
	private String                      Cod_Zona;
        /** Descripcion:                    String con la descripcion de la zona */
        private String                      Descripcion;
        /** Nivel:                          Nivel necesario para acceder a la zona */
        private Nivel                       Nivel;
        /** Llaves:                         Llaves que pertenecen a la zona */
        private HashMap<String,Llave>       Llaves;

        /** UBD:                            Objeto de tipo UtilidadesBD */
        public static UtilidadesBD          UBD = new UtilidadesBD();
        /** UCAL:                           Objeto de tipo UtilidadesCalendar */
        public static UtilidadesCalendar    UCAL = new UtilidadesCalendar();

	
	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
        /** Constructor por defecto
         *
         * @param   void
         * @return  Zona      Objeto de tipo Zona
         */
        public Zona () throws DataException, RemoteException {}
	/** Constructor: Constructor de clase que crea el objeto DescripZona con los valores pasados parametricamente
	 *
	 * @param   Cod_Zona        String con el codigo de zona
         * @param   F_Alta_Z        Fecha y hora en la que es dada de alta la zona
         * @param   Edificio        Edificio en el que se encuentra situada la zona
         * @param   Descripcion     String con la descripcion de la zona
         * @param   Llaves          Llaves que pertenecen a la zona
	 * @return  Zona            Objeto de tipo Zona
	 */
	public Zona (String Cod_Zona, Calendar F_Alta_Z, String Descripcion, HashMap<String,Llave> Llaves) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(F_Alta_Z);
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Zona == null) || (Descripcion == null) || (Llaves == null) || (Llaves.isEmpty()) || (Cod_Zona.length()>10) || (Descripcion.length()>100))
                    throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Zona = Cod_Zona.toUpperCase();
            this.Descripcion = Descripcion;
            this.Nivel = null;
            this.Llaves = Llaves;
	}
        /** Constructor: Constructor de clase que crea el objeto DescripZona con los valores pasados parametricamente
	 *
	 * @param   Cod_Zona        String con el codigo de zona
         * @param   F_Alta_Z        Fecha y hora en la que es dada de alta la zona
         * @param   Edificio        Edificio en el que se encuentra situada la zona
         * @param   Descripcion     String con la descripcion de la zona
         * @param   Nivel           Nivel necesario para acceder a la zona
         * @param   Llaves          Llaves que pertenecen a la zona
	 * @return  Zona            Objeto de tipo Zona
	 */
	public Zona (String Cod_Zona, Calendar F_Alta_Z, String Descripcion, Nivel Nivel, HashMap<String,Llave> Llaves) throws DataException, RemoteException
        {
            // Llamamos al constructor base
            super(F_Alta_Z);

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Zona == null) || (Descripcion == null) || (Cod_Zona.length()>10) || (Descripcion.length()>100))
                    throw new DataException();

            // Realizamos la asignacion de valores a los atributos
            this.Cod_Zona = Cod_Zona.toUpperCase();
            this.Descripcion = Descripcion;
            this.Nivel = Nivel;
            this.Llaves = Llaves;
	}
	/** Constructor: Constructor de clase que crea el objeto Zona con los valores pasados parametricamente
	 *
	 * @param   Cod_Zona    Codigo de zona
         * @param   F_Alta_Z    Fecha en la que ha sido dada de alta la zona
	 * @return  Zona	Objeto de tipo Zona
	 */
	public Zona(String Cod_Zona, String F_Alta_Z) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Zona == null) || (F_Alta_Z == null) || (Cod_Zona.length()>10) || (F_Alta_Z.length()>19))
                throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Zona WHERE Cod_Zona = \"" + Cod_Zona +
                "\" AND F_Alta_Z = \"" + F_Alta_Z + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Zona = rs.getString(1);
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            this.Descripcion = rs.getString(3);
            rs.getObject(4);
            if (rs.wasNull())       this.Nivel = null;
            else                    this.Nivel = new Nivel(rs.getInt(4),rs.getString(5).substring(0,19));
            this.Llaves = getLlavesBD();

            // Cerramos la conexion
            UBD.cerrarConexion();
	}
	/** Constructor: Constructor de clase que crea el objeto Zona con los valores pasados parametricamente
	 *
	 * @param   Descripcion     Descripcion de zona
	 * @return  Zona            Objeto de tipo Zona
	 */
	public Zona(String Descripcion) throws DataException, SQLException, RemoteException
	{
            // Llamamos al constructor base
            super();

            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Descripcion.length()>100))            throw new DataException();

            // Obtenemos los valores de la tupla existente
            String query = "SELECT * FROM Zona WHERE Descripcion = \"" + Descripcion + "\";";
            ResultSet rs = UBD.ejecutarQuery(query);

            // Realizamos la asignacion de valores a los atributos
            rs.next();
            this.Cod_Zona = rs.getString(1);
            setFechaI(stringToCalendar(rs.getString(2).substring(0,19)));
            this.Descripcion = rs.getString(3);
            rs.getObject(4);
            if (rs.wasNull())       this.Nivel = null;
            else                    this.Nivel = new Nivel(rs.getInt(4),rs.getString(5).substring(0,19));
            this.Llaves = getLlavesBD();

            // Cerramos la conexion
            UBD.cerrarConexion();
	}

        
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getCodZona: Metodo que obtiene el valor del atributo Cod_Zona
	 *
	 * @param   void
	 * @return  String	String con el codigo de zona
	 */
	public String getCodZona()			{ return this.Cod_Zona; }
        /** setCodZona: Metodo que establece el valor del atributo Cod_Zona
	 *
	 * @param   Cod_Zona     String     String con el codigo de zona
	 * @return  void
	 */
	public void setCodZona(String Cod_Zona) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Cod_Zona == null) || (Cod_Zona.length()>10))   throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Cod_Zona = Cod_Zona.toUpperCase();
        }

        /** getDescripcion: Metodo que obtiene el valor del atributo Descripcion
	 *
	 * @param   void
	 * @return  String     Descripcion de la zona
	 */
	public String getDescripcion()                  { return this.Descripcion; }
        /** setDescripcion: Metodo que establece el valor del atributo Descripcion
	 *
	 * @param   Descripcion     String     String con la descripcion de la zona
	 * @return  void
	 */
	public void setDescripcion(String Descripcion) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Descripcion == null) || (Descripcion.length()>100))    throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Descripcion = Descripcion;
        }

        /** getNivel: Metodo que obtiene el valor del atributo Nivel
	 *
	 * @param   void
	 * @return  Nivel     Nivel necesario para acceder a la zona
	 */
	public Nivel getNivel()                         { return this.Nivel; }
        /** setNivel: Metodo que establece el valor del atributo Nivel
	 *
	 * @param   Nivel     Nivel     Nivel  necesario para acceder a la zona
	 * @return  void
	 */
	public void setNivel(Nivel Nivel)               { this.Nivel = Nivel; }

        /** getLlaves: Metodo que obtiene el valor del atributo Llaves
	 *
	 * @param   void
	 * @return  HashMapa	Llaves que pertenecen a la zona
	 */
	public HashMap<String,Llave> getLlaves()                      { return this.Llaves; }
        /** setLlaves: Metodo que establece el valor del atributo Zonas
	 *
	 * @param   Llaves     HashMap     Llaves que pertenecen a la zona
	 * @return  void
	 */
	public void setLlaves(HashMap<String,Llave> Llaves) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llaves == null) || (Llaves.isEmpty()))                 throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.Llaves = Llaves;
        }
        /** addLlave: Metodo que añade una llave a las ya existentes
         *
         * @param   Llave        Llave  Llave nueva a añadir
         * @return  void
         */
        public void addLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().put("L - " + Llave.getCodigo() + " - " + calendarToString(Llave.getFechaI()),Llave);
        }
        /** updateLlave: Metodo que actualiza una llave existente
         *
         * @param   Llave_old       Llave       Llave a modificar
         * @param   Llave_new       Llave       Llave por la que reemplazar
         * @return  void
         */
        public void updateLlave(Llave Llave_old, Llave Llave_new) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llave_old == null) || (Llave_new == null))   throw new DataException();
            // Reemplazamos la zona vieja por la nueva
            if (getLlaves().containsKey("L - " + Llave_old.getCodigo() + " - " + calendarToString(Llave_old.getFechaI())));
            {
                removeLlave(Llave_old);
                addLlave(Llave_new);
            }
        }
        /** removeLlave: Metodo que elimina una llave de las existentes
         *
         * @param   Llave        Llave  Llave a eliminar
         * @return  void
         */
        public void removeLlave(Llave Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                              throw new DataException();
            // Realizamos la asignacion de valores a los atributos
            this.getLlaves().remove("L - " + Llave.getCodigo() + " - " + calendarToString(Llave.getFechaI()));
        }
        /** getLlave: Metodo que obtiene una Llave de las existentes
         *
         * @param   String          Key_Llave   Key de la Llave a obtener
         * @return  Llave                       Llave solicitada
         */
        public Llave getLlave(String Key_Llave) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Key_Llave == null) || (Key_Llave.length()>29))     throw new DataException();
            // Devolvemos el resultado
            return this.getLlaves().get(Key_Llave);
        }


        // TOSTRING
        /** toString: Metodo que devuelve una String con el estado del objeto
	 *
	 * @param   void
	 * @return  s       String con informacion del estado actual del objeto
	 */
        @Override
	public String toString()
	{
            // String con las llaves pertenecientes a la Zona
            String ls = "";

            if ((getLlaves() != null) && (!getLlaves().isEmpty()))
            {
                // Añadimos las Strings con todas las llaves
                Iterator it = getLlaves().entrySet().iterator();
                while (it.hasNext())
                {
                    Map.Entry VKLlave = (Map.Entry) it.next();
                    Llave l = (Llave) VKLlave.getValue();
                    ls = ls + l.toString();
                }
            }

            // String resultado
            String s = "ZONA:\nCod_Zonaf: " + getCodZona() + "\n" +
            "F_Alta_Z: " + calendarToString(getFechaI()) + "\n" +
            "Descripcion: " + getDescripcion() + "\n" +
            "Cod_Niv: " + getNivel().getCodNiv() + "\n\n" +
            ls + "\n";

            // Devolvemos el resultado
            return s;
	}

        // DML
        /** insertLlaveBD: Metodo que inserta una Llave en la zona en la BD
         *
         * @param   Llave   Llave   Llave a insertar en la zona
         * @return  int             Numero de filas insertadas
         */
        public int insertLlaveBD(Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                                      throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Llave VALUES (" +
                Llave.getCodigo() + ",\"" +
                UCAL.calendarToString(Llave.getFechaI()) + "\"," +
                (Llave.getFechaJ() == null ? "null" : "\"" + UCAL.calendarToString(Llave.getFechaJ()) + "\"") + ",\"" +
                getCodZona() + "\",\"" +
                UCAL.calendarToString(getFechaI()) + "\",\"" +
                Llave.getDescripcion() + "\"," +
                (Llave.getNotas() == null ? "null" : "\"" + Llave.getNotas() + "\"") + ");";

                // Ejecutamos la consulta y devolvemos el resultado
                return UBD.ejecutarUpdate(query);
        }
        /** updateLlaveBD: Metodo que actualiza una llave de la zona en la BD
         *
         * @param   Llave_old       Llave a modificar
         * @param   Llave_new       Llave de reemplazo
         * @return  int             Resultado
         */
        public int updateLlaveBD(Llave Llave_old, Llave Llave_new) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Llave_old == null) || (Llave_new == null))         throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llave SET " +
                "Cod_Llave = " + Llave_new.getCodigo() + "," +
                "F_Alta_L = \"" + UCAL.calendarToString(Llave_new.getFechaI()) + "\"," +
                "F_Baja_L = " + (Llave_new.getFechaJ() == null ? "null" : "\"" + UCAL.calendarToString(Llave_new.getFechaJ())+ "\"") + "," +
                "Descripcion = \"" + Llave_new.getDescripcion() + "\"," +
                "Notas = \"" + Llave_new.getNotas() +
                "\" WHERE Cod_Llave = " + Llave_old.getCodigo() + " AND F_Alta_L = \"" +
                UCAL.calendarToString(Llave_old.getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** darBajaLlaveBD: Metodo que da de baja la Llave en la BD
         *
         * @param   void
         * @return  int         Numero de filas actualizadas
         */
        public int darBajaLlaveBD(Llave llave) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llave" +
                    " SET F_Baja_L = \"" + calendarToString(Calendar.getInstance()) + "\"" +
                    " WHERE Cod_Llave = " + llave.getCodigo() + " AND F_Alta_L = \"" + calendarToString(llave.getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** removeLlaveBD: Metodo que borra una llave de la zona en la BD
         *
         * @param   Llave    Llave     Llave a eliminar
         * @return  int                Numero de filas eliminadas
         */
        public int removeLlaveBD(Llave Llave) throws DataException, SQLException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Llave == null)                                  throw new DataException();

            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Llave  WHERE Cod_Llave = " + Llave.getCodigo() +
                        " AND F_Alta_L = \"" + UCAL.calendarToString(Llave.getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }

        /** insertLlavesBD: Metodo que inserta las Llaves de la Zona en la BD
         *
         * @param   void
         * @return  int                 Numero de filas insertadas
         */
        public int insertLlavesBD() throws DataException, SQLException
        {
            // Obtenemos las llaves
            HashMap<String,Llave> Llaves = getLlaves();

            // Insertamos cada Llave
            Iterator it = Llaves.entrySet().iterator();
            while (it.hasNext())
            {
                // Obtenemos la llave
                Map.Entry VKLlaves = (Map.Entry) it.next();
                Llave ll = (Llave) VKLlaves.getValue();

                // Insertamos la llave
                insertLlaveBD (ll);
            }

            return Llaves.size();
        }
        /** updateLlavesBD: Metodo que actualiza las Llaves de la Zona en la BD
         *
         * @param   void
         * @return  int                                     Numero de filas (Llaves) actualizadas
         */
        public int updateLlavesBD() throws DataException, SQLException, RemoteException
        {
            // Filas actualizadas
            int res = 0;

            // Recorremos el HashMap nuevo, actualizando las Llaves existentes e insertando las nuevas
            Iterator it_new = getLlaves().entrySet().iterator();
            while(it_new.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_new.next();
                Llave llave_new = (Llave) VKLlaves.getValue();
                String llave_key = (String) VKLlaves.getKey();

                // Si existe la llave -> La actualizamos
                if (getLlaves().containsKey(llave_key))         updateLlaveBD(getLlaves().get(llave_key),llave_new);
                // Si no existe -> La insertamos
                else                                            insertLlaveBD(llave_new);
                // Actualizamos el contador de filas actualizadas
                res++;
            }

            // Finalmente, recorremos el HashMap viejo eliminando aquellas Llaves que no consten en el nuevo
            HashMap<String,Llave> Llaves_actu = getLlavesBD();
            Iterator it_old = Llaves_actu.entrySet().iterator();
            while(it_old.hasNext())
            {
                Map.Entry VKLlaves = (Map.Entry) it_old.next();
                Llave llave_old = (Llave) VKLlaves.getValue();
                String llave_key = (String) VKLlaves.getKey();

                // Si la Llave vieja no existe como nueva -> La eliminamos y actualizamos el contador de filas actualizadas
                if (!getLlaves().containsKey(llave_key))
                {
                    removeLlaveBD(llave_old);
                    res++;
                }
            }

            // Devolvemos el resultado
            return res;
        }
        /** removeLlavesBD: Metodo que borra las Llaves de la Zona en la BD
         *
         * @param   void
         * @return  int             Numero de filas (Llaves) eliminadas
         */
        public int removeLlavesBD() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Llave WHERE Cod_Zona = \"" + getCodZona() + "\" AND F_Alta_Z = \"" + UCAL.calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }
        /** getLlavesBD: Metodo que obtiene todas las llaves de una zona del edificio
         *
         * @param   void
         * @return  LinkedHashMap<Integer,Llave>       Llaves               HashMap con todas las llaves
         */
        public LinkedHashMap<String,Llave> getLlavesBD () throws DataException, SQLException, RemoteException
        {
           // Creamos el HashMap resultado
           LinkedHashMap<String,Llave> Llaves = new LinkedHashMap();

           // Introducimos las llaves
           String query = "SELECT Cod_Llave,F_Alta_L FROM Llave WHERE Cod_Zona = \"" + Cod_Zona + "\";" ;
           ResultSet rs = UBD.ejecutarQuery(query);

            while (rs.next())
            {
                // Creamos la llave y la introduciomos
                Llave ll = new Llave(rs.getInt(1),rs.getString(2).substring(0,19));
                Llaves.put("L - " + ll.getCodigo() + " - " + UCAL.calendarToString(ll.getFechaI()),ll);
            }

            // Cerramos la conexion
            UBD.cerrarConexion();

            // Devolvemos el resultado
            return Llaves;
        }

        /** insertZonaBD: Metodo que inserta la Zona en la BD
         *
         * @param   void
         * @return  int             Numero de filas (Zonas) insertadas
         */
        public int insertZonaBD() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "INSERT INTO Zona VALUES (\"" +
                getCodZona() + "\",\"" +
                UCAL.calendarToString(getFechaI()) + "\",\"" +
                getDescripcion () + "\"," +
                (getNivel()== null ? "null,null" : getNivel().getCodNiv() + ",\"" +
                UCAL.calendarToString(getNivel().getFechaI()) + "\"") + ");";

            // Ejecutamos la consulta
            int res = UBD.ejecutarUpdate(query);

            // Si se ha insertado correctamente la zona -> INSERTAMOS LAS LLAVES
            if (res == 1)       { insertLlavesBD(); }

            // Devolvemos el resultado
            return res;
        }
        /** updateZonaBD: Metodo que actualiza la Zona en la BD
         *
         * @param   Zona            Zona de reemplazo
         * @return  int             Resultado
         */
        public int updateZonaBD(Zona Zona) throws DataException, SQLException, RemoteException
        {
            // Comprobamos que los parametros pasados son correctos
            if (Zona == null)                                      throw new DataException();

            // ACTUALIZAMOS LOS DATOS
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Zona SET " +
                "Cod_Zona = \"" + Zona.getCodZona() + "\"," +
                "F_Alta_Z = \"" + UCAL.calendarToString(Zona.getFechaI()) + "\"," +
                "Descripcion = \"" + Zona.getDescripcion() + "\"," +
                "Cod_Niv = " + (Zona.getNivel() == null ? "null" : Zona.getNivel().getCodNiv()) + "," +
                "F_Alta_N = " + (Zona.getNivel() == null ? "null" : ("\"" + UCAL.calendarToString(Zona.getNivel().getFechaI()) + "\"")) +
                " WHERE Cod_Zona = \"" + getCodZona()+ "\" AND F_Alta_Z = \"" +
                UCAL.calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta
            UBD.ejecutarUpdate(query);

            // Cambiamos los atributos primarios para que se realicen correctamente las posteriores actualizaciones
            setCodZona(Zona.getCodZona());
            setFechaI(Zona.getFechaI());

            // ACTUALIZAMOS LAS LLAVES
            return updateLlavesBD();

        }
        /** removeZonaBD: Metodo que borra la Zona de la BD
         *
         * @param   void
         * @return  int                 Numero de filas eliminadas
         */
        public int removeZonaBD() throws DataException, SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "DELETE FROM Zona WHERE Cod_Zona = \"" + getCodZona() +
                    "\" AND F_Alta_Z = \"" + UCAL.calendarToString(getFechaI()) + "\";";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarUpdate(query);
        }


        // FUNCIONALIDADES
        /* obtenerZonas: Metodo que devuelve las zonas existentes en el centro
         *
         * @param   void
         * @return  ResultSet   rs          Zonas existentes en el centro
         */
        public ResultSet obtenerZonas() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Descripcion Nombre, Cod_Niv Nivel" +
                    " FROM Zona" +
                    " ORDER BY Descripcion;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* obtenerLlaves: Metodo que devuelve las llaves pertenecientes a la zona
         *
         * @param   void
         * @return  ResultSet   rs          Llaves pertenecientes a la zona
         */
        public ResultSet obtenerLlaves() throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "SELECT Cod_Llave Código, L.Descripcion Descripción" +
                    " FROM Zona JOIN Llave L USING (Cod_Zona,F_Alta_Z)" +
                    " WHERE Cod_Zona = \"" + getCodZona() + "\" AND F_Alta_Z = \"" + calendarToString(getFechaI()).substring(0,19) + "\"" +
                    " AND F_Baja_L IS NULL;";

            // Ejecutamos la consulta y devolvemos el resultado
            return UBD.ejecutarQuery(query);
        }

        /* disparador: Metodo que ejecuta el disparador sobre BEFORE INSERT ON Llave
         *
         * @param   Llave   Llave a insertar
         * @return  int
         */
        public int disparador(Llave Llave) throws SQLException
        {
            // Creamos la String con la sentencia a realizar
            String query = "UPDATE Llave SET F_Baja_L = CURDATE() WHERE Cod_Llave = " + Llave.getCodigo() + " AND F_Baja_L IS NULL;";
            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }

        /* disparador: Metodo que ejecuta el disparador sobre AFTER DELETE ON Llave
         *
         * @param   void
         * @return  int
         */
        public int disparador() throws SQLException
        {
            // Eliminamos los llaveros que no dispongan de llaves
            String query = "DELETE FROM Llavero WHERE (Cod_Llavero,F_Alta_Ll) NOT IN (SELECT DISTINCT Cod_Llavero, F_Alta_Ll FROM Composicion_Llavero);";
            UBD.ejecutarUpdate(query);

            // Eliminamos todos los almacenes que no contengan ningun elemento
            query = "DELETE FROM Almacen WHERE Cod_Alm NOT IN (SELECT DISTINCT Cod_Alm FROM Almacen_Llave JOIN Llave USING (Cod_Llave,F_Alta_L) WHERE F_Baja_L IS NULL UNION SELECT DISTINCT Cod_Alm FROM Almacen_Llavero JOIN Llavero USING (Cod_Llavero,F_Alta_Ll) WHERE F_Baja_Ll IS NULL);";            UBD.ejecutarUpdate(query);

            // Devolvemos el resultado
            return 0;
        }
}