// PAQUETE
package API.Utilidades;

// IMPORTACIONES
import API.Interfaces_RMI.UtilidadesBD_I;
import java.sql.*;



/** CLASE UTILIDADESBD:  Clase que proporciona todas las utilidades necesarias para conectar con la BD
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class UtilidadesBD implements UtilidadesBD_I
{
 	// ATRIBUTOS -----------------------------------------------------------
        /* DireccionURL:                    Direccion URL de la base de datos */
        private String                      DireccionURL = "jdbc:mysql://localhost/bd";
        /* Propiedades:                     Usuario de la base dtos */
        private String                      Usuario = "root";
        /* Propiedades:                     Contraseña de la base de datos */
        private String                      Pass = "root";
        /* Con:                             Conexion con la base de datos */
        protected Connection                Con;


	// METODOS -------------------------------------------------------------
	// OBTENCION Y ESTABLECIMIENTO DE PARAMETROS
        /** getDireccionURL: Metodo que obtiene el valor del atributo DireccionURL
	 *
         * @param   void
	 * @return  String      DireccionURL
	 */
        public String getDireccionURL()                             { return this.DireccionURL; }
        /** setDireccionURL: Metodo que establece el valor del atributo DireccionURL
	 *
         * @param   String      DireccionURL            Direccion URL de la base de datos
	 * @return  void
	 */
        // setDireccionURL: Metodo que establece el valor del atributo DireccionURL
        public void setDireccionURL (String DireccionURL)           { this.DireccionURL = DireccionURL; }


	// EJECUTAR CONSULTAS
        /** abrirConexion: Metodo que abre una conexion con la BD
	 *
	 * Posibles valores erroneos: Ninguno
	 *
         * @param   void
	 * @return  void
	 */
        public Statement abrirConexion() throws SQLException
        {
            // Conectamos con la BD
            Con = DriverManager.getConnection(DireccionURL, Usuario, Pass);

            // Creamos el Statement y lo devolvemos
            return Con.createStatement();
        }
        
        /** cerrarConexion: Metodo que cierra la conexion con la BD
	 *
	 * Posibles valores erroneos: Ninguno
	 *
         * @param   void
	 * @return  void
	 */
        public void cerrarConexion() throws SQLException
        {
            Con.close();
        }

        /** ejecutarQuery: Metodo que conecta con una BD, ejecuta una consulta
         * DML y devuelve el resultado obtenido
	 *
	 * Posibles valores erroneos: Que la direccion, usuario, password o consulta sean erroneos
	 *
         * @param   Query           Consulta
	 * @return  ResultSet       ResultSet con el resultado obtenido de la consulta
	 */
        public ResultSet ejecutarQuery(String Query) throws SQLException
        {
            //Conectamos con la BD y ejecutamos la consulta
            return ejecutarQuery(DireccionURL,Usuario,Pass,Query);
        }
        /** ejecutarQuery: Metodo que conecta con una BD, ejecuta una consulta
         * DML y devuelve el resultado obtenido
	 *
	 * Posibles valores erroneos: Que la direccion, usuario, password o consulta sean erroneos
	 *
	 * @param   DireccionURL    Direccion URL en formato MySQL que referencia a la BD con la que conectar
         * @param   Usuario         Nombre de usuario
         * @param   Pass            Contraseña
         * @param   Query           Consulta
	 * @return  ResultSet       ResultSet con el resultado obtenido de la consulta
	 */
        public ResultSet ejecutarQuery(String DireccionURL, String Usuario, String Pass, String Query) throws SQLException
        {
            // Conectamos con la BD
            Statement st = abrirConexion();
            // Ejecutamos la consulta
            return st.executeQuery(Query);
        }

        /** ejecutarUpdate: Metodo que conecta con una BD, ejecuta una consulta
         * de DDL y devuelve el numero de filas actualizadas
	 *
	 * Posibles valores erroneos: Que la direccion, usuario, password o consulta sean erroneos
	 *
	 * @param   DireccionURL    Direccion URL en formato MySQL que referencia a la BD con la que conectar
         * @param   Usuario         Nombre de usuario
         * @param   Pass            Contraseña
         * @param   Query           Consulta
	 * @return  ResultSet       ResultSet con el resultado obtenido de la consulta
	 */
        public int ejecutarUpdate(String Query) throws SQLException
        {
            // Conectamos con la BD y ejecutamos la consulta
            return ejecutarUpdate(DireccionURL,Usuario,Pass,Query);
        }
        
        /** ejecutarUpdate: Metodo que conecta con una BD, ejecuta una consulta
         * de DDL y devuelve el numero de filas actualizadas
	 *
	 * Posibles valores erroneos: Que la direccion, usuario, password o consulta sean erroneos
	 *
	 * @param   DireccionURL    Direccion URL en formato MySQL que referencia a la BD con la que conectar
         * @param   Usuario         Nombre de usuario
         * @param   Pass            Contraseña
         * @param   Query           Consulta
	 * @return  ResultSet       ResultSet con el resultado obtenido de la consulta
	 */
        public int ejecutarUpdate(String DireccionURL, String Usuario, String Pass, String Query) throws SQLException
        {
            // Tipo de operacion
            String operacion = "insertada(s)";
            // Numero de filas tratadas
            int num_filas;

            // Conectamos con la BD
            Statement st = abrirConexion();

            // Ejecutamos la consulta
            num_filas = st.executeUpdate(Query);

            // Cerramos la conexion
            cerrarConexion();

            // Mostramos el resultado en funcion del tipo de consulta realizada
            if (Query.startsWith("UPDATE"))         operacion = "actualizada(s)";
            else if (Query.startsWith("DELETE"))    operacion = "eliminada(s)";
            System.out.println("AVISO: " + num_filas + " fila(s) " + operacion);
          
            // Devolvemos el resultado
            return num_filas;
        }
}