// PAQUETE
package API.Utilidades;

// IMPORTACIONES
import API.Interfaces_RMI.UtilidadesCalendar_I;
import API.Excepciones.DataException;
import java.util.Calendar;
import java.util.StringTokenizer;
import java.text.SimpleDateFormat;



/** CLASE UTILIDADESCALENDAR:  Clase que proporciona todas las utilidades necesarias para manejar objetos de tipo Calendar
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class UtilidadesCalendar implements UtilidadesCalendar_I
{      
	// METODOS -------------------------------------------------------------
	// CONVERSION ENTRE OBJETOS
        /** calendarToString: Metodo que pasa una fecha y hora a una String
	 *
	 * Posibles valores erroneos: Ninguno
	 *
	 * @param   fecha      Calendar     Objeto Calendar a pasar a String
	 * @return  void
	 */
        public String calendarToString (Calendar Fecha)
        {
             SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

             return dateFormat.format(Fecha.getTime());
        }

        /** stringToCalendar: Metodo que pasa una String a objeto Calendar
	 *
	 * Posibles valores erroneos: Ninguno
	 *
	 * @param   fecha      String     Objeto String a pasar a Calendar
	 * @return  void
	 */
        public Calendar stringToCalendar(String Fecha) throws DataException
        {
            // Comprobamos que los parametros pasados son correctos
            if ((Fecha == null) || (Fecha.length()>19))                 throw new DataException();

            // Tratamos el String en el que viene la fecha para tener todo separado por '-'
            Fecha = Fecha.replace(' ','-').replace(':', '-');
            // Creamos un StringTokenizer con el que recorrer la String
            StringTokenizer st = new StringTokenizer(Fecha);
            // Creamos un objeto Calendar
            Calendar calendar = Calendar.getInstance();
            // Establecemos el estado del objeto Calendar con los datos obtenidos del String de entrada
            calendar.set(Integer.parseInt(st.nextToken("-")),Integer.parseInt(st.nextToken())-1,
                    Integer.parseInt(st.nextToken()),Integer.parseInt(st.nextToken()),
                    Integer.parseInt(st.nextToken()),Integer.parseInt(st.nextToken().substring(0,2)));

            // Devolvemos el resultado
            return calendar;
        }

        /** stringToTime: Metodo que pasa una String a objeto Calendar con solo hora
	 *
	 * Posibles valores erroneos: Ninguno
	 *
	 * @param   Fecha      String     Objeto String a pasar a Calendar (con solo hora)
	 * @return  void
	 */
        public Calendar stringToTime(String Hora)
        {
            // Creamos un StringTokenizer con el que recorrer la String
            StringTokenizer st = new StringTokenizer(Hora);
            // Creamos un objeto Calendar
            Calendar calendar = Calendar.getInstance();
            // Establecemos el estado del objeto Calendar con los datos obtenidos del String de entrada
            calendar.set(0,0,0,Integer.parseInt(st.nextToken(":")),Integer.parseInt(st.nextToken()),
                    Integer.parseInt(st.nextToken().substring(0,2)));

            // Devolvemos el resultado
            return calendar;
        }
}