// PAQUETE
package API.Utilidades;

// IMPORTACIONES
import API.Interfaces_RMI.UtilidadesGUI_I;
import API.Excepciones.DataException;
import API.Tablas.Departamento;
import API.Tablas.Empresa;
import API.Tablas.GLL_Dev;
import API.Tablas.Llave;
import java.rmi.RemoteException;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;



/** CLASE UTILIDADESGUI:  Clase que proporciona todas las utilidades necesarias para manejar los componentes de la GUI
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class UtilidadesGUI implements UtilidadesGUI_I
{       
	// METODOS -------------------------------------------------------------
	// JTABLES
        /** rsToTableModel: Metodo que crea un DefaultTableModel a partir de un ResultSet
	 *
	 * Posibles valores erroneos: Si el ResultSet no es valido
	 *
         * @param   ResultSet           rs      ResultSet con el resultado de la consulta
	 * @return  DefaultTableModel           DefaultTableModel a usar en la JTable
	 */
        public DefaultTableModel rsToTableModel(ResultSet rs) throws SQLException
        {
            // Obtenemos los metadatos
            ResultSetMetaData metaDatos = rs.getMetaData();

            // Numero de columnas
            int numCols = metaDatos.getColumnCount();

            // Se crea el array de etiquetas a rellenar
            Object[] etiquetas = new Object[numCols];

            // Se obtiene cada una de las etiquetas para cada columna
            for (int i = 0; i < numCols; i++)   { etiquetas[i] = metaDatos.getColumnLabel(i+1); }

            return new DefaultTableModel(etiquetas,0);
        }

        /** rellenarTabla: Metodo que rellena una JTable con un ResultSet
	 *
	 * Posibles valores erroneos: Si el JTable o el ResultSet no son validos
	 *
         * @param   JTable              tabla   JTable a rellenar
         * @param   ResultSet           rs      ResultSet con el resultado de la consulta
	 * @return  void
	 */
        public void rellenarTabla(JTable tabla, ResultSet rs) throws SQLException
        {
            // Reestablecemos el modelo de la tabla
            DefaultTableModel modelo = rsToTableModel(rs);
            tabla.setModel(modelo);

            // Obtenemos el numero de columnas
            int numCols = rs.getMetaData().getColumnCount();

            // Introducimos el resultado de la consulta en la tabla
            while (rs.next())
            {
               // Se crea un array que responda a cada fila
               Object [] fila = new Object[numCols];

               // Se rellena cada posición del array con una de las columnas de la tabla
               for (int i=0;i<numCols;i++)
               {
                   // Obtenemos el objeto
                   Object o = rs.getObject(i+1);

                   // Si es null
                   if (rs.wasNull())                    fila[i] = new String("");

                   // Si es un booleano
                   if ((o instanceof Integer) && ((rs.getMetaData().getColumnName(i+1).equals("Devuelta"))))
                   {
                        int in = (Integer) o;
                        if (in == 1)                    fila[i] = "Sí";
                        else                            fila[i] = "No";
                   }
                   // Si es cualquier otro tipo de dato
                   else                                 fila[i] = rs.getObject(i+1);
               }
               // Se añade al modelo la fila completa
               modelo.addRow(fila);
            }
        }

        /** pintarTabla: Metodo que pinta las celdas de una JTable segun un ArrayList de colores
	 *
	 * Posibles valores erroneos: Si el JTable o ArrayList no son correctos
	 *
         * @param   JTable              tabla   JTable a rellenar
         * @param   ArrayList           al      ArrayList con los colores
	 * @return  void
	 */
        public void pintarTabla(JTable tabla, ArrayList al) throws DataException, SQLException, RemoteException
        {
            tabla.setDefaultRenderer(Object.class,new Render(al));
        }

        /** rellenarTablaTelefonos: Metodo que rellena el JTable de telefonos con un ArrayList
	 *
	 * Posibles valores erroneos: Si el JTable o el ArrayList no son validos
	 *
         * @param   JTable                      tabla   JTable a rellenar
         * @param   ArrayList<String>           al      ArrayList con lso telefonos a introducir en la tabla
	 * @return  void
	 */
        public void rellenarTablaTelefonos(JTable tabla, ArrayList<String> al) throws SQLException
        {
            // Reestablecemos el modelo de la tabla
            String[] Etiquetas = {"Teléfonos"};
            DefaultTableModel modelo = new DefaultTableModel(Etiquetas,0);
            tabla.setModel(modelo);

            // Introducimos el ArrayList en la tabla
            Iterator it = al.iterator();
            while (it.hasNext())
            {
                // Se crea un array que responda a cada fila
                Object [] fila = new Object[Etiquetas.length];

                // Se rellena la fila
                fila[0] = (String) it.next();

                // Se añade al modelo la fila completa
                modelo.addRow(fila);
            }
        }
        
        /** rellenarTablaPrestamos: Metodo que rellena el JTable de prestamos con un HashMap
	 *
	 * Posibles valores erroneos: Si el JTable o el HashMap no son validos
	 *
         * @param   JTable                      tabla   JTable a rellenar
         * @param   HashMap<String,GLL_Dev>     hm      HashMap con los datos a emplear para rellenar la tabla
	 * @return  void
	 */
        public void rellenarTablaPrestamos(JTable tabla, HashMap<String,GLL_Dev> hm) throws SQLException
        {
            // Reestablecemos el modelo de la tabla
            String[] Etiquetas = {"Tipo","Código", "Devuelta"};
            DefaultTableModel modelo = new DefaultTableModel(Etiquetas,0);
            tabla.setModel(modelo);

            // Introducimos el hashMap en la tabla
            Iterator it = hm.entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKGLLD = (Map.Entry) it.next();
                GLL_Dev GLLD = (GLL_Dev) VKGLLD.getValue();

                // Se crea un array que responda a cada fila
                Object [] fila = new Object[Etiquetas.length];

                // Se rellena la fila
                // Tipo
                String key = (String)VKGLLD.getKey();
                fila[0] = (key.startsWith("L -") ? "Llave" : "Llavero");
                // Codigo
                fila[1] = GLLD.getLlaveLlavero().getCodigo();
                // Devuelta
                fila[2] = (GLLD.getDev() ? "Sí" : "No");

                // Se añade al modelo la fila completa
                modelo.addRow(fila);
            }
        }

        /** rellenarTablaEmpresas: Metodo que rellena el JTable de Empresas con las empresas existentes y el HashMap de empresas a introducir
	 *
	 * Posibles valores erroneos: Si el JTable o el HashMap no son validos
	 *
         * @param   JTable                      tabla   JTable a rellenar
         * @param   ResultSet                   rs      ResultSet con las empresas existentes
         * @param   HashMap<Integer,Empresa>    hm      HashMap con los datos a emplear para rellenar la tabla
	 * @return  void
	 */
        public void rellenarTablaEmpresas(JTable tabla, ResultSet rs, HashMap<Integer,Empresa> hm) throws SQLException
        {
            // Creamos el modelo con las empresas existentes en la BD
            DefaultTableModel modelo= rsToTableModel(rs);
            rellenarTabla(tabla,rs);
            modelo = (DefaultTableModel) tabla.getModel();

            // Añadimos el hashMap en la tabla
            Iterator it = hm.entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKEmp = (Map.Entry) it.next();
                Empresa emp = (Empresa) VKEmp.getValue();

                // Se crea un array que responda a cada fila
                Object [] fila = new Object[tabla.getModel().getColumnCount()];

                // Se rellena la fila
                // Nombre de empresa
                fila[0] = emp.getNombre();

                // Se añade al modelo la fila completa
                modelo.addRow(fila);
            }
        }

        /** rellenarTablaDepartamentos: Metodo que rellena el JTable de Departamentos con el HashMap de empresas a introducir
	 *
	 * Posibles valores erroneos: Si el JTable o el HashMap no son validos
	 *
         * @param   JTable                      tabla       JTable a rellenar
         * @param   HashMap<Integer,Empresa>    hm          HashMap con los datos a emplear para rellenar la tabla
         * @param   String                      Nom_Emp     Nombre de empresa seleccionada
	 * @return  void
	 */
        public void rellenarTablaDepartamentos(JTable tabla, HashMap<Integer,Empresa> hm, int Emp_i) throws SQLException
        {
            // Añadimos cada departamento a la tabla
            Iterator it = hm.get(Emp_i).getDepartamentos().entrySet().iterator();
            
            // Si existen departamentos
            if (it.hasNext())
            {
                while (it.hasNext())
                {
                    Map.Entry VKDep = (Map.Entry) it.next();
                    Departamento dep = (Departamento) VKDep.getValue();

                    // Se crea un array que responda a cada fila
                    Object [] fila = new Object[tabla.getModel().getColumnCount()];

                    // Se rellena la fila
                    // Nombre de departamento
                    fila[0] = dep.getNombre();
                    // Actividad
                    fila[1] = dep.getAcceso().getActividad();

                    // Se añade al modelo la fila completa
                    DefaultTableModel modelo = (DefaultTableModel) tabla.getModel();
                    modelo.addRow(fila);
                    tabla.setModel(modelo);
                }
            }
            // Si no existen departamentos
            else
            {
                // Insertamos un modelo con una sola fila
                Object[] etiquetas = {"Nombre","Actividad"};
                tabla.setModel(new DefaultTableModel(etiquetas,0));
            }
        }

        /** rellenarTablaLs: Metodo que rellena el JTable de llaves con un HashMap de llaves
	 *
	 * Posibles valores erroneos: Si el JTable o el HashMap no son validos
	 *
         * @param   JTable                      tabla       JTable a rellenar
         * @param   HashMap<Integer,Empresa>    hm          HashMap con los datos a emplear para rellenar la tabla
	 * @return  void
	 */
        public void rellenarTablaLs(JTable tabla, HashMap<String,Llave> hm) throws SQLException
        {
            // Añadimos cada llave a la tabla
            Iterator it = hm.entrySet().iterator();

            // Si existen llaves
            if (it.hasNext())
            {
                while (it.hasNext())
                {
                    Map.Entry VKLlave = (Map.Entry) it.next();
                    Llave l = (Llave) VKLlave.getValue();

                    // Se crea un array que responda a cada fila
                    Object [] fila = new Object[tabla.getModel().getColumnCount()];

                    // Se rellena la fila
                    // Codigo de llave
                    fila[0] = l.getCodigo();
                    // Descripcion
                    fila[1] = l.getDescripcion();

                    // Se añade al modelo la fila completa
                    DefaultTableModel modelo = (DefaultTableModel) tabla.getModel();
                    modelo.addRow(fila);
                    tabla.setModel(modelo);
                }
            }
            // Si no existen llaves
            else
            {
                // Insertamos un modelo con una sola fila
                Object[] etiquetas = {"Código","Descripción"};
                tabla.setModel(new DefaultTableModel(etiquetas,0));
            }
        }


        // JCOMBOBOXES
        /** rellenarJCB: Metodo que rellena un JComboBox con un ResultSet
	 *
	 * Posibles valores erroneos: Si el JComboBox o el ResultSet no son validos
	 *
         * @param   JComboBox           JCB     JComboBox a rellenar
         * @param   ResultSet           rs      ResultSet con el resultado de la consulta
	 * @return  void
	 */
        public void rellenarJCB(JComboBox JCB, ResultSet rs) throws SQLException
        {
            // Reestablecemos el modelo del JComboBox
            DefaultComboBoxModel modelo = new DefaultComboBoxModel();
            JCB.setModel(modelo);

            // Introducimos el resultado de la consulta en el JComboBox
            while (rs.next())   modelo.addElement(rs.getString(1));
        }
}