// PAQUETE
package GUI.UIs;

// IMPORTACIONES
import API.Tablas.Ocasional;
import API.Tablas.Llave;
import API.Utilidades.UtilidadesBD;
import API.Utilidades.UtilidadesGUI;
import java.awt.Color;
import javax.swing.JOptionPane;



/** CLASE JDLSOCAS:  Clase que representa el JDialog con el que el usuario puede asignar a una persona ocasional un conjunto de llaves como prestables
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class JDLsOcas extends javax.swing.JDialog
{
	// ATRIBUTOS -----------------------------------------------------------
        /** DNI:                            DNI de la persona */
        static String                       DNI;
        /** Empresa:                        Empresa para la que trabaja */
        static String                       Empresa;


        /* UGUI:                            Utilidades para la interfaz */
        private UtilidadesGUI               UGUI = new UtilidadesGUI();
        /* UBD:                             Utilidades para realizar consultas a la BD */
        private UtilidadesBD                UBD = new UtilidadesBD();



	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
	/** Constructor: Constructor de clase que crea el objeto JDLsOcas con los valores pasados parametricamente
         *
         * @param   Frame       parente     Frame padre del JDialog
         * @param   modal       modal       Denota si el JDialog debe ser modal o no
         * @param   String      DNI         DNI de la persona
         * @param   String      Empresa     Empresa para la que trabaja
         * @return  JDialog     JDLsOcas     Objeto de tipo JDLsOcas
         */
        public JDLsOcas(java.awt.Frame parent, boolean modal, String DNI, String Empresa)
        {
            // Llamamos al constructor base
            super(parent, modal);
            // Asignamos los valores a las variables
            this.DNI = DNI;
            this.Empresa = Empresa;
            // Dibujamos todos los componentes
            initComponents();
            // Iniciamos todos los datos necesarios del JDialog
            iniciarDatos();
        }


	// DIBUJO
        /** initComponents: Metodo que dibuja toda la GUI de JDLsOcas
	 *
	 * @param   void
	 * @return  void
	 */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        JTLlaves = new javax.swing.JTable();
        JBAsignar = new javax.swing.JButton();
        JTFBuscar = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Llaves");

        JTLlaves.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Zona", "Descripción"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane1.setViewportView(JTLlaves);
        JTLlaves.getAccessibleContext().setAccessibleName("JTLlaves");

        JBAsignar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBAsignar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAsignarActionPerformed(evt);
            }
        });

        JTFBuscar.setForeground(new java.awt.Color(204, 204, 204));
        JTFBuscar.setText("Buscar ...");
        JTFBuscar.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFBuscarMouseClicked(evt);
            }
        });
        JTFBuscar.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFBuscarKeyReleased(evt);
            }
            public void keyTyped(java.awt.event.KeyEvent evt) {
                JTFBuscarKeyTyped(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 452, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(JTFBuscar, javax.swing.GroupLayout.PREFERRED_SIZE, 180, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 223, Short.MAX_VALUE)
                        .addComponent(JBAsignar)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTFBuscar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JBAsignar))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 352, Short.MAX_VALUE)
                .addContainerGap())
        );

        JBAsignar.getAccessibleContext().setAccessibleName("JBAsignar");

        pack();
    }// </editor-fold>//GEN-END:initComponents


 	// INICIALIZACION
        /** iniciarDatos: Metodo que inicializa todos los datos de los componentes de la interfaz grafica
         *
         * @param   void
         * @return  void
         */
        private void iniciarDatos()
        {
            try
            {
                // Creamos un objeto de tipo Ocasional
                Ocasional ocas = new Ocasional(DNI,Empresa);

                // Refrescamos la tabla de llaves
                UGUI.rellenarTabla(JTLlaves,ocas.obtenerLsPorPrestarOcas(JTFBuscar.getText().equals("Buscar ...") ? "" : JTFBuscar.getText()));
                ocas.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las llaves.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }


    	// EVENTOS
    private void JBAsignarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAsignarActionPerformed
        try
        {
            // Avisamos de que vamos a añadir las llaves seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a añadir las llaves seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Añadimos las llaves
            if (seleccion == 0)
            {
                // Obtenemos las llaves seleccionadas
                int[] lsSelecs = JTLlaves.getSelectedRows();

                // Obtenemos la persona ocasional
                Ocasional ocas = new Ocasional (DNI,Empresa);

                Llave l;
                for (int i=0; i<lsSelecs.length; i++)
                {
                    // Obtenemos la llave
                    l = new Llave((Integer) JTLlaves.getModel().getValueAt(lsSelecs[i],0));

                    // Añadimos la llave a las prestables
                    ocas.insertLlaveBD(l);
                }

                // Ocultamos el panel
                this.setVisible(false);

                // Advertimos de que las llaves han sido insertadas correctamente
                JOptionPane.showMessageDialog(this,"Las llaves han sido insertadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible añadir las llaves seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
                e.printStackTrace();
        }
    }//GEN-LAST:event_JBAsignarActionPerformed

    private void JTFBuscarMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFBuscarMouseClicked
        // Eliminamos el texto y ponemos la letra a color negro
        JTFBuscar.setText("");
        JTFBuscar.setForeground(Color.BLACK);
}//GEN-LAST:event_JTFBuscarMouseClicked

    private void JTFBuscarKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarKeyReleased
        // Refrescamos el conjunto de llaves en funcion del filtro escrito hasta el momento
        iniciarDatos();
}//GEN-LAST:event_JTFBuscarKeyReleased

    private void JTFBuscarKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarKeyTyped
        // TODO add your handling code here:
    }//GEN-LAST:event_JTFBuscarKeyTyped



	// MAIN
        /** main: Metodo principal ejecutable de la clase, encargado de desplegar el JDialog
	 *
	 * @param   String args[]
	 * @return  void
	 */
        public static void main(String args[])
        {
            java.awt.EventQueue.invokeLater(new Runnable()
            {
                public void run()
                {
                    JDLsOcas dialog = new JDLsOcas(new javax.swing.JFrame(), true, DNI, Empresa);
                    dialog.addWindowListener(new java.awt.event.WindowAdapter() { public void windowClosing(java.awt.event.WindowEvent e) { System.exit(0); }});
                    dialog.setVisible(true);
                }
            });
        }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton JBAsignar;
    private javax.swing.JTextField JTFBuscar;
    private javax.swing.JTable JTLlaves;
    private javax.swing.JScrollPane jScrollPane1;
    // End of variables declaration//GEN-END:variables

}
