-- *********************************************************************************************************************
-- ******** CREACION																						************
-- *********************************************************************************************************************

	-- Eliminamos la base de datos si existe
	DROP DATABASE IF EXISTS bd;
	
	-- Creamos la base de datos 'bd'
	CREATE DATABASE bd;
	
	-- Alteramos su codificacion a UTF8 en español para que acepte eñes y tildes
	ALTER DATABASE bd DEFAULT CHARACTER SET utf8 COLLATE utf8_spanish_ci;
	
	-- Convertimos a 'bd' en la base de datos actual
	USE bd;
	
-- *********************************************************************************************************************
-- *********************************************************************************************************************




-- *********************************************************************************************************************
-- ******** TABLAS																							************
-- *********************************************************************************************************************

	-- TABLA 'T_Usuario'
	CREATE TABLE T_Usuario
	(
		Tipo     		varchar(20) 					NOT NULL			comment 'Tipo de usuario', 
		Descripcion 	varchar(50) 					NOT NULL 			comment 'Descripción del tipo de usuario', 
			PRIMARY KEY (Tipo)
	) type=InnoDB;
	
  	-- TABLA 'Nivel'
	CREATE TABLE Nivel
	(
		Cod_Niv     	int(1) 							NOT NULL			comment 'Nivel de acceso', 
		F_Alta_N    	datetime 						NOT NULL 			comment 'Fecha y horade inserción del nivel', 
		F_Baja_N    	datetime 		DEFAULT null 						comment 'Fecha y hora de baja del nivel', 
		Descripcion 	varchar(50) 					NOT NULL 			comment 'Descripción del nivel de acceso', 
			PRIMARY KEY (Cod_Niv, F_Alta_N)
	) type=InnoDB;
	
	-- TABLA 'Color'
	CREATE TABLE Color
	(
		Cod_Color 		int(8) 							NOT NULL 			comment 'Color de la tarjeta', 
		F_Alta_C  		datetime 						NOT NULL 			comment 'Fecha y horade inserción del color', 
		F_Baja_C  		datetime 		DEFAULT null 						comment 'Fecha y hora de baja del color', 
		Cod_Niv   		int(1) 												comment 'Nivel de acceso', 
		F_Alta_N  		datetime 											comment 'Fecha y horade inserción del nivel', 
			PRIMARY KEY (Cod_Color, F_Alta_C)
	) type=InnoDB;
	
  	-- TABLA 'T_Acceso'	
	CREATE TABLE T_Acceso
	(
		Cod_Acc   		varchar(4) 						NOT NULL 			comment 'Código de acceso', 
		Actividad 		varchar(30) 					NOT NULL 			comment 'Actividad para la cual se concede el acceso', 
		Dias      		varchar(7) 						NOT NULL 			comment 'Días permitidos de acceso', 
		Hora_Inic 		time 							NOT NULL 			comment 'Hora de inicio a partir de la cual está permitido el acceso', 
		Hora_Fin  		time 							NOT NULL 			comment 'Hora hasta la que está permitido el acceso al centro', 
		Aut_Tel   		boolean 						NOT NULL 			comment 'Denota si es necesario el aviso telefónico a la persona responsable del departamento cada vez que un empleado de ésta accede', 
		Cod_Niv   		int(1) 												comment 'Nivel de acceso al centro', 
		F_Alta_N  datetime 													comment 'Fecha y hora de inserción del nivel', 
			PRIMARY KEY (Cod_Acc)
	) type=InnoDB;

	-- TABLA 'Empresa'	
	CREATE TABLE Empresa
	(
		Cod_Emp   		varchar(4) 						NOT NULL 			comment 'Código de la empresa', 
		CIF       		varchar(9) 		UNIQUE			NOT NULL 			comment 'Código de identificación fiscal de la empresa', 
		Nombre    		varchar(50) 	UNIQUE			NOT NULL 			comment 'Nombre de la empresa', 
		Direccion 		varchar(100)					NOT NULL 			comment 'Dirección de las oficinas de la empresa', 
			PRIMARY KEY (Cod_Emp)
	) type=InnoDB;
	
	-- TABLA 'Personal'
	CREATE TABLE Personal
	(
		DNI       		varchar(9) 						NOT NULL 			comment 'DNI', 
		Apellido1 		varchar(25) 					NOT NULL 			comment 'Primer apellido', 
		Apellido2 		varchar(25) 					NOT NULL 			comment 'Segundo apellido', 
		Nombre    		varchar(20) 					NOT NULL 			comment 'Nombre', 
		DNI_Ins  		varchar(9) 											comment 'DNI del usuario que ha realizado el alta de esta persona', 
			PRIMARY KEY (DNI)
	) type=InnoDB;
  	
	-- TABLA 'Residente'
	CREATE TABLE Residente
	(
		DNI      		varchar(9) 						NOT NULL 			comment 'DNI', 
		Cod_Niv  		int(1) 												comment 'Nivel de acceso al centro que posee', 
		F_Alta_N 		datetime 											comment 'Fecha y hora de inserción del nivel.', 
			PRIMARY KEY (DNI)
	) type=InnoDB;
	
	-- TABLA 'Ocasional'
	CREATE TABLE Ocasional
	(
		DNI         	varchar(9) 						NOT NULL 			comment 'DNI',
		Empresa     	varchar(50) 					NOT NULL 			comment 'Empresa para la que trabaja el trabajador ocasional', 
		Cod_Acc     	varchar(4) 						NOT NULL			comment 'Código de acceso proporcionado', 
		F_Caducidad		datetime						NOT NULL			comment 'Fecha de caducidad del permiso para acceder al centro',
		Notas       	varchar(250) 	DEFAULT null 						comment 'Otros datos', 
			PRIMARY KEY (DNI, Empresa)
	) type=InnoDB;
	
	-- TABLA 'Departamento'	
	CREATE TABLE Departamento
	(
		Cod_Emp      	varchar(4) 						NOT NULL 			comment 'Código de empresa', 
		Cod_Dep      	varchar(4) 						NOT NULL 			comment 'Código de departamento', 
		Nombre       	varchar(50) 					NOT NULL 			comment 'Nombre del departamento', 
		Cod_Acc      	varchar(4)								 			comment 'Código de acceso que posee', 
		Periodicidad 	int(3) 							NOT NULL 			comment 'Periodicidad con la que renueva la lista de personal que trabaja en el centro', 
		F_Ult_Actu   	datetime 						NOT NULL 			comment 'Fecha y hora de la última actualización de personal realizada', 
		DNI          	varchar(9) 											comment 'DNI del autorizador responsable del departamento', 
			PRIMARY KEY (Cod_Emp, Cod_Dep)
	) type=InnoDB;

  	-- TABLA 'Cargo'
	CREATE TABLE Cargo
	(
		Cod_Cargo   	varchar(5) 						NOT NULL 			comment 'Código de cargo', 
		Descripcion 	varchar(50) 					NOT NULL 			comment 'Descripción del cargo', 
			PRIMARY KEY (Cod_Cargo)
	) type=InnoDB;

	-- TABLA 'Periodico'
	CREATE TABLE Periodico
	(
		DNI     		varchar(9) 						NOT NULL 			comment 'DNI', 
		Cod_Emp 		varchar(4) 						NOT NULL 			comment 'Código de empresa para la que trabaja', 
		Cod_Dep 		varchar(4) 						NOT NULL 			comment 'Código de departamento para el que trabaja', 
			PRIMARY KEY (DNI)
	) type=InnoDB;
	
	-- TABLA 'Contacto'
	CREATE TABLE Contacto
	(
		DNI     		varchar(9) 						NOT NULL 			comment 'DNI',
		Cod_Emp 		varchar(4) 						NOT NULL 			comment 'Código de empresa para la que trabaja', 
		Cod_Dep 		varchar(4) 						NOT NULL 			comment 'Código de departamento para el que trabaja', 
		Cod_Cargo   	varchar(5) 						NOT NULL 			comment 'Cargo desempeñado', 		  
			PRIMARY KEY (DNI, Cod_Emp, Cod_Dep, Cod_Cargo)
	) type=InnoDB;
	  
  	-- TABLA 'Vigilante'
	CREATE TABLE Vigilante
	(
		DNI     		varchar(9) 						NOT NULL 			comment 'DNI',
		Cod_Vig 		int(6) 		UNIQUE				NOT NULL 			comment 'Código de vigilante', 
			PRIMARY KEY (DNI)
	) type=InnoDB;
  
    -- TABLA 'Parte_Seguridad'
	CREATE TABLE Parte_Seguridad
	(
		Fecha       	datetime 						NOT NULL 			comment 'Fecha y hora del servicio', 
		Cod_Vig 		int(6) 							NOT NULL 			comment 'Código de vigilante',
		Incidencias 	varchar(1000) 					NOT NULL 			comment 'Incidencias acontecidas durante el servicio', 
			PRIMARY KEY (Fecha)
	) type=InnoDB;
	
  	-- TABLA 'Cuenta'	
	CREATE TABLE Cuenta
	(
		Usuario 		varchar(20) 						NOT NULL 		comment 'Nombre de usuario', 
		Pass    		varchar(20) 	DEFAULT 'cambiar' 	NOT NULL 		comment 'Contraseña', 
		Tipo			varchar(15)							NOT NULL		comment 'Tipo de usuario',
		DNI				varchar(9)							NOT NULL		comment 'DNI de la persona a la que pertenece la cuenta',
			PRIMARY KEY (Usuario)
	) type=InnoDB;
   
  	-- TABLA 'Tarjeta'
	CREATE TABLE Tarjeta
	(
		Cod_Tarj  		int(3) 							NOT NULL					comment 'Número de tarjeta', 
		F_Alta_T  		datetime 						NOT NULL 					comment 'Fecha y hora de inserción de la tarjeta', 
		F_Baja_T  		datetime 		DEFAULT null 								comment 'Fecha y hora de baja de la tarjeta', 
		Cod_Color 		int(8) 							NOT NULL 					comment 'Color de la tarjeta', 
		F_Alta_C  		datetime 						NOT NULL 					comment 'Fecha y hora de inserción del color', 
		DNI       		varchar(9)													comment 'DNI de la persona que la tiene en su posesión', 
			PRIMARY KEY (Cod_Tarj, F_Alta_T)
		) type=InnoDB;
 
    -- TABLA 'Telefono'
	CREATE TABLE Telefono
	(
		DNI  			varchar(9) 						NOT NULL 			comment 'DNI', 
		Tfno 			varchar(12)						NOT NULL 			comment 'Número de teléfono de contacto', 
			PRIMARY KEY (DNI, Tfno)
	) type=InnoDB;
  
    -- TABLA 'Email'	
	CREATE TABLE Email
	(
		DNI   			varchar(9) 						NOT NULL 			comment 'DNI', 
		Email 			varchar(50) 					NOT NULL 			comment 'Dirección de correo electrónico de contacto', 
			PRIMARY KEY (DNI, Email)
	) type=InnoDB;
  
    -- TABLA 'Almacen'	
	CREATE TABLE Almacen
	(
		Cod_Alm     	varchar(10) 					NOT NULL 			comment 'Código de almacén', 
		Descripcion 	varchar(30)		UNIQUE			NOT NULL 			comment 'Descripción del almacén', 
			PRIMARY KEY (Cod_Alm)
	) type=InnoDB;
  
    -- TABLA 'Zona'	
	CREATE TABLE Zona
	(
		Cod_Zona    	varchar(10) 					NOT NULL 			comment 'Código de zona', 
		F_Alta_Z   	 	datetime						NOT NULL 			comment 'Fecha y hora de inserción de la zona', 
		Descripcion 	varchar(100) 	UNIQUE			NOT NULL 			comment 'Descripción de la zona', 
		Cod_Niv     	int(1) 												comment 'Nivel necesario para acceder a ella', 
		F_Alta_N    	datetime 											comment 'Fecha y hora de inserción del nivel', 
			PRIMARY KEY (Cod_Zona, F_Alta_Z)
	) type=InnoDB;
 
  	-- TABLA 'Llave'
	CREATE TABLE Llave
	(
		Cod_Llave   	int(3) 							NOT NULL		 	comment 'Número de llave', 
		F_Alta_L    	datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la llave', 
		F_Baja_L    	datetime 		DEFAULT null 						comment 'Fecha y hora de baja de la llave',
		Cod_Zona    	varchar(10) 					NOT NULL 			comment 'Zona a la que pertenece', 
		F_Alta_Z    	datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la zona', 
		Descripcion 	varchar(100) 					NOT NULL 			comment 'Descripción de la cerradura que abre', 
		Notas      	 	varchar(150) 	DEFAULT null 						comment 'Otros datos a tener en cuenta', 
			PRIMARY KEY (Cod_Llave, F_Alta_L)
	) type=InnoDB;
  
  	-- TABLA 'Prestamo_Departamento'
	CREATE TABLE Prestamo_Departamento
	(
		Cod_Llave   	int(3) 							NOT NULL		 	comment 'Número de llave', 
		F_Alta_L    	datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la llave',
		Cod_Emp      	varchar(4) 						NOT NULL 			comment 'Código de empresa', 
		Cod_Dep      	varchar(4) 						NOT NULL 			comment 'Código de departamento', 
			PRIMARY KEY (Cod_Llave, F_Alta_L, Cod_Emp, Cod_Dep)
	) type=InnoDB;
  
    -- TABLA 'Prestamo_Ocasional'
	CREATE TABLE Prestamo_Ocasional
	(
		Cod_Llave   	int(3) 							NOT NULL		 	comment 'Número de llave', 
		F_Alta_L    	datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la llave',
		DNI         	varchar(9) 						NOT NULL 			comment 'DNI',
		Empresa     	varchar(50) 					NOT NULL 			comment 'Empresa para la que trabaja el trabajador ocasional', 
			PRIMARY KEY (Cod_Llave, F_Alta_L, DNI, Empresa)
	) type=InnoDB;
  
    -- TABLA 'Almacen_Llave'	
	CREATE TABLE Almacen_Llave
	(
		Cod_Alm    		varchar(4) 						NOT NULL 			comment 'Lugar de almacenamiento', 
		Cod_Llave  		int(3) 							NOT NULL 			comment 'Número de llave', 
		F_Alta_L   		datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la llave', 
		Num_Copias 		int(2) 							NOT NULL 			comment 'Número de copias existentes', 
			PRIMARY KEY (Cod_Alm, Cod_Llave, F_Alta_L)
	) type=InnoDB;
  
  	-- TABLA 'Llavero'
	CREATE TABLE Llavero
	(
		Cod_Llavero 	int(3) 							NOT NULL 			comment 'Número de llavero', 
		F_Alta_Ll   	datetime 						NOT NULL 			comment 'Fecha y hora de inserción del llavero', 
		F_Baja_Ll   	datetime 		DEFAULT null 						comment 'Fecha y hora de baja del llavero', 
		Descripcion 	varchar(100) 					NOT NULL 			comment 'Descripción del llavero', 
			PRIMARY KEY (Cod_Llavero, F_Alta_Ll)
	) type=InnoDB;
  
   	-- TABLA 'Almacen_Llavero'
	CREATE TABLE Almacen_Llavero
	(
		Cod_Alm     	varchar(4) 						NOT NULL 			comment 'Lugar de almacenamiento', 
		Cod_Llavero 	int(3) 							NOT NULL 			comment 'Número de llavero', 
		F_Alta_Ll   	datetime 						NOT NULL 			comment 'Fecha y hora de inserción del llavero', 
		Num_Copias  	int(2) 							NOT NULL 			comment 'Número de copias existentes', 
			PRIMARY KEY (Cod_Alm, Cod_Llavero, F_Alta_Ll)
	) type=InnoDB;
  
    -- TABLA 'Composicion_Llavero'
	CREATE TABLE Composicion_Llavero
	(
		Cod_Llavero 	int(3) 							NOT NULL 			comment 'Número de llavero', 
		F_Alta_Ll   	datetime 						NOT NULL 			comment 'Fecha y hora de inserción del llavero', 
		Cod_Llave   	int(3) 							NOT NULL 			comment 'Número de llave', 
		F_Alta_L    	datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la llave', 
			PRIMARY KEY (Cod_Llavero, F_Alta_Ll, Cod_Llave, F_Alta_L)
	) type=InnoDB;
  
    -- TABLA 'Acceso'
	CREATE TABLE Acceso
	(
		DNI         	varchar(9) 						NOT NULL 			comment 'DNI de la persona que accede', 
		Entrada     	datetime 						NOT NULL 			comment 'Fecha y hora de entrada', 
		Salida      	datetime 		DEFAULT null 						comment 'Fecha y hora de salida', 
		Motivo      	varchar(100) 					NOT NULL 			comment 'Motivo de acceso', 
		Cod_Tarj    	int(3) 							NOT NULL			comment 'Número de tarjeta empleada', 
		F_Alta_T    	datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la tarjeta', 
		Otros_Datos 	varchar(150) 	DEFAULT null 						comment 'Otros datos de interés', 
		DNI_Ins     	varchar(9) 						NOT NULL 			comment 'DNI del usuario que realiza la inserción del acceso (por lo general el vigilante de seguridad)', 
			PRIMARY KEY (Entrada)
	) type=InnoDB;
  
    -- TABLA 'Llave_Prestada'
	CREATE TABLE Llave_Prestada
	(
		Entrada   		datetime 						NOT NULL 			comment 'Fecha y hora de acceso', 
		Cod_Llave 		int(3) 							NOT NULL 			comment 'Número de llave prestada', 
		F_Alta_L  		datetime 						NOT NULL 			comment 'Fecha y hora de inserción de la llave.', 
		Devuelta  		boolean 		DEFAULT False	NOT NULL 			comment 'Denota si la llave ha sido devuelta', 
			PRIMARY KEY (Entrada, Cod_Llave, F_Alta_L)) type=InnoDB;
			
    -- TABLA 'Llavero_Prestado'
	CREATE TABLE Llavero_Prestado
	(
		Entrada     	datetime 						NOT NULL 			comment 'Fecha y hora de acceso', 
		Cod_Llavero 	int(3) 							NOT NULL 			comment 'Número de llavero prestado', 
		F_Alta_Ll   	datetime 						NOT NULL 			comment 'Fecha y hora de inserción del llavero', 
		Devuelto    	boolean 		DEFAULT False 						comment 'Denota si el llavero ha sido devuelto', 

			PRIMARY KEY (Entrada, Cod_Llavero, F_Alta_Ll)) type=InnoDB;
  
  	-- TABLA 'Nota'	
	CREATE TABLE Nota
	(
		Fecha 			datetime 						NOT NULL 			comment 'Fecha y hora de redacción', 
		DNI_A 			varchar(9) 						NOT NULL 			comment 'DNI del autor', 
		Tema  			varchar(50) 					NOT NULL 			comment 'Tema/título de la nota.', 
		Texto 			varchar(500) 					NOT NULL 			comment 'Texto/contenido de la nota', 
		Descartada	 	boolean 		DEFAULT False 	NOT NULL 			comment 'Denota si ha sido marcada como descartada', 
		PRIMARY KEY (Fecha, DNI_A)
	) type=InnoDB;
  
  	-- TABLA 'Destinatario'	
	CREATE TABLE Destinatario
	(
		Fecha      		datetime 						NOT NULL 			comment 'Fecha y hora de redacción', 
		DNI_A      		varchar(9) 						NOT NULL 			comment 'DNI del autor', 
		DNI_D      		varchar(9) 						NOT NULL 			comment 'DNI del destinatario', 
		Leida     	 	boolean 		DEFAULT False 	NOT NULL			comment 'Denota si ha sido marcada como leída', 
		Descartada	 	boolean 		DEFAULT False 	NOT NULL 			comment 'Denota si ha sido marcada como descartada', 
			PRIMARY KEY (Fecha, DNI_A, DNI_D)
	) type=InnoDB;
   
  	-- TABLA 'Acceso_Sistema'	
	CREATE TABLE Acceso_Sistema
	(
		Usuario 		varchar(20) 					NOT NULL 			comment 'Nombre de usuario', 
		Fecha     		datetime 						NOT NULL 			comment 'Fecha y hora de entrada en el sistema', 
			PRIMARY KEY (Usuario, Fecha)
	) type=InnoDB;
	
	-- TABLA 'Codigo'
	CREATE TABLE Codigo
	(
		Codigo			int(3)												comment 'Codigo',
			PRIMARY KEY (Codigo)
	) type=InnoDB;
	
	
  
	ALTER TABLE Color 							ADD INDEX FK_Color_Nivel (Cod_Niv, F_Alta_N), 					ADD CONSTRAINT FK_Color_Nivel FOREIGN KEY (Cod_Niv, F_Alta_N) REFERENCES Nivel (Cod_Niv, F_Alta_N) ON UPDATE Cascade ON DELETE Set null;
	ALTER TABLE T_Acceso 						ADD INDEX FK_TAcceso_Nivel (Cod_Niv, F_Alta_N), 				ADD CONSTRAINT FK_TAcceso_Nivel FOREIGN KEY (Cod_Niv, F_Alta_N) REFERENCES Nivel (Cod_Niv, F_Alta_N) ON UPDATE Cascade ON DELETE Set null;
	ALTER TABLE Residente 						ADD INDEX FK_Resid_Personal (DNI), 								ADD CONSTRAINT FK_Resid_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Residente 						ADD INDEX FK_Resid_Nivel (Cod_Niv, F_Alta_N), 					ADD CONSTRAINT FK_Resid_Nivel FOREIGN KEY (Cod_Niv, F_Alta_N) REFERENCES Nivel (Cod_Niv, F_Alta_N) ON UPDATE Cascade ON DELETE Set null;
	ALTER TABLE Ocasional 						ADD INDEX FK_Ocasional_Personal (DNI), 							ADD CONSTRAINT FK_Ocasional_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Ocasional 						ADD INDEX FK_Ocasional_TAcceso (Cod_Acc), 						ADD CONSTRAINT FK_Ocasional_TAcceso FOREIGN KEY (Cod_Acc) REFERENCES T_Acceso (Cod_Acc) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Departamento 					ADD INDEX FK_Depart_Empresa (Cod_Emp), 							ADD CONSTRAINT FK_Depart_Empresa FOREIGN KEY (Cod_Emp) REFERENCES Empresa (Cod_Emp) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Departamento 					ADD INDEX FK_Depart_Residente (DNI), 							ADD CONSTRAINT FK_Depart_Residente FOREIGN KEY (DNI) REFERENCES Residente (DNI) ON UPDATE Cascade ON DELETE Set null;
	ALTER TABLE Departamento 					ADD INDEX FK_Depart_TAcceso (Cod_Acc), 							ADD CONSTRAINT FK_Depart_TAcceso FOREIGN KEY (Cod_Acc) REFERENCES T_Acceso (Cod_Acc) ON UPDATE Cascade ON DELETE Set null;
	ALTER TABLE Periodico						ADD INDEX FK_Periodico_Personal (DNI),							ADD CONSTRAINT FK_Periodico_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade ;
	ALTER TABLE Periodico						ADD INDEX FK_PerCont_Depart (Cod_Emp, Cod_Dep),					ADD CONSTRAINT FK_PerCont_Depart FOREIGN KEY (Cod_Emp, Cod_Dep) REFERENCES Departamento (Cod_Emp, Cod_Dep) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Contacto						ADD INDEX FK_Contacto_Personal (DNI),							ADD CONSTRAINT FK_Contacto_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Contacto						ADD INDEX FK_Contacto_Depart (Cod_Emp, Cod_Dep),				ADD CONSTRAINT FK_Contacto_Depart FOREIGN KEY (Cod_Emp, Cod_Dep) REFERENCES Departamento (Cod_Emp, Cod_Dep) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Contacto						ADD INDEX FK_Contacto_Cargo (Cod_Cargo),						ADD CONSTRAINT FK_Contacto_Cargo FOREIGN KEY (Cod_Cargo) REFERENCES Cargo (Cod_Cargo) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Vigilante 						ADD INDEX FK_Vigilante_Periodico (DNI), 						ADD CONSTRAINT FK_Vigilante_Periodico FOREIGN KEY (DNI) REFERENCES Periodico (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Cuenta 							ADD INDEX FK_Cuenta_TUsuario (Tipo),							ADD CONSTRAINT FK_Cuenta_TUsuario FOREIGN KEY (Tipo) REFERENCES T_Usuario (Tipo) ON UPDATE Cascade ON DELETE Cascade;  
	ALTER TABLE Cuenta 							ADD INDEX FK_Cuenta_Personal (DNI),								ADD CONSTRAINT FK_Cuenta_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;  
	ALTER TABLE Telefono 						ADD INDEX FK_Telefono_Personal (DNI), 							ADD CONSTRAINT FK_Telefono_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Tarjeta 						ADD INDEX FK_Tarjeta_Color (Cod_Color, F_Alta_C), 				ADD CONSTRAINT FK_Tarjeta_Color FOREIGN KEY (Cod_Color, F_Alta_C) REFERENCES Color (Cod_Color, F_Alta_C) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Tarjeta 						ADD INDEX FK_Tarjeta_Personal (DNI), 							ADD CONSTRAINT FK_Tarjeta_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Email 							ADD INDEX FK_Email_Personal (DNI), 								ADD CONSTRAINT FK_Email_Personal FOREIGN KEY (DNI) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Zona 							ADD INDEX FK_Zona_Nivel (Cod_Niv, F_Alta_N),				 	ADD CONSTRAINT FK_Zona_Nivel FOREIGN KEY (Cod_Niv, F_Alta_N) REFERENCES Nivel (Cod_Niv, F_Alta_N) ON UPDATE Cascade ON DELETE Set null;
	ALTER TABLE Llave 							ADD INDEX FK_Llave_Zona (Cod_Zona, F_Alta_Z), 					ADD CONSTRAINT FK_Llave_Zona FOREIGN KEY (Cod_Zona, F_Alta_Z) REFERENCES Zona (Cod_Zona, F_Alta_Z) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Prestamo_Departamento			ADD INDEX FK_PrestDepart_Llave (Cod_Llave, F_Alta_L),			ADD CONSTRAINT FK_PrestDepart_Llave FOREIGN KEY (Cod_Llave, F_Alta_L) REFERENCES Llave (Cod_Llave, F_Alta_L) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Prestamo_Departamento			ADD INDEX FK_PrestDepart_Depart (Cod_Emp, Cod_Dep),				ADD CONSTRAINT FK_PrestDepart_Depart FOREIGN KEY (Cod_Emp, Cod_Dep) REFERENCES Departamento (Cod_Emp, Cod_Dep) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Prestamo_Ocasional				ADD INDEX FK_PrestOcas_Llave (Cod_Llave, F_Alta_L),				ADD CONSTRAINT FK_PrestOcas_Llave FOREIGN KEY (Cod_Llave, F_Alta_L) REFERENCES Llave (Cod_Llave, F_Alta_L) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Prestamo_Ocasional				ADD INDEX FK_PrestOcas_Ocasional (DNI, Empresa),				ADD CONSTRAINT FK_PrestOcas_Ocasional FOREIGN KEY (DNI, Empresa) REFERENCES Ocasional (DNI, Empresa) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Almacen_Llave 					ADD INDEX FK_AlmLlave_Almacen (Cod_Alm), 						ADD CONSTRAINT FK_AlmLlave_Almacen FOREIGN KEY (Cod_Alm) REFERENCES Almacen (Cod_Alm) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Almacen_Llave 					ADD INDEX FK_AlmLlave_Llave (Cod_Llave, F_Alta_L), 				ADD CONSTRAINT FK_AlmLlave_Llave FOREIGN KEY (Cod_Llave, F_Alta_L) REFERENCES Llave (Cod_Llave, F_Alta_L) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Almacen_Llavero 				ADD INDEX FK_AlmLl_Almacen (Cod_Alm), 							ADD CONSTRAINT FK_AlmLl_Almacen FOREIGN KEY (Cod_Alm) REFERENCES Almacen (Cod_Alm) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Almacen_Llavero 				ADD INDEX FK_AlmLl_Llavero (Cod_Llavero, F_Alta_Ll), 			ADD CONSTRAINT FK_AlmLl_Llavero FOREIGN KEY (Cod_Llavero, F_Alta_Ll) REFERENCES Llavero (Cod_Llavero, F_Alta_Ll) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Composicion_Llavero	 			ADD INDEX FK_ComposLl_Llave (Cod_Llave, F_Alta_L), 				ADD CONSTRAINT FK_ComposLl_Llave FOREIGN KEY (Cod_Llave, F_Alta_L) REFERENCES Llave (Cod_Llave, F_Alta_L) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Composicion_Llavero 			ADD INDEX FK_ComposLl_Llavero (Cod_Llavero, F_Alta_Ll), 		ADD CONSTRAINT FK_ComposLl_Llavero FOREIGN KEY (Cod_Llavero, F_Alta_Ll) REFERENCES Llavero (Cod_Llavero, F_Alta_Ll) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Llave_Prestada 					ADD INDEX FK_LlavePrest_Llave (Cod_Llave, F_Alta_L), 			ADD CONSTRAINT FK_LlavePrest_Llave FOREIGN KEY (Cod_Llave, F_Alta_L) REFERENCES Llave (Cod_Llave, F_Alta_L) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Llave_Prestada 					ADD INDEX FK_LlavePrest_Acceso (Entrada), 						ADD CONSTRAINT FK_LlavePrest_Acceso FOREIGN KEY (Entrada) REFERENCES Acceso (Entrada) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Llavero_Prestado 				ADD INDEX FK_LlaveroPrest_Acceso (Entrada), 					ADD CONSTRAINT FK_LlaveroPrest_Acceso FOREIGN KEY (Entrada) REFERENCES Acceso (Entrada) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Llavero_Prestado 				ADD INDEX FK_LlaveroPrest_Llavero (Cod_Llavero, F_Alta_Ll),		ADD CONSTRAINT FK_LlaveroPrest_Llavero FOREIGN KEY (Cod_Llavero, F_Alta_Ll) REFERENCES Llavero (Cod_Llavero, F_Alta_Ll) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Destinatario 					ADD INDEX FK_Destinatario_Nota (Fecha, DNI_A), 					ADD CONSTRAINT FK_Destinatario_Nota FOREIGN KEY (Fecha, DNI_A) REFERENCES Nota (Fecha, DNI_A) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Destinatario					ADD INDEX FK_Destinatario_Personal (DNI_D),						ADD CONSTRAINT FK_Destinatario_Personal FOREIGN KEY (DNI_D) REFERENCES Personal (DNI) ON UPDATE Cascade ON DELETE Cascade;
	ALTER TABLE Acceso_Sistema					ADD INDEX FK_AccesoSistema_Cuenta (Usuario),					ADD CONSTRAINT FK_AccesoSistema_Cuenta FOREIGN KEY (Usuario) REFERENCES Cuenta (Usuario) ON UPDATE Cascade ON DELETE Cascade;
	
-- *********************************************************************************************************************
-- *********************************************************************************************************************




-- *********************************************************************************************************************
-- ******** POBLACION																						************
-- *********************************************************************************************************************

	-- TABLA 'T_Usuario'
	INSERT INTO T_Usuario VALUES
		('ADMINISTRADOR', 'Administrador del sistema'),
		('AUTORIZADOR', 'Autorizador de departamentos'),
		('VIGILANTE', 'Vigilante del centro'),
		('RP', 'Responsable de personal de departamento(s)');
		
	-- TABLA 'Nivel'
	INSERT INTO Nivel (Cod_Niv,F_Alta_N,Descripcion) VALUES
		(0,'2010-01-01 00:00:01','Acceso restringido: Zonas comunes'),
		(1,'2010-01-01 00:00:01','Acceso total: Zonas comunes + Host');

	-- TABLA 'Color'
	INSERT INTO Color (Cod_Color,F_Alta_C,Cod_Niv,F_Alta_N) VALUES
		(-1,'2010-01-01 00:00:01',0,'2010-01-01 00:00:01'),
		(-256,'2010-01-01 00:00:01',0,'2010-01-01 00:00:01'),
		(-16776961,'2010-01-01 00:00:01',1,'2010-01-01 00:00:01');

	-- TABLA 'T_Acceso'
	INSERT INTO T_Acceso VALUES
		('VIG','Vigilancia','LMXJVSD','00:00:00','23:59:59',False,1,'2010-01-01 00:00:01'),
		('MHT','Mantenimiento','LMXJV','07:00','21:00',False,1,'2010-01-01 00:00:01'),
		('MHC','Mantenimiento','LMXJV','07:00','21:00',False,0,'2010-01-01 00:00:01'),
		('MHTA','Mantenimiento','LMXJV','07:00','21:00',True,1,'2010-01-01 00:00:01'),
		('MHCA','Mantenimiento','LMXJV','07:00','21:00',True,0,'2010-01-01 00:00:01'),
		('MLT','Mantenimiento','LMXJVSD','00:00:00','23:59:59',False,1,'2010-01-01 00:00:01'),
		('MLC','Mantenimiento','LMXJVSD','00:00:00','23:59:59',False,0,'2010-01-01 00:00:01'),
		('MLTA','Manteniento','LMXJVSD','00:00:00','23:59:59',True,1,'2010-01-01 00:00:01'),
		('MLCA','Mantenimiento','LMXJVSD','00:00:00','23:59:59',True,0,'2010-01-01 00:00:01'),
		('THT','Trabajo','LMXJV','09:00','21:00',False,1,'2010-01-01 00:00:01'),
		('THC','Trabajo','LMXJV','09:00','21:00',False,0,'2010-01-01 00:00:01'),
		('THTA','Trabajo','LMXJV','09:00','21:00',True,null,null),
		('THCA','Trabajo','LMXJV','09:00','21:00',True,0,'2010-01-01 00:00:01');

	-- TABLA 'Empresa'
	INSERT INTO Empresa	VALUES
		('CIS','B82738212','Compañía Integral de Seguridad','C/Castillo de Maya, 2\n31015 Pamplona'),
		('CONA','B87268920','Conasa','C/Santa Engracia, 43\n31014 Pamplona'),
		('INDR','B73621917','Indra','Polígono Landazábal\nCalle 1, Nave 15\nVillava'),
		('CADO','B31168479','Cador','Polígono Talluntxe\nCalle C, Números 15167772157\n31110 Noain'),
		('COMF','B37392112','Comflo','R. Ruben, 12\n08004 Barcelona'),
		('CONT','B82517261','Conatel','C/Serapio Huici, 22\n31017 Villava'),
		('ELEC','B53017290','Electrolumen','C/Mayor, 55\n31018 Burlada'),
		('ELKO','B43114827','Elkor','C/Ezkaba, 5\n31014 Pamplona'),
		('IBM','B19273928','IBM','Avda. Villava, 44\n31014 Pamplona'),
		('INGE','B53820172','Ingered','Avda. Marcelo Celayeta,\n99\n31013 Pamplona'),
		('ISTH','B91729271','Isis Thot','Avda. San Jorge, 71\n31012 Pamplona'),
		('LILE','B54232432','Limpiezas Leyre','C/Monasterio de la Oliva, 25\n31011 Pamplona'),
		('MEYS','B92872623','Meysh','Avda. Barañain, 52\n28080 Barcelona'),
		('OPNA','B52820172','Opnatel','Avda. Pamplona, 1\n26281 Madrid'),
		('ORAI','B86281625','Oraintxe','C/Irunlarrea, 2\n56241 Bilbao'),
		('PROD','B91726528','Prodein','Avda. Pio XII, 31\n31015 Pamplona'),
		('PSH','B72128952','PSH','Avda. Baja Navarra, 34\n31013 Ansoain'),
		('ROLL','B71629108','Rolltore','Paseo Ibaialde, 187\n31010 Mutilva Baja'),
		('SERN','B51728261','Sernaman','Plaza San Pedro, 1\n28081 Barcelona'),
		('STOR','B91729272','Storagetek','C/ San Saturnino, 6\n45627 Valencia'),
		('SVS','B66738279','SVS','C/Amaya, 22 . 31015 Pamplona'),
		('RAXE','B91728722','Rank Xerox','Avda. Guipúzcoa, 7\n31009 Noain'),
		('GUPO','B23132581','Gupost','C/Maria Viscarret, 46\n31018 Artica');

	-- TABLA 'Personal'
	INSERT INTO Personal VALUES
		('00000000A','Napal','Basterrica','Héctor',null),
		('00000001A','Salinas','Hernández','Rosa','00000198A'),
		('00000002A','Haydee','Duck','Graciela','00000000A'),
		('00000003A','Casado','Sierra','José Manuel','00000198A'),
		('00000018A','Teresa','Gamiz','María','00000000A'),
		('00000028A','Trillo','Carmona','Laura','00000000A'),
		('00000043A','Flores','Cavido','Tamara','00000000A'),
		('00000004A','Sierra','Casado','Natividad','00000018A'),
		('00000005A','Díaz','Boró','Natalia','00000000A'),
		('00000006A','Nieves','Lopez','Rosa','00000163A'),
		('00000007A','Fuerte','Terol','Rosa','00000190A'),
		('00000008A','Barbero','Garrido','Francisco Javier','00000000A'),
		('00000009A','Pinteno','Correa','Jose','00000000A'),
		('00000010A','Rodríguez','Arnoso','Ramón','00000190A'),
		('00000011A','Hervas','Pardo','Lidia','00000018A'),
		('00000012A','Belén','Gonzalo','Ana','00000000A'),
		('00000013A','Vives','Gatón','Alicia','00000190A'),
		('00000014A','Llorente','Pérez','Siro','00000001A'),
		('00000015A','Mendoza','Ponce','Alma','00000190A'),
		('00000019A','Bescos','Garcia','Luis','00000002A'),
		('00000020A','Boro','Diaz','Laura','00000003A'),
		('00000021A','Bartels','Ferran','Mireia','00000164A'),
		('00000022A','Caballero','García','Montserrat','00000000A'),
		('00000023A','Morales','Lorenzo','Inmaculada','00000000A'),
		('00000024A','Boró','Díaz','Rafaél','00000000A'),
		('00000025A','Vergara','Cela','Alberto','00000164A'),
		('00000026A','Andreu','García','Maria del Carmen','00000191A'),
		('00000027A','Cebada','Sanchez','Patricia','00000166A'),
		('00000029A','Laforga','Galindo','Javier','00000192A'),
		('00000030A','Guirado','Gálvez','Catalina','00000193A'),
		('00000031A','Rojas','Suez','Carlos','00000001A'),
		('00000032A','Marin','Gallardo','Rosa María','00000193A'),
		('00000033A','Perez','Gutierrez','Belén','00000000A'),
		('00000034A','Garcia','Moreno','Cristina','00000002A'),
		('00000035A','López','Pérez','Elena','00000193A'),
		('00000036A','López','Plaza','Ana','00000000A'),
		('00000037A','Granado','Torices','Encarna','00000193A'),
		('00000038A','Salgado','Rodríguez','Sandra','00000167A'),
		('00000039A','Arbona','Moragues','Silvia','00000000A'),
		('00000040A','Llamas','Aylagas','Melanie','00000003A'),
		('00000041A','Garriga','Murcientes','Jonathan','00000167A'),
		('00000042A','Camargo','Herves','Luis Miguel','00000000A'),
		('00000044A','Espinosa','Márquez','Dolores','00000168A'),
		('00000045A','Zeballos','Giers','Mirtha','00000001A'),
		('00000046A','Mañá','Pérez','Alejandro','00000169A'),
		('00000047A','Gil','Paul','Miguel','00000169A'),
		('00000048A','Gómez','Sánchez','Ana','00000169A'),
		('00000050A','Gómez','Sánchez','Rosa','00000169A'),
		('00000051A','Morrison','Aguiar','John','00000169A'),
		('00000052A','Manrique','Ramirez','Juan Manuel','00000000A'),
		('00000053A','Garcia','Barrero','Ismael','00000170A'),
		('00000054A','Elkoro','Garmendia','Marijo','00000170A'),
		('00000055A','Pita','Cañas','Javier','00000170A'),
		('00000056A','Andres','Lozano','Pablo','00000001A'),
		('00000057A','Vázquez','Jiménez','Alvaro','00000171A'),
		('00000058A','Vázquez','Jiménez','Flory','00000194A'),
		('00000059A','Muñoz','Collazos','Adriana','00000195A'),
		('00000060A','Vargas','Pinto','Morella','00000172A'),
		('00000061A','María','Guzmán','Margarita','00000000A'),
		('00000062A','Valencia','Marín','Verónica','00000172A'),
		('00000063A','Cadena','Cruz','Tatiana','00000000A'),
		('00000064A','Cataño','Herrera','María','00000188A'),
		('00000065A','González','Echeverrí','Catalina','00000188A'),
		('00000066A','Alonso','Nule','Elias','00000188A'),
		('00000067A','Teresa','Cáceres','Claudia','00000002A'),
		('00000068A','Caro','Espinosa','Carlos','00000198A'),
		('00000069A','Gabriel','Vázquez','Marcos','00000198A'),
		('00000070A','Uribe','Tobón','Carlos','00000198A'),
		('00000071A','López','Mérida','Rafael','00000198A'),
		('00000072A','Gonzalez','Treviño','Margarita','00000000A'),
		('00000073A','Velasco','Peinado','David','00000000A'),
		('00000074A','Serramitjana','Gómez','David','00000198A'),
		('00000075A','Montero','Quesada','José','00000198A'),
		('00000076A','Niño','Pérez','Jorge','00000198A'),
		('00000077A','Casarrubios','Pinilla','Nuria','00000198A'),
		('00000078A','Massa','Achútegui','Alberto','00000198A'),
		('00000079A','Aragón','Castro','Nuria','00000198A'),
		('00000080A','Carcel','Romero','Eva','00000198A'),
		('00000081A','Fuentes','Sahagún','Ramón','00000198A'),
		('00000082A','Pardo','Encinas','JoséMaría','00000000A'),
		('00000083A','Martínez','Sanz','Juan Antonio','00000173A'),
		('00000084A','Cruz','Elosegi','José','00000173A'),
		('00000085A','Garcia','Chica','Ildefonso','00000000A'),
		('00000086A','Gárnica','Silva','Margot','00000174A'),
		('00000087A','Granado','Torices','Belén','00000174A'),
		('00000088A','Vicente','Santayana','Gema','00000174A'),
		('00000089A','Martin','Cuevas','Elisa','00000175A'),
		('00000090A','Lopetegi','Peñagarikano','Pablo','00000176A'),
		('00000091A','Santibañez','Arévalo','Jorge','00000176A'),
		('00000092A','Campos','Juarez','Sandra','00000176A'),
		('00000093A','Merchan','Company','Juan','00000176A'),
		('00000094A','Medina','Rivera','Carlos','00000000A'),
		('00000095A','Carmona','Garcia','Sofia','00000176A'),
		('00000096A','King','Sakura','Yan','00000177A'),
		('00000097A','Ibañez','Alonso','Miguel','00000177A'),
		('00000098A','Leyva','Perez','Ezequiel','00000177A'),
		('00000099A','Alvarez','Pascal','Paulina','00000177A'),
		('00000100A','Rodríguez','Cervantes','María','00000178A'),
		('00000101A','Vozmediano','Gallego','Ignacio','00000179A'),
		('00000102A','Torres','Villar','Cecill','00000180A'),
		('00000103A','Montijo','Robledo','Sergio','00000000A'),
		('00000104A','Villar','Aguirre','Sonia','00000180A'),
		('00000105A','Velasco','Peinado','Nuria','00000180A'),
		('00000106A','Barrigón','Fernández','Mónica','00000000A'),
		('00000107A','Alanís','Menéndez','Eloísa','00000000A'),
		('00000108A','Gutierrez','Lancha','Susana','00000180A'),
		('00000109A','Díaz','Prado','Laura','00000109A'),
		('00000111A','Díaz','Hipólito','Francisco','00000109A'),
		('00000112A','Prado','Barrero','Inés','00000181A'),
		('00000113A','Hernández','Truebas','Rosa','00000181A'),
		('00000114A','Torres','Alejandre','Juan','00000182A'),
		('00000115A','Chia','Perea','Bertha','00000182A'),
		('00000116A','Valderrama','Fabian','Erika','00000000A'),
		('00000117A','Sanchez','Mosqueda','Sara','00000182A'),
		('00000118A','Antonio','Robles','Pablo','00000183A'),
		('00000119A','Valdivia','Germain','Cristián','00000183A'),
		('00000120A','Navarro','Ortiz','Mayka','00000183A'),
		('00000121A','Sans','Blanco','Ana','00000183A'),
		('00000122A','Talavera','Tassara','Alfonso','00000184A'),
		('00000123A','García','Ordóñez','Noemí','00000184A'),
		('00000124A','Remedios','Giner','María','00000184A'),
		('00000125A','Fernando','Rosas','Andrés','00000184A'),
		('00000126A','Castillo','Villegas','Eduardo','00000000A'),
		('00000127A','Linares','Miró','Josep','00000184A'),
		('00000128A','Pacheco','Villar','Mónica','00000184A'),
		('00000129A','Vazquez','Ferreiro','Raquel','00000185A'),
		('00000130A','Gonzalez','Lupiañez','Gabriel','00000185A'),
		('00000131A','Martínez','Sendino','Raquel','00000186A'),
		('00000132A','Mas','Magallón','Montse','00000186A'),
		('00000133A','Mun','Mas','Carles','00000186A'),
		('00000134A','Martines','Garcia','Miquel','00000186A'),
		('00000135A','Basaganya','Obiols','Rosa','00000186A'),
		('00000136A','Basaganya','Obiols','Teresa','00000186A'),
		('00000137A','Vicente','Mateos','Emilio','00000186A'),
		('00000138A','Agüero','Venturi','Carlos','00000187A'),
		('00000139A','Blasón','Cynthia','Desireé','00000187A'),
		('00000140A','Villagrán','Gómez','Patricia','00000172A'),
		('00000141A','Ivette','Cuello','Graciela','00000188A'),
		('00000142A','Canop','Mazar','Daniela','00000188A'),
		('00000143A','Canop','Mazar','Gabriela','00000198A'),
		('00000144A','Marta','Piazzini','Graciela','00000198A'),
		('00000145A','Leonardo','Baez','Iván','00000198A'),
		('00000146A','Casielles','Martinez','Paola','00000198A'),
		('00000147A','Infanzón','Domínguez','Fernando','00000198A'),
		('00000148A','Gómez','Alustiza','Mariana','00000198A'),
		('00000149A','Romero','Guerra','Pío','00000198A'),
		('00000150A','María','Alvarez','Cecilia','00000198A'),
		('00000151A','Hernández','Valerio','Edwin','00000198A'),
		('00000152A','Méndez','Signoret','Almudena','00000163A'),
		('00000153A','Yamilet','Salazar','Patricia','00000163A'),
		('00000154A','García','Sánchez','Jorge','00000163A'),
		('00000155A','Aurelia','Quesada','María','00000190A'),
		('00000156A','Poblador','García','Maite','00000190A'),
		('00000157A','Emilia','Durán','María','00000190A'),
		('00000158A','Durán','Quesada','Miriam','00000190A'),
		('00000159A','Reyes','Rios','Patricia','00000190A'),
		('00000160A','Martinez','Carne','Pedro','00000190A'),
		('00000161A','Romero','Riveroll','María Teresa','00000190A'),
		('00000162A','Alejandra','Cortez','María','00000164A'),
		('00001000A','Tirado','Jimenez','Eva María','00000018A'),
		('00000163A','García','Pérez','Javier','00000018A'),
		('00000190A','Valencia','Valle','Juana María','00000018A'),
		('00000164A','Aldasoro','Ortega','Edurne','00000028A'),
		('00000165A','Beguiristain','Amatriain','Verónica','00000163A'),
		('00000191A','Vera','Gil','Raquel','00000043A'),
		('00000166A','Dominguez','Santillana','Pedro','00000043A'),
		('00000192A','Acosta','Moreno','Jose Manuel','00000043A'),
		('00000193A','Aguilera','Frias','Jose Manuel','00000028A'),
		('00000167A','Rojas','Garcia','Claudia','00000028A'),
		('00000168A','Gonzalez','Cuadrado','Maribel','00000043A'),
		('00000169A','Van','Elsen','Eric','00000028A'),
		('00000170A','Hyun','Kim','Yong','00000028A'),
		('00000171A','Gonzalez','Quiroga','Paloma','00000018A'),
		('00000194A','Abellán','Abellán','Celia','00000043A'),
		('00000195A','Baeza','Gomis','Esther','00000028A'),
		('00000172A','Oteo','Latorre','Maria Elena','00000028A'),
		('00000196A','Calderón','Ayala','Maria Olvido','00000043A'),
		('00000198A','Delicado','Davo','Gaspar','00000018A'),
		('00000188A','Borde','Membibre','Irene','00000043A'),
		('00000189A','Cestau','Baraibar','Begoña','00000198A'),
		('00000173A','Mosquera','López','Sandra','00000028A'),
		('00000174A','Valdivia','Lizana','Cristina','00000018A'),
		('00000175A','Zúñiga','Espinoza','Cristóbal','00000018A'),
		('00000176A','Lara','Díaz','Carla','00000028A'),
		('00000177A','Lopez','Mendez','Margarita','00000043A'),
		('00000178A','Moreno','Mohedano','Leticia','00000043A'),
		('00000179A','Estero','Olmo','Antonio','00000018A'),
		('00000180A','Barbero','Pardal','Carmen','00000043A'),
		('00000181A','Estero','Barbero','Gessica','00000043A'),
		('00000182A','Sevilla','Espina','Coral','00000018A'),
		('00000183A','Chávez','Cáceres','Catalina','00000028A'),
		('00000184A','Moratilla','López','Beatriz','00000043A'),
		('00000185A','Ruiz','Aguilar','Clara','00000018A'),
		('00000186A','Reula','Gómez','Esther','00000043A'),
		('00000187A','Hernández','Estiarte','Xavier','00000018A');
		
	-- TABLA 'Residente'
	INSERT INTO Residente VALUES
		('00000000A',1,'2010-01-01 00:00:01'),
		('00000005A',1,'2010-01-01 00:00:01'),
		('00000006A',1,'2010-01-01 00:00:01'),
		('00000008A',1,'2010-01-01 00:00:01'),
		('00000009A',1,'2010-01-01 00:00:01'),
		('00000014A',1,'2010-01-01 00:00:01'),
		('00000018A',1,'2010-01-01 00:00:01'),
		('00000019A',1,'2010-01-01 00:00:01'),
		('00000028A',1,'2010-01-01 00:00:01'),
		('00000033A',1,'2010-01-01 00:00:01'),
		('00000036A',1,'2010-01-01 00:00:01'),
		('00000041A',1,'2010-01-01 00:00:01'),
		('00000043A',1,'2010-01-01 00:00:01'),
		('00000051A',1,'2010-01-01 00:00:01'),
		('00000060A',1,'2010-01-01 00:00:01'),
		('00000063A',1,'2010-01-01 00:00:01'),
		('00000071A',0,'2010-01-01 00:00:01'),
		('00000073A',1,'2010-01-01 00:00:01'),
		('00000081A',1,'2010-01-01 00:00:01'),
		('00000085A',1,'2010-01-01 00:00:01'),
		('00000094A',1,'2010-01-01 00:00:01');
	
	-- TABLA 'Ocasional'
	INSERT INTO Ocasional VALUES
		('00000020A','Endesa','THTA','2013-07-07 08:46:04',null),
		('00000031A','Electricidad Manuel','THT','2012-07-21 22:20:21','No dejarle ninguna llave.'),
		('00000034A','Particular','THTA','2012-06-28 07:57:42',null),
		('00000040A','Fontaneros S.A.','THT','2012-05-16 23:00:21','Decirle que acuda en primer lugar a hablar con Raquel Martínez.'),
		('00000045A','Gobierno de Navarra','THTA','2012-04-05 16:21:47',null),
		('00000056A','Fontaneros S.A.','MHTA','2011-07-02 19:22:47',null),
		('00000067A','Particular','MHTA','2015-02-15 21:51:47',null);
		
	-- TABLA 'Departamento'
	INSERT INTO Departamento VALUES
		('CIS','VA','Vigilancia con arma','VIG','365','2010-01-31','00000018A'),
		('CONA','MANT','Mantenimiento H/S 24x7','MLTA','60','2010-01-01','00000018A'),
		('CONA','CAU','Centro Atención Usuario 24x7','MLT','30','2010-01-02','00000018A'),
		('CONA','EMER','Emergencias 24x7','MLT','30','2010-01-02','00000028A'),
		('CONA','SIST','Sistemas','MHTA','60','2010-01-03','00000028A'),
		('CONA','ATEC','Asistencia Técnica','THTA','60','2010-01-09','00000043A'),
		('INDR','CLYO','Consultoría legal y organizativa','THCA','90','2010-01-21','00000043A'),
		('INDR','FYPR','Formación y pruebas','THCA','30','2010-01-21','00000043A'),
		('INDR','LOIN','Logística e infraestructuras','THC','120','2010-01-22','00000018A'),
		('INDR','CTYC','Consultoría técnica y comunicaciones','THT','90','2010-01-30','00000028A'),
		('CADO','DICO','Diseño y construcción','THT','60','2010-01-07','00000028A'),
		('COMF','DEPV','Departamento de ventas','THCA','60','2010-01-03','00000043A'),
		('CONT','DIRE','Diseño de redes','THT','365','2010-01-09','00000028A'),
		('CONT','MARE','Mantenimiento de redes','MHT','30','2010-01-01','00000028A'),
		('ELEC','CONS','Consultoría','THTA','30','2010-01-08','00000028A'),
		('ELEC','INMA','Instalación y mantenimiento de equipos','MHT','90','2010-01-25','00000018A'),
		('ELKO','INEQ','Instalación de equipos','MHT','30','2010-01-22','00000018A'),
		('ELKO','MANT','Mantenimiento','MHT','120','2010-01-01','00000043A'),
		('ELKO','EMER','Servicio de emergencia 24h','MLTA','90','2010-01-01','00000028A'),
		('IBM','ESAR','Dep. Estrategia y arquitectura','THC','60','2010-01-02','00000028A'),
		('IBM','MIDD','Dep. Middleware','MHTA','60','2010-01-24','00000043A'),
		('IBM','COIN','Dep. Comunicaciones integradas','MHTA','90','2010-01-28','00000028A'),
		('IBM','SEPR','Dep. Seguridad y privacidad','MHT','60','2010-01-30','00000018A'),
		('IBM','ALDA','Dep. Almacenamiento y datos','MHT','30','2010-01-01','00000018A'),
		('INGE','INMA','Instalación y mantenimiento','MHCA','30','2010-01-02','00000028A'),
		('ISTH','DESO','Desarrollo de Software','MHC','90','2010-01-01','00000018A'),
		('LILE','LIGE','Limpieza general','MHT','30','2010-02-02','00000018A'),
		('MEYS','TELE','Dep. Telecomunicaciones','MHT','30','2010-01-01','00000028A'),
		('MEYS','SEIN','Dep. Seguridad contra incendios','MHTA','60','2010-01-01','00000018A'),
		('OPNA','DRED','Dep. Redes','MHT','60','2010-01-20','00000043A'),
		('ORAI','MENS','Mensajería','MHCA','90','2010-01-03','00000043A'),
		('PROD','MINC','Material contra incendios','MHT','60','2010-01-09','00000018A'),
		('PSH','DELE','Dep. Electricidad','MHT','60','2010-01-01','00000043A'),
		('PSH','DTEL','Dep. Telecomunicaciones','MHT','90','2010-01-28','00000028A'),
		('ROLL','INMA','Instalación y mantenimiento de puertas','MHTA','60','2010-01-03','00000043A'),
		('SERN','MAIN','Mantenimiento de instalaciones','MHT','30','2010-01-01','00000018A'),
		('STOR','ANDI','Análisis y diseño','THT','60','2010-01-01','00000028A'),
		('STOR','MASE','Mantenimiento de servidores','MHT','30','2010-01-07','00000043A'),
		('SVS','SVIS','Sistemas de vigilancia y seguridad','MLTA','60','2010-01-24','00000018A'),
		('RAXE','DVEN','Dep. Ventas','THCA','90','2010-01-22','00000028A'),
		('RAXE','DMAN','Dep. Mantenimiento','MHTA','60','2010-01-29','00000043A'),
		('GUPO','DMAR','Dep. Marketing','THCA','30','2010-01-30','00000018A');
		
	-- TABLA 'Cargo'
	INSERT INTO Cargo VALUES
		('RP', 'Responsable de personal'),
		('ANAL','Analista'),
		('GERE','Gerente');
		
	-- TABLA 'Periodico'
	INSERT INTO Periodico VALUES
		('00000001A','CIS','VA'),
		('00000002A','CIS','VA'),
		('00000003A','CIS','VA'),
		('00000163A','CONA','MANT'),
		('00000164A','CONA','SIST'),
		('00000165A','CONA','MANT'),
		('00000190A','CONA','CAU'),	
		('00000169A','CONT','DIRE'),	
		('00000171A','ELKO','INEQ'),
		('00000188A','IBM','MIDD'),
		('00000198A','IBM','COIN'),
		('00000174A','ISTH','DESO'),
		('00000176A','MEYS','TELE'),
		('00000177A','OPNA','DRED'),
		('00000178A','ORAI','MENS'),
		('00000179A','PROD','MINC'),
		('00000180A','PSH','DELE'),
		('00000181A','ROLL','INMA'),
		('00000182A','SERN','MAIN'),
		('00000183A','STOR','ANDI'),
		('00000184A','STOR','MASE'),
		('00000185A','SVS','SVIS'),
		('00000186A','RAXE','DMAN'),
		('00000187A','GUPO','DMAR'),
		('00000191A','CONA','ATEC'),		
		('00000166A','INDR','CLYO'),
		('00000192A','INDR','FYPR'),
		('00000193A','INDR','LOIN'),
		('00000167A','CADO','DICO'),
		('00000168A','COMF','DEPV'),
		('00000170A','ELEC','CONS'),
		('00000194A','ELKO','MANT'),
		('00000195A','ELKO','EMER'),
		('00000172A','IBM','ESAR'),
		('00000196A','IBM','MIDD'),
		('00000173A','INGE','INMA'),
		('00000175A','LILE','LIGE'),
		('00000004A','CONA','MANT'),
		('00000007A','CONA','CAU'),
		('00000010A','CONA','CAU'),
		('00000013A','CONA','EMER'),
		('00000015A','CONA','EMER'),
		('00000021A','CONA','SIST'),
		('00000025A','CONA','SIST'),
		('00000026A','CONA','ATEC'),
		('00000027A','INDR','CLYO'),
		('00000029A','INDR','FYPR'),
		('00000030A','INDR','LOIN'),
		('00000032A','INDR','LOIN'),
		('00000035A','INDR','CTYC'),
		('00000037A','INDR','CTYC'),
		('00000038A','CADO','DICO'),
		('00000041A','CADO','DICO'),
		('00000044A','COMF','DEPV'),
		('00000046A','CONT','DIRE'),
		('00000047A','CONT','DIRE'),
		('00000048A','CONT','DIRE'),
		('00000050A','CONT','MARE'),
		('00000051A','ELEC','CONS'),
		('00000053A','ELEC','INMA'),
		('00000054A','ELEC','INMA'),
		('00000055A','ELEC','INMA'),
		('00000057A','ELKO','INEQ'),
		('00000058A','ELKO','MANT'),
		('00000059A','ELKO','EMER'),
		('00000060A','IBM','ESAR'),
		('00000062A','IBM','ESAR'),
		('00000064A','IBM','MIDD'),
		('00000065A','IBM','MIDD'),
		('00000066A','IBM','MIDD'),
		('00000068A','IBM','COIN'),
		('00000069A','IBM','COIN'),
		('00000070A','IBM','COIN'),
		('00000071A','IBM','SEPR'),
		('00000074A','IBM','SEPR'),
		('00000075A','IBM','SEPR'),
		('00000076A','IBM','SEPR'),
		('00000077A','IBM','ALDA'),
		('00000078A','IBM','ALDA'),
		('00000079A','IBM','ALDA'),
		('00000080A','IBM','ALDA'),
		('00000083A','INGE','INMA'),
		('00000084A','INGE','INMA'),
		('00000086A','ISTH','DESO'),
		('00000087A','ISTH','DESO'),
		('00000088A','ISTH','DESO'),
		('00000089A','LILE','LIGE'),
		('00000090A','MEYS','TELE'),
		('00000091A','MEYS','TELE'),
		('00000092A','MEYS','TELE'),
		('00000093A','MEYS','SEIN'),
		('00000095A','MEYS','SEIN'),
		('00000096A','OPNA','DRED'),
		('00000097A','OPNA','DRED'),
		('00000098A','OPNA','DRED'),
		('00000099A','OPNA','DRED'),
		('00000100A','ORAI','MENS'),
		('00000101A','PROD','MINC'),
		('00000102A','PSH','DELE'),
		('00000104A','PSH','DELE'),
		('00000105A','PSH','DELE'),
		('00000108A','PSH','DELE'),
		('00000109A','PSH','DTEL'),
		('00000111A','PSH','DTEL'),
		('00000112A','ROLL','INMA'),
		('00000113A','ROLL','INMA'),
		('00000114A','SERN','MAIN'),
		('00000115A','SERN','MAIN'),
		('00000117A','SERN','MAIN'),
		('00000118A','STOR','ANDI'),
		('00000119A','STOR','ANDI'),
		('00000120A','STOR','ANDI'),
		('00000121A','STOR','ANDI'),
		('00000122A','STOR','MASE'),
		('00000123A','STOR','MASE'),
		('00000124A','STOR','MASE'),
		('00000125A','STOR','MASE'),
		('00000127A','STOR','MASE'),
		('00000128A','STOR','MASE'),
		('00000129A','SVS','SVIS'),
		('00000130A','SVS','SVIS'),
		('00000131A','RAXE','DVEN'),
		('00000132A','RAXE','DVEN'),
		('00000133A','RAXE','DMAN'),
		('00000134A','RAXE','DMAN'),
		('00000135A','RAXE','DMAN'),
		('00000136A','RAXE','DMAN'),
		('00000137A','RAXE','DMAN'),
		('00000138A','GUPO','DMAR'),
		('00000139A','GUPO','DMAR'),
		('00000140A','IBM','ESAR'),
		('00000141A','IBM','MIDD'),
		('00000142A','IBM','MIDD'),
		('00000143A','IBM','COIN'),
		('00000144A','IBM','COIN'),
		('00000145A','IBM','COIN'),
		('00000146A','IBM','SEPR'),
		('00000147A','IBM','SEPR'),
		('00000148A','IBM','ALDA'),
		('00000149A','IBM','ALDA'),
		('00000150A','IBM','ALDA'),
		('00000151A','IBM','ALDA'),
		('00000152A','CONA','MANT'),
		('00000153A','CONA','MANT'),
		('00000154A','CONA','MANT'),
		('00000155A','CONA','CAU'),
		('00000156A','CONA','CAU'),
		('00000157A','CONA','CAU'),
		('00000158A','CONA','CAU'),
		('00000159A','CONA','EMER'),
		('00000160A','CONA','EMER'),
		('00000161A','CONA','EMER'),
		('00000162A','CONA','SIST'),
		('00000189A','IBM','COIN');
		
	-- TABLA 'Contacto'
	INSERT INTO Contacto VALUES
		('00001000A','CIS','VA','RP'),
		('00000190A','CONA','CAU','RP'),	
		('00000169A','CONT','DIRE','RP'),	
		('00000171A','ELKO','INEQ','RP'),
		('00000198A','IBM','COIN','RP'),
		('00000198A','IBM','ALDA','RP'),
		('00000174A','ISTH','DESO','RP'),
		('00000176A','MEYS','TELE','RP'),
		('00000176A','MEYS','SEIN','RP'),
		('00000177A','OPNA','DRED','RP'),
		('00000178A','ORAI','MENS','RP'),
		('00000179A','PROD','MINC','RP'),
		('00000180A','PSH','DELE','RP'),
		('00000109A','PSH','DTEL','RP'),
		('00000181A','ROLL','INMA','RP'),
		('00000182A','SERN','MAIN','RP'),
		('00000183A','STOR','ANDI','RP'),
		('00000184A','STOR','MASE','RP'),
		('00000185A','SVS','SVIS','RP'),
		('00000186A','RAXE','DVEN','RP'),
		('00000186A','RAXE','DMAN','RP'),
		('00000187A','GUPO','DMAR','RP'),
		('00000190A','CONA','EMER','RP'),
		('00000190A','CONA','SIST','RP'),
		('00000191A','CONA','ATEC','RP'),		
		('00000166A','INDR','CLYO','RP'),
		('00000192A','INDR','FYPR','RP'),
		('00000193A','INDR','LOIN','RP'),
		('00000193A','INDR','CTYC','RP'),
		('00000167A','CADO','DICO','RP'),
		('00000168A','COMF','DEPV','RP'),
		('00000169A','CONT','MARE','RP'),
		('00000169A','ELEC','CONS','RP'),		
		('00000170A','ELEC','CONS','ANAL'),
		('00000170A','ELEC','INMA','RP'),
		('00000194A','ELKO','MANT','RP'),
		('00000195A','ELKO','EMER','RP'),
		('00000172A','IBM','ESAR','RP'),
		('00000196A','IBM','MIDD','RP'),
		('00000198A','IBM','SEPR','RP'),	
		('00000173A','INGE','INMA','RP'),
		('00000175A','LILE','LIGE','RP'),
		('00000004A','CONA','MANT','RP'),
		('00000011A','CONA','CAU','GERE');
		
	-- TABLA 'Vigilante'
	INSERT INTO Vigilante VALUES
		('00000001A','000000'),
		('00000002A','000001'),
		('00000003A','000002');

	-- TABLA 'Parte_Seguridad'
	INSERT INTO Parte_Seguridad VALUES
		('2010-01-23 - 15:00','000000','Abro y cierro el servicio sin incidencias.'),
		('2010-02-12 - 23:00','000001','Abro y cierro el servicio sin incidencias.'),
		('2010-02-24 - 23:00','000000',concat('23:00 - Inicio servicio s/n aparente.\n\n23:30-Ronda al centro s/n. ',
			'\n\n07:00 - Finalizo servicio s/n.')),
		('2010-03-02 - 07:00','000002',concat('07:00 - Inicio servicio s/n aparente.\n\n13:00 - 00000117A comunica ',
			' que han sido conectados armarios de UPS a la central de alarmas.\n\n15:00 - Finalizo servicio s/n')),
		('2010-04-15 - 15:00','000000', concat('15:00 - Inicio servicio s/n aparente.\n\n20:00 - Sonda ambiente 20,',
			'6º.\n\n21:30 - Ronda por el centro apagando luces.\n\n23:00 - Finalizo servicio s/n')),
		('2010-04-16 - 19:00','000001', concat('19:00 - Inicio servicio s/n aparente.\n\n20:00 - Sonda ambiente 20,',
			'4º.\n\n22:00 - Ronda por el centro s/n.\n\n07:00 - Finalizo servicio s/n')),
		('2010-05-26 - 23:00','000001', concat('23:00 - Inicio servicio s/n aparente.\n\n02:00 - Ronda por el centro s/n.\n\n07:00 - Finalizo servicio s/n'));
		
	-- TABLA 'Cuenta'
	INSERT INTO Cuenta (Usuario, Tipo, DNI) VALUES
		('mgamiz','AUTORIZADOR','00000018A'),
		('lcarmona','AUTORIZADOR','00000028A'),
		('tcavido','AUTORIZADOR','00000043A'),
		('hnapalb','ADMINISTRADOR','00000000A'),
		('rsalinash','VIGILANTE','00000001A'),
		('ghaydeed','VIGILANTE','00000002A'),
		('jcasados','VIGILANTE','00000003A'),
		('ejimenezt','RP','00001000A'),
		('jgarciap','RP','00000163A'),
		('jvalenciav','RP','00000190A'),
		('rverag','RP','00000191A'),		
		('pdominguezs','RP','00000166A'),
		('jacostam','RP','00000192A'),
		('jaguileraf','RP','00000193A'),
		('crojasg','RP','00000167A'),
		('mgonzalezc','RP','00000168A'),
		('evane','RP','00000169A'),
		('yhyunk','RP','00000170A'),
		('pgonzalezq','RP','00000171A'),
		('cabellana','RP','00000194A'),
		('ebaezag','RP','00000195A'),
		('moteol','RP','00000172A'),
		('mcalderona','RP','00000196A'),
		('gdelicadod','RP','00000198A'),
		('smosqueral','RP','00000173A'),
		('cvaldivial','RP','00000174A'),
		('czuñigae','RP','00000175A'),
		('clarad','RP','00000176A'),	
		('mlopezm','RP','00000177A'),
		('lmorenom','RP','00000178A'),
		('aesteroo','RP','00000179A'),
		('cbarberop','RP','00000180A'),
		('gesterob','RP','00000181A'),
		('csevillae','RP','00000182A'),
		('cchavezc','RP','00000183A'),
		('bmoratillal','RP','00000184A'),
		('cruiza','RP','00000185A'),
		('ereulag','RP','00000186A'),
		('xhernandeze','RP','00000187A');
			
	-- TABLA 'Tarjeta'
	INSERT INTO Tarjeta (Cod_Tarj,F_Alta_T,Cod_Color,F_Alta_C,DNI) VALUES
		('000','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01',null),
		('001','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01',null),
		('002','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01',null),
		('003','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01','00000150A'),
		('004','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01',null),
		('005','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01','00000118A'),
		('006','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01',null),
		('007','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01',null),
		('008','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01','00000008A'),
		('009','2010-01-01 00:00:01',-1,'2010-01-01 00:00:01',null),
		('010','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01','00000044A'),
		('011','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('012','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('013','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('014','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('015','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('016','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('017','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('018','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('019','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('020','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('021','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('022','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('023','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('024','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('025','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('026','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01','00000010A'),
		('027','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('028','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('029','2010-01-01 00:00:01',-256,'2010-01-01 00:00:01',null),
		('030','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('031','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('032','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('033','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('034','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01','00000160A'),
		('035','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01','00000050A'),
		('036','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('037','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('038','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01','00000097A'),
		('039','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('040','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('041','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('042','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('043','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('044','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('045','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('046','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('047','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('048','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null),
		('049','2010-01-01 00:00:01',-16776961,'2010-01-01 00:00:01',null);
		
	-- TABLA 'Telefono'
	INSERT INTO Telefono VALUES
		('00000018A','+34687658362'),
		('00000028A','+34848638269'),
		('00000043A','+34965835295'),
		('00000001A','+34673638261'),
		('00000002A','+34691729362'),
		('00000003A','+34692738228'),
		('00000004A','+34613426124'),
		('00000006A','+34682028009'),
		('00000007A','+34653456257'),
		('00000010A','+34682928372'),
		('00000011A','+34698981923'),
		('00000013A','+34672637817'),
		('00000015A','+34682638162'),
		('00000011A','+34692793731'),
		('00000021A','+34672635293'),
		('00000025A','+34693729378'),
		('00000026A','+34682719278'),
		('00000027A','+34682361822'),
		('00000029A','+34632837123'),
		('00000030A','+34648936734'),
		('00000032A','+34683272833'),
		('00000035A','+34638372937'),
		('00000037A','+34632827388'),
		('00000038A','+34683721821'),
		('00000041A','+34672621726'),
		('00000044A','+34638200832'),
		('00000046A','+34637362538'),
		('00000047A','+34638203938'),
		('00000048A','+34638273821'),
		('00000048A','+34623728271'),
		('00000050A','+34637281762'),
		('00000051A','+34637281728'),
		('00000053A','+34627182718'),
		('00000054A','+34625162432'),
		('00000055A','+34635253421'),
		('00000057A','+34682167123'),
		('00000058A','+34687912364'),
		('00000059A','+34683267140'),
		('00000060A','+34680123482'),
		('00000062A','+34673169406'),
		('00000064A','+34682316921'),
		('00000065A','+34608713647'),
		('00000066A','+34670136740'),
		('00000068A','+34670263412'),
		('00000069A','+34601234076'),
		('00000070A','+34607216492'),
		('00000071A','+34678236407'),
		('00000074A','+34607114037'),
		('00000075A','+34683791872'),
		('00000076A','+34682169273'),
		('00000077A','+34683217022'),
		('00000078A','+34689172029'),
		('00000079A','+34652189721'),
		('00000080A','+34672169382'),
		('00000081A','+34617621092'),
		('00000083A','+34681728371'),
		('00000084A','+34682179279'),
		('00000086A','+34687217291'),
		('00000087A','+34682729382'),
		('00000088A','+34681728182'),
		('00000089A','+34672819283'),
		('00000090A','+34681829370'),
		('00000091A','+34681920382'),
		('00000092A','+34682187281'),
		('00000093A','+34681729817'),
		('00000095A','+34681728178'),
		('00000096A','+34681928998'),
		('00000097A','+34621872681'),
		('00000098A','+34681271829'),
		('00000099A','+34618273821'),
		('00000100A','+34681732091'),
		('00000101A','+34689172912'),
		('00000102A','+34682736528'),
		('00000104A','+34687291829'),
		('00000105A','+34682729289'),
		('00000108A','+34672821212'),
		('00000109A','+34621321234'),
		('00000111A','+34671628167'),
		('00000112A','+34687327394'),
		('00000113A','+34690879872'),
		('00000114A','+34671632976'),
		('00000115A','+34687123123'),
		('00000117A','+34687912098'),
		('00000118A','+34687172987'),
		('00000119A','+34698172919'),
		('00000120A','+34687239182'),
		('00000121A','+34698791023'),
		('00000122A','+34601830810'),
		('00000123A','+34691723987'),
		('00000124A','+34609102391'),
		('00000125A','+34609182092'),
		('00000127A','+34601923801'),
		('00000128A','+34601231233'),
		('00000129A','+34601092013'),
		('00000130A','+34601821093'),
		('00000131A','+34672516251'),
		('00000132A','+34672046523'),
		('00000133A','+34670136459'),
		('00000134A','+34615109897'),
		('00000135A','+34671639841'),
		('00000136A','+34670123740'),
		('00000137A','+34609731641'),
		('00000138A','+34617386412'),
		('00000139A','+34681734209'),
		('00000140A','+34687635482'),
		('00000141A','+34601403829'),
		('00000142A','+34609187391'),
		('00000143A','+34609213792'),
		('00000144A','+34601902983'),
		('00000145A','+34609198273'),
		('00000146A','+34682738271'),
		('00000147A','+34683729182'),
		('00000148A','+34672918291'),
		('00000149A','+34682190283'),
		('00000150A','+34610937981'),
		('00000151A','+34681030291'),
		('00000152A','+34603988104'),
		('00000153A','+34620129833'),
		('00000154A','+34601929438'),
		('00000155A','+34673928371'),
		('00000156A','+34691782392'),
		('00000157A','+34601932812'),
		('00000158A','+34601389021'),
		('00000159A','+34601926831'),
		('00000160A','+34603844012'),
		('00000161A','+34602023349'),
		('00000162A','+34601802434'),
		('00000163A','+34603135809'),
		('00001000A','+34948527192'),
		('00000163A','+34948739172'),
		('00000165A','+34675241926'),
		('00000190A','+34617826378'),
		('00000190A','+34948128293'),
		('00000190A','+34687291827'),
		('00000164A','+34948128729'),
		('00000191A','+34948928721'),
		('00000166A','+34948715281'),
		('00000192A','+34617367889'),
		('00000193A','+34667892127'),
		('00000193A','+34661292827'),
		('00000167A','+34673826356'),
		('00000168A','+34691628315'),
		('00000169A','+34678895378'),
		('00000169A','+34672715327'),
		('00000170A','+34665232345'),
		('00000170A','+34665172615'),		
		('00000171A','+34948672431'),
		('00000194A','+34612819283'),
		('00000195A','+94817293740'),
		('00000172A','+34676890120'),
		('00000196A','+34948372812'),
		('00000188A','+34678123456'),
		('00000189A','+34678524398'),
		('00000198A','+34948232821'),
		('00000173A','+34948124567'),
		('00000174A','+34689554287'),
		('00000175A','+34948456239'),
		('00000176A','+34662213448'),
		('00000176A','+34602934648'),
		('00000177A','+34671928273'),
		('00000178A','+34681524390'),
		('00000179A','+34948674251'),
		('00000180A','+34948252637'),
		('00000181A','+34948161734'),
		('00000182A','+34677918252'),
		('00000183A','+34948567239'),
		('00000184A','+34634281677'),
		('00000185A','+34687391235'),
		('00000186A','+34687453686'),
		('00000186A','+34657536886'),
		('00000187A','+34948126792');
		
	-- TABLA 'Email'
	INSERT INTO Email VALUES
		('00000006A','nieves@terra.es'),
		('00000097A','mickey1986@hotmail.com'),
		('00000097A','hectornapal@gmail.com'),
		('00001000A','jimenez@cis.com'),
		('00000163A','garcia@conasa.es'),
		('00000165A','beguiristain@conasa.es'),
		('00000190A','valencia@conasa.es'),
		('00000190A','valencia@hotmail.es'),
		('00000190A','valencia@gmail.es'),
		('00000164A','aldasoro@conasa.es'),
		('00000191A','vera@conasa.es'),
		('00000166A','dominguez@indra.es'),
		('00000192A','acosta@indra.es'),
		('00000193A','aguilera@indra.es'),
		('00000193A','aguilera@gmail.es'),
		('00000167A','rojas@cador.com'),
		('00000168A','gonzalez@comflo.es'),
		('00000169A','velsen@conatel.com'),
		('00000169A','velsen@yahoo.com'),
		('00000170A','hyun@electrolumen.com'),
		('00000170A','hyun@hotmail.com'),		
		('00000171A','gonzalez@elkor.com'),
		('00000194A','abellan@elkor.com'),
		('00000195A','baeza@elkor.com'),
		('00000172A','oteo@ibm.com'),
		('00000196A','calderon@ibm.com'),
		('00000188A','borde@ibm.es'),
		('00000198A','delicado@ibm.com'),
		('00000189A','cestau@ibm.es'),
		('00000198A','delicado@gmail.com'),
		('00000173A','mosquera@ingered.es'),
		('00000174A','valdivia@isistoth.com'),
		('00000175A','zespinoza@limpiezasleire.com'),
		('00000176A','lara@meysh.es'),
		('00000176A','lara@ono.es'),
		('00000177A','lopez@opnatel.org'),
		('00000178A','moreno@oraintxe.net'),
		('00000179A','estero@prodeis.net'),
		('00000180A','barbero@psh.es'),
		('00000181A','estero@rolltore.com'),
		('00000182A','sevilla@sernaman.es'),
		('00000183A','chavez@storagetek.com'),
		('00000184A','moratilla@storagetek.com'),
		('00000185A','ruiz@svs.com'),
		('00000186A','reula@rankxerox.com'),
		('00000186A','reula@hotmail.com'),
		('00000187A','hernandez@gupost.net');
		
	-- TABLA 'Almacen'
	INSERT INTO Almacen VALUES
		('A','Armario'),
		('CC','Caja de copias'),
		('V','Varios');
		
	-- TABLA 'Zona'
	INSERT INTO Zona VALUES
		('GRUPELEC','2010-01-01 00:00:01','Grupo electrógeno',0,'2010-01-01 00:00:01'),
		('SALAMAQS','2010-01-01 00:00:01 ','Sala de máquinas',0,'2010-01-01 00:00:01'),
		('UPS','2010-01-01 00:00:01','UPS',0,'2010-01-01 00:00:01'),
		('CONDENS','2010-01-01 00:00:01','Sala de condensadoras',0,'2010-01-01 00:00:01'),
		('SALACOMS','2010-01-01 00:00:01 ','Sala de comunicaciones',1,'2010-01-01 00:00:01');

		
	-- TABLA 'Llave'
	INSERT INTO Llave (Cod_Llave,F_Alta_L,Cod_Zona,F_Alta_Z,Descripcion) VALUES
		(000,'2010-01-01 00:00:01','GRUPELEC','2010-01-01 00:00:01','Puerta de acceso'),
		(001,'2010-01-01 00:00:01','GRUPELEC','2010-01-01 00:00:01','Armario técnico'),
		(002,'2010-01-01 00:00:01','GRUPELEC','2010-01-01 00:00:01','Armario principal'),
		(003,'2010-01-01 00:00:01','GRUPELEC','2010-01-01 00:00:01','Salida a patio'),
		(005,'2010-01-01 00:00:01','GRUPELEC','2010-01-01 00:00:01','Contadores'),
		(006,'2010-01-01 00:00:01','SALAMAQS','2010-01-01 00:00:01','Puerta de acceso'),
		(007,'2010-01-01 00:00:01','SALAMAQS','2010-01-01 00:00:01','Puerta 2'),
		(008,'2010-01-01 00:00:01','SALAMAQS','2010-01-01 00:00:01','Armario 1'),
		(009,'2010-01-01 00:00:01','SALAMAQS','2010-01-01 00:00:01','Armario 2'),
		(010,'2010-01-01 00:00:01','SALAMAQS','2010-01-01 00:00:01','Cuadros de fuerzas'),
		(011,'2010-01-01 00:00:01','UPS','2010-01-01 00:00:01','Puertade acceso'),
		(012,'2010-01-01 00:00:01','UPS','2010-01-01 00:00:01','Baterias'),
		(013,'2010-01-01 00:00:01','UPS','2010-01-01 00:00:01','Armarios'),
		(014,'2010-01-01 00:00:01','UPS','2010-01-01 00:00:01','Cuadros'),
		(015,'2010-01-01 00:00:01','CONDENS','2010-01-01 00:00:01','Puerta de acceso'),
		(016,'2010-01-01 00:00:01','CONDENS','2010-01-01 00:00:01','Armario 1'),
		(017,'2010-01-01 00:00:01','CONDENS','2010-01-01 00:00:01','Armario 2'),
		(018,'2010-01-01 00:00:01','CONDENS','2010-01-01 00:00:01','Mantenimiento'),
		(019,'2010-01-01 00:00:01','CONDENS','2010-01-01 00:00:01','Cuadros'),
		(020,'2010-01-01 00:00:01','CONDENS','2010-01-01 00:00:01','Cajones'),
		(021,'2010-01-01 00:00:01','SALACOMS','2010-01-01 00:00:01','Puerta de acceso'),
		(022,'2010-01-01 00:00:01','SALACOMS','2010-01-01 00:00:01','Armario 1'),
		(023,'2010-01-01 00:00:01','SALACOMS','2010-01-01 00:00:01','Armario 2'),
		(024,'2010-01-01 00:00:01','SALACOMS','2010-01-01 00:00:01','Técnicos'),
		(025,'2010-01-01 00:00:01','SALACOMS','2010-01-01 00:00:01','Servidores'),
		(026,'2010-01-01 00:00:01','SALACOMS','2010-01-01 00:00:01','Mantenimiento');

	-- TABLA 'Prestamo_Ocasional'
	INSERT INTO Prestamo_Departamento VALUES
		(006,'2010-01-01 00:00:01','CIS','VA'),
		(007,'2010-01-01 00:00:01','CIS','VA'),
		(008,'2010-01-01 00:00:01','CIS','VA'),
		(021,'2010-01-01 00:00:01','CONA','MANT'),
        (026,'2010-01-01 00:00:01','CONA','MANT'),
		(021,'2010-01-01 00:00:01','CONA','CAU'),
		(022,'2010-01-01 00:00:01','CONA','CAU'),
		(023,'2010-01-01 00:00:01','CONA','EMER'),
		(021,'2010-01-01 00:00:01','CONA','SIST'),
		(024,'2010-01-01 00:00:01','CONA','SIST'),
		(025,'2010-01-01 00:00:01','CONA','SIST'),
		(021,'2010-01-01 00:00:01','CONA','ATEC'),
		(006,'2010-01-01 00:00:01','INDR','CLYO'),
		(007,'2010-01-01 00:00:01','INDR','CLYO'),
		(008,'2010-01-01 00:00:01','INDR','CLYO'),
		(009,'2010-01-01 00:00:01','INDR','CLYO'),
		(006,'2010-01-01 00:00:01','INDR','FYPR'),
		(009,'2010-01-01 00:00:01','INDR','FYPR'),
		(021,'2010-01-01 00:00:01','INDR','CTYC'),
		(023,'2010-01-01 00:00:01','INDR','CTYC'),
		(011,'2010-01-01 00:00:01','CONT','DIRE'),
		(012,'2010-01-01 00:00:01','CONT','MARE'),
		(013,'2010-01-01 00:00:01','CONT','MARE'),
		(014,'2010-01-01 00:00:01','CONT','MARE'),
		(015,'2010-01-01 00:00:01','ELEC','CONS'),
		(016,'2010-01-01 00:00:01','ELEC','CONS'),
		(015,'2010-01-01 00:00:01','ELEC','INMA'),
		(018,'2010-01-01 00:00:01','ELEC','INMA'),
		(000,'2010-01-01 00:00:01','ELKO','INEQ'),
		(001,'2010-01-01 00:00:01','ELKO','INEQ'),
		(000,'2010-01-01 00:00:01','ELKO','MANT'),
		(001,'2010-01-01 00:00:01','ELKO','MANT'),
		(002,'2010-01-01 00:00:01','ELKO','MANT'),
		(003,'2010-01-01 00:00:01','ELKO','MANT'),
		(011,'2010-01-01 00:00:01','ELKO','EMER'),
		(013,'2010-01-01 00:00:01','ELKO','EMER'),
		(014,'2010-01-01 00:00:01','ELKO','EMER'),
		(006,'2010-01-01 00:00:01','IBM','ESAR'),
		(007,'2010-01-01 00:00:01','IBM','ESAR'),
		(008,'2010-01-01 00:00:01','IBM','ESAR'),
		(021,'2010-01-01 00:00:01','IBM','MIDD'),
		(025,'2010-01-01 00:00:01','IBM','MIDD'),
		(021,'2010-01-01 00:00:01','IBM','COIN'),
		(024,'2010-01-01 00:00:01','IBM','COIN'),
		(021,'2010-01-01 00:00:01','IBM','ALDA'),
		(022,'2010-01-01 00:00:01','IBM','ALDA'),
		(023,'2010-01-01 00:00:01','IBM','ALDA'),
		(026,'2010-01-01 00:00:01','IBM','ALDA'),
		(000,'2010-01-01 00:00:01','INGE','INMA'),
		(002,'2010-01-01 00:00:01','INGE','INMA'),
		(006,'2010-01-01 00:00:01','ISTH','DESO'),
		(011,'2010-01-01 00:00:01','LILE','LIGE'),
		(013,'2010-01-01 00:00:01','LILE','LIGE'),
		(014,'2010-01-01 00:00:01','LILE','LIGE'),
		(021,'2010-01-01 00:00:01','MEYS','TELE'),
		(025,'2010-01-01 00:00:01','MEYS','TELE'),
		(000,'2010-01-01 00:00:01','MEYS','SEIN'),
		(002,'2010-01-01 00:00:01','MEYS','SEIN'),
		(021,'2010-01-01 00:00:01','OPNA','DRED'),
		(024,'2010-01-01 00:00:01','OPNA','DRED'),
		(025,'2010-01-01 00:00:01','OPNA','DRED'),
		(026,'2010-01-01 00:00:01','OPNA','DRED'),
		(000,'2010-01-01 00:00:01','PROD','MINC'),
		(001,'2010-01-01 00:00:01','PROD','MINC'),
		(002,'2010-01-01 00:00:01','PROD','MINC'),
		(011,'2010-01-01 00:00:01','PSH','DELE'),
		(014,'2010-01-01 00:00:01','PSH','DELE'),
		(006,'2010-01-01 00:00:01','ROLL','INMA'),
		(007,'2010-01-01 00:00:01','ROLL','INMA'),
		(006,'2010-01-01 00:00:01','STOR','ANDI'),
		(007,'2010-01-01 00:00:01','STOR','ANDI'),
		(008,'2010-01-01 00:00:01','STOR','ANDI'),
		(021,'2010-01-01 00:00:01','STOR','MASE'),
		(025,'2010-01-01 00:00:01','STOR','MASE'),
		(026,'2010-01-01 00:00:01','STOR','MASE'),
		(011,'2010-01-01 00:00:01','SVS','SVIS'),
		(006,'2010-01-01 00:00:01','RAXE','DVEN'),
		(007,'2010-01-01 00:00:01','RAXE','DVEN'),
		(008,'2010-01-01 00:00:01','RAXE','DVEN'),
		(015,'2010-01-01 00:00:01','RAXE','DMAN'),
		(016,'2010-01-01 00:00:01','RAXE','DMAN'),
		(017,'2010-01-01 00:00:01','GUPO','DMAR'),
		(006,'2010-01-01 00:00:01','GUPO','DMAR');
	
	-- TABLA 'Prestamo_Ocasional'
	INSERT INTO Prestamo_Ocasional VALUES
		(017,'2010-01-01 00:00:01','00000020A','Endesa'),
		(019,'2010-01-01 00:00:01','00000020A','Endesa'),
		(011,'2010-01-01 00:00:01','00000031A','Electricidad Manuel'),
		(013,'2010-01-01 00:00:01','00000031A','Electricidad Manuel'),
		(014,'2010-01-01 00:00:01','00000031A','Electricidad Manuel'),
		(006,'2010-01-01 00:00:01','00000040A','Fontaneros S.A.'),
		(021,'2010-01-01 00:00:01','00000045A','Gobierno de Navarra'),
		(024,'2010-01-01 00:00:01','00000045A','Gobierno de Navarra'),
		(025,'2010-01-01 00:00:01','00000045A','Gobierno de Navarra'),
		(000,'2010-01-01 00:00:01','00000056A','Fontaneros S.A.');

	-- TABLA 'Almacen_Llave'
	INSERT INTO Almacen_Llave VALUES
		('A',000,'2010-01-01 00:00:01',1),
		('CC',000,'2010-01-01 00:00:01',2),
		('CC',002,'2010-01-01 00:00:01',1),
		('V',002,'2010-01-01 00:00:01',1),
		('CC',003,'2010-01-01 00:00:01',1),
		('V',003,'2010-01-01 00:00:01',1),
		('A',005,'2010-01-01 00:00:01',2),
		('CC',005,'2010-01-01 00:00:01',1),
		('V',005,'2010-01-01 00:00:01',1),
		('A',006,'2010-01-01 00:00:01',2),
		('CC',006,'2010-01-01 00:00:01',1),
		('V',006,'2010-01-01 00:00:01',1),
		('A',007,'2010-01-01 00:00:01',1),
		('V',007,'2010-01-01 00:00:01',1),
		('A',008,'2010-01-01 00:00:01',1),
		('A',009,'2010-01-01 00:00:01',1),
		('A',010,'2010-01-01 00:00:01',2),
		('V',010,'2010-01-01 00:00:01',1),
		('V',011,'2010-01-01 00:00:01',2),
		('CC',012,'2010-01-01 00:00:01',1),
		('V',012,'2010-01-01 00:00:01',1),
		('CC',013,'2010-01-01 00:00:01',1),
		('V',013,'2010-01-01 00:00:01',1),
		('CC',014,'2010-01-01 00:00:01',1),
		('V',014,'2010-01-01 00:00:01',1),
		('A',015,'2010-01-01 00:00:01',1),
		('A',016,'2010-01-01 00:00:01',1),
		('V',016,'2010-01-01 00:00:01',1),
		('A',017,'2010-01-01 00:00:01',3),
		('CC',017,'2010-01-01 00:00:01',1),
		('V',017,'2010-01-01 00:00:01',1),
		('A',018,'2010-01-01 00:00:01',2),
		('CC',018,'2010-01-01 00:00:01',1),
		('V',018,'2010-01-01 00:00:01',1),
		('A',019,'2010-01-01 00:00:01',2),
		('CC',019,'2010-01-01 00:00:01',1),
		('V',019,'2010-01-01 00:00:01',1),
		('A',020,'2010-01-01 00:00:01',2),
		('CC',020,'2010-01-01 00:00:01',1),
		('V',020,'2010-01-01 00:00:01',1),
		('A',021,'2010-01-01 00:00:01',2),
		('CC',021,'2010-01-01 00:00:01',1),
		('V',021,'2010-01-01 00:00:01',1),
		('A',022,'2010-01-01 00:00:01',2),
		('CC',022,'2010-01-01 00:00:01',1),
		('V',022,'2010-01-01 00:00:01',1),
		('A',023,'2010-01-01 00:00:01',2),
		('CC',023,'2010-01-01 00:00:01',1),
		('V',023,'2010-01-01 00:00:01',1),
		('A',024,'2010-01-01 00:00:01',2),
		('CC',024,'2010-01-01 00:00:01',1),
		('V',024,'2010-01-01 00:00:01',1),
		('A',025,'2010-01-01 00:00:01',2),
		('CC',025,'2010-01-01 00:00:01',1),
		('V',025,'2010-01-01 00:00:01',1),
		('A',026,'2010-01-01 00:00:01',2),
		('CC',026,'2010-01-01 00:00:01',1),
		('V',026,'2010-01-01 00:00:01',1);
		
	-- TABLA 'Llavero'
	INSERT INTO Llavero (Cod_Llavero,F_Alta_Ll,Descripcion) VALUES
		(000,'2010-01-01 00:00:01','Grupo Electrogeno'),
		(001,'2010-01-01 00:00:01','Sala Maquinas'),
		(002,'2010-01-01 00:00:01','UPS'),
		(003,'2010-01-01 00:00:01','Sala Comunicaciones');
		
	-- TABLA 'Almacen_Llavero'
	INSERT INTO Almacen_Llavero VALUES
		('A',000,'2010-01-01 00:00:01',2),
		('A',001,'2010-01-01 00:00:01',4),
		('CC',000,'2010-01-01 00:00:01',3),
		('CC',001,'2010-01-01 00:00:01',2),
		('CC',002,'2010-01-01 00:00:01',2),
		('V',000,'2010-01-01 00:00:01',1),
		('V',003,'2010-01-01 00:00:01',2);
		
	-- TABLA 'Composicion_Llavero'
	INSERT INTO Composicion_Llavero VALUES	
		(000,' 2010-01-01 00:00:01',000,'2010-01-01 00:00:01'),
		(000,' 2010-01-01 00:00:01',001,'2010-01-01 00:00:01'),
		(000,' 2010-01-01 00:00:01',002,'2010-01-01 00:00:01'),
		(000,' 2010-01-01 00:00:01',003,'2010-01-01 00:00:01'),
		(001,' 2010-01-01 00:00:01',006,'2010-01-01 00:00:01'),
		(001,' 2010-01-01 00:00:01',007,'2010-01-01 00:00:01'),
		(002,' 2010-01-01 00:00:01',011,'2010-01-01 00:00:01'),
		(002,' 2010-01-01 00:00:01',012,'2010-01-01 00:00:01'),
		(002,' 2010-01-01 00:00:01',013,'2010-01-01 00:00:01'),
		(002,' 2010-01-01 00:00:01',014,'2010-01-01 00:00:01'),
		(003,' 2010-01-01 00:00:01',021,'2010-01-01 00:00:01'),
		(003,' 2010-01-01 00:00:01',022,'2010-01-01 00:00:01'),
		(003,' 2010-01-01 00:00:01',023,'2010-01-01 00:00:01'),
		(003,' 2010-01-01 00:00:01',024,'2010-01-01 00:00:01'),
		(003,' 2010-01-01 00:00:01',025,'2010-01-01 00:00:01');


	-- TABLA 'Acceso'
	INSERT INTO Acceso VALUES
		('00000099A','2008-12-22 07:05','2008-12-22 08:30','Host','034','2010-01-01 00:00:01','Fibra optica','00000001A'),
		('00000010A','2008-12-22 07:30','2008-12-22 14:45','Trabajo','026','2010-01-01 00:00:01','Olvido la suya','00000001A'),
		('00000050A','2008-12-22 07:45','2008-12-22 09:05','Host','035','2010-01-01 00:00:01','Mantenimiento','00000001A'),
		('00000035A','2008-12-22 08:31','2008-12-22 11:15','Trabajo','034','2010-01-01 00:00:01','Sistemas','00000001A'),
		('00000037A','2008-12-22 08:32','2008-12-22 11:15','Trabajo','036','2010-01-01 00:00:01','Sistemas','00000001A'),
		('00000129A','2008-12-22 09:10:02','2008-12-22 13:48','Mantenimiento','035','2010-01-01 00:00:01','Revisión','00000001A'),
		('00000130A','2008-12-22 09:10:52','2008-12-22 13:48','Mantenimiento','037','2010-01-01 00:00:01','Revisión','00000001A'),
		('00000135A','2008-12-22 10:20','2008-12-22 12:50','Trabajo','038','2010-01-01 00:00:01','Impresoras','00000001A'),
		('00000118A','2008-12-22 11:40:22','2008-12-22 14:20','Sistemas','005','2010-01-01 00:00:01','Visita','00000001A'),
		('00000119A','2008-12-22 11:40:46','2008-12-22 14:25','Sistemas','036','2010-01-01 00:00:01','Visita','00000001A'),
		('00000120A','2008-12-22 11:40:57','2008-12-22 14:25','Sistemas','008','2010-01-01 00:00:01','Visita','00000001A'),
		('00000104A','2008-12-22 13:00:02','2008-12-22 14:10','Trabajo','038','2010-01-01 00:00:01','Revisión','00000001A'),
		('00000105A','2008-12-22 13:02:21','2008-12-22 14:10','Trabajo','041','2010-01-01 00:00:01','Revisión','00000001A'),
		('00000032A','2008-12-22 13:45','2008-12-22 14:10','Material informatico','010','2010-01-01 00:00:01','4 cajas','00000001A'),
		('00000086A','2008-12-22 13:50:01','2008-12-22 14:55','Trabajo','011','2010-01-01 00:00:01','Operación','00000001A'),
		('00000087A','2008-12-22 13:50:20','2008-12-22 14:55','Trabajo','012','2010-01-01 00:00:01','Operación','00000001A'),
		('00000044A','2008-12-22 15:50:29','2008-12-22 16:50','Revisión','010','2010-01-01 00:00:01','Cortadora papel','00000002A'),
		('00000148A','2008-12-22 15:55:12','2008-12-22 20:05','Trabajo','034','2010-01-01 00:00:01','Analisis','00000002A'),
		('00000149A','2008-12-22 15:55:23','2008-12-22 20:05','Trabajo','035','2010-01-01 00:00:01','Analisis','00000002A'),
		('00000150A','2008-12-22 15:55:52','2008-12-22 20:05','Trabajo','003','2010-01-01 00:00:01','Analisis','00000002A'),
		('00000114A','2008-12-22 16:02','2008-12-22 19:10','Revisión','037','2010-01-01 00:00:01','Aire Acondicionado','00000002A'),
		('00000098A','2008-12-22 18:05','2008-12-22 20:15','Host','039','2010-01-01 00:00:01','Intranet','00000002A'),
		('00000045A','2008-12-22 20:00','2008-12-22 20:20','Aprender el servicio','010','2010-01-01 00:00:01','Visita de un nuevo vigilante de seguridad para aprender el servicio','00000002A'),
		('00000160A','2008-12-23 07:05','2008-12-23 02:40','Trabajo','034','2010-01-01 00:00:01','Host','00000003A'),
		('00000006A','2008-12-22 13:00:05','2008-12-22 14:02','Trabajo','039','2010-01-01 00:00:01',null,'00000001A'),
		('00000045A','2010-04-25 13:02:55','2010-04-25 14:22:15','Trabajo','012','2010-01-01 00:00:01','Host','00000003A'),
		('00000006A','2010-04-26 09:00:03','2010-04-26 09:21:21','Trabajo','013','2010-01-01 00:00:01','Host','00000003A'),
		('00000006A','2010-06-22 08:00:00',null,'Trabajo','012','2010-01-01 00:00:01','Host','00000002A'),
		('00000169A','2010-06-22 19:23:02',null,'Trabajo','013','2010-01-01 00:00:01','Host','00000003A'),
		('00000006A','2010-06-24 08:00:00',null,'Trabajo','013','2010-01-01 00:00:01','Host','00000002A'),
		('00000097A','2010-06-24 08:00:05',null,'Host','038','2010-01-01 00:00:01','Intranet','00000002A'),
		('00000169A','2010-06-24 09:23:02',null,'Trabajo','013','2010-01-01 00:00:01','Host','00000003A'),
		('00000045A','2010-06-24 10:00:00',null,'Trabajo','010','2010-01-01 00:00:01',null,'00000002A'),
		('00000006A','2010-06-24 13:12:05',null,'Trabajo','039','2010-01-01 00:00:01',null,'00000001A'),
		('00000160A','2010-06-24 14:20:55',null,'Trabajo','034','2010-01-01 00:00:01','Host','00000003A');
		
	-- TABLA 'Llave_Prestada'
	INSERT INTO Llave_Prestada VALUES
		('2010-06-24 08:00:05','001','2010-01-01 00:00:01',False),
		('2010-06-24 08:00:05','002','2010-01-01 00:00:01',False),
		('2010-06-24 08:00:00','017','2010-01-01 00:00:01',False),
		('2010-06-24 08:00:00','018','2010-01-01 00:00:01',False),
		('2010-06-24 08:00:00','019','2010-01-01 00:00:01',False),
		('2010-06-24 08:00:00','020','2010-01-01 00:00:01',False);
		
	-- TABLA 'Llavero_Prestado'
	INSERT INTO Llavero_Prestado VALUES
		('2008-12-23 07:05','003','2010-01-01 00:00:01',True),
		('2010-06-24 08:00:05','002','2010-01-01 00:00:01',False),
		('2010-06-24 08:00:00','001','2010-01-01 00:00:01',False);
		
	-- TABLA 'Nota'
	INSERT INTO Nota VALUES
		('2008-12-04 11:30','00000018A','Informar de entrega de llaves',
			concat('Cuando el personal de cualquier expresa externa\nal centro nos soliciten llaves, debeis informarme,',
			'\nespecialmente si son llaves del centro de servidores.'),False),
		('2008-12-10 11:42','00000043A','Empresas externas',
			concat('Tamara Flores Cavido se va de vacaciones, en \ncaso de que accedan al centro personal suyo que',
			'\nnecesite autorización, avisadme a mi.'),False),
		('2008-12-11 18:38','00000043A','Alta de empresas',
			concat('Antes de dar de alta a una empresa, consúltalo conmigo.'),False),
		('2008-12-15 16:00','00000001A','Cambio de turno',
			concat('Hola, Estaría interesado en cambiar el\nturno de pasado mañana. ¿Qué te parece?'),False),
		('2008-12-16 15:14','00000003A','Actualización de personal',
			concat('Hola, Hoy ha venido la persona con DNI 00000040A.\nTras hacer las diversas comprobaciones se llega ',
			'\na la conclusión de que debería estar insertada como persona\nperiódica del departamento de Sistemas de ',
			'\nConasa. Por favor, borrala de ocasional e introducela\nen la tabla que le corresponde. Muchas gracias.'),False);
			
	-- TABLA 'Destinatario'
	INSERT INTO Destinatario VALUES
		('2008-12-04 11:30','00000018A','00000001A',False, False),
		('2008-12-04 11:30','00000018A','00000002A',False, False),
		('2008-12-04 11:30','00000018A','00000003A',False, False),
		('2008-12-10 11:42','00000043A','00000001A',False, False),
		('2008-12-10 11:42','00000043A','00000002A',False, False),
		('2008-12-10 11:42','00000043A','00000003A',False, False),
		('2008-12-11 18:38','00000043A','00000028A',True, True),
		('2008-12-15 16:00','00000001A','00000002A',False, False),
		('2008-12-16 15:14','00000003A','00000000A',False, False);
		
	-- TABLA 'Codigo'
	INSERT INTO Codigo VALUES
		(0),(1),(2),(3),(4),(5),(6),(7),(8),(9),(10),(11),(12),(13),(14),(15),(16),(17),(18),(19),(20),
		(21),(22),(23),(24),(25),(26),(27),(28),(29) ,(30),(31),(32),(33),(34),(35),(36),(37),(38),(39),(40),
		(41),(42),(43),(44),(45),(46),(47),(48),(49) ,(50),(51),(52),(53),(54),(55),(56),(57),(58),(59),(60),
		(61),(62),(63),(64),(65),(66),(67),(68),(69) ,(70),(71),(72),(73),(74),(75),(76),(77),(78),(79),(80),
		(81),(82),(83),(84),(85),(86),(87),(88),(89) ,(90),(91),(92),(93),(94),(95),(96),(97),(98),(99),(100),
		(101),(102),(103),(104),(105),(106),(107),(108),(109),(110),(111),(112),(113),(114),(115),(116),(117),(118),(119),(120),
		(121),(122),(123),(124),(125),(126),(127),(128),(129),(130),(131),(132),(133),(134),(135),(136),(137),(138),(139),(140),
		(141),(142),(143),(144),(145),(146),(147),(148),(149),(150),(151),(152),(153),(154),(155),(156),(157),(158),(159),(160),
		(161),(162),(163),(164),(165),(166),(167),(168),(169),(170),(171),(172),(173),(174),(175),(176),(177),(178),(179),(180),
		(181),(182),(183),(184),(185),(186),(187),(188),(189),(190),(191),(192),(193),(194),(195),(196),(197),(198),(199),(200),
		(201),(202),(203),(204),(205),(206),(207),(208),(209),(210),(211),(212),(213),(214),(215),(216),(217),(218),(219),(220),
		(221),(222),(223),(224),(225),(226),(227),(228),(229),(230),(231),(232),(233),(234),(235),(236),(237),(238),(239),(240),
		(241),(242),(243),(244),(245),(246),(247),(248),(249),(250),(251),(252),(253),(254),(255),(256),(257),(258),(259),(260),
		(261),(262),(263),(264),(265),(266),(267),(268),(269),(270),(271),(272),(273),(274),(275),(276),(277),(278),(279),(280),
		(281),(282),(283),(284),(285),(286),(287),(288),(289),(290),(291),(292),(293),(294),(295),(296),(297),(298),(299),(300),
		(301),(302),(303),(304),(305),(306),(307),(308),(309),(310),(311),(312),(313),(314),(315),(316),(317),(318),(319),(320),
		(321),(322),(323),(324),(325),(326),(327),(328),(329),(330),(331),(332),(333),(334),(335),(336),(337),(338),(339),(340),
		(341),(342),(343),(344),(345),(346),(347),(348),(349),(350),(351),(352),(353),(354),(355),(356),(357),(358),(359),(360),
		(361),(362),(363),(364),(365),(366),(367),(368),(369),(370),(371),(372),(373),(374),(375),(376),(377),(378),(379),(380),
		(381),(382),(383),(384),(385),(386),(387),(388),(389),(390),(391),(392),(393),(394),(395),(396),(397),(398),(399),(400),
		(401),(402),(403),(404),(405),(406),(407),(408),(409),(410),(411),(412),(413),(414),(415),(416),(417),(418),(419),(420),
		(421),(422),(423),(424),(425),(426),(427),(428),(429),(430),(431),(432),(433),(434),(435),(436),(437),(438),(439),(440),
		(441),(442),(443),(444),(445),(446),(447),(448),(449),(450),(451),(452),(453),(454),(455),(456),(457),(458),(459),(460),
		(461),(462),(463),(464),(465),(466),(467),(468),(469),(470),(471),(472),(473),(474),(475),(476),(477),(478),(479),(480),
		(481),(482),(483),(484),(485),(486),(487),(488),(489),(490),(491),(492),(493),(494),(495),(496),(497),(498),(499),(500),
		(501),(502),(503),(504),(505),(506),(507),(508),(509),(510),(511),(512),(513),(514),(515),(516),(517),(518),(519),(520),
		(521),(522),(523),(524),(525),(526),(527),(528),(529),(530),(531),(532),(533),(534),(535),(536),(537),(538),(539),(540),
		(541),(542),(543),(544),(545),(546),(547),(548),(549),(550),(551),(552),(553),(554),(555),(556),(557),(558),(559),(560),
		(561),(562),(563),(564),(565),(566),(567),(568),(569),(570),(571),(572),(573),(574),(575),(576),(577),(578),(579),(580),
		(581),(582),(583),(584),(585),(586),(587),(588),(589),(590),(591),(592),(593),(594),(595),(596),(597),(598),(599),(600),
		(601),(602),(603),(604),(605),(606),(607),(608),(609),(610),(611),(612),(613),(614),(615),(616),(617),(618),(619),(620),
		(621),(622),(623),(624),(625),(626),(627),(628),(629),(630),(631),(632),(633),(634),(635),(636),(637),(638),(639),(640),
		(641),(642),(643),(644),(645),(646),(647),(648),(649),(650),(651),(652),(653),(654),(655),(656),(657),(658),(659),(660),
		(661),(662),(663),(664),(665),(666),(667),(668),(669),(670),(671),(672),(673),(674),(675),(676),(677),(678),(679),(680),
		(681),(682),(683),(684),(685),(686),(687),(688),(689),(690),(691),(692),(693),(694),(695),(696),(697),(698),(699),(700),
		(701),(702),(703),(704),(705),(706),(707),(708),(709),(710),(711),(712),(713),(714),(715),(716),(717),(718),(719),(720),
		(721),(722),(723),(724),(725),(726),(727),(728),(729),(730),(731),(732),(733),(734),(735),(736),(737),(738),(739),(740),
		(741),(742),(743),(744),(745),(746),(747),(748),(749),(750),(751),(752),(753),(754),(755),(756),(757),(758),(759),(760),
		(761),(762),(763),(764),(765),(766),(767),(768),(769),(770),(771),(772),(773),(774),(775),(776),(777),(778),(779),(780),
		(781),(782),(783),(784),(785),(786),(787),(788),(789),(790),(791),(792),(793),(794),(795),(796),(797),(798),(799),(800),
		(801),(802),(803),(804),(805),(806),(807),(808),(809),(810),(811),(812),(813),(814),(815),(816),(817),(818),(819),(820),
		(821),(822),(823),(824),(825),(826),(827),(828),(829),(830),(831),(832),(833),(834),(835),(836),(837),(838),(839),(840),
		(841),(842),(843),(844),(845),(846),(847),(848),(849),(850),(851),(852),(853),(854),(855),(856),(857),(858),(859),(860),
		(861),(862),(863),(864),(865),(866),(867),(868),(869),(870),(871),(872),(873),(874),(875),(876),(877),(878),(879),(880),
		(881),(882),(883),(884),(885),(886),(887),(888),(889),(890),(891),(892),(893),(894),(895),(896),(897),(898),(899),(900),
		(901),(902),(903),(904),(905),(906),(907),(908),(909),(910),(911),(912),(913),(914),(915),(916),(917),(918),(919),(920),
		(921),(922),(923),(924),(925),(926),(927),(928),(929),(930),(931),(932),(933),(934),(935),(936),(937),(938),(939),(940),
		(941),(942),(943),(944),(945),(946),(947),(948),(949),(950),(951),(952),(953),(954),(955),(956),(957),(958),(959),(960),
		(961),(962),(963),(964),(965),(966),(967),(968),(969),(970),(971),(972),(973),(974),(975),(976),(977),(978),(979),(980),
		(981),(982),(983),(984),(985),(986),(987),(988),(989),(990),(991),(992),(993),(994),(995),(996),(997),(998),(999);

-- *********************************************************************************************************************
-- *********************************************************************************************************************




-- *********************************************************************************************************************
-- ******** DISPARADORES																					************
-- *********************************************************************************************************************
	
	CREATE TRIGGER borr_col
		AFTER DELETE ON Color FOR EACH ROW
			DELETE FROM Nivel WHERE Cod_Niv NOT IN (SELECT Cod_Niv FROM Color);
			
			
	-- Tabla 'Almacen_Llave'
	CREATE TRIGGER borr_al
		AFTER DELETE ON Almacen_Llave FOR EACH ROW
			DELETE FROM Llave WHERE Cod_Llave NOT IN (	SELECT DISTINCT Cod_Llave FROM Composicion_Llavero
															UNION
														SELECT DISTINCT Cod_Llave FROM Almacen_Llave);


	-- Tabla 'Cuenta'
	CREATE TRIGGER borr_Cuenta
		AFTER DELETE ON Cuenta FOR EACH ROW
			DELETE FROM T_Usuario WHERE Tipo NOT IN (SELECT DISTINCT Tipo FROM Cuenta);
			
			
	-- Tabla 'Acceso'
	CREATE TRIGGER ins_acc
		AFTER INSERT ON Acceso FOR EACH ROW
			UPDATE Tarjeta SET DNI = NEW.DNI WHERE Cod_Tarj = NEW.Cod_Tarj;
	
-- *********************************************************************************************************************
-- *********************************************************************************************************************