// PAQUETE
package GUI.UIs;

// IMPORTACIONES
import API.PDFs.GeneradorInformes;
import API.Tablas.*;
import API.Utilidades.*;
import API.Correos.Cartero;
import API.Excepciones.DataException;
import java.awt.Color;
import java.io.File;
import java.io.FileReader;
import java.io.PrintWriter;
import java.sql.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;
import java.rmi.RemoteException;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;



/** CLASE APLICACION:  Clase que representa el JFrame desde el que el usuario accedera a toda la funcionalidad de la aplicacion
 *
 * @author  Napal Basterrica, Hector
 * @version 1.0
 *
 */
public class Aplicacion extends JFrame
{
	// ATRIBUTOS -----------------------------------------------------------
        // GENERALES
        /* Usuario:                         Nombre de usuario */
        private static String               Usuario;
        /* Tipo_Usuario:                    Tipo de usuario */
        private static String               Tipo_Usuario;
        /* DNI_Usuario:                     DNI del usuario */
        private static String               DNI_Usuario;


        // TABS
        // -> 'ACCESOS'
        /* DNI_Usu:                         DNI del usuario */
        private String                      DNI_Usu;
        /* HashMap:                         HashMap con las llaves y llaveros a prestar */
        private HashMap<String,GLL_Dev>     HM_GLLDs = new HashMap();

        // -> 'EMPRESAS Y PERSONAL'
        /* DNI_Personal:                    DNI de la persona seleccionada */
        private String                      DNI_Personal;

        // -> 'PRESTAMOS'
        /* DNI_Prest:                       DNI del prestatrio */
        private String                      DNI_Prest;

        // -> 'PARTES DE SEGURIDAD'
        /* Tipo_Usu:                        Tipo de usuario */
        private String                      Tipo_Usu;
        /* Fecha:                           Fecha del parte de seguridad editable */
        private String                      Fecha_Parte;

        // CORREOS
        /* Cartero:                         Objeto de tipo Cartero empleado en el envio de E-Mails */
        private Cartero                     Cartero;


        // UTILIDADES
        /* UGUI:                            Utilidades para la interfaz */
        private UtilidadesGUI               UGUI = new UtilidadesGUI();
        /* UBD:                             Utilidades para realizar consultas a la BD */
        private UtilidadesBD                UBD = new UtilidadesBD();
        /* UCAL:                            Utilidades para el manejo de fechar */
        private UtilidadesCalendar          UCAL = new UtilidadesCalendar();
        


	// METODOS -------------------------------------------------------------
	// CONSTRUCTORES
	/** Constructor: Constructor de clase que crea el objeto Aplicacion con los valores pasados parametricamente
         *
         * @param   String      Usuario         Usuario logueado en el sistema
         * @param   String      Tipo_Usuario    Tipo de usuario logueado en el sistema
         * @para    String      DNI_Usuario     DNI del usuario logueado en el sistema
         * @return  JFrame      Aplicacion      Objeto de tipo Aplicacion
         */
        public Aplicacion(String Usuario, String Tipo_Usuario, String DNI_Usuario)
        {
            try
            {
                // Dibujamos todos los componentes
                initComponents();

                // Establecemos el valor de los atributos
                this.Usuario = Usuario;
                this.DNI_Usu = DNI_Usuario;
                this.Tipo_Usu = Tipo_Usuario;
                this.Cartero = new Cartero("smtp.gmail.com","true","587","informaticanavarrasa@gmail.com","true","informaticanavarrasa@gmail.com","informaticanavarra");

                // Registramos el acceso en el sistema
                Acceso_Sistema as = new Acceso_Sistema(new Cuenta(DNI_Usuario,Tipo_Usuario),Calendar.getInstance());
                as.insertAccesoSistemaBD();

                // Si se trata del primer vigilante que accede al sistema en el dia de hoy
                if (as.primerVigilante())
                {
                    // Actualizamos la tabla Acceso, asigando salida a todos los accesos que no la posean
                    // mantenerAcceso();
                    // Enviamos los E-Mails oportunos: Peticiones de devolucion de prestamos y actualizacion de personal
                    // enviarEmails();
                    // Eliminamos todos los datos antiguos / en desuso
                    // eliminarDatosAntiguos(365);
                }
                // Si se trata de un responsable de personal
                else if (as.getCuenta().getTipo().getTipo().equals("RP"))
                {
                    // Actualizamos la fecha de ultima actualizacion a la de hoy
                    Periodico_Contacto pc = new Periodico_Contacto(as.getCuenta().getPersona().getDNI());
                    pc.actualizarFUA();
                }
                

                // Inicializamos todos los datos
                iniciarDatos();

                // Pintamos los componentes necesarios
                pintar();

                // Borramos todos los componentes graficos innecesarios segun el tipo de usuario
                visualizacion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible iniciar correctamente la aplicación.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }


	// DIBUJO
        /** initComponents: Metodo que dibuja toda la GUI de JDCuenta
	 *
	 * @param   void
	 * @return  void
	 */
        @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        BGEntradaSalida = new javax.swing.ButtonGroup();
        JTPFuncionalidades = new javax.swing.JTabbedPane();
        JPAccesos = new javax.swing.JPanel();
        JSPAccesos = new javax.swing.JScrollPane();
        JTAccesos = new javax.swing.JTable();
        JCCFechaInic = new org.freixas.jcalendar.JCalendarCombo();
        JCCFechaFin = new org.freixas.jcalendar.JCalendarCombo();
        JCBTodoDia = new javax.swing.JCheckBox();
        JCBHoraInic = new javax.swing.JComboBox();
        JCBHoraFin = new javax.swing.JComboBox();
        JRBEntrada = new javax.swing.JRadioButton();
        JRSalida = new javax.swing.JRadioButton();
        JPDatosPersonales = new javax.swing.JPanel();
        JLDNI = new javax.swing.JLabel();
        JLApellido1 = new javax.swing.JLabel();
        JLApellido2 = new javax.swing.JLabel();
        JLNombre = new javax.swing.JLabel();
        JSPMotivo = new javax.swing.JScrollPane();
        JTAMotivo = new javax.swing.JTextArea();
        JLMotivo = new javax.swing.JLabel();
        JLOtrosDatos = new javax.swing.JLabel();
        JSPOtrosDatos = new javax.swing.JScrollPane();
        JTAOtrosDatos = new javax.swing.JTextArea();
        JTFDNI = new javax.swing.JTextField();
        JTFApellido1 = new javax.swing.JTextField();
        JTFApellido2 = new javax.swing.JTextField();
        JTFNombre = new javax.swing.JTextField();
        JPAccesosB1 = new javax.swing.JPanel();
        JBAnadirPersona = new javax.swing.JButton();
        JBEliminarAcceso = new javax.swing.JButton();
        JBAnadirAcceso = new javax.swing.JButton();
        JBDarSalidaConDev = new javax.swing.JButton();
        JBSalida = new javax.swing.JButton();
        JPTarjeta = new javax.swing.JPanel();
        JTFCodTarj = new javax.swing.JTextField();
        JPColorTarjeta = new javax.swing.JPanel();
        JCBTarjeta = new javax.swing.JCheckBox();
        JTFBuscar = new javax.swing.JTextField();
        JPLlsLLls = new javax.swing.JPanel();
        JSPLLs = new javax.swing.JScrollPane();
        JTLsLls = new javax.swing.JTable();
        JPAccesosB2 = new javax.swing.JPanel();
        JBAnadirPrestamo = new javax.swing.JButton();
        JBEliminarPrestamo = new javax.swing.JButton();
        JBDevolverPrestamo = new javax.swing.JButton();
        JPAvisoTfno = new javax.swing.JPanel();
        JTFAvisoTfno = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        JTTelefonos = new javax.swing.JTable();
        JBPDFAccs = new javax.swing.JButton();
        JPEmpresasPersonal = new javax.swing.JPanel();
        JPEmpresas = new javax.swing.JPanel();
        JSPEmpresas = new javax.swing.JScrollPane();
        JTEmpresas = new javax.swing.JTable();
        JPEmpresasD = new javax.swing.JPanel();
        JLCodEmp = new javax.swing.JLabel();
        JTFCodEmp = new javax.swing.JTextField();
        JLCIFEmp = new javax.swing.JLabel();
        JTFCIFEmp = new javax.swing.JTextField();
        JLNomEmp = new javax.swing.JLabel();
        JTFNomEmp = new javax.swing.JTextField();
        JLDirEmp = new javax.swing.JLabel();
        jScrollPane19 = new javax.swing.JScrollPane();
        JTADirEmp = new javax.swing.JTextArea();
        jPanel6 = new javax.swing.JPanel();
        JBModificarEmpresa = new javax.swing.JButton();
        JTFNombreEmp = new javax.swing.JTextField();
        JTFCIF = new javax.swing.JTextField();
        JPEmpresasB = new javax.swing.JPanel();
        JTBAnadirEmpresa = new javax.swing.JToggleButton();
        JBEliminarEmpresa = new javax.swing.JButton();
        JTBModificarEmpresa = new javax.swing.JToggleButton();
        JPDepartamentos = new javax.swing.JPanel();
        JSPDepartamentos = new javax.swing.JScrollPane();
        JTDepartamentos = new javax.swing.JTable();
        JTFNombreDep = new javax.swing.JTextField();
        JTFActividad = new javax.swing.JTextField();
        JPDepartamentosB = new javax.swing.JPanel();
        JBEliminarDepartamento = new javax.swing.JButton();
        JTBAnadirDepartamento = new javax.swing.JToggleButton();
        JTBModificarDepartamento = new javax.swing.JToggleButton();
        JPDepartamentosD = new javax.swing.JPanel();
        JLCodDep = new javax.swing.JLabel();
        JTFCodDep = new javax.swing.JTextField();
        JLNomDep = new javax.swing.JLabel();
        JTFNomDep = new javax.swing.JTextField();
        JLCodAcc = new javax.swing.JLabel();
        JLPeriodicidad = new javax.swing.JLabel();
        JTFPeriodicidad = new javax.swing.JTextField();
        JCBActividad = new javax.swing.JComboBox();
        JBModificarEmpresa1 = new javax.swing.JButton();
        JPJCBDNIDep = new javax.swing.JPanel();
        JCBDNIDep = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        JLFUltActu = new javax.swing.JLabel();
        JCCFUltActu = new org.freixas.jcalendar.JCalendarCombo();
        jPanel1 = new javax.swing.JPanel();
        JBEliminarPrestDep = new javax.swing.JButton();
        JBAnadirPrestDep = new javax.swing.JButton();
        jScrollPane18 = new javax.swing.JScrollPane();
        JTLsLlsPers = new javax.swing.JTable();
        JPPersonal = new javax.swing.JPanel();
        JTPDepartamentos = new javax.swing.JTabbedPane();
        JPEmpleados = new javax.swing.JPanel();
        JSPEmpleados = new javax.swing.JScrollPane();
        JTEmpleados = new javax.swing.JTable();
        JPContactos = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        JTContactos = new javax.swing.JTable();
        jScrollPane16 = new javax.swing.JScrollPane();
        JTCargos = new javax.swing.JTable();
        JPElimC = new javax.swing.JPanel();
        JCBCargos = new javax.swing.JComboBox();
        JEliminarCargo = new javax.swing.JButton();
        jScrollPane14 = new javax.swing.JScrollPane();
        JTEmails = new javax.swing.JTable();
        jScrollPane15 = new javax.swing.JScrollPane();
        JTTfns = new javax.swing.JTable();
        JTFFiltro = new javax.swing.JTextField();
        JPPersonalB = new javax.swing.JPanel();
        JTBAnadirPerContac = new javax.swing.JToggleButton();
        JTBModificarPerContac = new javax.swing.JToggleButton();
        JBEliminarPerContac = new javax.swing.JButton();
        JPDP = new javax.swing.JPanel();
        JLDNIDP = new javax.swing.JLabel();
        JLApellido1DP = new javax.swing.JLabel();
        JLApellido2DP = new javax.swing.JLabel();
        JLNombreDP = new javax.swing.JLabel();
        JTFApellido2DP = new javax.swing.JTextField();
        JTFApellido1DP = new javax.swing.JTextField();
        JTFDNIDP = new javax.swing.JTextField();
        JTFNombreDP = new javax.swing.JTextField();
        JLTfnoDP = new javax.swing.JLabel();
        JTFTfnoEmailDP = new javax.swing.JTextField();
        JBGrabarPerContac = new javax.swing.JButton();
        JBAnadirTfno = new javax.swing.JButton();
        JBAnadirEmail = new javax.swing.JButton();
        JPDNIAuto = new javax.swing.JPanel();
        JCBOcasional = new javax.swing.JComboBox();
        JLDNIOcas = new javax.swing.JLabel();
        JPResidente = new javax.swing.JPanel();
        JLNivel = new javax.swing.JLabel();
        JCBNivel = new javax.swing.JComboBox();
        JPVigilante = new javax.swing.JPanel();
        JLCodV = new javax.swing.JLabel();
        JTFCodV = new javax.swing.JTextField();
        JPOcasional = new javax.swing.JPanel();
        JLEmpresas = new javax.swing.JLabel();
        JLActiv = new javax.swing.JLabel();
        JLFCaducidad = new javax.swing.JLabel();
        JLNotas = new javax.swing.JLabel();
        JCCFCaducidad = new org.freixas.jcalendar.JCalendarCombo();
        jScrollPane17 = new javax.swing.JScrollPane();
        JTANotas = new javax.swing.JTextArea();
        JCBEmpresas = new javax.swing.JComboBox();
        JCBActividad1 = new javax.swing.JComboBox();
        jScrollPane38 = new javax.swing.JScrollPane();
        JTLsLlsPers1 = new javax.swing.JTable();
        jPanel5 = new javax.swing.JPanel();
        JBAnadirPrestDep1 = new javax.swing.JButton();
        JBEliminarPrestDep1 = new javax.swing.JButton();
        JPElimTE = new javax.swing.JPanel();
        JBEliminarTfno = new javax.swing.JButton();
        JEliminarEmail = new javax.swing.JButton();
        JBMostrarTodo = new javax.swing.JButton();
        JPPrestsPDF2 = new javax.swing.JPanel();
        JBPrestsPDF2 = new javax.swing.JButton();
        JLPrestTablas2 = new javax.swing.JLabel();
        JCBPDFTodas2 = new javax.swing.JCheckBox();
        JCBPDFAlms2 = new javax.swing.JCheckBox();
        JCBPDFLs2 = new javax.swing.JCheckBox();
        JCBPDFLls2 = new javax.swing.JCheckBox();
        JTBActivarPanelPDF2 = new javax.swing.JToggleButton();
        JPTACs = new javax.swing.JPanel();
        JPTarjetas = new javax.swing.JPanel();
        jScrollPane25 = new javax.swing.JScrollPane();
        JTTarjetas = new javax.swing.JTable();
        JPTarjetasB = new javax.swing.JPanel();
        JBEliminarTarj = new javax.swing.JButton();
        JTBAnadirTarjeta = new javax.swing.JToggleButton();
        JBBajaTarj = new javax.swing.JButton();
        JTBModificarTarj = new javax.swing.JToggleButton();
        JPTarjetasD = new javax.swing.JPanel();
        JPTarjetaD = new javax.swing.JPanel();
        JTFCodTarjD = new javax.swing.JTextField();
        JBGrabarTarjeta = new javax.swing.JButton();
        JPColores = new javax.swing.JPanel();
        jScrollPane24 = new javax.swing.JScrollPane();
        JTColores = new javax.swing.JTable();
        JPColorD = new javax.swing.JPanel();
        JBColor = new javax.swing.JButton();
        JTFColorNiv = new javax.swing.JTextField();
        JBGrabarColor = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        JTBAnadirColor = new javax.swing.JToggleButton();
        JBEliminarColor = new javax.swing.JButton();
        JBBajaColor = new javax.swing.JButton();
        JTBModificarColor = new javax.swing.JToggleButton();
        JPNiveles = new javax.swing.JPanel();
        jScrollPane23 = new javax.swing.JScrollPane();
        JTNiveles = new javax.swing.JTable();
        JPNivelD = new javax.swing.JPanel();
        JLCodNiv = new javax.swing.JLabel();
        JLDescripNiv = new javax.swing.JLabel();
        JTFDescripNiv = new javax.swing.JTextField();
        JTFCodNiv = new javax.swing.JTextField();
        JBGrabarNivel = new javax.swing.JButton();
        JPNivelesB = new javax.swing.JPanel();
        JBEliminarNivel = new javax.swing.JButton();
        JTBAnadirNivel = new javax.swing.JToggleButton();
        JTBModificarNivel = new javax.swing.JToggleButton();
        JBBajaNivel = new javax.swing.JButton();
        JPTAccesos = new javax.swing.JPanel();
        jScrollPane35 = new javax.swing.JScrollPane();
        JTTAccesos = new javax.swing.JTable();
        JPTAccesoD = new javax.swing.JPanel();
        JLCodTAcceso = new javax.swing.JLabel();
        JLActivTAcceso = new javax.swing.JLabel();
        JTFActivTAcceso = new javax.swing.JTextField();
        JTFCodTAcceso = new javax.swing.JTextField();
        JBGrabarNivel1 = new javax.swing.JButton();
        JLDiasTAcceso = new javax.swing.JLabel();
        JLHorasTAcceso = new javax.swing.JLabel();
        JTFDiasTAcceso = new javax.swing.JTextField();
        JCBHITAccesos = new javax.swing.JComboBox();
        JCBHFTAccesos = new javax.swing.JComboBox();
        JCBAutTelTAccesos = new javax.swing.JCheckBox();
        JCBNivelTAcceso = new javax.swing.JComboBox();
        JLNivTAcceso = new javax.swing.JLabel();
        JPTAccesoB = new javax.swing.JPanel();
        JBEliminarTAcceso = new javax.swing.JButton();
        JTBAnadirTAcceso = new javax.swing.JToggleButton();
        JTBModificarTAcceso = new javax.swing.JToggleButton();
        JPCargos = new javax.swing.JPanel();
        jScrollPane37 = new javax.swing.JScrollPane();
        JTCargoss = new javax.swing.JTable();
        JPCargoD = new javax.swing.JPanel();
        JLCodNiv1 = new javax.swing.JLabel();
        JLDescripNiv1 = new javax.swing.JLabel();
        JTFDescripCargo = new javax.swing.JTextField();
        JTFCodCargo = new javax.swing.JTextField();
        JBGrabarCargo = new javax.swing.JButton();
        JPCargoss = new javax.swing.JPanel();
        JBEliminarCargo = new javax.swing.JButton();
        JTBAnadirCargo = new javax.swing.JToggleButton();
        JTBModificarCargo = new javax.swing.JToggleButton();
        JPZsLsLls = new javax.swing.JPanel();
        JPZona = new javax.swing.JPanel();
        jScrollPane26 = new javax.swing.JScrollPane();
        JTLsZona = new javax.swing.JTable();
        jScrollPane27 = new javax.swing.JScrollPane();
        JTLsPrestEmpDep = new javax.swing.JTable();
        jScrollPane28 = new javax.swing.JScrollPane();
        JTLsPrestOcas = new javax.swing.JTable();
        JPLlaveB = new javax.swing.JPanel();
        JBEliminarL = new javax.swing.JButton();
        JTBAnadirL = new javax.swing.JToggleButton();
        JTBModificarLlave = new javax.swing.JToggleButton();
        JBDarBajaL = new javax.swing.JButton();
        JPLsPrestB = new javax.swing.JPanel();
        JBAnadirLPrestEmpDep = new javax.swing.JButton();
        JBEliminarLPrestEmpDep = new javax.swing.JButton();
        JBAnadirLPrestOcas = new javax.swing.JButton();
        JBEliminarLPrestOcas = new javax.swing.JButton();
        jScrollPane29 = new javax.swing.JScrollPane();
        JTLsPrestEmpDepResto = new javax.swing.JTable();
        jScrollPane30 = new javax.swing.JScrollPane();
        JTLsPrestOcasResto = new javax.swing.JTable();
        JPLlaveD = new javax.swing.JPanel();
        JLCodL = new javax.swing.JLabel();
        JLDescripL = new javax.swing.JLabel();
        JLNotasL = new javax.swing.JLabel();
        JTFDescripL = new javax.swing.JTextField();
        JTFNotasL = new javax.swing.JTextField();
        JTFCodL = new javax.swing.JTextField();
        JBGrabarL = new javax.swing.JButton();
        JBAnadirAlmLl = new javax.swing.JButton();
        JPZonas = new javax.swing.JPanel();
        jScrollPane13 = new javax.swing.JScrollPane();
        JTZonas = new javax.swing.JTable();
        JPZonaB = new javax.swing.JPanel();
        JBEliminarZona = new javax.swing.JButton();
        JTBModificarZona = new javax.swing.JToggleButton();
        JTBAnadirZona = new javax.swing.JToggleButton();
        JPZonaD = new javax.swing.JPanel();
        JCBNivelZona = new javax.swing.JComboBox();
        JLNivelZona = new javax.swing.JLabel();
        JLDescripZona = new javax.swing.JLabel();
        JTFDescripZona = new javax.swing.JTextField();
        JTFCodZona = new javax.swing.JTextField();
        JLCodZona = new javax.swing.JLabel();
        JBGrabarZona = new javax.swing.JButton();
        JPAlmacenes = new javax.swing.JPanel();
        jScrollPane31 = new javax.swing.JScrollPane();
        JTAlms = new javax.swing.JTable();
        jPanel3 = new javax.swing.JPanel();
        JTBAnadirAlm = new javax.swing.JToggleButton();
        JBEliminarAlm = new javax.swing.JButton();
        JTBModificarAlm = new javax.swing.JToggleButton();
        JPAlmD = new javax.swing.JPanel();
        JLCodAlm = new javax.swing.JLabel();
        JLDescripAlm = new javax.swing.JLabel();
        JTFDescripAlm = new javax.swing.JTextField();
        JTFCodAlm = new javax.swing.JTextField();
        JBGrabarAlm = new javax.swing.JButton();
        jScrollPane33 = new javax.swing.JScrollPane();
        JTLls = new javax.swing.JTable();
        jScrollPane32 = new javax.swing.JScrollPane();
        JTLs = new javax.swing.JTable();
        jPanel4 = new javax.swing.JPanel();
        JTBAnadirLl = new javax.swing.JToggleButton();
        JTBModificarLl = new javax.swing.JToggleButton();
        JBDarBajaLl = new javax.swing.JButton();
        JBEliminarLl = new javax.swing.JButton();
        JPLlaveroD = new javax.swing.JPanel();
        JLCodLl = new javax.swing.JLabel();
        JLDescripLl = new javax.swing.JLabel();
        JTFDescripLl = new javax.swing.JTextField();
        JTFCodLl = new javax.swing.JTextField();
        JBGrabarLl = new javax.swing.JButton();
        JBAnadirLsLl = new javax.swing.JButton();
        JBEliminarLsLl = new javax.swing.JButton();
        JPAlmsB = new javax.swing.JPanel();
        JTBNumCopsL = new javax.swing.JToggleButton();
        JPJTFNumCopsL = new javax.swing.JPanel();
        JTFNumCopsL = new javax.swing.JTextField();
        JBGrabarAlmL = new javax.swing.JButton();
        JBEliminarAlmL = new javax.swing.JButton();
        JTBAnadirAlmL = new javax.swing.JToggleButton();
        JTBAnadirAlmLlB = new javax.swing.JToggleButton();
        JBEliminarAlmLl = new javax.swing.JButton();
        JTBNumCopsLl = new javax.swing.JToggleButton();
        JPJTFNumCopsLl = new javax.swing.JPanel();
        JTFNumCopsLl = new javax.swing.JTextField();
        JBGrabarAlmLl = new javax.swing.JButton();
        JPLls = new javax.swing.JPanel();
        JCBLlaveros = new javax.swing.JComboBox();
        JTFBuscarLLl = new javax.swing.JTextField();
        JPPrestsPDF1 = new javax.swing.JPanel();
        JBPrestsPDF1 = new javax.swing.JButton();
        JLPrestTablas1 = new javax.swing.JLabel();
        JCBPDFTodas1 = new javax.swing.JCheckBox();
        JCBPDFAlms1 = new javax.swing.JCheckBox();
        JCBPDFLs1 = new javax.swing.JCheckBox();
        JCBPDFLls1 = new javax.swing.JCheckBox();
        JTBActivarPanelPDF1 = new javax.swing.JToggleButton();
        JPPrestamos = new javax.swing.JPanel();
        JPTarjsPrest = new javax.swing.JPanel();
        jScrollPane20 = new javax.swing.JScrollPane();
        JTTarjsPrest = new javax.swing.JTable();
        JPLsLlsPrest = new javax.swing.JPanel();
        jScrollPane21 = new javax.swing.JScrollPane();
        JTLsPrest = new javax.swing.JTable();
        jScrollPane22 = new javax.swing.JScrollPane();
        JTLlsPrest = new javax.swing.JTable();
        JPDatosPrest = new javax.swing.JPanel();
        jScrollPane34 = new javax.swing.JScrollPane();
        JTTfnsPrest = new javax.swing.JTable();
        JBRecordPrest = new javax.swing.JButton();
        JPDPPrest = new javax.swing.JPanel();
        JLNombrePrest = new javax.swing.JLabel();
        JTFNombrePrest = new javax.swing.JTextField();
        JLEmpresaPrest = new javax.swing.JLabel();
        JTFEmpresaPrest = new javax.swing.JTextField();
        JLTipoPrest = new javax.swing.JLabel();
        JTFTipoPrest = new javax.swing.JTextField();
        JBDevolver = new javax.swing.JButton();
        JPPrestsPDF = new javax.swing.JPanel();
        JBPrestsPDF = new javax.swing.JButton();
        JLPrestTablas = new javax.swing.JLabel();
        JCBPDFTodas = new javax.swing.JCheckBox();
        JCBPDFTarjs = new javax.swing.JCheckBox();
        JCBPDFLs = new javax.swing.JCheckBox();
        JCBPDFLls = new javax.swing.JCheckBox();
        JTBActivarPanelPDF = new javax.swing.JToggleButton();
        JBDeseleccionar = new javax.swing.JButton();
        JPCuentas = new javax.swing.JPanel();
        JPInfoCuentas = new javax.swing.JPanel();
        jScrollPane5 = new javax.swing.JScrollPane();
        JTCuentas = new javax.swing.JTable();
        JPDatosCuenta = new javax.swing.JPanel();
        JLDNICuenta = new javax.swing.JLabel();
        JCBDNICuenta = new javax.swing.JComboBox();
        JCBTipoCuenta = new javax.swing.JComboBox();
        JTFUsuarioCuenta = new javax.swing.JTextField();
        JTFPassCuenta = new javax.swing.JTextField();
        JLTipoCuenta = new javax.swing.JLabel();
        JLUsuarioCuenta = new javax.swing.JLabel();
        JLPassCuenta = new javax.swing.JLabel();
        JBGrabarCuenta = new javax.swing.JButton();
        JTFBuscarCuenta = new javax.swing.JTextField();
        JPCuentasB = new javax.swing.JPanel();
        JTBModificarCuenta = new javax.swing.JToggleButton();
        JBEliminarCuenta = new javax.swing.JButton();
        JTBAnadirCuenta = new javax.swing.JToggleButton();
        JBPDFCuentas = new javax.swing.JButton();
        JPHistorialAccesosSist = new javax.swing.JPanel();
        jScrollPane36 = new javax.swing.JScrollPane();
        JTHistorialAccesosSist = new javax.swing.JTable();
        JTFBuscarAccesosSist = new javax.swing.JTextField();
        JPAccsSistB = new javax.swing.JPanel();
        JCCFechaFinAccsSist = new org.freixas.jcalendar.JCalendarCombo();
        JCCFechaInicAccsSist = new org.freixas.jcalendar.JCalendarCombo();
        JCBHoraInicAccsSist = new javax.swing.JComboBox();
        JCBHoraFinAccsSist = new javax.swing.JComboBox();
        JCBTodoDiaAccsSist = new javax.swing.JCheckBox();
        JBPDFAccsSist = new javax.swing.JButton();
        JPTUsuarios1 = new javax.swing.JPanel();
        JPTUsuariosB = new javax.swing.JPanel();
        JTBModificarTUsuario = new javax.swing.JToggleButton();
        JBEliminarTUsuario = new javax.swing.JButton();
        JTBAnadirTUsuario = new javax.swing.JToggleButton();
        JPDatosTUsuario = new javax.swing.JPanel();
        JLTipoTUsuario = new javax.swing.JLabel();
        JLDescripcionTUsuario = new javax.swing.JLabel();
        JTFDescripTUsuario = new javax.swing.JTextField();
        JTFTUsuario = new javax.swing.JTextField();
        JBGrabarTUsuario = new javax.swing.JButton();
        jScrollPane6 = new javax.swing.JScrollPane();
        JTTUsuarios = new javax.swing.JTable();
        JPPartes = new javax.swing.JPanel();
        JPParte = new javax.swing.JPanel();
        JLIncidencias = new javax.swing.JLabel();
        JLCodVig = new javax.swing.JLabel();
        JTFCodVig = new javax.swing.JTextField();
        JLFecha = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        JTAIncidencias = new javax.swing.JTextArea();
        JCCFechaParte = new org.freixas.jcalendar.JCalendarCombo();
        JCBParte = new javax.swing.JComboBox();
        JLPartesLogo = new javax.swing.JLabel();
        JPPDFB = new javax.swing.JPanel();
        JBPDFParte = new javax.swing.JButton();
        jScrollPane4 = new javax.swing.JScrollPane();
        JTPartes = new javax.swing.JTable();
        JPPartesBuscarB = new javax.swing.JPanel();
        JCBCodVig = new javax.swing.JComboBox();
        JCBFechasPartes = new javax.swing.JCheckBox();
        JCCFechaInicParte = new org.freixas.jcalendar.JCalendarCombo();
        JCCFechaFinParte = new org.freixas.jcalendar.JCalendarCombo();
        JCBTodoDiaParte = new javax.swing.JCheckBox();
        JCBHoraFinParte = new javax.swing.JComboBox();
        JCBHoraInicParte = new javax.swing.JComboBox();
        JLCodVigParte = new javax.swing.JLabel();
        JPPartesB = new javax.swing.JPanel();
        JBAnadirParte = new javax.swing.JButton();
        JBEliminarParte = new javax.swing.JButton();
        JBModificarParte = new javax.swing.JButton();
        JPNotas = new javax.swing.JPanel();
        JTPNotas = new javax.swing.JTabbedPane();
        JPRedactar = new javax.swing.JPanel();
        jScrollPane7 = new javax.swing.JScrollPane();
        JTATexto = new javax.swing.JTextArea();
        JLAsunto = new javax.swing.JLabel();
        JTFAsunto = new javax.swing.JTextField();
        JPContactosNotas = new javax.swing.JPanel();
        jScrollPane8 = new javax.swing.JScrollPane();
        JTContactosNotas = new javax.swing.JTable();
        JBEnviar = new javax.swing.JButton();
        JTFBuscarContac = new javax.swing.JTextField();
        JPBandejas = new javax.swing.JPanel();
        JPContenidoNota = new javax.swing.JPanel();
        JTFAsuntoNota = new javax.swing.JTextField();
        JTFRedactorNota = new javax.swing.JTextField();
        JTFEnviado = new javax.swing.JTextField();
        JLEnviado = new javax.swing.JLabel();
        JLPara = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jScrollPane10 = new javax.swing.JScrollPane();
        JTATextoNota = new javax.swing.JTextArea();
        jScrollPane11 = new javax.swing.JScrollPane();
        JTAPara = new javax.swing.JTextArea();
        JTPES = new javax.swing.JTabbedPane();
        JPEntrada = new javax.swing.JPanel();
        jScrollPane9 = new javax.swing.JScrollPane();
        JTEntrada = new javax.swing.JTable();
        JPSalida = new javax.swing.JPanel();
        jScrollPane12 = new javax.swing.JScrollPane();
        JTSalida = new javax.swing.JTable();
        JBDescartar = new javax.swing.JButton();
        jMenuBar1 = new javax.swing.JMenuBar();
        JMAccessManager = new javax.swing.JMenu();
        JMIBD = new javax.swing.JMenu();
        JMIImportar = new javax.swing.JMenuItem();
        JMIExportar = new javax.swing.JMenuItem();
        JMICambiarDatos = new javax.swing.JMenuItem();
        JMISalir = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("accessManager");

        JTPFuncionalidades.setTabPlacement(javax.swing.JTabbedPane.LEFT);
        JTPFuncionalidades.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTPFuncionalidadesMouseClicked(evt);
            }
        });

        JPAccesos.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JPAccesosMouseClicked(evt);
            }
        });
        JPAccesos.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPAccesosComponentShown(evt);
            }
        });

        JTAccesos.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "DNI", "Entrada", "Salida", "Motivo"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTAccesos.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTAccesosMouseClicked(evt);
            }
        });
        JSPAccesos.setViewportView(JTAccesos);
        JTAccesos.getAccessibleContext().setAccessibleName("");

        JCCFechaInic.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JCCFechaInicMouseClicked(evt);
            }
        });
        JCCFechaInic.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFechaInicDateChanged(evt);
            }
        });
        JCCFechaInic.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCCFechaInicActionPerformed(evt);
            }
        });

        JCCFechaFin.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFechaFinDateChanged(evt);
            }
        });
        JCCFechaFin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCCFechaFinActionPerformed(evt);
            }
        });

        JCBTodoDia.setSelected(true);
        JCBTodoDia.setText("Todo el día");
        JCBTodoDia.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                JCBTodoDiaStateChanged(evt);
            }
        });
        JCBTodoDia.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBTodoDiaActionPerformed(evt);
            }
        });

        JCBHoraInic.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHoraInic.setEnabled(false);
        JCBHoraInic.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHoraInicItemStateChanged(evt);
            }
        });
        JCBHoraInic.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHoraInicActionPerformed(evt);
            }
        });

        JCBHoraFin.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHoraFin.setEnabled(false);
        JCBHoraFin.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHoraFinItemStateChanged(evt);
            }
        });
        JCBHoraFin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHoraFinActionPerformed(evt);
            }
        });

        BGEntradaSalida.add(JRBEntrada);
        JRBEntrada.setSelected(true);
        JRBEntrada.setText("Entrada");
        JRBEntrada.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JRBEntradaMouseClicked(evt);
            }
        });
        JRBEntrada.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                JRBEntradaStateChanged(evt);
            }
        });

        BGEntradaSalida.add(JRSalida);
        JRSalida.setLabel("Salida");
        JRSalida.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JRSalidaMouseClicked(evt);
            }
        });
        JRSalida.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                JRSalidaStateChanged(evt);
            }
        });

        JPDatosPersonales.setBorder(javax.swing.BorderFactory.createTitledBorder("Datos"));

        JLDNI.setText("DNI");

        JLApellido1.setText("Primer apellido");

        JLApellido2.setText("Segundo apellido");

        JLNombre.setText("Nombre");

        JTAMotivo.setColumns(20);
        JTAMotivo.setEditable(false);
        JTAMotivo.setRows(5);
        JSPMotivo.setViewportView(JTAMotivo);
        JTAMotivo.getAccessibleContext().setAccessibleName("JTAMotivo");

        JLMotivo.setText("Motivo");

        JLOtrosDatos.setText("Otros datos");

        JTAOtrosDatos.setColumns(20);
        JTAOtrosDatos.setEditable(false);
        JTAOtrosDatos.setRows(5);
        JSPOtrosDatos.setViewportView(JTAOtrosDatos);
        JTAOtrosDatos.getAccessibleContext().setAccessibleName("JTAOtrosDatos");

        JTFDNI.setEditable(false);

        JTFApellido1.setEditable(false);

        JTFApellido2.setEditable(false);

        JTFNombre.setEditable(false);

        JBAnadirPersona.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/door_in.png"))); // NOI18N
        JBAnadirPersona.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JBAnadirPersonaMouseClicked(evt);
            }
        });
        JBAnadirPersona.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirPersonaActionPerformed(evt);
            }
        });

        JBEliminarAcceso.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarAcceso.setEnabled(false);
        JBEliminarAcceso.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JBEliminarAccesoMouseClicked(evt);
            }
        });
        JBEliminarAcceso.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarAccesoActionPerformed(evt);
            }
        });

        JBAnadirAcceso.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBAnadirAcceso.setEnabled(false);
        JBAnadirAcceso.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirAccesoActionPerformed(evt);
            }
        });

        JBDarSalidaConDev.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/door_open.png"))); // NOI18N
        JBDarSalidaConDev.setEnabled(false);
        JBDarSalidaConDev.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JBDarSalidaConDevMouseClicked(evt);
            }
        });
        JBDarSalidaConDev.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBDarSalidaConDevActionPerformed(evt);
            }
        });

        JBSalida.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/door_out.png"))); // NOI18N
        JBSalida.setEnabled(false);
        JBSalida.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JBSalidaMouseClicked(evt);
            }
        });
        JBSalida.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBSalidaActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPAccesosB1Layout = new javax.swing.GroupLayout(JPAccesosB1);
        JPAccesosB1.setLayout(JPAccesosB1Layout);
        JPAccesosB1Layout.setHorizontalGroup(
            JPAccesosB1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAccesosB1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBAnadirPersona)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPAccesosB1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(JBEliminarAcceso, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JBAnadirAcceso))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPAccesosB1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(JBSalida, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JBDarSalidaConDev))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPAccesosB1Layout.setVerticalGroup(
            JPAccesosB1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAccesosB1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAccesosB1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPAccesosB1Layout.createSequentialGroup()
                        .addGroup(JPAccesosB1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JBSalida)
                            .addComponent(JBAnadirAcceso))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPAccesosB1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JBDarSalidaConDev, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(JBEliminarAcceso, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addComponent(JBAnadirPersona))
                .addContainerGap())
        );

        JBAnadirPersona.getAccessibleContext().setAccessibleName("JBAnadirPersona");
        JBEliminarAcceso.getAccessibleContext().setAccessibleName("JBEliminarAcceso");
        JBAnadirAcceso.getAccessibleContext().setAccessibleName("JBAnadirAcceso");
        JBDarSalidaConDev.getAccessibleContext().setAccessibleName("JBDarSalidaConDev");
        JBSalida.getAccessibleContext().setAccessibleName("JBSalida");

        javax.swing.GroupLayout JPDatosPersonalesLayout = new javax.swing.GroupLayout(JPDatosPersonales);
        JPDatosPersonales.setLayout(JPDatosPersonalesLayout);
        JPDatosPersonalesLayout.setHorizontalGroup(
            JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDatosPersonalesLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLMotivo)
                    .addComponent(JLNombre)
                    .addComponent(JLDNI)
                    .addComponent(JLApellido1)
                    .addComponent(JLApellido2)
                    .addComponent(JLOtrosDatos))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPAccesosB1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(JSPMotivo)
                        .addComponent(JTFDNI, javax.swing.GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
                        .addComponent(JTFNombre, javax.swing.GroupLayout.DEFAULT_SIZE, 255, Short.MAX_VALUE)
                        .addComponent(JTFApellido2)
                        .addComponent(JTFApellido1, javax.swing.GroupLayout.DEFAULT_SIZE, 249, Short.MAX_VALUE)
                        .addComponent(JSPOtrosDatos)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPDatosPersonalesLayout.setVerticalGroup(
            JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDatosPersonalesLayout.createSequentialGroup()
                .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(JPDatosPersonalesLayout.createSequentialGroup()
                        .addComponent(JLNombre)
                        .addGap(9, 9, 9)
                        .addComponent(JLMotivo)
                        .addGap(26, 26, 26))
                    .addGroup(JPDatosPersonalesLayout.createSequentialGroup()
                        .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTFDNI, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLDNI))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTFApellido1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLApellido1))
                        .addGap(4, 4, 4)
                        .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTFApellido2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLApellido2))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JTFNombre, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(9, 9, 9)
                        .addComponent(JSPMotivo, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)))
                .addGroup(JPDatosPersonalesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JSPOtrosDatos, javax.swing.GroupLayout.PREFERRED_SIZE, 36, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLOtrosDatos))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPAccesosB1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        JLDNI.getAccessibleContext().setAccessibleName("JLDNI");
        JSPMotivo.getAccessibleContext().setAccessibleName("JSPMotivo");
        JLOtrosDatos.getAccessibleContext().setAccessibleName("JLOtrosDatos");
        JSPOtrosDatos.getAccessibleContext().setAccessibleName("JSPOtrosDatos");
        JTFDNI.getAccessibleContext().setAccessibleName("JTFDNI");
        JTFApellido1.getAccessibleContext().setAccessibleName("JTFApellido1");
        JTFApellido2.getAccessibleContext().setAccessibleName("JTFApellido2");
        JTFNombre.getAccessibleContext().setAccessibleName("JTFNombre");
        JPAccesosB1.getAccessibleContext().setAccessibleName("JPAccesosB1");

        JPTarjeta.setBorder(javax.swing.BorderFactory.createTitledBorder("Tarjeta"));

        JTFCodTarj.setEditable(false);
        JTFCodTarj.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        JTFCodTarj.setBorder(null);

        JPColorTarjeta.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        javax.swing.GroupLayout JPColorTarjetaLayout = new javax.swing.GroupLayout(JPColorTarjeta);
        JPColorTarjeta.setLayout(JPColorTarjetaLayout);
        JPColorTarjetaLayout.setHorizontalGroup(
            JPColorTarjetaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 148, Short.MAX_VALUE)
        );
        JPColorTarjetaLayout.setVerticalGroup(
            JPColorTarjetaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 47, Short.MAX_VALUE)
        );

        JCBTarjeta.setText("Devuelta");
        JCBTarjeta.setEnabled(false);
        JCBTarjeta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBTarjetaActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPTarjetaLayout = new javax.swing.GroupLayout(JPTarjeta);
        JPTarjeta.setLayout(JPTarjetaLayout);
        JPTarjetaLayout.setHorizontalGroup(
            JPTarjetaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetaLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTarjetaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPTarjetaLayout.createSequentialGroup()
                        .addComponent(JTFCodTarj, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 54, Short.MAX_VALUE)
                        .addComponent(JCBTarjeta))
                    .addComponent(JPColorTarjeta, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        JPTarjetaLayout.setVerticalGroup(
            JPTarjetaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetaLayout.createSequentialGroup()
                .addComponent(JPColorTarjeta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPTarjetaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTFCodTarj, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JCBTarjeta)))
        );

        JTFCodTarj.getAccessibleContext().setAccessibleName("JTFCodTarj");
        JPColorTarjeta.getAccessibleContext().setAccessibleName("JPColorTarjeta");
        JCBTarjeta.getAccessibleContext().setAccessibleName("JCBTarjeta");

        JTFBuscar.setForeground(new java.awt.Color(204, 204, 204));
        JTFBuscar.setText("Buscar ...");
        JTFBuscar.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFBuscarMouseClicked(evt);
            }
        });
        JTFBuscar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTFBuscarActionPerformed(evt);
            }
        });
        JTFBuscar.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFBuscarKeyReleased(evt);
            }
            public void keyTyped(java.awt.event.KeyEvent evt) {
                JTFBuscarKeyTyped(evt);
            }
        });

        JPLlsLLls.setBorder(javax.swing.BorderFactory.createTitledBorder("Llaves y llaveros"));

        JTLsLls.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Tipo", "Código", "Devuelta"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        JTLsLls.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        JTLsLls.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsLlsMouseClicked(evt);
            }
        });
        JSPLLs.setViewportView(JTLsLls);
        JTLsLls.getAccessibleContext().setAccessibleName("JTLsLls");

        JBAnadirPrestamo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_add.png"))); // NOI18N
        JBAnadirPrestamo.setActionCommand("-");
        JBAnadirPrestamo.setEnabled(false);
        JBAnadirPrestamo.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JBAnadirPrestamoMouseClicked(evt);
            }
        });
        JBAnadirPrestamo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirPrestamoActionPerformed(evt);
            }
        });

        JBEliminarPrestamo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_delete.png"))); // NOI18N
        JBEliminarPrestamo.setEnabled(false);
        JBEliminarPrestamo.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JBEliminarPrestamoMouseClicked(evt);
            }
        });
        JBEliminarPrestamo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarPrestamoActionPerformed(evt);
            }
        });

        JBDevolverPrestamo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_go.png"))); // NOI18N
        JBDevolverPrestamo.setEnabled(false);
        JBDevolverPrestamo.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JBDevolverPrestamoMouseClicked(evt);
            }
        });
        JBDevolverPrestamo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBDevolverPrestamoActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPAccesosB2Layout = new javax.swing.GroupLayout(JPAccesosB2);
        JPAccesosB2.setLayout(JPAccesosB2Layout);
        JPAccesosB2Layout.setHorizontalGroup(
            JPAccesosB2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAccesosB2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBAnadirPrestamo)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarPrestamo)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBDevolverPrestamo)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPAccesosB2Layout.setVerticalGroup(
            JPAccesosB2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAccesosB2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAccesosB2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBAnadirPrestamo)
                    .addComponent(JBEliminarPrestamo)
                    .addComponent(JBDevolverPrestamo))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBAnadirPrestamo.getAccessibleContext().setAccessibleName("JBAnadirPrestamo");
        JBEliminarPrestamo.getAccessibleContext().setAccessibleName("JBEliminarPrestamo");
        JBDevolverPrestamo.getAccessibleContext().setAccessibleName("JBDevolverPrestamo");

        javax.swing.GroupLayout JPLlsLLlsLayout = new javax.swing.GroupLayout(JPLlsLLls);
        JPLlsLLls.setLayout(JPLlsLLlsLayout);
        JPLlsLLlsLayout.setHorizontalGroup(
            JPLlsLLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlsLLlsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLlsLLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JSPLLs, javax.swing.GroupLayout.DEFAULT_SIZE, 356, Short.MAX_VALUE)
                    .addComponent(JPAccesosB2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPLlsLLlsLayout.setVerticalGroup(
            JPLlsLLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlsLLlsLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JSPLLs, javax.swing.GroupLayout.PREFERRED_SIZE, 168, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPAccesosB2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(33, Short.MAX_VALUE))
        );

        JSPLLs.getAccessibleContext().setAccessibleName("JSPLLs");
        JPAccesosB2.getAccessibleContext().setAccessibleName("JPAccesosB2");

        JPAvisoTfno.setBorder(javax.swing.BorderFactory.createTitledBorder("Aviso telefónico"));

        JTFAvisoTfno.setEditable(false);

        JTTelefonos.setAutoCreateRowSorter(true);
        JTTelefonos.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Teléfonos"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane2.setViewportView(JTTelefonos);
        JTTelefonos.getAccessibleContext().setAccessibleName("JTTelefonos");

        javax.swing.GroupLayout JPAvisoTfnoLayout = new javax.swing.GroupLayout(JPAvisoTfno);
        JPAvisoTfno.setLayout(JPAvisoTfnoLayout);
        JPAvisoTfnoLayout.setHorizontalGroup(
            JPAvisoTfnoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPAvisoTfnoLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAvisoTfnoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 152, Short.MAX_VALUE)
                    .addComponent(JTFAvisoTfno, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 152, Short.MAX_VALUE))
                .addContainerGap())
        );
        JPAvisoTfnoLayout.setVerticalGroup(
            JPAvisoTfnoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAvisoTfnoLayout.createSequentialGroup()
                .addComponent(JTFAvisoTfno, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(34, Short.MAX_VALUE))
        );

        JTFAvisoTfno.getAccessibleContext().setAccessibleName("JTFAvisoTfno");

        JBPDFAccs.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_white_acrobat.png"))); // NOI18N
        JBPDFAccs.setBorderPainted(false);
        JBPDFAccs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBPDFAccsActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPAccesosLayout = new javax.swing.GroupLayout(JPAccesos);
        JPAccesos.setLayout(JPAccesosLayout);
        JPAccesosLayout.setHorizontalGroup(
            JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPAccesosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPAccesosLayout.createSequentialGroup()
                        .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPAccesosLayout.createSequentialGroup()
                                .addComponent(JCCFechaInic, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addComponent(JCBHoraInic, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addComponent(JCBTodoDia))
                            .addGroup(JPAccesosLayout.createSequentialGroup()
                                .addComponent(JCCFechaFin, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addComponent(JCBHoraFin, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(18, 18, 18)
                        .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPAccesosLayout.createSequentialGroup()
                                .addComponent(JRBEntrada)
                                .addGap(66, 66, 66)
                                .addComponent(JTFBuscar, javax.swing.GroupLayout.PREFERRED_SIZE, 149, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(JRSalida))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(JBPDFAccs))
                    .addComponent(JSPAccesos, javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPAccesosLayout.createSequentialGroup()
                        .addComponent(JPDatosPersonales, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPLlsLLls, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JPAvisoTfno, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(JPTarjeta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(126, Short.MAX_VALUE))
        );
        JPAccesosLayout.setVerticalGroup(
            JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAccesosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPAccesosLayout.createSequentialGroup()
                        .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPAccesosLayout.createSequentialGroup()
                                .addGap(1, 1, 1)
                                .addComponent(JCCFechaInic, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(JCBHoraInic, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(JCBTodoDia)
                                .addComponent(JRBEntrada)
                                .addComponent(JTFBuscar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPAccesosLayout.createSequentialGroup()
                                .addGap(4, 4, 4)
                                .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JCCFechaFin, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(JCBHoraFin, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addGroup(JPAccesosLayout.createSequentialGroup()
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(JRSalida))))
                    .addComponent(JBPDFAccs))
                .addGap(13, 13, 13)
                .addComponent(JSPAccesos, javax.swing.GroupLayout.PREFERRED_SIZE, 241, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPAccesosLayout.createSequentialGroup()
                        .addComponent(JPTarjeta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPAvisoTfno, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addComponent(JPLlsLLls, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JPDatosPersonales, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(399, 399, 399))
        );

        JSPAccesos.getAccessibleContext().setAccessibleName("JSPAccesos");
        JCCFechaInic.getAccessibleContext().setAccessibleName("JCCFechaInic");
        JCCFechaFin.getAccessibleContext().setAccessibleName("JCCFechaFin");
        JCBTodoDia.getAccessibleContext().setAccessibleName("CBTodoDia");
        JCBHoraInic.getAccessibleContext().setAccessibleName("JCBHoraInic");
        JCBHoraFin.getAccessibleContext().setAccessibleName("JCBHoraFin");
        JRBEntrada.getAccessibleContext().setAccessibleName("JRBEntrada");
        JPDatosPersonales.getAccessibleContext().setAccessibleName("JPDatosPersonales");
        JPTarjeta.getAccessibleContext().setAccessibleName("JPTarjeta");
        JTFBuscar.getAccessibleContext().setAccessibleName("JTFBuscar");
        JPLlsLLls.getAccessibleContext().setAccessibleName("JPLlsLLls");
        JPAvisoTfno.getAccessibleContext().setAccessibleName("JPAvisoTfno");
        JBPDFAccs.getAccessibleContext().setAccessibleName("JBPDFAccs");

        JTPFuncionalidades.addTab("<html><br><br>Accesos<br><br></html>", JPAccesos);
        JPAccesos.getAccessibleContext().setAccessibleName("JPAccesos");

        JPEmpresasPersonal.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPEmpresasPersonalComponentShown(evt);
            }
        });

        JPEmpresas.setBorder(javax.swing.BorderFactory.createTitledBorder("Empresas"));

        JTEmpresas.setAutoCreateRowSorter(true);
        JTEmpresas.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Nombre"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTEmpresas.setColumnSelectionAllowed(true);
        JTEmpresas.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        JTEmpresas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTEmpresasMouseClicked(evt);
            }
        });
        JSPEmpresas.setViewportView(JTEmpresas);
        JTEmpresas.getColumnModel().getSelectionModel().setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JTEmpresas.getAccessibleContext().setAccessibleName("");

        JLCodEmp.setText("Código");

        JLCIFEmp.setText("CIF");

        JLNomEmp.setText("Nombre");

        JLDirEmp.setText("Dirección");

        JTADirEmp.setColumns(15);
        JTADirEmp.setRows(3);
        jScrollPane19.setViewportView(JTADirEmp);
        JTADirEmp.getAccessibleContext().setAccessibleName("JTADirEmp");

        JBModificarEmpresa.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBModificarEmpresa.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBModificarEmpresaActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel6Layout.createSequentialGroup()
                .addContainerGap(38, Short.MAX_VALUE)
                .addComponent(JBModificarEmpresa))
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel6Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(JBModificarEmpresa)
                .addContainerGap())
        );

        JBModificarEmpresa.getAccessibleContext().setAccessibleName("JBModificarEmpresa");

        javax.swing.GroupLayout JPEmpresasDLayout = new javax.swing.GroupLayout(JPEmpresasD);
        JPEmpresasD.setLayout(JPEmpresasDLayout);
        JPEmpresasDLayout.setHorizontalGroup(
            JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPEmpresasDLayout.createSequentialGroup()
                        .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLNomEmp)
                            .addComponent(JLCIFEmp)
                            .addComponent(JLCodEmp))
                        .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(JPEmpresasDLayout.createSequentialGroup()
                                .addGap(18, 18, 18)
                                .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                    .addComponent(JTFCIFEmp, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JTFNomEmp, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jScrollPane19, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPEmpresasDLayout.createSequentialGroup()
                                        .addGap(10, 10, 10)
                                        .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                            .addGroup(JPEmpresasDLayout.createSequentialGroup()
                                .addGap(18, 18, 18)
                                .addComponent(JTFCodEmp))))
                    .addComponent(JLDirEmp))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPEmpresasDLayout.setVerticalGroup(
            JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTFCodEmp, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLCodEmp))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCIFEmp)
                    .addComponent(JTFCIFEmp, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLNomEmp)
                    .addComponent(JTFNomEmp, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPEmpresasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLDirEmp)
                    .addComponent(jScrollPane19, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        JLCodEmp.getAccessibleContext().setAccessibleName("JLCodEmp");
        JTFCodEmp.getAccessibleContext().setAccessibleName("JTFCodEmp");
        JLCIFEmp.getAccessibleContext().setAccessibleName("JLCIFEmp");
        JTFCIFEmp.getAccessibleContext().setAccessibleName("JTFCIFEmp");
        JLNomEmp.getAccessibleContext().setAccessibleName("JLNomEmp");
        JTFNomEmp.getAccessibleContext().setAccessibleName("JTFNomEmp");
        JLDirEmp.getAccessibleContext().setAccessibleName("JLDirEmp");

        JTFNombreEmp.setForeground(new java.awt.Color(204, 204, 204));
        JTFNombreEmp.setText("Nombre ...");
        JTFNombreEmp.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFNombreEmpMouseClicked(evt);
            }
        });
        JTFNombreEmp.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFNombreEmpKeyReleased(evt);
            }
        });

        JTFCIF.setForeground(new java.awt.Color(204, 204, 204));
        JTFCIF.setText("CIF ...");
        JTFCIF.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFCIFMouseClicked(evt);
            }
        });
        JTFCIF.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFCIFKeyReleased(evt);
            }
            public void keyTyped(java.awt.event.KeyEvent evt) {
                JTFCIFKeyTyped(evt);
            }
        });

        JTBAnadirEmpresa.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirEmpresa.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirEmpresaActionPerformed(evt);
            }
        });

        JBEliminarEmpresa.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarEmpresa.setEnabled(false);
        JBEliminarEmpresa.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarEmpresaActionPerformed(evt);
            }
        });

        JTBModificarEmpresa.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarEmpresa.setEnabled(false);
        JTBModificarEmpresa.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarEmpresaActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPEmpresasBLayout = new javax.swing.GroupLayout(JPEmpresasB);
        JPEmpresasB.setLayout(JPEmpresasBLayout);
        JPEmpresasBLayout.setHorizontalGroup(
            JPEmpresasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirEmpresa)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarEmpresa)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarEmpresa)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPEmpresasBLayout.setVerticalGroup(
            JPEmpresasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPEmpresasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirEmpresa)
                    .addComponent(JBEliminarEmpresa)
                    .addComponent(JTBModificarEmpresa))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBAnadirEmpresa.getAccessibleContext().setAccessibleName("JTBAnadirEmpresa");
        JBEliminarEmpresa.getAccessibleContext().setAccessibleName("JBEliminarEmpresa");
        JTBModificarEmpresa.getAccessibleContext().setAccessibleName("JTBModificarEmpresa");

        javax.swing.GroupLayout JPEmpresasLayout = new javax.swing.GroupLayout(JPEmpresas);
        JPEmpresas.setLayout(JPEmpresasLayout);
        JPEmpresasLayout.setHorizontalGroup(
            JPEmpresasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPEmpresasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPEmpresasLayout.createSequentialGroup()
                        .addGroup(JPEmpresasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPEmpresasLayout.createSequentialGroup()
                                .addComponent(JTFCIF, javax.swing.GroupLayout.PREFERRED_SIZE, 103, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addComponent(JTFNombreEmp, javax.swing.GroupLayout.PREFERRED_SIZE, 105, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(JSPEmpresas, javax.swing.GroupLayout.DEFAULT_SIZE, 354, Short.MAX_VALUE)
                            .addComponent(JPEmpresasB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addContainerGap())
                    .addComponent(JPEmpresasD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );
        JPEmpresasLayout.setVerticalGroup(
            JPEmpresasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasLayout.createSequentialGroup()
                .addGroup(JPEmpresasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTFCIF, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JTFNombreEmp, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JSPEmpresas, javax.swing.GroupLayout.PREFERRED_SIZE, 86, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPEmpresasB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPEmpresasD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JSPEmpresas.getAccessibleContext().setAccessibleName("JSPEmpresas");
        JPEmpresasD.getAccessibleContext().setAccessibleName("JPEmpresasD");
        JTFNombreEmp.getAccessibleContext().setAccessibleName("JTFNombreEmp");
        JTFCIF.getAccessibleContext().setAccessibleName("JTFCIF");

        JPDepartamentos.setBorder(javax.swing.BorderFactory.createTitledBorder("Departamentos"));

        JTDepartamentos.setAutoCreateRowSorter(true);
        JTDepartamentos.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Nombre", "Actividad"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTDepartamentos.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTDepartamentosMouseClicked(evt);
            }
        });
        JSPDepartamentos.setViewportView(JTDepartamentos);
        JTDepartamentos.getAccessibleContext().setAccessibleName("");

        JTFNombreDep.setForeground(new java.awt.Color(204, 204, 204));
        JTFNombreDep.setText("Nombre ...");
        JTFNombreDep.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFNombreDepMouseClicked(evt);
            }
        });
        JTFNombreDep.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFNombreDepKeyReleased(evt);
            }
        });

        JTFActividad.setForeground(new java.awt.Color(204, 204, 204));
        JTFActividad.setText("Actividad ...");
        JTFActividad.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFActividadMouseClicked(evt);
            }
        });
        JTFActividad.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFActividadKeyReleased(evt);
            }
        });

        JBEliminarDepartamento.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarDepartamento.setEnabled(false);
        JBEliminarDepartamento.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarDepartamentoActionPerformed(evt);
            }
        });

        JTBAnadirDepartamento.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirDepartamento.setEnabled(false);
        JTBAnadirDepartamento.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirDepartamentoActionPerformed(evt);
            }
        });

        JTBModificarDepartamento.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarDepartamento.setEnabled(false);
        JTBModificarDepartamento.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarDepartamentoActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPDepartamentosBLayout = new javax.swing.GroupLayout(JPDepartamentosB);
        JPDepartamentosB.setLayout(JPDepartamentosBLayout);
        JPDepartamentosBLayout.setHorizontalGroup(
            JPDepartamentosBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDepartamentosBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirDepartamento)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarDepartamento)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarDepartamento)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPDepartamentosBLayout.setVerticalGroup(
            JPDepartamentosBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDepartamentosBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDepartamentosBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBEliminarDepartamento)
                    .addComponent(JTBAnadirDepartamento)
                    .addComponent(JTBModificarDepartamento))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBEliminarDepartamento.getAccessibleContext().setAccessibleName("JBEliminarDepartamento");
        JTBAnadirDepartamento.getAccessibleContext().setAccessibleName("JTBAnadirDepartamento");
        JTBModificarDepartamento.getAccessibleContext().setAccessibleName("JTBModificarDepartamento");

        JLCodDep.setText("Código");

        JLNomDep.setText("Nombre");

        JLCodAcc.setText("Cód. acc.");

        JLPeriodicidad.setText("Periodicidad");

        JBModificarEmpresa1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBModificarEmpresa1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBModificarEmpresa1ActionPerformed(evt);
            }
        });

        jLabel1.setText("DNI");

        JLFUltActu.setText("F. últ. actu.");

        JCCFUltActu.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFUltActuDateChanged(evt);
            }
        });

        javax.swing.GroupLayout JPJCBDNIDepLayout = new javax.swing.GroupLayout(JPJCBDNIDep);
        JPJCBDNIDep.setLayout(JPJCBDNIDepLayout);
        JPJCBDNIDepLayout.setHorizontalGroup(
            JPJCBDNIDepLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPJCBDNIDepLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPJCBDNIDepLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLFUltActu)
                    .addComponent(jLabel1))
                .addGap(18, 26, Short.MAX_VALUE)
                .addGroup(JPJCBDNIDepLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(JCBDNIDep, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JCCFUltActu, javax.swing.GroupLayout.DEFAULT_SIZE, 203, Short.MAX_VALUE))
                .addContainerGap())
        );
        JPJCBDNIDepLayout.setVerticalGroup(
            JPJCBDNIDepLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPJCBDNIDepLayout.createSequentialGroup()
                .addGroup(JPJCBDNIDepLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLFUltActu)
                    .addComponent(JCCFUltActu, javax.swing.GroupLayout.DEFAULT_SIZE, 22, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPJCBDNIDepLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(JCBDNIDep, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        JCBDNIDep.getAccessibleContext().setAccessibleName("JCBDNIDep");
        JLFUltActu.getAccessibleContext().setAccessibleName("JLFUltActu");

        JBEliminarPrestDep.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_delete.png"))); // NOI18N
        JBEliminarPrestDep.setEnabled(false);
        JBEliminarPrestDep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarPrestDepActionPerformed(evt);
            }
        });

        JBAnadirPrestDep.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_add.png"))); // NOI18N
        JBAnadirPrestDep.setEnabled(false);
        JBAnadirPrestDep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirPrestDepActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBAnadirPrestDep)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarPrestDep)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBAnadirPrestDep)
                    .addComponent(JBEliminarPrestDep))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBEliminarPrestDep.getAccessibleContext().setAccessibleName("JBEliminarPrestDep");
        JBAnadirPrestDep.getAccessibleContext().setAccessibleName("JBAnadirPrestDep");

        JTLsLlsPers.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Zona", "Descripción"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsLlsPers.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsLlsPersMouseClicked(evt);
            }
        });
        jScrollPane18.setViewportView(JTLsLlsPers);

        javax.swing.GroupLayout JPDepartamentosDLayout = new javax.swing.GroupLayout(JPDepartamentosD);
        JPDepartamentosD.setLayout(JPDepartamentosDLayout);
        JPDepartamentosDLayout.setHorizontalGroup(
            JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDepartamentosDLayout.createSequentialGroup()
                .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPDepartamentosDLayout.createSequentialGroup()
                        .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPDepartamentosDLayout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JLPeriodicidad)
                                    .addComponent(JLCodAcc)
                                    .addComponent(JLNomDep)
                                    .addComponent(JLCodDep))
                                .addGap(20, 20, 20)
                                .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JTFNomDep, javax.swing.GroupLayout.DEFAULT_SIZE, 211, Short.MAX_VALUE)
                                    .addComponent(JCBActividad, 0, 211, Short.MAX_VALUE)
                                    .addComponent(JTFPeriodicidad, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 211, Short.MAX_VALUE)
                                    .addComponent(JTFCodDep, javax.swing.GroupLayout.DEFAULT_SIZE, 211, Short.MAX_VALUE))
                                .addGap(8, 8, 8))
                            .addComponent(JPJCBDNIDep, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addGap(10, 10, 10))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPDepartamentosDLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(JBModificarEmpresa1)
                        .addGap(106, 106, 106)))
                .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jScrollPane18, javax.swing.GroupLayout.PREFERRED_SIZE, 148, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPDepartamentosDLayout.setVerticalGroup(
            JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDepartamentosDLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPDepartamentosDLayout.createSequentialGroup()
                        .addComponent(jScrollPane18, javax.swing.GroupLayout.PREFERRED_SIZE, 146, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPDepartamentosDLayout.createSequentialGroup()
                        .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLCodDep)
                            .addComponent(JTFCodDep, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(5, 5, 5)
                        .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLNomDep)
                            .addComponent(JTFNomDep, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLCodAcc)
                            .addComponent(JCBActividad, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPDepartamentosDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLPeriodicidad)
                            .addComponent(JTFPeriodicidad, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPJCBDNIDep, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JBModificarEmpresa1)))
                .addContainerGap())
        );

        JLCodDep.getAccessibleContext().setAccessibleName("JLCodDep");
        JTFCodDep.getAccessibleContext().setAccessibleName("JTFCodDep");
        JLNomDep.getAccessibleContext().setAccessibleName("JLNomDep");
        JTFNomDep.getAccessibleContext().setAccessibleName("JTFNomDep");
        JLCodAcc.getAccessibleContext().setAccessibleName("JLCodAcc");
        JLPeriodicidad.getAccessibleContext().setAccessibleName("JLPeriodicidad");
        JTFPeriodicidad.getAccessibleContext().setAccessibleName("JTFPeriodicidad");
        JCBActividad.getAccessibleContext().setAccessibleName("JCBActividad");
        JPJCBDNIDep.getAccessibleContext().setAccessibleName("JPJCBDNIDep");

        javax.swing.GroupLayout JPDepartamentosLayout = new javax.swing.GroupLayout(JPDepartamentos);
        JPDepartamentos.setLayout(JPDepartamentosLayout);
        JPDepartamentosLayout.setHorizontalGroup(
            JPDepartamentosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDepartamentosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDepartamentosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPDepartamentosD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPDepartamentosLayout.createSequentialGroup()
                        .addComponent(JTFNombreDep, javax.swing.GroupLayout.PREFERRED_SIZE, 117, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JTFActividad, javax.swing.GroupLayout.PREFERRED_SIZE, 117, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(JPDepartamentosB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JSPDepartamentos, javax.swing.GroupLayout.PREFERRED_SIZE, 460, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPDepartamentosLayout.setVerticalGroup(
            JPDepartamentosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDepartamentosLayout.createSequentialGroup()
                .addGroup(JPDepartamentosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTFNombreDep, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JTFActividad, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JSPDepartamentos, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPDepartamentosB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPDepartamentosD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTFNombreDep.getAccessibleContext().setAccessibleName("JTFNombreDep");
        JTFActividad.getAccessibleContext().setAccessibleName("JTFActividad");
        JPDepartamentosD.getAccessibleContext().setAccessibleName("JPDepartamentosD");

        JPPersonal.setBorder(javax.swing.BorderFactory.createTitledBorder("Personal"));

        JTPDepartamentos.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTPDepartamentosMouseClicked(evt);
            }
        });

        JTEmpleados.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {

            }
        ));
        JTEmpleados.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTEmpleadosMouseClicked(evt);
            }
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                JTEmpleadosMouseEntered(evt);
            }
        });
        JSPEmpleados.setViewportView(JTEmpleados);
        JTEmpleados.getAccessibleContext().setAccessibleName("JTEmpleados");

        javax.swing.GroupLayout JPEmpleadosLayout = new javax.swing.GroupLayout(JPEmpleados);
        JPEmpleados.setLayout(JPEmpleadosLayout);
        JPEmpleadosLayout.setHorizontalGroup(
            JPEmpleadosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpleadosLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JSPEmpleados, javax.swing.GroupLayout.DEFAULT_SIZE, 586, Short.MAX_VALUE)
                .addContainerGap())
        );
        JPEmpleadosLayout.setVerticalGroup(
            JPEmpleadosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpleadosLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JSPEmpleados, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        JTPDepartamentos.addTab("Empleados", JPEmpleados);
        JPEmpleados.getAccessibleContext().setAccessibleName("JPEmpleados");

        JTContactos.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {

            }
        ));
        JTContactos.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTContactosMouseClicked(evt);
            }
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                JTContactosMouseEntered(evt);
            }
        });
        jScrollPane1.setViewportView(JTContactos);
        JTContactos.getAccessibleContext().setAccessibleName("");

        JTCargos.setAutoCreateRowSorter(true);
        JTCargos.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Cargos"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane16.setViewportView(JTCargos);
        JTCargos.getAccessibleContext().setAccessibleName("JTCargos");

        JCBCargos.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBCargosItemStateChanged(evt);
            }
        });
        JCBCargos.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBCargosActionPerformed(evt);
            }
        });

        JEliminarCargo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JEliminarCargo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JEliminarCargoActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPElimCLayout = new javax.swing.GroupLayout(JPElimC);
        JPElimC.setLayout(JPElimCLayout);
        JPElimCLayout.setHorizontalGroup(
            JPElimCLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPElimCLayout.createSequentialGroup()
                .addComponent(JCBCargos, 0, 58, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JEliminarCargo))
        );
        JPElimCLayout.setVerticalGroup(
            JPElimCLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPElimCLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(JCBCargos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(JEliminarCargo))
        );

        JCBCargos.getAccessibleContext().setAccessibleName("JCBCargos");

        javax.swing.GroupLayout JPContactosLayout = new javax.swing.GroupLayout(JPContactos);
        JPContactos.setLayout(JPContactosLayout);
        JPContactosLayout.setHorizontalGroup(
            JPContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPContactosLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 467, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane16, javax.swing.GroupLayout.DEFAULT_SIZE, 113, Short.MAX_VALUE)
                    .addComponent(JPElimC, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        JPContactosLayout.setVerticalGroup(
            JPContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPContactosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPContactosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPContactosLayout.createSequentialGroup()
                        .addComponent(jScrollPane16, javax.swing.GroupLayout.DEFAULT_SIZE, 72, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPElimC, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 103, Short.MAX_VALUE))
                .addContainerGap())
        );

        JPElimC.getAccessibleContext().setAccessibleName("JPElimC");

        JTPDepartamentos.addTab("Contactos", JPContactos);
        JPContactos.getAccessibleContext().setAccessibleName("JPContactos");

        JTEmails.setAutoCreateRowSorter(true);
        JTEmails.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "E-Mails"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        jScrollPane14.setViewportView(JTEmails);
        JTEmails.getAccessibleContext().setAccessibleName("JTEmails");

        JTTfns.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Telefonos"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTTfns.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTTfnsMouseClicked(evt);
            }
        });
        jScrollPane15.setViewportView(JTTfns);
        JTTfns.getAccessibleContext().setAccessibleName("JTTfns");

        JTFFiltro.setForeground(new java.awt.Color(204, 204, 204));
        JTFFiltro.setText("Buscar ...");
        JTFFiltro.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFFiltroMouseClicked(evt);
            }
        });
        JTFFiltro.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFFiltroKeyReleased(evt);
            }
        });

        JTBAnadirPerContac.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_add.png"))); // NOI18N
        JTBAnadirPerContac.setEnabled(false);
        JTBAnadirPerContac.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirPerContacActionPerformed(evt);
            }
        });

        JTBModificarPerContac.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_edit.png"))); // NOI18N
        JTBModificarPerContac.setEnabled(false);
        JTBModificarPerContac.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarPerContacActionPerformed(evt);
            }
        });

        JBEliminarPerContac.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_delete.png"))); // NOI18N
        JBEliminarPerContac.setEnabled(false);
        JBEliminarPerContac.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarPerContacActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPPersonalBLayout = new javax.swing.GroupLayout(JPPersonalB);
        JPPersonalB.setLayout(JPPersonalBLayout);
        JPPersonalBLayout.setHorizontalGroup(
            JPPersonalBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPersonalBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirPerContac)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarPerContac)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarPerContac)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPPersonalBLayout.setVerticalGroup(
            JPPersonalBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPersonalBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPersonalBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JTBModificarPerContac)
                    .addComponent(JBEliminarPerContac)
                    .addComponent(JTBAnadirPerContac))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBAnadirPerContac.getAccessibleContext().setAccessibleName("JTBAnadirPerContac");
        JTBModificarPerContac.getAccessibleContext().setAccessibleName("JTBModificarPerContac");
        JBEliminarPerContac.getAccessibleContext().setAccessibleName("JBEliminarPerContac");

        JLDNIDP.setText("DNI");

        JLApellido1DP.setText("Primer apellido");

        JLApellido2DP.setText("Segundo apellido");

        JLNombreDP.setText("Nombre");

        JLTfnoDP.setText("Tfno./E-Mail");

        JBGrabarPerContac.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarPerContac.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarPerContacActionPerformed(evt);
            }
        });

        JBAnadirTfno.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/phone_add.png"))); // NOI18N
        JBAnadirTfno.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirTfnoActionPerformed(evt);
            }
        });

        JBAnadirEmail.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/email_add.png"))); // NOI18N
        JBAnadirEmail.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirEmailActionPerformed(evt);
            }
        });

        JCBOcasional.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBOcasionalItemStateChanged(evt);
            }
        });
        JCBOcasional.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBOcasionalActionPerformed(evt);
            }
        });

        JLDNIOcas.setText("DNI");

        javax.swing.GroupLayout JPDNIAutoLayout = new javax.swing.GroupLayout(JPDNIAuto);
        JPDNIAuto.setLayout(JPDNIAutoLayout);
        JPDNIAutoLayout.setHorizontalGroup(
            JPDNIAutoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPDNIAutoLayout.createSequentialGroup()
                .addComponent(JLDNIOcas)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 74, Short.MAX_VALUE)
                .addComponent(JCBOcasional, javax.swing.GroupLayout.PREFERRED_SIZE, 131, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        JPDNIAutoLayout.setVerticalGroup(
            JPDNIAutoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPDNIAutoLayout.createSequentialGroup()
                .addContainerGap(22, Short.MAX_VALUE)
                .addGroup(JPDNIAutoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JCBOcasional, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLDNIOcas)))
        );

        JCBOcasional.getAccessibleContext().setAccessibleName("JCBOcasional");
        JLDNIOcas.getAccessibleContext().setAccessibleName("JLDNIOcas");

        javax.swing.GroupLayout JPDPLayout = new javax.swing.GroupLayout(JPDP);
        JPDP.setLayout(JPDPLayout);
        JPDPLayout.setHorizontalGroup(
            JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDPLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(JPDPLayout.createSequentialGroup()
                        .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLApellido1DP)
                            .addComponent(JLDNIDP)
                            .addComponent(JLApellido2DP))
                        .addGap(10, 10, 10)
                        .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JTFApellido2DP, javax.swing.GroupLayout.DEFAULT_SIZE, 132, Short.MAX_VALUE)
                            .addComponent(JTFApellido1DP, javax.swing.GroupLayout.DEFAULT_SIZE, 132, Short.MAX_VALUE)
                            .addComponent(JTFDNIDP, javax.swing.GroupLayout.DEFAULT_SIZE, 132, Short.MAX_VALUE)))
                    .addComponent(JPDNIAuto, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPDPLayout.createSequentialGroup()
                        .addComponent(JLNombreDP)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 31, Short.MAX_VALUE)
                        .addComponent(JTFNombreDP, javax.swing.GroupLayout.PREFERRED_SIZE, 119, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPDPLayout.createSequentialGroup()
                        .addComponent(JLTfnoDP)
                        .addGap(10, 10, 10)
                        .addComponent(JTFTfnoEmailDP, javax.swing.GroupLayout.DEFAULT_SIZE, 119, Short.MAX_VALUE))
                    .addComponent(JBGrabarPerContac)
                    .addGroup(JPDPLayout.createSequentialGroup()
                        .addComponent(JBAnadirTfno)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JBAnadirEmail)))
                .addContainerGap())
        );
        JPDPLayout.setVerticalGroup(
            JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDPLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLDNIDP)
                    .addComponent(JTFDNIDP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLNombreDP)
                    .addComponent(JTFNombreDP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLApellido1DP)
                    .addComponent(JTFApellido1DP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLTfnoDP)
                    .addComponent(JTFTfnoEmailDP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(JBAnadirTfno)
                        .addComponent(JBAnadirEmail))
                    .addGroup(JPDPLayout.createSequentialGroup()
                        .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTFApellido2DP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLApellido2DP))
                        .addGap(3, 3, 3)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPDPLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(JBGrabarPerContac)
                    .addComponent(JPDNIAuto, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(11, 11, 11))
        );

        JLDNIDP.getAccessibleContext().setAccessibleName("JLDNIDP");
        JLApellido1DP.getAccessibleContext().setAccessibleName("JLApellido1DP");
        JLApellido2DP.getAccessibleContext().setAccessibleName("JLApellido2DP");
        JLNombreDP.getAccessibleContext().setAccessibleName("JLNombreDP");
        JTFApellido2DP.getAccessibleContext().setAccessibleName("JTFApellido2DP");
        JTFApellido1DP.getAccessibleContext().setAccessibleName("JTFApellido1DP");
        JTFDNIDP.getAccessibleContext().setAccessibleName("JTFDNIDP");
        JTFNombreDP.getAccessibleContext().setAccessibleName("JTFNombreDP");
        JLTfnoDP.getAccessibleContext().setAccessibleName("JLTfnoDP");
        JTFTfnoEmailDP.getAccessibleContext().setAccessibleName("JTFTfnoDP");
        JBGrabarPerContac.getAccessibleContext().setAccessibleName("JBGrabarPerContac");
        JBAnadirTfno.getAccessibleContext().setAccessibleName("JBAnadirTfno");
        JBAnadirEmail.getAccessibleContext().setAccessibleName("JBAnadirEmail");
        JPDNIAuto.getAccessibleContext().setAccessibleName("JPDNIAuto");

        JLNivel.setText("Nivel");

        javax.swing.GroupLayout JPResidenteLayout = new javax.swing.GroupLayout(JPResidente);
        JPResidente.setLayout(JPResidenteLayout);
        JPResidenteLayout.setHorizontalGroup(
            JPResidenteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPResidenteLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JLNivel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JCBNivel, javax.swing.GroupLayout.PREFERRED_SIZE, 131, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPResidenteLayout.setVerticalGroup(
            JPResidenteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPResidenteLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(JPResidenteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLNivel)
                    .addComponent(JCBNivel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        JLNivel.getAccessibleContext().setAccessibleName("JLNivel");
        JCBNivel.getAccessibleContext().setAccessibleName("JCBNivel");

        JLCodV.setText("Código de vig.");

        javax.swing.GroupLayout JPVigilanteLayout = new javax.swing.GroupLayout(JPVigilante);
        JPVigilante.setLayout(JPVigilanteLayout);
        JPVigilanteLayout.setHorizontalGroup(
            JPVigilanteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPVigilanteLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JLCodV)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JTFCodV, javax.swing.GroupLayout.PREFERRED_SIZE, 133, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPVigilanteLayout.setVerticalGroup(
            JPVigilanteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPVigilanteLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPVigilanteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodV)
                    .addComponent(JTFCodV, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JLCodV.getAccessibleContext().setAccessibleName("JLCodV");
        JTFCodV.getAccessibleContext().setAccessibleName("JTFCodV");

        JLEmpresas.setText("Empresas");

        JLActiv.setText("Actividad");

        JLFCaducidad.setText("Caducidad");

        JLNotas.setText("Notas");

        JCCFCaducidad.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFCaducidadDateChanged(evt);
            }
        });
        JCCFCaducidad.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCCFCaducidadItemStateChanged(evt);
            }
        });
        JCCFCaducidad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCCFCaducidadActionPerformed(evt);
            }
        });

        JTANotas.setColumns(15);
        JTANotas.setRows(2);
        jScrollPane17.setViewportView(JTANotas);
        JTANotas.getAccessibleContext().setAccessibleName("JTANotas");

        JCBEmpresas.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBEmpresasActionPerformed(evt);
            }
        });

        JTLsLlsPers1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Zona", "Descripción"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsLlsPers1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsLlsPers1MouseClicked(evt);
            }
        });
        jScrollPane38.setViewportView(JTLsLlsPers1);

        JBAnadirPrestDep1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_add.png"))); // NOI18N
        JBAnadirPrestDep1.setEnabled(false);
        JBAnadirPrestDep1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirPrestDep1ActionPerformed(evt);
            }
        });

        JBEliminarPrestDep1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_delete.png"))); // NOI18N
        JBEliminarPrestDep1.setEnabled(false);
        JBEliminarPrestDep1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarPrestDep1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBAnadirPrestDep1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarPrestDep1)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel5Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBAnadirPrestDep1)
                    .addComponent(JBEliminarPrestDep1))
                .addContainerGap())
        );

        javax.swing.GroupLayout JPOcasionalLayout = new javax.swing.GroupLayout(JPOcasional);
        JPOcasional.setLayout(JPOcasionalLayout);
        JPOcasionalLayout.setHorizontalGroup(
            JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPOcasionalLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLFCaducidad)
                    .addComponent(JLNotas)
                    .addComponent(JLActiv)
                    .addComponent(JLEmpresas))
                .addGap(18, 18, 18)
                .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane17, javax.swing.GroupLayout.DEFAULT_SIZE, 167, Short.MAX_VALUE)
                    .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(JCBEmpresas, 0, 166, Short.MAX_VALUE)
                        .addComponent(JCCFCaducidad, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(JCBActividad1, 0, 167, Short.MAX_VALUE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jScrollPane38, javax.swing.GroupLayout.PREFERRED_SIZE, 148, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPOcasionalLayout.setVerticalGroup(
            JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPOcasionalLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(JPOcasionalLayout.createSequentialGroup()
                        .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLEmpresas)
                            .addComponent(JCBEmpresas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLActiv)
                            .addComponent(JCBActividad1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLFCaducidad)
                            .addComponent(JCCFCaducidad, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPOcasionalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLNotas)
                            .addComponent(jScrollPane17, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(jScrollPane38, 0, 0, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        JLEmpresas.getAccessibleContext().setAccessibleName("JLEmpresas");
        JLActiv.getAccessibleContext().setAccessibleName("JLActiv");
        JLNotas.getAccessibleContext().setAccessibleName("JLNotas");
        JCCFCaducidad.getAccessibleContext().setAccessibleName("JCCFCaducidad");
        JCBEmpresas.getAccessibleContext().setAccessibleName("JCBEmpresas");

        JBEliminarTfno.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/phone_delete.png"))); // NOI18N
        JBEliminarTfno.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarTfnoActionPerformed(evt);
            }
        });

        JEliminarEmail.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/email_delete.png"))); // NOI18N
        JEliminarEmail.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JEliminarEmailActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPElimTELayout = new javax.swing.GroupLayout(JPElimTE);
        JPElimTE.setLayout(JPElimTELayout);
        JPElimTELayout.setHorizontalGroup(
            JPElimTELayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPElimTELayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPElimTELayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JBEliminarTfno)
                    .addComponent(JEliminarEmail))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPElimTELayout.setVerticalGroup(
            JPElimTELayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPElimTELayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBEliminarTfno)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 40, Short.MAX_VALUE)
                .addComponent(JEliminarEmail)
                .addContainerGap())
        );

        JBEliminarTfno.getAccessibleContext().setAccessibleName("JBEliminarTfno");
        JEliminarEmail.getAccessibleContext().setAccessibleName("JEliminarEmail");

        javax.swing.GroupLayout JPPersonalLayout = new javax.swing.GroupLayout(JPPersonal);
        JPPersonal.setLayout(JPPersonalLayout);
        JPPersonalLayout.setHorizontalGroup(
            JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPersonalLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JTFFiltro, javax.swing.GroupLayout.PREFERRED_SIZE, 213, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPPersonalLayout.createSequentialGroup()
                        .addComponent(JTPDepartamentos, javax.swing.GroupLayout.PREFERRED_SIZE, 611, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jScrollPane15, 0, 0, Short.MAX_VALUE)
                            .addComponent(jScrollPane14, javax.swing.GroupLayout.PREFERRED_SIZE, 136, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPElimTE, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPPersonalB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPPersonalLayout.createSequentialGroup()
                        .addComponent(JPDP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JPResidente, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JPVigilante, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPOcasional, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
        );
        JPPersonalLayout.setVerticalGroup(
            JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPersonalLayout.createSequentialGroup()
                .addComponent(JTFFiltro, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JTPDepartamentos, javax.swing.GroupLayout.PREFERRED_SIZE, 153, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPPersonalLayout.createSequentialGroup()
                        .addGap(21, 21, 21)
                        .addGroup(JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JPPersonalB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(JPPersonalLayout.createSequentialGroup()
                                .addComponent(jScrollPane15, javax.swing.GroupLayout.PREFERRED_SIZE, 62, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jScrollPane14, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(JPElimTE, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPDP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPPersonalLayout.createSequentialGroup()
                        .addComponent(JPResidente, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPVigilante, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(JPOcasional, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTPDepartamentos.getAccessibleContext().setAccessibleName("JTPDatosDerivados");
        JTFFiltro.getAccessibleContext().setAccessibleName("JTFFiltro");
        JPPersonalB.getAccessibleContext().setAccessibleName("JPPersonalB");
        JPDP.getAccessibleContext().setAccessibleName("JPDP");
        JPResidente.getAccessibleContext().setAccessibleName("JPResidente");
        JPVigilante.getAccessibleContext().setAccessibleName("JPVigilante");
        JPOcasional.getAccessibleContext().setAccessibleName("JPOcasional");
        JPElimTE.getAccessibleContext().setAccessibleName("JPElimTE");

        JBMostrarTodo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/sum.png"))); // NOI18N
        JBMostrarTodo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBMostrarTodoActionPerformed(evt);
            }
        });

        JBPrestsPDF2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBPrestsPDF2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBPrestsPDF2ActionPerformed(evt);
            }
        });

        JLPrestTablas2.setText("Tablas a reportar:");

        JCBPDFTodas2.setText("Todas");
        JCBPDFTodas2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFTodas2ActionPerformed(evt);
            }
        });

        JCBPDFAlms2.setText("Empresas");

        JCBPDFLs2.setText("Departamentos");
        JCBPDFLs2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFLs2ActionPerformed(evt);
            }
        });

        JCBPDFLls2.setText("Personal");
        JCBPDFLls2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFLls2ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPPrestsPDF2Layout = new javax.swing.GroupLayout(JPPrestsPDF2);
        JPPrestsPDF2.setLayout(JPPrestsPDF2Layout);
        JPPrestsPDF2Layout.setHorizontalGroup(
            JPPrestsPDF2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestsPDF2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPrestsPDF2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLPrestTablas2)
                    .addGroup(JPPrestsPDF2Layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(JPPrestsPDF2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JCBPDFAlms2)
                            .addComponent(JCBPDFTodas2)
                            .addComponent(JCBPDFLs2)
                            .addComponent(JCBPDFLls2)
                            .addComponent(JBPrestsPDF2))))
                .addContainerGap(17, Short.MAX_VALUE))
        );
        JPPrestsPDF2Layout.setVerticalGroup(
            JPPrestsPDF2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestsPDF2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(JLPrestTablas2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFTodas2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFAlms2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFLs2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFLls2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBPrestsPDF2))
        );

        JTBActivarPanelPDF2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_white_acrobat.png"))); // NOI18N
        JTBActivarPanelPDF2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBActivarPanelPDF2ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPEmpresasPersonalLayout = new javax.swing.GroupLayout(JPEmpresasPersonal);
        JPEmpresasPersonal.setLayout(JPEmpresasPersonalLayout);
        JPEmpresasPersonalLayout.setHorizontalGroup(
            JPEmpresasPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasPersonalLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPEmpresasPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPPersonal, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPEmpresasPersonalLayout.createSequentialGroup()
                        .addComponent(JPEmpresas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPDepartamentos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPEmpresasPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPEmpresasPersonalLayout.createSequentialGroup()
                                .addComponent(JTBActivarPanelPDF2)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(JBMostrarTodo))
                            .addComponent(JPPrestsPDF2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPEmpresasPersonalLayout.setVerticalGroup(
            JPEmpresasPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEmpresasPersonalLayout.createSequentialGroup()
                .addGroup(JPEmpresasPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPEmpresasPersonalLayout.createSequentialGroup()
                        .addGap(19, 19, 19)
                        .addGroup(JPEmpresasPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTBActivarPanelPDF2)
                            .addComponent(JBMostrarTodo))
                        .addGap(4, 4, 4)
                        .addComponent(JPPrestsPDF2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPEmpresasPersonalLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(JPEmpresasPersonalLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JPDepartamentos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JPEmpresas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPPersonal, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(153, 153, 153))
        );

        JPEmpresas.getAccessibleContext().setAccessibleName("JPEmpresas");
        JPDepartamentos.getAccessibleContext().setAccessibleName("JPDepartamentos");
        JPPersonal.getAccessibleContext().setAccessibleName("JPPersonal");
        JBMostrarTodo.getAccessibleContext().setAccessibleName("JBMostrarTodo");

        JTPFuncionalidades.addTab("<html><br><br>Empresas y personal<br><br></html>", JPEmpresasPersonal);
        JPEmpresasPersonal.getAccessibleContext().setAccessibleName("JPEmpresasPersonal");

        JPTACs.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JPTACsMouseClicked(evt);
            }
        });
        JPTACs.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPTACsComponentShown(evt);
            }
        });

        JPTarjetas.setBorder(javax.swing.BorderFactory.createTitledBorder("Tarjetas"));

        JTTarjetas.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        JTTarjetas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTTarjetasMouseClicked(evt);
            }
        });
        JTTarjetas.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                JTTarjetasCaretPositionChanged(evt);
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                JTTarjetasInputMethodTextChanged(evt);
            }
        });
        JTTarjetas.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTTarjetasKeyReleased(evt);
            }
        });
        jScrollPane25.setViewportView(JTTarjetas);
        JTTarjetas.getAccessibleContext().setAccessibleName("JTTarjetas");

        JBEliminarTarj.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarTarj.setEnabled(false);
        JBEliminarTarj.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarTarjActionPerformed(evt);
            }
        });

        JTBAnadirTarjeta.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirTarjeta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirTarjetaActionPerformed(evt);
            }
        });

        JBBajaTarj.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_down.png"))); // NOI18N
        JBBajaTarj.setEnabled(false);
        JBBajaTarj.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBBajaTarjActionPerformed(evt);
            }
        });

        JTBModificarTarj.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarTarj.setEnabled(false);
        JTBModificarTarj.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarTarjActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPTarjetasBLayout = new javax.swing.GroupLayout(JPTarjetasB);
        JPTarjetasB.setLayout(JPTarjetasBLayout);
        JPTarjetasBLayout.setHorizontalGroup(
            JPTarjetasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetasBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirTarjeta)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarTarj)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarTarj)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBBajaTarj)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPTarjetasBLayout.setVerticalGroup(
            JPTarjetasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetasBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTarjetasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirTarjeta)
                    .addComponent(JBEliminarTarj)
                    .addComponent(JTBModificarTarj)
                    .addComponent(JBBajaTarj))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBEliminarTarj.getAccessibleContext().setAccessibleName("JBEliminarTarj");
        JTBAnadirTarjeta.getAccessibleContext().setAccessibleName("JTBAnadirTarjeta");
        JBBajaTarj.getAccessibleContext().setAccessibleName("JBBajaTarj");
        JTBModificarTarj.getAccessibleContext().setAccessibleName("JTBModificarTarj");

        JPTarjetaD.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        JTFCodTarjD.setHorizontalAlignment(javax.swing.JTextField.CENTER);

        javax.swing.GroupLayout JPTarjetaDLayout = new javax.swing.GroupLayout(JPTarjetaD);
        JPTarjetaD.setLayout(JPTarjetaDLayout);
        JPTarjetaDLayout.setHorizontalGroup(
            JPTarjetaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetaDLayout.createSequentialGroup()
                .addGap(76, 76, 76)
                .addComponent(JTFCodTarjD, javax.swing.GroupLayout.PREFERRED_SIZE, 67, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(77, Short.MAX_VALUE))
        );
        JPTarjetaDLayout.setVerticalGroup(
            JPTarjetaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetaDLayout.createSequentialGroup()
                .addGap(34, 34, 34)
                .addComponent(JTFCodTarjD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(35, Short.MAX_VALUE))
        );

        JTFCodTarjD.getAccessibleContext().setAccessibleName("JTFCodTarjD");

        JBGrabarTarjeta.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarTarjeta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarTarjetaActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPTarjetasDLayout = new javax.swing.GroupLayout(JPTarjetasD);
        JPTarjetasD.setLayout(JPTarjetasDLayout);
        JPTarjetasDLayout.setHorizontalGroup(
            JPTarjetasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetasDLayout.createSequentialGroup()
                .addGroup(JPTarjetasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPTarjetasDLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(JPTarjetaD, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(JPTarjetasDLayout.createSequentialGroup()
                        .addGap(92, 92, 92)
                        .addComponent(JBGrabarTarjeta)))
                .addContainerGap())
        );
        JPTarjetasDLayout.setVerticalGroup(
            JPTarjetasDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetasDLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JPTarjetaD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBGrabarTarjeta)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPTarjetaD.getAccessibleContext().setAccessibleName("JPTarjetaD");
        JBGrabarTarjeta.getAccessibleContext().setAccessibleName("JBAnadirTarjeta");

        javax.swing.GroupLayout JPTarjetasLayout = new javax.swing.GroupLayout(JPTarjetas);
        JPTarjetas.setLayout(JPTarjetasLayout);
        JPTarjetasLayout.setHorizontalGroup(
            JPTarjetasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetasLayout.createSequentialGroup()
                .addGroup(JPTarjetasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(JPTarjetasD, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPTarjetasLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(JPTarjetasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(JPTarjetasB, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jScrollPane25, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPTarjetasLayout.setVerticalGroup(
            JPTarjetasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjetasLayout.createSequentialGroup()
                .addComponent(jScrollPane25, javax.swing.GroupLayout.PREFERRED_SIZE, 156, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPTarjetasB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPTarjetasD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        JPTarjetasB.getAccessibleContext().setAccessibleName("JPTarjetasB");
        JPTarjetasD.getAccessibleContext().setAccessibleName("JPTarjetasD");

        JPColores.setBorder(javax.swing.BorderFactory.createTitledBorder("Colores"));

        JTColores.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Colores"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTColores.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTColoresMouseClicked(evt);
            }
        });
        JTColores.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                JTColoresCaretPositionChanged(evt);
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                JTColoresInputMethodTextChanged(evt);
            }
        });
        JTColores.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTColoresKeyReleased(evt);
            }
        });
        jScrollPane24.setViewportView(JTColores);
        JTColores.getAccessibleContext().setAccessibleName("JTColores");

        JBColor.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/color_wheel.png"))); // NOI18N
        JBColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBColorActionPerformed(evt);
            }
        });

        JTFColorNiv.setEditable(false);
        JTFColorNiv.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTFColorNivActionPerformed(evt);
            }
        });

        JBGrabarColor.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarColorActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPColorDLayout = new javax.swing.GroupLayout(JPColorD);
        JPColorD.setLayout(JPColorDLayout);
        JPColorDLayout.setHorizontalGroup(
            JPColorDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPColorDLayout.createSequentialGroup()
                .addGroup(JPColorDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPColorDLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(JTFColorNiv, javax.swing.GroupLayout.PREFERRED_SIZE, 95, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JBColor))
                    .addGroup(JPColorDLayout.createSequentialGroup()
                        .addGap(32, 32, 32)
                        .addComponent(JBGrabarColor)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPColorDLayout.setVerticalGroup(
            JPColorDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPColorDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPColorDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTFColorNiv, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JBColor))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBGrabarColor)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBColor.getAccessibleContext().setAccessibleName("JBColor");
        JTFColorNiv.getAccessibleContext().setAccessibleName("JTFColorNiv");
        JBGrabarColor.getAccessibleContext().setAccessibleName("JBGrabarColor");

        JTBAnadirColor.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirColorActionPerformed(evt);
            }
        });

        JBEliminarColor.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarColor.setEnabled(false);
        JBEliminarColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarColorActionPerformed(evt);
            }
        });

        JBBajaColor.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_down.png"))); // NOI18N
        JBBajaColor.setEnabled(false);
        JBBajaColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBBajaColorActionPerformed(evt);
            }
        });

        JTBModificarColor.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarColor.setEnabled(false);
        JTBModificarColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarColorActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirColor)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarColor)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarColor)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBBajaColor)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirColor)
                    .addComponent(JBEliminarColor)
                    .addComponent(JTBModificarColor)
                    .addComponent(JBBajaColor))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBAnadirColor.getAccessibleContext().setAccessibleName("JTBAnadirColor");
        JBEliminarColor.getAccessibleContext().setAccessibleName("JBEliminarColor");
        JBBajaColor.getAccessibleContext().setAccessibleName("JBBajaColor");
        JTBModificarColor.getAccessibleContext().setAccessibleName("JTBModificarColor");

        javax.swing.GroupLayout JPColoresLayout = new javax.swing.GroupLayout(JPColores);
        JPColores.setLayout(JPColoresLayout);
        JPColoresLayout.setHorizontalGroup(
            JPColoresLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPColoresLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPColoresLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPColorD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPColoresLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jScrollPane24, javax.swing.GroupLayout.Alignment.LEADING, 0, 0, Short.MAX_VALUE)
                        .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap(12, Short.MAX_VALUE))
        );
        JPColoresLayout.setVerticalGroup(
            JPColoresLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPColoresLayout.createSequentialGroup()
                .addComponent(jScrollPane24, javax.swing.GroupLayout.PREFERRED_SIZE, 155, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPColorD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        JPColorD.getAccessibleContext().setAccessibleName("JPColorD");

        JPNiveles.setBorder(javax.swing.BorderFactory.createTitledBorder("Niveles"));

        JTNiveles.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        JTNiveles.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTNivelesMouseClicked(evt);
            }
        });
        JTNiveles.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                JTNivelesCaretPositionChanged(evt);
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                JTNivelesInputMethodTextChanged(evt);
            }
        });
        JTNiveles.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTNivelesKeyReleased(evt);
            }
        });
        jScrollPane23.setViewportView(JTNiveles);
        JTNiveles.getAccessibleContext().setAccessibleName("JTNiveles");

        JLCodNiv.setText("Código");

        JLDescripNiv.setText("Descripción");

        JBGrabarNivel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarNivel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarNivelActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPNivelDLayout = new javax.swing.GroupLayout(JPNivelD);
        JPNivelD.setLayout(JPNivelDLayout);
        JPNivelDLayout.setHorizontalGroup(
            JPNivelDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNivelDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPNivelDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPNivelDLayout.createSequentialGroup()
                        .addGroup(JPNivelDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLDescripNiv)
                            .addComponent(JLCodNiv))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(JPNivelDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JTFCodNiv, javax.swing.GroupLayout.DEFAULT_SIZE, 150, Short.MAX_VALUE)
                            .addComponent(JTFDescripNiv, javax.swing.GroupLayout.DEFAULT_SIZE, 150, Short.MAX_VALUE))
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPNivelDLayout.createSequentialGroup()
                        .addComponent(JBGrabarNivel)
                        .addGap(62, 62, 62))))
        );
        JPNivelDLayout.setVerticalGroup(
            JPNivelDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNivelDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPNivelDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodNiv)
                    .addComponent(JTFCodNiv, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPNivelDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLDescripNiv)
                    .addComponent(JTFDescripNiv, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBGrabarNivel)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JLCodNiv.getAccessibleContext().setAccessibleName("JLCodNiv");
        JLDescripNiv.getAccessibleContext().setAccessibleName("JLDescripNiv");
        JTFDescripNiv.getAccessibleContext().setAccessibleName("JTFDescripNiv");
        JTFCodNiv.getAccessibleContext().setAccessibleName("JTFCodNiv");
        JBGrabarNivel.getAccessibleContext().setAccessibleName("JBGrabarNivel");

        JBEliminarNivel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarNivel.setEnabled(false);
        JBEliminarNivel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarNivelActionPerformed(evt);
            }
        });

        JTBAnadirNivel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirNivel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirNivelActionPerformed(evt);
            }
        });

        JTBModificarNivel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarNivel.setEnabled(false);
        JTBModificarNivel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarNivelActionPerformed(evt);
            }
        });

        JBBajaNivel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_down.png"))); // NOI18N
        JBBajaNivel.setEnabled(false);
        JBBajaNivel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBBajaNivelActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPNivelesBLayout = new javax.swing.GroupLayout(JPNivelesB);
        JPNivelesB.setLayout(JPNivelesBLayout);
        JPNivelesBLayout.setHorizontalGroup(
            JPNivelesBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNivelesBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirNivel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarNivel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarNivel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBBajaNivel)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPNivelesBLayout.setVerticalGroup(
            JPNivelesBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNivelesBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPNivelesBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBEliminarNivel)
                    .addComponent(JTBAnadirNivel)
                    .addComponent(JTBModificarNivel)
                    .addComponent(JBBajaNivel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBEliminarNivel.getAccessibleContext().setAccessibleName("JBEliminarNivel");
        JTBAnadirNivel.getAccessibleContext().setAccessibleName("JTBAnadirNivel");
        JTBModificarNivel.getAccessibleContext().setAccessibleName("JTBModificarNivel");
        JBBajaNivel.getAccessibleContext().setAccessibleName("JBBajaNivel");

        javax.swing.GroupLayout JPNivelesLayout = new javax.swing.GroupLayout(JPNiveles);
        JPNiveles.setLayout(JPNivelesLayout);
        JPNivelesLayout.setHorizontalGroup(
            JPNivelesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNivelesLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPNivelesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane23, javax.swing.GroupLayout.DEFAULT_SIZE, 270, Short.MAX_VALUE)
                    .addGroup(JPNivelesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(JPNivelD, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(JPNivelesB, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );
        JPNivelesLayout.setVerticalGroup(
            JPNivelesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNivelesLayout.createSequentialGroup()
                .addComponent(jScrollPane23, javax.swing.GroupLayout.PREFERRED_SIZE, 155, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPNivelesB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPNivelD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPNivelD.getAccessibleContext().setAccessibleName("JPNivelD");
        JPNivelesB.getAccessibleContext().setAccessibleName("JPNivelesB");

        JPTAccesos.setBorder(javax.swing.BorderFactory.createTitledBorder("Tipos de acceso"));

        JTTAccesos.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Actividad", "Días", "Hora de inicio", "Hora de fin"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTTAccesos.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTTAccesosMouseClicked(evt);
            }
        });
        JTTAccesos.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                JTTAccesosCaretPositionChanged(evt);
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                JTTAccesosInputMethodTextChanged(evt);
            }
        });
        JTTAccesos.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTTAccesosKeyReleased(evt);
            }
        });
        jScrollPane35.setViewportView(JTTAccesos);
        JTTAccesos.getAccessibleContext().setAccessibleName("JTTAccesos");

        JLCodTAcceso.setText("Código");

        JLActivTAcceso.setText("Actividad");

        JBGrabarNivel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarNivel1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarNivel1ActionPerformed(evt);
            }
        });

        JLDiasTAcceso.setText("Días");

        JLHorasTAcceso.setText("Horas");

        JCBHITAccesos.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHITAccesos.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHITAccesosItemStateChanged(evt);
            }
        });
        JCBHITAccesos.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHITAccesosActionPerformed(evt);
            }
        });

        JCBHFTAccesos.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHFTAccesos.setSelectedIndex(47);
        JCBHFTAccesos.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHFTAccesosItemStateChanged(evt);
            }
        });
        JCBHFTAccesos.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHFTAccesosActionPerformed(evt);
            }
        });

        JCBAutTelTAccesos.setText("Aut. tfno.");

        JLNivTAcceso.setText("Nivel");

        javax.swing.GroupLayout JPTAccesoDLayout = new javax.swing.GroupLayout(JPTAccesoD);
        JPTAccesoD.setLayout(JPTAccesoDLayout);
        JPTAccesoDLayout.setHorizontalGroup(
            JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTAccesoDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPTAccesoDLayout.createSequentialGroup()
                        .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPTAccesoDLayout.createSequentialGroup()
                                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JLActivTAcceso)
                                    .addComponent(JLCodTAcceso))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addComponent(JTFActivTAcceso)
                                    .addComponent(JTFCodTAcceso, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)))
                            .addComponent(JCBAutTelTAccesos))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPTAccesoDLayout.createSequentialGroup()
                                .addComponent(JLNivTAcceso)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(JCBNivelTAcceso, 0, 117, Short.MAX_VALUE))
                            .addGroup(JPTAccesoDLayout.createSequentialGroup()
                                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JLHorasTAcceso)
                                    .addComponent(JLDiasTAcceso))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JTFDiasTAcceso, javax.swing.GroupLayout.DEFAULT_SIZE, 118, Short.MAX_VALUE)
                                    .addGroup(JPTAccesoDLayout.createSequentialGroup()
                                        .addComponent(JCBHITAccesos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 12, Short.MAX_VALUE)
                                        .addComponent(JCBHFTAccesos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))))
                    .addComponent(JBGrabarNivel1))
                .addContainerGap())
        );
        JPTAccesoDLayout.setVerticalGroup(
            JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTAccesoDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodTAcceso)
                    .addComponent(JTFCodTAcceso, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JTFDiasTAcceso, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLDiasTAcceso))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLActivTAcceso)
                    .addComponent(JTFActivTAcceso, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLHorasTAcceso)
                    .addComponent(JCBHITAccesos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JCBHFTAccesos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPTAccesoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JCBAutTelTAccesos)
                    .addComponent(JLNivTAcceso)
                    .addComponent(JCBNivelTAcceso, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JBGrabarNivel1)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JLCodTAcceso.getAccessibleContext().setAccessibleName("JLCodTAcceso");
        JLActivTAcceso.getAccessibleContext().setAccessibleName("JLActivTAcceso");
        JTFActivTAcceso.getAccessibleContext().setAccessibleName("JTFActivTAcceso");
        JTFCodTAcceso.getAccessibleContext().setAccessibleName("JTFCodTAcceso");
        JLDiasTAcceso.getAccessibleContext().setAccessibleName("JLDiasTAcceso");
        JLHorasTAcceso.getAccessibleContext().setAccessibleName("JLHorasTAcceso");
        JTFDiasTAcceso.getAccessibleContext().setAccessibleName("JTFDiasTAcceso");
        JCBHITAccesos.getAccessibleContext().setAccessibleName("JCBHITAccesos");
        JCBHFTAccesos.getAccessibleContext().setAccessibleName("JCBHFTAccesos");
        JCBAutTelTAccesos.getAccessibleContext().setAccessibleName("JCBAutTelTAccesos");
        JCBNivelTAcceso.getAccessibleContext().setAccessibleName("JCBNivelTAcceso");
        JLNivTAcceso.getAccessibleContext().setAccessibleName("JLNivTAcceso");

        JBEliminarTAcceso.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarTAcceso.setEnabled(false);
        JBEliminarTAcceso.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarTAccesoActionPerformed(evt);
            }
        });

        JTBAnadirTAcceso.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirTAcceso.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirTAccesoActionPerformed(evt);
            }
        });

        JTBModificarTAcceso.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarTAcceso.setEnabled(false);
        JTBModificarTAcceso.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarTAccesoActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPTAccesoBLayout = new javax.swing.GroupLayout(JPTAccesoB);
        JPTAccesoB.setLayout(JPTAccesoBLayout);
        JPTAccesoBLayout.setHorizontalGroup(
            JPTAccesoBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTAccesoBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirTAcceso)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarTAcceso)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarTAcceso)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPTAccesoBLayout.setVerticalGroup(
            JPTAccesoBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTAccesoBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTAccesoBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBEliminarTAcceso)
                    .addComponent(JTBAnadirTAcceso)
                    .addComponent(JTBModificarTAcceso))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBEliminarTAcceso.getAccessibleContext().setAccessibleName("JBEliminarTAcceso");
        JTBAnadirTAcceso.getAccessibleContext().setAccessibleName("JTBAnadirTAcceso");
        JTBModificarTAcceso.getAccessibleContext().setAccessibleName("JTBModificarTAcceso");

        javax.swing.GroupLayout JPTAccesosLayout = new javax.swing.GroupLayout(JPTAccesos);
        JPTAccesos.setLayout(JPTAccesosLayout);
        JPTAccesosLayout.setHorizontalGroup(
            JPTAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTAccesosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane35, javax.swing.GroupLayout.DEFAULT_SIZE, 403, Short.MAX_VALUE)
                    .addComponent(JPTAccesoB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPTAccesosLayout.createSequentialGroup()
                        .addGap(1, 1, 1)
                        .addComponent(JPTAccesoD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        JPTAccesosLayout.setVerticalGroup(
            JPTAccesosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTAccesosLayout.createSequentialGroup()
                .addComponent(jScrollPane35, javax.swing.GroupLayout.PREFERRED_SIZE, 129, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPTAccesoB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPTAccesoD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPTAccesoD.getAccessibleContext().setAccessibleName("JPTAccesoD");
        JPTAccesoB.getAccessibleContext().setAccessibleName("JPTAccesoB");

        JPCargos.setBorder(javax.swing.BorderFactory.createTitledBorder("Cargos"));

        JTCargoss.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Descripción"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        JTCargoss.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTCargossMouseClicked(evt);
            }
        });
        JTCargoss.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                JTCargossCaretPositionChanged(evt);
            }
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                JTCargossInputMethodTextChanged(evt);
            }
        });
        JTCargoss.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTCargossKeyReleased(evt);
            }
        });
        jScrollPane37.setViewportView(JTCargoss);
        JTCargoss.getAccessibleContext().setAccessibleName("JTCargoss");

        JLCodNiv1.setText("Código");

        JLDescripNiv1.setText("Descripción");

        JBGrabarCargo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarCargo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarCargoActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPCargoDLayout = new javax.swing.GroupLayout(JPCargoD);
        JPCargoD.setLayout(JPCargoDLayout);
        JPCargoDLayout.setHorizontalGroup(
            JPCargoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCargoDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPCargoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLDescripNiv1)
                    .addComponent(JLCodNiv1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPCargoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(JTFCodCargo)
                    .addComponent(JTFDescripCargo, javax.swing.GroupLayout.DEFAULT_SIZE, 166, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPCargoDLayout.createSequentialGroup()
                .addContainerGap(131, Short.MAX_VALUE)
                .addComponent(JBGrabarCargo)
                .addGap(70, 70, 70))
        );
        JPCargoDLayout.setVerticalGroup(
            JPCargoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCargoDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPCargoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodNiv1)
                    .addComponent(JTFCodCargo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPCargoDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLDescripNiv1)
                    .addComponent(JTFDescripCargo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBGrabarCargo)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTFDescripCargo.getAccessibleContext().setAccessibleName("JTFDescripCargo");
        JTFCodCargo.getAccessibleContext().setAccessibleName("JTFCodCargo");
        JBGrabarCargo.getAccessibleContext().setAccessibleName("JBGrabarCargo");

        JBEliminarCargo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarCargo.setEnabled(false);
        JBEliminarCargo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarCargoActionPerformed(evt);
            }
        });

        JTBAnadirCargo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirCargo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirCargoActionPerformed(evt);
            }
        });

        JTBModificarCargo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarCargo.setEnabled(false);
        JTBModificarCargo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarCargoActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPCargossLayout = new javax.swing.GroupLayout(JPCargoss);
        JPCargoss.setLayout(JPCargossLayout);
        JPCargossLayout.setHorizontalGroup(
            JPCargossLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCargossLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirCargo)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarCargo)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarCargo)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPCargossLayout.setVerticalGroup(
            JPCargossLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCargossLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPCargossLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBEliminarCargo)
                    .addComponent(JTBAnadirCargo)
                    .addComponent(JTBModificarCargo))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBEliminarCargo.getAccessibleContext().setAccessibleName("JBEliminarCargo");
        JTBAnadirCargo.getAccessibleContext().setAccessibleName("JTBAnadirCargo");
        JTBModificarCargo.getAccessibleContext().setAccessibleName("JTBModificarCargo");

        javax.swing.GroupLayout JPCargosLayout = new javax.swing.GroupLayout(JPCargos);
        JPCargos.setLayout(JPCargosLayout);
        JPCargosLayout.setHorizontalGroup(
            JPCargosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCargosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPCargosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane37, javax.swing.GroupLayout.DEFAULT_SIZE, 363, Short.MAX_VALUE)
                    .addComponent(JPCargoss, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPCargoD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPCargosLayout.setVerticalGroup(
            JPCargosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCargosLayout.createSequentialGroup()
                .addComponent(jScrollPane37, javax.swing.GroupLayout.PREFERRED_SIZE, 128, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPCargoss, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPCargoD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPCargoD.getAccessibleContext().setAccessibleName("JPCargoD");
        JPCargoss.getAccessibleContext().setAccessibleName("JPCargoss");

        javax.swing.GroupLayout JPTACsLayout = new javax.swing.GroupLayout(JPTACs);
        JPTACs.setLayout(JPTACsLayout);
        JPTACsLayout.setHorizontalGroup(
            JPTACsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTACsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTACsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPTACsLayout.createSequentialGroup()
                        .addComponent(JPTAccesos, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGap(18, 18, 18)
                        .addComponent(JPCargos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPTACsLayout.createSequentialGroup()
                        .addComponent(JPTarjetas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPColores, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPNiveles, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(827, 827, 827))
        );
        JPTACsLayout.setVerticalGroup(
            JPTACsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTACsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTACsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPTarjetas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPColores, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPNiveles, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPTACsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPCargos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPTAccesos, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPTarjetas.getAccessibleContext().setAccessibleName("JPTarjetas");
        JPColores.getAccessibleContext().setAccessibleName("JPColores");
        JPNiveles.getAccessibleContext().setAccessibleName("JPNiveles");
        JPTAccesos.getAccessibleContext().setAccessibleName("JPTAccesos");

        JTPFuncionalidades.addTab("<html><br>Tarjetas, tipos de<br>acceso y cargos<br><br></html>", JPTACs);
        JPTACs.getAccessibleContext().setAccessibleName("JPTACs");

        JPZsLsLls.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPZsLsLlsComponentShown(evt);
            }
        });

        JPZona.setBorder(javax.swing.BorderFactory.createTitledBorder("Llaves"));

        JTLsZona.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Descripción"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsZona.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsZonaMouseClicked(evt);
            }
        });
        jScrollPane26.setViewportView(JTLsZona);
        JTLsZona.getAccessibleContext().setAccessibleName("JTLsZona");

        JTLsPrestEmpDep.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Empresa", "Departamento"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsPrestEmpDep.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsPrestEmpDepMouseClicked(evt);
            }
        });
        jScrollPane27.setViewportView(JTLsPrestEmpDep);
        JTLsPrestEmpDep.getAccessibleContext().setAccessibleName("JTLsPrestEmpDep");

        JTLsPrestOcas.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "DNI", "Empresa"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsPrestOcas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsPrestOcasMouseClicked(evt);
            }
        });
        jScrollPane28.setViewportView(JTLsPrestOcas);
        JTLsPrestOcas.getAccessibleContext().setAccessibleName("JTLsPrestOcas");

        JBEliminarL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_delete.png"))); // NOI18N
        JBEliminarL.setEnabled(false);
        JBEliminarL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarLActionPerformed(evt);
            }
        });

        JTBAnadirL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_add.png"))); // NOI18N
        JTBAnadirL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirLActionPerformed(evt);
            }
        });

        JTBModificarLlave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarLlave.setEnabled(false);
        JTBModificarLlave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarLlaveActionPerformed(evt);
            }
        });

        JBDarBajaL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_down.png"))); // NOI18N
        JBDarBajaL.setEnabled(false);
        JBDarBajaL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBDarBajaLActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPLlaveBLayout = new javax.swing.GroupLayout(JPLlaveB);
        JPLlaveB.setLayout(JPLlaveBLayout);
        JPLlaveBLayout.setHorizontalGroup(
            JPLlaveBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlaveBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirL)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarL)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarLlave)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBDarBajaL)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPLlaveBLayout.setVerticalGroup(
            JPLlaveBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlaveBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLlaveBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirL)
                    .addComponent(JBEliminarL)
                    .addComponent(JTBModificarLlave)
                    .addComponent(JBDarBajaL))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBEliminarL.getAccessibleContext().setAccessibleName("JBEliminarL");
        JTBAnadirL.getAccessibleContext().setAccessibleName("JTBAnadirL");
        JTBModificarLlave.getAccessibleContext().setAccessibleName("JTBModificarLlave");
        JBDarBajaL.getAccessibleContext().setAccessibleName("JBDarBajaL");

        JBAnadirLPrestEmpDep.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_left.png"))); // NOI18N
        JBAnadirLPrestEmpDep.setEnabled(false);
        JBAnadirLPrestEmpDep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirLPrestEmpDepActionPerformed(evt);
            }
        });

        JBEliminarLPrestEmpDep.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_right.png"))); // NOI18N
        JBEliminarLPrestEmpDep.setEnabled(false);
        JBEliminarLPrestEmpDep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarLPrestEmpDepActionPerformed(evt);
            }
        });

        JBAnadirLPrestOcas.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_left.png"))); // NOI18N
        JBAnadirLPrestOcas.setEnabled(false);
        JBAnadirLPrestOcas.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirLPrestOcasActionPerformed(evt);
            }
        });

        JBEliminarLPrestOcas.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_right.png"))); // NOI18N
        JBEliminarLPrestOcas.setEnabled(false);
        JBEliminarLPrestOcas.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarLPrestOcasActionPerformed(evt);
            }
        });

        JTLsPrestEmpDepResto.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Empresa", "Departamento"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsPrestEmpDepResto.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsPrestEmpDepRestoMouseClicked(evt);
            }
        });
        jScrollPane29.setViewportView(JTLsPrestEmpDepResto);
        JTLsPrestEmpDepResto.getAccessibleContext().setAccessibleName("JTLsPrestEmpDepResto");

        JTLsPrestOcasResto.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "DNI", "Empresa"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsPrestOcasResto.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsPrestOcasRestoMouseClicked(evt);
            }
        });
        jScrollPane30.setViewportView(JTLsPrestOcasResto);
        JTLsPrestOcasResto.getAccessibleContext().setAccessibleName("JTLsPrestOcasResto");

        javax.swing.GroupLayout JPLsPrestBLayout = new javax.swing.GroupLayout(JPLsPrestB);
        JPLsPrestB.setLayout(JPLsPrestBLayout);
        JPLsPrestBLayout.setHorizontalGroup(
            JPLsPrestBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLsPrestBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLsPrestBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPLsPrestBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addComponent(JBAnadirLPrestOcas)
                        .addComponent(JBEliminarLPrestOcas))
                    .addGroup(JPLsPrestBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(JBAnadirLPrestEmpDep, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(JBEliminarLPrestEmpDep, javax.swing.GroupLayout.Alignment.TRAILING)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPLsPrestBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane30, javax.swing.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE)
                    .addComponent(jScrollPane29, javax.swing.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE))
                .addContainerGap())
        );
        JPLsPrestBLayout.setVerticalGroup(
            JPLsPrestBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLsPrestBLayout.createSequentialGroup()
                .addComponent(jScrollPane29, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane30, javax.swing.GroupLayout.PREFERRED_SIZE, 94, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(61, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPLsPrestBLayout.createSequentialGroup()
                .addGap(27, 27, 27)
                .addComponent(JBEliminarLPrestEmpDep)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBAnadirLPrestEmpDep)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 41, Short.MAX_VALUE)
                .addComponent(JBEliminarLPrestOcas)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBAnadirLPrestOcas)
                .addGap(77, 77, 77))
        );

        JBAnadirLPrestEmpDep.getAccessibleContext().setAccessibleName("JBAnadirLPrestEmpDep");
        JBEliminarLPrestEmpDep.getAccessibleContext().setAccessibleName("JBEliminarLPrestEmpDep");
        JBAnadirLPrestOcas.getAccessibleContext().setAccessibleName("JBAnadirLPrestOcas");
        JBEliminarLPrestOcas.getAccessibleContext().setAccessibleName("JBEliminarLPrestOcas");

        JLCodL.setText("Código");

        JLDescripL.setText("Descripción");

        JLNotasL.setText("Notas");

        JBGrabarL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarLActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPLlaveDLayout = new javax.swing.GroupLayout(JPLlaveD);
        JPLlaveD.setLayout(JPLlaveDLayout);
        JPLlaveDLayout.setHorizontalGroup(
            JPLlaveDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlaveDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLlaveDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLDescripL)
                    .addComponent(JLNotasL)
                    .addComponent(JLCodL))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPLlaveDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(JTFDescripL)
                    .addComponent(JTFCodL, javax.swing.GroupLayout.DEFAULT_SIZE, 138, Short.MAX_VALUE)
                    .addComponent(JTFNotasL))
                .addContainerGap())
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPLlaveDLayout.createSequentialGroup()
                .addContainerGap(120, Short.MAX_VALUE)
                .addComponent(JBGrabarL)
                .addGap(53, 53, 53))
        );
        JPLlaveDLayout.setVerticalGroup(
            JPLlaveDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlaveDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLlaveDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodL)
                    .addComponent(JTFCodL, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPLlaveDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLDescripL)
                    .addComponent(JTFDescripL, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPLlaveDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLNotasL)
                    .addComponent(JTFNotasL, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JBGrabarL)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JLCodL.getAccessibleContext().setAccessibleName("JLCodL");
        JLDescripL.getAccessibleContext().setAccessibleName("JLDescripL");
        JLNotasL.getAccessibleContext().setAccessibleName("JLNotasL");
        JTFDescripL.getAccessibleContext().setAccessibleName("JTFDescripL");
        JTFNotasL.getAccessibleContext().setAccessibleName("JTFNotasL");
        JTFCodL.getAccessibleContext().setAccessibleName("JTFCodL");
        JBGrabarL.getAccessibleContext().setAccessibleName("JBGrabarL");

        javax.swing.GroupLayout JPZonaLayout = new javax.swing.GroupLayout(JPZona);
        JPZona.setLayout(JPZonaLayout);
        JPZonaLayout.setHorizontalGroup(
            JPZonaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZonaLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPZonaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane26, javax.swing.GroupLayout.PREFERRED_SIZE, 253, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPLlaveB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPLlaveD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(7, 7, 7)
                .addGroup(JPZonaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane27, javax.swing.GroupLayout.PREFERRED_SIZE, 163, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jScrollPane28, javax.swing.GroupLayout.PREFERRED_SIZE, 163, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPLsPrestB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(18, Short.MAX_VALUE))
        );
        JPZonaLayout.setVerticalGroup(
            JPZonaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZonaLayout.createSequentialGroup()
                .addGroup(JPZonaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jScrollPane27, 0, 0, Short.MAX_VALUE)
                    .addComponent(jScrollPane26, javax.swing.GroupLayout.DEFAULT_SIZE, 95, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPZonaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPZonaLayout.createSequentialGroup()
                        .addComponent(JPLlaveB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPLlaveD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jScrollPane28, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE)))
            .addComponent(JPLsPrestB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        );

        JPLlaveB.getAccessibleContext().setAccessibleName("JPLlaveB");
        JPLsPrestB.getAccessibleContext().setAccessibleName("JPLsPrestB");
        JPLlaveD.getAccessibleContext().setAccessibleName("JPLlaveD");

        JBAnadirAlmLl.setText("+");
        JBAnadirAlmLl.setEnabled(false);

        JPZonas.setBorder(javax.swing.BorderFactory.createTitledBorder("Zonas"));

        JTZonas.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Nombre", "Nivel"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTZonas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTZonasMouseClicked(evt);
            }
        });
        jScrollPane13.setViewportView(JTZonas);
        JTZonas.getAccessibleContext().setAccessibleName("JTZonas");

        JBEliminarZona.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/map_delete.png"))); // NOI18N
        JBEliminarZona.setEnabled(false);
        JBEliminarZona.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarZonaActionPerformed(evt);
            }
        });

        JTBModificarZona.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/map_edit.png"))); // NOI18N
        JTBModificarZona.setEnabled(false);
        JTBModificarZona.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarZonaActionPerformed(evt);
            }
        });

        JTBAnadirZona.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/map_add.png"))); // NOI18N
        JTBAnadirZona.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirZonaActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPZonaBLayout = new javax.swing.GroupLayout(JPZonaB);
        JPZonaB.setLayout(JPZonaBLayout);
        JPZonaBLayout.setHorizontalGroup(
            JPZonaBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZonaBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirZona)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarZona)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarZona)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPZonaBLayout.setVerticalGroup(
            JPZonaBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPZonaBLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(JPZonaBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirZona)
                    .addComponent(JBEliminarZona)
                    .addComponent(JTBModificarZona))
                .addContainerGap())
        );

        JBEliminarZona.getAccessibleContext().setAccessibleName("JBEliminarZona");
        JTBModificarZona.getAccessibleContext().setAccessibleName("JTBModificarZona");
        JTBAnadirZona.getAccessibleContext().setAccessibleName("JTBAnadirZona");

        JLNivelZona.setText("Nivel");

        JLDescripZona.setText("Descripción");

        JLCodZona.setText("Código");

        JBGrabarZona.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarZona.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarZonaActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPZonaDLayout = new javax.swing.GroupLayout(JPZonaD);
        JPZonaD.setLayout(JPZonaDLayout);
        JPZonaDLayout.setHorizontalGroup(
            JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZonaDLayout.createSequentialGroup()
                .addGroup(JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPZonaDLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLDescripZona)
                            .addComponent(JLCodZona)
                            .addComponent(JLNivelZona))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(JCBNivelZona, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(JTFDescripZona)
                            .addComponent(JTFCodZona, javax.swing.GroupLayout.DEFAULT_SIZE, 160, Short.MAX_VALUE)))
                    .addGroup(JPZonaDLayout.createSequentialGroup()
                        .addGap(96, 96, 96)
                        .addComponent(JBGrabarZona)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPZonaDLayout.setVerticalGroup(
            JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZonaDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodZona)
                    .addComponent(JTFCodZona, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLDescripZona)
                    .addComponent(JTFDescripZona, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPZonaDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLNivelZona)
                    .addComponent(JCBNivelZona, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JBGrabarZona)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JCBNivelZona.getAccessibleContext().setAccessibleName("JCBNivelZona");
        JLNivelZona.getAccessibleContext().setAccessibleName("JLNivelZona");
        JLDescripZona.getAccessibleContext().setAccessibleName("JLDescripZona");
        JTFDescripZona.getAccessibleContext().setAccessibleName("JTFDescripZona");
        JTFCodZona.getAccessibleContext().setAccessibleName("JTFCodZona");
        JLCodZona.getAccessibleContext().setAccessibleName("JLCodZona");
        JBGrabarZona.getAccessibleContext().setAccessibleName("JBGrabarZona");

        javax.swing.GroupLayout JPZonasLayout = new javax.swing.GroupLayout(JPZonas);
        JPZonas.setLayout(JPZonasLayout);
        JPZonasLayout.setHorizontalGroup(
            JPZonasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZonasLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPZonasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane13, javax.swing.GroupLayout.DEFAULT_SIZE, 272, Short.MAX_VALUE)
                    .addComponent(JPZonaB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPZonaD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPZonasLayout.setVerticalGroup(
            JPZonasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZonasLayout.createSequentialGroup()
                .addComponent(jScrollPane13, javax.swing.GroupLayout.PREFERRED_SIZE, 140, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPZonaB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPZonaD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        JPZonaB.getAccessibleContext().setAccessibleName("JPZonaB");
        JPZonaD.getAccessibleContext().setAccessibleName("JPZonaD");

        JPAlmacenes.setBorder(javax.swing.BorderFactory.createTitledBorder("Almacenes"));

        JTAlms.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Almacenes"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTAlms.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTAlmsMouseClicked(evt);
            }
        });
        jScrollPane31.setViewportView(JTAlms);
        JTAlms.getAccessibleContext().setAccessibleName("JTAlms");

        JTBAnadirAlm.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/package_add.png"))); // NOI18N
        JTBAnadirAlm.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirAlmActionPerformed(evt);
            }
        });

        JBEliminarAlm.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/package_delete.png"))); // NOI18N
        JBEliminarAlm.setEnabled(false);
        JBEliminarAlm.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarAlmActionPerformed(evt);
            }
        });

        JTBModificarAlm.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBModificarAlm.setEnabled(false);
        JTBModificarAlm.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarAlmActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirAlm)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarAlm)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarAlm)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirAlm)
                    .addComponent(JBEliminarAlm)
                    .addComponent(JTBModificarAlm))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBAnadirAlm.getAccessibleContext().setAccessibleName("JTBAnadirAlm");
        JBEliminarAlm.getAccessibleContext().setAccessibleName("JBEliminarAlm");
        JTBModificarAlm.getAccessibleContext().setAccessibleName("JTBModificarAlm");

        JLCodAlm.setText("Código");

        JLDescripAlm.setText("Descripción");

        JBGrabarAlm.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarAlm.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarAlmActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPAlmDLayout = new javax.swing.GroupLayout(JPAlmD);
        JPAlmD.setLayout(JPAlmDLayout);
        JPAlmDLayout.setHorizontalGroup(
            JPAlmDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAlmDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAlmDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPAlmDLayout.createSequentialGroup()
                        .addGroup(JPAlmDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLDescripAlm)
                            .addComponent(JLCodAlm))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(JPAlmDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JTFCodAlm, javax.swing.GroupLayout.DEFAULT_SIZE, 152, Short.MAX_VALUE)
                            .addComponent(JTFDescripAlm, javax.swing.GroupLayout.DEFAULT_SIZE, 152, Short.MAX_VALUE))
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPAlmDLayout.createSequentialGroup()
                        .addComponent(JBGrabarAlm)
                        .addGap(66, 66, 66))))
        );
        JPAlmDLayout.setVerticalGroup(
            JPAlmDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAlmDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAlmDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodAlm)
                    .addComponent(JTFCodAlm, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPAlmDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLDescripAlm)
                    .addComponent(JTFDescripAlm, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JBGrabarAlm)
                .addContainerGap(18, Short.MAX_VALUE))
        );

        JLCodAlm.getAccessibleContext().setAccessibleName("JLCodAlm");
        JLDescripAlm.getAccessibleContext().setAccessibleName("JLDescripAlm");
        JTFDescripAlm.getAccessibleContext().setAccessibleName("JTFDescripAlm");
        JTFCodAlm.getAccessibleContext().setAccessibleName("JTFCodAlm");
        JBGrabarAlm.getAccessibleContext().setAccessibleName("JBGrabarAlm");

        JTLls.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Descripción", "Total"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLls.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLlsMouseClicked(evt);
            }
        });
        jScrollPane33.setViewportView(JTLls);
        JTLls.getAccessibleContext().setAccessibleName("JTLls");

        JTLs.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Zona", "Descripción", "Total"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLs.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsMouseClicked(evt);
            }
        });
        jScrollPane32.setViewportView(JTLs);
        JTLs.getAccessibleContext().setAccessibleName("JTLs");

        JTBAnadirLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/link_add.png"))); // NOI18N
        JTBAnadirLl.setEnabled(false);
        JTBAnadirLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirLlActionPerformed(evt);
            }
        });

        JTBModificarLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/link_edit.png"))); // NOI18N
        JTBModificarLl.setEnabled(false);
        JTBModificarLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarLlActionPerformed(evt);
            }
        });

        JBDarBajaLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_down.png"))); // NOI18N
        JBDarBajaLl.setEnabled(false);
        JBDarBajaLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBDarBajaLlActionPerformed(evt);
            }
        });

        JBEliminarLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/link_delete.png"))); // NOI18N
        JBEliminarLl.setEnabled(false);
        JBEliminarLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarLlActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirLl)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarLl)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarLl)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBDarBajaLl)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBEliminarLl)
                    .addComponent(JTBModificarLl)
                    .addComponent(JBDarBajaLl)
                    .addComponent(JTBAnadirLl))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBAnadirLl.getAccessibleContext().setAccessibleName("JTBAnadirLl");
        JTBModificarLl.getAccessibleContext().setAccessibleName("JTBModificarLl");
        JBDarBajaLl.getAccessibleContext().setAccessibleName("JBDarBajaLl");
        JBEliminarLl.getAccessibleContext().setAccessibleName("JBEliminarLl");

        JLCodLl.setText("Código");

        JLDescripLl.setText("Descripción");

        JBGrabarLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarLlActionPerformed(evt);
            }
        });

        JBAnadirLsLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_add.png"))); // NOI18N
        JBAnadirLsLl.setEnabled(false);
        JBAnadirLsLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirLsLlActionPerformed(evt);
            }
        });

        JBEliminarLsLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/key_delete.png"))); // NOI18N
        JBEliminarLsLl.setEnabled(false);
        JBEliminarLsLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarLsLlActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPLlaveroDLayout = new javax.swing.GroupLayout(JPLlaveroD);
        JPLlaveroD.setLayout(JPLlaveroDLayout);
        JPLlaveroDLayout.setHorizontalGroup(
            JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlaveroDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLDescripLl)
                    .addComponent(JLCodLl))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(JTFDescripLl)
                    .addComponent(JTFCodLl, javax.swing.GroupLayout.DEFAULT_SIZE, 93, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JBEliminarLsLl)
                    .addComponent(JBAnadirLsLl, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBGrabarLl)
                .addContainerGap())
        );
        JPLlaveroDLayout.setVerticalGroup(
            JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlaveroDLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JBGrabarLl)
                    .addGroup(JPLlaveroDLayout.createSequentialGroup()
                        .addGroup(JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(JBAnadirLsLl)
                            .addGroup(JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(JLCodLl)
                                .addComponent(JTFCodLl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPLlaveroDLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLDescripLl)
                            .addComponent(JTFDescripLl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JBEliminarLsLl))))
                .addContainerGap(19, Short.MAX_VALUE))
        );

        JLCodLl.getAccessibleContext().setAccessibleName("JLCodLl");
        JLDescripLl.getAccessibleContext().setAccessibleName("JLDescripLl");
        JTFDescripLl.getAccessibleContext().setAccessibleName("JTFDescripLl");
        JTFCodLl.getAccessibleContext().setAccessibleName("JTFCodLl");
        JBGrabarLl.getAccessibleContext().setAccessibleName("JBGrabarLl");
        JBAnadirLsLl.getAccessibleContext().setAccessibleName("JBAnadirLsLl");
        JBEliminarLsLl.getAccessibleContext().setAccessibleName("JBEliminarLsLl");

        JTBNumCopsL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBNumCopsL.setEnabled(false);
        JTBNumCopsL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBNumCopsLActionPerformed(evt);
            }
        });

        JTFNumCopsL.setText("1");
        JTFNumCopsL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTFNumCopsLActionPerformed(evt);
            }
        });

        JBGrabarAlmL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarAlmL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarAlmLActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPJTFNumCopsLLayout = new javax.swing.GroupLayout(JPJTFNumCopsL);
        JPJTFNumCopsL.setLayout(JPJTFNumCopsLLayout);
        JPJTFNumCopsLLayout.setHorizontalGroup(
            JPJTFNumCopsLLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPJTFNumCopsLLayout.createSequentialGroup()
                .addComponent(JTFNumCopsL, javax.swing.GroupLayout.PREFERRED_SIZE, 47, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 63, Short.MAX_VALUE)
                .addComponent(JBGrabarAlmL))
        );
        JPJTFNumCopsLLayout.setVerticalGroup(
            JPJTFNumCopsLLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPJTFNumCopsLLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPJTFNumCopsLLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JTFNumCopsL, javax.swing.GroupLayout.PREFERRED_SIZE, 22, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JBGrabarAlmL))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTFNumCopsL.getAccessibleContext().setAccessibleName("JTFNumCopsL");
        JBGrabarAlmL.getAccessibleContext().setAccessibleName("JBGrabarAlmL");

        JBEliminarAlmL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarAlmL.setEnabled(false);
        JBEliminarAlmL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarAlmLActionPerformed(evt);
            }
        });

        JTBAnadirAlmL.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirAlmL.setEnabled(false);
        JTBAnadirAlmL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirAlmLActionPerformed(evt);
            }
        });

        JTBAnadirAlmLlB.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/add.png"))); // NOI18N
        JTBAnadirAlmLlB.setEnabled(false);
        JTBAnadirAlmLlB.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirAlmLlBActionPerformed(evt);
            }
        });

        JBEliminarAlmLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/delete.png"))); // NOI18N
        JBEliminarAlmLl.setEnabled(false);
        JBEliminarAlmLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarAlmLlActionPerformed(evt);
            }
        });

        JTBNumCopsLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_refresh.png"))); // NOI18N
        JTBNumCopsLl.setEnabled(false);
        JTBNumCopsLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBNumCopsLlActionPerformed(evt);
            }
        });

        JPJTFNumCopsLl.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        JTFNumCopsLl.setText("1");
        JPJTFNumCopsLl.add(JTFNumCopsLl, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 10, 48, -1));
        JTFNumCopsLl.getAccessibleContext().setAccessibleName("JTFNumCopsLl");

        JBGrabarAlmLl.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarAlmLl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarAlmLlActionPerformed(evt);
            }
        });
        JPJTFNumCopsLl.add(JBGrabarAlmLl, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 10, -1, -1));
        JBGrabarAlmLl.getAccessibleContext().setAccessibleName("JBGrabarAlmLl");

        javax.swing.GroupLayout JPLlsLayout = new javax.swing.GroupLayout(JPLls);
        JPLls.setLayout(JPLlsLayout);
        JPLlsLayout.setHorizontalGroup(
            JPLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(JCBLlaveros, 0, 43, Short.MAX_VALUE)
        );
        JPLlsLayout.setVerticalGroup(
            JPLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLlsLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JCBLlaveros, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JCBLlaveros.getAccessibleContext().setAccessibleName("JCBLlaveros");

        JPJTFNumCopsLl.add(JPLls, new org.netbeans.lib.awtextra.AbsoluteConstraints(60, 0, -1, -1));
        JPLls.getAccessibleContext().setAccessibleName("JPLls");

        javax.swing.GroupLayout JPAlmsBLayout = new javax.swing.GroupLayout(JPAlmsB);
        JPAlmsB.setLayout(JPAlmsBLayout);
        JPAlmsBLayout.setHorizontalGroup(
            JPAlmsBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPAlmsBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAlmsBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(JPJTFNumCopsLl, javax.swing.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .addComponent(JPJTFNumCopsL, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPAlmsBLayout.createSequentialGroup()
                        .addComponent(JTBAnadirAlmL)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JBEliminarAlmL)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JTBNumCopsL))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPAlmsBLayout.createSequentialGroup()
                        .addComponent(JTBAnadirAlmLlB)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JBEliminarAlmLl)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JTBNumCopsLl)))
                .addContainerGap())
        );
        JPAlmsBLayout.setVerticalGroup(
            JPAlmsBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAlmsBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAlmsBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirAlmL)
                    .addComponent(JBEliminarAlmL)
                    .addComponent(JTBNumCopsL))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPJTFNumCopsL, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 62, Short.MAX_VALUE)
                .addGroup(JPAlmsBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirAlmLlB)
                    .addComponent(JBEliminarAlmLl)
                    .addComponent(JTBNumCopsLl))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPJTFNumCopsLl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(51, 51, 51))
        );

        JTBNumCopsL.getAccessibleContext().setAccessibleName("JTBNumCopsL");
        JPJTFNumCopsL.getAccessibleContext().setAccessibleName("JPJTFNumCopsL");
        JBEliminarAlmL.getAccessibleContext().setAccessibleName("JBEliminarAlmL");
        JTBAnadirAlmL.getAccessibleContext().setAccessibleName("JTBAnadirAlmL");
        JTBAnadirAlmLlB.getAccessibleContext().setAccessibleName("JTBAnadirAlmLlB");
        JBEliminarAlmLl.getAccessibleContext().setAccessibleName("JBEliminarAlmLl");
        JTBNumCopsLl.getAccessibleContext().setAccessibleName("JTBNumCopiasLl");

        JTFBuscarLLl.setForeground(new java.awt.Color(204, 204, 204));
        JTFBuscarLLl.setText("Buscar ...");
        JTFBuscarLLl.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFBuscarLLlMouseClicked(evt);
            }
        });
        JTFBuscarLLl.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFBuscarLLlKeyReleased(evt);
            }
        });

        JBPrestsPDF1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBPrestsPDF1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBPrestsPDF1ActionPerformed(evt);
            }
        });

        JLPrestTablas1.setText("Tablas a reportar:");

        JCBPDFTodas1.setText("Todas");
        JCBPDFTodas1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFTodas1ActionPerformed(evt);
            }
        });

        JCBPDFAlms1.setText("Almacenes");

        JCBPDFLs1.setText("Llaves");

        JCBPDFLls1.setText("Llaveros");
        JCBPDFLls1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFLls1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPPrestsPDF1Layout = new javax.swing.GroupLayout(JPPrestsPDF1);
        JPPrestsPDF1.setLayout(JPPrestsPDF1Layout);
        JPPrestsPDF1Layout.setHorizontalGroup(
            JPPrestsPDF1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestsPDF1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPrestsPDF1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLPrestTablas1)
                    .addGroup(JPPrestsPDF1Layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(JPPrestsPDF1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JCBPDFAlms1)
                            .addComponent(JCBPDFTodas1)
                            .addComponent(JCBPDFLs1)
                            .addComponent(JBPrestsPDF1)
                            .addComponent(JCBPDFLls1))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPPrestsPDF1Layout.setVerticalGroup(
            JPPrestsPDF1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestsPDF1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JLPrestTablas1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JCBPDFTodas1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFAlms1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFLs1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFLls1)
                .addGap(7, 7, 7)
                .addComponent(JBPrestsPDF1)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBActivarPanelPDF1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_white_acrobat.png"))); // NOI18N
        JTBActivarPanelPDF1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBActivarPanelPDF1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPAlmacenesLayout = new javax.swing.GroupLayout(JPAlmacenes);
        JPAlmacenes.setLayout(JPAlmacenesLayout);
        JPAlmacenesLayout.setHorizontalGroup(
            JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAlmacenesLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPAlmacenesLayout.createSequentialGroup()
                        .addGroup(JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane31, javax.swing.GroupLayout.PREFERRED_SIZE, 268, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JPAlmD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGroup(JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPAlmacenesLayout.createSequentialGroup()
                                .addGap(18, 18, 18)
                                .addGroup(JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addComponent(jScrollPane32, javax.swing.GroupLayout.PREFERRED_SIZE, 327, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(jScrollPane33, javax.swing.GroupLayout.PREFERRED_SIZE, 327, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addGroup(JPAlmacenesLayout.createSequentialGroup()
                                .addGap(12, 12, 12)
                                .addGroup(JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(JPLlaveroD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                        .addGap(10, 10, 10)
                        .addComponent(JPAlmsB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JTBActivarPanelPDF1)
                            .addComponent(JPPrestsPDF1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(JTFBuscarLLl, javax.swing.GroupLayout.PREFERRED_SIZE, 107, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(48, Short.MAX_VALUE))
        );
        JPAlmacenesLayout.setVerticalGroup(
            JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAlmacenesLayout.createSequentialGroup()
                .addComponent(JTFBuscarLLl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(6, 6, 6)
                .addGroup(JPAlmacenesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPAlmsB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPAlmacenesLayout.createSequentialGroup()
                        .addGap(11, 11, 11)
                        .addComponent(JTBActivarPanelPDF1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPPrestsPDF1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPAlmacenesLayout.createSequentialGroup()
                        .addComponent(jScrollPane31, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPAlmD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPAlmacenesLayout.createSequentialGroup()
                        .addComponent(jScrollPane32, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jScrollPane33, javax.swing.GroupLayout.PREFERRED_SIZE, 73, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPLlaveroD, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPAlmD.getAccessibleContext().setAccessibleName("JPAlmD");
        JPLlaveroD.getAccessibleContext().setAccessibleName("JPLlaveroD");
        JPAlmsB.getAccessibleContext().setAccessibleName("JPAlmsB");
        JTFBuscarLLl.getAccessibleContext().setAccessibleName("JTFBuscarLLl");

        javax.swing.GroupLayout JPZsLsLlsLayout = new javax.swing.GroupLayout(JPZsLsLls);
        JPZsLsLls.setLayout(JPZsLsLlsLayout);
        JPZsLsLlsLayout.setHorizontalGroup(
            JPZsLsLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZsLsLlsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPZsLsLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPAlmacenes, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPZsLsLlsLayout.createSequentialGroup()
                        .addGap(1624, 1624, 1624)
                        .addComponent(JBAnadirAlmLl))
                    .addGroup(JPZsLsLlsLayout.createSequentialGroup()
                        .addComponent(JPZonas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPZona, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPZsLsLlsLayout.setVerticalGroup(
            JPZsLsLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPZsLsLlsLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPZsLsLlsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPZonas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPZona, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPAlmacenes, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(250, 250, 250)
                .addComponent(JBAnadirAlmLl)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPZona.getAccessibleContext().setAccessibleName("JPZona");
        JBAnadirAlmLl.getAccessibleContext().setAccessibleName("JBAnadirAlmLl");
        JPZonas.getAccessibleContext().setAccessibleName("JPZonas");
        JPAlmacenes.getAccessibleContext().setAccessibleName("JPAlmacenes");

        JTPFuncionalidades.addTab("<html><br><br>Zonas, llaves<br>y llaveros<br><br></html>", JPZsLsLls);
        JPZsLsLls.getAccessibleContext().setAccessibleName("JPZsLsLls");

        JPPrestamos.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPPrestamosComponentShown(evt);
            }
        });

        JPTarjsPrest.setBorder(javax.swing.BorderFactory.createTitledBorder("Tarjetas"));

        JTTarjsPrest.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Fecha", "DNI"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTTarjsPrest.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTTarjsPrestMouseClicked(evt);
            }
        });
        jScrollPane20.setViewportView(JTTarjsPrest);
        JTTarjsPrest.getAccessibleContext().setAccessibleName("JTTarjsPrest");

        javax.swing.GroupLayout JPTarjsPrestLayout = new javax.swing.GroupLayout(JPTarjsPrest);
        JPTarjsPrest.setLayout(JPTarjsPrestLayout);
        JPTarjsPrestLayout.setHorizontalGroup(
            JPTarjsPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjsPrestLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane20, javax.swing.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE)
                .addContainerGap())
        );
        JPTarjsPrestLayout.setVerticalGroup(
            JPTarjsPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTarjsPrestLayout.createSequentialGroup()
                .addComponent(jScrollPane20, javax.swing.GroupLayout.DEFAULT_SIZE, 347, Short.MAX_VALUE)
                .addContainerGap())
        );

        JPLsLlsPrest.setBorder(javax.swing.BorderFactory.createTitledBorder("Llaves y llaveros"));

        JTLsPrest.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Fecha", "DNI"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLsPrest.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLsPrestMouseClicked(evt);
            }
        });
        jScrollPane21.setViewportView(JTLsPrest);
        JTLsPrest.getAccessibleContext().setAccessibleName("JTLsPrest");

        JTLlsPrest.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Código", "Fecha", "DNI"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTLlsPrest.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTLlsPrestMouseClicked(evt);
            }
        });
        jScrollPane22.setViewportView(JTLlsPrest);
        JTLlsPrest.getAccessibleContext().setAccessibleName("JTLlsPrest");

        javax.swing.GroupLayout JPLsLlsPrestLayout = new javax.swing.GroupLayout(JPLsLlsPrest);
        JPLsLlsPrest.setLayout(JPLsLlsPrestLayout);
        JPLsLlsPrestLayout.setHorizontalGroup(
            JPLsLlsPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLsLlsPrestLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPLsLlsPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(jScrollPane21, javax.swing.GroupLayout.Alignment.LEADING, 0, 0, Short.MAX_VALUE)
                    .addComponent(jScrollPane22, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 348, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPLsLlsPrestLayout.setVerticalGroup(
            JPLsLlsPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPLsLlsPrestLayout.createSequentialGroup()
                .addComponent(jScrollPane21, javax.swing.GroupLayout.PREFERRED_SIZE, 166, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane22, javax.swing.GroupLayout.DEFAULT_SIZE, 175, Short.MAX_VALUE)
                .addContainerGap())
        );

        JTTfnsPrest.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Teléfonos"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane34.setViewportView(JTTfnsPrest);
        JTTfnsPrest.getAccessibleContext().setAccessibleName("JTTfnsPrest");

        JBRecordPrest.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/email.png"))); // NOI18N
        JBRecordPrest.setEnabled(false);
        JBRecordPrest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBRecordPrestActionPerformed(evt);
            }
        });

        JPDPPrest.setBackground(new java.awt.Color(255, 255, 255));
        JPDPPrest.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        JLNombrePrest.setText("Nombre");

        JTFNombrePrest.setEditable(false);
        JTFNombrePrest.setBorder(null);

        JLEmpresaPrest.setText("Empresa");

        JTFEmpresaPrest.setEditable(false);
        JTFEmpresaPrest.setBorder(null);

        JLTipoPrest.setText("Tipo");

        JTFTipoPrest.setEditable(false);
        JTFTipoPrest.setBorder(null);

        javax.swing.GroupLayout JPDPPrestLayout = new javax.swing.GroupLayout(JPDPPrest);
        JPDPPrest.setLayout(JPDPPrestLayout);
        JPDPPrestLayout.setHorizontalGroup(
            JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDPPrestLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPDPPrestLayout.createSequentialGroup()
                        .addComponent(JLNombrePrest)
                        .addGap(22, 22, 22)
                        .addComponent(JTFNombrePrest, javax.swing.GroupLayout.PREFERRED_SIZE, 196, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPDPPrestLayout.createSequentialGroup()
                        .addGroup(JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JLTipoPrest)
                            .addComponent(JLEmpresaPrest))
                        .addGap(18, 18, 18)
                        .addGroup(JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JTFEmpresaPrest, javax.swing.GroupLayout.DEFAULT_SIZE, 196, Short.MAX_VALUE)
                            .addComponent(JTFTipoPrest, javax.swing.GroupLayout.DEFAULT_SIZE, 196, Short.MAX_VALUE))))
                .addGap(182, 182, 182))
        );
        JPDPPrestLayout.setVerticalGroup(
            JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDPPrestLayout.createSequentialGroup()
                .addGap(43, 43, 43)
                .addGroup(JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLNombrePrest)
                    .addComponent(JTFNombrePrest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLTipoPrest)
                    .addComponent(JTFTipoPrest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPDPPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLEmpresaPrest)
                    .addComponent(JTFEmpresaPrest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(72, Short.MAX_VALUE))
        );

        JLNombrePrest.getAccessibleContext().setAccessibleName("JLNombrePrest");
        JTFNombrePrest.getAccessibleContext().setAccessibleName("JTFNombrePrest");
        JLEmpresaPrest.getAccessibleContext().setAccessibleName("JLEmpresaPrest");
        JTFEmpresaPrest.getAccessibleContext().setAccessibleName("JTFEmpresaPrest");
        JLTipoPrest.getAccessibleContext().setAccessibleName("JLTipoPrest");
        JTFTipoPrest.getAccessibleContext().setAccessibleName("JTFTipoPrest");

        JBDevolver.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_right.png"))); // NOI18N
        JBDevolver.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBDevolverActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPDatosPrestLayout = new javax.swing.GroupLayout(JPDatosPrest);
        JPDatosPrest.setLayout(JPDatosPrestLayout);
        JPDatosPrestLayout.setHorizontalGroup(
            JPDatosPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPDatosPrestLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JPDPPrest, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPDatosPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPDatosPrestLayout.createSequentialGroup()
                        .addComponent(JBRecordPrest)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JBDevolver))
                    .addComponent(jScrollPane34, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPDatosPrestLayout.setVerticalGroup(
            JPDatosPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDatosPrestLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDatosPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(JPDPPrest, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPDatosPrestLayout.createSequentialGroup()
                        .addComponent(jScrollPane34, javax.swing.GroupLayout.PREFERRED_SIZE, 152, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPDatosPrestLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JBRecordPrest)
                            .addComponent(JBDevolver))))
                .addContainerGap(21, Short.MAX_VALUE))
        );

        JBRecordPrest.getAccessibleContext().setAccessibleName("JBRecordPrest");
        JPDPPrest.getAccessibleContext().setAccessibleName("JPDPPrest");
        JBDevolver.getAccessibleContext().setAccessibleName("JBDevolver");

        JBPrestsPDF.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBPrestsPDF.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBPrestsPDFActionPerformed(evt);
            }
        });

        JLPrestTablas.setText("Tablas a reportar:");

        JCBPDFTodas.setText("Todas");
        JCBPDFTodas.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFTodasActionPerformed(evt);
            }
        });

        JCBPDFTarjs.setText("Tarjetas");
        JCBPDFTarjs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFTarjsActionPerformed(evt);
            }
        });

        JCBPDFLs.setText("Llaves");
        JCBPDFLs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFLsActionPerformed(evt);
            }
        });

        JCBPDFLls.setText("Llaveros");
        JCBPDFLls.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBPDFLlsActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPPrestsPDFLayout = new javax.swing.GroupLayout(JPPrestsPDF);
        JPPrestsPDF.setLayout(JPPrestsPDFLayout);
        JPPrestsPDFLayout.setHorizontalGroup(
            JPPrestsPDFLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestsPDFLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPrestsPDFLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLPrestTablas)
                    .addGroup(JPPrestsPDFLayout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(JPPrestsPDFLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JCBPDFTarjs)
                            .addComponent(JCBPDFTodas)
                            .addComponent(JCBPDFLs)
                            .addComponent(JBPrestsPDF)
                            .addComponent(JCBPDFLls))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPPrestsPDFLayout.setVerticalGroup(
            JPPrestsPDFLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestsPDFLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JLPrestTablas)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(JCBPDFTodas)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFTarjs)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFLs)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JCBPDFLls)
                .addGap(7, 7, 7)
                .addComponent(JBPrestsPDF)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBPrestsPDF.getAccessibleContext().setAccessibleName("JBPrestsPDF");
        JLPrestTablas.getAccessibleContext().setAccessibleName("JLPrestTablas");
        JCBPDFTodas.getAccessibleContext().setAccessibleName("JCBPDFTodas");
        JCBPDFTarjs.getAccessibleContext().setAccessibleName("JCBPDFTarjs");
        JCBPDFLs.getAccessibleContext().setAccessibleName("JCBPDFLs");
        JCBPDFLls.getAccessibleContext().setAccessibleName("JCBPDFLls");

        JTBActivarPanelPDF.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_white_acrobat.png"))); // NOI18N
        JTBActivarPanelPDF.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBActivarPanelPDFActionPerformed(evt);
            }
        });

        JBDeseleccionar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/arrow_undo.png"))); // NOI18N
        JBDeseleccionar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBDeseleccionarActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPPrestamosLayout = new javax.swing.GroupLayout(JPPrestamos);
        JPPrestamos.setLayout(JPPrestamosLayout);
        JPPrestamosLayout.setHorizontalGroup(
            JPPrestamosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestamosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPrestamosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPPrestamosLayout.createSequentialGroup()
                        .addComponent(JPTarjsPrest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPLsLlsPrest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPPrestamosLayout.createSequentialGroup()
                        .addComponent(JPDatosPrest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPPrestamosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(JPPrestamosLayout.createSequentialGroup()
                                .addComponent(JTBActivarPanelPDF)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(JBDeseleccionar))
                            .addComponent(JPPrestsPDF, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(370, Short.MAX_VALUE))
        );
        JPPrestamosLayout.setVerticalGroup(
            JPPrestamosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPrestamosLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPrestamosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(JPLsLlsPrest, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(JPTarjsPrest, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPPrestamosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPDatosPrest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPPrestamosLayout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(JPPrestamosLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTBActivarPanelPDF)
                            .addComponent(JBDeseleccionar))
                        .addGap(4, 4, 4)
                        .addComponent(JPPrestsPDF, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(373, 373, 373))
        );

        JPTarjsPrest.getAccessibleContext().setAccessibleName("JPTarjsPrest");
        JPLsLlsPrest.getAccessibleContext().setAccessibleName("JPLsLlsPrest");
        JPDatosPrest.getAccessibleContext().setAccessibleName("JPDatosPrest");
        JPPrestsPDF.getAccessibleContext().setAccessibleName("JPPrestsPDF");
        JTBActivarPanelPDF.getAccessibleContext().setAccessibleName("JTBActivarPanelPDF");
        JBDeseleccionar.getAccessibleContext().setAccessibleName("JBDeseleccionar");

        JTPFuncionalidades.addTab("<html><br><br>Préstamos<br><br></html>", JPPrestamos);
        JPPrestamos.getAccessibleContext().setAccessibleName("JPPrestamos");

        JPCuentas.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPCuentasComponentShown(evt);
            }
        });

        JPInfoCuentas.setBorder(javax.swing.BorderFactory.createTitledBorder("Cuentas"));

        JTCuentas.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "DNI", "Tipo", "Usuario", "Contraseña"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTCuentas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTCuentasMouseClicked(evt);
            }
        });
        jScrollPane5.setViewportView(JTCuentas);
        JTCuentas.getAccessibleContext().setAccessibleName("JTCuentas");

        JLDNICuenta.setText("DNI");

        JCBDNICuenta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBDNICuentaActionPerformed(evt);
            }
        });
        JCBDNICuenta.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JCBDNICuentaKeyReleased(evt);
            }
        });

        JLTipoCuenta.setText("Tipo");

        JLUsuarioCuenta.setText("Usuario");

        JLPassCuenta.setText("Contraseña");

        JBGrabarCuenta.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarCuenta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarCuentaActionPerformed(evt);
            }
        });

        JTFBuscarCuenta.setForeground(new java.awt.Color(204, 204, 204));
        JTFBuscarCuenta.setText("Buscar ...");
        JTFBuscarCuenta.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFBuscarCuentaMouseClicked(evt);
            }
        });
        JTFBuscarCuenta.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFBuscarCuentaKeyReleased(evt);
            }
            public void keyTyped(java.awt.event.KeyEvent evt) {
                JTFBuscarCuentaKeyTyped(evt);
            }
        });

        javax.swing.GroupLayout JPDatosCuentaLayout = new javax.swing.GroupLayout(JPDatosCuenta);
        JPDatosCuenta.setLayout(JPDatosCuentaLayout);
        JPDatosCuentaLayout.setHorizontalGroup(
            JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDatosCuentaLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLPassCuenta)
                    .addComponent(JLDNICuenta)
                    .addComponent(JLUsuarioCuenta)
                    .addComponent(JLTipoCuenta))
                .addGap(10, 10, 10)
                .addGroup(JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPDatosCuentaLayout.createSequentialGroup()
                        .addComponent(JCBDNICuenta, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JTFBuscarCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, 154, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(6, 6, 6)
                        .addComponent(JBGrabarCuenta))
                    .addComponent(JCBTipoCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JTFUsuarioCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JTFPassCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPDatosCuentaLayout.setVerticalGroup(
            JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDatosCuentaLayout.createSequentialGroup()
                .addGap(12, 12, 12)
                .addGroup(JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JBGrabarCuenta)
                    .addGroup(JPDatosCuentaLayout.createSequentialGroup()
                        .addGroup(JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTFBuscarCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JCBDNICuenta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLDNICuenta))
                        .addGap(5, 5, 5)
                        .addGroup(JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JCBTipoCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLTipoCuenta))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTFUsuarioCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLUsuarioCuenta))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPDatosCuentaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLPassCuenta)
                            .addComponent(JTFPassCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap())
        );

        JLDNICuenta.getAccessibleContext().setAccessibleName("JLDNICuenta");
        JCBDNICuenta.getAccessibleContext().setAccessibleName("JCBDNICuenta");
        JCBTipoCuenta.getAccessibleContext().setAccessibleName("JCBTipoCuenta");
        JTFUsuarioCuenta.getAccessibleContext().setAccessibleName("JTFUsuarioCuenta");
        JTFPassCuenta.getAccessibleContext().setAccessibleName("JTFPassCuenta");
        JLTipoCuenta.getAccessibleContext().setAccessibleName("JLTipoCuenta");
        JLUsuarioCuenta.getAccessibleContext().setAccessibleName("JLUsuarioCuenta");
        JLPassCuenta.getAccessibleContext().setAccessibleName("JLPassCuenta");
        JBGrabarCuenta.getAccessibleContext().setAccessibleName("JBGrabarCuenta");
        JTFBuscarCuenta.getAccessibleContext().setAccessibleName("JTFBuscarCuenta");

        JTBModificarCuenta.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_edit.png"))); // NOI18N
        JTBModificarCuenta.setEnabled(false);
        JTBModificarCuenta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarCuentaActionPerformed(evt);
            }
        });

        JBEliminarCuenta.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_delete.png"))); // NOI18N
        JBEliminarCuenta.setEnabled(false);
        JBEliminarCuenta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarCuentaActionPerformed(evt);
            }
        });

        JTBAnadirCuenta.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_add.png"))); // NOI18N
        JTBAnadirCuenta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirCuentaActionPerformed(evt);
            }
        });

        JBPDFCuentas.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_white_acrobat.png"))); // NOI18N
        JBPDFCuentas.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBPDFCuentasActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPCuentasBLayout = new javax.swing.GroupLayout(JPCuentasB);
        JPCuentasB.setLayout(JPCuentasBLayout);
        JPCuentasBLayout.setHorizontalGroup(
            JPCuentasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCuentasBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirCuenta)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarCuenta)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarCuenta)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBPDFCuentas)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPCuentasBLayout.setVerticalGroup(
            JPCuentasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCuentasBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPCuentasBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTBAnadirCuenta)
                    .addComponent(JBEliminarCuenta)
                    .addComponent(JTBModificarCuenta)
                    .addComponent(JBPDFCuentas))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBModificarCuenta.getAccessibleContext().setAccessibleName("JTBModificarCuenta");
        JBEliminarCuenta.getAccessibleContext().setAccessibleName("JBEliminarCuenta");
        JTBAnadirCuenta.getAccessibleContext().setAccessibleName("JTBAnadirCuenta");
        JBPDFCuentas.getAccessibleContext().setAccessibleName("JBPDFCuentas");

        javax.swing.GroupLayout JPInfoCuentasLayout = new javax.swing.GroupLayout(JPInfoCuentas);
        JPInfoCuentas.setLayout(JPInfoCuentasLayout);
        JPInfoCuentasLayout.setHorizontalGroup(
            JPInfoCuentasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPInfoCuentasLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPInfoCuentasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane5, javax.swing.GroupLayout.DEFAULT_SIZE, 517, Short.MAX_VALUE)
                    .addComponent(JPCuentasB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPDatosCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPInfoCuentasLayout.setVerticalGroup(
            JPInfoCuentasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPInfoCuentasLayout.createSequentialGroup()
                .addComponent(jScrollPane5, javax.swing.GroupLayout.PREFERRED_SIZE, 171, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPCuentasB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPDatosCuenta, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        JPDatosCuenta.getAccessibleContext().setAccessibleName("JPDatosCuenta");
        JPCuentasB.getAccessibleContext().setAccessibleName("JPCuentasB");

        JPHistorialAccesosSist.setBorder(javax.swing.BorderFactory.createTitledBorder("Historial de accesos al sistema"));

        JTHistorialAccesosSist.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Usuario", "Tipo", "DNI", "Fecha"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane36.setViewportView(JTHistorialAccesosSist);
        JTHistorialAccesosSist.getAccessibleContext().setAccessibleName("JTHistorialAccesosSist");

        JTFBuscarAccesosSist.setForeground(new java.awt.Color(204, 204, 204));
        JTFBuscarAccesosSist.setText("Buscar ...");
        JTFBuscarAccesosSist.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFBuscarAccesosSistMouseClicked(evt);
            }
        });
        JTFBuscarAccesosSist.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFBuscarAccesosSistKeyReleased(evt);
            }
        });

        JCCFechaFinAccsSist.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFechaFinAccsSistDateChanged(evt);
            }
        });
        JCCFechaFinAccsSist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCCFechaFinAccsSistActionPerformed(evt);
            }
        });

        JCCFechaInicAccsSist.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JCCFechaInicAccsSistMouseClicked(evt);
            }
        });
        JCCFechaInicAccsSist.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFechaInicAccsSistDateChanged(evt);
            }
        });
        JCCFechaInicAccsSist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCCFechaInicAccsSistActionPerformed(evt);
            }
        });

        JCBHoraInicAccsSist.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHoraInicAccsSist.setEnabled(false);
        JCBHoraInicAccsSist.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHoraInicAccsSistItemStateChanged(evt);
            }
        });
        JCBHoraInicAccsSist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHoraInicAccsSistActionPerformed(evt);
            }
        });

        JCBHoraFinAccsSist.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHoraFinAccsSist.setEnabled(false);
        JCBHoraFinAccsSist.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHoraFinAccsSistItemStateChanged(evt);
            }
        });
        JCBHoraFinAccsSist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHoraFinAccsSistActionPerformed(evt);
            }
        });

        JCBTodoDiaAccsSist.setSelected(true);
        JCBTodoDiaAccsSist.setText("Todo el día");
        JCBTodoDiaAccsSist.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                JCBTodoDiaAccsSistStateChanged(evt);
            }
        });
        JCBTodoDiaAccsSist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBTodoDiaAccsSistActionPerformed(evt);
            }
        });

        JBPDFAccsSist.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_white_acrobat.png"))); // NOI18N
        JBPDFAccsSist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBPDFAccsSistActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPAccsSistBLayout = new javax.swing.GroupLayout(JPAccsSistB);
        JPAccsSistB.setLayout(JPAccsSistBLayout);
        JPAccsSistBLayout.setHorizontalGroup(
            JPAccsSistBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAccsSistBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAccsSistBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPAccsSistBLayout.createSequentialGroup()
                        .addComponent(JCCFechaInicAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JCBHoraInicAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JCBTodoDiaAccsSist))
                    .addGroup(JPAccsSistBLayout.createSequentialGroup()
                        .addComponent(JCCFechaFinAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JCBHoraFinAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(JBPDFAccsSist)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPAccsSistBLayout.setVerticalGroup(
            JPAccsSistBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPAccsSistBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPAccsSistBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPAccsSistBLayout.createSequentialGroup()
                        .addGap(1, 1, 1)
                        .addComponent(JCCFechaInicAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(JCBHoraInicAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JCBTodoDiaAccsSist))
                .addGap(4, 4, 4)
                .addGroup(JPAccsSistBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JCCFechaFinAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPAccsSistBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(JCBHoraFinAccsSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(JBPDFAccsSist)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JCCFechaFinAccsSist.getAccessibleContext().setAccessibleName("JCCFechaFinAccsSist");
        JCCFechaInicAccsSist.getAccessibleContext().setAccessibleName("JCCFechaInicAccsSist");
        JCBHoraInicAccsSist.getAccessibleContext().setAccessibleName("JCBHoraInicAccsSist");
        JCBHoraFinAccsSist.getAccessibleContext().setAccessibleName("JCBHoraFinAccsSist");
        JCBTodoDiaAccsSist.getAccessibleContext().setAccessibleName("JCBTodoDiaAccsSist");
        JBPDFAccsSist.getAccessibleContext().setAccessibleName("JBPDFAccsSist");

        javax.swing.GroupLayout JPHistorialAccesosSistLayout = new javax.swing.GroupLayout(JPHistorialAccesosSist);
        JPHistorialAccesosSist.setLayout(JPHistorialAccesosSistLayout);
        JPHistorialAccesosSistLayout.setHorizontalGroup(
            JPHistorialAccesosSistLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPHistorialAccesosSistLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPHistorialAccesosSistLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane36, javax.swing.GroupLayout.DEFAULT_SIZE, 902, Short.MAX_VALUE)
                    .addGroup(JPHistorialAccesosSistLayout.createSequentialGroup()
                        .addComponent(JTFBuscarAccesosSist, javax.swing.GroupLayout.PREFERRED_SIZE, 143, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 422, Short.MAX_VALUE)
                        .addComponent(JPAccsSistB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        JPHistorialAccesosSistLayout.setVerticalGroup(
            JPHistorialAccesosSistLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPHistorialAccesosSistLayout.createSequentialGroup()
                .addGroup(JPHistorialAccesosSistLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPAccsSistB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPHistorialAccesosSistLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(JTFBuscarAccesosSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane36, javax.swing.GroupLayout.PREFERRED_SIZE, 143, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        JTFBuscarAccesosSist.getAccessibleContext().setAccessibleName("JTFBuscarAccesosSist");
        JPAccsSistB.getAccessibleContext().setAccessibleName("JPAccsSistB");

        JPTUsuarios1.setBorder(javax.swing.BorderFactory.createTitledBorder("Tipos de usuario"));

        JTBModificarTUsuario.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_orange.png"))); // NOI18N
        JTBModificarTUsuario.setEnabled(false);
        JTBModificarTUsuario.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBModificarTUsuarioActionPerformed(evt);
            }
        });

        JBEliminarTUsuario.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_red.png"))); // NOI18N
        JBEliminarTUsuario.setEnabled(false);
        JBEliminarTUsuario.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarTUsuarioActionPerformed(evt);
            }
        });

        JTBAnadirTUsuario.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/user_green.png"))); // NOI18N
        JTBAnadirTUsuario.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JTBAnadirTUsuarioActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPTUsuariosBLayout = new javax.swing.GroupLayout(JPTUsuariosB);
        JPTUsuariosB.setLayout(JPTUsuariosBLayout);
        JPTUsuariosBLayout.setHorizontalGroup(
            JPTUsuariosBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTUsuariosBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTBAnadirTUsuario)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarTUsuario)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTBModificarTUsuario)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPTUsuariosBLayout.setVerticalGroup(
            JPTUsuariosBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTUsuariosBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTUsuariosBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(JTBAnadirTUsuario)
                    .addGroup(JPTUsuariosBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(JBEliminarTUsuario)
                        .addComponent(JTBModificarTUsuario)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JTBModificarTUsuario.getAccessibleContext().setAccessibleName("JTBModificarTUsuario");
        JBEliminarTUsuario.getAccessibleContext().setAccessibleName("JBEliminarTUsuario");
        JTBAnadirTUsuario.getAccessibleContext().setAccessibleName("JTBAnadirTUsuario");

        JLTipoTUsuario.setText("Tipo");

        JLDescripcionTUsuario.setText("Descripción");

        JBGrabarTUsuario.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBGrabarTUsuario.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBGrabarTUsuarioActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPDatosTUsuarioLayout = new javax.swing.GroupLayout(JPDatosTUsuario);
        JPDatosTUsuario.setLayout(JPDatosTUsuarioLayout);
        JPDatosTUsuarioLayout.setHorizontalGroup(
            JPDatosTUsuarioLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDatosTUsuarioLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDatosTUsuarioLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLTipoTUsuario)
                    .addComponent(JLDescripcionTUsuario))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPDatosTUsuarioLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JTFDescripTUsuario, javax.swing.GroupLayout.DEFAULT_SIZE, 184, Short.MAX_VALUE)
                    .addComponent(JTFTUsuario, javax.swing.GroupLayout.DEFAULT_SIZE, 184, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBGrabarTUsuario)
                .addContainerGap())
        );
        JPDatosTUsuarioLayout.setVerticalGroup(
            JPDatosTUsuarioLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPDatosTUsuarioLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPDatosTUsuarioLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JBGrabarTUsuario)
                    .addGroup(JPDatosTUsuarioLayout.createSequentialGroup()
                        .addGroup(JPDatosTUsuarioLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLTipoTUsuario)
                            .addComponent(JTFTUsuario, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(JPDatosTUsuarioLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JLDescripcionTUsuario)
                            .addComponent(JTFDescripTUsuario, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JLTipoTUsuario.getAccessibleContext().setAccessibleName("JLTipoTUsuario");
        JLDescripcionTUsuario.getAccessibleContext().setAccessibleName("JLDescripcionTUsuario");
        JTFDescripTUsuario.getAccessibleContext().setAccessibleName("JTFDescripTUsuario");
        JTFTUsuario.getAccessibleContext().setAccessibleName("JTFTUsuario");
        JBGrabarTUsuario.getAccessibleContext().setAccessibleName("JBGrabarTUsuario");

        JTTUsuarios.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Tipo", "Descripción"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTTUsuarios.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTTUsuariosMouseClicked(evt);
            }
        });
        jScrollPane6.setViewportView(JTTUsuarios);
        JTTUsuarios.getAccessibleContext().setAccessibleName("JTTUsuarios");

        javax.swing.GroupLayout JPTUsuarios1Layout = new javax.swing.GroupLayout(JPTUsuarios1);
        JPTUsuarios1.setLayout(JPTUsuarios1Layout);
        JPTUsuarios1Layout.setHorizontalGroup(
            JPTUsuarios1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTUsuarios1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPTUsuarios1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane6, javax.swing.GroupLayout.DEFAULT_SIZE, 393, Short.MAX_VALUE)
                    .addComponent(JPTUsuariosB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPDatosTUsuario, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        JPTUsuarios1Layout.setVerticalGroup(
            JPTUsuarios1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPTUsuarios1Layout.createSequentialGroup()
                .addComponent(jScrollPane6, javax.swing.GroupLayout.PREFERRED_SIZE, 170, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPTUsuariosB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPDatosTUsuario, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPTUsuariosB.getAccessibleContext().setAccessibleName("JPTUsuariosB");
        JPDatosTUsuario.getAccessibleContext().setAccessibleName("JPDatosTUsuario");

        javax.swing.GroupLayout JPCuentasLayout = new javax.swing.GroupLayout(JPCuentas);
        JPCuentas.setLayout(JPCuentasLayout);
        JPCuentasLayout.setHorizontalGroup(
            JPCuentasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCuentasLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPCuentasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPHistorialAccesosSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPCuentasLayout.createSequentialGroup()
                        .addComponent(JPInfoCuentas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPTUsuarios1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(108, Short.MAX_VALUE))
        );
        JPCuentasLayout.setVerticalGroup(
            JPCuentasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPCuentasLayout.createSequentialGroup()
                .addGap(11, 11, 11)
                .addGroup(JPCuentasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPInfoCuentas, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JPTUsuarios1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JPHistorialAccesosSist, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JPInfoCuentas.getAccessibleContext().setAccessibleName("JPInfoCuentas");
        JPHistorialAccesosSist.getAccessibleContext().setAccessibleName("JPHistorialAccesosSist");
        JPTUsuarios1.getAccessibleContext().setAccessibleName("JPTUsuarios1");

        JTPFuncionalidades.addTab("<html><br><br>Cuentas<br><br></html>", JPCuentas);
        JPCuentas.getAccessibleContext().setAccessibleName("JPCuentas");

        JPPartes.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPPartesComponentShown(evt);
            }
        });

        JPParte.setBackground(new java.awt.Color(255, 255, 255));
        JPParte.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        JLIncidencias.setText("Incidencias");

        JLCodVig.setText("Código de vigilante");

        JTFCodVig.setEditable(false);

        JLFecha.setText("Fecha");

        JTAIncidencias.setColumns(20);
        JTAIncidencias.setEditable(false);
        JTAIncidencias.setLineWrap(true);
        JTAIncidencias.setRows(5);
        JTAIncidencias.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTAIncidenciasKeyReleased(evt);
            }
        });
        jScrollPane3.setViewportView(JTAIncidencias);
        JTAIncidencias.getAccessibleContext().setAccessibleName("JTAIncidencias");

        JCCFechaParte.setEnabled(false);
        JCCFechaParte.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                JCCFechaParteMouseReleased(evt);
            }
        });
        JCCFechaParte.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFechaParteDateChanged(evt);
            }
        });

        JCBParte.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBParte.setEnabled(false);
        JCBParte.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                JCBParteMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                JCBParteMouseReleased(evt);
            }
        });
        JCBParte.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBParteItemStateChanged(evt);
            }
        });
        JCBParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBParteActionPerformed(evt);
            }
        });

        JLPartesLogo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Imgs/CIS_Logo.png"))); // NOI18N

        JPPDFB.setBackground(new java.awt.Color(255, 255, 255));

        JBPDFParte.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_white_acrobat.png"))); // NOI18N
        JBPDFParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBPDFParteActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPPDFBLayout = new javax.swing.GroupLayout(JPPDFB);
        JPPDFB.setLayout(JPPDFBLayout);
        JPPDFBLayout.setHorizontalGroup(
            JPPDFBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPDFBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBPDFParte)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPPDFBLayout.setVerticalGroup(
            JPPDFBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPDFBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBPDFParte)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBPDFParte.getAccessibleContext().setAccessibleName("JBPDFParte");

        javax.swing.GroupLayout JPParteLayout = new javax.swing.GroupLayout(JPParte);
        JPParte.setLayout(JPParteLayout);
        JPParteLayout.setHorizontalGroup(
            JPParteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPParteLayout.createSequentialGroup()
                .addGroup(JPParteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(JPParteLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(JLIncidencias)
                        .addGap(18, 18, 18)
                        .addGroup(JPParteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(JPParteLayout.createSequentialGroup()
                                .addComponent(JLCodVig)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(JTFCodVig, javax.swing.GroupLayout.PREFERRED_SIZE, 55, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addComponent(JLFecha)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(JCCFechaParte, javax.swing.GroupLayout.PREFERRED_SIZE, 225, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(JCBParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(jScrollPane3))
                        .addGap(17, 17, 17))
                    .addGroup(JPParteLayout.createSequentialGroup()
                        .addGap(9, 9, 9)
                        .addComponent(JLPartesLogo)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(JPPDFB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(20, Short.MAX_VALUE))
        );
        JPParteLayout.setVerticalGroup(
            JPParteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPParteLayout.createSequentialGroup()
                .addGap(11, 11, 11)
                .addGroup(JPParteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLPartesLogo)
                    .addComponent(JPPDFB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(21, 21, 21)
                .addGroup(JPParteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodVig)
                    .addComponent(JTFCodVig, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JCBParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JCCFechaParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JLFecha))
                .addGap(41, 41, 41)
                .addGroup(JPParteLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLIncidencias)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 379, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(40, Short.MAX_VALUE))
        );

        JLIncidencias.getAccessibleContext().setAccessibleName("JLIncidencias");
        JLCodVig.getAccessibleContext().setAccessibleName("JLCodVig");
        JTFCodVig.getAccessibleContext().setAccessibleName("JTFCodVig");
        JLFecha.getAccessibleContext().setAccessibleName("JLFecha");
        JCCFechaParte.getAccessibleContext().setAccessibleName("JCCFechaParte");
        JCBParte.getAccessibleContext().setAccessibleName("JCBParte");
        JLPartesLogo.getAccessibleContext().setAccessibleName("JLPartesLogo");
        JPPDFB.getAccessibleContext().setAccessibleName("JPPDFB");

        JTPartes.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Fecha"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTPartes.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTPartesMouseClicked(evt);
            }
        });
        jScrollPane4.setViewportView(JTPartes);
        JTPartes.getAccessibleContext().setAccessibleName("JTPartes");

        JCBCodVig.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBCodVigActionPerformed(evt);
            }
        });

        JCBFechasPartes.setText("Activar fechas");
        JCBFechasPartes.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBFechasPartesActionPerformed(evt);
            }
        });

        JCCFechaInicParte.setEnabled(false);
        JCCFechaInicParte.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFechaInicParteDateChanged(evt);
            }
        });
        JCCFechaInicParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCCFechaInicParteActionPerformed(evt);
            }
        });

        JCCFechaFinParte.setEnabled(false);
        JCCFechaFinParte.addDateListener(new org.freixas.jcalendar.DateListener() {
            public void dateChanged(org.freixas.jcalendar.DateEvent evt) {
                JCCFechaFinParteDateChanged(evt);
            }
        });
        JCCFechaFinParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCCFechaFinParteActionPerformed(evt);
            }
        });

        JCBTodoDiaParte.setSelected(true);
        JCBTodoDiaParte.setText("Todo el día");
        JCBTodoDiaParte.setEnabled(false);
        JCBTodoDiaParte.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                JCBTodoDiaParteStateChanged(evt);
            }
        });
        JCBTodoDiaParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBTodoDiaParteActionPerformed(evt);
            }
        });

        JCBHoraFinParte.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHoraFinParte.setEnabled(false);
        JCBHoraFinParte.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHoraFinParteItemStateChanged(evt);
            }
        });
        JCBHoraFinParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHoraFinParteActionPerformed(evt);
            }
        });

        JCBHoraInicParte.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "00:00", "00:30", "01:00", "01:30", "02:00", "02:30", "03:00", "03:30", "04:00", "04:30", "05:00", "05:30", "06:00", "06:30", "07:00", "07:30", "08:00", "08:30", "09:00", "09:30", "10:00", "10:30", "11:00", "11:30", "12:00", "12:30", "13:00", "13:30", "14:00", "14:30", "15:00", "15:30", "16:00", "16:30", "17:00", "17:30", "18:00", "18:30", "19:00", "19:30", "20:00", "20:30", "21:00", "21:30", "22:00", "22:30", "23:00", "23:30" }));
        JCBHoraInicParte.setEnabled(false);
        JCBHoraInicParte.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                JCBHoraInicParteItemStateChanged(evt);
            }
        });
        JCBHoraInicParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JCBHoraInicParteActionPerformed(evt);
            }
        });

        JLCodVigParte.setText("Cod. Vig.");

        javax.swing.GroupLayout JPPartesBuscarBLayout = new javax.swing.GroupLayout(JPPartesBuscarB);
        JPPartesBuscarB.setLayout(JPPartesBuscarBLayout);
        JPPartesBuscarBLayout.setHorizontalGroup(
            JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPartesBuscarBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JCCFechaFinParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPPartesBuscarBLayout.createSequentialGroup()
                        .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(JCCFechaInicParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JCBFechasPartes)
                            .addGroup(JPPartesBuscarBLayout.createSequentialGroup()
                                .addComponent(JLCodVigParte)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(JCBCodVig, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(JCBTodoDiaParte)
                            .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                .addComponent(JCBHoraFinParte, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(JCBHoraInicParte, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPPartesBuscarBLayout.setVerticalGroup(
            JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPartesBuscarBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JLCodVigParte)
                    .addComponent(JCBCodVig, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JCBFechasPartes)
                    .addComponent(JCBTodoDiaParte))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JCCFechaInicParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JCBHoraInicParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPPartesBuscarBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JCCFechaFinParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JCBHoraFinParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JCBCodVig.getAccessibleContext().setAccessibleName("JCBCodVig");
        JCBFechasPartes.getAccessibleContext().setAccessibleName("JCBFechasPartes");
        JCCFechaInicParte.getAccessibleContext().setAccessibleName("JCCFechaInicParte");
        JCCFechaFinParte.getAccessibleContext().setAccessibleName("JCCFechaFinParte");
        JCBTodoDiaParte.getAccessibleContext().setAccessibleName("JCBTodoDiaParte");
        JCBHoraFinParte.getAccessibleContext().setAccessibleName("JCBHoraFinParte");
        JCBHoraInicParte.getAccessibleContext().setAccessibleName("JCBHoraInicParte");
        JLCodVigParte.getAccessibleContext().setAccessibleName("JLCodVigParte");

        JBAnadirParte.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/report_add.png"))); // NOI18N
        JBAnadirParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBAnadirParteActionPerformed(evt);
            }
        });

        JBEliminarParte.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/report_delete.png"))); // NOI18N
        JBEliminarParte.setEnabled(false);
        JBEliminarParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEliminarParteActionPerformed(evt);
            }
        });

        JBModificarParte.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/accept.png"))); // NOI18N
        JBModificarParte.setEnabled(false);
        JBModificarParte.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBModificarParteActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPPartesBLayout = new javax.swing.GroupLayout(JPPartesB);
        JPPartesB.setLayout(JPPartesBLayout);
        JPPartesBLayout.setHorizontalGroup(
            JPPartesBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPartesBLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JBAnadirParte)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBEliminarParte)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JBModificarParte)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        JPPartesBLayout.setVerticalGroup(
            JPPartesBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPartesBLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPPartesBLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JBAnadirParte)
                    .addComponent(JBEliminarParte)
                    .addComponent(JBModificarParte))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        JBAnadirParte.getAccessibleContext().setAccessibleName("JBAnadirParte");
        JBEliminarParte.getAccessibleContext().setAccessibleName("JBEliminarParte");
        JBModificarParte.getAccessibleContext().setAccessibleName("JBModificarParte");

        javax.swing.GroupLayout JPPartesLayout = new javax.swing.GroupLayout(JPPartes);
        JPPartes.setLayout(JPPartesLayout);
        JPPartesLayout.setHorizontalGroup(
            JPPartesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPartesLayout.createSequentialGroup()
                .addGroup(JPPartesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPPartesLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(JPPartesBuscarB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPPartesLayout.createSequentialGroup()
                        .addGap(20, 20, 20)
                        .addGroup(JPPartesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane4, javax.swing.GroupLayout.PREFERRED_SIZE, 272, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JPPartesB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addGap(18, 18, 18)
                .addComponent(JPParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(752, 752, 752))
        );
        JPPartesLayout.setVerticalGroup(
            JPPartesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPPartesLayout.createSequentialGroup()
                .addGap(11, 11, 11)
                .addGroup(JPPartesLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPParte, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(JPPartesLayout.createSequentialGroup()
                        .addComponent(JPPartesBuscarB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jScrollPane4, javax.swing.GroupLayout.PREFERRED_SIZE, 177, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JPPartesB, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(15, Short.MAX_VALUE))
        );

        JPParte.getAccessibleContext().setAccessibleName("JPParte");
        JPPartesBuscarB.getAccessibleContext().setAccessibleName("JPPartesBuscarB");
        JPPartesB.getAccessibleContext().setAccessibleName("JPPartesB");

        JTPFuncionalidades.addTab("<html><br><br>Partes de seguridad<br><br></html>", JPPartes);
        JPPartes.getAccessibleContext().setAccessibleName("JPPartes");

        JPNotas.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                JPNotasComponentShown(evt);
            }
        });

        JTPNotas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTPNotasMouseClicked(evt);
            }
        });

        JPRedactar.setInheritsPopupMenu(true);

        JTATexto.setColumns(20);
        JTATexto.setRows(5);
        jScrollPane7.setViewportView(JTATexto);
        JTATexto.getAccessibleContext().setAccessibleName("JTATexto");

        JLAsunto.setText("Asunto");

        JPContactosNotas.setBorder(javax.swing.BorderFactory.createTitledBorder("Contactos"));

        JTContactosNotas.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Primer apellido", "Segundo apellido", "Nombre"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTContactosNotas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTContactosNotasMouseClicked(evt);
            }
        });
        jScrollPane8.setViewportView(JTContactosNotas);
        JTContactosNotas.getAccessibleContext().setAccessibleName("JTContactosNotas");

        JBEnviar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/note_go.png"))); // NOI18N
        JBEnviar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBEnviarActionPerformed(evt);
            }
        });

        JTFBuscarContac.setForeground(new java.awt.Color(204, 204, 204));
        JTFBuscarContac.setText("Buscar ...");
        JTFBuscarContac.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTFBuscarContacMouseClicked(evt);
            }
        });
        JTFBuscarContac.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                JTFBuscarContacKeyReleased(evt);
            }
        });

        javax.swing.GroupLayout JPContactosNotasLayout = new javax.swing.GroupLayout(JPContactosNotas);
        JPContactosNotas.setLayout(JPContactosNotasLayout);
        JPContactosNotasLayout.setHorizontalGroup(
            JPContactosNotasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPContactosNotasLayout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addGroup(JPContactosNotasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane8, 0, 0, Short.MAX_VALUE)
                    .addGroup(JPContactosNotasLayout.createSequentialGroup()
                        .addComponent(JTFBuscarContac, javax.swing.GroupLayout.PREFERRED_SIZE, 176, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(JBEnviar, javax.swing.GroupLayout.DEFAULT_SIZE, 63, Short.MAX_VALUE)))
                .addContainerGap())
        );
        JPContactosNotasLayout.setVerticalGroup(
            JPContactosNotasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPContactosNotasLayout.createSequentialGroup()
                .addGap(1, 1, 1)
                .addGroup(JPContactosNotasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(JTFBuscarContac, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(JBEnviar))
                .addGap(8, 8, 8)
                .addComponent(jScrollPane8, javax.swing.GroupLayout.PREFERRED_SIZE, 307, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(11, 11, 11))
        );

        JBEnviar.getAccessibleContext().setAccessibleName("JBEnviar");
        JTFBuscarContac.getAccessibleContext().setAccessibleName("JTFBuscarContac");

        javax.swing.GroupLayout JPRedactarLayout = new javax.swing.GroupLayout(JPRedactar);
        JPRedactar.setLayout(JPRedactarLayout);
        JPRedactarLayout.setHorizontalGroup(
            JPRedactarLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPRedactarLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPRedactarLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(JPRedactarLayout.createSequentialGroup()
                        .addComponent(JLAsunto)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(JTFAsunto))
                    .addComponent(jScrollPane7, javax.swing.GroupLayout.PREFERRED_SIZE, 621, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addComponent(JPContactosNotas, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        JPRedactarLayout.setVerticalGroup(
            JPRedactarLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, JPRedactarLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPRedactarLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(JPContactosNotas, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(JPRedactarLayout.createSequentialGroup()
                        .addGroup(JPRedactarLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(JTFAsunto, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(JLAsunto))
                        .addGap(8, 8, 8)
                        .addComponent(jScrollPane7, javax.swing.GroupLayout.DEFAULT_SIZE, 351, Short.MAX_VALUE)))
                .addContainerGap())
        );

        JLAsunto.getAccessibleContext().setAccessibleName("JLAsunto");
        JTFAsunto.getAccessibleContext().setAccessibleName("JTFAsunto");
        JPContactosNotas.getAccessibleContext().setAccessibleName("JPContactosNotas");

        JTPNotas.addTab("                                        Redactar                                        ", JPRedactar);
        JPRedactar.getAccessibleContext().setAccessibleName("JPRedactar");

        JPBandejas.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JPBandejasMouseClicked(evt);
            }
        });

        JPContenidoNota.setBackground(new java.awt.Color(255, 255, 255));
        JPContenidoNota.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        JTFAsuntoNota.setEditable(false);
        JTFAsuntoNota.setFont(new java.awt.Font("Tahoma", 1, 13));
        JTFAsuntoNota.setBorder(null);

        JTFRedactorNota.setEditable(false);
        JTFRedactorNota.setFont(new java.awt.Font("Tahoma", 0, 12));
        JTFRedactorNota.setBorder(null);

        JTFEnviado.setEditable(false);
        JTFEnviado.setFont(new java.awt.Font("Tahoma", 0, 10));
        JTFEnviado.setBorder(null);

        JLEnviado.setText("Enviado:");

        JLPara.setText("Para:");

        JTATextoNota.setColumns(20);
        JTATextoNota.setEditable(false);
        JTATextoNota.setRows(5);
        JTATextoNota.setBorder(null);
        jScrollPane10.setViewportView(JTATextoNota);
        JTATextoNota.getAccessibleContext().setAccessibleName("JTATextoNota");

        JTAPara.setColumns(20);
        JTAPara.setEditable(false);
        JTAPara.setFont(new java.awt.Font("Tahoma", 0, 10));
        JTAPara.setRows(2);
        JTAPara.setBorder(null);
        jScrollPane11.setViewportView(JTAPara);
        JTAPara.getAccessibleContext().setAccessibleName("JTAPara");

        javax.swing.GroupLayout JPContenidoNotaLayout = new javax.swing.GroupLayout(JPContenidoNota);
        JPContenidoNota.setLayout(JPContenidoNotaLayout);
        JPContenidoNotaLayout.setHorizontalGroup(
            JPContenidoNotaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPContenidoNotaLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPContenidoNotaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JTFRedactorNota, javax.swing.GroupLayout.DEFAULT_SIZE, 502, Short.MAX_VALUE)
                    .addGroup(JPContenidoNotaLayout.createSequentialGroup()
                        .addComponent(JLEnviado)
                        .addGap(10, 10, 10)
                        .addComponent(JTFEnviado, javax.swing.GroupLayout.DEFAULT_SIZE, 450, Short.MAX_VALUE))
                    .addComponent(JTFAsuntoNota, javax.swing.GroupLayout.DEFAULT_SIZE, 502, Short.MAX_VALUE)
                    .addGroup(JPContenidoNotaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jSeparator1, javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, JPContenidoNotaLayout.createSequentialGroup()
                            .addComponent(JLPara)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jScrollPane11, javax.swing.GroupLayout.PREFERRED_SIZE, 443, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addComponent(jScrollPane10, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 494, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        JPContenidoNotaLayout.setVerticalGroup(
            JPContenidoNotaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPContenidoNotaLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTFAsuntoNota, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(JTFRedactorNota, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPContenidoNotaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLEnviado)
                    .addGroup(JPContenidoNotaLayout.createSequentialGroup()
                        .addGap(1, 1, 1)
                        .addComponent(JTFEnviado, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(JPContenidoNotaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JLPara)
                    .addComponent(jScrollPane11, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(11, 11, 11)
                .addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, 9, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane10, javax.swing.GroupLayout.PREFERRED_SIZE, 226, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        JTFAsuntoNota.getAccessibleContext().setAccessibleName("JTFAsuntoNota");
        JTFRedactorNota.getAccessibleContext().setAccessibleName("JTFRedactorNota");
        JTFEnviado.getAccessibleContext().setAccessibleName("JTFEnviado");
        JLEnviado.getAccessibleContext().setAccessibleName("JLEnviado");
        JLPara.getAccessibleContext().setAccessibleName("JLPara");

        JTEntrada.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "De", "Asunto", "Fecha"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTEntrada.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTEntradaMouseClicked(evt);
            }
        });
        jScrollPane9.setViewportView(JTEntrada);
        JTEntrada.getAccessibleContext().setAccessibleName("JTEntrada");

        javax.swing.GroupLayout JPEntradaLayout = new javax.swing.GroupLayout(JPEntrada);
        JPEntrada.setLayout(JPEntradaLayout);
        JPEntradaLayout.setHorizontalGroup(
            JPEntradaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEntradaLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane9, javax.swing.GroupLayout.DEFAULT_SIZE, 337, Short.MAX_VALUE)
                .addContainerGap())
        );
        JPEntradaLayout.setVerticalGroup(
            JPEntradaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPEntradaLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane9, javax.swing.GroupLayout.DEFAULT_SIZE, 291, Short.MAX_VALUE)
                .addContainerGap())
        );

        JTPES.addTab("                    Entrada                   ", JPEntrada);
        JPEntrada.getAccessibleContext().setAccessibleName("JPEntrada");

        JTSalida.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Asunto", "Fecha"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        JTSalida.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JTSalidaMouseClicked(evt);
            }
        });
        jScrollPane12.setViewportView(JTSalida);
        JTSalida.getAccessibleContext().setAccessibleName("JTSalida");

        javax.swing.GroupLayout JPSalidaLayout = new javax.swing.GroupLayout(JPSalida);
        JPSalida.setLayout(JPSalidaLayout);
        JPSalidaLayout.setHorizontalGroup(
            JPSalidaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPSalidaLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane12, javax.swing.GroupLayout.DEFAULT_SIZE, 337, Short.MAX_VALUE)
                .addContainerGap())
        );
        JPSalidaLayout.setVerticalGroup(
            JPSalidaLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPSalidaLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane12, javax.swing.GroupLayout.DEFAULT_SIZE, 291, Short.MAX_VALUE)
                .addContainerGap())
        );

        JTPES.addTab("                    Salida                   ", JPSalida);
        JPSalida.getAccessibleContext().setAccessibleName("JPSalida");

        JBDescartar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/note_delete.png"))); // NOI18N
        JBDescartar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JBDescartarActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout JPBandejasLayout = new javax.swing.GroupLayout(JPBandejas);
        JPBandejas.setLayout(JPBandejasLayout);
        JPBandejasLayout.setHorizontalGroup(
            JPBandejasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPBandejasLayout.createSequentialGroup()
                .addGroup(JPBandejasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(JPBandejasLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(JTPES, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(JPBandejasLayout.createSequentialGroup()
                        .addGap(159, 159, 159)
                        .addComponent(JBDescartar)))
                .addGap(18, 18, 18)
                .addComponent(JPContenidoNota, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(103, 103, 103))
        );
        JPBandejasLayout.setVerticalGroup(
            JPBandejasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPBandejasLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(JPBandejasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(JPContenidoNota, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(JPBandejasLayout.createSequentialGroup()
                        .addComponent(JTPES, javax.swing.GroupLayout.PREFERRED_SIZE, 341, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(JBDescartar)))
                .addContainerGap())
        );

        JPContenidoNota.getAccessibleContext().setAccessibleName("JPContenidoNota");
        JTPES.getAccessibleContext().setAccessibleName("JTPES");
        JBDescartar.getAccessibleContext().setAccessibleName("JBDescartar");

        JTPNotas.addTab("                                        Bandejas                                        ", JPBandejas);
        JPBandejas.getAccessibleContext().setAccessibleName("JPBandejas");

        javax.swing.GroupLayout JPNotasLayout = new javax.swing.GroupLayout(JPNotas);
        JPNotas.setLayout(JPNotasLayout);
        JPNotasLayout.setHorizontalGroup(
            JPNotasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNotasLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTPNotas, javax.swing.GroupLayout.PREFERRED_SIZE, 945, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(143, Short.MAX_VALUE))
        );
        JPNotasLayout.setVerticalGroup(
            JPNotasLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(JPNotasLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTPNotas, javax.swing.GroupLayout.PREFERRED_SIZE, 424, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(195, Short.MAX_VALUE))
        );

        JTPNotas.getAccessibleContext().setAccessibleName("JTPNotas");

        JTPFuncionalidades.addTab("<html><br><br>Notas<br><br></html>", JPNotas);
        JPNotas.getAccessibleContext().setAccessibleName("JPNotas");

        JMAccessManager.setText("accessManager");

        JMIBD.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/database.png"))); // NOI18N
        JMIBD.setText("BD");

        JMIImportar.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_I, java.awt.event.InputEvent.CTRL_MASK));
        JMIImportar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/database_add.png"))); // NOI18N
        JMIImportar.setText("Importar");
        JMIImportar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JMIImportarActionPerformed(evt);
            }
        });
        JMIBD.add(JMIImportar);
        JMIImportar.getAccessibleContext().setAccessibleName("JMIImportar");

        JMIExportar.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK));
        JMIExportar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/database_save.png"))); // NOI18N
        JMIExportar.setText("Exportar");
        JMIExportar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JMIExportarActionPerformed(evt);
            }
        });
        JMIBD.add(JMIExportar);
        JMIExportar.getAccessibleContext().setAccessibleName("JMIExportar");

        JMAccessManager.add(JMIBD);
        JMIBD.getAccessibleContext().setAccessibleName("JMIBD");

        JMICambiarDatos.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.CTRL_MASK));
        JMICambiarDatos.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/page_user_dark.gif"))); // NOI18N
        JMICambiarDatos.setText("Datos cuenta");
        JMICambiarDatos.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JMICambiarDatosActionPerformed(evt);
            }
        });
        JMAccessManager.add(JMICambiarDatos);
        JMICambiarDatos.getAccessibleContext().setAccessibleName("JMICambiarDatos");

        JMISalir.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ESCAPE, 0));
        JMISalir.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/Icons/cross.png"))); // NOI18N
        JMISalir.setText("Salir");
        JMISalir.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                JMISalirMouseClicked(evt);
            }
        });
        JMISalir.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                JMISalirActionPerformed(evt);
            }
        });
        JMAccessManager.add(JMISalir);
        JMISalir.getAccessibleContext().setAccessibleName("JMISalir");

        jMenuBar1.add(JMAccessManager);
        JMAccessManager.getAccessibleContext().setAccessibleName("JMAccessManager");

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTPFuncionalidades, javax.swing.GroupLayout.PREFERRED_SIZE, 1218, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(18, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(JTPFuncionalidades, javax.swing.GroupLayout.PREFERRED_SIZE, 635, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(19, Short.MAX_VALUE))
        );

        JTPFuncionalidades.getAccessibleContext().setAccessibleName("JTPFuncionalidades");

        pack();
    }// </editor-fold>//GEN-END:initComponents


 	// INICIALIZACION
        /** iniciarDatos: Metodo que inicializa todos los datos de los componentes de la interfaz grafica
         *
         * @param   void
         * @return  void
         */
        private void iniciarDatos()
        {           
            // ACCESOS
            // Iniciamos todo el contenido del tab 'Accesos'
            iniciarAccesos();

            // EMPRESAS Y PERSONAL
            // Rellenamos todas las tablas de empresas y personal
            iniciarEmpresasPersonal();

            // TARJETAS, COLORES, NIVELES Y T_ACCESOS
            // Rellenamos las tablas de tarjetas, colores, niveles, tipos de accesos y cargos
            iniciarTACs();

            // ZONAS, LLAVES, LLAVEROS Y ALMACENES
            // Rellenamos las tablas de zonas, llaves y llaveros
            iniciarZonas();

            // PRESTAMOS
            // Iniciamos todo el contenido del tab 'Prestamos'
            iniciarPrestamos();

            // PARTES DE SEGURIDAD
            // Iniciamos todo el contenido del tab 'Partes de seguridad'
            iniciarPartes();

            // CUENTAS
            // Iniciamos todo el contenido del tab 'Cuentas'
            iniciarCuentas();

            // NOTAS
            // Iniciamos todo el contenido del tab 'Notas'
            iniciarNotas();
        }

        /** pintar: Metodo que pinta los componentes graficos necesarios
         *
         * @param   void
         * @return  void
         */
        private void pintar()
        {
            // 'Prestamos'
            JTFNombrePrest.setBackground(Color.WHITE);
            JTFTipoPrest.setBackground(Color.WHITE);
            JTFEmpresaPrest.setBackground(Color.WHITE);

            // 'Notas'
            JTFAsuntoNota.setBackground(Color.WHITE);
            JTFRedactorNota.setBackground(Color.WHITE);
            JTFEnviado.setBackground(Color.WHITE);
        }

        /** visualizacion: Metodo que borra todos los componentes graficos innecesarios segun el tipo de usuario
         *
         * @param   void
         * @return  void
         */
        private void visualizacion()
        {
            if (Tipo_Usu.equals("ADMINISTRADOR"))
            {
                // Ocultamos las botoneras de 'Accesos'
                JPAccesosB1.setVisible(false);
                JPAccesosB2.setVisible(false);

                // Ocultamos el boton de 'Añadir parte' y 'Modificar/Grabar parte'
                JBAnadirParte.setVisible(false);
                JBModificarParte.setVisible(false);
            }
            else if (Tipo_Usu.equals("VIGILANTE"))
            {
                // Borramos los paneles y menus
                JTPFuncionalidades.remove(5);
                JTPFuncionalidades.remove(4);
                JTPFuncionalidades.remove(3);
                JTPFuncionalidades.remove(2);
                JTPFuncionalidades.remove(1);
                JMIBD.setVisible(false);
            }
            else
            {
                // Borramos los paneles y menus
                if (Tipo_Usu.equals("RP"))
                {
                    JPEmpresasB.setVisible(false);
                    JPDepartamentosB.setVisible(false);
                    JTPFuncionalidades.remove(7);
                }
                JTPFuncionalidades.remove(6);
                JTPFuncionalidades.remove(5);
                JTPFuncionalidades.remove(4);
                JTPFuncionalidades.remove(3);
                JTPFuncionalidades.remove(2);
                JTPFuncionalidades.remove(0);
                JMIBD.setVisible(false);
            }
        }



   	// EVENTOS
    private void JCCFechaInicDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFechaInicDateChanged
        // Refrescamos la JTable de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JCCFechaInicDateChanged

    private void JTAccesosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTAccesosMouseClicked
        // Mostramos la informacion del acceso
        mostrarAcceso();
    }//GEN-LAST:event_JTAccesosMouseClicked

    private void JTEmpresasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTEmpresasMouseClicked
        try
        {
            // Escondemos los paneles de departamentos y personal
            JPDepartamentosD.setVisible(false);
            mostrarJPanelsPersonal("");

            // Borramos la tabla de llaves
            String[] Etiquetas = {"Código","Zona","Descripción"};
            JTLsLlsPers.setModel(new DefaultTableModel(Etiquetas,0));

            // Siempre que 'JTBAnadirEmpresa' no este seleccionado -> Mostramos los datos de la empresa
            if (!JTBAnadirEmpresa.isSelected())
            {
                // En un principio ocultamos todos los paneles
                mostrarJPanelsPersonal("");

                // Si 'JTBModificarEmpresa' no esta seleccionado
                if (!JTBModificarEmpresa.isSelected())
                {
                    // Desactivamos el boton de modificar empresa (lo activaremos en caso que sea posible la modificacion)
                    JTBModificarEmpresa.setSelected(false);
                    JTBModificarEmpresa.setEnabled(false);
                }

                // Desactivamos los botones de eliminar y modificar persona
                JBEliminarPerContac.setEnabled(false);
                JTBModificarPerContac.setSelected(false);
                JTBModificarPerContac.setEnabled(false);

                // Obtenemos el nombre de la empresa
                String Nombre_Empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);

                // Mostramos los datos de la empresa
                if (Nombre_Empresa.equals("Ocasional"))
                {
                    // Borramos los datos del formulario
                    borrarDatosEmp();

                    // Mostramos el nombre
                    JTFNomEmp.setText("Ocasional");

                    // Activamos/Desactivamos botones
                    JTBAnadirDepartamento.setSelected(false);
                    JTBAnadirDepartamento.setEnabled(false);
                    JTBAnadirPerContac.setSelected(false);
                    JTBAnadirPerContac.setEnabled(true);
                }
                else if (Nombre_Empresa.equals("Informática Navarra S.A."))
                {
                    // Borramos los datos del formulario
                    borrarDatosEmp();

                    // Mostramos el nombre
                    JTFNomEmp.setText("Informática Navarra S.A.");

                    // Activamos/Desactivamos botones
                    JTBAnadirDepartamento.setSelected(false);
                    JTBAnadirDepartamento.setEnabled(false);
                    JTBAnadirPerContac.setSelected(false);
                    JTBAnadirPerContac.setEnabled(true);
                }
                else
                {
                    // Rellenamos los datos de la empresa
                    Empresa emp = new Empresa();
                    emp.restablecerDatos(Nombre_Empresa);
                    JTFCodEmp.setText(emp.getCodEmp());
                    JTFCIFEmp.setText(emp.getCIF());
                    JTFNomEmp.setText(emp.getNombre());
                    JTADirEmp.setText(emp.getDireccion());

                    // Activamos los botones de 'Eliminar empresa', 'Modificar empresa' y 'Anadir departamento'
                    JBEliminarEmpresa.setEnabled(true);
                    
                    if (!JTBModificarEmpresa.isSelected())
                    {
                        JTBModificarEmpresa.setSelected(false);
                        JTBModificarEmpresa.setEnabled(true);
                    }

                    JTBAnadirDepartamento.setSelected(false);
                    JTBAnadirDepartamento.setEnabled(true);

                    // Desactivamos los botones para eliminar o modificar departamentos y añadir persona/contacto
                    JBEliminarDepartamento.setEnabled(false);
                    JTBModificarDepartamento.setSelected(false);
                    JTBModificarDepartamento.setEnabled(false);
                    JTBAnadirPerContac.setSelected(false);
                    JTBAnadirPerContac.setEnabled(false);
                }

                // Desactivamos los botones de añadir y borrar llaves
                JBAnadirPrestDep.setEnabled(false);
                JBEliminarPrestDep.setEnabled(false);

                // Refrescamos todas las tablas salvo la de empresas
                refrescarTablasEmpresasPersonal("DGC");
                String [] Etiquetas1 = {"Teléfonos"};
                JTTfns.setModel(new DefaultTableModel(Etiquetas1,0));
                Etiquetas1[0] = "E-Mails";
                JTEmails.setModel(new DefaultTableModel(Etiquetas1,0));
                Etiquetas1[0] = "Cargos";
                JTCargos.setModel(new DefaultTableModel(Etiquetas1,0));

                // Escondemos el panel
                JPElimC.setVisible(false);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la empresa.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTEmpresasMouseClicked

    private void JTDepartamentosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTDepartamentosMouseClicked
        try
        {
            // Escondemos los paneles de personal
            mostrarJPanelsPersonal("");

            // Siempre que 'JTBAnadirDepartamento' no este seleccionado y que exista una empresa seleccionada -> Mostramos los datos del departamento
            if ((!JTBAnadirDepartamento.isSelected()) && (JTEmpresas.getSelectedRow() != -1))
            {
                // Obtenemos el nombre de la empresa
                String Nombre_Empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);
                // Obtenemos el nombre del departamento
                String Nombre_Departamento = (String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0);

                // Mostramos los datos del departamento
                Empresa emp = new Empresa();
                emp.setNombre((String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0));
                emp.setCodEmp(emp.obtenerCodEmp());
                Departamento dep = emp.getDepartamentoBD(emp.obtenerCodDep(Nombre_Departamento));
                JTFCodDep.setText(dep.getCodDep());
                JTFNomDep.setText(dep.getNombre());

                // Buscamos el indice del elemento a seleccionar
                int ind = 0;
                boolean encontrado = false;
                while ((ind<JCBActividad.getModel().getSize()) && (!encontrado))
                {
                    String elemento = (String) JCBActividad.getModel().getElementAt(ind);
                    if (elemento.startsWith(dep.getAcceso().getCodAcc()))   encontrado = true;
                    else                                                    ind++;
                }
                // Seleccionamos el tipo de acceso
                JCBActividad.setSelectedIndex(ind);
                // Establecemos la periodicidad y autorizador
                JTFPeriodicidad.setText(new Integer(dep.getPeriodicidad()).toString());
                JCCFUltActu.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(dep.getFechaI().getTime()));
                if (dep.getAutorizador() != null)       JCBDNIDep.getModel().setSelectedItem((String) dep.getAutorizador().getDNI());
                else                                    JCBDNIDep.getModel().setSelectedItem("Ninguno");

                // Refrescamos las tablas de empleados, contactos, llaves, telefonos, E-Mails y cargos
                refrescarTablasEmpresasPersonal("GCL");
                String[] Etiquetas = {"Teléfonos"};
                JTTfns.setModel(new DefaultTableModel(Etiquetas,0));
                Etiquetas[0] = "E-Mails";
                JTEmails.setModel(new DefaultTableModel(Etiquetas,0));
                Etiquetas[0] = "Cargos";
                JTCargos.setModel(new DefaultTableModel(Etiquetas,0));
                JPElimC.setVisible(false);

                // Activamos los botones de 'Modificar empresa', 'Añadir llaves', 'Eliminar departamento', 'Modificar departamento' y 'Anadir persona/contacto'
                JTBModificarEmpresa.setEnabled(true);
                JBAnadirPrestDep.setEnabled(true);
                JBEliminarDepartamento.setEnabled(true);

                // Si 'JTBModificarEmpresa' no esta seleccionado
                if (!JTBModificarDepartamento.isSelected())
                {
                    JTBModificarDepartamento.setSelected(false);
                    JTBModificarDepartamento.setEnabled(true);
                }

                JTBAnadirPerContac.setSelected(false);
                JTBAnadirPerContac.setEnabled(true);
                JBEliminarPerContac.setEnabled(false);
                JTBModificarPerContac.setSelected(false);
                JTBModificarPerContac.setEnabled(false);

                // Desactivamos el boton de borrado de llaves
                JBEliminarPrestDep.setEnabled(false);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos del departamento.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTDepartamentosMouseClicked

    private void JTEmpleadosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTEmpleadosMouseClicked
        try
        {
            // Registramos el DNI seleccionado
            DNI_Personal = (String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0);

            // Siempre que 'JTBAnadirPerContac' no este seleccionado -> Mostramos los datos de la persona
            if (!JTBAnadirPerContac.isSelected())
            {
                // Activamos el boton de eliminar y modificar persona
                if ((!JTBModificarPerContac.isSelected()) && ((JTDepartamentos.getModel().getRowCount() == 0) || (JTDepartamentos.getSelectedRow() != -1)))
                {
                    JBEliminarPerContac.setEnabled(true);
                    JTBModificarPerContac.setSelected(false);
                    JTBModificarPerContac.setEnabled(true);
                }
                
                // Mostramos los datos generales
                mostrarDP(DNI_Personal);

                // Rellenamos los datos
                Personal pers = new Personal();
                pers.setDNI(DNI_Personal);
                String tipo = pers.obtenerTipo();

                if (tipo.equals("Ocasional"))
                {
                    // Rellenamos el JComboBox de empresas
                    JCBEmpresas.setEditable(false);
                    refrescarCBEmpresas();

                    // Rellenamos los datos especificos con la primera empresa de la lista
                    Ocasional oca = new Ocasional(DNI_Personal,(String) JCBEmpresas.getModel().getSelectedItem());
                    JCBEmpresas.setSelectedIndex(0);
                    JCBActividad1.getModel().setSelectedItem(oca.getAcceso().getCodAcc() + " - " + oca.getAcceso().getActividad());
                    JCCFCaducidad.setDate(oca.getFCaducidad().getTime());
                    JCCFCaducidad.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(oca.getFCaducidad().getTime()));
                    JTANotas.setText(oca.getNotas());

                    // Rellenamos la tabla de llaves
                    UGUI.rellenarTabla(JTLsLlsPers1, oca.obtenerLlaves());
                    oca.UBD.cerrarConexion();
                }
                else if (tipo.equals("Residente"))
                {
                    // Rellenamos los datos especificos
                    Residente res = new Residente(DNI_Personal);
                    if (res.getNivel() == null)     JCBNivel.setSelectedIndex(0);
                    else                            JCBNivel.getModel().setSelectedItem(res.getNivel().getCodNiv());
                }
                else if (tipo.equals("Vigilante"))
                {
                    // Rellenamos los datos especificos
                    Vigilante vig = new Vigilante(DNI_Personal);
                    JTFCodV.setText(new Integer(vig.getCodVig()).toString());
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la persona.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTEmpleadosMouseClicked

    private void JTContactosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTContactosMouseClicked
            // Siempre que 'JTBAnadirPerContac' no este seleccionado -> Mostramos los datos de la empresa
            if (!JTBAnadirPerContac.isSelected())
            {
                // Registramos el DNI seleccionado
                DNI_Personal = (String) JTContactos.getModel().getValueAt(JTContactos.getSelectedRow(),0);

                // Activamos el boton de eliminar y modificar persona
                if ((!JTBModificarPerContac.isSelected()) && ((JTDepartamentos.getModel().getRowCount() == 0) || (JTDepartamentos.getSelectedRow() != -1)))
                {
                    JBEliminarPerContac.setEnabled(true);
                    JTBModificarPerContac.setSelected(false);
                    JTBModificarPerContac.setEnabled(true);
                }

                // Mostramos los datos generales
                mostrarDP(DNI_Personal);

                // Refrescanos las tablas de telefonos, E-Mails y cargos
                refrescarTablasEmpresasPersonal("TMX");
            }
    }//GEN-LAST:event_JTContactosMouseClicked

    private void JCCFechaFinDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFechaFinDateChanged
        // Refrescamos la JTable de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JCCFechaFinDateChanged

    private void JCCFechaInicActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCCFechaInicActionPerformed
    }//GEN-LAST:event_JCCFechaInicActionPerformed

    private void JCBTodoDiaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBTodoDiaActionPerformed
        // Activamos/descativamos los JComboBox que delimitan las horas
        if (JCBHoraFin.isEnabled())
        {
            JCBHoraInic.setEnabled(false);
            JCBHoraFin.setEnabled(false);
        }
        else
        {
            JCBHoraInic.setEnabled(true);
            JCBHoraFin.setEnabled(true);
        }
        // Refrescamos la JTable de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JCBTodoDiaActionPerformed

    private void JCBTodoDiaStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_JCBTodoDiaStateChanged
    }//GEN-LAST:event_JCBTodoDiaStateChanged

    private void JRBEntradaStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_JRBEntradaStateChanged
    }//GEN-LAST:event_JRBEntradaStateChanged

    private void JCBHoraInicItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHoraInicItemStateChanged
    }//GEN-LAST:event_JCBHoraInicItemStateChanged

    private void JCBHoraFinItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHoraFinItemStateChanged
    }//GEN-LAST:event_JCBHoraFinItemStateChanged

    private void JBSalidaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JBSalidaMouseClicked
    }//GEN-LAST:event_JBSalidaMouseClicked

    private void JTFBuscarKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarKeyReleased
        // Refrescamos la tabla de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JTFBuscarKeyReleased

    private void JRBEntradaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JRBEntradaMouseClicked
        // Refrescamos la JTable de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JRBEntradaMouseClicked

    private void JRSalidaStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_JRSalidaStateChanged
    }//GEN-LAST:event_JRSalidaStateChanged

    private void JRSalidaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JRSalidaMouseClicked
        // Refrescamos la JTable de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JRSalidaMouseClicked

    private void JBSalidaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBSalidaActionPerformed
        // Obtenemos la fila seleccionada
        int filSelec = JTAccesos.getSelectedRow();

        // Damos salida al acceso
        try
        {
            // Avisamos de que vamos a dar salida al acceso
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar salida al acceso seleccionado.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Damos salida
            if (seleccion == 0)
            {
                // Creamos el acceso
                Acceso acc = new Acceso(JTAccesos.getModel().getValueAt(filSelec,1).toString().substring(0,19));

                // Devolvemos las llaves y llaveros
                int[] filasSelecs = JTLsLls.getSelectedRows();
                for (int i=0; i<filasSelecs.length; i++)
                {
                    // Obtenemos el tipo de prestamo (Llave o llavero) y el codigo de llave o llavero
                    String tipo = (String) JTLsLls.getModel().getValueAt(filasSelecs[i],0);
                    int codigo = (Integer) JTLsLls.getModel().getValueAt(filasSelecs[i],1);

                    // Devolvemos el prestamo
                    acc.devGLLDBD(tipo,codigo);

                    // Cerramos la conexion
                    acc.UBD.cerrarConexion();
                }

                // Refrescamos JTLsLls (tabla de prestamos de llaves y llaveros)
                UGUI.rellenarTabla(JTLsLls,acc.obtenerLsLlsAcceso());
                // Cerramos la conexion
                acc.UBD.cerrarConexion();

                // Devolvemos la tarjeta en caso de estar JTBTarjeta presionado
                if (JCBTarjeta.isSelected())
                {
                    acc.getTarjeta().devTarjetaBD();
                    JCBTarjeta.setSelected(true);
                }

                // Damos salida al acceso
                acc.darSalida();

                // Refrescamos las tabs de accesos y prestamos
                iniciarAccesos();
                iniciarPrestamos();

                // Cerramos la conexion
                acc.UBD.cerrarConexion();

                // Advertimos de que se ha dado salida al acceso seleccionado
                JOptionPane.showMessageDialog(this,"Se ha proporcionado salida al acceso seleccionado.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible dar salida al acceso seleccionado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBSalidaActionPerformed

    private void JBEliminarPrestamoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JBEliminarPrestamoMouseClicked
    }//GEN-LAST:event_JBEliminarPrestamoMouseClicked

    private void JTLsLlsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsLlsMouseClicked
        // Obtenemos el numero de filas seleccionadas
        int numSelecs = JTLsLls.getSelectedRowCount();

        // Comprobamos si todas las filas seleccionadas corresponden a prestamos sin devolver
        boolean noDevueltas = true;
        int[] filasSelecs = JTLsLls.getSelectedRows();

        int i = 0;
        while ((i<filasSelecs.length) && (noDevueltas))
        {
            String devuelta = (String) JTLsLls.getModel().getValueAt(filasSelecs[i],2);
            if (devuelta.equals("Sí"))  noDevueltas = false;
            i++;
        }

        // Si existe algun acceso seleccionado
        if (JTAccesos.getSelectedRow() != -1)
        {
            // Si existe alguna fila seleccionada y todas ellas estan en no-devueltas -> Activamos los botones de 'Eliminar prestamos' y 'Devolver prestamos'
            if (noDevueltas) // ((numSelecs>0) && (noDevueltas))
            {
                JBEliminarPrestamo.setEnabled(true);
                JBDevolverPrestamo.setEnabled(true);
            }
            // Si existe alguna fila seleccionada pero existe alguna con prestamo devuelto -> Activamos solo el boton de 'Eliminar prestamos'
            else // if ((numSelecs>0) && (!noDevueltas))
            {
                JBEliminarPrestamo.setEnabled(true);
                JBDevolverPrestamo.setEnabled(false);
            }
        }
        // En caso contrario -> Activamos el boton de 'Eliminar prestamos'
        else    JBEliminarPrestamo.setEnabled(true);
    }//GEN-LAST:event_JTLsLlsMouseClicked

    private void JBDevolverPrestamoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JBDevolverPrestamoMouseClicked
    }//GEN-LAST:event_JBDevolverPrestamoMouseClicked

    private void JBAnadirPrestamoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JBAnadirPrestamoMouseClicked
    }//GEN-LAST:event_JBAnadirPrestamoMouseClicked

    private void JBDarSalidaConDevMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JBDarSalidaConDevMouseClicked
    }//GEN-LAST:event_JBDarSalidaConDevMouseClicked

    private void JBEliminarAccesoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JBEliminarAccesoMouseClicked
    }//GEN-LAST:event_JBEliminarAccesoMouseClicked

    private void JBAnadirPrestamoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirPrestamoActionPerformed
       try
       {
            // Fila seleccionada
            int filaSelec = JTAccesos.getSelectedRow();
            // Entrada
            String Entrada = ((filaSelec!=-1) ? JTAccesos.getModel().getValueAt(filaSelec,1).toString().substring(0,19) : "");
            
            // Creamos el acceso
            Acceso acc = null;

            // Creamos el acceso
            if (filaSelec != -1)        acc = new Acceso(Entrada);

            // Creamos el JDialog para añadir prestamos
            JDAccesoLsLls jdll = new JDAccesoLsLls(this,true,JTFDNI.getText(),Entrada);
            jdll.setVisible(true);

            // Obtenemos el HashMap de GLL_Devs compuesto por las llaves y llaveros seleccionados
            HM_GLLDs = jdll.getLsLls();
            boolean pulsarExit = jdll.getPulsarExit();

            // Cerramos el dialogo de llaves y llaveros
            jdll.dispose();

            // Si el prestamo se realiza a un acceso ya existente
            if ((filaSelec != -1) && (!HM_GLLDs.isEmpty()) && (!pulsarExit))
            {
                // Avisamos de que vamos a realizar el prestamo de las llaves y llaveros seleccionados
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a prestar las llaves y llaveros seleccionados.\n" +
                                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);
                // Si el usuario acepta realizar la operacion, realizamos el prestamo de llaves y llaveros
                if (seleccion == 0)
                {
                    // Creamos el acceso
                    acc = new Acceso(Entrada);

                    // Insertamos todos los prestamos existentes en el HashMap
                    acc.insertGLLDsBD(HM_GLLDs);

                    // Refrescamos la tabla de prestamos
                    mostrarPrestamos(acc);

                    // Advertimos de que los prestamos han sido insertador correctamente
                    JOptionPane.showMessageDialog(this,"Los prestamos seleccionados han sido insertados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // Si no se realiza a un acceso existente, las llaves y llaveros seleccionados permaneceran en la JTable.

            // Refrescamos la tabla de prestamos y el tab de prestamos pendientes
            mostrarPrestamos(acc);
            iniciarPrestamos();
       }
       catch (Exception e)
       {
            JOptionPane.showMessageDialog(this,"No ha sido posible prestar las llaves y/o llaveros seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
       }
    }//GEN-LAST:event_JBAnadirPrestamoActionPerformed

    private void JBEliminarPrestamoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarPrestamoActionPerformed
        // Avisamos de que vamos a eliminar los prestamos seleccionados
        int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los prestamos seleccionados.\n" +
                "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

        // Si el usuario acepta realizar la operacion -> Eliminamos los prestamos seleccionados
        if (seleccion == 0)
        {
            try
            {
                // Si se trata de un acceso existente
                if (JTAccesos.getSelectedRow() != -1)
                {
                    // Obtenemos el acceso
                    Acceso acc = new Acceso(JTAccesos.getModel().getValueAt(JTAccesos.getSelectedRow(),1).toString().substring(0,19));

                    // Para cada fila seleccionada en JTLsLls -> Eliminar el prestamo de la BD
                    int[] filasSelecs = JTLsLls.getSelectedRows();
                    for (int i=0; i<filasSelecs.length; i++)
                    {
                        // Obtenemos el tipo de prestamo (Llave o llavero) y el codigo de llave o llavero
                        String tipo = (String) JTLsLls.getModel().getValueAt(filasSelecs[i],0);
                        int codigo = (Integer) JTLsLls.getModel().getValueAt(filasSelecs[i],1);

                        // Borramos el prestamo de la BD
                        GLL_Dev glld = acc.getGLLDBD(tipo, codigo);
                        acc.UBD.cerrarConexion();
                        acc.removeGLLDBD(glld);
                        acc.UBD.cerrarConexion();
                    }

                    // Refrescamos JTLsLls (tabla de llaves y llaveros prestados)
                    UGUI.rellenarTabla(JTLsLls,acc.obtenerLsLlsAcceso());

                    // Refrescamos el tab de prestamos
                    iniciarPrestamos();

                    // Cerramos la conexion
                    acc.UBD.cerrarConexion();
                }

                // Si se trata de un acceso por realizar
                else
                {
                    // Obtenemos las filas seleccionadas
                    int[] filasSelecs = JTLsLls.getSelectedRows();
                    // Eliminamos cada prestamo del modelo, creando uno nuevo y asignandoselo a la tabla
                    DefaultTableModel modelo = (DefaultTableModel) JTLsLls.getModel();
                    for (int i=filasSelecs.length; i>0; i--)    modelo.removeRow(filasSelecs[i-1]);
                    JTLsLls.setModel(modelo);
                }

                // Advertimos de que los prestamos seleccionados han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los préstamos seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);

                // Desactivamos los botones de 'Eliminar prestamo' y 'Devolver prestamo'
                JBEliminarPrestamo.setEnabled(false);
                JBDevolverPrestamo.setEnabled(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los prestamos seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_JBEliminarPrestamoActionPerformed

    private void JBDevolverPrestamoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBDevolverPrestamoActionPerformed
        // Avisamos de que vamos a devolver las llaves y llaveros seleccionados
        int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a devolver los prestamos seleccionados.\n" +
                "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

        // Si el usuario acepta realizar la operacion -> Devolvemos los prestamos seleccionados
        if (seleccion == 0)
        {
            try
            {
                // Para cada fila seleccionada en JTLsLls -> Devolver el prestamo de la BD
                Acceso acc = new Acceso(JTAccesos.getModel().getValueAt(JTAccesos.getSelectedRow(),1).toString().substring(0,19));

                int[] filasSelecs = JTLsLls.getSelectedRows();
                for (int i=0; i<filasSelecs.length; i++)
                {
                    // Obtenemos el tipo de prestamo (Llave o llavero) y el codigo de llave o llavero
                    String tipo = (String) JTLsLls.getModel().getValueAt(filasSelecs[i],0);
                    int codigo = (Integer) JTLsLls.getModel().getValueAt(filasSelecs[i],1);

                    // Devolvemos el prestamo
                    acc.devGLLDBD(tipo,codigo);

                    // Cerramos la conexion
                    acc.UBD.cerrarConexion();
                }

                // Refrescar JTLsLls
                UGUI.rellenarTabla(JTLsLls,acc.obtenerLsLlsAcceso());

                // Refrescar el tab de prestamos
                iniciarPrestamos();

                // Cerramos la conexion
                acc.UBD.cerrarConexion();

                // Advertimos de que los prestamos seleccionados han sido devueltas correctamente
                JOptionPane.showMessageDialog(this,"Los prestamos seleccionados han sido devueltos correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible devolver los prestamos seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_JBDevolverPrestamoActionPerformed

    private void JBEliminarAccesoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarAccesoActionPerformed
        // Avisamos de que vamos a eliminar los accesos seleccionados
        int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los accesos seleccionados.\n" +
                "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

        // Si el usuario acepta realizar la operacion -> Eliminamos los accesos seleccionados
        if (seleccion == 0)
        {
            try
            {
                // Obtenemos los accesos seleccionados
                int[] filasSelecs = JTAccesos.getSelectedRows();

                // Creamos un objeto de tipo Acceso
                Acceso acc = new Acceso();

                // Eliminamos cada acceso
                for (int i=0; i<filasSelecs.length; i++)
                {
                    // Establecemos la PK del Acceso
                    acc.setFechaI(UCAL.stringToCalendar(JTAccesos.getModel().getValueAt(filasSelecs[i],1).toString().substring(0,19)));

                    // Eliminamos el acceso
                    acc.removeAccesoBD();
                }

                // Refrescamos las tabs de accesos y prestamos
                iniciarAccesos();
                iniciarPrestamos();

                // Borramos el formulario de acceso
                borrarFormAcceso();

                // Advertimos de que los accesos seleccionados han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los accesos seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los accesos seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_JBEliminarAccesoActionPerformed

    private void JBDarSalidaConDevActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBDarSalidaConDevActionPerformed
        // Avisamos de que vamos a dar salida al acceso, devolviendo las llaves, llaveros y tarjeta prestada
        int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar salida a la persona, devolviendo todos los préstamos del acceso pendientes.\n" +
                "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

        // Si el usuario acepta realizar la operacion -> Se devuelven todos los prestamos y se da salida
        if (seleccion == 0)
        {
            try
            {
                // Creamos el acceso
                Acceso acc = new Acceso(JTAccesos.getModel().getValueAt(JTAccesos.getSelectedRow(),1).toString().substring(0,19));

                // Devolvemos las llaves y llaveros
                acc.devolverLlavesLlaveros();

                // Refrescamos JTLsLls (tabla de prestamos de llaves y llaveros)
                UGUI.rellenarTabla(JTLsLls,acc.obtenerLsLlsAcceso());
                // Cerramos la conexion
                acc.UBD.cerrarConexion();

                // Devolvemos la tarjeta
                acc.getTarjeta().devTarjetaBD();
                JCBTarjeta.setSelected(true);

                // Damos salida a la persona
                acc.darSalida();

                // Refrescamos la JTable de accesos
                iniciarAccesos();

                // Advertimos de que se ha realizado la devolucion de todos los prestamos y se ha dado salida al acceso seleccionado
                JOptionPane.showMessageDialog(this,"Se ha realizado la devolución de préstamos correctamente, así como la proporción de salida al acceso seleccionado.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible dar salida al acceso seleccionado.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_JBDarSalidaConDevActionPerformed

    private void JCBTarjetaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBTarjetaActionPerformed
    }//GEN-LAST:event_JCBTarjetaActionPerformed

    private void JBAnadirAccesoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirAccesoActionPerformed
        try
        {
            // Creamos el Acceso
            Personal Persona = new Personal(JTFDNI.getText());
            Calendar Entrada = Calendar.getInstance();
            Calendar Salida = null;
            String Motivo = JTAMotivo.getText();
            Tarjeta Tarjeta = new Tarjeta(Integer.parseInt(JTFCodTarj.getText()));
            String OtrosDatos = JTAOtrosDatos.getText();
            Personal Insertador = new Personal(DNI_Usu);
            Acceso acc_new = new Acceso(Persona,Entrada,Salida,Motivo,Tarjeta,HM_GLLDs,OtrosDatos,Insertador);

            // Comprobamos si se trata de una insercion o de una modificacion
            // -> Si se trata de una insercion
            if (JTAccesos.getSelectedRow() == -1)
            {
                // Avisamos de que vamos a añadir el acceso
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a insertar el acceso confeccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si el usuario acepta realizar la operacion -> Se inserta el acceso
                if (seleccion == 0)
                {
                    // Insertamos el acceso en la BD
                    acc_new.insertAccesoBD();

                    // Borramos los datos del acceso
                    borrarFormAcceso();

                    // Refrescamos la tabla de Accesos
                    iniciarAccesos();

                    // Advertimos de que el acceso ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El acceso ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Si se trata de una modificacion
            else
            {
                // Avisamos de que vamos a modificar el acceso
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el acceso seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si el usuario acepta realizar la operacion -> Se modifica el acceso
                if (seleccion == 0)
                {
                    // Obtenemos el acceso viejo
                    String Entr = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTAccesos.getModel().getValueAt(JTAccesos.getSelectedRow(),1));
                    Acceso acc_old = new Acceso(Entr);

                    // Modificamos el acceso viejo con el nuevo
                    acc_old.updateDatosAccesoBD(acc_new);

                    // Refrescamos la tabla de Accesos
                    iniciarAccesos();

                    // Habilitamos el JCB de la tarjeta
                    JCBTarjeta.setEnabled(true);

                    // Advertimos de que el acceso ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El acceso ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }

                // Desactivamos los prestamos
                JBAnadirPrestamo.setEnabled(false);
                JBEliminarPrestamo.setEnabled(false);
                JBDevolverPrestamo.setEnabled(false);
                // Desactivamos el poder dar salidas
                JBSalida.setEnabled(false);
                JBDarSalidaConDev.setEnabled(false);
                // No se pueden modificar JTAMotivo y JTAOtrosDatos
                JTAMotivo.setEditable(false);
                JTAOtrosDatos.setEditable(false);
                JBAnadirAcceso.setEnabled(false);
            }

            // Refrescamos todo el tab de prestamos
            iniciarPrestamos();
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el acceso.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBAnadirAccesoActionPerformed

    private void JCCFechaInicParteDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFechaInicParteDateChanged
        // Refrescamos la JTable de partes
        refrescarTPartes();
    }//GEN-LAST:event_JCCFechaInicParteDateChanged

    private void JCCFechaInicParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCCFechaInicParteActionPerformed
    }//GEN-LAST:event_JCCFechaInicParteActionPerformed

    private void JCCFechaFinParteDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFechaFinParteDateChanged
        // Refrescamos la JTable de partes
        refrescarTPartes();
    }//GEN-LAST:event_JCCFechaFinParteDateChanged

    private void JCBHoraFinParteItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHoraFinParteItemStateChanged
    }//GEN-LAST:event_JCBHoraFinParteItemStateChanged

    private void JCBHoraInicParteItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHoraInicParteItemStateChanged
    }//GEN-LAST:event_JCBHoraInicParteItemStateChanged

    private void JCBTodoDiaParteStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_JCBTodoDiaParteStateChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBTodoDiaParteStateChanged

    private void JCBTodoDiaParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBTodoDiaParteActionPerformed
        if (JCBFechasPartes.isSelected())
        {
            // Activamos/descativamos los JComboBox que delimitan las horas si el de fechas esta activado
            if (JCBHoraFinParte.isEnabled())
            {
                JCBHoraInicParte.setEnabled(false);
                JCBHoraFinParte.setEnabled(false);
            }
            else
            {
                JCBHoraInicParte.setEnabled(true);
                JCBHoraFinParte.setEnabled(true);
            }
            // Refrescamos la JTable de partes
            refrescarTPartes();
        }
    }//GEN-LAST:event_JCBTodoDiaParteActionPerformed

    private void JCBFechasPartesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBFechasPartesActionPerformed
        // Activamos/descativamos los JCalendar que delimitan las fechas y el JCheckBox que activa la busqued apor horas
        if (JCCFechaInicParte.isEnabled())
        {
            JCCFechaInicParte.setEnabled(false);
            JCCFechaFinParte.setEnabled(false);
            JCBTodoDiaParte.setSelected(true);
            JCBHoraInicParte.setEnabled(false);
            JCBHoraFinParte.setEnabled(false);
            JCBTodoDiaParte.setEnabled(false);
        }
        else
        {
            JCCFechaInicParte.setEnabled(true);
            JCCFechaFinParte.setEnabled(true);
            JCBTodoDiaParte.setEnabled(true);
        }
        
        // Refrescamos la JTable de partes
        refrescarTPartes();
    }//GEN-LAST:event_JCBFechasPartesActionPerformed

    private void JCBHoraInicParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHoraInicParteActionPerformed
        // Refrescamos la JTable de partes
        refrescarTPartes();
    }//GEN-LAST:event_JCBHoraInicParteActionPerformed

    private void JCCFechaFinParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCCFechaFinParteActionPerformed
    }//GEN-LAST:event_JCCFechaFinParteActionPerformed

    private void JCBHoraFinParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHoraFinParteActionPerformed
        // Refrescamos la JTable de partes
        refrescarTPartes();
    }//GEN-LAST:event_JCBHoraFinParteActionPerformed

    private void JCBCodVigActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBCodVigActionPerformed
        // Refrescamos la tabla de partes
        refrescarTPartes();
    }//GEN-LAST:event_JCBCodVigActionPerformed

    private void JCBHoraInicActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHoraInicActionPerformed
        // Refrescamos la JTable de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JCBHoraInicActionPerformed

    private void JCCFechaFinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCCFechaFinActionPerformed
    }//GEN-LAST:event_JCCFechaFinActionPerformed

    private void JCBHoraFinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHoraFinActionPerformed
        // Refrescamos la JTable de accesos
        iniciarAccesos();
    }//GEN-LAST:event_JCBHoraFinActionPerformed

    private void JTPartesMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTPartesMouseClicked
        try
        {
            // Obtenemos la fecha del parte seleccionado
            String Fecha = new SimpleDateFormat("yyyy-MM-dd  HH:mm:ss").format(JTPartes.getModel().getValueAt(JTPartes.getSelectedRow(),0));

            // Creamos el parte de seguridad
            Parte_Seguridad parte = new Parte_Seguridad(Fecha.substring(0,19));

            // Obtenemos los partes editables
            ArrayList partesEditables = parte.obtenerPartesEditables(DNI_Usu);

            // Si el parte es editable
            if (partesEditables.contains(UCAL.calendarToString(parte.getFechaI())))
            {
                // Guardamos la fecha del parte, para asi poder identificarlo de ser necesaria una posterior modificacion
                Fecha_Parte = UCAL.calendarToString(parte.getFechaI());

                // Habilitamos los campos de modificacion
                JCCFechaParte.setEnabled(true);
                JCBParte.setEnabled(true);
                JTAIncidencias.setEditable(true);

                // Activamos el boton de modificacion
                JBModificarParte.setEnabled(true);
            }
            else
            {
                // Desactivamos los campos de modificacion
                JCCFechaParte.setEnabled(false);
                JCBParte.setEnabled(false);
                JTAIncidencias.setEditable(false);

                // Desactivamos el boton de modificacion
                JBModificarParte.setEnabled(false);
            }

            // Activacion/desactivacion del boton de 'Eliminar partes'
            // Si el usuario es de tipo Administrador -> Puede eliminar los partes que desee
            if (Tipo_Usu.equals("ADMINISTRADOR"))        JBEliminarParte.setEnabled(true);
            // En cualquier otro caso (Vigilante)
            else
            {
                // Si todos los partes seleccionados son editables -> Activamos el boton de 'Eliminar partes'
                int[] partesSelecs = JTPartes.getSelectedRows();
                boolean eliminables = true;
                int i=0;
                while ((eliminables) && (i<partesSelecs.length))
                {
                    String key_parte = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTPartes.getModel().getValueAt(partesSelecs[i],0));
                    if (!partesEditables.contains(key_parte))           eliminables = false;
                    i++;
                }
                JBEliminarParte.setEnabled(eliminables);
            }

            // Mostramos la informacion del parte seleccionado
            JTFCodVig.setText(new Integer(parte.getVigilante().getCodVig()).toString());
            JTAIncidencias.setText(parte.getIncidencias());
            JCCFechaParte.setDate(parte.getFechaI().getTime());
            JCCFechaParte.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(parte.getFechaI().getTime()));
            JCBParte.getModel().setSelectedItem(UCAL.calendarToString(parte.getFechaI()).substring(11,16));

            // Si el formulario de muestra de partes esta oculto, lo mostramos
            if (!JPParte.isVisible())   JPParte.setVisible(true);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la información del parte seleccionado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTPartesMouseClicked

    private void JCBParteItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBParteItemStateChanged

    }//GEN-LAST:event_JCBParteItemStateChanged

    private void JCBParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBParteActionPerformed
    }//GEN-LAST:event_JCBParteActionPerformed

    private void JBModificarParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBModificarParteActionPerformed
        try
        {
            // Creamos el nuevo parte de seguridad
            Calendar cal = JCCFechaParte.getCalendar();
            String hora = (String) JCBParte.getModel().getSelectedItem();
            StringTokenizer st = new StringTokenizer(hora);
            int horas = Integer.parseInt(st.nextToken(":"));
            int minutos = Integer.parseInt(st.nextToken());
            cal.set(Calendar.HOUR,horas);
            cal.set(Calendar.MINUTE,minutos);
            Vigilante vig = new Vigilante();
            vig.setCodVig(Integer.parseInt(JTFCodVig.getText()));
            vig = new Vigilante(vig.obtenerDNI());

            Parte_Seguridad parte_new = new Parte_Seguridad(cal,vig,JTAIncidencias.getText());

            // Si Fecha_Parte es == null -> Insercion de parte
            if (Fecha_Parte == null)
            {
                // Avisamos de que vamos a insertar el parte de seguridad
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a insertar el parte de seguridad.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Insertamos el parte
                if (seleccion == 0)
                {
                    // Insertamos el parte de seguridad
                    parte_new.insertParteSeguridadBD();

                    // Refrescamos la tabla de partes
                    refrescarTPartes();

                    // Advertimos de que el parte ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El parte ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // Si Fecha_Parte no es nula -> Modificacion de parte
            else
            {
                // Avisamos de que vamos a actualizar el parte de seguridad
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el parte de seguridad.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el parte
                if (seleccion == 0)
                {
                    // Obtenemos el parte antes de ser modificado
                    Parte_Seguridad parte_old = new Parte_Seguridad(Fecha_Parte);

                    // Actualizamos el parte de seguridad viejo con el nuevo
                    parte_old.updateParteSeguridadBD(parte_new);

                    // Refrescamos la tabla de partes
                    refrescarTPartes();

                    // Advertimos de que el parte ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El parte ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el parte de seguridad.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBModificarParteActionPerformed

    private void JBEliminarParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarParteActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los partes de seguridad seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los partes de seguridad seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los partes
            if (seleccion == 0)
            {
                // Obtenemos los partes seleccionados
                int[] partesSelecs = JTPartes.getSelectedRows();

                // Eliminamos cada parte seleccionado
                Parte_Seguridad parte = new Parte_Seguridad();
                for (int i=0; i<partesSelecs.length; i++)
                {
                    // Creamos el parte a eliminar
                    String Fecha = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTPartes.getModel().getValueAt(partesSelecs[i],0));
                    parte.setFechaI(UCAL.stringToCalendar(Fecha.substring(0,19)));

                    // Eliminamos el parte de seguridad
                    parte.removeParteSeguridadBD();
                }

                // Escondemos el formulario
                JPParte.setVisible(false);

                // Refrescamos la lista de partes
                refrescarTPartes();

                // Advertimos de que los partes seleccionados han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los partes de seguridad seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar el parte de seguridad.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarParteActionPerformed

    private void JBAnadirParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirParteActionPerformed
        try
        {
            // Borramos el formulario
            JCBParte.getModel().setSelectedItem("");
            JTAIncidencias.setText("");

            // Establecemos el codigo de vigilante
            Vigilante vig = new Vigilante(DNI_Usu);
            JTFCodVig.setText(new Integer(vig.getCodVig()).toString());
            // Establecemos el dia de redaccion del parte
            JCCFechaParte.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(Calendar.getInstance().getTime()));
            // Establecemos la fecha del parte a la actual
            Calendar cal = Calendar.getInstance();
            int Hora = cal.get(Calendar.HOUR_OF_DAY);
            int Minutos = cal.get(Calendar.MINUTE);
            if (Minutos>30)     Minutos = 3;
            else                Minutos = 0;
            JCBParte.getModel().setSelectedItem(Hora + ":" + Minutos + "0");

            // Habilitamos los campos del formulario
            JCCFechaParte.setEnabled(true);
            JCBParte.setEnabled(true);
            JTAIncidencias.setEditable(true);

            // Desactivamos el boton de 'Eliminar parte' y activamos el de 'Modificar parte'
            JBEliminarParte.setEnabled(false);
            JBModificarParte.setEnabled(true);

            // Establecemos Fecha_Parte a null
            this.Fecha_Parte = null;

            // Si el formulario de muestra de partes esta oculto, lo mostramos
            if (!JPParte.isVisible())   JPParte.setVisible(true);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar el formulario de inserción del parte de seguridad.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBAnadirParteActionPerformed

    private void JTCuentasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTCuentasMouseClicked
        try
        {
            // Siempre que 'JTBAnadirCuenta' no este seleccionado -> Mostramos los datos de la cuenta seleccionada
            if (!JTBAnadirCuenta.isSelected())
            {
                // Creamos la cuenta
                Cuenta cuenta = new Cuenta ((String) JTCuentas.getModel().getValueAt(JTCuentas.getSelectedRow(),2));

                // Rellenamos los componentes del JTable
                JCBDNICuenta.getModel().setSelectedItem(cuenta.getPersona().getDNI());
                JCBTipoCuenta.getModel().setSelectedItem(cuenta.getTipo().getTipo());
                JTFUsuarioCuenta.setText(cuenta.getUsuario());
                JTFPassCuenta.setText(cuenta.getPass());

                // Activamos los botones de 'Eliminar cuenta' y 'Modificar cuenta'
                JBEliminarCuenta.setEnabled(true);
                JTBModificarCuenta.setEnabled(true);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la información de la cuenta.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTCuentasMouseClicked

    private void JTTUsuariosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTTUsuariosMouseClicked
        try
        {
            // Siempre que 'JTBAnadirT_Usuario' no este seleccionado -> Mostramos los datos del tipo de usuario seleccionado
            if (!JTBAnadirTUsuario.isSelected())
            {
                // Creamos el tipo de usuario
                T_Usuario t_usuario = new T_Usuario ((String) JTTUsuarios.getModel().getValueAt(JTTUsuarios.getSelectedRow(),0));

                // Rellenamos los componentes del JPanel de datos
                JTFTUsuario.setText(t_usuario.getTipo());
                JTFDescripTUsuario.setText(t_usuario.getDescripcion());

                // Activamos los botones de 'Eliminar tipo de usuario' y 'Modificar tipo de usuario'
                JBEliminarTUsuario.setEnabled(true);
                JTBModificarTUsuario.setEnabled(true);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la información del tipo de usuario.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTTUsuariosMouseClicked

    private void JBGrabarCuentaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarCuentaActionPerformed
        try
        {
            // Recogemos los datos
            String Usuario = JTFUsuarioCuenta.getText();
            String Pass = JTFPassCuenta.getText();
            String T_Usuario = (String) JCBTipoCuenta.getModel().getSelectedItem();
            String DNI = (String) JCBDNICuenta.getModel().getSelectedItem();

            // Creamos la cuenta nueva
            Cuenta cuenta_new;
            if (Pass.equals(""))    cuenta_new = new Cuenta(Usuario,new T_Usuario(T_Usuario), new Personal(DNI.substring(0,9)));
            else                    cuenta_new = new Cuenta(Usuario,Pass,new T_Usuario(T_Usuario), new Personal(DNI.substring(0,9)));

            // Comprobamos si se trata de una insercion o de una modificacion
            // -> Insercion
            if (JTBAnadirCuenta.isSelected())
            {
                // Avisamos de que vamos a dar de alta la cuenta
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta la cuenta.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta la cuenta
                if (seleccion == 0)
                {
                    // Insertamos la cuenta en la BD
                    cuenta_new.insertCuentaBD();
                    
                    // Actualizamos la tabla de cuentas
                    refrescarTCuentas();

                    // Advertimos de que la cuenta ha sido insertada correctamente
                    JOptionPane.showMessageDialog(this,"La cuenta ha sido insertada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // --> Modificacion
            else
            {
                // Avisamos de que vamos a modificar la cuenta
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar la cuenta seleccionada.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos la cuenta
                if (seleccion == 0)
                {
                    // Creamos la cuenta vieja
                    Cuenta cuenta_old = new Cuenta((String) JTCuentas.getModel().getValueAt(JTCuentas.getSelectedRow(),2));

                    // Modificamos la cuenta vieja con la nueva
                    cuenta_old.updateCuentaBD(cuenta_new);

                    // Actualizamos la tabla de cuentas
                    refrescarTCuentas();

                    // Advertimos de que la cuenta ha sido modificada correctamente
                    JOptionPane.showMessageDialog(this,"La cuenta seleccionada ha sido modificada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar la cuenta indicada.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarCuentaActionPerformed

    private void JTBModificarCuentaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarCuentaActionPerformed
        // Si JTBModificarCuenta esta seleccionado
        if (JTBModificarCuenta.isSelected())
        {
            // Mostramos el JPanel de introduccion de datos
            JPDatosCuenta.setVisible(true);

            // Des-seleccionamos el boton de 'Añadir cuenta' y el buscador
            JTBAnadirCuenta.setSelected(false);
            JTFBuscarCuenta.setEditable(false);
        }
        // Si no lo esta -> Escondemos el panel de datos y activamos el buscador
        else
        {
            JPDatosCuenta.setVisible(false);
            JTFBuscarCuenta.setEditable(true);
        }
    }//GEN-LAST:event_JTBModificarCuentaActionPerformed

    private void JBEliminarCuentaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarCuentaActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las cuentas seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las cuentas seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las cuentas
            if (seleccion == 0)
            {
                // Obtenemos las cuentas seleccionadas
                int[] cuentasSelecs = JTCuentas.getSelectedRows();

                // Eliminamos cada cuenta seleccionada
                for (int i=0; i<cuentasSelecs.length; i++)
                {
                    // Creamos la cuenta a eliminar
                    Cuenta cuenta = new Cuenta ();
                    cuenta.setUsuario((String) JTCuentas.getModel().getValueAt(cuentasSelecs[i],2));
                    
                    // Eliminamos la cuenta
                    cuenta.removeCuentaBD();
                }
                
                // Refrescamos las tablas de cuentas y tipos de usuario y desactivamos los botones de 'Eliminar cuenta' y 'Modificar cuenta'
                refrescarTCuentas();
                refrescarTUsuarios();
                JBEliminarCuenta.setEnabled(false);
                JTBModificarCuenta.setSelected(false);
                JTBModificarCuenta.setEnabled(false);

                // Advertimos de que las cuentas han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las cuentas seleccionadas han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las cuentas seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarCuentaActionPerformed

    private void JTBModificarTUsuarioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarTUsuarioActionPerformed
        // Si JTBModificarTUsuario esta seleccionado
        if (JTBModificarTUsuario.isSelected())
        {
            // Mostramos el JPanel de introduccion de datos
            JPDatosTUsuario.setVisible(true);

            // Des-seleccionamos el boton de 'Añadir tipo de usuario'
            JTBAnadirTUsuario.setSelected(false);
        }
        // Si no lo esta -> Escondemos el panel de datos
        else        JPDatosTUsuario.setVisible(false);
    }//GEN-LAST:event_JTBModificarTUsuarioActionPerformed

    private void JBEliminarTUsuarioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarTUsuarioActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los tipos de usuario
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los tipos de usuario seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los tipos de usuarios
            if (seleccion == 0)
            {
                // Obtenemos los tipos de usuario seleccionados
                int[] tiposSelecs = JTTUsuarios.getSelectedRows();

                // Eliminamos cada tipo de usuario seleccionado
                for (int i=0; i<tiposSelecs.length; i++)
                {
                    // Creamos el tipo de usuario a eliminar
                    T_Usuario t_usuario = new T_Usuario ();
                    t_usuario.setTipo((String) JTTUsuarios.getModel().getValueAt(tiposSelecs[i],0));

                    // Eliminamos el tipo de usuario
                    t_usuario.removeTUsuarioBD();
                }

                // Refrecamos las tablas del tab
                refrescarCuentas();

                // Advertimos de que los tipos de usuario han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los tipos de usuarios han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los tipos de usuarios seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarTUsuarioActionPerformed

    private void JBGrabarTUsuarioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarTUsuarioActionPerformed
        try
        {
            // Creamos el nuevo tipo de usuario
            T_Usuario t_usuario_new = new T_Usuario(JTFTUsuario.getText(),JTFDescripTUsuario.getText());

            // Comprobamos si se trata de una insercion o de una actualizacion
            // -> Insercion
            if (JTBAnadirTUsuario.isSelected())
            {
                // Avisamos de que vamos a dar de alta el tipo de usuario
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta el tipo de usuario.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta el tipo de usuario
                if (seleccion == 0)
                {
                    // Insertamos el tipo de usuario en la BD
                    t_usuario_new.insertTUsuarioBD();

                    // Actualizamos la tabla de tipos de usuarios
                    refrescarTUsuarios();

                    // Advertimos de que el tipo de usuario ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El tipo de usuario ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Modificacion
            else
            {
                // Avisamos de que vamos a modificar el tipo de usuario
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el tipo de usuario seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el tipo de usuario
                if (seleccion == 0)
                {
                    // Creamos el tipo de usuario viejo
                    T_Usuario t_usuario_old = new T_Usuario((String) JTTUsuarios.getModel().getValueAt(JTTUsuarios.getSelectedRow(),0));

                    // Modificamos el viejo tipo de usuario con el nuevo
                    t_usuario_old.updateTUsuarioBD(t_usuario_new);

                    // Actualizamos la tablas de cuentas
                    refrescarCuentas();

                    // Advertimos de que el tipo de usuario ha sido modificado
                    JOptionPane.showMessageDialog(this,"El tipo de usuario ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el tipo de usuario.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarTUsuarioActionPerformed

    private void JCBDNICuentaKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JCBDNICuentaKeyReleased
    }//GEN-LAST:event_JCBDNICuentaKeyReleased

    private void JCBDNICuentaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBDNICuentaActionPerformed
    }//GEN-LAST:event_JCBDNICuentaActionPerformed

    private void JTFBuscarCuentaKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarCuentaKeyReleased
        // Con cada letra hacemos que se muestren solo los DNIs de aquellas personas que cumplan con el criterio de busqueda
        refrescarCBDNIs();
    }//GEN-LAST:event_JTFBuscarCuentaKeyReleased

    private void JTFBuscarCuentaKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarCuentaKeyTyped
    }//GEN-LAST:event_JTFBuscarCuentaKeyTyped

    private void JTFBuscarKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarKeyTyped
    }//GEN-LAST:event_JTFBuscarKeyTyped

    private void JTPNotasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTPNotasMouseClicked
}//GEN-LAST:event_JTPNotasMouseClicked

    private void JBDescartarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBDescartarActionPerformed
        try
        {
            // Avisamos de que vamos a descartar la nota
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a descartar las notas seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Descartamos las notas
            if (seleccion == 0)
            {
                // Obtenemos las notas
                int[] notasSelecs;
                if (JTEntrada.isShowing())      notasSelecs = JTEntrada.getSelectedRows();
                else                            notasSelecs = JTSalida.getSelectedRows();

                // Descartamos cada nota
                for (int i=0; i<notasSelecs.length; i++)
                {
                    // Creamos la nota
                    String Fecha;
                    String DNI;
                    if (JTEntrada.isShowing())
                    {
                        Fecha = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTEntrada.getModel().getValueAt(notasSelecs[i],2));
                        DNI = new Personal().obtenerDNI((String) JTEntrada.getModel().getValueAt(notasSelecs[i],0));
                    }
                    else
                    {
                        Fecha = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTSalida.getModel().getValueAt(notasSelecs[i],1));
                        DNI = DNI_Usu;
                    }
                    
                    Nota nota = new Nota(Fecha,DNI);
                    
                    // Descartamos la nota
                    // Si se trata de un destinatario
                    if (JTEntrada.isShowing())
                    {
                        nota.descartada(DNI_Usu);

                        // Ejecutamos el disparador
                        nota.disparadorDestinatario();
                    }
                    // Si se trata del autor
                    else
                    {
                        nota.descartada();

                        // Ejecutamos el disparador
                        nota.disparadorAutor();
                    }
                }

                // Escondemos el formulario
                JPContenidoNota.setVisible(false);

                // Refrescamos la bandeja de entrada o de salida, segun corresponda
                if (JTEntrada.isShowing())  refrescarBandejaEntrada();
                else                        refrescarBandejaSalida();

                // Advertimos de que las notas han sido descartada correctamente
                JOptionPane.showMessageDialog(this,"Las notas han sido descartadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible descartar las notas seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
}//GEN-LAST:event_JBDescartarActionPerformed

    private void JTEntradaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTEntradaMouseClicked
        try
        {
            // Si el formulario de muestra de notas esta oculto, lo mostramos
            if (!JPContenidoNota.isVisible())   JPContenidoNota.setVisible(true);

            // Obtenemos la nota
            String apellidosNombre = (String) JTEntrada.getModel().getValueAt(JTEntrada.getSelectedRow(),0);
            String Fecha = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTEntrada.getModel().getValueAt(JTEntrada.getSelectedRow(),2));
            String DNI_A = new Personal().obtenerDNI(apellidosNombre);
            Nota nota = new Nota(Fecha,DNI_A);

            // Mostramos la nota
            mostrarNota(nota);

            // Marcamos la nota como leida si es que no lo estaba
            if (!nota.getDestinatario(DNI_Usu).getLeida())      nota.leida(DNI_Usu);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la nota.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
}//GEN-LAST:event_JTEntradaMouseClicked

    private void JBEnviarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEnviarActionPerformed
        // Avisamos de que vamos a enviar la nota
        int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a enviar la nota.\n" +
                "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

        // Si se acepta la operacion -> Enviamos la nota
        if (seleccion == 0)     enviarNota();
}//GEN-LAST:event_JBEnviarActionPerformed

    private void JTSalidaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTSalidaMouseClicked
        try
        {
            // Si el formulario de muestra de notas esta oculto, lo mostramos
            if (!JPContenidoNota.isVisible())   JPContenidoNota.setVisible(true);

            // Obtenemos la nota
            String Fecha = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTSalida.getModel().getValueAt(JTSalida.getSelectedRow(),1));
            Nota nota = new Nota(Fecha,DNI_Usu);

            // Mostramos la nota
            mostrarNota(nota);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la nota.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTSalidaMouseClicked

    private void JPBandejasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JPBandejasMouseClicked
        // Refrecamos las bandejas de entrada y salida
        refrescarBandejas();
    }//GEN-LAST:event_JPBandejasMouseClicked

    private void JMISalirMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JMISalirMouseClicked
    }//GEN-LAST:event_JMISalirMouseClicked

    private void JMISalirActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JMISalirActionPerformed
        // Cerramos la aplicacion
        System.exit(0);
    }//GEN-LAST:event_JMISalirActionPerformed

    private void JMIImportarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JMIImportarActionPerformed
        try
        {
            // Desplegamos un JFileChooser en modo 'Abrir'
            JFileChooser jfc = new JFileChooser();
            int opcion = jfc.showOpenDialog(this);

            // Si se pulsa 'Aceptar'
            if (opcion == jfc.APPROVE_OPTION)
            {
                // Recogemos el fichero seleccionado
                File fichero = jfc.getSelectedFile();

                // Avisamos de que vamos a importar la base de datos
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a importar la BD.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Importamos la BD
                if (seleccion == 0)
                {
                    // Abrimos el fichero
                    FileReader fr = new FileReader(fichero);
                    
                    // Procedemos a la lectura del fichero -> Sentencias a ejecutar
                    int caract = fr.read();
                    String query = "";
                    while(caract != -1)
                    {
                        // Añadimos el caracter al String
                        // Si leemos cualquier caracter distinto del tabulador
                        if (caract != '\t')             query = query + (char) caract;
                        // Si leemos un tabulador
                        else if (caract == '\t')         query = query + ' ';

                        // Si nos encontramos en una linea de comentarios
                        if ((caract == '-') && (query.trim().startsWith("--")))
                        {
                            // Leemos hasta encontrar \n y reiniciamos la String
                            caract = fr.read();
                            while ((caract != -1) && (caract != '\n'))  { caract = fr.read(); }
                            query = "";
                        }
                        // Si hemos terminado de leer una sentencia -> La ejecutamos y pasamos a la siguiente String
                        else if (caract == ';')
                        {
                            // Ejecutamos la query
                            UBD.ejecutarUpdate(query.substring(1));
                            if (query.endsWith("DROP DATABASE IF EXISTS bd;"))  UBD.setDireccionURL("jdbc:mysql://localhost/");
                            else if  (query.endsWith("CREATE DATABASE bd;"))    UBD.setDireccionURL("jdbc:mysql://localhost/bd");

                            // Reiniciamos la String
                            query = "";
                        }
                        caract = fr.read();
                    }
                    //Cerramos el fichero
                    fr.close();

                    // Advertimos de que la base de datos ha sido importada correctamente
                    JOptionPane.showMessageDialog(this,"La base de datos ha sido importada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible importar toda la BD.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
        finally
        {
            // Inicializamos todas las JTables,...
            iniciarDatos();

            // Seleccionamos la primera pestaña (Accesos)
            JTPFuncionalidades.setSelectedIndex(0);
        }
    }//GEN-LAST:event_JMIImportarActionPerformed

    private void JMIExportarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JMIExportarActionPerformed
        try
        {
            // Desplegamos un JFileChooser en modo 'Guardar'
            JFileChooser jfc = new JFileChooser();
            int opcion = jfc.showSaveDialog(this);

            // Si se pulsa 'Aceptar'
            if (opcion == jfc.APPROVE_OPTION)
            {
                // Recogemos el fichero seleccionado
                File fichero = jfc.getSelectedFile();

                // Avisamos de que vamos a exportar la base de datos
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a exportar la BD.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Exportamos la BD
                if (seleccion == 0)
                {
                    // Obtenemos las tablas existentes
                    String query = "SELECT table_name FROM information_schema.tables WHERE table_schema = 'bd';";
                    ResultSet rs = UBD.ejecutarQuery(query);
                    ArrayList tablas = new ArrayList();
                    while (rs.next())   { tablas.add(rs.getString(1)); }
                    UBD.cerrarConexion();

                    // Creamos un PrintWriter con el que escribir en el fichero
                    PrintWriter writer = new PrintWriter(fichero);

                    // Para cada tabla -> Obtenemos la poblacion
                    Iterator it = tablas.iterator();
                    while (it.hasNext())
                    {
                        // Obtenemos el nombre de la tabla
                        String nomTabla = (String) it.next();
                        // Obtenemos la poblacion de la tabla
                        query = "SELECT * FROM " + nomTabla + ";";
                        rs = UBD.ejecutarQuery(query);

                        // Obtenemos el numero de campos
                        int numCamps = rs.getMetaData().getColumnCount();

                        // Escribimos la cabecera de la poblacion
                        writer.println("-- TABLA '" + nomTabla + "'");
                        // Escribimos en el fichero cada tupla
                        String linea = "INSERT INTO " + nomTabla + " VALUES (";
                        String values = "";
                        while (rs.next())
                        {
                            for (int i=0; i<numCamps; i++)
                            {
                                rs.getObject(i+1);
                                if (rs.wasNull())   values = values + "null, ";
                                else
                                {
                                    if (rs.getObject(i+1).getClass() == Integer.class)  values = values + rs.getInt(i+1) + ", ";
                                    else                                                values = values + "'" + rs.getObject(i+1) + "', ";
                                }
                            }
                            writer.println(linea + values.substring(0,values.length()-2) + ");");
                            values = "";
                        }
                        writer.println();
                        writer.println();
                    }

                    // Cerramos el fichero
                    writer.close();

                    // Advertimos de que la base de datos ha sido exportada correctamente
                    JOptionPane.showMessageDialog(this,"La base de datos ha sido exportada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible exportar la BD.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JMIExportarActionPerformed

    private void JMICambiarDatosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JMICambiarDatosActionPerformed
        // Desplegamos el JDCuenta, para qu el usuario pueda configurar su cuenta a su gusto
        new JDCuenta(this,true,Usuario).setVisible(true);
    }//GEN-LAST:event_JMICambiarDatosActionPerformed

    private void JTFCIFKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFCIFKeyTyped
    }//GEN-LAST:event_JTFCIFKeyTyped

    private void JTFCIFKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFCIFKeyReleased
            // Rellenamos la tabla de empresas
            refrescarTablasEmpresasPersonal("E");
    }//GEN-LAST:event_JTFCIFKeyReleased

    private void JTFNombreEmpKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFNombreEmpKeyReleased
            // Rellenamos la tabla de empresas
            refrescarTablasEmpresasPersonal("E");
    }//GEN-LAST:event_JTFNombreEmpKeyReleased

    private void JTFNombreDepKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFNombreDepKeyReleased
            // Rellenamos la tabla de empresas
            refrescarTablasEmpresasPersonal("D");
    }//GEN-LAST:event_JTFNombreDepKeyReleased

    private void JTFActividadKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFActividadKeyReleased
            // Rellenamos la tabla de empresas
            refrescarTablasEmpresasPersonal("D");
    }//GEN-LAST:event_JTFActividadKeyReleased

    private void JTFFiltroKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFFiltroKeyReleased
            // Rellenamos la tabla de empresas
            refrescarTablasEmpresasPersonal("GC");
    }//GEN-LAST:event_JTFFiltroKeyReleased

    private void JBMostrarTodoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBMostrarTodoActionPerformed
        // Deseleccionamos todos los JTables
        JTEmpresas.clearSelection();
        JTDepartamentos.clearSelection();
        JTEmpleados.clearSelection();
        JTContactos.clearSelection();

        // Borramos los datos generales
        borrarFormDP();
        JCBDNIDep.setModel(new DefaultComboBoxModel());

        // Escondemos los JPanels
        mostrarJPanelsPersonal("");
        JPEmpresasD.setVisible(false);
        JPDepartamentosD.setVisible(false);
        
        // Borramos los JTextFields de busqueda
        JTFCIF.setText("CIF ...");
        JTFCIF.setForeground(new Color(204,204,204));
        JTFNombreEmp.setText("Nombre ...");
        JTFNombreEmp.setForeground(new Color(204,204,204));
        JTFNombreDep.setText("Nombre ...");
        JTFNombreDep.setForeground(new Color(204,204,204));
        JTFActividad.setText("Actividad ...");
        JTFActividad.setForeground(new Color(204,204,204));
        JTFFiltro.setText("Buscar ...");
        JTFFiltro.setForeground(new Color(204,204,204));

        // Refrescamos todos los JTables
        refrescarTablasEmpresasPersonal("EDGC");

        // Vaciamos las tablas de telefonos, E-Mails y cargos
        String[] Etiquetas = {"Teléfonos"};
        JTTfns.setModel(new DefaultTableModel(Etiquetas,0));
        Etiquetas[0] = "E-Mails";
        JTEmails.setModel(new DefaultTableModel(Etiquetas,0));
        Etiquetas[0] = "Cargos";
        JTCargos.setModel(new DefaultTableModel(Etiquetas,0));

        // Restablecemos todos los botones a su estado original
        refrescarBotonesEmpPers();
    }//GEN-LAST:event_JBMostrarTodoActionPerformed

    private void JTPDepartamentosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTPDepartamentosMouseClicked
        try
        {
            String empresa = "";
            if (JTEmpresas.getSelectedRow() != -1) empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);

            // Refrescamos las JTables de telefonos e e-mails
            refrescarTablasEmpresasPersonal("TM");

            // Mostramos el JPanel
            if (JTBAnadirPerContac.isSelected() && !JTBModificarPerContac.isSelected())
            {
                borrarFormDP();
                JPElimTE.setVisible(false);
                JPDNIAuto.setVisible(false);
                JTFDNIDP.setEditable(true);
                JBAnadirPrestDep1.setEnabled(false);
                if (empresa.equals("Ocasional"))   refrescarCBOcasional();
                else                               refrescarCBContactos();
                mostrarForm();
                JPElimC.setVisible(false);
            }
            // Si JTBModificarPerContac esta seleccionado o no esta seleccionado ninguno de los dos botones
            else
            {
                // Deseleccionamos el boton de modificar personal/contacto
                JTBModificarPerContac.setSelected(false);
                JTBModificarPerContac.setEnabled(false);
                JBEliminarPerContac.setEnabled(false);
                mostrarJPanelsPersonal("");

                // Refrescamos las tablas de telefonos, E-Mails y cargos
                refrescarTablasEmpresasPersonal("TMX");
            }
            if ((empresa.equals("Informática Navarra S.A.") || empresa.equals("Ocasional")) && (JTContactos.isShowing()))
                mostrarJPanelsPersonal("");
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la persona seleccionada.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTPDepartamentosMouseClicked

    private void JBEliminarEmpresaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarEmpresaActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las empresas seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las empresas seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las empresas
            if (seleccion == 0)
            {
                // Obtenemos las empresas seleccionadas
                int[] empSelecs = JTEmpresas.getSelectedRows();

                // Eliminamos cada empresa
                Empresa emp = new Empresa();
                for (int i=0; i<empSelecs.length; i++)
                {
                    // Obtenemos el codigo de empresa
                    emp.setNombre((String) JTEmpresas.getModel().getValueAt(empSelecs[i],0));
                    emp.setCodEmp(emp.obtenerCodEmp());

                    // Eliminamos la empresa
                    emp.removeEmpresaBD();
                }

                // Refrescamos todo el TabbedPane de empresas y personal
                iniciarEmpresasPersonal();

                // Restablecemos todos los botones a su estado original
                refrescarBotonesEmpPers();

                // Escondemos el JPanel de empresas
                JPEmpresasD.setVisible(false);

                // Advertimos de que las empresas han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las empresas han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            iniciarEmpresasPersonal();
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar todas las empresas seleccionadas.\nRecuerda que las empresas 'Informática Navarra S.A.' y 'Ocasional' no pueden ser eliminadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarEmpresaActionPerformed

    private void JBEliminarDepartamentoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarDepartamentoActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los departamentos seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los departamentos seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los departamentos
            if (seleccion == 0)
            {
                // Obtenemos los departamentos seleccionados
                int[] depSelecs = JTDepartamentos.getSelectedRows();

                // Obtenemos el codigo de empresa
                Empresa emp = new Empresa();
                emp.setNombre((String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0));
                emp.setCodEmp(emp.obtenerCodEmp());

                // Eliminamos cada departamento
                Departamento dep = new Departamento();
                for (int i=0; i<depSelecs.length; i++)
                {
                    // Obtenemos el codigo de departamento
                    String Cod_Dep = emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(depSelecs[i],0));

                    // Creamos el departamento a eliminar
                    dep.setCodDep(Cod_Dep);

                    // Borramos el departamento
                    emp.removeDepartamentoBD(dep);
                }

                // Refrescamos todo el TabbedPane de empresas y personal
                iniciarEmpresasPersonal();

                // Advertimos de que las empresas han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Los departamentos han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los departamentos seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarDepartamentoActionPerformed

    private void JBEliminarPerContacActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarPerContacActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las personas seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las personas seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las personas
            if (seleccion == 0)
            {
                // Obtenemos las personas seleccionadas
                int[] perSelecs;
                if (JTEmpleados.isShowing())        perSelecs = JTEmpleados.getSelectedRows();
                else                                perSelecs = JTContactos.getSelectedRows();

                // Obtenemos el nombre de la empresa
                String Nombre_Empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);

                // PERSONAL OCASIONAL
                if (Nombre_Empresa.equals("Ocasional"))
                {
                    // Eliminamos cada persona
                    Ocasional ocas;
                    for (int i=0; i<perSelecs.length; i++)
                    {
                        // Creamos la persona a eliminar
                        ocas = new Ocasional((String) JTEmpleados.getModel().getValueAt(perSelecs[i],0));

                        // Eliminamos la persona
                        ocas.removeOcasional();
                    }
                }
                // PERSONAL RESIDENTE Y EMPLEADOS VIGILANTES
                else if (Nombre_Empresa.equals("Informática Navarra S.A.") ||
                        (Nombre_Empresa.equals("Compañía Integral de Seguridad") && (JTEmpleados.isShowing())))
                {
                    // Creamos la persona a eliminar
                    Personal pers = new Personal();

                    // Eliminamos cada persona
                    for (int i=0; i<perSelecs.length; i++)
                    {
                        // obtenemos la persona
                        pers.setDNI((String) JTEmpleados.getModel().getValueAt(perSelecs[i],0));

                        // Eliminamos la persona
                        pers.removePersonalBD();
                    }
                }
                // PERSONAL PERIODICO O CONTACTOS
                else
                {
                    // Obtenemos la empresa y departamento
                    Empresa emp = new Empresa();
                    emp.setNombre(Nombre_Empresa);
                    emp.setCodEmp(emp.obtenerCodEmp());
                    Departamento dep = new Departamento();
                    dep.setCodDep(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));

                    // Creamos la persona a eliminar
                    Periodico_Contacto pc;

                    // Eliminamos cada persona
                    for (int i=0; i<perSelecs.length; i++)
                    {
                        // Obtenemos la persona
                        if (JTEmpleados.isShowing())    pc = (Periodico_Contacto) emp.getPersonalBD(dep.getCodDep()).get((String) JTEmpleados.getModel().getValueAt(perSelecs[i],0));
                        else                            pc = (Periodico_Contacto) emp.getPersonalBD(dep.getCodDep()).get((String) JTContactos.getModel().getValueAt(perSelecs[i],0));
                        
                        // Eliminamos la persona
                        emp.removePeriodicoContactoBD(dep, pc);
                    }
                }

                // Ejecutamos el disparador
                disparador();
                
                // Refrescamos todo el TabbedPane de empresas y personal
                iniciarEmpresasPersonal();

                // Escondemos los paneles de edicion innecesarios
                JPElimTE.setVisible(false);
                JPElimC.setVisible(false);

                // Advertimos de que las personas han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las personas han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las personas seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarPerContacActionPerformed

    private void JCBEmpresasActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBEmpresasActionPerformed
        try
        {
            // Si nos encontramos visualizando/modificando datos
            if (!JTBAnadirPerContac.isSelected())
            {
                // Rellenamos los datos especificos
                Ocasional oca = new Ocasional(DNI_Personal,(String) JCBEmpresas.getModel().getSelectedItem());
                JCBActividad1.getModel().setSelectedItem(oca.getAcceso().getCodAcc() + " - " + oca.getAcceso().getActividad());
                JCCFCaducidad.setDate(oca.getFCaducidad().getTime());
                JCCFCaducidad.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(oca.getFCaducidad().getTime()));
                JTANotas.setText(oca.getNotas());
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la persona seleccionada.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JCBEmpresasActionPerformed

    private void JBGrabarPerContacActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarPerContacActionPerformed
        try
        {
            // Obtenemos los datos personales
            String DNI = JTFDNIDP.getText();
            String Apellido1 = JTFApellido1DP.getText();
            String Apellido2 = JTFApellido2DP.getText();
            String Nombre = JTFNombreDP.getText();

            // Obtenemos los nuevos telefonos e E-Mails
            ArrayList<String> telefonos = new ArrayList();
            DefaultTableModel dtm = (DefaultTableModel) JTTfns.getModel();
            Vector v = dtm.getDataVector();
            Iterator it = v.iterator();
            while (it.hasNext())
            {
                Vector ve = (Vector) it.next();
                telefonos.add((String) ve.get(0));
            }

            ArrayList<String> emails = new ArrayList();
            dtm = (DefaultTableModel) JTEmails.getModel();
            v = dtm.getDataVector();
            it = v.iterator();
            while (it.hasNext())
            {
                Vector ve = (Vector) it.next();
                emails.add((String) ve.get(0));
            }

            // Comprobamos si se trata de una insercion o una modificacion de persona
            // -> Insercion
            if (JTBAnadirPerContac.isSelected())
            {
                // Avisamos de que vamos a dar de alta a la persona
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta la persona.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta la persona
                if (seleccion == 0)
                {
                    // Obtenemos los cargos
                    HashMap<String,Cargo> cargos = new HashMap();
                    if (JTContactos.isShowing())
                    {
                        dtm = (DefaultTableModel) JTCargos.getModel();
                        v = dtm.getDataVector();
                        it = v.iterator();
                        Cargo car = new Cargo();
                        while (it.hasNext())
                        {
                            Vector ve = (Vector) it.next();
                            car = new Cargo (car.obtenerCodCargo((String) ve.get(0)));
                            cargos.put((String) ve.get(0), car);
                        }
                    }

                    // Obtenemos el nombre de la empresa
                    String Nombre_Empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);

                    // PERSONAL OCASIONAL
                    if (Nombre_Empresa.equals("Ocasional"))
                    {
                        // Creamos la persona
                        StringTokenizer st = new StringTokenizer((String)JCBActividad1.getModel().getSelectedItem());
                        T_Acceso acc = new T_Acceso(st.nextToken());
                        Ocasional oca = new Ocasional(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), (String) JCBEmpresas.getModel().getSelectedItem(), acc, JCCFCaducidad.getCalendar(), JTANotas.getText(), new HashMap(), telefonos, emails);

                        // Insertamos la persona
                        oca.insertOcasionalBD();
                    }
                    // PERSONAL RESIDENTE
                    else if (Nombre_Empresa.equals("Informática Navarra S.A."))
                    {
                        // Creamos la persona
                        String nivel = (String) JCBNivel.getModel().getSelectedItem();
                        Nivel niv = null;
                        if (!nivel.equals("Ninguno"))   niv = new Nivel(Integer.parseInt(nivel));
                        Residente res = new Residente(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), niv, telefonos, emails);

                        // Insertamos la persona
                        res.insertResidenteBD();
                    }
                    // PERSONAL VIGILANTE
                    else if (Nombre_Empresa.equals("Compañía Integral de Seguridad"))
                    {
                        // Obtenemos la empresa y departamento
                        Empresa emp = new Empresa();
                        emp.setNombre(Nombre_Empresa);
                        emp.setCodEmp(emp.obtenerCodEmp());
                        Departamento dep = new Departamento();
                        dep.setCodDep(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));

                        // Si se trata de un vigilante
                        if (JTEmpleados.isShowing())
                        {
                            // Creamos la persona
                            Vigilante vig = new Vigilante(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), Integer.parseInt(JTFCodV.getText()), telefonos, emails);
                            
                            // Insertamos la persona
                            emp.insertVigilanteBD(dep, vig);
                        }
                        // Si se trata de un contacto
                        else
                        {
                            // Creamos la persona
                            cargos.put("RP", new Cargo("RP"));
                            Periodico_Contacto pc = new Periodico_Contacto (DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu),cargos,telefonos,emails);

                            // Insertamos la persona
                            emp.insertPersonaBD(dep, pc);
                            
                            // Creamos e insertamos una cuenta para la persona si es que no dispone de ninguna
                            String usuario = pc.getNombre().substring(0,1) + pc.getApellido1() + pc.getApellido2().substring(0,1);
                            Cuenta cuenta = new Cuenta(usuario.toLowerCase(),new T_Usuario("RP"),pc);
                            cuenta.removeCuentaBD();
                            cuenta.insertCuentaBD();
                        }
                    }
                    // PERSONAL PERIODICO O CONTACTOS
                    else
                    {
                        // Creamos la persona
                        Periodico_Contacto percontac = new Periodico_Contacto(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), cargos, telefonos, emails);

                        // Insertamos la persona con cargo por defecto RP y le asignamos una cuenta nueva
                        Empresa emp = new Empresa();
                        emp.setNombre(Nombre_Empresa);
                        emp.setCodEmp(emp.obtenerCodEmp());
                        Departamento dep = new Departamento();
                        dep.setCodDep(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));
                        if (JTContactos.isShowing())    cargos.put("RP", new Cargo("RP"));
                        emp.insertPeriodicoContactoBD(dep, percontac);
                        if (JTContactos.isShowing())
                        {
                            // Creamos una cuenta para la persona
                            String usuario = percontac.getNombre().substring(0,1) + percontac.getApellido1() + percontac.getApellido2().substring(0,1);
                            Cuenta cuenta = new Cuenta(usuario.toLowerCase(),new T_Usuario("RP"),percontac);
                            cuenta.removeCuentaBD();
                            cuenta.insertCuentaBD();
                        }
                    }

                    // Escondemos los paneles de edicion y de datos innecesarios
                    JPElimTE.setVisible(false);
                    JPElimC.setVisible(false);
                    mostrarJPanelsPersonal("");

                    // Deseleccionamos el boton de adicion
                    JTBAnadirPerContac.setSelected(false);

                    // Refrescamos la tabla de empleados, contactos, telefonos, E-Mails y cargos
                    refrescarTablasEmpresasPersonal("GC");

                    // Borramos las tablas de telefonos, E-Mails y cargos
                    String[] Etiquetas = {"Teléfonos"};
                    JTTfns.setModel(new DefaultTableModel(Etiquetas,0));
                    Etiquetas[0] = "E-Mails";
                    JTEmails.setModel(new DefaultTableModel(Etiquetas,0));
                    Etiquetas[0] = "Cargos";
                    JTCargos.setModel(new DefaultTableModel(Etiquetas,0));

                    // Advertimos de que la persona ha sido insertada correctamente
                    JOptionPane.showMessageDialog(this,"La persona ha sido insertada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Modificacion
            else
            {
                // Avisamos de que vamos a modificar a la persona
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar la persona seleccionada.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos la persona
                if (seleccion == 0)
                {
                    // Obtenemos los cargos
                    HashMap<String,Cargo> cargos2 = new HashMap();
                    if (JTContactos.isShowing())
                    {
                        dtm = (DefaultTableModel) JTCargos.getModel();
                        v = dtm.getDataVector();
                        it = v.iterator();
                        Cargo car = new Cargo();
                        while (it.hasNext())
                        {
                            Vector ve = (Vector) it.next();
                            car = new Cargo (car.obtenerCodCargo((String) ve.get(0)));
                            cargos2.put(car.obtenerCodCargo((String) ve.get(0)), car);
                        }
                    }

                    // Obtenemos el nombre de la empresa
                    String Nombre_Empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);

                    // PERSONAL OCASIONAL
                    if (Nombre_Empresa.equals("Ocasional"))
                    {
                        // Creamos la persona
                        StringTokenizer st = new StringTokenizer((String)JCBActividad1.getModel().getSelectedItem());
                        T_Acceso acc = new T_Acceso(st.nextToken());
                        Ocasional oca = new Ocasional(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), (String) JCBEmpresas.getModel().getSelectedItem(), acc, JCCFCaducidad.getCalendar(), JTANotas.getText(), new HashMap(), telefonos, emails);

                        // Obtenemos la persona vieja
                        Ocasional oca_old = new Ocasional ((String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0), (String) JCBEmpresas.getModel().getSelectedItem());

                        // Actualizamos la persona vieja con la nueva
                        oca_old.updateOcasionalBD(oca);
                    }
                    // PERSONAL RESIDENTE
                    else if (Nombre_Empresa.equals("Informática Navarra S.A."))
                    {
                        // Creamos la persona
                        String nivel = (String) JCBNivel.getModel().getSelectedItem();
                        Nivel niv = null;
                        if (!nivel.equals("Ninguno"))   niv = new Nivel(Integer.parseInt(nivel));
                        Residente res = new Residente(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), niv, telefonos, emails);

                        // Obtenemos la persona vieja
                        Residente res_old = new Residente((String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0));

                        // Actualizamos la persona vieja con la nueva
                        res_old.updateResidenteBD(res);
                    }
                    // PERSONAL VIGILANTE
                    else if (Nombre_Empresa.equals("Compañía Integral de Seguridad"))
                    {
                        // Obtenemos la empresa y departamento
                        Empresa emp = new Empresa();
                        emp.setNombre(Nombre_Empresa);
                        emp.setCodEmp(emp.obtenerCodEmp());
                        Departamento dep = new Departamento();
                        dep.setCodDep(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));

                        // Si se trata de un vigilante
                        if (JTEmpleados.isShowing())
                        {
                            // Creamos la persona
                            Vigilante vig = new Vigilante(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), Integer.parseInt(JTFCodV.getText()), telefonos, emails);

                            // Obtenemos la persona vieja
                            Vigilante vig_old = new Vigilante ((String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0));

                            // Actualizamos la persona vieja con la nueva
                            emp.updatePersonaBD(dep, vig_old, vig);
                        }
                        // Si se trata de un contacto
                        else
                        {
                            // Creamos la persona
                            Periodico_Contacto pc = new Periodico_Contacto (DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu),cargos2,telefonos,emails);
                            
                            // Obtenemos la persona vieja
                            Periodico_Contacto pc_old = (Periodico_Contacto) emp.getPersonalBD(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0))).get(DNI);

                            // Actualizamos la persona vieja con la nueva
                            emp.updatePersonaBD(dep, pc_old, pc);

                            // Si desempeña como responsables de personal y antes no lo hacia -> Le creamos e insertamos una cuenta
                            if ((!pc_old.getCargos().containsKey("RP")) && (pc.getCargos().containsKey("RP")))
                            {
                                Cuenta cuenta = new Cuenta(pc.getNombre().substring(0,1) + pc.getApellido1() + pc.getApellido2().substring(0,1),new T_Usuario("RP"),pc);
                                cuenta.insertCuentaBD();
                            }
                        }
                    }
                    // PERSONAL PERIODICO O CONTACTOS
                    else
                    {
                        // Creamos la persona
                        Periodico_Contacto percontac = new Periodico_Contacto(DNI, Apellido1, Apellido2, Nombre, new Personal(DNI_Usu), cargos2, telefonos, emails);

                        // Obtenemos la persona vieja
                        if (JTEmpleados.isShowing())    DNI = (String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0);
                        else                            DNI = (String) JTContactos.getModel().getValueAt(JTContactos.getSelectedRow(),0);

                        // Actualizamos la persona vieja con la nueva
                        Empresa emp = new Empresa();
                        emp.setNombre(Nombre_Empresa);
                        emp.setCodEmp(emp.obtenerCodEmp());
                        Departamento dep = new Departamento();
                        dep.setCodDep(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));
                        Periodico_Contacto percontac_old = (Periodico_Contacto) emp.getPersonalBD(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0))).get(DNI);
                        emp.updatePersonaBD(dep, percontac_old, percontac);

                        // Si desempeña como responsables de personal y antes no lo hacia -> Le creamos e insertamos una cuenta
                        if ((!percontac_old.getCargos().containsKey("RP")) && (percontac.getCargos().containsKey("RP")))
                        {
                            Cuenta cuenta = new Cuenta(percontac.getNombre().substring(0,1) + percontac.getApellido1() + percontac.getApellido2().substring(0,1),new T_Usuario("RP"),percontac);
                            cuenta.insertCuentaBD();
                        }

                        // Escondemos los paneles
                        JPElimTE.setVisible(false);
                        JPElimC.setVisible(false);
                    }
                    
                    // Escondemos los paneles innecesarios
                    JPElimTE.setVisible(false);
                    JPElimC.setVisible(false);
                    mostrarJPanelsPersonal("");

                    // Deseleccionamos el boton de modificacion
                    JTBModificarPerContac.setSelected(false);
                    JTBModificarPerContac.setEnabled(false);

                    // Refrescamos la tabla de empleados, contactos, telefonos, E-Mails y cargos
                    refrescarTablasEmpresasPersonal("GC");

                    // Borramos las tablas de telefonos, E-Mails y cargos
                    String[] Etiquetas = {"Teléfonos"};
                    JTTfns.setModel(new DefaultTableModel(Etiquetas,0));
                    Etiquetas[0] = "E-Mails";
                    JTEmails.setModel(new DefaultTableModel(Etiquetas,0));
                    Etiquetas[0] = "Cargos";
                    JTCargos.setModel(new DefaultTableModel(Etiquetas,0));

                    // Advertimos de que la persona ha sido insertada correctamente
                    JOptionPane.showMessageDialog(this,"La persona seleccionada ha sido modificada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar o modificar la información solicitada.","ERROR",JOptionPane.ERROR_MESSAGE);

            // Si la excepcion es producida por un vigilante -> Lo borrarmos
            String Nombre_Empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);
            if (Nombre_Empresa.equals("Compañía Integral de Seguridad") && JTEmpleados.isShowing() && JTBAnadirPerContac.isSelected())
            {
                try
                {
                    // Obtenemos la empresa y departamento
                    Empresa emp = new Empresa();
                    emp.setNombre(Nombre_Empresa);
                    emp.setCodEmp(emp.obtenerCodEmp());
                    Departamento dep = new Departamento();
                    dep.setCodDep(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));
                    Vigilante vig = new Vigilante(JTFDNIDP.getText(), Integer.parseInt(JTFCodV.getText()));
                    emp.removeVigilanteBD(vig);
                }
                catch (Exception e2) {}
            }
        }
    }//GEN-LAST:event_JBGrabarPerContacActionPerformed

    private void JBAnadirTfnoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirTfnoActionPerformed
        // Obtenemos el modelo actual
        DefaultTableModel modelo = (DefaultTableModel) JTTfns.getModel();

        // Si el telefono introducido es valido
        if ((JTFTfnoEmailDP.getText().length()<13) && (!JTFTfnoEmailDP.getText().equals("")))
        {
            // Introducimos el telefono
            Vector v = new Vector();
            v.add(JTFTfnoEmailDP.getText());
            modelo.addRow(v);
            JTTfns.setModel(modelo);

            // Borramos el telefono de JTFTfnoEmailDP
            JTFTfnoEmailDP.setText("");
        }
        // Si el telefono introducido no es valido
        else
        {
            // Avisamos del error
            JOptionPane.showMessageDialog(this,"El número de teléfono no debe contener más de 12 caracteres\ny no puede ser vacío.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBAnadirTfnoActionPerformed

    private void JBAnadirEmailActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirEmailActionPerformed
        // Obtenemos el modelo actual
        DefaultTableModel modelo = (DefaultTableModel) JTEmails.getModel();

        // Si el E-Mail introducido es valido
        if ((JTFTfnoEmailDP.getText().length()<51) && (JTFTfnoEmailDP.getText().contains("@")))
        {
            // Introducimos el E-Mail
            Vector v = new Vector();
            v.add(JTFTfnoEmailDP.getText());
            modelo.addRow(v);
            JTEmails.setModel(modelo);

            // Borramos el E-Mail de JTFTfnoEmailDP
            JTFTfnoEmailDP.setText("");
        }
        // Si el E-Mail introducido no es valido
        else
        {
            // Avisamos del error
            JOptionPane.showMessageDialog(this,"La dirección de correo electrónico no debe contener más\nde 50 caracteres y debe contener el caracter '@'.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBAnadirEmailActionPerformed

    private void JBEliminarTfnoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarTfnoActionPerformed
        // Obtenemos los telefonos seleccionados
        int[] tfnsSelecs = JTTfns.getSelectedRows();

        // Eliminamos cada telefono
        DefaultTableModel modelo = (DefaultTableModel) JTTfns.getModel();
        for (int i=tfnsSelecs.length-1; i>=0; i--)      modelo.removeRow(tfnsSelecs[i]);

        // Refrescamos la tabla
        JTTfns.setModel(modelo);
    }//GEN-LAST:event_JBEliminarTfnoActionPerformed

    private void JEliminarEmailActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JEliminarEmailActionPerformed
        // Obtenemos los E-Mails seleccionados
        int[] emailsSelecs = JTEmails.getSelectedRows();

        // Eliminamos cada E-Mail
        DefaultTableModel modelo = (DefaultTableModel) JTEmails.getModel();
        for (int i=emailsSelecs.length-1; i>=0; i--)      modelo.removeRow(emailsSelecs[i]);

        // Refrescamos la tabla
        JTEmails.setModel(modelo);
    }//GEN-LAST:event_JEliminarEmailActionPerformed

    private void JTEmpleadosMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTEmpleadosMouseEntered
        // TODO add your handling code here:
    }//GEN-LAST:event_JTEmpleadosMouseEntered

    private void JTBAnadirPerContacActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirPerContacActionPerformed
        String empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);

        // Si el boton se encuentra pulsado -> Mostramos el formulario de insercion de datos que corresponda
        if (JTBAnadirPerContac.isSelected())
        {
            // Escondemos los paneles de insercion de empresas y departamentos
            JPEmpresasD.setVisible(false);
            JPDepartamentosD.setVisible(false);

            // Refrescamos todas las tablas salvo la de empresas y departamentos
            refrescarTablasEmpresasPersonal("GC");
            String [] Etiquetas1 = {"Teléfonos"};
            JTTfns.setModel(new DefaultTableModel(Etiquetas1,0));
            Etiquetas1[0] = "E-Mails";
            JTEmails.setModel(new DefaultTableModel(Etiquetas1,0));
            Etiquetas1[0] = "Cargos";
            JTCargos.setModel(new DefaultTableModel(Etiquetas1,0));
            String[] Etiquetas2 = {"Código","Zona","Descripción"};
            JTLsLlsPers1.setModel(new DefaultTableModel(Etiquetas2,0));

            borrarFormDP();
            JPElimTE.setVisible(false);
            JPDNIAuto.setVisible(false);
            JTFDNIDP.setEditable(true);
            JBAnadirPrestDep1.setEnabled(false);
            if (empresa.equals("Ocasional"))   refrescarCBOcasional();
            else                               refrescarCBContactos();
            mostrarForm();
            JPElimC.setVisible(false);
        }
        // Si no se encuentra pulsado
        else
        {
            // Escondemos los paneles innecesarios y refrescamos las tablas de telefonos, E-Mails y cargps
            JPElimTE.setVisible(false);
            JPElimC.setVisible(false);
            JPDNIAuto.setVisible(false);
            mostrarJPanelsPersonal("");
            refrescarTablasEmpresasPersonal("TMX");
        }
        if ((empresa.equals("Informática Navarra S.A.") || empresa.equals("Ocasional")) && (JTContactos.isShowing()))
            mostrarJPanelsPersonal("");
    }//GEN-LAST:event_JTBAnadirPerContacActionPerformed

    private void JCBOcasionalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBOcasionalActionPerformed
    }//GEN-LAST:event_JCBOcasionalActionPerformed

    private void JCBOcasionalItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBOcasionalItemStateChanged
        // Rellenamos los datos personales de la persona escogida o los borramos
        String DNI = (String) JCBOcasional.getModel().getSelectedItem();
        if ((JTBModificarPerContac.isSelected()))      borrarFormDP();
        if ((JTBAnadirPerContac.isSelected()))         mostrarDP(DNI);
    }//GEN-LAST:event_JCBOcasionalItemStateChanged

    private void JBModificarEmpresaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBModificarEmpresaActionPerformed
        try
        {
            // Creamos la empresa nueva
            Empresa emp = new Empresa(JTFCodEmp.getText(),JTFCIFEmp.getText(),JTFNomEmp.getText(),JTADirEmp.getText(), new HashMap());

            // Comprobamos si se trata de una insercion o una modificacion de empresa
            // -> Insercion
            if (JTBAnadirEmpresa.isSelected())
            {
                // Avisamos de que vamos a dar de alta la empresa
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta la empresa.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta la empresa
                if (seleccion == 0)
                {
                    // Insertamos la empresa
                    emp.insertEmpresaBD();

                    // Refrescamos todo el TabbedPane de empresas y personal
                    iniciarEmpresasPersonal();

                    // Restablecemos todos los botones a su estado original
                    refrescarBotonesEmpPers();

                    // Escondemos el JPanel
                    JPEmpresasD.setVisible(false);

                    // Advertimos de que la empresa ha sido insertada correctamente
                    JOptionPane.showMessageDialog(this,"La empresa ha sido insertada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // ->Modificacion
            else
            {
                // Avisamos de que vamos a modificar la empresa
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar la empresa seleccionada.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos la empresa seleccionada
                if (seleccion == 0)
                {
                    // Creamos la empresa vieja
                    Empresa emp_old = new Empresa();
                    emp_old.restablecerDatos((String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0));

                    // Modificamos la empresa vieja con la nueva
                    emp_old.updateDatosBD(emp);

                    // Refrescamos todo el TabbedPane de empresas y personal
                    iniciarEmpresasPersonal();

                    // Restablecemos todos los botones a su estado original
                    refrescarBotonesEmpPers();

                    // Escondemos el JPanel
                    JPEmpresasD.setVisible(false);

                    // Advertimos de que la empresa ha sido modificada correctamente
                    JOptionPane.showMessageDialog(this,"La empresa ha sido modificada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar la empresa.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBModificarEmpresaActionPerformed

    private void JTBAnadirEmpresaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirEmpresaActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirEmpresa.isSelected())
        {
            try
            {               
                // Borramos todos los campos y escondemos los paneles de añadir departamentos y personal
                borrarDatosEmp();
                JPDepartamentosD.setVisible(false);
                mostrarJPanelsPersonal("");
                
                // Hacemos visible el panel
                JPEmpresasD.setVisible(true);
                
                // Desactivamos los botones de 'Eliminar empresa', 'Modificar empresa' y 'Añadir departamento'
                JBEliminarEmpresa.setEnabled(false);
                JTBModificarEmpresa.setSelected(false);
                JTBModificarEmpresa.setEnabled(false);
                JTBAnadirDepartamento.setSelected(false);
                JTBAnadirDepartamento.setEnabled(false);

                // Deseleccionamos cualquier fila
                JTEmpresas.clearSelection();
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido mostrar el formulario de inserción de empresas.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // En caso contrario -> Escondemos el JPanel de insercion de empresa
        else        JPEmpresasD.setVisible(false);
    }//GEN-LAST:event_JTBAnadirEmpresaActionPerformed

    private void JBDevolverActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBDevolverActionPerformed
        try
        {
            // Avisamos de que vamos a devolver los elementos seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a devolver todos los elementos seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Devolvemos todos los elementos seleccionados en todas las tarjetas
            if (seleccion == 0)
            {
                // Recogemos las tarjetas, llaves y llaveros seleccionados
                int[] tarjsSelecs = JTTarjsPrest.getSelectedRows();
                int[] lsSelecs = JTLsPrest.getSelectedRows();
                int[] llsSelecs = JTLlsPrest.getSelectedRows();

                // Devolvemos todas las TARJETAS
                String entrada;
                Acceso acc;
                for (int i=0; i<tarjsSelecs.length; i++)
                {
                    // Obtenemos el acceso en el que se produjo el prestamo de la tarjeta
                    entrada = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTTarjsPrest.getModel().getValueAt(tarjsSelecs[i],1));
                    acc = new Acceso(entrada);
                    // Devolvemos la tarjeta
                    acc.getTarjeta().devTarjetaBD();
                }

                // Devolvemos todas las LLAVES
                int codigo;
                for (int i=0; i<lsSelecs.length; i++)
                {
                    // Obtenemos el acceso en el que se produjo el prestamo de la llave
                    codigo = (Integer) JTLsPrest.getModel().getValueAt(lsSelecs[i],0);
                    entrada = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTLsPrest.getModel().getValueAt(lsSelecs[i],1));
                    acc = new Acceso(entrada);
                    // Creamos la key del GLL_Dev
                    String key = "L - " + codigo;

                    // Devolvemos el GLL_Dev
                    acc.devGLLDBD(acc.getGLLDBD(key));
                }

                // Devolvemos todos los LLAVEROS
                for (int i=0; i<llsSelecs.length; i++)
                {
                    // Obtenemos el acceso en el que se produjo el prestamo del llavero
                    codigo = (Integer) JTLlsPrest.getModel().getValueAt(llsSelecs[i],0);
                    entrada = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTLlsPrest.getModel().getValueAt(llsSelecs[i],1));
                    acc = new Acceso(entrada);
                    // Creamos la key del GLL_Dev
                    String key = "Ll - " + codigo;

                    // Devolvemos el GLL_Dev
                    acc.devGLLDBD(acc.getGLLDBD(key));
                }

                // Refrescamos todas las tablas de prestamos
                iniciarPrestamos();

                // Advertimos de que los elementos seleccionados han sido devueltos correctamente
                JOptionPane.showMessageDialog(this,"Los elementos seleccionados han sido devueltos correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible realizar las devoluciones solicitadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
        
    }//GEN-LAST:event_JBDevolverActionPerformed

    private void JTNivelesKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTNivelesKeyReleased
    }//GEN-LAST:event_JTNivelesKeyReleased

    private void JTNivelesCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTNivelesCaretPositionChanged
    }//GEN-LAST:event_JTNivelesCaretPositionChanged

    private void JTNivelesInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTNivelesInputMethodTextChanged
    }//GEN-LAST:event_JTNivelesInputMethodTextChanged

    private void JTColoresCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTColoresCaretPositionChanged
    }//GEN-LAST:event_JTColoresCaretPositionChanged

    private void JTColoresInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTColoresInputMethodTextChanged
    }//GEN-LAST:event_JTColoresInputMethodTextChanged

    private void JTColoresKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTColoresKeyReleased
    }//GEN-LAST:event_JTColoresKeyReleased

    private void JTTarjetasCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTTarjetasCaretPositionChanged
    }//GEN-LAST:event_JTTarjetasCaretPositionChanged

    private void JTTarjetasInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTTarjetasInputMethodTextChanged
    }//GEN-LAST:event_JTTarjetasInputMethodTextChanged

    private void JTTarjetasKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTTarjetasKeyReleased
    }//GEN-LAST:event_JTTarjetasKeyReleased

    private void JTColoresMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTColoresMouseClicked
        try
        {
            // Obtenemos el color
            ArrayList colores = new API.Tablas.Color().obtenerColores();

            // Siempre que 'JTBAnadirColor' no este seleccionado -> Coloreamos y activamos botones
            if (!JTBAnadirColor.isSelected())
            {
                // Pintamos el color a actualizar
                JTFColorNiv.setBackground(new Color((Integer) colores.get(JTColores.getSelectedRow())));
                // Actualizamos el nivel del color a modificar
                JTFColorNiv.setText(new Integer((Integer) JTColores.getModel().getValueAt(JTColores.getSelectedRow(),0)).toString());

                // Activamos los botones de 'Eliminar color', 'Dar de baja color' y 'Modificar color'
                JBEliminarColor.setEnabled(true);
                JTBModificarColor.setEnabled(true);
                JBBajaColor.setEnabled(true);
            }
            // Pintamos la tarjeta a insertar
            JPTarjetaD.setBackground(new Color((Integer) colores.get(JTColores.getSelectedRow())));
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible colorear la tarjeta o el color a modificar.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTColoresMouseClicked

    private void JBEliminarTarjActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarTarjActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las tarjetas seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las tarjetas seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las tarjetas
            if (seleccion == 0)
            {
                // Obtenemos las tarjetas seleccionadas
                int[] tarjsSelec = JTTarjetas.getSelectedRows();

                // Eliminamos cada tarjeta
                Tarjeta tarj;
                for (int i=0; i<tarjsSelec.length; i++)
                {
                    // Creamos la tarjeta
                    tarj = new Tarjeta((Integer) JTTarjetas.getModel().getValueAt(tarjsSelec[i],0));
                    // Borramos la tarjeta
                    tarj.removeTarjetaBD();
                }

                // Refrescamos las tablas de tarjetas, colores y niveles, asi como la de tarjetas en prestamo
                iniciarTACs();
                refrescarTarjsPrest();
                
                // Advertimos de que las tarjetas han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las tarjetas seleccionadas han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las tarjetas seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarTarjActionPerformed

    private void JTBAnadirTarjetaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirTarjetaActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirTarjeta.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Seleccionamos el de 'Añadir tarjeta'
                JTBAnadirTarjeta.setSelected(true);
                // Escondemos los paneles de colores, niveles, tipos de acceso y cargos
                JPColorD.setVisible(false);
                JPNivelD.setVisible(false);
                JPTAccesoD.setVisible(false);
                JPCargoD.setVisible(false);

                // Coloreamos la tarjeta con el color de menor numeracion y menor nivel
                JPTarjetaD.setBackground(new Color(new API.Tablas.Color().colorPorDefecto()));
                // Indicamos la numeracion por defecto que deberia llevar la nueva tarjeta
                JTFCodTarjD.setText(new Integer(new Tarjeta().numTarjPorDefecto()).toString());
                
                // Mostramos el panel
                JPTarjetasD.setVisible(true);
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible establecer el color y/o número de tarjeta por defecto.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // En caso contrario -> Escondemos el JPanel de insercion de tarjeta
        else        JPTarjetasD.setVisible(false);
    }//GEN-LAST:event_JTBAnadirTarjetaActionPerformed

    private void JBGrabarTarjetaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarTarjetaActionPerformed
        try
        {
            // Creamos la tarjeta nueva
            Tarjeta tarj = new Tarjeta(Integer.parseInt(JTFCodTarjD.getText()),Calendar.getInstance(),new API.Tablas.Color(JPTarjetaD.getBackground().getRGB()));

            // Comprobamos si se trata de una insercion o una modificacion de tarjeta
            // -> Insercion
            if (JTBAnadirTarjeta.isSelected())
            {
                // Avisamos de que vamos a dar de alta la tarjeta
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta la tarjeta.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta la tarjeta
                if (seleccion == 0)
                {
                    // Ejecutamos el disparador
                    tarj.disparador();
                    
                    // Insertamos la tarjeta
                    tarj.insertTarjetaBD();

                    // Refrescamos la tablas
                    iniciarTACs();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que la tarjeta ha sido insertada correctamente
                    JOptionPane.showMessageDialog(this,"La tarjeta ha sido insertada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // ->Modificacion
            else
            {
                // Avisamos de que vamos a modificar la tarjeta
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar la tarjeta seleccionada.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos la tarjeta seleccionada
                if (seleccion == 0)
                {
                    // Ejecutamos el disparador
                    tarj.disparador();
                    
                    // Creamos la tarjeta vieja
                    Tarjeta tarj_old = new Tarjeta((Integer) JTTarjetas.getModel().getValueAt(JTTarjetas.getSelectedRow(),0));

                    // Modificamos la tarjeta vieja con la nueva
                    tarj_old.updateTarjetaBD(tarj);

                    // Refrescamos las tablas de tarjetas, colores y niveles, asi como la de tarjetas en prestamo
                    iniciarTACs();
                    refrescarTarjsPrest();

                    // Advertimos de que la tarjeta ha sido modificada correctamente
                    JOptionPane.showMessageDialog(this,"La tarjeta ha sido modificada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar la tarjeta.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarTarjetaActionPerformed

    private void JBBajaTarjActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBBajaTarjActionPerformed
        try
        {
            // Avisamos de que vamos a dar de baja las tarjetas seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de baja las tarjetas seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Damos de baja las tarjetas
            if (seleccion == 0)
            {
                // Obtenemos las tarjetas seleccionadas
                int[] tarjsSelec = JTTarjetas.getSelectedRows();

                // Damos de baja cada tarjeta
                Tarjeta tarj;
                for (int i=0; i<tarjsSelec.length; i++)
                {
                    // Creamos la tarjeta
                    tarj = new Tarjeta((Integer) JTTarjetas.getModel().getValueAt(tarjsSelec[i],0));
                    // Damos de baja la tarjeta
                    tarj.darBajaTarjetaBD();
                }
                
                // Refrescamos las tablas de tarjetas, colores y niveles, asi como la de tarjetas en prestamo
                iniciarTACs();
                refrescarTarjsPrest();

                // Advertimos de que las tarjetas han sido dadas de baja correctamente
                JOptionPane.showMessageDialog(this,"Las tarjetas seleccionadas han sido dadas de baja correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible dar de baja las tarjetas seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBBajaTarjActionPerformed

    private void JTBModificarTarjActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarTarjActionPerformed
        // Si el boton esta pulsado
        if (JTBModificarTarj.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Activamos los botones de 'Eliminar tarjeta', 'Modificar tarjeta' y 'Dar de baja tarjeta'
                JBEliminarTarj.setEnabled(true);
                JTBModificarTarj.setSelected(true);
                JTBModificarTarj.setEnabled(true);
                JBBajaTarj.setEnabled(true);
                // Establecemos el boton de 'Añadir tarjeta' a no seleccionado
                JTBAnadirTarjeta.setSelected(false);
                // Escondemos los paneles de colores, niveles, tipos de acceso y cargos
                JPColorD.setVisible(false);
                JPNivelD.setVisible(false);
                JPTAccesoD.setVisible(false);
                JPCargoD.setVisible(false);

                // Mostramos el panel de datos
                JPTarjetasD.setVisible(true);
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar el color y número de tarjeta.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // Escondemos el JPanel de insercion de tarjeta en caso contrario
        else        JPTarjetasD.setVisible(false);
    }//GEN-LAST:event_JTBModificarTarjActionPerformed

    private void JTTarjetasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTTarjetasMouseClicked
        try
        {
            // Siempre que 'JTBAnadirTarj' no este seleccionado -> Coloreamos y activamos botones
            if (!JTBAnadirTarjeta.isSelected())
            {
                // Coloreamos y dotamos de numero a la tarjeta
                JPTarjetaD.setBackground(new Color ((Integer) new Tarjeta().obtenerColoresTarjs().get(JTTarjetas.getSelectedRow())));
                JTFCodTarjD.setText(new Integer((Integer) JTTarjetas.getModel().getValueAt(JTTarjetas.getSelectedRow(),0)).toString());

                // Activamos los botones de 'Eliminar tarjeta', 'Dar de baja tarjeta' y 'Modificar tarjeta'
                JBEliminarTarj.setEnabled(true);
                JBBajaTarj.setEnabled(true);
                JTBModificarTarj.setEnabled(true);
            }
        }
        catch(Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la tarjeta.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTTarjetasMouseClicked

    private void JTFColorNivActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTFColorNivActionPerformed
    }//GEN-LAST:event_JTFColorNivActionPerformed

    private void JBColorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBColorActionPerformed
        // Desplegamos el JColorChooser
        JColorChooser jcc = new JColorChooser();
        Color color = jcc.showDialog(this,"Colores",Color.white);

        // Establecemos el color del JTextField al seleccionado
        JTFColorNiv.setBackground(color);
    }//GEN-LAST:event_JBColorActionPerformed

    private void JTBAnadirColorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirColorActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirColor.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones y paneles
                activDesactivBotonesTACs();
                // Establecemos el boton de 'Añadir color' a seleccionado
                JTBAnadirColor.setSelected(true);
                // Escondemos los paneles de tarjetas, niveles, tipos de acceso y cargos
                JPTarjetasD.setVisible(false);
                JPNivelD.setVisible(false);
                JPTAccesoD.setVisible(false);
                JPCargoD.setVisible(false);

                // Establecemos el color de menor codigo que se corresponde con el menor nivel
                JTFColorNiv.setBackground(new Color(new API.Tablas.Color().colorPorDefecto()));
                // Establecemos el nivel del color al menor posible
                JTFColorNiv.setText(new Integer(new Nivel().nivelMinimo()).toString());

                // Mostramos el panel
                JPColorD.setVisible(true);
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible establecer el color y/o nivel mínimos.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // En caso contrario -> Escondemos el JPanel de insercion de color
        else        JPColorD.setVisible(false);
    }//GEN-LAST:event_JTBAnadirColorActionPerformed

    private void JBEliminarColorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarColorActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los colores seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los colores seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los colores
            if (seleccion == 0)
            {
                // Obtenemos los colores
                ArrayList colores = new API.Tablas.Color().obtenerColores();

                // Obtenemos los colores seleccionados
                int[] colsSelec = JTColores.getSelectedRows();

                // Eliminamos cada color
                API.Tablas.Color col = new  API.Tablas.Color();
                for (int i=0; i<colsSelec.length; i++)
                {
                    // Creamos el color
                    col = new API.Tablas.Color((Integer) colores.get(colsSelec[i]));
                    // Borramos el color
                    col.removeColorBD();
                }

                // Ejecutamos el disparador
                col.disparadorRemove();

                // Refrescamos las tablas de tarjetas, niveles y colores, asi como la de tarjetas en prestamo
                iniciarTACs();
                refrescarTarjsPrest();

                // Advertimos de que los colores han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los colores seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los colores seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarColorActionPerformed

    private void JTBModificarColorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarColorActionPerformed
        // Si el boton esta pulsado
        if (JTBModificarColor.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Activamos los botones de 'Eliminar color', 'Modificar color' y 'Dar de baja color'
                JBEliminarColor.setEnabled(true);
                JTBModificarColor.setSelected(true);
                JTBModificarColor.setEnabled(true);
                JBBajaColor.setEnabled(true);
                // Establecemos el boton de 'Añadir color' a no seleccionado
                JTBAnadirColor.setSelected(false);
                // Escondemos los paneles de tarjetas, niveles, tipos de acceso y cargos
                JPTarjetasD.setVisible(false);
                JPNivelD.setVisible(false);
                JPTAccesoD.setVisible(false);
                JPCargoD.setVisible(false);

                // Mostramos el panel de datos
                JPColorD.setVisible(true);
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar el color y nivel del color.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // Escondemos el JPanel de insercion de color en caso contrario
        else        JPColorD.setVisible(false);
    }//GEN-LAST:event_JTBModificarColorActionPerformed

    private void JBBajaColorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBBajaColorActionPerformed
        try
        {
            // Avisamos de que vamos a dar de baja los colores seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de baja los colores seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Damos de baja los colores
            if (seleccion == 0)
            {
                // Obtenemos los colores seleccionados
                int[] colsSelec = JTColores.getSelectedRows();

                // Damos de baja cada color
                API.Tablas.Color col = new API.Tablas.Color();
                for (int i=0; i<colsSelec.length; i++)
                {
                    // Creamos el color
                    col = new API.Tablas.Color((Integer) new API.Tablas.Color().obtenerColores().get(colsSelec[i]));
                    // Damos de baja el color
                    col.darBajaColorBD();
                }

                // Ejecutamos el disparador
                col.disparadorRemove();

                // Refrescamos las tablas de tarjetas, colores y niveles, asi como la de tarjetas en prestamo
                iniciarTACs();
                refrescarTarjsPrest();

                // Advertimos de que los colores han sido dados de baja correctamente
                JOptionPane.showMessageDialog(this,"Los colores seleccionados han sido dados de baja correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible dar de baja los colores seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBBajaColorActionPerformed

    private void JBGrabarColorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarColorActionPerformed
        try
        {
            // Creamos el color nuevo
            API.Tablas.Color col_new = new API.Tablas.Color(JTFColorNiv.getBackground().getRGB(),Calendar.getInstance(),new Nivel(Integer.parseInt(JTFColorNiv.getText())));

            // Comprobamos si se trata de una insercion o una modificacion de color
            // -> Insercion
            if (JTBAnadirColor.isSelected())
            {
                // Avisamos de que vamos a dar de alta el color
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta el color.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta el color
                if (seleccion == 0)
                {
                    // Ejecutamos el disparador
                    col_new.disparadorInsert();

                    // Insertamos el color
                    col_new.insertColorBD();

                    // Refrescamos la tablas
                    iniciarTACs();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que el color ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El color ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // ->Modificacion
            else
            {
                // Avisamos de que vamos a modificar el color
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el color seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el color seleccionado
                if (seleccion == 0)
                {
                    // Creamos el color viejo
                    API.Tablas.Color col_old = new API.Tablas.Color((Integer) new API.Tablas.Color().obtenerColores().get((Integer) JTColores.getSelectedRow()));

                    // Modificamos el color viejo con el nuevo
                    col_old.updateColorBD(col_new);

                    // Ejecutamos el disparador
                    col_old.disparadorUpdate();

                    // Refrescamos todas las tablas del tab y la de tarjetas en prestamo
                    iniciarTACs();
                    refrescarTarjsPrest();

                    // Advertimos de que el color ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El color ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el color.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarColorActionPerformed

    private void JTNivelesMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTNivelesMouseClicked
        try
        {
            // Siempre que 'JTBAnadirNivel' no este seleccionado -> Rellenamos y activamos botones
            if (!JTBAnadirNivel.isSelected())
            {
                // Rellenamos los campos de codigo y descripcion
                Nivel niv = new Nivel((Integer) JTNiveles.getModel().getValueAt(JTNiveles.getSelectedRow(),0));
                JTFCodNiv.setText(new Integer(niv.getCodNiv()).toString());
                JTFDescripNiv.setText(niv.getDescripcion());

                // Activamos los botones de 'Eliminar nivel', 'Dar de baja nivel' y 'Modificar nivel'
                JBEliminarNivel.setEnabled(true);
                JTBModificarNivel.setEnabled(true);
                JBBajaNivel.setEnabled(true);
            }
            JTFColorNiv.setText(new Integer((Integer) JTNiveles.getModel().getValueAt(JTNiveles.getSelectedRow(),0)).toString());
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible colorear la tarjeta o el color a modificar.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTNivelesMouseClicked

    private void JTBAnadirNivelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirNivelActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirNivel.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Establecemos el boton de 'Añadir nivel' a seleccionado
                JTBAnadirNivel.setSelected(true);
                // Escondemos los paneles de tarjetas, colores, tipos de acceso y cargos
                JPTarjetasD.setVisible(false);
                JPColorD.setVisible(false);
                JPTAccesoD.setVisible(false);
                JPCargoD.setVisible(false);

                // Establecemos el nivel por defecto
                JTFCodNiv.setText(new Integer(new Nivel().nivelPorDefecto()).toString());
                // Borramos el campo de la descripcion del nivel
                JTFDescripNiv.setText("");

                // Mostramos el panel
                JPNivelD.setVisible(true);
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible establecer el nivel por defecto.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // Escondemos el JPanel de insercion de nivel y deseleccionamos los niveles que pudieran estar
        // seleccionados en la tabla, dejandola "intacta"
        else
        {
            JPNivelD.setVisible(false);
            JTNiveles.clearSelection();
        }
    }//GEN-LAST:event_JTBAnadirNivelActionPerformed

    private void JBEliminarNivelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarNivelActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los niveles seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los niveles seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los niveles
            if (seleccion == 0)
            {
                // Obtenemos los niveles seleccionados
                int[] nivsSelec = JTNiveles.getSelectedRows();

                // Eliminamos cada nivel
                Nivel niv;
                for (int i=0; i<nivsSelec.length; i++)
                {
                    // Creamos el nivel
                    niv = new Nivel((Integer) JTNiveles.getModel().getValueAt(nivsSelec[i],0));
                    // Borramos el nivel
                    niv.removeNivelBD();
                }

                // Refrecamos las tablas de tarjetas, colores, niveles, tipos de acceso y cargos, asi como la de tarjetas en prestamo
                iniciarTACs();
                refrescarTarjsPrest();

                // Advertimos de que los niveles han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los niveles seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los niveles seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarNivelActionPerformed

    private void JTBModificarNivelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarNivelActionPerformed
        // Si el boton esta pulsado
        if (JTBModificarNivel.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Activamos los botones de 'Eliminar nivel', 'Modificar nivel' y 'Dar de baja nivel'
                JBEliminarNivel.setEnabled(true);
                JTBModificarNivel.setSelected(true);
                JTBModificarNivel.setEnabled(true);
                JBBajaNivel.setEnabled(true);
                // Establecemos el boton de 'Añadir nivel' a no seleccionado
                JTBAnadirNivel.setSelected(false);

                // Escondemos los paneles de tarjetas, colores, tipos de accesos y cargos
                JPTarjetasD.setVisible(false);
                JPColorD.setVisible(false);
                JPTAccesoD.setVisible(false);
                JPCargoD.setVisible(false);

                // Mostramos el panel de datos
                JPNivelD.setVisible(true);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar el código y descripción del nivel.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // Escondemos el JPanel de insercion de nivel en caso contrario
        else        JPNivelD.setVisible(false);
    }//GEN-LAST:event_JTBModificarNivelActionPerformed

    private void JBBajaNivelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBBajaNivelActionPerformed
        try
        {
            // Avisamos de que vamos a dar de baja los niveles seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de baja los niveles seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Damos de baja los niveles
            if (seleccion == 0)
            {
                // Obtenemos los niveles seleccionados
                int[] nivsSelec = JTNiveles.getSelectedRows();

                // Damos de baja cada nivel
                Nivel niv;
                for (int i=0; i<nivsSelec.length; i++)
                {
                    // Creamos el nivel
                    niv = new Nivel((Integer) JTNiveles.getModel().getValueAt(JTNiveles.getSelectedRow(),0));
                    // Damos de baja el nivel
                    niv.darBajaNivelBD();
                }

                // Refrecamos las tablas de tarjetas, colores y niveles, asi como la de tarjetas en prestamo
                iniciarTACs();
                refrescarTarjsPrest();

                // Advertimos de que los niveles han sido dados de baja correctamente
                JOptionPane.showMessageDialog(this,"Los niveles seleccionados han sido dados de baja correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible dar de baja los niveles seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBBajaNivelActionPerformed

    private void JBGrabarNivelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarNivelActionPerformed
        try
        {
            // Creamos el nivel nuevo
            Nivel niv_new = new Nivel(Integer.parseInt(JTFCodNiv.getText()),Calendar.getInstance(),JTFDescripNiv.getText());

            // Comprobamos si se trata de una insercion o una modificacion de nivel
            // -> Insercion
            if (JTBAnadirNivel.isSelected())
            {
                // Avisamos de que vamos a dar de alta el nivel
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta el nivel.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta el nivel
                if (seleccion == 0)
                {
                    // Ejecutamos el disparador
                    niv_new.disparadorInsert();

                    // Insertamos el nivel
                    niv_new.insertNivelBD();

                    // Refrescamos la tablas
                    iniciarTACs();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que el nivel ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El nivel ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // ->Modificacion
            else
            {
                // Avisamos de que vamos a modificar el nivel
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el nivel seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el nivel seleccionado
                if (seleccion == 0)
                {
                    // Creamos el nivel viejo
                    Nivel niv_old = new Nivel((Integer) JTNiveles.getModel().getValueAt(JTNiveles.getSelectedRow(),0));

                    // Modificamos el nivel viejo con el nuevo
                    niv_old.updateNivelBD(niv_new);

                    // Ejecutamos el disparador
                    niv_old.disparadorUpdate();

                    // Refrescamos la tablas
                    iniciarTACs();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que el nivel ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El nivel ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el nivel.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarNivelActionPerformed

    private void JTZonasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTZonasMouseClicked
        try
        {
            // Si el boton de añadir zona no esta seleccionado
            if (!JTBAnadirZona.isSelected())
            {
                // Mostramos los botones de llaves
                JPLlaveB.setVisible(true);

                // Vaciamos las tablas de llaves, empresas-departamentos y personal ocasional
                vaciarTablasZonas();

                // Obtenemos la zona seleccionada
                Zona z = new Zona((String) JTZonas.getModel().getValueAt(JTZonas.getSelectedRow(),0));

                // Rellenamos los datos de esta
                JTFCodZona.setText(z.getCodZona());
                JTFDescripZona.setText(z.getDescripcion());

                // Recorremos el JCB para obtener el indice del elemento que corresponda por el nivel de la zona
                DefaultComboBoxModel dcm = (DefaultComboBoxModel) JCBNivelZona.getModel();
                int index = 1;
                boolean encontrado = false;
                if (z.getNivel() != null)
                {
                    while ((index<dcm.getSize()) && (!encontrado))
                    {
                        String elem = (String) dcm.getElementAt(index);
                        if (elem.startsWith(new Integer(z.getNivel().getCodNiv()).toString()))  encontrado = true;
                        index++;
                    }
                }

                // Escogemos el item del JCB que corresponde
                JCBNivelZona.setSelectedIndex(index-1);

                // Mostramos las llaves correspondientes a la zona
                UGUI.rellenarTabla(JTLsZona, z.obtenerLlaves());
            }

            // Activamos los botones de 'Eliminar zona' y 'Modificar zona'
            JTBAnadirL.setSelected(false);
            JTBAnadirL.setEnabled(true);
            JBEliminarZona.setEnabled(true);
            JTBModificarZona.setEnabled(true);

            // Desactivamos los botones de 'Añadir llaves a llavero', 'Eliminar llave', 'Modificar llave' y 'Dar baja llave'
            JBAnadirLsLl.setEnabled(false);
            JBEliminarL.setEnabled(false);
            JTBModificarLlave.setSelected(false);
            JTBModificarLlave.setEnabled(false);
            JBDarBajaL.setEnabled(false);
            // Desactivamos los botones y campos de añadir llave y llavero al almacen
            JPJTFNumCopsL.setVisible(false);
            JTBAnadirAlmL.setEnabled(false);
            JPJTFNumCopsLl.setVisible(false);
            JTBAnadirAlmLlB.setSelected(false);
            JBAnadirAlmLl.setEnabled(false);
            JPLlaveD.setVisible(false);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la zona.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTZonasMouseClicked

    private void JCCFechaInicMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JCCFechaInicMouseClicked
        // TODO add your handling code here:
    }//GEN-LAST:event_JCCFechaInicMouseClicked

    private void JBGrabarZonaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarZonaActionPerformed
        try
        {
            // Creamos la zona nueva
            String nivel = (String) JCBNivelZona.getModel().getSelectedItem();
            Nivel niv = null;
            if (!nivel.equals("Ninguno"))   niv = new Nivel(Integer.parseInt(nivel));
            Zona zona_new = new Zona(JTFCodZona.getText(),Calendar.getInstance(),JTFDescripZona.getText(),niv,new HashMap());

            // Comprobamos si se trata de una insercion o de una actualizacion
            // -> Insercion
            if (JTBAnadirZona.isSelected())
            {
                    // Avisamos de que vamos a insertar la zona
                    int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a insertar la zona.\n" +
                            "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                    // Si se acepta la operacion -> Insertamos la nueva zona
                    if (seleccion == 0)
                    {
                        // Insertamos la nueva zona
                        zona_new.insertZonaBD();

                        // Actualizamos todas las tablas de las zonas
                        refrescarZonas();

                        // Des-seleccionamos el boton de 'Añadir zona' y borramos el panel de datos
                        JTBAnadirZona.setSelected(false);
                        JPZonaD.setVisible(false);
                        // Des-seleccionamos y descativamos el boton de modificar y eliminar zona
                        JTBModificarZona.setSelected(false);
                        JTBModificarZona.setEnabled(false);
                        JBEliminarZona.setEnabled(false);

                        // Advertimos de que la zona ha sido insertada correctamente
                        JOptionPane.showMessageDialog(this,"La zona ha sido insertada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                    }
            }
            // -> Actualización
            else
            {
                    // Avisamos de que vamos a modificar la zona
                    int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar la zona seleccionado.\n" +
                            "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                    // Si se acepta la operacion -> Modificamos la zona seleccionada
                    if (seleccion == 0)
                    {
                        // Obtenemos la zona vieja
                        Zona zona_old = new Zona((String) JTZonas.getModel().getValueAt(JTZonas.getSelectedRow(),0));

                        // Actualizamos la zona vieja por la nueva
                        zona_old.updateZonaBD(zona_new);

                        // Actualizamos todas las tablas de las zonas
                        refrescarZonas();
                        
                        // Des-seleccionamos el boton de 'Añadir zona' y borramos el panel de datos
                        JTBAnadirZona.setSelected(false);
                        JPZonaD.setVisible(false);
                        // Des-seleccionamos y descativamos el boton de modificar y eliminar zona
                        JTBModificarZona.setSelected(false);
                        JTBModificarZona.setEnabled(false);
                        JBEliminarZona.setEnabled(false);

                        // Advertimos de que la zona ha sido modificada correctamente
                        JOptionPane.showMessageDialog(this,"La zona ha sido modificada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                    }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar la zona.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarZonaActionPerformed

    private void JBEliminarZonaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarZonaActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las zonas seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las zonas seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las zonas
            if (seleccion == 0)
            {
                // Obtenemos las zonas seleccionadas
                int[] zonasSelecs = JTZonas.getSelectedRows();

                // Damos de baja cada zona
                Zona z;
                for (int i=0; i<zonasSelecs.length; i++)
                {
                    // Obtenemos la zona
                    z = new Zona((String) JTZonas.getModel().getValueAt(zonasSelecs[i],0));
                    // Damos de baja la zona
                    z.removeZonaBD();
                }

                // Refrescamos las tablas de zonas y deshabilitamos los botones que correspondan
                refrescarZonas();
                deshabilitarBotonesZonas();

                // Refrescamos los almacenes
                refrescarAlmacenes();

                // Desactivamos los botones de 'Eliminar zona' y 'Modificar zona'
                JBEliminarZona.setEnabled(false);
                JTBModificarZona.setEnabled (false);

                // Advertimos de que las zonas han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las zonas seleccionadas han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las zonas seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarZonaActionPerformed

    private void JTBAnadirZonaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirZonaActionPerformed
        // Si hemos seleccionado el boton
        if (JTBAnadirZona.isSelected())
        {
            // Des-seleccionamos y desactivamos el boton de modificar y eliminar zona
            JTBModificarZona.setSelected(false);
            JTBModificarZona.setEnabled(false);
            JBEliminarZona.setEnabled(false);

            // Des-seleccionamos cualquier zona que pudiera estar clicada
            JTZonas.clearSelection();

            // Vaciamos las tablas de zonas
            vaciarTablasZonas();

            // Borramos los campos para introducir una nueva zona
            JTFCodZona.setText("");
            JTFDescripZona.setText("");
            JCBNivelZona.setSelectedIndex(0);
            // Mostramos el panel de insercion de datos
            JPZonaD.setVisible(true);
        }
        // Si lo hemos des-seleccionado
        else
        {
            // Escondemos el panel de datos y los botones de añadir/eliminar prestables y todos lo referente llaves
            refrescarZonas();
            deshabilitarBotonesZonas();
            JBEliminarZona.setEnabled(false);
            JTBModificarZona.setSelected(false);
            JTBModificarZona.setEnabled(false);
            
            // Seleccionamos el primer almacen
            JTAlms.setRowSelectionInterval(0,0);
        }
    }//GEN-LAST:event_JTBAnadirZonaActionPerformed

    private void JTBModificarZonaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarZonaActionPerformed
        // Si esta el boton seleccionado
        if (JTBModificarZona.isSelected())
        {
            // Des-seleccionamos el boton de añadir zona
            JTBAnadirZona.setSelected(false);

            // Mostramos el panel de datos
            JPZonaD.setVisible(true);
        }
        // Si no lo esta -> Escondemos el panel de datos y los botones de añadir/eliminar prestables
        else    deshabilitarBotonesZonas();
    }//GEN-LAST:event_JTBModificarZonaActionPerformed

    private void JTLsZonaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsZonaMouseClicked
        try
        {
            // Activo los botones de modificar, eliminar llave y dar de baja llave
            JTBModificarLlave.setEnabled(true);
            JBEliminarL.setEnabled(true);
            JBDarBajaL.setEnabled(true);

            // Obtenemos la llave clicada
            Llave l = new Llave ((Integer) JTLsZona.getModel().getValueAt(JTLsZona.getSelectedRow(),0));

            // Establecemos el valor de todos los campos y mostramos el panel
            JTFCodL.setText(new Integer(l.getCodigo()).toString());
            JTFDescripL.setText(l.getDescripcion());
            JTFNotasL.setText(l.getNotas() == null ? "" : l.getNotas());

            // Mostramos las tablas de prestatarios actuales...
            activarTablasPrestablesActuales();
            // ... y los posibles si es que el boton de modificacion de llave esta pulsado
            if (JTBModificarLlave.isSelected())     activarTablasPrestablesPosibles();

            // Si tenemos seleccionado un almacen distinto de 'Todos' y alguna llave
            // en JTLsZona -> Activamos el boton para poder añadirla al almacen
            String AlmSelec = (String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0);
            if ((!AlmSelec.equals("Todos")) && (JTLsZona.getSelectedRow() != -1))
            {
                JTBAnadirAlmL.setSelected(false);
                JTBAnadirAlmL.setEnabled(true);
            }
            else
            {
                JTBAnadirAlmL.setSelected(false);
                JTBAnadirAlmL.setEnabled(false);
            }

            // Hacemos visibles los botones de añadir/eliminar prestables si los botones de insercion o modificacion de zonas estan activados
            if (JTBAnadirZona.isSelected() || JTBModificarZona.isSelected())
            {
                JBAnadirLPrestEmpDep.setEnabled(false);
                JBAnadirLPrestOcas.setEnabled(false);
                JBEliminarLPrestEmpDep.setEnabled(false);
                JBEliminarLPrestOcas.setEnabled(false);
                activarTablasPrestablesPosibles();
            }

            // Si tenemos seleccionado el almacen 'Todos' y un llavero
            // seleccionado -> Activar o desactivar botones de modificacion de las llaves del llavero
            if ((AlmSelec.equals("Todos")) && (JTLls.getSelectedRow() != -1))
            {
                // Si tenemos escogida alguna llave de el apartado de zonas -> Habilitar boton de añadir llaves al llavero
                JBAnadirLsLl.setEnabled(true);
                JBEliminarLsLl.setEnabled(false);
                    
                // Si tenemos escogida alguna llave de el apartado de zonas y alguna otra de las que componen el
                // llavero -> Habilitar boton de añadir llaves al llavero
                if (JTLs.getSelectedRow() != -1)
                {
                    JBAnadirLsLl.setEnabled(true);
                    JBEliminarLsLl.setEnabled(true);
                }
            }
            else
            {
                JBAnadirLsLl.setEnabled(false);
                JBEliminarLsLl.setEnabled(false);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las prestabilidades de la llave seleccionada.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTLsZonaMouseClicked

    private void JTLsPrestEmpDepMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsPrestEmpDepMouseClicked
        // Activamos el boton de eliminar prestable
        JBEliminarLPrestEmpDep.setEnabled(true);
    }//GEN-LAST:event_JTLsPrestEmpDepMouseClicked

    private void JTLsPrestOcasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsPrestOcasMouseClicked
        // Activamos el boton de eliminar prestable
        JBEliminarLPrestOcas.setEnabled(true);
    }//GEN-LAST:event_JTLsPrestOcasMouseClicked

    private void JBEliminarLPrestEmpDepActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarLPrestEmpDepActionPerformed
        // Creamos los nuevos TableModels a raiz de los existentes y la seleccion de filas
        int[] filasSelecs = JTLsPrestEmpDep.getSelectedRows();

        // Obtenemos los modelos viejos
        DefaultTableModel mod_old = (DefaultTableModel) JTLsPrestEmpDep.getModel();
        DefaultTableModel mod_r_old = (DefaultTableModel) JTLsPrestEmpDepResto.getModel();

        Vector datos = mod_old.getDataVector();
        Vector datos_r = mod_r_old.getDataVector();

        // Recorremos todas las filas seleccionadas
        for (int i=0; i<filasSelecs.length; i++)
        {
            // Añadimos la fila al resto
            datos_r.add(datos.get(filasSelecs[i]-i));
            // Eliminamos la fila
            datos.removeElementAt(filasSelecs[i]-i);
        }

        // Creamos los nuevos table models
        Vector etiquetas = new Vector ();
        etiquetas.add("Empresa");
        etiquetas.add("Departamento");

        DefaultTableModel mod_new = new DefaultTableModel(datos,etiquetas);
        DefaultTableModel mod_r_new = new DefaultTableModel(datos_r,etiquetas);

        // Asignamos los nuevos modelos a las tablas
        JTLsPrestEmpDep.setModel(mod_new);
        JTLsPrestEmpDepResto.setModel(mod_r_new);

        // Desactivamos el boton de eliminar prestables
        JBEliminarLPrestEmpDep.setEnabled(false);
    }//GEN-LAST:event_JBEliminarLPrestEmpDepActionPerformed

    private void JBEliminarLPrestOcasActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarLPrestOcasActionPerformed
        // Creamos los nuevos TableModels a raiz de los existentes y la seleccion de filas
        int[] filasSelecs = JTLsPrestOcas.getSelectedRows();

        // Obtenemos los viejos table models
        DefaultTableModel mod_old = (DefaultTableModel) JTLsPrestOcas.getModel();
        DefaultTableModel mod_r_old = (DefaultTableModel) JTLsPrestOcasResto.getModel();

        Vector datos = mod_old.getDataVector();
        Vector datos_r = mod_r_old.getDataVector();

        // Recorremos todas las filas seleccionadas
        for (int i=0; i<filasSelecs.length; i++)
        {
            // Añadimos la fila al resto
            datos_r.add(datos.get(filasSelecs[i]-i));
            // Eliminamos la fila
            datos.removeElementAt(filasSelecs[i]-i);
        }

        // Creamos los nuevos table models
        Vector etiquetas = new Vector ();
        etiquetas.add("DNI");
        etiquetas.add("Empresa");
        DefaultTableModel mod_new = new DefaultTableModel(datos,etiquetas);
        DefaultTableModel mod_r_new = new DefaultTableModel(datos_r,etiquetas);

        // Asignamos los nuevos modelos a las tablas
        JTLsPrestOcas.setModel(mod_new);
        JTLsPrestOcasResto.setModel(mod_r_new);

        // Desactivamos el boton de eliminar prestables
        JBEliminarLPrestOcas.setEnabled(false);
    }//GEN-LAST:event_JBEliminarLPrestOcasActionPerformed

    private void JTLsPrestEmpDepRestoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsPrestEmpDepRestoMouseClicked
        // Activamos el boton de añadir prestable
        JBAnadirLPrestEmpDep.setEnabled(true);
    }//GEN-LAST:event_JTLsPrestEmpDepRestoMouseClicked

    private void JTLsPrestOcasRestoMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsPrestOcasRestoMouseClicked
        // Activamos el boton de añadir prestable
        JBAnadirLPrestOcas.setEnabled(true);
    }//GEN-LAST:event_JTLsPrestOcasRestoMouseClicked

    private void JBAnadirLPrestEmpDepActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirLPrestEmpDepActionPerformed
        // Creamos los nuevos TableModels a raiz de los existentes y la seleccion de filas
        int[] filasSelecs = JTLsPrestEmpDepResto.getSelectedRows();

        // Obtenemos los viejos table models
        DefaultTableModel mod_old = (DefaultTableModel) JTLsPrestEmpDep.getModel();
        DefaultTableModel mod_r_old = (DefaultTableModel) JTLsPrestEmpDepResto.getModel();

        Vector datos = mod_old.getDataVector();
        Vector datos_r = mod_r_old.getDataVector();

        // Recorremos todas las filas seleccionadas
        for (int i=0; i<filasSelecs.length; i++)
        {
            // Añadimos la fila al resto
            datos.add(datos_r.get(filasSelecs[i]-i));
            // Eliminamos la fila
            datos_r.removeElementAt(filasSelecs[i]-i);
        }

        // Creamos los nuevos table models
        Vector etiquetas = new Vector ();
        etiquetas.add("Empresa");
        etiquetas.add("Departamento");
        DefaultTableModel mod_new = new DefaultTableModel(datos,etiquetas);
        DefaultTableModel mod_r_new = new DefaultTableModel(datos_r,etiquetas);

        // Asignamos los nuevos modelos a las tablas
        JTLsPrestEmpDep.setModel(mod_new);
        JTLsPrestEmpDepResto.setModel(mod_r_new);

        // Desactivamos el boton de añadir prestables
        JBAnadirLPrestEmpDep.setEnabled(false);
    }//GEN-LAST:event_JBAnadirLPrestEmpDepActionPerformed

    private void JBAnadirLPrestOcasActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirLPrestOcasActionPerformed
        // Creamos los nuevos TableModels a raiz de los existentes y la seleccion de filas
        int[] filasSelecs = JTLsPrestOcasResto.getSelectedRows();

        // Obtenemos los viejos table models
        DefaultTableModel mod_old = (DefaultTableModel) JTLsPrestOcas.getModel();
        DefaultTableModel mod_r_old = (DefaultTableModel) JTLsPrestOcasResto.getModel();

        Vector datos = mod_old.getDataVector();
        Vector datos_r = mod_r_old.getDataVector();

        // Recorremos las filas seleccionadas
        for (int i=0; i<filasSelecs.length; i++)
        {
            // Añadimos la fila al resto
            datos.add(datos_r.get(filasSelecs[i]-i));
            // Eliminamos la fila
            datos_r.removeElementAt(filasSelecs[i]-i);
        }

        // Creamos los nuevos table models
        Vector etiquetas = new Vector ();
        etiquetas.add("Empresa");
        etiquetas.add("Departamento");
        DefaultTableModel mod_new = new DefaultTableModel(datos,etiquetas);
        DefaultTableModel mod_r_new = new DefaultTableModel(datos_r,etiquetas);

        // Asignamos los nuevos modelos a las tablas
        JTLsPrestOcas.setModel(mod_new);
        JTLsPrestOcasResto.setModel(mod_r_new);

        // Desactivamos el boton de añadir prestables
        JBAnadirLPrestOcas.setEnabled(false);
    }//GEN-LAST:event_JBAnadirLPrestOcasActionPerformed

    private void JTBAnadirLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirLActionPerformed
        try
        {
            // Si el boton esta pulsado
            if (JTBAnadirL.isSelected())
            {
                // Des-seleccionamos y desactivamos los botones de modificar, eliminar llave y dar de baja llave
                JTBModificarLlave.setSelected(false);
                JTBModificarLlave.setEnabled(false);
                JBEliminarL.setEnabled(false);
                JBDarBajaL.setEnabled(false);

                // Des-seleccionamos y desactivamos el boton de añadir llave al llavero
                JTBAnadirAlmL.setSelected(false);
                JTBAnadirAlmL.setEnabled(false);

                // Deseleccionamos cualquier llave seleccionada
                JTLsZona.clearSelection();

                // Borramos los campos y mostramos el panel
                Llave l = new Llave();
                JTFCodL.setText(new Integer(l.codigoPorDefecto()).toString());
                JTFDescripL.setText("");
                JTFNotasL.setText("");
                JPLlaveD.setVisible(true);

                // Preparamos las tablas de prestables actuales y posibles
                vaciarTablasPrestables();
                activarTablasPrestablesPosibles();
            }
            // Si no lo esta -> Escondemos el panel de datos de la llave y los de prestables
            else
            {
                JPLlaveD.setVisible(false);
                JPLsPrestB.setVisible(false);
            }
        }
        catch(Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible preparar los paneles para insertar la llave.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTBAnadirLActionPerformed

    private void JTBModificarLlaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarLlaveActionPerformed
        // Si el boton esta pulsado
        if (JTBModificarLlave.isSelected())
        {
            // Mostramos el panel
            JPLlaveD.setVisible(true);

            // Mostramos las tablas de prestables
            activarTablasPrestablesPosibles();
        }
        // Si no lo esta -> Habilitamos el boton de añadir llave y escondemos el panel de datos de la llave
        else
        {
            JTBAnadirL.setEnabled(true);
            JPLlaveD.setVisible(false);
            JPLsPrestB.setVisible(false);
        }
    }//GEN-LAST:event_JTBModificarLlaveActionPerformed

    private void JBGrabarLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarLActionPerformed
        try
        {
            // Obtenemos la zona
            Zona zona = new Zona ((String) JTZonas.getModel().getValueAt(JTZonas.getSelectedRow(),0));
            // Creamos la nueva llave
            Llave l_new = new Llave(new Integer(JTFCodL.getText()),Calendar.getInstance(),null,JTFDescripL.getText(),(JTFNotasL.getText().equals("") ? null : JTFNotasL.getText()));

            // Comprobamos si se trata de una insercion o de una modificacion
            // -> Insercion
            int seleccion;
            if (JTBAnadirL.isSelected())
            {
                // Avisamos de que vamos a añadir la llave proporcionada
                seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta la llave.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta la llave
                if (seleccion == 0)
                {
                    // Ejecutamos el disparador
                    zona.disparador(l_new);

                    // Insertamos la llave en la zona
                    zona.insertLlaveBD(l_new);

                    // Insertamos los prestables
                    insertarPrestables(l_new);

                    // Refrescamos la tabla de llaves, borramos las de prestables y escondemos las del resto de prestables
                    UGUI.rellenarTabla(JTLsZona, zona.obtenerLlaves());

                    // Refrescamos los almacenes
                    refrescarAlmacenes();

                    // Ocultamos todo lo innecesario
                    deshabilitarBotonesZonas();

                    // Advertimos de que la llave ha sido insertada correctamente
                    JOptionPane.showMessageDialog(this,"La llave ha sido insertada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Modificacion
            else
            {
                // Avisamos de que vamos a modificar la llave seleccionada
                seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar la llave seleccionada.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos la llave
                if (seleccion == 0)
                {
                    // Obtenemos la llave vieja
                    Llave l_old = new Llave ((Integer) JTLsZona.getModel().getValueAt(JTLsZona.getSelectedRow(),0));

                    // Modificamos la llave vieja con la nueva
                    zona.updateLlaveBD(l_old, l_new);

                    // Eliminamos la llave de todos los prestables
                    l_new.eliminarPrestables();

                    // Insertamos los prestables
                    insertarPrestables(l_new);

                    // Refrescamos la tabla de llaves, borramos las de prestables y escondemos las del resto de prestables
                    UGUI.rellenarTabla(JTLsZona, zona.obtenerLlaves());

                    // Refrescamos los almacenes
                    refrescarAlmacenes();

                    // Ocultamos todo lo innecesario
                    deshabilitarBotonesZonas();

                    // Advertimos de que la llave ha sido modificada correctamente
                    JOptionPane.showMessageDialog(this,"La llave ha sido modificada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar la llave indicada.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarLActionPerformed

    private void JBEliminarLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarLActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las llaves seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las llaves seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las llaves
            if (seleccion == 0)
            {
                // Obtenemos las llaves seleccionadas
                int[] llavesSelecs = JTLsZona.getSelectedRows();

                // Eliminamos cada llave
                Llave l;
                Zona z = new Zona ((String) JTZonas.getModel().getValueAt(JTZonas.getSelectedRow(),0));
                for (int i=0; i<llavesSelecs.length; i++)
                {
                    // Obtenemos la llave
                    l = new Llave((Integer) JTLsZona.getModel().getValueAt(llavesSelecs[i],0));
                    // Eliminamos la llave
                    z.removeLlaveBD(l);
                }

                // Ejecutamos el disparador
                z.disparador();

                // Refrescamos las tablas de llaves y prestables
                UGUI.rellenarTabla(JTLsZona, z.obtenerLlaves());
                vaciarTablasPrestables();

                // Refrescamos los almacenes
                refrescarAlmacenes();

                // Desactivamos los motones de 'Eliminar llave', 'Modificar llave' y 'Dar baja llave'
                JBEliminarL.setEnabled(false);
                JTBModificarLlave.setSelected(false);
                JTBModificarLlave.setEnabled(false);
                JBDarBajaL.setEnabled(false);

                // Advertimos de que las llaves han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las llaves seleccionadas han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las llaves seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarLActionPerformed

    private void JBDarBajaLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBDarBajaLActionPerformed
        try
        {
            // Avisamos de que vamos a dar de baja las llaves seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de baja las llaves seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Damos de baja las llaves
            if (seleccion == 0)
            {
                // Obtenemos las llaves seleccionadas
                int[] llavesSelecs = JTLsZona.getSelectedRows();

                // Damos de baja cada llave
                Llave l;
                Zona z = new Zona ((String) JTZonas.getModel().getValueAt(JTZonas.getSelectedRow(),0));
                for (int i=0; i<llavesSelecs.length; i++)
                {
                    // Obtenemos la llave
                    l = new Llave((Integer) JTLsZona.getModel().getValueAt(llavesSelecs[i],0));
                    // Damos de baja la llave
                    z.darBajaLlaveBD(l);
                }

                // Refrescamos las tablas de llaves
                UGUI.rellenarTabla(JTLsZona, z.obtenerLlaves());
                vaciarTablasPrestables();

                // Refrescamos los almacenes
                refrescarAlmacenes();

                // Desactivamos los motones de 'Eliminar llave', 'Modificar llave' y 'Dar baja llave'
                JBEliminarL.setEnabled(false);
                JTBModificarLlave.setEnabled (false);
                JBDarBajaL.setEnabled(false);

                // Advertimos de que las llaves han sido dadas de baja correctamente
                JOptionPane.showMessageDialog(this,"Las llaves seleccionadas han sido dadas de baja correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible dar de baja las llaves seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBDarBajaLActionPerformed

    private void JTAlmsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTAlmsMouseClicked
            // Mostramos las llaves y llaveros almacenados en el almacen
            try
            {
                if (JTBModificarAlm.isSelected())   JPAlmD.setVisible(true);
                JPLlaveroD.setVisible(false);
                if (!JTBAnadirAlm.isSelected())
                {
                    // Rellenamos la tabla de llaves
                    Almacen a;
                    String DescripAlm = ((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));

                    // Si se trata de todos los almacenes
                    if (DescripAlm.equals("Todos"))
                    {
                        a = new Almacen();
                        a.setCodAlm("TODO");
                        a.setDescripcion("Todos");
                        UGUI.rellenarTabla(JTLs, a.obtenerLlavesTotales((JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText())));
                        a.UBD.cerrarConexion();
                        UGUI.rellenarTabla(JTLls, a.obtenerLlaverosTotales(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                        a.UBD.cerrarConexion();
                    }
                    // En caso de tratarse de un almacen en concreto
                    else
                    {
                        a = new Almacen(DescripAlm);
                        UGUI.rellenarTabla(JTLs, a.obtenerLlaves(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                        a.UBD.cerrarConexion();
                        UGUI.rellenarTabla(JTLls, a.obtenerLlaveros(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                        a.UBD.cerrarConexion();

                        if (JTBModificarAlm.isSelected() && !JTBAnadirAlm.isSelected())     JTBModificarAlm.setSelected(true);
                        else                                                                JTBModificarAlm.setSelected(false);
                        JTBModificarAlm.setEnabled(true);
                        JBEliminarAlm.setEnabled(true);
                    }

                    // Actualizamos los datos en el panel de informacion
                    JTFCodAlm.setText(a.getCodAlm());
                    JTFDescripAlm.setText(a.getDescripcion());

                    // Si tenemos seleccionado un almacen distinto de 'Todos' y alguna llave
                    // en JTLsZona -> Activamos el boton para poder añadirla al almacen
                    String AlmSelec = (String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0);
                    if ((!AlmSelec.equals("Todos")) && (JTLsZona.getSelectedRow() != -1))
                    {
                        JTBAnadirAlmL.setSelected(false);
                        JTBAnadirAlmL.setEnabled(true);
                    }
                    else
                    {
                        JTBAnadirAlmL.setSelected(false);
                        JTBAnadirAlmL.setEnabled(false);
                    }

                    // Si tenemos seleccionado un almacen distinto de 'Todos' -> Activamos el boton para añadir un llavero
                    if (!AlmSelec.equals("Todos"))
                    {
                        JTBAnadirAlmLlB.setSelected(false);
                        JTBAnadirAlmLlB.setEnabled(true);
                    }
                    else
                    {
                        JTBAnadirAlmLlB.setSelected(false);
                        JTBAnadirAlmLlB.setEnabled(false);
                    }

                    // Si tenemos seleccionado el almacen 'Todos' y un llavero
                    // seleccionado -> Activar o desactivar botones de modificacion de las llaves del llavero
                    if ((AlmSelec.equals("Todos")) && (JTLls.getSelectedRow() != -1))
                    {
                        // Si tenemos escogida alguna llave de el apartado de zonas y alguna otra de las que componen el
                        // llavero -> Habilitar boton de añadir llaves al llavero
                        if ((JTLsZona.getSelectedRow() != -1) && (JTLs.getSelectedRow() != -1))
                        {
                            JBAnadirLsLl.setEnabled(true);
                            JBEliminarLsLl.setEnabled(true);
                        }
                        // Si tenemos escogida alguna llave de el apartado de zonas -> Habilitar boton de añadir llaves al llavero
                        else if (JTLsZona.getSelectedRow() != -1)
                        {
                            JBAnadirLsLl.setEnabled(true);
                            JBEliminarLsLl.setEnabled(false);
                        }
                        // Si tenemos escogida alguna llave de las que componen el llavero -> Habilitar el boton de eliminar llaves al llavero
                        else if (JTLs.getSelectedRow() != -1)
                        {
                            JBAnadirLsLl.setEnabled(false);
                            JBEliminarLsLl.setEnabled(true);
                        }
                        // En cualquier otro caso -> Deshabilitamos ambos botones
                        else
                        {
                            JBAnadirLsLl.setEnabled(false);
                            JBEliminarLsLl.setEnabled(false);
                        }
                    }
                    else
                    {
                        JBAnadirLsLl.setEnabled(false);
                        JBEliminarLsLl.setEnabled(false);
                    }

                    // Activamos/Desactivamos todos los botones de eliminacion y modificacion de copias
                    JBEliminarAlmL.setEnabled(false);
                    JBEliminarAlmLl.setEnabled(false);
                    JTBNumCopsLl.setSelected(false);
                    JTBNumCopsLl.setEnabled(false);
                    JPJTFNumCopsLl.setVisible(false);
                    JTBNumCopsL.setSelected(false);
                    JTBNumCopsL.setEnabled(false);
                    JPJTFNumCopsL.setVisible(false);

                    // Desactivamos los botones correspondientes a los llaveros
                    JTBAnadirLl.setSelected(false);
                    JTBAnadirLl.setEnabled(false);
                    JBEliminarLl.setEnabled(false);
                    JTBModificarLl.setSelected(false);
                    JTBModificarLl.setEnabled(false);
                    JBDarBajaLl.setEnabled(false);
                }
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las llaves y llaveros que conforman el almacén.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
    }//GEN-LAST:event_JTAlmsMouseClicked

    private void JTLlsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLlsMouseClicked
        try
        {
            // Activo los botones de modificar, eliminar llavero y dar de baja llavero
            JTBModificarLl.setEnabled(true);
            JBEliminarLl.setEnabled(true);
            JBDarBajaLl.setEnabled(true);

            // Rellenamos la tabla de llaves con las llaves que conforman el llavero
            Llavero ll = new Llavero((Integer) JTLls.getModel().getValueAt(JTLls.getSelectedRow(),0));
            UGUI.rellenarTabla(JTLs, ll.obtenerLlaves(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
            ll.UBD.cerrarConexion();

            // Rellenamos los datos del llavero
            // Establecemos el valor de todos los campos
            JTFCodLl.setText(new Integer(ll.getCodigo()).toString());
            JTFDescripLl.setText(ll.getDescripcion());
            
            // Activamos el JTB de modificar numero de copias y el de eliminar del almacen si es que existe algun almacen seleccionado distinto del general ('Todos')
            String AlmSelec = (String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0);
            if ((JTAlms.getSelectedRow() != -1) && (!AlmSelec.equals("Todos")))
            {
                JTBNumCopsL.setSelected(false);
                JTBNumCopsL.setEnabled(false);
                JTBNumCopsLl.setSelected(false);
                JTBNumCopsLl.setEnabled(true);
                JBEliminarAlmLl.setEnabled(true);
                JBEliminarAlmLl.setEnabled(true);
            }
            // En cualquier otro caso
            else
            {
                JTBNumCopsL.setSelected(false);
                JTBNumCopsL.setEnabled(false);
                JTBNumCopsLl.setSelected(false);
                JTBNumCopsLl.setEnabled(false);
                JBEliminarAlmLl.setEnabled(false);
            }

            // Si tenemos seleccionado el almacen 'Todos' y un llavero
            // seleccionado -> Activar o desactivar botones de modificacion de las llaves del llavero
            if (AlmSelec.equals("Todos"))
            {
                // Si tenemos escogida alguna llave de el apartado de zonas y alguna otra de las que componen el
                // llavero -> Habilitar boton de añadir llaves al llavero
                if ((JTLsZona.getSelectedRow() != -1) && (JTLs.getSelectedRow() != -1))
                {
                    JBAnadirLsLl.setEnabled(true);
                    JBEliminarLsLl.setEnabled(true);
                }
                // Si tenemos escogida alguna llave de el apartado de zonas -> Habilitar boton de añadir llaves al llavero
                else if (JTLsZona.getSelectedRow() != -1)
                {
                    JBAnadirLsLl.setEnabled(true);
                    JBEliminarLsLl.setEnabled(false);
                }
                // Si tenemos escogida alguna llave de las que componen el llavero -> Habilitar el boton de eliminar llaves al llavero
                else if (JTLs.getSelectedRow() != -1)
                {
                    JBAnadirLsLl.setEnabled(false);
                    JBEliminarLsLl.setEnabled(true);
                }
                // En cualquier otro caso -> Deshabilitamos ambos botones
                else
                {
                    JBAnadirLsLl.setEnabled(false);
                    JBEliminarLsLl.setEnabled(false);
                }
            }
            else
            {
                JBAnadirLsLl.setEnabled(false);
                JBEliminarLsLl.setEnabled(false);
            }

            // Desactivamos el JTB y JTF de numero de copias de llaves
            JTBNumCopsL.setSelected(false);
            JTBNumCopsL.setEnabled(false);
            JPJTFNumCopsL.setVisible(false);
            JBEliminarAlmL.setEnabled(false);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las llaves que conforman el llavero.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTLlsMouseClicked

    private void JTFBuscarLLlKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarLLlKeyReleased
        try
        {
            // Buscamos almacenes, llaves y llaveros cuya descripcion contenga el filtro introducido hasta el momento
            Almacen a = new Almacen();
            String Filtro = JTFBuscarLLl.getText();
            UGUI.rellenarTabla(JTAlms, a.obtenerAlmacenes(Filtro));
            a.UBD.cerrarConexion();
            UGUI.rellenarTabla(JTLs, a.obtenerLlavesTotales(Filtro));
            a.UBD.cerrarConexion();
            UGUI.rellenarTabla(JTLls, a.obtenerLlaverosTotales(Filtro));
            a.UBD.cerrarConexion();
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible acotar la búsqueda.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTFBuscarLLlKeyReleased

    private void JTLsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsMouseClicked
        // Activamos el JTB de modificar numero de copias y el de eliminar del almacen si es que existe algun almacen seleccionado distinto del general ('Todos')
        String AlmSelec = (String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0);
        if ((JTAlms.getSelectedRow() != -1) && (!AlmSelec.equals("Todos")))
        {
            if (JTLls.getSelectedRow() != -1)
            {
                JBEliminarAlmL.setEnabled(false);
                JTBNumCopsL.setSelected(false);
                JTBNumCopsL.setEnabled(false);
            }
            else
            {
                JBEliminarAlmL.setEnabled(true);
                JTBNumCopsL.setSelected(false);
                JTBNumCopsL.setEnabled(true);
            }
        }
        else
        {
            JTBNumCopsL.setSelected(false);
            JTBNumCopsL.setEnabled(false);
            JBEliminarAlmL.setEnabled(false);
        }

        // Si hemos seleccionado varias llaves -> Activamos el boton de crear llavero
        if (JTLs.getSelectedRowCount()>1)
        {
            JTBAnadirLl.setSelected(false);
            JTBAnadirLl.setEnabled(true);
        }
        // Si no, lo desactivamos
        else
        {
            JTBAnadirLl.setSelected(false);
            JTBAnadirLl.setEnabled(false);
        }

        // Si tenemos seleccionado el almacen 'Todos' y un llavero
        // seleccionado -> Activar o desactivar botones de modificacion de las llaves del llavero
        if ((AlmSelec.equals("Todos")) && (JTLls.getSelectedRow() != -1))
        {
            // Si tenemos escogida alguna llave de las que componen el llavero -> Habilitar el boton de eliminar llaves al llavero
            JBAnadirLsLl.setEnabled(false);
            JBEliminarLsLl.setEnabled(true);

            // Si tenemos escogida alguna llave de el apartado de zonas -> Habilitar boton de añadir llaves al llavero
            if (JTLsZona.getSelectedRow() != -1)
            {
                JBAnadirLsLl.setEnabled(true);
            }
        }
        else
        {
            JBAnadirLsLl.setEnabled(false);
            JBEliminarLsLl.setEnabled(false);
        }
    }//GEN-LAST:event_JTLsMouseClicked

    private void JTBNumCopsLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBNumCopsLActionPerformed
        // Si el boton esta activado
        if (JTBNumCopsL.isSelected())
        {
            Integer NumCopias = (Integer) JTLs.getModel().getValueAt(JTLs.getSelectedRow(),3);
            JTFNumCopsL.setText(NumCopias.toString());
            JPJTFNumCopsL.setVisible(true);
            JBEliminarAlmL.setEnabled(true);
            JTBAnadirAlmL.setSelected(false);
            JTBAnadirAlmL.setEnabled(false);
        }
        // Si no esta activado
        else
        {
            JPJTFNumCopsL.setVisible(false);
            JTBAnadirAlmL.setSelected(false);
            JTBAnadirAlmL.setEnabled(true);
        }
    }//GEN-LAST:event_JTBNumCopsLActionPerformed

    private void JTBNumCopsLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBNumCopsLlActionPerformed
        // Si el boton esta activado
        if (JTBNumCopsLl.isSelected())
        {
            Integer NumCopias = (Integer) JTLls.getModel().getValueAt(JTLls.getSelectedRow(),2);
            JTFNumCopsLl.setText(NumCopias.toString());
            JPJTFNumCopsLl.setVisible(true);
            JBEliminarAlmLl.setEnabled(true);
            JTBAnadirAlmLlB.setSelected(false);
            JTBAnadirAlmLlB.setEnabled(false);
            JPLls.setVisible(false);
        }
        // Si no esta activado
        else
        {
            JPJTFNumCopsLl.setVisible(false);
            JTBAnadirAlmLlB.setSelected(false);
            JTBAnadirAlmLlB.setEnabled(true);
        }
    }//GEN-LAST:event_JTBNumCopsLlActionPerformed

    private void JBEliminarAlmLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarAlmLActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las llaves seleccionadas del almacen
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las llaves seleccionadas del almacen.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las llaves del almacen
            if (seleccion == 0)
            {
                // Obtenemos el almacen
                Almacen alm = new Almacen ((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
                // Obtenemos las llaves seleccionadas
                int[] llavesSelecs = JTLs.getSelectedRows();

                // Eliminamos cada llave
                GLL_Copias gllc;
                for (int i=0; i<llavesSelecs.length; i++)
                {
                    // Obtenemos la llave (GLL_Copias)
                    gllc = new GLL_Copias(new Llave((Integer) JTLs.getModel().getValueAt(llavesSelecs[i],0)),(Integer) JTLs.getModel().getValueAt(JTLs.getSelectedRow(),3));
                    // Eliminamos la llave
                    alm.removeGLLCBD(gllc);
                }

                // Ejecutamos el disparador
                alm.disparadorAlmL();

                // Refrescamos la tabla de llaves asociada a zonas si existe alguna de ellas seleccionada
                int fila = JTZonas.getSelectedRow();
                if (fila != -1)
                {
                    Zona z = new Zona((String) JTZonas.getModel().getValueAt(fila,0));
                    UGUI.rellenarTabla(JTLsZona, z.obtenerLlaves());
                    z.UBD.cerrarConexion();
                }

                // Refrescamos las tablas de almacenes
                refrescarAlmacenes();

                // Desactivamos los motones de 'Eliminar llave', 'Modificar llave' y 'Añadir llavero'
                JBEliminarAlmL.setEnabled(false);
                JTBNumCopsL.setSelected(false);
                JTBNumCopsL.setEnabled (false);
                JTBAnadirLl.setSelected(false);
                JTBAnadirLl.setEnabled (false);

                // Advertimos de que las llaves han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las llaves seleccionadas han sido eliminadas correctamente del almacén.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar del almacén las llaves seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarAlmLActionPerformed

    private void JBEliminarAlmLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarAlmLlActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los llaveros seleccionados del almacen
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los llaveros seleccionados del almacen.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los llaveros del almacen
            if (seleccion == 0)
            {
                // Obtenemos el almacen
                Almacen alm = new Almacen ((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
                // Obtenemos los llaveros seleccionados
                int[] llaverosSelecs = JTLls.getSelectedRows();

                // Eliminamos cada llavero
                GLL_Copias gllc;
                for (int i=0; i<llaverosSelecs.length; i++)
                {
                    // Obtenemos la llave (GLL_Copias)
                    gllc = new GLL_Copias(new Llavero((Integer) JTLls.getModel().getValueAt(llaverosSelecs[i],0)),(Integer) JTLls.getModel().getValueAt(JTLls.getSelectedRow(),2));
                    // Eliminamos el llavero
                    alm.removeGLLCBD(gllc);
                }
                // Ejecutamos el disparador
                alm.disparadorAlmLl();

                // Refrescamos las tablas de almacenes
                refrescarAlmacenes();

                // Desactivamos los botones de 'Añadir llavero', 'Eliminar llavero' y 'Modificar llavero'
                JTBAnadirAlmLlB.setSelected(false);
                JTBAnadirAlmLlB.setEnabled(false);
                JBEliminarAlmLl.setEnabled(false);
                JTBNumCopsLl.setSelected(false);
                JTBNumCopsLl.setEnabled (false);

                // Advertimos de que los llaveros han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los llaveros seleccionados han sido eliminados correctamente del almacén.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar del almacén los llaveros seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarAlmLlActionPerformed

    private void JBGrabarAlmLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarAlmLActionPerformed
        try
        {
            // Obtenemos el almacen y el numero de copias
            Almacen alm = new Almacen((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
            int NumCopias = new Integer(JTFNumCopsL.getText()).intValue();

            // Comprobamos si se trata de una insercion o de una modificacion de llave
            // -> Insercion
            if (JTBAnadirAlmL.isSelected())
            {
                // Avisamos de que vamos a añadir la llave y copias indicadas al almacen
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a añadir la llave y copias indicadas al almacén seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Añadimos la llave y copias al almacen
                if (seleccion == 0)
                {
                    // Obtenemos la llave seleccionada
                    Llave l = new Llave ((Integer) JTLsZona.getModel().getValueAt(JTLsZona.getSelectedRow(),0));

                    // Si la llave existe -> Actualizamos su numero de copias
                    String key = "L - " + l.getCodigo() + " - " + UCAL.calendarToString(l.getFechaI()).substring(0,19);
                    if (alm.getGLLCs().containsKey(key))
                    {
                        //Actualizamos el numero de copias de la llave
                        GLL_Copias gllc_old = alm.getGLLCBD(key);
                        GLL_Copias gllc_new = alm.getGLLCBD(key);
                        gllc_new.setNumCopias(gllc_new.getNumCopias()+ NumCopias);
                        alm.updateGLLCBD(gllc_old, gllc_new);
                    }
                    // Si no existe -> Introducimos la llave como nueva en el almacen
                    else
                    {
                        // Creamos el GLL_Copias
                        GLL_Copias gllc = new GLL_Copias(l,NumCopias);
                        // Insertamos el GLL_Copias en el almacen
                        alm.insertGLLCBD(gllc);
                    }

                    // Actualizamos la tabla de llaves
                    UGUI.rellenarTabla(JTLs,alm.obtenerLlaves(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    alm.UBD.cerrarConexion();

                    // Escondemos el panel de insercion de copias
                    JPJTFNumCopsL.setVisible(false);

                    // Des-seleccionamos/desactivamos los botones necesarios
                    JTBAnadirAlmL.setSelected(false);
                    JTBAnadirAlmL.setEnabled(false);
                    JBEliminarAlmL.setEnabled(false);
                    JTBNumCopsL.setSelected(false);
                    JTBNumCopsL.setEnabled(false);

                    // Advertimos de que la llave ha sido insertada correctamente en el almacen
                    JOptionPane.showMessageDialog(this,"La llave ha sido insertada correctamente en el almacén.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Modificacion
            else
            {
                // Avisamos de que vamos a modificar la llave indicada
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar la llave indicada del almacén seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos la llave del almacen
                if (seleccion == 0)
                {
                    // Obtenemos la llave seleccionada
                    Llave l = new Llave ((Integer) JTLs.getModel().getValueAt(JTLs.getSelectedRow(),0));

                    // Modificamos el numero de copias existentes
                    String key = "L - " + l.getCodigo() + " - " + UCAL.calendarToString(l.getFechaI()).substring(0,19);

                    //Actualizamos el numero de copias de la llave
                    GLL_Copias gllc_old = alm.getGLLCBD(key);
                    GLL_Copias gllc_new = alm.getGLLCBD(key);
                    gllc_new.setNumCopias(NumCopias);
                    alm.updateGLLCBD(gllc_old, gllc_new);

                    // Actualizamos la tabla de llaves
                    UGUI.rellenarTabla(JTLs,alm.obtenerLlaves(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    alm.UBD.cerrarConexion();

                    // Escondemos el panel de insercion de copias
                    JPJTFNumCopsL.setVisible(false);

                    // Des-seleccionamos/desactivamos los botones necesarios
                    JTBAnadirAlmL.setSelected(false);
                    JTBAnadirAlmL.setEnabled(false);
                    JBEliminarAlmL.setEnabled(false);
                    JTBNumCopsL.setSelected(false);
                    JTBNumCopsL.setEnabled(false);

                    // Advertimos de que la llave ha sido modificada correctamente
                    JOptionPane.showMessageDialog(this,"La llave ha sido modificada correctamente en el almacén.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar la llave en el almacén.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarAlmLActionPerformed

    private void JTBAnadirAlmLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirAlmLActionPerformed
        // Si esta activado -> Mostramos JPJTFNumCopsL y desactivamos botones
        if (JTBAnadirAlmL.isSelected())
        {
            JTFNumCopsL.setText("1");
            JPJTFNumCopsL.setVisible(true);
            JBEliminarAlmL.setEnabled(false);
            JTBNumCopsL.setSelected(false);
            JTBNumCopsL.setEnabled(false);
        }
        // Si no lo ocultamos
        else                            JPJTFNumCopsL.setVisible(false);
    }//GEN-LAST:event_JTBAnadirAlmLActionPerformed

    private void JBGrabarAlmLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarAlmLlActionPerformed
        try
        {
            // Obtenemos el almacen y el numero de copias
            Almacen alm = new Almacen((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
            int NumCopias = new Integer(JTFNumCopsLl.getText()).intValue();

            // Comprobamos si se trata de una insercion o de una modificacion de llavero
            // -> Insercion
            if (JTBAnadirAlmLlB.isSelected())
            {
                // Avisamos de que vamos a añadir el llavero y copias indicadas al almacen
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a añadir el llavero y copias indicadas al almacén seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Añadimos el llavero y copias al almacen
                if (seleccion == 0)
                {
                    // Obtenemos el llavero seleccionado
                    Llavero ll = new Llavero (Integer.parseInt((String)JCBLlaveros.getModel().getSelectedItem()));

                    // Si el llavero existe -> Actualizamos su numero de copias
                    String key = "LL - " + ll.getCodigo() + " - " + UCAL.calendarToString(ll.getFechaI()).substring(0,19);
                    if (alm.getGLLCs().containsKey(key))
                    {
                        //Actualizamos el numero de copias del llavero
                        GLL_Copias gllc_old = alm.getGLLCBD(key);
                        GLL_Copias gllc_new = alm.getGLLCBD(key);
                        gllc_new.setNumCopias(gllc_old.getNumCopias() + NumCopias);
                        alm.updateGLLCBD(gllc_old, gllc_new);
                    }
                    // Si no existe -> Introducimos el llavero como nuevo en el almacen
                    else
                    {
                        // Creamos el GLL_Copias
                        GLL_Copias gllc= new GLL_Copias(ll,NumCopias);
                        // Insertamos el GLL_Copias en el almacen
                        alm.insertGLLCBD(gllc);
                    }

                    // Actualizamos la tabla de llaveros
                    UGUI.rellenarTabla(JTLls,alm.obtenerLlaveros(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    alm.UBD.cerrarConexion();

                    // Escondemos el panel de insercion de copias
                    JPJTFNumCopsLl.setVisible(false);

                    // Escondemos los paneles de llavero y de insercion de copias
                    JPLls.setVisible(false);
                    JPJTFNumCopsLl.setVisible(false);

                    // Des-seleccionamos/desactivamos los botones necesarios
                    JTBAnadirAlmLlB.setSelected(false);
                    JTBAnadirAlmLlB.setEnabled(false);
                    JBEliminarAlmLl.setEnabled(false);
                    JTBNumCopsLl.setSelected(false);
                    JTBNumCopsLl.setEnabled(false);

                    // Advertimos de que el llavero ha sido insertado correctamente en el almacen
                    JOptionPane.showMessageDialog(this,"El llavero ha sido insertado correctamente en el almacén.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Modificacion
            else
            {
                // Avisamos de que vamos a modificar el llavero indicado
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el llavero indicado del almacén seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el llavero del almacen
                if (seleccion == 0)
                {
                    // Obtenemos el llavero seleccionado
                    Llavero ll = new Llavero ((Integer) JTLls.getModel().getValueAt(JTLls.getSelectedRow(),0));

                    // Modificamos el numero de copias existentes
                    String key = "LL - " + ll.getCodigo() + " - " + UCAL.calendarToString(ll.getFechaI()).substring(0,19);

                    //Actualizamos el numero de copias del llavero
                    GLL_Copias gllc_old = alm.getGLLCBD(key);
                    GLL_Copias gllc_new = alm.getGLLCBD(key);
                    gllc_new.setNumCopias(NumCopias);
                    alm.updateGLLCBD(gllc_old, gllc_new);

                    // Actualizamos la tabla de llaveros
                    UGUI.rellenarTabla(JTLls,alm.obtenerLlaveros(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    alm.UBD.cerrarConexion();

                    // Escondemos los paneles de llavero y de insercion de copias
                    JPLls.setVisible(false);
                    JPJTFNumCopsLl.setVisible(false);

                    // Des-seleccionamos/desactivamos los botones necesarios
                    JTBAnadirAlmLlB.setSelected(false);
                    JTBAnadirAlmLlB.setEnabled(false);
                    JBEliminarAlmLl.setEnabled(false);
                    JTBNumCopsLl.setSelected(false);
                    JTBNumCopsLl.setEnabled(false);

                    // Advertimos de que el llavero ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El llavero ha sido modificado correctamente en el almacén.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el llavero en el almacén.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarAlmLlActionPerformed

    private void JTBAnadirAlmActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirAlmActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirAlm.isSelected())
        {
            // Mostramos el panel de datos
            JTFCodAlm.setText("");
            JTFDescripAlm.setText("");
            JPAlmD.setVisible(true);
            // Desactivamos el resto de botones y la seleccion
            JBEliminarAlm.setEnabled(false);
            JTBModificarAlm.setSelected(false);
            JTBModificarAlm.setEnabled(false);
            JTAlms.clearSelection();
        }
        // Si no esta pulsado
        else
        {
            // Activamos el resto de botones y la seleccion
            JBEliminarAlm.setEnabled(true);
            JTBModificarAlm.setSelected(false);
            JTBModificarAlm.setEnabled(true);
            
            // Escondemos el panel de datos
            JPAlmD.setVisible(false);
            // Refrescamos las tablas de almacenes
            refrescarAlmacenes();
        }
    }//GEN-LAST:event_JTBAnadirAlmActionPerformed

    private void JBEliminarAlmActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarAlmActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los almacenes seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los almacenes seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los almacenes
            if (seleccion == 0)
            {
                // Obtenemos los almacenes seleccionados
                int[] almsSelecs = JTAlms.getSelectedRows();

                // Eliminamos cada almacen
                Almacen alm = new Almacen();
                for (int i=0; i<almsSelecs.length; i++)
                {
                    // Obtenemos el almacen
                    alm = new Almacen ((String) JTAlms.getModel().getValueAt(almsSelecs[i],0));
                    // Eliminamos el almacen
                    alm.removeAlmacenBD();
                }

                // Refrescamos las tablas de almacenes
                refrescarAlmacenes();

                // Establecemos el estado de los botones
                JTBAnadirAlm.setSelected(false);
                JTBAnadirAlm.setEnabled(true);
                JTBModificarAlm.setSelected(false);
                JTBModificarAlm.setEnabled(false);
                JBEliminarAlm.setEnabled(false);

                // Advertimos de que los almacenes han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los almacenes seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los almacenes seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarAlmActionPerformed

    private void JTBModificarAlmActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarAlmActionPerformed
        try
        {
            // Si el boton esta pulsado
            if (JTBModificarAlm.isSelected())
            {
                // Mostramos el panel de datos
                String almacen = (String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0);
                Almacen alm;
                if (!almacen.equals("Todos"))    alm = new Almacen(almacen);
                else
                {
                    alm = new Almacen();
                    alm.setCodAlm("TODO");
                    alm.setDescripcion("Todos");
                }
                JTFCodAlm.setText(alm.getCodAlm());
                JTFDescripAlm.setText(alm.getDescripcion());
                JPAlmD.setVisible(true);

                // Desactivamos el resto de botones
                JBEliminarAlm.setEnabled(true);
                JTBAnadirAlm.setSelected(false);
                JTBAnadirAlm.setEnabled(true);
            }
            // Si no esta pulsado
            else
            {
                // Escondemos el panel de datos
                JPAlmD.setVisible(false);
                // Activamos el boton de añadir almacen
                JTBAnadirAlm.setSelected(false);
                JTBAnadirAlm.setEnabled(true);
                // Seleccionamos el primer almacen
                JTAlms.setRowSelectionInterval(0,0);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos del almacén.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTBModificarAlmActionPerformed

    private void JBGrabarAlmActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarAlmActionPerformed
        try
        {
            // Creamos el almacen nuevo
            Almacen alm_new = new Almacen(JTFCodAlm.getText(),JTFDescripAlm.getText(),new HashMap());

            // Comprobamos si se trata de una insercion o de una actualizacion
            // -> Insercion
            if (JTBAnadirAlm.isSelected())
            {
                    // Avisamos de que vamos a modificar el nivel
                    int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a insertar el almacén.\n" +
                            "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                    // Si se acepta la operacion -> Insertamos el nuevo almacen
                    if (seleccion == 0)
                    {
                        // Insertamos el nuevo almacen
                        alm_new.insertAlmacenBD();

                        // Refrescamos las tablas de almacenes
                        refrescarAlmacenes();

                        // Establecemos el estado de los botones y borramos el panel de datos
                        JTBAnadirAlm.setSelected(false);
                        JTBAnadirAlm.setEnabled(true);
                        JBEliminarAlm.setEnabled(false);
                        JTBModificarAlm.setSelected(false);
                        JTBModificarAlm.setEnabled(false);
                        JPAlmD.setVisible(false);
                        JBEliminarAlm.setEnabled(false);

                        // Advertimos de que el almacen ha sido insertado correctamente
                        JOptionPane.showMessageDialog(this,"El almacén ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                    }
            }
            // -> Actualización
            else
            {
                    // Avisamos de que vamos a modificar el almacén
                    int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el almacén seleccionado.\n" +
                            "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                    // Si se acepta la operacion -> Modificamos el almacen seleccionado
                    if (seleccion == 0)
                    {
                        // Obtenemos el almacen viejo
                        Almacen alm_old = new Almacen((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));

                        // Actualizamos el almacen viejo por el nuevo
                        alm_new.setGLLCs(alm_old.getGLLCs());
                        alm_old.updateAlmacenBD(alm_new);

                        // Refrescamos las tablas de almacenes
                        refrescarAlmacenes();

                        // Establecemos el estado de los botones y borramos el panel de datos
                        JTBAnadirAlm.setSelected(false);
                        JTBAnadirAlm.setEnabled(true);
                        JBEliminarAlm.setEnabled(false);
                        JTBModificarAlm.setSelected(false);
                        JTBModificarAlm.setEnabled(false);
                        JPAlmD.setVisible(false);
                        JBEliminarAlm.setEnabled(false);

                        // Advertimos de que el almacen ha sido modificado correctamente
                        JOptionPane.showMessageDialog(this,"El almacén ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                    }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el almacén.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarAlmActionPerformed

    private void JTBAnadirLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirLlActionPerformed
        try
        {
            // Si el boton esta pulsado
            if (JTBAnadirLl.isSelected())
            {
                // Des-seleccionamos y descativamos los botones de modificar y dar de baja el llavero
                JTBModificarLl.setSelected(false);
                JTBModificarLl.setEnabled(false);
                JBDarBajaLl.setEnabled(false);

                // Deseleccionamos cualquier llavero seleccionado
                JTLls.clearSelection();

                // Borramos los campos y mostramos el panel
                Llavero ll = new Llavero();
                JTFCodLl.setText(new Integer(ll.codigoPorDefecto()).toString());
                JTFDescripLl.setText("");
                JPLlaveroD.setVisible(true);
                JBAnadirLsLl.setEnabled(false);
                JBEliminarLsLl.setEnabled(false);
            }
            // Si no lo esta -> Escondemos el panel de datos del llavero
            else        JPLlaveroD.setVisible(false);
        }
        catch(Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible preparar el panel para insertar el llavero.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTBAnadirLlActionPerformed

    private void JTBModificarLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarLlActionPerformed
        // Si el boton esta pulsado
        if (JTBModificarLl.isSelected())
        {
            // Des-seleccionamos y desactivamos el boton de añadir llavero
            JTBAnadirLl.setSelected(false);
            JTBAnadirLl.setEnabled(false);

            // Si estamos tratando todos los almacenes y el llavero y llave que constan en el estan seleccionados, permitimos que se pueda eliminar la segunda
            String AlmSelec = (String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0);
            if ((AlmSelec.equals("Todos") && (JTLs.getSelectedRow() != -1) && (JTLls.getSelectedRow() != -1)))      JBEliminarLsLl.setEnabled(true);
            // Si llavero y llave de una zona estan seleccionados, permitimos que se pueda añadir la segunda
            if ((JTLsZona.getSelectedRow() != -1) && (JTLls.getSelectedRow() != -1))                                JBAnadirLsLl.setEnabled(true);

            // Mostramos el panel
            JPLlaveroD.setVisible(true);
        }
        // Si no lo esta -> Escondemos el panel de datos del llavero
        else    JPLlaveroD.setVisible(false);
    }//GEN-LAST:event_JTBModificarLlActionPerformed

    private void JBEliminarLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarLlActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los llaveros seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los llaveros seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los llaveros
            if (seleccion == 0)
            {
                // Obtenemos los llaveros seleccionados
                int[] llaverosSelecs = JTLls.getSelectedRows();

                // Eliminamos cada llavero
                Llavero ll;
                for (int i=0; i<llaverosSelecs.length; i++)
                {
                    // Obtenemos el llavero
                    ll = new Llavero((Integer) JTLls.getModel().getValueAt(llaverosSelecs[i],0));
                    // Eliminamos el llavero
                    ll.removeLlaveroBD();
                    // Ejecutamos el disparador
                    ll.disparadorRemove();
                }

                // Refrescamos la tabla de llaveros
                String DescripAlm = ((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
                Almacen a;
                if (DescripAlm.equals("Todos"))
                {
                    a = new Almacen();
                    UGUI.rellenarTabla(JTLls, a.obtenerLlaverosTotales(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    a.UBD.cerrarConexion();
                }
                else
                {
                    a = new Almacen(DescripAlm);
                    UGUI.rellenarTabla(JTLls, a.obtenerLlaveros(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    a.UBD.cerrarConexion();
                }

                // Vaciamos la tabla de llaves
                String[] Etiquetas = {"Código","Zona","Descripción"};
                JTLs.setModel(new DefaultTableModel(Etiquetas,0));

                // Desactivamos los motones de 'Eliminar llavero', 'Modificar llavero' y 'Dar baja llavero'
                JBEliminarLl.setSelected(false);
                JBEliminarLl.setEnabled(false);
                JTBModificarLl.setSelected(false);
                JTBModificarLl.setEnabled (false);
                JBDarBajaLl.setEnabled(false);
                JBEliminarAlmLl.setEnabled(false);
                JTBNumCopsLl.setSelected(false);
                JTBNumCopsLl.setEnabled(false);

                // Advertimos de que los llaveros han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los llaveros seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los llaveros seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarLlActionPerformed

    private void JBDarBajaLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBDarBajaLlActionPerformed
        try
        {
            // Avisamos de que vamos a dar de baja los llaveros seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de baja los llaveros seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Damos de baja los llaveros
            if (seleccion == 0)
            {
                // Obtenemos los llaveros seleccionados
                int[] llaverosSelecs = JTLls.getSelectedRows();

                // Damos de baja cada llavero
                Llavero ll;
                for (int i=0; i<llaverosSelecs.length; i++)
                {
                    // Obtenemos el llavero
                    ll = new Llavero((Integer) JTLls.getModel().getValueAt(llaverosSelecs[i],0));
                    // Damos de baja el llavero
                    ll.darBajaLlaveroBD();
                }

                // Refrescamos la tabla de llaveros
                String DescripAlm = ((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
                Almacen a;
                // Si se trata del almacen general
                if (DescripAlm.equals("Todos"))
                {
                    a = new Almacen();
                    UGUI.rellenarTabla(JTLls, a.obtenerLlaverosTotales(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    a.UBD.cerrarConexion();
                }
                // Si se trata de un almacen concreto
                else
                {
                    a = new Almacen(DescripAlm);
                    UGUI.rellenarTabla(JTLls, a.obtenerLlaveros(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    a.UBD.cerrarConexion();
                }

                // Desactivamos los motones de 'Eliminar llave', 'Modificar llave' y 'Dar baja llave'
                JBEliminarLl.setEnabled(false);
                JTBModificarLl.setEnabled (false);
                JBDarBajaLl.setEnabled(false);

                // Refrescamos los almacenes
                refrescarAlmacenes();

                // Advertimos de que los llaveros han sido dados de baja correctamente
                JOptionPane.showMessageDialog(this,"Los llaveros seleccionados han sido dados de baja correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible dar de baja los llaveros seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBDarBajaLlActionPerformed

    private void JBGrabarLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarLlActionPerformed
        try
        {
            // Creamos el nuevo llavero
            Llavero ll_new = new Llavero(new Integer(JTFCodLl.getText()),Calendar.getInstance(),null,JTFDescripLl.getText(),new HashMap());

            // Obtenemos las llaves seleccionadas y las introducimos en el llavero
            int[] llavesSelecs = JTLs.getSelectedRows();
            Llave l;
            for (int i=0; i<llavesSelecs.length; i++)
            {
                // Obtenemos la llave
                l = new Llave ((Integer) JTLs.getModel().getValueAt(llavesSelecs[i],0));
                // Insertamos la llave
                ll_new.addLlave(l);
            }

            // Comprobamos si se trata de una insercion o de una modificacion
            // -> Insercion
            int seleccion;
            if (JTBAnadirLl.isSelected())
            {
                // Avisamos de que vamos a añadir el llavero proporcionado
                seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta el llavero.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta el llavero
                if (seleccion == 0)
                {
                    // Ejecutamos el disparador
                    ll_new.disparadorInsert();

                    // Insertamos el llavero
                    ll_new.insertLlaveroBD();

                    // Refrescamos la tabla de llaveros
                    Almacen a = new Almacen();
                    String DescripAlm = ((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
                    if (DescripAlm.equals("Todos")) UGUI.rellenarTabla(JTLls, a.obtenerLlaverosTotales((JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText())));
                    else
                    {
                        a = new Almacen(DescripAlm);
                        UGUI.rellenarTabla(JTLls, a.obtenerLlaveros(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    }
                    a.UBD.cerrarConexion();
                    JTLs.clearSelection();
                    JBEliminarAlmLl.setEnabled(false);
                    JTBNumCopsLl.setSelected(false);
                    JTBNumCopsLl.setEnabled(false);

                    // Ocultamos todo lo innecesario
                    JPLlaveroD.setVisible(false);
                    JTBAnadirLl.setSelected(false);
                    JTBAnadirLl.setEnabled(false);
                    JBEliminarLl.setEnabled(false);

                    // Advertimos de que el llavero ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El llavero ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Modificacion
            else
            {
                // Avisamos de que vamos a modificar el llavero seleccionado
                seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el llavero seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el llavero
                if (seleccion == 0)
                {
                    // Obtenemos el llavero viejo
                    Llavero ll_old = new Llavero ((Integer) JTLls.getModel().getValueAt(JTLls.getSelectedRow(),0));

                    // Modificamos el llavero viejo con el nuevo
                    ll_new.setLlaves(ll_old.getLlaves());
                    ll_old.updateDatosBD(ll_new);

                    // Refrescamos la tabla de llaveros
                    Almacen a = new Almacen();
                    String DescripAlm = ((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
                    if (DescripAlm.equals("Todos")) UGUI.rellenarTabla(JTLls, a.obtenerLlaverosTotales((JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText())));
                    else
                    {
                        a = new Almacen(DescripAlm);
                        UGUI.rellenarTabla(JTLls, a.obtenerLlaveros(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                    }
                    a.UBD.cerrarConexion();
                    JTLs.clearSelection();
                    JBEliminarAlmLl.setEnabled(false);
                    JTBNumCopsLl.setSelected(false);
                    JTBNumCopsLl.setEnabled(false);
                    
                    // Ocultamos todo lo innecesario
                    JPLlaveroD.setVisible(false);
                    JBEliminarLl.setEnabled(false);
                    JTBModificarLl.setSelected(false);
                    JTBModificarLl.setEnabled(false);

                    // Advertimos de que el llavero ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El llavero ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            JPJTFNumCopsL.setVisible(false);
            JPJTFNumCopsLl.setVisible(false);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el llavero indicado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarLlActionPerformed

    private void JTBAnadirAlmLlBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirAlmLlBActionPerformed
        try
        {
            // Si esta activado -> Mostramos JPJTFNumCopsLl, rellenamos los llaveros existentes y desactivamos botones
            if (JTBAnadirAlmLlB.isSelected())
            {
                JTFNumCopsLl.setText("1");
                Almacen alm = new Almacen((String) JTAlms.getModel().getValueAt(JTAlms.getSelectedRow(),0));
                UGUI.rellenarJCB(JCBLlaveros, alm.obtenerCodLls());
                alm.UBD.cerrarConexion();
                JPJTFNumCopsLl.setVisible(true);
                JPLls.setVisible(true);
                JBEliminarAlmL.setEnabled(false);
                JTBNumCopsL.setSelected(false);
                JTBNumCopsL.setEnabled(false);
            }
            // Si no lo ocultamos
            else                            JPJTFNumCopsLl.setVisible(false);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible obtener los llaveros existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTBAnadirAlmLlBActionPerformed

    private void JBAnadirLsLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirLsLlActionPerformed
        try
        {
            // Avisamos de que vamos a añadir las llaves al llavero seleccionado
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a añadir las llaves seleccionadas al llavero.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Añadimos las llaves seleccionadas al llavero
            if (seleccion == 0)
            {
                // Obtenemos el llavero
                Llavero ll_old = new Llavero ((Integer)JTLls.getModel().getValueAt(JTLls.getSelectedRow(),0));
                Llavero ll_new = new Llavero ((Integer)JTLls.getModel().getValueAt(JTLls.getSelectedRow(),0));

                // Obtenemos las llaves
                int[] llavesSelecs = JTLsZona.getSelectedRows();

                // Insertamos cada llave en el llavero
                Llave l;
                for (int i=0; i<llavesSelecs.length; i++)
                {
                    // Obtenemos la llave
                    l = new Llave((Integer)JTLsZona.getModel().getValueAt(llavesSelecs[i],0));
                    // Insertamos la llave
                    ll_new.addLlave(l);
                }

                // Actualizamos el llavero viejo por el nuevo
                ll_old.updateLlaveroBD(ll_new);

                // Refrescamos la tabla de llaves
                UGUI.rellenarTabla(JTLs,ll_new.obtenerLlaves(JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText()));
                ll_new.UBD.cerrarConexion();

                // Dejamos los campos del llavero como debieran estar (segun el llavero seleccionado)
                JTFCodLl.setText(new Integer(ll_old.getCodigo()).toString());
                JTFDescripLl.setText(ll_old.getDescripcion());
                
                // Des-seleccionamos las llaves en JTLsZona y el boton de modificar llavero, escondiendo ademas el paneld e datos
                JTLsZona.clearSelection();
                JTBModificarLl.setSelected(false);
                JPLlaveroD.setVisible(false);

                // Advertimos de que las llaves han sido insertadas correctamente
                JOptionPane.showMessageDialog(this,"Las llaves han sido insertadas correctamente en el llavero.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar las llaves en el llavero.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBAnadirLsLlActionPerformed

    private void JBEliminarLsLlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarLsLlActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las llaves del llavero seleccionado
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las llaves seleccionadas del llavero.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las llaves seleccionadas del llavero
            if (seleccion == 0)
            {
                // Obtenemos el llavero
                Llavero ll_old = new Llavero ((Integer)JTLls.getModel().getValueAt(JTLls.getSelectedRow(),0));
                Llavero ll_new = new Llavero ((Integer)JTLls.getModel().getValueAt(JTLls.getSelectedRow(),0));

                // Obtenemos las llaves
                int[] llavesSelecs = JTLs.getSelectedRows();

                // Eliminamos cada llave del llavero
                Llave l;
                for (int i=0; i<llavesSelecs.length; i++)
                {
                    // Obtenemos la llave
                    l = new Llave((Integer)JTLs.getModel().getValueAt(llavesSelecs[i],0));
                    // Eliminamos la llave
                    ll_new.removeLlave(l);
                }

                // Actualizamos el llavero viejo por el nuevo
                ll_old.updateLlaveroBD(ll_new);

                // Refrescamos las tablas de los almacenes
                refrescarAlmacenes();

                // Des-seleccionamos las llaves en JTLsZona y el boton de modificar llavero, escondiendo ademas el paneld e datos
                JTLsZona.clearSelection();
                JTBModificarLl.setSelected(false);
                JPLlaveroD.setVisible(false);

                // Advertimos de que las llaves han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las llaves han sido eliminadas correctamente del llavero.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las llaves del llavero.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarLsLlActionPerformed

    private void JTTarjsPrestMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTTarjsPrestMouseClicked
        try
        {
            // Actualizamos el DNI del prestatario y mostramos sus datos personales
            Tarjeta tarj = new Tarjeta((Integer) JTTarjsPrest.getModel().getValueAt(JTTarjsPrest.getSelectedRow(),0));
            DNI_Prest = tarj.getPersona().getDNI();
            mostrarDatosPrestatario(tarj.getPersona());
            tarj.UBD.cerrarConexion();
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos personales del prestatario.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTTarjsPrestMouseClicked

    private void JTLsPrestMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsPrestMouseClicked
        try
        {
            // Actualizamos el DNI del prestatrio y mostramos sus datos personales
            String Entrada = new SimpleDateFormat("yyyy-MM-dd - HH:mm:ss").format(JTLsPrest.getModel().getValueAt(JTLsPrest.getSelectedRow(),1));
            Acceso acc = new Acceso(Entrada);
            DNI_Prest = acc.getPersona().getDNI();
            mostrarDatosPrestatario(acc.getPersona());
            acc.UBD.cerrarConexion();
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos personales del prestatario.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTLsPrestMouseClicked

    private void JTLlsPrestMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLlsPrestMouseClicked
        try
        {
            // Actualizamos el DNI del prestatrio y rellenamos sus datos personales
            String Entrada = new SimpleDateFormat("yyyy-MM-dd - HH:mm:ss").format(JTLlsPrest.getModel().getValueAt(JTLlsPrest.getSelectedRow(),1));
            Acceso acc = new Acceso(Entrada);
            DNI_Prest = acc.getPersona().getDNI();
            mostrarDatosPrestatario(acc.getPersona());
            acc.UBD.cerrarConexion();
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos personales del prestatario.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTLlsPrestMouseClicked

    private void JBRecordPrestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBRecordPrestActionPerformed
        try
        {
            // Avisamos de que vamos a enviar el recordatorio
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a enviar el recordatorio de préstamos pendientes.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Enviamos el recordatorio
            if (seleccion == 0)
            {
                // Creamos un objeto de tipo Cartero
                Cartero car = new Cartero("smtp.gmail.com","true","587","informaticanavarrasa@gmail.com","true","informaticanavarrasa@gmail.com","informaticanavarra");

                // Enviamos e-mails a la ultima persona seleccionada, recordandole todos los elementos que tiene pendiente de devolucion
                car.enviarCorreosPrestamosPendientes(DNI_Prest);
                
                // Advertimos de que el recordatorio ha sido enviado correctamente
                JOptionPane.showMessageDialog(this,"El recordatorio ha sido enviado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible enviar los correos de recordatorio al prestatario.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBRecordPrestActionPerformed

    private void JTTAccesosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTTAccesosMouseClicked
        try
        {
            // Siempre que 'JTBAnadirTAcceso' no este seleccionado -> Rellenamos y activamos botones
            if (!JTBAnadirTAcceso.isSelected())
            {
                // Mostramos los datos del tipo de acceso
                T_Acceso ta = new T_Acceso((String) JTTAccesos.getModel().getValueAt(JTTAccesos.getSelectedRow(),0));
                JTFCodTAcceso.setText(ta.getCodAcc());
                JTFActivTAcceso.setText(ta.getActividad());
                JTFDiasTAcceso.setText(ta.getDias());
                JCBHITAccesos.setSelectedItem(ta.timeToString(ta.getFechaI()).substring(0,5));
                JCBHFTAccesos.setSelectedItem(ta.timeToString(ta.getFechaJ()).substring(0,5));
                JCBAutTelTAccesos.setSelected(ta.getAutTel());
                JCBNivelTAcceso.getModel().setSelectedItem(ta.getNivel() == null ? "Ninguno" : ta.getNivel().getCodNiv());

                // Activamos los botones de 'Eliminar tipo de acceso' y 'Modificar tipo de acceso'
                JBEliminarTAcceso.setEnabled(true);
                JTBModificarTAcceso.setEnabled(true);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos del tipo de acceso.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTTAccesosMouseClicked

    private void JTTAccesosCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTTAccesosCaretPositionChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JTTAccesosCaretPositionChanged

    private void JTTAccesosInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTTAccesosInputMethodTextChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JTTAccesosInputMethodTextChanged

    private void JTTAccesosKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTTAccesosKeyReleased
        // TODO add your handling code here:
    }//GEN-LAST:event_JTTAccesosKeyReleased

    private void JBGrabarNivel1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarNivel1ActionPerformed
        try
        {
            // Creamos el tipo de acceso nuevo
            T_Acceso ta_new = new T_Acceso();
            
            Calendar Hora_Inic = ta_new.stringToTime((String) JCBHITAccesos.getSelectedItem() + ":00");
            Calendar Hora_Fin = ta_new.stringToTime((String) JCBHFTAccesos.getSelectedItem() + ":00");
            Hora_Fin.set(Hora_Fin.YEAR, Hora_Fin.get(Hora_Fin.YEAR)+1);

            String nivel = (String) JCBNivelTAcceso.getModel().getSelectedItem();
            Nivel niv = null;
            if (!nivel.equals("Ninguno"))   niv = new Nivel(Integer.parseInt(nivel));
            ta_new = new T_Acceso(JTFCodTAcceso.getText(),JTFActivTAcceso.getText(),JTFDiasTAcceso.getText(),Hora_Inic,Hora_Fin,
                    JCBAutTelTAccesos.isSelected(),niv);

            // Comprobamos si se trata de una insercion o una modificacion de tipo de acceso
            // -> Insercion
            if (JTBAnadirTAcceso.isSelected())
            {
                // Avisamos de que vamos a dar de alta el tipo de acceso
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta el tipo de acceso.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta el tipo de acceso
                if (seleccion == 0)
                {
                    // Insertamos el tipo de acceso
                    ta_new.insertTAccesoBD();

                    // Refrescamos la tabla de tipos de acceso
                    refrescarTAccesos();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que el tipo de acceso ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El tipo de acceso ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // ->Modificacion
            else
            {
                // Avisamos de que vamos a modificar el tipo de acceso
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el tipo de acceso seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el tipo de acceso seleccionado
                if (seleccion == 0)
                {
                    // Creamos el tipo de acceso viejo
                    T_Acceso ta_old = new T_Acceso((String) JTTAccesos.getModel().getValueAt(JTTAccesos.getSelectedRow(),0));

                    // Modificamos el tipo de acceso viejo con el nuevo
                    ta_old.updateTAccesoBD(ta_new);

                    // Refrescamos la tabla de tipos de acceso
                    refrescarTAccesos();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que el tipo de acceso ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El tipo de acceso ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el tipo de acceso.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarNivel1ActionPerformed

    private void JBEliminarTAccesoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarTAccesoActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los tipos de acceso seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los tipos de acceso seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los tipos de acceso
            if (seleccion == 0)
            {
                // Obtenemos los tipos de acceso seleccionados
                int[] taccsSelec = JTTAccesos.getSelectedRows();

                // Eliminamos cada tipo de acceso
                T_Acceso ta;
                for (int i=0; i<taccsSelec.length; i++)
                {
                    // Creamos el tipo de acceso
                    ta = new T_Acceso((String) JTTAccesos.getModel().getValueAt(taccsSelec[i],0));
                    // Borramos el tipo de acceso
                    ta.removeTAccesoBD();
                }

                // Refrecamos las tablas de niveles y tipos de accesos
                refrescarNiveles();
                refrescarTAccesos();

                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();

                // Advertimos de que los tipos de acceso han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los tipos de acceso seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los tipos de acceso seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarTAccesoActionPerformed

    private void JTBAnadirTAccesoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirTAccesoActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirTAcceso.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Establecemos el boton de 'Añadir tipo de acceso' a seleccionado
                JTBAnadirTAcceso.setSelected(true);
                // Escondemos los paneles de niveles, tarjetas, colores y cargos
                JPNivelD.setVisible(false);
                JPTarjetasD.setVisible(false);
                JPColorD.setVisible(false);
                JPCargoD.setVisible(false);

                // Preparamos el formulario de introduccion del nuevo tipo de acceso
                JTFCodTAcceso.setText("");
                JTFActivTAcceso.setText("");
                JTFDiasTAcceso.setText("");
                JCBHITAccesos.setSelectedIndex(0);
                JCBHFTAccesos.setSelectedIndex(0);
                JCBAutTelTAccesos.setSelected(false);
                JCBNivelTAcceso.setSelectedIndex(0);

                // Mostramos el panel
                JPTAccesoD.setVisible(true);
                JCBNivelTAcceso.setSelectedIndex(0);
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar el panel de inserción de tipo de acceso.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // Escondemos el JPanel de insercion de nivel
        else        JPTAccesoD.setVisible(false);
    }//GEN-LAST:event_JTBAnadirTAccesoActionPerformed

    private void JTBModificarTAccesoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarTAccesoActionPerformed
        try
        {
            // Si el boton esta pulsado
            if (JTBModificarTAcceso.isSelected())
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Activamos los botones de 'Eliminar tipo de acceso' y 'Modificar tipo de acceso'
                JBEliminarTAcceso.setEnabled(true);
                JTBModificarTAcceso.setSelected(true);
                JTBModificarTAcceso.setEnabled(true);
                // Establecemos el boton de 'Añadir tipo de acceso' a no seleccionado
                JTBAnadirTAcceso.setSelected(false);
                // Escondemos los paneles de tarjetas, colores, niveles y cargos
                JPTarjetasD.setVisible(false);
                JPColorD.setVisible(false);
                JPNivelD.setVisible(false);
                JPCargoD.setVisible(false);

                // Mostramos el panel de datos
                JPTAccesoD.setVisible(true);
            }
            // Escondemos el JPanel de insercion de tipo de acceso en caso contrario
            else        JPTAccesoD.setVisible(false);
        }
        catch(Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos del tipo de acceso.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTBModificarTAccesoActionPerformed

    private void JTFBuscarAccesosSistKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarAccesosSistKeyReleased
        // Refrescamos la tablas de accesos al sistema
        refrescarTAccsSist();
    }//GEN-LAST:event_JTFBuscarAccesosSistKeyReleased

    private void JCCFechaInicAccsSistMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JCCFechaInicAccsSistMouseClicked
        // TODO add your handling code here:
    }//GEN-LAST:event_JCCFechaInicAccsSistMouseClicked

    private void JCCFechaInicAccsSistDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFechaInicAccsSistDateChanged
        // Refrescamos la tabla de accesos al sistema
        refrescarTAccsSist();
    }//GEN-LAST:event_JCCFechaInicAccsSistDateChanged

    private void JCCFechaInicAccsSistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCCFechaInicAccsSistActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCCFechaInicAccsSistActionPerformed

    private void JCCFechaFinAccsSistDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFechaFinAccsSistDateChanged
        // Refrescamos la tabla de accesos al sistema
        refrescarTAccsSist();
    }//GEN-LAST:event_JCCFechaFinAccsSistDateChanged

    private void JCCFechaFinAccsSistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCCFechaFinAccsSistActionPerformed
        // Refrescamos la tabla de accesos al sistema
        refrescarTAccsSist();
    }//GEN-LAST:event_JCCFechaFinAccsSistActionPerformed

    private void JCBHoraFinAccsSistItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHoraFinAccsSistItemStateChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBHoraFinAccsSistItemStateChanged

    private void JCBHoraFinAccsSistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHoraFinAccsSistActionPerformed
        // Refrescamos la tabla de accesos al sistema
        refrescarTAccsSist();
    }//GEN-LAST:event_JCBHoraFinAccsSistActionPerformed

    private void JCBHoraInicAccsSistItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHoraInicAccsSistItemStateChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBHoraInicAccsSistItemStateChanged

    private void JCBHoraInicAccsSistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHoraInicAccsSistActionPerformed
        // Refrescamos la tabla de accesos al sistema
        refrescarTAccsSist();
    }//GEN-LAST:event_JCBHoraInicAccsSistActionPerformed

    private void JCBTodoDiaAccsSistStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_JCBTodoDiaAccsSistStateChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBTodoDiaAccsSistStateChanged

    private void JCBTodoDiaAccsSistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBTodoDiaAccsSistActionPerformed
        // Activamos/descativamos los JComboBox que delimitan las horas
        if (JCBHoraFinAccsSist.isEnabled())
        {
            JCBHoraInicAccsSist.setEnabled(false);
            JCBHoraFinAccsSist.setEnabled(false);
        }
        else
        {
            JCBHoraInicAccsSist.setEnabled(true);
            JCBHoraFinAccsSist.setEnabled(true);
        }
        // Refrescamos la JTable de accesos al sistema
        this.refrescarTAccsSist();
    }//GEN-LAST:event_JCBTodoDiaAccsSistActionPerformed

    private void JBPDFAccsSistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBPDFAccsSistActionPerformed
        try
        {
            // Desplegamos un JFileChooser en modo 'Guardar'
            JFileChooser jfc = new JFileChooser();
            int opcion = jfc.showSaveDialog(this);

            // Si se pulsa 'Aceptar'
            if (opcion == jfc.APPROVE_OPTION)
            {
                // Recogemos el fichero seleccionado
                File fichero = jfc.getSelectedFile();
                String Ruta = fichero.getAbsolutePath();

                // Creamos un objeto de tipo GeneradorInformes
                GeneradorInformes gi = new GeneradorInformes(Ruta);
                // Generamos el informe
                gi.generarInforme("ACCESOS AL SISTEMA","Historial de accesos al sistema:",JTHistorialAccesosSist);

                // Advertimos de que el informe ha sido generado correctamente
                JOptionPane.showMessageDialog(this,"Reporte generado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible realizar el reporte solicitado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBPDFAccsSistActionPerformed

    private void JTFBuscarContacMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFBuscarContacMouseClicked
        // Eliminamos 'Buscar...' y establecemos el color de texto a negor
        JTFBuscarContac.setText("");
        JTFBuscarContac.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFBuscarContacMouseClicked

    private void JTFBuscarContacKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTFBuscarContacKeyReleased
        // Refrescamos los contactos de la JTable
        refrescarContactos();
    }//GEN-LAST:event_JTFBuscarContacKeyReleased

    private void JTContactosNotasMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTContactosNotasMouseClicked
    }//GEN-LAST:event_JTContactosNotasMouseClicked

    private void JTFBuscarAccesosSistMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFBuscarAccesosSistMouseClicked
        // Eliminamos 'Buscar...' y establecemos el color de texto a negor
        JTFBuscarAccesosSist.setText("");
        JTFBuscarAccesosSist.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFBuscarAccesosSistMouseClicked

    private void JTFBuscarCuentaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFBuscarCuentaMouseClicked
        // Eliminamos 'Buscar...' y establecemos el color de texto a negor
        JTFBuscarCuenta.setText("");
        JTFBuscarCuenta.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFBuscarCuentaMouseClicked

    private void JBPDFCuentasActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBPDFCuentasActionPerformed
        try
        {
            // Desplegamos un JFileChooser en modo 'Guardar'
            JFileChooser jfc = new JFileChooser();
            int opcion = jfc.showSaveDialog(this);

            // Si se pulsa 'Aceptar'
            if (opcion == jfc.APPROVE_OPTION)
            {
                // Recogemos el fichero seleccionado
                File fichero = jfc.getSelectedFile();
                String Ruta = fichero.getAbsolutePath();

                // Creamos un objeto de tipo GeneradorInformes
                GeneradorInformes gi = new GeneradorInformes(Ruta);
                // Generamos el informe
                gi.generarInforme("CUENTAS","Cuentas existentes en el sistema:",JTCuentas);

                // Advertimos de que el informe ha sido generado correctamente
                JOptionPane.showMessageDialog(this,"Reporte generado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible realizar el reporte solicitado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBPDFCuentasActionPerformed

    private void JTBAnadirCuentaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirCuentaActionPerformed
        // Si JTBAnadirCuenta esta seleccionado
        if (JTBAnadirCuenta.isSelected())
        {
            // Borramos los campos de usuario y contraseña
            JTFUsuarioCuenta.setText("");
            JTFPassCuenta.setText("");

            // Seleccionamos los primeros elementos de ambos JComboBoxes
            JCBDNICuenta.setSelectedIndex(0);
            JCBTipoCuenta.setSelectedIndex(0);

            // Mostramos el JPanel de introduccion de datos
            JPDatosCuenta.setVisible(true);

            // Desactivamos los botones de 'Eliminar cuenta' y Modificar cuenta'
            JBEliminarCuenta.setEnabled(false);
            JTBModificarCuenta.setSelected(false);
            JTBModificarCuenta.setEnabled(false);

            // Activamos el buscador
            JTFBuscarCuenta.setEditable(true);
        }
        // Si no lo esta
        else
        {
            // Escondemos el panel de datos
            JPDatosCuenta.setVisible(false);
        }

        // Deseleccionamos cualquier fila
        JTCuentas.clearSelection();
    }//GEN-LAST:event_JTBAnadirCuentaActionPerformed

    private void JTBAnadirTUsuarioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirTUsuarioActionPerformed
        // Si JTBAnadirTUsuario esta seleccionado
        if (JTBAnadirTUsuario.isSelected())
        {
            // Borramos los campos de tipo y descripcion
            JTFTUsuario.setText("");
            JTFDescripTUsuario.setText("");

            // Mostramos el JPanel de introduccion de datos
            JPDatosTUsuario.setVisible(true);

            // Desactivamos los botones de 'Eliminar tipo de usuario' y Modificar tipo de usuario'
            JBEliminarTUsuario.setEnabled(false);
            JTBModificarTUsuario.setSelected(false);
            JTBModificarTUsuario.setEnabled(false);
        }
        // Si no lo esta
        else
        {
            // Escondemos el panel de datos
            JPDatosTUsuario.setVisible(false);
        }

        // Deseleccionamos cualquier fila
        JTTUsuarios.clearSelection();
    }//GEN-LAST:event_JTBAnadirTUsuarioActionPerformed

    private void JTAIncidenciasKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTAIncidenciasKeyReleased
    }//GEN-LAST:event_JTAIncidenciasKeyReleased

    private void JCCFechaParteDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFechaParteDateChanged
    }//GEN-LAST:event_JCCFechaParteDateChanged

    private void JCCFechaParteMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JCCFechaParteMouseReleased

    }//GEN-LAST:event_JCCFechaParteMouseReleased

    private void JCBParteMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JCBParteMousePressed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBParteMousePressed

    private void JCBParteMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JCBParteMouseReleased

    }//GEN-LAST:event_JCBParteMouseReleased

    private void JBPDFParteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBPDFParteActionPerformed
        try
        {
            // Desplegamos un JFileChooser en modo 'Guardar'
            JFileChooser jfc = new JFileChooser();
            int opcion = jfc.showSaveDialog(this);

            // Si se pulsa 'Aceptar'
            if (opcion == jfc.APPROVE_OPTION)
            {
                // Recogemos el fichero seleccionado
                File fichero = jfc.getSelectedFile();
                String Ruta = fichero.getAbsolutePath();
                // Creamos un objeto de tipo GeneradorInformes
                GeneradorInformes gi = new GeneradorInformes(Ruta);
                // Generamos el reporte
                JPPDFB.setVisible(false);
                gi.generarParte(JPParte);
                JPPDFB.setVisible(true);
                // Advertimos de que el informe ha sido generado correctamente
                JOptionPane.showMessageDialog(this,"Reporte generado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible realizar el reporte solicitado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBPDFParteActionPerformed

    private void JTBActivarPanelPDFActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBActivarPanelPDFActionPerformed
        // Si 'JTBActivarPanelPDF' esta pulsado
        if (JTBActivarPanelPDF.isSelected())
        {
            // Inicializamos el panel
            JCBPDFTodas.setSelected(false);
            JCBPDFTodas.setEnabled(true);
            JCBPDFTarjs.setSelected(false);
            JCBPDFTarjs.setEnabled(true);
            JCBPDFLs.setSelected(false);
            JCBPDFLs.setEnabled(true);
            JCBPDFLls.setSelected(false);
            JCBPDFLls.setEnabled(true);

            // Mostramos el panel de generar reportes
            JPPrestsPDF.setVisible(true);
        }
        // En caso contrario -> Lo escondemos
        else
        {
            JPPrestsPDF.setVisible(false);
        }
    }//GEN-LAST:event_JTBActivarPanelPDFActionPerformed

    private void JBPrestsPDFActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBPrestsPDFActionPerformed
        try
        {
            // Si existe algun JCheckBox seleccionado
            if (JCBPDFTodas.isSelected() || JCBPDFTarjs.isSelected() || JCBPDFLs.isSelected() || JCBPDFLls.isSelected())
            {
                // Desplegamos un JFileChooser en modo 'Guardar'
                JFileChooser jfc = new JFileChooser();
                int opcion = jfc.showSaveDialog(this);

                // Si se pulsa 'Aceptar'
                if (opcion == jfc.APPROVE_OPTION)
                {
                    // Recogemos el fichero seleccionado
                    File fichero = jfc.getSelectedFile();
                    String Ruta = fichero.getAbsolutePath();

                    // Creamos un objeto de tipo GeneradorInformes
                    GeneradorInformes gi = new GeneradorInformes(Ruta);
                    // Generamos el reporte
                    gi.generarInformeAvanzado("PRÉSTAMOS PENDIENTES","Préstamos pendientes de devolución:","Tarjetas","Llaves","Llaveros",JTTarjsPrest,JTLsPrest,JTLlsPrest,JCBPDFTodas.isSelected(),JCBPDFTarjs.isSelected(),JCBPDFLs.isSelected(),JCBPDFLls.isSelected());

                    // Des-seleccionamos el boton de generar PDF
                    JTBActivarPanelPDF.setSelected(false);

                    // Cerramos el panel de PDF
                    JPPrestsPDF.setVisible(false);

                    // Advertimos de que el informe ha sido generado correctamente
                    JOptionPane.showMessageDialog(this,"Reporte generado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // Si no hemos seleccionado ningun elemento, avisamos del error
            else    JOptionPane.showMessageDialog(this,"No se ha seleccionado ningún elemento a reportar.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible realizar el reporte solicitado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBPrestsPDFActionPerformed

    private void JCBPDFLlsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFLlsActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBPDFLlsActionPerformed

    private void JCBPDFTodasActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFTodasActionPerformed
        // Si esta seleccionado -> Desactivamos el resto de JCheckBoxes
        if (JCBPDFTodas.isSelected())
        {
            JCBPDFTarjs.setEnabled(false);
            JCBPDFLs.setEnabled(false);
            JCBPDFLls.setEnabled(false);
        }
        // En caso contrario -> Habilitamos el resto de JCheckBoxes
        else
        {
            JCBPDFTarjs.setEnabled(true);
            JCBPDFLs.setEnabled(true);
            JCBPDFLls.setEnabled(true);
        }
    }//GEN-LAST:event_JCBPDFTodasActionPerformed

    private void JTFBuscarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTFBuscarActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JTFBuscarActionPerformed

    private void JTFBuscarMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFBuscarMouseClicked
        // Eliminamos 'Buscar...' y establecemos el color de texto a negor
        JTFBuscar.setText("");
        JTFBuscar.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFBuscarMouseClicked

    private void JBPDFAccsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBPDFAccsActionPerformed
        try
        {
            // Desplegamos un JFileChooser en modo 'Guardar'
            JFileChooser jfc = new JFileChooser();
            int opcion = jfc.showSaveDialog(this);

            // Si se pulsa 'Aceptar'
            if (opcion == jfc.APPROVE_OPTION)
            {
                // Recogemos el fichero seleccionado
                String Ruta = jfc.getSelectedFile().getAbsolutePath();

                // Creamos un objeto de tipo GeneradorInformes
                GeneradorInformes gi = new GeneradorInformes(Ruta);
                // Generamos el informe
                gi.generarInforme("ACCESOS","Accesos realizados:",JTAccesos);

                // Advertimos de que el informe ha sido generado correctamente
                JOptionPane.showMessageDialog(this,"Reporte generado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible realizar el reporte solicitado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBPDFAccsActionPerformed

    private void JTCargossMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTCargossMouseClicked
        try
        {
            // Siempre que 'JTBAnadirCargo' no este seleccionado -> Rellenamos y activamos botones
            if (!JTBAnadirCargo.isSelected())
            {
                // Rellenamos los campos de codigo y descripcion
                JTFCodCargo.setText((String) JTCargoss.getModel().getValueAt(JTCargoss.getSelectedRow(),0));
                JTFDescripCargo.setText((String) JTCargoss.getModel().getValueAt(JTCargoss.getSelectedRow(),1));

                // Activamos los botones de 'Eliminar cargo' y 'Modificar cargo'
                JBEliminarCargo.setEnabled(true);
                JTBModificarCargo.setEnabled(true);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos del cargo.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTCargossMouseClicked

    private void JTCargossCaretPositionChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTCargossCaretPositionChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JTCargossCaretPositionChanged

    private void JTCargossInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_JTCargossInputMethodTextChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JTCargossInputMethodTextChanged

    private void JTCargossKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_JTCargossKeyReleased
        // TODO add your handling code here:
    }//GEN-LAST:event_JTCargossKeyReleased

    private void JBGrabarCargoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBGrabarCargoActionPerformed
        try
        {
            // Creamos el cargo nuevo
            Cargo car_new = new Cargo(JTFCodCargo.getText(),JTFDescripCargo.getText());

            // Comprobamos si se trata de una insercion o una modificacion de cargo
            // -> Insercion
            if (JTBAnadirCargo.isSelected())
            {
                // Avisamos de que vamos a dar de alta el cargo
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta el cargo.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta el cargo
                if (seleccion == 0)
                {
                    // Insertamos el cargo
                    car_new.insertCargoBD();

                    // Refrescamos la tabla de cargos
                    refrescarTCargos();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que el cargo ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El cargo ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // ->Modificacion
            else
            {
                // Avisamos de que vamos a modificar el cargo
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el cargo seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el cargo seleccionado
                if (seleccion == 0)
                {
                    // Creamos el cargo viejo
                    Cargo car_old = new Cargo((String) JTCargoss.getModel().getValueAt(JTCargoss.getSelectedRow(),0));

                    // Modificamos el cargo viejo con el nuevo
                    car_old.updateCargoBD(car_new);

                    // Refrescamos las tabla de cargos
                    refrescarTCargos();

                    // Activamos/Desactivamos todos los botones
                    activDesactivBotonesTACs();

                    // Advertimos de que el cargo ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El cargo ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el cargo.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBGrabarCargoActionPerformed

    private void JBEliminarCargoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarCargoActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar los cargos seleccionados
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar los cargos seleccionados.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos los cargos
            if (seleccion == 0)
            {
                // Obtenemos los cargos seleccionados
                int[] cargosSelec = JTCargoss.getSelectedRows();

                // Eliminamos cada cargo
                Cargo car;
                for (int i=0; i<cargosSelec.length; i++)
                {
                    // Creamos el cargo
                    car = new Cargo((String) JTCargoss.getModel().getValueAt(cargosSelec[i],0));
                    // Borramos el cargo
                    car.removeCargoBD();
                }

                // Refrecamos la tabla de cargos
                refrescarTCargos();

                // Advertimos de que los cargos han sido eliminados correctamente
                JOptionPane.showMessageDialog(this,"Los cargos seleccionados han sido eliminados correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
                JOptionPane.showMessageDialog(this,"No ha sido posible eliminar los cargos seleccionados.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarCargoActionPerformed

    private void JTBAnadirCargoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirCargoActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirCargo.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Establecemos el boton de 'Añadir cargo' a seleccionado
                JTBAnadirCargo.setSelected(true);
                // Escondemos los paneles de tarjetas, colores, niveles y tipos de acceso
                JPTarjetasD.setVisible(false);
                JPColorD.setVisible(false);
                JPNivelD.setVisible(false);
                JPTAccesoD.setVisible(false);

                // Borramos el formulario
                JTFCodCargo.setText("");
                JTFDescripCargo.setText("");

                // Mostramos el panel
                JPCargoD.setVisible(true);
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar el panel de inserción de cargo.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // Escondemos el JPanel de insercion de cargo y deseleccionamos los cargos que pudieran estar
        // seleccionados en la tabla, dejandola "intacta"
        else
        {
            JPCargoD.setVisible(false);
            JTCargoss.clearSelection();
        }
    }//GEN-LAST:event_JTBAnadirCargoActionPerformed

    private void JTBModificarCargoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarCargoActionPerformed
        // Si el boton esta pulsado
        if (JTBModificarCargo.isSelected())
        {
            try
            {
                // Activamos/Desactivamos todos los botones
                activDesactivBotonesTACs();
                // Activamos los botones de 'Eliminar cargo' y 'Modificar cargo'
                JBEliminarCargo.setEnabled(true);
                JTBModificarCargo.setSelected(true);
                JTBModificarCargo.setEnabled(true);
                // Establecemos el boton de 'Añadir cargo' a no seleccionado
                JTBAnadirCargo.setSelected(false);

                // Escondemos los paneles de tarjetas, colores, niveles y tipos de accesos
                JPTarjetasD.setVisible(false);
                JPColorD.setVisible(false);
                JPNivelD.setVisible(false);
                JPTAccesoD.setVisible(false);

                // Mostramos el panel de datos
                JPCargoD.setVisible(true);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar el código y descripción del cargo.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // Escondemos el JPanel de insercion de cargo en caso contrario
        else        JPCargoD.setVisible(false);
    }//GEN-LAST:event_JTBModificarCargoActionPerformed

    private void JCBHITAccesosItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHITAccesosItemStateChanged
    }//GEN-LAST:event_JCBHITAccesosItemStateChanged

    private void JCBHFTAccesosItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBHFTAccesosItemStateChanged
    }//GEN-LAST:event_JCBHFTAccesosItemStateChanged

    private void JCBHFTAccesosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHFTAccesosActionPerformed
        // Comprobamos la correcion de las horas introducidas
        comprobarHorasTAcceso();
    }//GEN-LAST:event_JCBHFTAccesosActionPerformed

    private void JCBHITAccesosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBHITAccesosActionPerformed
        // Comprobamos la correcion de las horas introducidas
        comprobarHorasTAcceso();
    }//GEN-LAST:event_JCBHITAccesosActionPerformed

    private void JBDeseleccionarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBDeseleccionarActionPerformed
        // Deseleccionamos cualquier fila de cualquier tabla
        JTTarjsPrest.clearSelection();
        JTLsPrest.clearSelection();
        JTLlsPrest.clearSelection();

        // Escondemos el panel de datos
        JPDatosPrest.setVisible(false);
    }//GEN-LAST:event_JBDeseleccionarActionPerformed

    private void JBAnadirPersonaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JBAnadirPersonaMouseClicked
        // TODO add your handling code here:
    }//GEN-LAST:event_JBAnadirPersonaMouseClicked

    private void JBAnadirPersonaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirPersonaActionPerformed
        // Descativamos cualquier fila seleccionada en el JTable de accesos
        JTAccesos.clearSelection();

        // Mostramos la informacion del acceso
        mostrarAcceso();
    }//GEN-LAST:event_JBAnadirPersonaActionPerformed

    private void JTPFuncionalidadesMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTPFuncionalidadesMouseClicked
        // TODO add your handling code here:

    }//GEN-LAST:event_JTPFuncionalidadesMouseClicked

    private void JPAccesosMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JPAccesosMouseClicked
    }//GEN-LAST:event_JPAccesosMouseClicked

    private void JPAccesosComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPAccesosComponentShown
        // Iniciamos todo el contenido del tab 'Accesos'
        iniciarAccesos();
    }//GEN-LAST:event_JPAccesosComponentShown

    private void JPTACsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JPTACsMouseClicked
        // TODO add your handling code here:
    }//GEN-LAST:event_JPTACsMouseClicked

    private void JPTACsComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPTACsComponentShown
        // Rellenamos las tablas de tarjetas, colores, niveles, tipos de accesos y cargos
        iniciarTACs();
    }//GEN-LAST:event_JPTACsComponentShown

    private void JPPrestamosComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPPrestamosComponentShown
        // Iniciamos todo el contenido del tab 'Prestamos'
        iniciarPrestamos();
    }//GEN-LAST:event_JPPrestamosComponentShown

    private void JPCuentasComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPCuentasComponentShown
        // Iniciamos todo el contenido del tab 'Cuentas'
        iniciarCuentas();
    }//GEN-LAST:event_JPCuentasComponentShown

    private void JPNotasComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPNotasComponentShown
        // Iniciamos todo el contenido del tab 'Notas'
        iniciarNotas();
    }//GEN-LAST:event_JPNotasComponentShown

    private void JPPartesComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPPartesComponentShown
        // Iniciamos todo el contenido del tab 'Partes de seguridad'
        iniciarPartes();
    }//GEN-LAST:event_JPPartesComponentShown

    private void JTFNumCopsLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTFNumCopsLActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JTFNumCopsLActionPerformed

    private void JTFBuscarLLlMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFBuscarLLlMouseClicked
        // Eliminamos 'Buscar...' y establecemos el color de texto a negor
        JTFBuscarLLl.setText("");
        JTFBuscarLLl.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFBuscarLLlMouseClicked

    private void JBPrestsPDF1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBPrestsPDF1ActionPerformed
        try
        {
            // Si existe algun JCheckBox seleccionado
            if (JCBPDFTodas1.isSelected() || JCBPDFAlms1.isSelected() || JCBPDFLs1.isSelected() || JCBPDFLls1.isSelected())
            {
                // Desplegamos un JFileChooser en modo 'Guardar'
                JFileChooser jfc = new JFileChooser();
                int opcion = jfc.showSaveDialog(this);

                // Si se pulsa 'Aceptar'
                if (opcion == jfc.APPROVE_OPTION)
                {
                    // Recogemos el fichero seleccionado
                    File fichero = jfc.getSelectedFile();
                    String Ruta = fichero.getAbsolutePath();

                    // Creamos un objeto de tipo GeneradorInformes
                    GeneradorInformes gi = new GeneradorInformes(Ruta);
                    // Generamos el reporte
                    gi.generarInformeAvanzado("ALMACENES DE LLAVES","Contenido de los almacenes:","Almacenes","Llaves","Llaveros",JTAlms,JTLs,JTLls,JCBPDFTodas1.isSelected(),JCBPDFAlms1.isSelected(),JCBPDFLs1.isSelected(),JCBPDFLls1.isSelected());

                    // Des-seleccionamos el boton de generar PDF
                    JTBActivarPanelPDF1.setSelected(false);

                    // Cerramos el panel de PDF
                    JPPrestsPDF1.setVisible(false);

                    // Advertimos de que el informe ha sido generado correctamente
                    JOptionPane.showMessageDialog(this,"Reporte generado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // Si no hemos seleccionado ningun elemento, avisamos del error
            else    JOptionPane.showMessageDialog(this,"No se ha seleccionado ningún elemento a reportar.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible realizar el reporte solicitado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBPrestsPDF1ActionPerformed

    private void JCBPDFTodas1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFTodas1ActionPerformed
        // Si esta seleccionado -> Desactivamos el resto de JCheckBoxes
        if (JCBPDFTodas1.isSelected())
        {
            JCBPDFAlms1.setEnabled(false);
            JCBPDFLs1.setEnabled(false);
            JCBPDFLls1.setEnabled(false);
        }
        // En caso contrario -> Habilitamos el resto de JCheckBoxes
        else
        {
            JCBPDFAlms1.setEnabled(true);
            JCBPDFLs1.setEnabled(true);
            JCBPDFLls1.setEnabled(true);
        }
    }//GEN-LAST:event_JCBPDFTodas1ActionPerformed

    private void JCBPDFLls1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFLls1ActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBPDFLls1ActionPerformed

    private void JTBActivarPanelPDF1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBActivarPanelPDF1ActionPerformed
        // Si 'JTBActivarPanelPDF1' esta pulsado
        if (JTBActivarPanelPDF1.isSelected())
        {
            // Inicializamos el panel
            JCBPDFTodas1.setSelected(false);
            JCBPDFTodas1.setEnabled(true);
            JCBPDFAlms1.setSelected(false);
            JCBPDFAlms1.setEnabled(true);
            JCBPDFLs1.setSelected(false);
            JCBPDFLs1.setEnabled(true);
            JCBPDFLls1.setSelected(false);
            JCBPDFLls1.setEnabled(true);

            // Mostramos el panel de generar reportes
            JPPrestsPDF1.setVisible(true);
        }
        // En caso contrario -> Lo escondemos
        else
        {
            JPPrestsPDF1.setVisible(false);
        }
    }//GEN-LAST:event_JTBActivarPanelPDF1ActionPerformed

    private void JCBPDFTarjsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFTarjsActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBPDFTarjsActionPerformed

    private void JCBPDFLsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFLsActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBPDFLsActionPerformed

    private void JPZsLsLlsComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPZsLsLlsComponentShown
        // Rellenamos las tablas de zonas, llaves y llaveros
        iniciarZonas();
    }//GEN-LAST:event_JPZsLsLlsComponentShown

    private void JTBModificarEmpresaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarEmpresaActionPerformed
        // Si JTBModificarEmpresa esta seleccionado
        if (JTBModificarEmpresa.isSelected())
        {
            // Mostramos el JPanel de introduccion de datos
            JPEmpresasD.setVisible(true);

            // Escondemos los paneles de insercion de departamentos y personal
            JPDepartamentosD.setVisible(false);
            mostrarJPanelsPersonal("");

            // Des-seleccionamos el boton de 'Añadir empresa'
            JTBAnadirEmpresa.setSelected(false);
        }
        // Si no lo esta -> Escondemos el panel de datos
        else    JPEmpresasD.setVisible(false);
    }//GEN-LAST:event_JTBModificarEmpresaActionPerformed

    private void JTBAnadirDepartamentoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBAnadirDepartamentoActionPerformed
        // Si el boton esta pulsado
        if (JTBAnadirDepartamento.isSelected())
        {
            try
            {
                // Borramos todos los campos
                borrarDatosDep();

                // Hacemos visible el panel
                JPDepartamentosD.setVisible(true);

                // Escondemos los paneles de insercion de empresas y personal
                JPEmpresasD.setVisible(false);
                mostrarJPanelsPersonal("");

                // Desactivamos los botones de 'Eliminar departamento', 'Modificar departamento' y añadir llaves
                JBEliminarDepartamento.setEnabled(false);
                JTBModificarDepartamento.setSelected(false);
                JTBModificarDepartamento.setEnabled(false);
                JBAnadirPrestDep.setEnabled(false);

                // Deseleccionamos cualquier fila
                JTDepartamentos.clearSelection();

                // Borramos la tabla de llaves
                String[] Etiquetas = {"Código","Zona","Descripción"};
                JTLsLlsPers.setModel(new DefaultTableModel(Etiquetas,0));
            }
            catch(Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido mostrar el formulario de inseción de departamentos.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // En caso contrario -> Escondemos el JPanel de insercion de departamento
        else        JPDepartamentosD.setVisible(false);
    }//GEN-LAST:event_JTBAnadirDepartamentoActionPerformed

    private void JTBModificarDepartamentoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarDepartamentoActionPerformed
        // Si JTBModificarDepartamento esta seleccionado
        if (JTBModificarDepartamento.isSelected())
        {
            // Mostramos el JPanel de introduccion de datos y el JComboBox de DNIs en caso de ser necesario
            JPJCBDNIDep.setVisible(false);
            if (Tipo_Usu.equals("ADMINISTRADOR"))   JPJCBDNIDep.setVisible(true);
            JPDepartamentosD.setVisible(true);

            // Escondemos los paneles de insercion de empresas y personal
            JPEmpresasD.setVisible(false);
            mostrarJPanelsPersonal("");

            // Des-seleccionamos el boton de 'Añadir departamento'
            JTBAnadirDepartamento.setSelected(false);
        }
        // Si no lo esta -> Escondemos el panel de datos
        else    JPDepartamentosD.setVisible(false);
    }//GEN-LAST:event_JTBModificarDepartamentoActionPerformed

    private void JBModificarEmpresa1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBModificarEmpresa1ActionPerformed
        try
        {
            // Creamos la empresa
            Empresa emp = new Empresa();
            emp.setNombre((String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0));
            emp.setCodEmp(emp.obtenerCodEmp());

            // Creamos el departamento nuevo
            Residente Resid = null;
            String DNI_Aut;
            if (Tipo_Usu.equals("ADMINISTRADOR"))
            {
                DNI_Aut = (String) JCBDNIDep.getModel().getSelectedItem();
                if (!DNI_Aut.equals("Ninguno")) Resid = new Residente(DNI_Aut);
            }
            else    Resid = new Residente(DNI_Usu);
            StringTokenizer st = new StringTokenizer ((String) JCBActividad.getModel().getSelectedItem());
            String Cod_Acc = st.nextToken();
            T_Acceso Acceso = new T_Acceso(Cod_Acc);
            Departamento dep = new Departamento(JTFCodDep.getText(),JTFNomDep.getText(), Acceso,Integer.parseInt(JTFPeriodicidad.getText()),JCCFUltActu.getCalendar(), Resid, new HashMap());

            // Comprobamos si se trata de una insercion o una modificacion de departamento
            // -> Insercion
            if (JTBAnadirDepartamento.isSelected())
            {
                // Avisamos de que vamos a dar de alta el departamento
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a dar de alta la empresa.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Damos de alta el departamento
                if (seleccion == 0)
                {
                    // Insertamos el departamento
                    emp.insertDepartamentoBD(dep);

                    // Refrescamos las tablas de departamentos, empleados, contactos, telefonos, E-Mails, cargos y llaves
                    refrescarTablasEmpresasPersonal("DGC");
                    String[] Etiquetas = {"Código","Zona","Descripción"};
                    JTLsLlsPers.setModel(new DefaultTableModel(Etiquetas,0));
                    String[] Etiquetas2 = {"Teléfonos"};
                    JTTfns.setModel(new DefaultTableModel(Etiquetas2,0));
                    Etiquetas2[0] = "E-Mails";
                    JTEmails.setModel(new DefaultTableModel(Etiquetas2,0));
                    Etiquetas2[0] = "Cargos";
                    JTCargos.setModel(new DefaultTableModel(Etiquetas2,0));

                    // Des-seleccionamos el boton de 'Añadir departamento' y deshabilitamos los de 'Eliminar departamento', 'Modificar departamento', 'Añadir persona/contacto' y 'Modificar persona/contacto'
                    JTBAnadirDepartamento.setSelected(false);
                    JTBAnadirDepartamento.setEnabled(true);
                    JBEliminarDepartamento.setEnabled(false);
                    JTBModificarDepartamento.setSelected(false);
                    JTBModificarDepartamento.setEnabled(false);
                    JTBAnadirPerContac.setSelected(false);
                    JTBAnadirPerContac.setEnabled(false);
                    JTBModificarPerContac.setSelected(false);
                    JTBModificarPerContac.setEnabled(false);

                    // Escondemos el JPanel
                    JPDepartamentosD.setVisible(false);

                    // Advertimos de que el departamento ha sido insertado correctamente
                    JOptionPane.showMessageDialog(this,"El departamento ha sido insertado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // -> Modificacion
            else
            {
                // Avisamos de que vamos a modificar el departamento
                int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a modificar el departamento seleccionado.\n" +
                        "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

                // Si se acepta la operacion -> Modificamos el departamento seleccionado
                if (seleccion == 0)
                {
                    // Creamos el departamento viejo
                    Departamento dep_old = emp.getDepartamentoBD((emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0))));

                    // Modificamos el departamento viejo con el nuevo
                    emp.updateDepartamentoBD(dep_old, dep);

                    // Refrescamos la tabla de departamento
                    refrescarTablasEmpresasPersonal("D");

                    // Des-seleccionamos el boton de 'Añadir departamento' y deshabilitamos los de 'Eliminar departamento', 'Modificar departamento', 'Añadir persona/contacto' y 'Modificar persona/contacto'
                    JTBAnadirDepartamento.setSelected(false);
                    JTBAnadirDepartamento.setEnabled(true);
                    JBEliminarDepartamento.setEnabled(false);
                    JTBModificarDepartamento.setSelected(false);
                    JTBModificarDepartamento.setEnabled(false);
                    JTBAnadirPerContac.setSelected(false);
                    JTBAnadirPerContac.setEnabled(false);
                    JTBModificarPerContac.setSelected(false);
                    JTBModificarPerContac.setEnabled(false);

                    // Escondemos el JPanel
                    JPDepartamentosD.setVisible(false);

                    // Advertimos de que el departamento ha sido modificado correctamente
                    JOptionPane.showMessageDialog(this,"El departamento ha sido modificado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible insertar/modificar el departamento.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBModificarEmpresa1ActionPerformed

    private void JBEliminarPrestDepActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarPrestDepActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las llaves seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las llaves seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las llaves
            if (seleccion == 0)
            {
                // Obtenemos las llaves seleccionadas
                int[] lsSelecs = JTLsLlsPers.getSelectedRows();

                // Obtenemos la empresa y departamento seleccionados
                Empresa emp = new Empresa();
                emp.setNombre((String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0));
                emp.setCodEmp(emp.obtenerCodEmp());
                Departamento dep = emp.getDepartamentoBD(emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));

                // Borramos cada llave
                Llave l;
                for (int i=0; i<lsSelecs.length; i++)
                {
                    // Obtenemos la llave
                    l = new Llave((Integer) JTLsLlsPers.getModel().getValueAt(lsSelecs[i],0));

                    // Borramos la llave
                    emp.removeLlaveBD(dep, l);
                }

                // Desactivamos el boton de borrado
                JBEliminarPrestDep.setEnabled(false);

                // Refrescamos la tabla de llaves
                refrescarTablasEmpresasPersonal("L");

                // Advertimos de que las llaves han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las llaves han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las llaves seleccionadas.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarPrestDepActionPerformed

    private void JTLsLlsPersMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsLlsPersMouseClicked
        // Si tenemos seleccionada alguna fila -> Activamos el boton de borrado
        if (JTLsLlsPers.getSelectedRow() != -1)     JBEliminarPrestDep.setEnabled(true);
    }//GEN-LAST:event_JTLsLlsPersMouseClicked

    private void JBAnadirPrestDepActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirPrestDepActionPerformed
        try
        {
            // Obtenemos la empresa
            Empresa emp = new Empresa();
            emp.setNombre((String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0));
            emp.setCodEmp(emp.obtenerCodEmp());

            // Abrimos el JDialog con el que poder añadir llaves prestables
            JDLsDep jdls = new JDLsDep(this,true,emp.getCodEmp(),emp.obtenerCodDep((String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0)));
            jdls.setVisible(true);

            // Refrescamos la tabla de llaves
            refrescarTablasEmpresasPersonal("L");

            // Cerramos el JDialog
            jdls.dispose();
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible abrir el dialogo de selección de llaves.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBAnadirPrestDepActionPerformed

    private void JTBModificarPerContacActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBModificarPerContacActionPerformed
        try
        {
            // Si JTBModificarPerContac esta seleccionado
            if (JTBModificarPerContac.isSelected())
            {
                // Escondemos los paneles de insercion de empresas y departamentos
                JPEmpresasD.setVisible(false);
                JPDepartamentosD.setVisible(false);

                // Mostramos/Escondemos los paneles y botones necesarios
                JPElimTE.setVisible(true);
                JPDNIAuto.setVisible(false);
                JTFDNIDP.setEditable(false);
                JBAnadirPrestDep1.setEnabled(true);
                JTBAnadirPerContac.setSelected(false);

                // Mostramos los paneles de datos que correspondan
                String DNI;
                if (JTEmpleados.isShowing())    DNI = (String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0);
                else                            DNI = (String) JTContactos.getModel().getValueAt(JTContactos.getSelectedRow(),0);
                Personal pers = new Personal (DNI);
                String tipo = pers.obtenerTipo();

                if (tipo.equals("Ocasional"))
                    if (JTEmpleados.isShowing())                                mostrarJPanelsPersonal("GOe");
                    else                                                        mostrarJPanelsPersonal("GOc");
                else if (tipo.equals("Residente"))
                    if (JTEmpleados.isShowing())                                mostrarJPanelsPersonal("GRe");
                    else                                                        mostrarJPanelsPersonal("GRc");
                else if (tipo.equals("Vigilante"))
                    if (JTEmpleados.isShowing())                                mostrarJPanelsPersonal("GVe");
                    else                                                        mostrarJPanelsPersonal("GVc");
                else
                {
                    // Refrescamos el JComboBox de cargos
                    Cargo car = new Cargo();
                    UGUI.rellenarJCB(JCBCargos, car.obtenerDescripCargos());
                    car.UBD.cerrarConexion();

                    // Mostramos los paneles de edicion
                    JPElimC.setVisible(true);
                    mostrarJPanelsPersonal("GP");
                    JPElimTE.setVisible(true);

                    // Eliminamos el cargo que se produce como consecuencia del refresco de JTCargos
                    DefaultTableModel modelo = (DefaultTableModel) JTCargos.getModel();
                    modelo.removeRow(modelo.getRowCount()-1);
                    JTCargos.setModel(modelo);
                }
            }
            // Si no lo esta -> Escondemos el panel de datos
            else
            {
                // Mostramos/Escondemos los paneles que correspondan
                JPDNIAuto.setVisible(false);
                JTFDNIDP.setEditable(true);
                JCBEmpresas.setEnabled(true);
                JBAnadirPrestDep1.setEnabled(false);
                JPElimTE.setVisible(false);
                JPElimC.setVisible(false);
                mostrarJPanelsPersonal("");

                // Refrescamos las tablas de telefonos, E-Mails y cargos
                refrescarTablasEmpresasPersonal("TMX");
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la persona seleccionada.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JTBModificarPerContacActionPerformed

    private void JCCFCaducidadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCCFCaducidadActionPerformed

    }//GEN-LAST:event_JCCFCaducidadActionPerformed

    private void JCCFCaducidadItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCCFCaducidadItemStateChanged

    }//GEN-LAST:event_JCCFCaducidadItemStateChanged

    private void JCCFCaducidadDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFCaducidadDateChanged
        // Si estamos creando o editando una persona ocasional
        if (JTBAnadirPerContac.isSelected() || JTBModificarPerContac.isSelected())
        {
            // Comprobamos que la fecha de caducidad no sea anterior al dia de hoy
            Calendar cal_selec = JCCFCaducidad.getCalendar();
            Calendar cal_ahora = Calendar.getInstance();
            cal_selec.set(cal_selec.HOUR_OF_DAY, 23);
            cal_selec.set(cal_selec.MINUTE, 23);
            cal_selec.set(cal_selec.SECOND, 59);

            // Si lo es, reajustamos el JCC avisando del error
            if (cal_selec.before(cal_ahora))
            {
                JCCFCaducidad.setDate(cal_ahora.getTime());
                JCCFCaducidad.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(JCCFCaducidad.getDate()));
                JOptionPane.showMessageDialog(this,"No puede seleccionarse una fecha de caducidad anterior a hoy.","ATENCIÓN",JOptionPane.WARNING_MESSAGE);
            }
        }
    }//GEN-LAST:event_JCCFCaducidadDateChanged

    private void JEliminarCargoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JEliminarCargoActionPerformed
        // Obtenemos los cargos seleccionados
        int[] cargosSelecs = JTCargos.getSelectedRows();

        // Eliminamos cada cargo
        DefaultTableModel modelo = (DefaultTableModel) JTCargos.getModel();
        for (int i=cargosSelecs.length-1; i>=0; i--)      modelo.removeRow(cargosSelecs[i]);

        // Refrescamos la tabla
        JTCargos.setModel(modelo);
    }//GEN-LAST:event_JEliminarCargoActionPerformed

    private void JCBCargosItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_JCBCargosItemStateChanged
    }//GEN-LAST:event_JCBCargosItemStateChanged

    private void JCBCargosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBCargosActionPerformed
        // Insertamos el cargo en JTCargos
        DefaultTableModel modelo = (DefaultTableModel) JTCargos.getModel();
        Vector v = new Vector();
        v.add((String) JCBCargos.getModel().getSelectedItem());
        modelo.addRow(v);
        JTCargos.setModel(modelo);
    }//GEN-LAST:event_JCBCargosActionPerformed

    private void JCCFUltActuDateChanged(org.freixas.jcalendar.DateEvent evt) {//GEN-FIRST:event_JCCFUltActuDateChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_JCCFUltActuDateChanged

    private void JTLsLlsPers1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTLsLlsPers1MouseClicked
        // Si tenemos seleccionada alguna fila -> Activamos el boton de borrado
        if (JTLsLlsPers1.getSelectedRow() != -1)     JBEliminarPrestDep1.setEnabled(true);
    }//GEN-LAST:event_JTLsLlsPers1MouseClicked

    private void JBEliminarPrestDep1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBEliminarPrestDep1ActionPerformed
        try
        {
            // Avisamos de que vamos a eliminar las llaves seleccionadas
            int seleccion = JOptionPane.showConfirmDialog(this,"Atención, se va a proceder a eliminar las llaves seleccionadas.\n" +
                    "¿Desea llevar a cabo esta operación?","ATENCIÓN",JOptionPane.YES_NO_OPTION);

            // Si se acepta la operacion -> Eliminamos las llaves
            if (seleccion == 0)
            {
                // Obtenemos las llaves seleccionadas
                int[] lsSelecs = JTLsLlsPers1.getSelectedRows();

                // Obtenemos la persona ocasional
                Ocasional ocas = new Ocasional((String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0), (String) JCBEmpresas.getModel().getSelectedItem());

                // Eliminamos cada llave
                Llave l;
                for (int i=0; i<lsSelecs.length; i++)
                {
                    // Obtenemos la llave
                    l = new Llave((Integer) JTLsLlsPers1.getModel().getValueAt(lsSelecs[i],0));

                    // Borramos la llave
                    ocas.removeLlaveBD(l);
                }

                // Desactivamos el boton de borrado
                JBEliminarPrestDep1.setEnabled(false);

                // Refrescamos la tabla de llaves
                refrescarTablasEmpresasPersonal("L1");

                // Advertimos de que las llaves han sido eliminadas correctamente
                JOptionPane.showMessageDialog(this,"Las llaves han sido eliminadas correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible eliminar las llaves seleccionadass.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBEliminarPrestDep1ActionPerformed

    private void JBAnadirPrestDep1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBAnadirPrestDep1ActionPerformed
        try
        {
            // Abrimos el JDialog con el que poder añadir llaves prestables a la persona ocasional
            JDLsOcas jdls = new JDLsOcas(this,true, (String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0), (String) JCBEmpresas.getModel().getSelectedItem());
            jdls.setVisible(true);

            // Cerramos el JDialog
            jdls.dispose();

            // Refrescamos la tabla de llaves
            refrescarTablasEmpresasPersonal("L1");
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible abrir el dialogo de selección de llaves.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBAnadirPrestDep1ActionPerformed

    private void JTContactosMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTContactosMouseEntered
        // TODO add your handling code here:
    }//GEN-LAST:event_JTContactosMouseEntered

    private void JTTfnsMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTTfnsMouseClicked
        // TODO add your handling code here:
    }//GEN-LAST:event_JTTfnsMouseClicked

    private void JTFNombreEmpMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFNombreEmpMouseClicked
        // Eliminamos 'Nombre ...' y establecemos el color de texto a negro
        JTFNombreEmp.setText("");
        JTFNombreEmp.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFNombreEmpMouseClicked

    private void JTFCIFMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFCIFMouseClicked
        // Eliminamos 'CIF ...' y establecemos el color de texto a negro
        JTFCIF.setText("");
        JTFCIF.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFCIFMouseClicked

    private void JTFNombreDepMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFNombreDepMouseClicked
        // Eliminamos 'Nombre ...' y establecemos el color de texto a negro
        JTFNombreDep.setText("");
        JTFNombreDep.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFNombreDepMouseClicked

    private void JTFActividadMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFActividadMouseClicked
        // Eliminamos 'Actividad ...' y establecemos el color de texto a negro
        JTFActividad.setText("");
        JTFActividad.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFActividadMouseClicked

    private void JTFFiltroMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_JTFFiltroMouseClicked
        // Eliminamos 'Buscar ...' y establecemos el color de texto a negro
        JTFFiltro.setText("");
        JTFFiltro.setForeground(Color.BLACK);
    }//GEN-LAST:event_JTFFiltroMouseClicked

    private void JPEmpresasPersonalComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_JPEmpresasPersonalComponentShown
        // Refrescamos toda la informacion de la pestañana
        iniciarEmpresasPersonal();
    }//GEN-LAST:event_JPEmpresasPersonalComponentShown

    private void JBPrestsPDF2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JBPrestsPDF2ActionPerformed
        try
        {
            // Si existe algun JCheckBox seleccionado
            if (JCBPDFTodas2.isSelected() || JCBPDFAlms2.isSelected() || JCBPDFLs2.isSelected() || JCBPDFLls2.isSelected())
            {
                // Desplegamos un JFileChooser en modo 'Guardar'
                JFileChooser jfc = new JFileChooser();
                int opcion = jfc.showSaveDialog(this);

                // Si se pulsa 'Aceptar'
                if (opcion == jfc.APPROVE_OPTION)
                {
                    // Recogemos el fichero seleccionado
                    File fichero = jfc.getSelectedFile();
                    String Ruta = fichero.getAbsolutePath();

                    // Creamos un objeto de tipo GeneradorInformes
                    GeneradorInformes gi = new GeneradorInformes(Ruta);
                    // Generamos el reporte
                    gi.generarInformeAvanzado("EMPRESAS Y PERSONAL","Empresas, departamentos y personal existente:","Empresas","Departamentos","Personal",JTEmpresas,JTDepartamentos,(JTEmpleados.isShowing() ? JTEmpleados : JTContactos),JCBPDFTodas2.isSelected(),JCBPDFAlms2.isSelected(),JCBPDFLs2.isSelected(),JCBPDFLls2.isSelected());

                    // Des-seleccionamos el boton de generar PDF
                    JTBActivarPanelPDF2.setSelected(false);

                    // Cerramos el panel de PDF
                    JPPrestsPDF2.setVisible(false);

                    // Advertimos de que el informe ha sido generado correctamente
                    JOptionPane.showMessageDialog(this,"Reporte generado correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
                }
            }
            // Si no hemos seleccionado ningun elemento, avisamos del error
            else    JOptionPane.showMessageDialog(this,"No se ha seleccionado ningún elemento a reportar.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,"No ha sido posible realizar el reporte solicitado.","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_JBPrestsPDF2ActionPerformed

    private void JCBPDFTodas2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFTodas2ActionPerformed
        // Si esta seleccionado -> Desactivamos el resto de JCheckBoxes
        if (JCBPDFTodas2.isSelected())
        {
            JCBPDFAlms2.setEnabled(false);
            JCBPDFLs2.setEnabled(false);
            JCBPDFLls2.setEnabled(false);
        }
        // En caso contrario -> Habilitamos el resto de JCheckBoxes
        else
        {
            JCBPDFAlms2.setEnabled(true);
            JCBPDFLs2.setEnabled(true);
            JCBPDFLls2.setEnabled(true);
        }
    }//GEN-LAST:event_JCBPDFTodas2ActionPerformed

    private void JCBPDFLls2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFLls2ActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBPDFLls2ActionPerformed

    private void JTBActivarPanelPDF2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JTBActivarPanelPDF2ActionPerformed
        // Si 'JTBActivarPanelPDF1' esta pulsado
        if (JTBActivarPanelPDF2.isSelected())
        {
            // Inicializamos el panel
            JCBPDFTodas2.setSelected(false);
            JCBPDFTodas2.setEnabled(true);
            JCBPDFAlms2.setSelected(false);
            JCBPDFAlms2.setEnabled(true);
            JCBPDFLs2.setSelected(false);
            JCBPDFLs2.setEnabled(true);
            JCBPDFLls2.setSelected(false);
            JCBPDFLls2.setEnabled(true);

            // Mostramos el panel de generar reportes
            JPPrestsPDF2.setVisible(true);
        }
        // En caso contrario -> Lo escondemos
        else
        {
            JPPrestsPDF2.setVisible(false);
        }
    }//GEN-LAST:event_JTBActivarPanelPDF2ActionPerformed

    private void JCBPDFLs2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_JCBPDFLs2ActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_JCBPDFLs2ActionPerformed




        // ***********************************************************************************************************
        // ***************************************** METODOS *********************************************************
        // ***********************************************************************************************************


        // *******************************************************************************************
        // ***************************************** ACCESOS *****************************************
        // *******************************************************************************************
        /* iniciarAccesos: Metodo que inicializa todos los elementos referentes al tab 'Accesos'
         *
         * @param   void
         * @return  void
         */
        private void iniciarAccesos()
        {
            try
            {
                // Creamos el objeto de tipo Acceso
                Acceso acc = new Acceso();

                // Comprobamos que las fechas y horas sean correctas
                comprobarFechasHorasAccesos();

                // Obtenemos la fecha y horas entre las que consultar los accesos, asi como el filtro
                boolean entrada = JRBEntrada.isSelected();
                String fecha_inic = new SimpleDateFormat("yyyy-MM-dd").format(JCCFechaInic.getDate());
                String fecha_fin = new SimpleDateFormat("yyyy-MM-dd").format(JCCFechaFin.getDate());
                String hora_inic = null;
                String hora_fin = null;
                String filtro = JTFBuscar.getText().equals("Buscar ...") ? "" : JTFBuscar.getText();

                if (!JCBTodoDia.isSelected())
                {
                    hora_inic = (String) JCBHoraInic.getSelectedItem();
                    hora_fin = (String) JCBHoraFin.getSelectedItem();
                }

                // Obtenemos los accesos realizados bajo los criterios de busqueda indicados, rellenando la tabla
                UGUI.rellenarTabla(JTAccesos,acc.obtenerAccesos(entrada,fecha_inic,fecha_fin,hora_inic,hora_fin,filtro));

                // Cerramos la conexion
                acc.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los accesos.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* comprobarFechasHorasAccesos: Metodo que comprueba que las fechas y horas de busqueda de accesos
         * sean correctas, realizando los cambios oportunos en caso contrario
         *
         * @param   void
         * @return  void
         */
        private void comprobarFechasHorasAccesos()
        {
            try
            {
                // Variable que denota si hay que avisar al usuario sobre el cambio de fecha-hora
                boolean aviso = false;

                // Comporbamos las fechas
                // Si la fecha de inicio es posterior a la de fin, avisamos del error y establecemos
                // la fecha de fin igual a la de inicio
                Calendar JCC = JCCFechaInic.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),0,0,0);
                Calendar ci = Calendar.getInstance();
                ci.setTime(JCC.getTime());
                JCC = JCCFechaFin.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),23,59,59);
                Calendar cf = Calendar.getInstance();
                cf.setTime(JCC.getTime());

                if (!cf.after(ci))
                {
                    JCCFechaFin.setDate(JCCFechaInic.getDate());
                    JCCFechaFin.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(JCCFechaInic.getDate()));
                    JCCFechaFin.setVisible(false);
                    JCCFechaFin.setVisible(true);
                    aviso = true;
                }

                // Comprobamos las horas en caso de ser necesario
                JCC = JCCFechaInic.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),23,59,59);
                cf = Calendar.getInstance();
                ci.setTime(JCC.getTime());
                JCC = JCCFechaFin.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),0,0,0);
                cf = Calendar.getInstance();
                cf.setTime(JCC.getTime());
                if (!JCBTodoDia.isSelected() && (!cf.after(ci)))
                {
                    DateFormat dt = new SimpleDateFormat ("HH:mm");
                    if (dt.parse((String) JCBHoraInic.getSelectedItem()).after(dt.parse((String) JCBHoraFin.getSelectedItem())))
                    {
                        JCBHoraFin.getModel().setSelectedItem((String) JCBHoraInic.getSelectedItem());
                        JCBHoraInic.getModel().setSelectedItem((String) JCBHoraFin.getSelectedItem());
                        aviso = true;
                    }
                }

                if (aviso)  JOptionPane.showMessageDialog(this,"La fecha-hora de fin debe ser posterior a la de inicio.\nSe han modificado la fecha y/o hora de fin.","AVISO",JOptionPane.WARNING_MESSAGE);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible comprobar las fechas-horas.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* mostrarAcceso: Metodo que muestra un acceso en la GUI
         *
         * @param   void
         * @return  void
         */
        private void mostrarAcceso()
        {
            try
            {
                // Inicializamos Acceso y HashMap de prestamos
                Acceso acc = null;
                HM_GLLDs = new HashMap();
                
                // Si existe algun acceso seleccionado en el JTable
                if (JTAccesos.getSelectedRow() != -1)
                {
                    // Obtenemos la Entrada que lo identifica
                    String Entrada = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(JTAccesos.getModel().getValueAt(JTAccesos.getSelectedRow(),1));

                    // Creamos el acceso
                    acc = new Acceso (Entrada);

                    // Habilitamos el JCB de la tarjeta
                    JCBTarjeta.setSelected(true);
                    JCBTarjeta.setEnabled(true);
                }

                // Mostramos los datos  si existe un DNI, bien cogido de la tabla de accesos o de la persona que va a acceder
                if (mostrarDatosPersonales(acc) == 0)
                {
                    // Mostramos los datos de la tarjeta
                    mostrarTarjeta(acc);

                    // Mostramos los prestamos
                    mostrarPrestamos(acc);

                    // Mostramos si es necesaria autorizacion telefonica
                    mostrarAutorizTfno();
                }
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos del acceso.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* mostrarDatosPersonales: Metodo que muestra los datos personales de un acceso
         *
         * @param   Acceso  acc     Acceso del que mostrar los datos personales del accesor
         * @return  int             0 si existe persona y 1 si no existe
         */
        private int mostrarDatosPersonales(Acceso acc) throws DataException, SQLException, RemoteException
        {
            String DNI = "";

            // Si se trata de un acceso existente
            if (acc != null)
            {
                // Mostramos los datos
                JTFDNI.setText(acc.getPersona().getDNI());
                JTFApellido1.setText(acc.getPersona().getApellido1());
                JTFApellido2.setText(acc.getPersona().getApellido2());
                JTFNombre.setText(acc.getPersona().getNombre());
                JTAMotivo.setText(acc.getMotivo());
                JTAOtrosDatos.setText(acc.getOtrosDatos());


                // Si se trata de un acceso con salida
                if (acc.getFechaJ() != null)
                {
                    // Desactivamos los prestamos
                    JBAnadirPrestamo.setEnabled(false);
                    JBEliminarPrestamo.setEnabled(false);
                    JBDevolverPrestamo.setEnabled(false);
                    // Desactivamos el poder dar salidas
                    JBSalida.setEnabled(false);
                    JBDarSalidaConDev.setEnabled(false);
                    // No se pueden modificar JTAMotivo y JTAOtrosDatos
                    JTAMotivo.setEditable(false);
                    JTAOtrosDatos.setEditable(false);
                    JBAnadirAcceso.setEnabled(false);
                }
                // Si se trata de un acceso sin salida
                else
                {
                    JBAnadirPrestamo.setEnabled(true);

                    // Se puede dar salida
                    JBSalida.setEnabled(true);
                    JBDarSalidaConDev.setEnabled(true);
                    // Se pueden modificar JTAMotivo y JTAOtrosDatos
                    JTAMotivo.setEditable(true);
                    JTAOtrosDatos.setEditable(true);
                    JBAnadirAcceso.setEnabled(true);

                    JCBTarjeta.setSelected(false);
                }
                // Esten con o sin salida, los accesos se pueden eliminar siempre
                JBEliminarAcceso.setEnabled(true);
            }
            // Si se trata de un acceso por introducir
            else
            {
                // Desplegamos el JDialog JDAccesoPers
                JDAccesoPers jda = new JDAccesoPers(this,true);
                jda.setVisible(true);
                // Recuperamos el DNI
                DNI = jda.getDNI();
                boolean pulsarExit = jda.getPulsarExit();
                // Cerramos el JDialog
                jda.dispose();
                if (pulsarExit)   DNI = null;
                else              JTAccesos.clearSelection();

                // Si hemos seleccionado un DNI (una persona)
                if (DNI != null)
                {
                    // Obtenemos la persona
                    Personal pers = new Personal(DNI);
                    // Rellenamos los campos
                    JTFDNI.setText(pers.getDNI());
                    JTFApellido1.setText(pers.getApellido1());
                    JTFApellido2.setText(pers.getApellido2());
                    JTFNombre.setText(pers.getNombre());
                    JTAMotivo.setText("");
                    JTAOtrosDatos.setText("");

                    // Activacion de campos/botones
                    // Campos de motivo y otros datos, a rellenar antes de proporcionar acceso
                    JTAMotivo.setEditable(true);
                    JTAOtrosDatos.setEditable(true);
                    // Se pueden añadir prestamos pero no eliminar ni devolver
                    JBAnadirPrestamo.setEnabled(true);
                    JBEliminarPrestamo.setEnabled(false);
                    JBDevolverPrestamo.setEnabled(false);
                    // No se puede dar salida
                    JBSalida.setEnabled(false);
                    JBDarSalidaConDev.setEnabled(false);
                    // Se puede dar acceso pero no eliminarlo (ya que no existe aun)
                    JBAnadirAcceso.setEnabled(true);
                    JBEliminarAcceso.setEnabled(false);
                }
                // En caso de no seleccionar persona alguna, no se hace nada
            }
            // Devolvemos el resultado, si disponemos de una persona (que ha accedido o desea acceder) o no
            if (DNI != null)    return 0;
            else                return 1;
        }

        /* mostrarTarjeta: Metodo que muestra los datos de la tarjeta empleada en el acceso
         *
         * @param   Acceso  acc     Acceso del que mostrar la informacion de la tarjeta
         * @return  void
         */
        private void mostrarTarjeta (Acceso acc) throws DataException, SQLException, RemoteException
        {
            // Numero de tarjeta
            int Num_Tarj;
            // Color de tarjeta
            int Color;

            // Si se trata de un acceso existente
            if (acc != null)
            {
                // Obtenemos los datos de la tarjeta
                Num_Tarj = acc.getTarjeta().getCodTarj();
                Color = acc.getTarjeta().getColor().getCodColor();

                // Si tiene fecha de salida -> Deshabilitamos el JCB de la tarjeta
                if (acc.getFechaJ() != null)
                {
                    JCBTarjeta.setSelected(false);
                    JCBTarjeta.setEnabled(false);
                }
                // Si no la tiene -> Habilitamos el JCB de la tarjeta
                else
                {
                    JCBTarjeta.setSelected(false);
                    JCBTarjeta.setEnabled(true);
                }
            }
            // Si se trata de un acceso nuevo -> Obtenemos el color de tarjeta que le corresponderia al usuario
            // y le ofrecemos la tarjeta no prestada, de dicho color y con menor numeracion
            else
            {
                // Creamos un objeto de tipo Color
                API.Tablas.Color col = new API.Tablas.Color();
                // Obtenemos el color por defecto
                Color = col.colorPorDefecto(JTFDNI.getText());

                // Creamos un objeto de tipo Tarjeta
                Tarjeta tarj = new Tarjeta();
                // Obtenemos el numero de tarjeta por defecto
                Num_Tarj = tarj.numTarjAcc(Color);
                // Deshabilitamos el JCB de la tarjeta
                JCBTarjeta.setSelected(false);
                JCBTarjeta.setEnabled(false);
            }

            // Rellenamos los campos que hacen referencia a la tarjeta a prestar
            JPColorTarjeta.setBackground(new Color (Color));
            JTFCodTarj.setText(new Integer(Num_Tarj).toString());
        }

        /* mostrarPrestamos: Metodo que muestra los prestamos del acceso proporcionado
         *
         * @param   Acceso  acc     Acceso del que mostrar los prestamos
         * @return  void
         */
        private void mostrarPrestamos(Acceso acc) throws DataException, SQLException, RemoteException
        {
            // Si se trata de un acceso existente
            if (acc != null)
            {
                // Rellenamos la tabla de prestamos de llaves y llaveros
                UGUI.rellenarTabla(JTLsLls,acc.obtenerLsLlsAcceso());

                // Cerramos la conexion
                acc.UBD.cerrarConexion();

                if (acc.getFechaJ() == null)
                {
                    if (JTLsLls.getSelectedRow() != -1)
                    {
                        JBEliminarPrestamo.setEnabled(true);
                        JBDevolverPrestamo.setEnabled(true);
                    }
                    else
                    {
                        JBEliminarPrestamo.setEnabled(false);
                        JBDevolverPrestamo.setEnabled(false);
                    }
                }
            }
            // Si se trata de un acceso por insertar -> Rellenamos la tabla mediante el HashMap de prestamos provisionales
            else
            {
                UGUI.rellenarTablaPrestamos(JTLsLls,HM_GLLDs);
            }
        }

        /* mostrarAutorizTfno: Metodo que muestra si es necesaria autorizacion telefonica
         *
         * @param   void
         * @return  void
         */
        private void mostrarAutorizTfno() throws DataException, SQLException, RemoteException
        {
            // Creamos un objeto de tipo Personal
            Personal pers = new Personal(JTFDNI.getText());

            // Obtenemos el DNI del autorizador del departamento para el que trabaja la persona accesora
            String DNI = pers.obtenerResponsable();
            
            // Si existe autorizador -> Obtenemos sus datos
            if (!DNI.equals(""))
            {
                // Creamos un objeto de tipo Personal que represente al responsable
                pers = new Personal(DNI);
                // Rellenamos el campo destinado al nombre y apellidos del autorizador
                JTFAvisoTfno.setText(pers.getApellido1() + " " + pers.getApellido2() + ", " + pers.getNombre());
                
                // Rellenamos los telefonos del autorizador
                UGUI.rellenarTablaTelefonos(JTTelefonos,pers.getTelefonos());
            }
            // Si no existe autorizador
            else
            {
                // Avisamos de que no es necesaria autorizacion telefonica y...
                JTFAvisoTfno.setText("No");

                // ... vaciamos la tabla de telefonos
                UGUI.rellenarTablaTelefonos(JTTelefonos,new ArrayList());
            }
        }

        /* borrarFormAcceso: Metodo que borrar todos los datos del formulario de acceso
         *
         * @param   void
         * @return  void
         */
        private void borrarFormAcceso()
        {
            // Borramos los datos personales
            borrarDatosPersonales();
            // Borramos los datos de la tarjeta
            borrarTarjeta();
            // Borramos los prestamos
            borrarPrestamos();
            // Borramos si es necesaria autorizacion telefonica
            borrarAutorizTfno();
        }

        /* borrarDatosPersonales: Metodo que borrar todos los datos personales del formulario de acceso
         *
         * @param   void
         * @return  void
         */
        private void borrarDatosPersonales()
        {
            // Borramos todos los campos correspondientes a datos personales
            JTFDNI.setText("");
            JTFApellido1.setText("");
            JTFApellido2.setText("");
            JTFNombre.setText("");
            JTAMotivo.setText("");
            JTAOtrosDatos.setText("");

            // Sin datos en el formulario, no se pueden dar accesos ni salidas
            JBAnadirAcceso.setEnabled(false);
            JBEliminarAcceso.setEnabled(false);
            JBSalida.setEnabled(false);
            JBDarSalidaConDev.setEnabled(false);
        }

        /* borrarTarjeta: Metodo que borrar todos los datos de la tarjeta del formulario de acceso
         *
         * @param   void
         * @return  void
         */
        private void borrarTarjeta()
        {
            // Borramos todos los campos correspondientes a la tarjeta
            JTFCodTarj.setText("");
            JPColorTarjeta.setBackground(new Color (244,244,244));

            // Desactivamos todos los campos correspondiente a la tarjeta
            JCBTarjeta.setSelected(false);
        }

        /* borrarPrestamos: Metodo que borrar todos los prestamos del formulario de acceso
         *
         * @param   void
         * @return  void
         */
        private void borrarPrestamos()
        {
            // Reiniciamos JTLsLls
            String[] Etiquetas = {"Tipo","Código","Devuelta"};
            JTLsLls.setModel(new DefaultTableModel(Etiquetas,0));

            // No se puede realizar ninguna accion que tenga que ver con prestamos
            JBAnadirPrestamo.setEnabled(false);
            JBDevolverPrestamo.setEnabled(false);
            JBEliminarPrestamo.setEnabled(false);
        }

        /* borrarAutorizTfno: Metodo que borrar todos los prestamos del formulario de acceso
         *
         * @param   void
         * @return  void
         */
        private void borrarAutorizTfno()
        {
            // Borramos todos los campos correspondientes a la autorizacion telefonica
            JTFAvisoTfno.setText("");

            // Vaciamos JTTelefonos
            String[] Etiquetas = {"Teléfonos"};
            JTTelefonos.setModel(new DefaultTableModel(Etiquetas,0));
        }
        // *******************************************************************************************



        // *******************************************************************************************
        // *********************************** EMPRESAS Y PERSONAL ***********************************
        // *******************************************************************************************
        /* iniciarEmpresasPersonal: Metodo que inicializa todos los elementos referentes al tab 'Empresas y personal'
         *
         * @param   void
         * @return  void
         */
        private void iniciarEmpresasPersonal ()
        {
            try
            {
                // Escondemos los JPanels de empresas, departamentos, datos personales y botones
                JPEmpresasD.setVisible(false);
                JPDepartamentosD.setVisible(false);
                mostrarJPanelsPersonal("");
                JPElimTE.setVisible(false);
                JPElimC.setVisible(false);

                // Deseleccionamos las JTables necesarias
                JTEmpresas.clearSelection();
                JTDepartamentos.clearSelection();
                JTEmpleados.clearSelection();
                JTContactos.clearSelection();

                // Inicializamos los JComboBoxes de actividades, autorizadores y niveles
                refrescarCBActivs();

                // Inicializamos el JComboBox de autorizadores y niveles
                Cuenta cuenta = new Cuenta();
                UGUI.rellenarJCB(JCBDNIDep, cuenta.obtenerDNIsAdmAut());
                cuenta.UBD.cerrarConexion();
                refrescarCBNivs(JCBNivel);

                // Refrescamos todas las JTables
                refrescarTablasEmpresasPersonal("EDGCTMX");

                // Escondemos el JPanel de PDFs
                JPPrestsPDF2.setVisible(false);

                // Restablecemos todos los botones a su estado original
                refrescarBotonesEmpPers();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos de la pestaña 'Empresas y personal'.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarTablasEmpresasPersonal: Metodo que refresca las tablas indicadas en el tab 'Empresas y personal'
         *
         * @param   String      tablas      Tablas a refrescar
         * @return  void
         */
        private void refrescarTablasEmpresasPersonal (String tablas)
        {
            try
            {
                // Obtenemos los filtros y los nombres de la empresa y departamento seleccionados
                String JTFCif = (JTFCIF.getText().equals("CIF ...") ? "" : JTFCIF.getText());
                String JTFNom_Emp = (JTFNombreEmp.getText().equals("Nombre ...") ? "" : JTFNombreEmp.getText());
                String JTFNom_Dep = (JTFNombreDep.getText().equals("Nombre ...") ? "" : JTFNombreDep.getText());
                String JTFActiv = (JTFActividad.getText().equals("Actividad ...") ? "" : JTFActividad.getText());
                String JTFPers = (JTFFiltro.getText().equals("Buscar ...") ? "" : JTFFiltro.getText());
                String Nom_Emp = (JTEmpresas.getSelectedRow() == -1 ? "" : (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(), 0));
                if (Nom_Emp == null)        Nom_Emp = "";
                String Nom_Dep = (JTDepartamentos.getSelectedRow() == -1 ? "" : (String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(), 0));
                String DNI = "";
                if ((JTEmpleados.isShowing()) && (JTEmpleados.getSelectedRow() != -1))          DNI = (JTEmpleados.getSelectedRow() == -1 ? "" : (String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(), 0));
                else if  ((JTContactos.isShowing()) && (JTContactos.getSelectedRow() != -1))    DNI = (JTContactos.getSelectedRow() == -1 ? "" : (String) JTContactos.getModel().getValueAt(JTContactos.getSelectedRow(), 0));

                // Refrescamos las tablas
                if (tablas.contains("E"))       refrescarJTEmpresas(JTFCif,JTFNom_Emp,Nom_Dep,DNI);
                if (tablas.contains("D"))       refrescarJTDepartamentos(Nom_Emp,JTFNom_Dep,JTFActiv,DNI);
                if (tablas.contains("G"))       refrescarJTEmpleados(Nom_Emp,Nom_Dep,JTFPers);
                if (tablas.contains("C"))       refrescarJTContactos(Nom_Emp,Nom_Dep,JTFPers);
                if (tablas.contains("T"))       refrescarJTTfns(DNI);
                if (tablas.contains("M"))       refrescarJTEmails(DNI);
                if (tablas.contains("L"))       refrescarJTLsLlsPers();
                if (tablas.contains("L1"))      refrescarJTLsLlsPers1();
                if (tablas.contains("X"))       refrescarJTCargos(Nom_Emp,Nom_Dep,DNI);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarJTEmpresas: Metodo que refresca las tabla de empresas
         *
         * @param   String      CIF         CIF
         * @param   String      Nom_Emp     Nombre de empresa
         * @param   String      Nom_Dep     Nombre de departamento
         * @para    String      DNI         DNI
         * @return  void
         */
        private void refrescarJTEmpresas (String CIF, String Nom_Emp, String Nom_Dep, String DNI) throws DataException, SQLException, RemoteException
        {
                // Creamos el objeto de tipo Empresa
                Empresa emp = new Empresa();

                // Obtenemos las empresas y rellenamos la tabla
                UGUI.rellenarTabla(JTEmpresas,emp.datosEmpresas(CIF,Nom_Emp,Nom_Dep,DNI,DNI_Usu,Tipo_Usu));

                // Cerramos la conexion
                emp.UBD.cerrarConexion();
        }

        /* refrescarJTDepartamentos: Metodo que refresca las tabla de departamentos
         *
         * @param   String      Nom_Emp     Nombre de empresa
         * @param   String      Nom_Dep     Nombre de departamento
         * @param   String      Actividad   Actividad
         * @para    String      DNI         DNI
         * @return  void
         */
        private void refrescarJTDepartamentos (String Nom_Emp, String Nom_Dep, String Actividad, String DNI) throws DataException, SQLException, RemoteException
        {
            // Creamos el objeto de tipo Empresa
            Empresa emp = new Empresa();

            // Obtenemos los departamentos y rellenamos la tabla
            UGUI.rellenarTabla(JTDepartamentos,emp.datosDepartamentos(Nom_Emp,Nom_Dep,Actividad,DNI,DNI_Usu,Tipo_Usu));

            // Cerramos la conexion
            emp.UBD.cerrarConexion();

            // Descativamos los botones de 'Eliminar departamento' y 'Modificar departamento'
            JBEliminarDepartamento.setEnabled(false);
            JTBModificarDepartamento.setEnabled(false);
        }

        /* refrescarJTEmpleados: Metodo que refresca las tabla de empleados
         *
         * @param   String      Nom_Emp     Nombre de empresa
         * @param   String      Nom_Dep     Nombre de departamento
         * @para    String      Filtro      Filtro
         * @return  void
         */
        private void refrescarJTEmpleados (String Nom_Emp, String Nom_Dep, String Filtro) throws DataException, SQLException, RemoteException
        {
            // Creamos el objeto de tipo Empresa
            Empresa emp = new Empresa();

            // Rellenamos la tabla de empleados
            UGUI.rellenarTabla(JTEmpleados,emp.datosEmpleados(Nom_Emp,Nom_Dep,Filtro,DNI_Usu,Tipo_Usu));

            // Cerramos la conexion
            emp.UBD.cerrarConexion();

            // Desactivamos el boton de 'Eliminar persona'
            JBEliminarPerContac.setEnabled(false);
        }

        /* refrescarJTContactos: Metodo que refresca las tabla de contactos
         *
         * @param   String      Nom_Emp     Nombre de empresa
         * @param   String      Nom_Dep     Nombre de departamento
         * @param   String      Persona     Persona
         * @return  void
         */
        private void refrescarJTContactos (String Nom_Emp, String Nom_Dep, String Persona) throws DataException, SQLException, RemoteException
        {
            // Creamos el objeto de tipo Empresa
            Empresa emp = new Empresa();

            // Obtenemos los contactos y rellenamos la tabla
            UGUI.rellenarTabla(JTContactos,emp.datosContactos(Nom_Emp,Nom_Dep,Persona,DNI_Usu,Tipo_Usu));

            // Cerramos la conexion
            emp.UBD.cerrarConexion();

            // Descativamos el boton de 'Eliminar persona'
            JBEliminarPerContac.setEnabled(false);
        }

        /* refrescarJTTfns: Metodo que refresca las tabla de telefonos
         *
         * @param   String      DNI     DNI
         * @return  void
         */
        private void refrescarJTTfns (String DNI) throws DataException, SQLException, RemoteException
        {
            // Creamos el objeto de tipo Personal
            Personal pers = new Personal(DNI);

            // Rellenamos la tabla de telefonos
            UGUI.rellenarTabla(JTTfns,pers.obtenerTfns());

            // Cerramos la conexion
            pers.UBD.cerrarConexion();
        }

        /* refrescarEmails: Metodo que refresca las tabla de E-Mails
         *
         * @param   String      DNI     DNI
         * @return  void
         */
        private void refrescarJTEmails (String DNI) throws DataException, SQLException, RemoteException
        {
            // Creamos el objeto de tipo Personal
            Personal pers = new Personal(DNI);

            // Rellenamos la tabla de e-mails
            UGUI.rellenarTabla(JTEmails,pers.obtenerEmails());

            // Cerramos la conexion
            pers.UBD.cerrarConexion();
        }

        /* refrescarJTLsLlsPers: Metodo que refresca las tabla de llaves prestable a un departamento
         *
         * @param   void
         * @return  void
         */
        private void refrescarJTLsLlsPers () throws DataException, SQLException, RemoteException
        {
            // Obtenemos la empresa y el departamento
            if ((JTEmpresas.getSelectedRow() != -1) && (JTDepartamentos.getSelectedRow() != -1))
            {
                String Nombre_Departamento = (String) JTDepartamentos.getModel().getValueAt(JTDepartamentos.getSelectedRow(),0);
                Empresa emp = new Empresa();
                emp.setNombre((String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0));
                emp.setCodEmp(emp.obtenerCodEmp());

                // Rellenamos la tabla de llaves
                UGUI.rellenarTabla(JTLsLlsPers,emp.obtenerLlaves(emp.obtenerCodDep(Nombre_Departamento)));

                // Cerramos la conexion
                emp.UBD.cerrarConexion();
            }
        }

        /* refrescarJTLsLlsPers1: Metodo que refresca las tabla de llaves prestable a una persona ocasional
         *
         * @param   void
         * @return  void
         */
        private void refrescarJTLsLlsPers1 () throws DataException, SQLException, RemoteException
        {
            // Obtenemos la la persona ocasional
            Ocasional ocas = new Ocasional((String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0), (String) JCBEmpresas.getSelectedItem());

            // Rellenamos la tabla de llaves
            UGUI.rellenarTabla(JTLsLlsPers1,ocas.obtenerLlaves());

            // Cerramos la conexion
            ocas.UBD.cerrarConexion();
        }

        /* refrescarJTCargos: Metodo que refresca las tabla de cargos desempeñados por un contacto
         *
         * @param   String      Nom_Emp     Nombre de empresa
         * @param   String      Nom_Dep     Nombre de departamento
         * @return  void
         */
        private void refrescarJTCargos (String Nom_Emp, String Nom_Dep, String DNI) throws DataException, SQLException, RemoteException
        {
            // Creamos el objeto de tipo Personal
            Personal pers = new Personal(DNI);

            // Obtenemos los cargos y rellenamos la tabla
            if (Nom_Emp.equals(""))                                     UGUI.rellenarTabla(JTCargos,pers.obtenerCargos());
            else if ((!Nom_Emp.equals("")) && (Nom_Dep.equals("")))     UGUI.rellenarTabla(JTCargos,pers.obtenerCargosEmp(Nom_Emp));
            else if ((!Nom_Emp.equals("")) && (!Nom_Dep.equals("")))    UGUI.rellenarTabla(JTCargos,pers.obtenerCargosDep(Nom_Emp,Nom_Dep));

            // Cerramos la conexion
            pers.UBD.cerrarConexion();
        }

        /* mostrarDP: Metodo que refresca los datos de una persona
         *
         * @param   String      DNI     DNI de la persona a mostrar
         * @return  void
         */
        private void mostrarDP (String DNI)
        {
            try
            {
                // Mostramos los datos personales
                Personal pers = new Personal(DNI);
                JTFDNIDP.setText(pers.getDNI());
                JTFApellido1DP.setText(pers.getApellido1());
                JTFApellido2DP.setText(pers.getApellido2());
                JTFNombreDP.setText(pers.getNombre());
                JTFTfnoEmailDP.setText("");
                JTFTfnoEmailDP.setText("");

                // Refrescamos las tablas de telefonos, E-Mails y cargos
                refrescarTablasEmpresasPersonal("TMX");
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos personales de la persona.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* mostrarForm: Metodo que prepara los formularios necesarios para realizar la insercion de cualquier tipo de persona
         *
         * @param   void
         * @return  void
         */
        private void mostrarForm()
        {
            try
            {
                // Deseleccionamos el resto de JToggleButtons
                JBEliminarPerContac.setEnabled(false);
                JTBModificarPerContac.setSelected(false);
                JTBModificarPerContac.setEnabled(false);

                // Deseleccionamos cualquier fila de cualquier tabla de personal
                JTEmpleados.clearSelection();
                JTContactos.clearSelection();
                JTCargos.clearSelection();

                // Obtenemos el tipo de JPanel de personal que debemos mostrar
                String empresa = (String) JTEmpresas.getModel().getValueAt(JTEmpresas.getSelectedRow(),0);

                // Borramos las tablas de telefonos e E-Mails
                String[] etiquetas = {"Teléfonos"};
                JTTfns.setModel(new DefaultTableModel(etiquetas,0));
                etiquetas[0] = "E-Mails";
                JTEmails.setModel(new DefaultTableModel(etiquetas,0));

                // Borramos el formulario de datos especificos y mostramos los paneles personales que correspondan
                // * Personal ocasional
                if (empresa.equals("Ocasional"))
                {
                    // Preparamos el panel
                    JPDNIAuto.setVisible(true);
                    JCBEmpresas.setModel(new DefaultComboBoxModel());
                    JCBEmpresas.setEditable(true);
                    JCBEmpresas.getModel().setSelectedItem("");
                    JCCFCaducidad.setDate(Calendar.getInstance().getTime());
                    JCCFCaducidad.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(Calendar.getInstance().getTime()));
                    JCBActividad1.setSelectedIndex(0);
                    JTANotas.setText("");

                    // Mostramos el JPanel
                    if (JTEmpleados.isShowing())                                mostrarJPanelsPersonal("GOe");
                    else                                                        mostrarJPanelsPersonal("GOc");
                }
                // * Personal residente
                else if (empresa.equals("Informática Navarra S.A."))
                {
                    // Preparamos el formulario
                    if (JTContactos.isShowing())    JPDNIAuto.setVisible(true);
                    JCBNivel.setSelectedIndex(0);

                    // Mostramos el JPanel
                    if (JTEmpleados.isShowing())                                mostrarJPanelsPersonal("GRe");
                    else                                                        mostrarJPanelsPersonal("GRc");
                }
                // * Personal vigilante
                else if (empresa.equals("Compañía Integral de Seguridad"))
                {
                    // Preparamos el formulario
                    JTFCodV.setText("");

                    // Mostramos el JPanel
                    if (JTEmpleados.isShowing())
                    {
                        mostrarJPanelsPersonal("GVe");
                        JPDNIAuto.setVisible(false);
                    }
                    else
                    {
                        mostrarJPanelsPersonal("GVc");
                        JPDNIAuto.setVisible(true);
                    }
                }
                // * Resto de personal
                else
                {
                    // Preparamos el formulario
                    JTFCodV.setText("");
                    JPElimTE.setVisible(true);
                    JPElimC.setVisible(true);
                    if (JTContactos.isShowing())    JPDNIAuto.setVisible(true);

                    // Refrescamos el JComboBox de cargos
                    Cargo car = new Cargo();
                    UGUI.rellenarJCB(JCBCargos, car.obtenerDescripCargos());
                    car.UBD.cerrarConexion();

                    // Eliminamos el ultimo cargo insertado por el evento itemChanged
                    DefaultTableModel modelo = (DefaultTableModel) JTCargos.getModel();
                    modelo.removeRow(modelo.getRowCount()-1);
                    JTCargos.setModel(modelo);

                    // Mostramos el JPanel
                    mostrarJPanelsPersonal("GP");
                }
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los paneles de información.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* mostrarJPanelsPersonal: Metodo que muestra los paneles de insercion de personal indicados
         *
         * @param   String  Paneles     Paneles a mostrar
         * @return  void
         */
        private void mostrarJPanelsPersonal(String Paneles)
        {
            // Si se debe mostrar el general
            if (Paneles.contains("G"))
            {
                JPDP.setVisible(true);
            }
            else                        JPDP.setVisible(false);

            // Si se debe mostrar el de datos especificos del personal ocasional (empleados)
            if (Paneles.contains("Oe"))
            {
                JPOcasional.setVisible(true);
                JPResidente.setVisible(false);
                JPVigilante.setVisible(false);
            }
            // Si se debe mostrar el de datos especificos del personal ocasional (contactos)
            else if (Paneles.contains("Oc"))
            {
                JPOcasional.setVisible(false);
                JPResidente.setVisible(false);
                JPVigilante.setVisible(false);
            }
            // Si se debe mostrar el de datos especificos del personal residente (empleados)
            else if (Paneles.contains("Re"))
            {
                JPOcasional.setVisible(false);
                JPResidente.setVisible(true);
                JPVigilante.setVisible(false);
            }
            // Si se debe mostrar el de datos especificos del personal residente (contactos)
            else if (Paneles.contains("Rc"))
            {
                JPOcasional.setVisible(false);
                JPResidente.setVisible(false);
                JPVigilante.setVisible(false);
            }
            // Si se debe mostrar el de datos especificos del personal vigilante
            else if (Paneles.contains("Ve"))
            {
                JPOcasional.setVisible(false);
                JPResidente.setVisible(false);
                JPVigilante.setVisible(true);
            }
            // Si se debe mostrar el de datos especificos de los contactos de la empresa encargada de la vigilancia
            else if (Paneles.contains("Vc"))
            {
                JPOcasional.setVisible(false);
                JPResidente.setVisible(false);
                JPVigilante.setVisible(false);
            }
            // Para el resto del personal
            else
            {
                JPOcasional.setVisible(false);
                JPResidente.setVisible(false);
                JPVigilante.setVisible(false);
                JPElimTE.setVisible(false);
            }
        }

        /* borrarDatosEmp: Metodo que borra los datos del formulario de insercion de empresas
         *
         * @param   void
         * @return  void
         */
        private void borrarDatosEmp()
        {
            // Borramos todos los campos
            JTFCodEmp.setText("");
            JTFCIFEmp.setText("");
            JTFNomEmp.setText("");
            JTADirEmp.setText("");
        }

        /* borrarDatosDep: Metodo que borra los datos del formulario de insercion de departamentos, dejandolo listo
         * para una nueva insercion
         *
         * @param   void
         * @return  void
         */
        private void borrarDatosDep()
        {
            // Borramos todos los campos
            JTFCodDep.setText("");
            JTFNomDep.setText("");
            JTFPeriodicidad.setText("");

            // Hacemos que la actividad eligida por defecto sea la primera y la fecha la de hoy
            JCBActividad.setSelectedIndex(0);
            JCCFUltActu.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(Calendar.getInstance().getTime()));

            // Hacemos que el selector de DNI solo este editable para el administrador
            JPJCBDNIDep.setVisible(false);
            if (Tipo_Usu.equals("ADMINISTRADOR"))   JPJCBDNIDep.setVisible(true);
        }

        /* borrarFormDP: Metodo que borra los datos del panel de datos personales
         *
         * @param   void
         * @return  void
         */
        private void borrarFormDP()
        {
            JTFDNIDP.setText("");
            JTFApellido1DP.setText("");
            JTFApellido2DP.setText("");
            JTFNombreDP.setText("");
            JTFTfnoEmailDP.setText("");
            JTFTfnoEmailDP.setText("");
            JCBDNIDep.setSelectedIndex(-1);
        }

        /* refrescarCBActivs: Metodo que refresca el contenido de los JComboBoxes de tipos de acceso
         *
         * @param   void
         * @return  void
         */
        private void refrescarCBActivs()
        {
            try
            {
                // Creamos un objeto de tipo T_Acceso
                T_Acceso ta = new T_Acceso();

                // Obtenemos los tipos de accesos existentes
                ResultSet rs = ta.obtenerTAccesos();
                Vector v = new Vector();
                while (rs.next())           { v.add(rs.getString(1) + " - " + rs.getString(2)); }
                // Cerramos la conexion
                ta.UBD.cerrarConexion();

                // Reestablecemos los modelos de los JComboBoxes
                JCBActividad.setModel(new DefaultComboBoxModel(v));
                JCBActividad1.setModel(new DefaultComboBoxModel(v));
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista de tipos de acceso.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarCBNivs: Metodo que refresca el contenido de un JComboBox con los niveles existentes
         *
         * @param   JComboBox   jcb     JComoBox a refrescar
         * @return  void
         */
        private void refrescarCBNivs(JComboBox jcb)
        {
            try
            {
                // Creamos un objeto de tipo Nivel
                Nivel niv = new Nivel();

                // Obtenemos los tipos de niveles existentes
                ResultSet rs = niv.obtenerNiveles();
                Vector v = new Vector();
                v.add("Ninguno");
                while (rs.next())           { v.add(rs.getString(1)); }

                // Cerramos la conexion
                niv.UBD.cerrarConexion();

                // Reestablecemos el modelo del JComboBox
                jcb.setModel(new DefaultComboBoxModel(v));
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista de niveles.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarCBActivs: Metodo que refresca el contenido del JComboBox de empresas
         *
         * @param   void
         * @return  void
         */
        private void refrescarCBEmpresas()
        {
            try
            {
                // Creamos un objeto de tipo Ocasional
                Ocasional oca = new Ocasional();

                // Obtenemos las empresas para las que trabaja
                String DNI = (String) JTEmpleados.getModel().getValueAt(JTEmpleados.getSelectedRow(),0);
                ResultSet rs = oca.obtenerEmpresas(DNI);
                Vector v = new Vector();
                while (rs.next())           { v.add(rs.getString(1)); }

                // Cerramos la conexion
                oca.UBD.cerrarConexion();

                // Reestablecemos el modelo del JComboBox
                JCBEmpresas.setModel(new DefaultComboBoxModel(v));
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista de niveles.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarCBOcasional: Metodo que refresca el contenido del JComboBox de personal ocasional
         *
         * @param   void
         * @return  void
         */
        private void refrescarCBOcasional()
        {
            try
            {
                // Creamos un objeto de tipo Ocasional
                Ocasional ocas = new Ocasional();

                // Obtenemos las personas ocasionales
                ResultSet rs = ocas.obtenerOcasional();
                Vector v = new Vector();
                v.add("Ninguno");
                while (rs.next())   v.add(rs.getString(1));

                // Cerramos la conexion
                ocas.UBD.cerrarConexion();

                // Reestablecemos el modelo del JComboBox
                JCBOcasional.setModel(new DefaultComboBoxModel(v));

                // Seleccionamos el elemento que corresponda
                JCBOcasional.setSelectedIndex(0);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista de personas ocasionales.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarCBContactos: Metodo que refresca el contenido del JComboBox de contactos
         *
         * @param   void
         * @return  void
         */
        private void refrescarCBContactos()
        {
            try
            {
                // Creamos un objeto de tipo Periodico_Contacto
                Periodico_Contacto pc = new Periodico_Contacto();

                // Obtenemos los contatcos
                ResultSet rs = pc.obtenerContactos();
                Vector v = new Vector();
                v.add("Ninguno");
                while (rs.next())   v.add(rs.getString(1));

                // Cerramos la conexion
                pc.UBD.cerrarConexion();

                // Reestablecemos el modelo del JComboBox
                JCBOcasional.setModel(new DefaultComboBoxModel(v));

                // Seleccionamos el elemento que corresponda
                JCBOcasional.setSelectedIndex(0);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista de contactos.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarBotonesEmpPers: Metodo que reestablece los botones de la pestaña a su estado original
         *
         * @param   void
         * @return  void
         */
        private void refrescarBotonesEmpPers()
        {
            // Restablecemos todos los botones a su estado original
            JTBAnadirEmpresa.setSelected(false);
            JTBAnadirEmpresa.setEnabled(true);
            JBEliminarEmpresa.setEnabled(false);
            JTBModificarEmpresa.setSelected(false);
            JTBModificarEmpresa.setEnabled(false);
            JTBAnadirDepartamento.setSelected(false);
            JTBAnadirDepartamento.setEnabled(false);
            JBEliminarDepartamento.setEnabled(false);
            JTBModificarDepartamento.setSelected(false);
            JTBModificarDepartamento.setEnabled(false);
            JTBAnadirPerContac.setSelected(false);
            JTBAnadirPerContac.setEnabled(false);
            JBEliminarPerContac.setEnabled(false);
            JTBModificarPerContac.setSelected(false);
            JTBModificarPerContac.setEnabled(false);
        }
        
        /* disparador: Metodo que ejecuta el disparador que elimina todas las empresas y departamentos que no disponen de personal
         *
         * @param   void
         * @return  void
         */
        private void disparador()
        {
            try
            {
                // Eliminamos las empresas sin personal
                String query = "DELETE FROM Empresa WHERE Cod_Emp NOT IN (SELECT DISTINCT Cod_Emp FROM Periodico UNION SELECT DISTINCT Cod_Emp FROM Contacto);";
                UBD.ejecutarUpdate(query);

                // Eliminamos los departamentos sin personal
                query = "DELETE FROM Departamento WHERE (Cod_Emp,Cod_Dep) NOT IN (SELECT DISTINCT Cod_Emp,Cod_Dep FROM Periodico UNION SELECT DISTINCT Cod_Emp,Cod_Dep FROM Contacto);";
                UBD.ejecutarUpdate(query);
            }
            catch (Exception e) {}
        }
        // *******************************************************************************************



        // *******************************************************************************************
        // **************************** TARJETAS, TIPOS DE ACCESO Y CARGOS ***************************
        // *******************************************************************************************
        /* iniciarTACs: Metodo que inicializa todos los elementos referentes al tab 'Tarjetas, tipos de accesos y cargos'
         *
         * @param   void
         * @return  void
         */
        private void iniciarTACs()
        {
            // Refrescamos las tablas de tarjetas, colores, niveles, tipos de accesos y cargos
            refrescarTarjetas();
            refrescarColores();
            refrescarNiveles();
            refrescarTAccesos();
            refrescarTCargos();

            // Activamos/Desactivamos todos los botones
            activDesactivBotonesTACs();
        }

        /* refrescarTarjetas: Metodo que refresca la tabla de tarjetas existentes
         *
         * @param   void
         * @return  void
         */
        private void refrescarTarjetas()
        {
            try
            {
                // Creamos un objeto de tipo Tarjeta
                Tarjeta tarj = new Tarjeta();

                // Obtenemos los datos de las tarjetas en circulacion y rellenamos su tabla
                UGUI.rellenarTabla(JTTarjetas, tarj.obtenerTarjetas());
                // Coloreamos la tabla
                UGUI.pintarTabla(JTTarjetas, tarj.obtenerColoresTarjs());

                // Cerramos la conexion
                tarj.UBD.cerrarConexion();

                // Escondemos el panel de datos
                JPTarjetasD.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las tarjetas existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarColores: Metodo que refresca la tabla de colores existentes
         *
         * @param   void
         * @return  void
         */
        private void refrescarColores()
        {
            try
            {
                // Creamos un objeto de tipo Color
                API.Tablas.Color col = new API.Tablas.Color();

                // Obtenemos los datos de los colores existentes y rellenamos su tabla
                UGUI.rellenarTabla(JTColores, col.obtenerCeldasColores());
                // Coloreamos la tabla
                UGUI.pintarTabla(JTColores, col.obtenerColores());

                // Cerramos la conexion
                col.UBD.cerrarConexion();

                // Escondemos el panel de colores
                JPColorD.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los colores existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarNiveles: Metodo que refresca la tabla de niveles existentes
         *
         * @param   void
         * @return  void
         */
        private void refrescarNiveles()
        {
            try
            {
                // Creamos un objeto de tipo Nivel
                Nivel niv = new Nivel();

                // Obtenemos los datos de los niveles existentes y rellenamos su tabla
                UGUI.rellenarTabla(JTNiveles, niv.obtenerNiveles());

                // Cerramos la conexion
                niv.UBD.cerrarConexion();

                // Escondemos el panel de datos
                JPNivelD.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los niveles existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarTAccesos: Metodo que refresca la tabla de tipos de acceso existentes
         *
         * @param   void
         * @return  void
         */
        private void refrescarTAccesos()
        {
            try
            {
                // Creamos un objeto de tipo T_Acceso y otro de tipo Nivel
                T_Acceso ta = new T_Acceso();
                Nivel niv = new Nivel();

                // Obtenemos los datos de los tipos de acceso existentes y rellenamos su tabla
                UGUI.rellenarTabla(JTTAccesos, ta.obtenerTAccesos());

                // Obtenemos los niveles existentes y rellenamos el JComboBox
                UGUI.rellenarJCB(JCBNivelTAcceso,niv.obtenerNivelesJCB());

                // Cerramos las conexiones
                ta.UBD.cerrarConexion();
                niv.UBD.cerrarConexion();

                // Escondemos el panel de datos
                JPTAccesoD.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los accesos existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarTCargos: Metodo que refresca la tabla de cargos existentes
         *
         * @param   void
         * @return  void
         */
        private void refrescarTCargos()
        {
            try
            {
                // Creamos un objeto de tipo Cargo
                Cargo car = new Cargo();

                // Obtenemos los datos de los cargos existentes y rellenamos su tabla
                UGUI.rellenarTabla(JTCargoss, car.obtenerCargos());

                // Cerramos la conexion
                car.UBD.cerrarConexion();

                // Escondemos el panel de datos
                JPCargoD.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los cargos existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* activDesactivBotonesTACs: Metodo que des-selecciona y desactiva todos los botones de todos los paneles (salvo los botones de adicion)
         *
         * @param   void
         * @return  void
         */
        private void activDesactivBotonesTACs()
        {
            // Panel NIVELES
            JTBAnadirNivel.setSelected(false);
            JTBAnadirNivel.setEnabled(true);
            JBEliminarNivel.setEnabled(false);
            JTBModificarNivel.setSelected(false);
            JTBModificarNivel.setEnabled(false);
            JBBajaNivel.setEnabled(false);

            // Panel COLORES
            JTBAnadirColor.setSelected(false);
            if (JTNiveles.getModel().getRowCount() == 0)    JTBAnadirColor.setEnabled(false);
            else                                            JTBAnadirColor.setEnabled(true);
            JBEliminarColor.setEnabled(false);
            JTBModificarColor.setSelected(false);
            JTBModificarColor.setEnabled(false);
            JBBajaColor.setEnabled(false);

            // Panel TARJETAS
            JTBAnadirTarjeta.setSelected(false);
            if (JTColores.getModel().getRowCount() == 0)    JTBAnadirTarjeta.setEnabled(false);
            else                                            JTBAnadirTarjeta.setEnabled(true);
            JBEliminarTarj.setEnabled(false);
            JTBModificarTarj.setSelected(false);
            JTBModificarTarj.setEnabled(false);
            JBBajaTarj.setEnabled(false);

            // Panel T_ACCESOS
            JTBAnadirTAcceso.setSelected(false);
            if (JTNiveles.getModel().getRowCount() == 0)    JTBAnadirTAcceso.setEnabled(false);
            else                                            JTBAnadirTAcceso.setEnabled(true);
            JBEliminarTAcceso.setEnabled(false);
            JTBModificarTAcceso.setSelected(false);
            JTBModificarTAcceso.setEnabled(false);

            // Panel CARGOS
            JTBAnadirCargo.setSelected(false);
            if (JTCargoss.getModel().getRowCount() == 0)    JTBAnadirCargo.setEnabled(false);
            else                                            JTBAnadirCargo.setEnabled(true);
            JBEliminarCargo.setEnabled(false);
            JTBModificarCargo.setSelected(false);
            JTBModificarCargo.setEnabled(false);
        }

        /* comprobarHorasTAcceso: Metodo que comprueba que las horas del tipo de acceso sean
         * correctas, realizando los cambios oportunos en caso contrario
         *
         * @param   void
         * @return  void
         */
        private void comprobarHorasTAcceso()
        {
            try
            {
                // Comprobacion de fechas
                DateFormat dt = new SimpleDateFormat ("HH:mm");
                if (dt.parse((String) JCBHITAccesos.getSelectedItem()).after(dt.parse((String) JCBHFTAccesos.getSelectedItem())))
                {
                    JCBHFTAccesos.getModel().setSelectedItem((String) JCBHITAccesos.getSelectedItem());
                    JCBHITAccesos.getModel().setSelectedItem((String) JCBHFTAccesos.getSelectedItem());
                    if (JTBAnadirTAcceso.isSelected())  JOptionPane.showMessageDialog(this,"La hora de fin debe ser posterior a la de inicio.\nSe ha modificado la hora de fin.","AVISO",JOptionPane.WARNING_MESSAGE);
                }
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible realizar la comprobación de horas.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // *******************************************************************************************



        // *******************************************************************************************
        // *************************** ZONAS, LLAVES, LLAVEROS Y ALMACENES ***************************
        // *******************************************************************************************
        /* iniciarZonas: Metodo que inicializa todos los elementos referentes al tab 'Zonas, llaves y llaveros'
         *
         * @param   void
         * @return  void
         */
        private void iniciarZonas()
        {
            // Refrescamos la tabla de zonas y llaves
            refrescarZonas();
            // Deshabilitamos todos los botones necesarios
            deshabilitarBotonesZonas();
            // Refrescamos las tablas de almacenes
            refrescarAlmacenes();
            // Escondemos el JPanel de PDFs
            JPPrestsPDF1.setVisible(false);
        }

        /* refrescarZonas: Metodo que refresca la tabla de zonas
         *
         * @param   void
         * @return  void
         */
        private void refrescarZonas()
        {
            try
            {
                // Refrescamos la tabla de zonas
                Zona z = new Zona();
                UGUI.rellenarTabla(JTZonas,z.obtenerZonas());

                // Cerramos la conexion
                z.UBD.cerrarConexion();

                // Vaciamos las tablas de zonas
                vaciarTablasZonas();

                // Refrescamos el JCB de niveles
                refrescarCBNivs(JCBNivelZona);

                // Ocultamos los botones de llaves
                JPLlaveB.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las zonas existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* deshabilitarBotonesZonas: Metodo que deshabilita todos los botones no iniciales
         *
         * @param   void
         * @return  void
         */
        private void deshabilitarBotonesZonas()
        {
                // Ocultamos el panel de datos, los botones de añadir y eliminar prestables
                JPZonaD.setVisible(false);
                JPLlaveD.setVisible(false);
                JTBAnadirL.setSelected(false);
                JTBAnadirL.setEnabled(true);
                JTBModificarLlave.setSelected(false);
                JTBModificarLlave.setEnabled(false);
                JBDarBajaL.setEnabled(false);
                JBEliminarL.setEnabled(false);
                JBAnadirLPrestEmpDep.setEnabled(false);
                JBAnadirLPrestOcas.setEnabled(false);
                JBEliminarLPrestEmpDep.setEnabled(false);
                JBEliminarLPrestOcas.setEnabled(false);
                JPLsPrestB.setVisible(false);
        }

        /* refrescarAlmacenes: Metodo que refresca las tablas de almacenes
         *
         * @param   void
         * @return  void
         */
        private void refrescarAlmacenes()
        {
            try
            {
                // Rellenamos la tabla de almacenes
                Almacen a = new Almacen();
                String Filtro = JTFBuscarLLl.getText().equals("Buscar ...") ? "" : JTFBuscarLLl.getText();
                // Almacenes
                UGUI.rellenarTabla(JTAlms, a.obtenerAlmacenes(Filtro));
                a.UBD.cerrarConexion();
                // Llaves
                UGUI.rellenarTabla(JTLs, a.obtenerLlavesTotales(Filtro));
                a.UBD.cerrarConexion();
                // Llaveros
                UGUI.rellenarTabla(JTLls, a.obtenerLlaverosTotales(Filtro));
                a.UBD.cerrarConexion();
                // Seleccionamos el primer almacen
                JTAlms.setRowSelectionInterval(0,0);

                // Escondemos los JPanels de numero de copias, de datos del almacen y de insercion de llaveros
                JPJTFNumCopsL.setVisible(false);
                JPJTFNumCopsLl.setVisible(false);
                JPLls.setVisible(false);
                JPAlmD.setVisible(false);
                JPLlaveroD.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los almacenes existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* vaciarTablasZonas: Metodo que vacia las tablas de llaves y prestables asociadas a las zonas
         *
         * @param   void
         * @return  void
         */
        private void vaciarTablasZonas()
        {
            // Llaves
            String[] Etiquetas = {"Código","Descripción"};
            JTLsZona.setModel((new DefaultTableModel(Etiquetas,0)));

            // Prestables
            vaciarTablasPrestables();

            // Escondemos los botones de añadir/eliminar prestables
            JPLsPrestB.setVisible(false);
        }

        /* vaciarTablasPrestables: Metodo que vacia las tablas de prestables
         *
         * @param   void
         * @return  void
         */
        private void vaciarTablasPrestables()
        {
            // Prestables
            String [] Etiquetas = {"Empresa","Departamento"};
            JTLsPrestEmpDep.setModel((new DefaultTableModel(Etiquetas,0)));
            Etiquetas[0] = "DNI";
            Etiquetas[1] = "Empresa";
            JTLsPrestOcas.setModel((new DefaultTableModel(Etiquetas,0)));
        }

         /* activarTablasPrestablesActuales: Metodo que crea y muestra las tabla con todos aquellos departamentos
         * y personal ocasional que optan al prestamo de la llaver al prestamo y no disponen de el aun
         *
         * @param   void
         * @return  void
         */
        private void activarTablasPrestablesActuales()
        {
            try
            {
                Llave l = new Llave ((Integer) JTLsZona.getModel().getValueAt(JTLsZona.getSelectedRow(),0));
                UGUI.rellenarTabla(JTLsPrestEmpDep, l.obtenerEmpDeps());
                UGUI.rellenarTabla(JTLsPrestOcas, l.obtenerOcasional());
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los prestatarios actuales posibles.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* activarTablasPrestablesPosibles: Metodo que crea y muestra las tabla con todos aquellos departamentos
         * y personal ocasional que pueden optar al prestamo y no disponen de el aun
         *
         * @param   void
         * @return  void
         */
        private void activarTablasPrestablesPosibles()
        {
            try
            {
                // Rellenamos las tablas de departamentos y personal ocasional
                int filaSelec = JTLsZona.getSelectedRow();
                Llave l;
                if (JTLsZona.getSelectedRow() != -1)    l = new Llave ((Integer) JTLsZona.getModel().getValueAt(filaSelec,0));
                else                                    l = new Llave();
                UGUI.rellenarTabla(JTLsPrestEmpDepResto, l.obtenerEmpDepsResto());
                UGUI.rellenarTabla(JTLsPrestOcasResto, l.obtenerOcasionalResto());

                // Mostramos el panel que aglutina a ambas tablas
                JPLsPrestB.setVisible(true);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las tablas de departamentos y personal ocasional.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* insertarPrestables: Metodo que proporciona a los departamentos y personal ocasional seleccionados en
         * las tablas de prestables la posibilidad de que puedan tomar en prestamo la llave indicada
         *
         * @param   Llave       Llave       Llave que puede ser prestada
         * @return  void
         */
        private void insertarPrestables(Llave Llave) throws DataException, SQLException, RemoteException
        {
            // Recorremos el vector generando cada empresa-departamento e insertandole la llave
            DefaultTableModel modelo = (DefaultTableModel) JTLsPrestEmpDep.getModel();
            Vector datos = modelo.getDataVector();
            Empresa emp = new Empresa();
            Iterator it = datos.iterator();
            String NomEmp;
            Vector tupla;
            Departamento dep = new Departamento();
            while (it.hasNext())
            {
                // Obtenemos el codigo de empresa
                tupla = (Vector) it.next();
                NomEmp = (String) tupla.get(0);
                emp.setNombre(NomEmp);
                emp.setCodEmp(emp.obtenerCodEmp());
                // Obtenemos el codigo de departamento
                dep.setCodDep(emp.obtenerCodDep((String) tupla.get(1)));

                // Insertamos la llave
                emp.insertLlaveBD(dep,Llave);
            }

            // Recorremos el vector generando cada persona ocasional e insertandole la llave
            modelo = (DefaultTableModel) JTLsPrestOcas.getModel();
            datos = modelo.getDataVector();
            Ocasional ocas;
            it = datos.iterator();
            while (it.hasNext())
            {
                // Obtenemos la persona ocasional
                tupla = (Vector) it.next();
                ocas = new Ocasional ((String)tupla.get(0),(String)tupla.get(1));

                // Insertamos la llave
                ocas.insertLlaveBD(Llave);
            }

            // Vaciamos las tablas de prestables para dejar todo como al principio
            vaciarTablasPrestables();
        }
        // *******************************************************************************************

        

        // *******************************************************************************************
        // **************************************** PRESTAMOS ****************************************
        // *******************************************************************************************
        /* iniciarPrestamos: Metodo que inicializa todos los elementos referentes al tab 'Prestamos'
         *
         * @param   void
         * @return  void
         */
        private void iniciarPrestamos()
        {
            // Refrescamos la tabla de tarjetas prestadas
            refrescarTarjsPrest();
            // Refrescamos las tablas de llaves y llaveros prestados
            refrescarLsLlsPrest();
            // Escondemos el panel de datos
            JPDatosPrest.setVisible(false);
            // Escondemos el JPanel de PDFs
            JPPrestsPDF.setVisible(false);
            // Des-seleccionamos el boton de reportes
            JTBActivarPanelPDF.setSelected(false);
        }

        /* refrescarTarjsPrest: Metodo que refresca la tabla de tarjetas prestadas
         *
         * @param   void
         * @return  void
         */
        private void refrescarTarjsPrest()
        {
            try
            {
                // Creamos un objeto de tipo tarjeta
                Tarjeta tarj = new Tarjeta();

                // Obtenemos los datos de las tarjetas en prestamo y refrescamos la tabla
                UGUI.rellenarTabla(JTTarjsPrest, tarj.obtenerPrestamos());
                // Coloreamos la tabla
                UGUI.pintarTabla(JTTarjsPrest, tarj.obtenerColoresTarjsPrest());
                
                // Cerramos la conexion
                tarj.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las tarjetas en préstamo.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarLsLlsPrest: Metodo que refresca las tablas de llaves y llaveros prestados
         *
         * @param   void
         * @return  void
         */
        private void refrescarLsLlsPrest()
        {
            try
            {
                // Creamos el objeto de tipo Llave
                Llave l = new Llave();
                // Creamos el objeto de tipo Llavero
                Llavero ll = new Llavero();

                // Refrescamos la tabla de llaves en prestamo
                UGUI.rellenarTabla(JTLsPrest,l.obtenerPrestamos());
                // Refrescamos la tabla de llaveros en prestamo
                UGUI.rellenarTabla(JTLlsPrest,ll.obtenerPrestamos());

                // Cerramos las conexiones
                l.UBD.cerrarConexion();
                ll.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las llaves y/o llaveros en préstamo.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* mostrarDatosPrestatario: Metodo que muestra los datos personales del prestatario
         *
         * @param   void
         * @return  void
         */
        private void mostrarDatosPrestatario(Personal Prestatario)
        {
            try
            {
                // Mostramos los datos personales
                JTFNombrePrest.setText(Prestatario.getApellido1() + " " + Prestatario.getApellido2() + ", " + Prestatario.getNombre());
                JTFTipoPrest.setText(Prestatario.obtenerTipo());
                JTFEmpresaPrest.setText(Prestatario.obtenerEmpresa());

                // Rellenamos los telefonos del prestatario
                UGUI.rellenarTabla(JTTfnsPrest, Prestatario.obtenerTfns());

                // Si dispone de algun e-mail -> Activamos el boton de enviar recordatorio
                if (Prestatario.getEmails().isEmpty())      JBRecordPrest.setEnabled(false);
                else                                        JBRecordPrest.setEnabled(true);

                // Mostramos el panel de datos si no estaba siendo mostrado
                if (!JPDatosPrest.isVisible())  JPDatosPrest.setVisible(true);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los datos personales del prestatario.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // *******************************************************************************************

        

        // *******************************************************************************************
        // *********************************** PARTES DE SEGURIDAD ***********************************
        // *******************************************************************************************
        /* iniciarPartes: Metodo que inicializa todos los elementos referentes al tab 'Partes de seguridad'
         *
         * @param   void
         * @return  void
         */
        private void iniciarPartes()
        {
            // Rellenamos el JComboBox con los vigilantes que hayan redactado partes
            refrescarCBVigs();
            // Rellenamos la JTable de partes redactados segun los criterios de busqueda iniciales
            refrescarTPartes();
            // Escondemos el formulario de muestra de partes
            JPParte.setVisible(false);
        }

        /* refrescarCBVigs: Metodo que rellena el JComboBox con los vigilantes que hayan redactado partes
         *
         * @param   void
         * @return  void
         */
        private void refrescarCBVigs()
        {
            try
            {
                // Creamos un objeto de tipo vigilante
                Vigilante vig = new Vigilante();

                // Obtenemos los codigos de vigilante de todos los vigilantes del centro
                ResultSet rs = vig.obtenerCodsVigilantes();
                Vector v = new Vector();
                v.add("Todos");
                while (rs.next())           { v.add(rs.getString(1)); }
                // Cerramos la conexion
                vig.UBD.cerrarConexion();

                // Reestablecemos el modelo del JComboBox
                JCBCodVig.setModel(new DefaultComboBoxModel(v));
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los códigos de los vigilantes de seguridad.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarTPartes: Metodo que refresca la tabla de partes de seguridad
         *
         * @param   void
         * @return  void
         */
        private void refrescarTPartes()
        {
            try
            {
                // Creamos el objeto de tipo Parte
                Parte_Seguridad parte = new Parte_Seguridad();

                // Comprobamos que las fechas y horas sean correctas
                comprobarFechasHorasPartes();

                // Obtenemos la fecha y horas entre las que consultar los partes, asi como el DNI
                String fecha_inic = null;
                String fecha_fin = null;
                String hora_inic = null;
                String hora_fin = null;
                String Cod_Vig = (String) JCBCodVig.getModel().getSelectedItem();

                if (JCBFechasPartes.isSelected())
                {
                    fecha_inic = new SimpleDateFormat("yyyy-MM-dd").format(JCCFechaInicParte.getDate());
                    fecha_fin = new SimpleDateFormat("yyyy-MM-dd").format(JCCFechaFinParte.getDate());
                }
                if (!JCBTodoDiaParte.isSelected())
                {
                    hora_inic = (String) JCBHoraInicParte.getSelectedItem();
                    hora_fin = (String) JCBHoraFinParte.getSelectedItem();
                }

                // Refrescamos la tabla de partes
                UGUI.rellenarTabla(JTPartes,parte.obtenerPartes(fecha_inic,fecha_fin,hora_inic,hora_fin,Cod_Vig));
                // Cerramos la conexion
                parte.UBD.cerrarConexion();

                // Desactivamos los botones de 'Eliminar parte' y 'Modificar parte'
                JBEliminarParte.setEnabled(false);
                JBModificarParte.setEnabled(false);

                // Desactivamos el formulario
                JCCFechaParte.setEnabled(false);
                JCBParte.setEnabled(false);
                JTAIncidencias.setEditable(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los partes de seguridad existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* comprobarFechasHorasPartes: Metodo que comprueba que las fechas y horas de busqueda de partes de seguridad
         * sean correctas, realizando los cambios oportunos en caso contrario
         *
         * @param   void
         * @return  void
         */
        private void comprobarFechasHorasPartes()
        {
            try
            {
                // Variable que denota si hay que avisar al usuario sobre el cambio de fecha-hora
                boolean aviso = false;

                // Comprobacion de fechas
                // Si la fecha de inicio es posterior a la de fin, avisamos del error y establecemos
                // la fecha de fin igual a la de inicio
                Calendar JCC = JCCFechaInicParte.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),0,0,0);
                Calendar ci = Calendar.getInstance();
                ci.setTime(JCC.getTime());
                JCC = JCCFechaFinParte.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),23,59,59);
                Calendar cf = Calendar.getInstance();
                cf.setTime(JCC.getTime());

                if (!cf.after(ci))
                {
                    JCCFechaFinParte.setDate(JCCFechaInicParte.getDate());
                    JCCFechaFinParte.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(JCCFechaInicParte.getDate()));
                    JCCFechaFinParte.setVisible(false);
                    JCCFechaFinParte.setVisible(true);
                    aviso = true;
                }

                // Comprobacion de horas, en caso necesario
                JCC = JCCFechaInicParte.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),23,59,59);
                cf = Calendar.getInstance();
                ci.setTime(JCC.getTime());
                JCC = JCCFechaFinParte.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),0,0,0);
                cf = Calendar.getInstance();
                cf.setTime(JCC.getTime());
                if (!JCBTodoDiaParte.isSelected() && (!cf.after(ci)))
                {
                    DateFormat dt = new SimpleDateFormat ("HH:mm");
                    if (dt.parse((String) JCBHoraInicParte.getSelectedItem()).after(dt.parse((String) JCBHoraFinParte.getSelectedItem())))
                    {
                        JCBHoraFinParte.getModel().setSelectedItem((String) JCBHoraInicParte.getSelectedItem());
                        JCBHoraInicParte.getModel().setSelectedItem((String) JCBHoraFinParte.getSelectedItem());
                        aviso = true;
                    }
                }

                if (aviso)  JOptionPane.showMessageDialog(this,"La fecha-hora de fin debe ser posterior a la de inicio.\nSe han modificado la fecha y/o hora de fin.","AVISO",JOptionPane.WARNING_MESSAGE);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible realizar la comprobación de fechas y horas.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // *******************************************************************************************



        // *******************************************************************************************
        // ***************************************** CUENTAS *****************************************
        // *******************************************************************************************
        /* iniciarCuentas: Metodo que inicializa todos los elementos referentes al tab 'Cuentas'
         *
         * @param   void
         * @return  void
         */
        private void iniciarCuentas()
        {
            // JPanel de cuentas
            // Inicializamos JCBDNI
            refrescarCBDNIs();
            // Inicializamos JCBTipo
            refrescarCBTUsuarios();
            // Rellenamos las tablas de cuentas, tipos de usuarios y accesos al sistema, escondiendo los JPanels de datos y estableciendo sus JTFields a ""
            refrescarCuentas();
        }
        
        /* refrescarCBDNIs: Metodo que rellena el JComboBox de DNIs del personal que puede tener cuentas en el sistema
         *
         * @param   void
         * @return  void
         */
        private void refrescarCBDNIs()
        {
            try
            {
                // Creamos un objeto de tipo Personal
                Personal pers = new Personal();

                // Obtenemos los datos del personal registrado en el sistema
                ResultSet rs = pers.obtenerPersonalCuentas(JTFBuscarCuenta.getText().equals("Buscar ...") ? "" : JTFBuscarCuenta.getText());
                Vector v = new Vector();
                while (rs.next())           { v.add(rs.getString(1)); }
                // Cerramos la conexion
                pers.UBD.cerrarConexion();

                // Reestablecemos el modelo del JComboBox
                JCBDNICuenta.setModel(new DefaultComboBoxModel(v));
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista del personal existente.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarCBTUsuarios: Metodo que rellena el JComboBox de tipos de usuarios (cuentas) que pueden existir en el sistema
         *
         * @param   void
         * @return  void
         */
        private void refrescarCBTUsuarios()
        {
            try
            {
                // Creamos un objeto de tipoT_Usuario
                T_Usuario tu = new T_Usuario();

                // Obtenemos los datos de los tipos de usuarios existentes
                ResultSet rs = tu.obtenerCodsTUsuarios();
                Vector v = new Vector();
                while (rs.next())           { v.add(rs.getString(1)); }

                // Cerramos la conexion
                tu.UBD.cerrarConexion();

                // Reestablecemos el modelo del JComboBox
                JCBTipoCuenta.setModel(new DefaultComboBoxModel(v));
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista de tipos de usuarios existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarCuentas: Metodo que refresca todas las tablas pertenecientes al tab 'Cuentas'
         *
         * @param   void
         * @return  void
         */
        private void refrescarCuentas()
        {
            // Refrescamos la tabla de CUENTAS
            refrescarTCuentas();

            // Refrescamos la tabla de T_USUARIOS
            refrescarTUsuarios();

            // Refrescamos la tabla de HISTORIAL DE ACCESOS AL SISTEMA
            refrescarTAccsSist();
        }

        /* refrescarTCuentas: Metodo que refresca la tabla de cuentas existentes en el sistema
         *
         * @param   void
         * @return  void
         */
        private void refrescarTCuentas()
        {
            try
            {
                // Creamos un objeto de tipo Cuenta
                Cuenta cu = new Cuenta();

                // Refrescamos la tabla de cuentas
                UGUI.rellenarTabla(JTCuentas,cu.obtenerCuentas());
                // Cerramos la conexion
                cu.UBD.cerrarConexion();

                // Des-seleccionamos el boton de 'Añadir cuenta' y deshabilitamos los de 'Eliminar cuenta' y 'Modificar cuenta'
                JTBAnadirCuenta.setSelected(false);
                JTBAnadirCuenta.setEnabled(true);
                JBEliminarCuenta.setEnabled(false);
                JTBModificarCuenta.setSelected(false);
                JTBModificarCuenta.setEnabled(false);

                // Escondemos el JPanel
                JPDatosCuenta.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar las cuentas existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarTUsuarios: Metodo que refresca la tabla de tipos de usuarios existentes en el sistema
         *
         * @param   void
         * @return  void
         */
        private void refrescarTUsuarios()
        {
            try
            {
                // Creamos un objeto de tipo T_Usuario
                T_Usuario tu = new T_Usuario();

                // Refrescamos la tabla de tipos de usuario
                UGUI.rellenarTabla(JTTUsuarios,tu.obtenerTUsuarios());
                // Cerramos la conexion
                tu.UBD.cerrarConexion();

                // Des-seleccionamos el boton de 'Añadir tipo de usuario' y deshabilitamos los de 'Eliminar tipo de usuario' y 'Modificar tipo de usuario'
                JTBAnadirTUsuario.setSelected(false);
                JTBAnadirTUsuario.setEnabled(true);
                JBEliminarTUsuario.setEnabled(false);
                JTBModificarTUsuario.setSelected(false);
                JTBModificarTUsuario.setEnabled(false);

                // Escondemos el JPanel
                JPDatosTUsuario.setVisible(false);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los tipos de usuarios existentes.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarTAccsSist: Metodo que refresca la tabla de los accesos realizados en el sistema
         *
         * @param   void
         * @return  void
         */
        private void refrescarTAccsSist()
        {
            try
            {
                // Creamos un objeto de tipo Acceso_Sistema
                Acceso_Sistema as = new Acceso_Sistema();

                // Comprobamos que las fechas y horas sean correctas
                comprobarFechasHorasAccsSistema();

                // Obtenemos la fecha y horas entre las que consultar los accesos al sistema, asi como el filtro
                String fecha_inic = new SimpleDateFormat("yyyy-MM-dd").format(JCCFechaInicAccsSist.getDate());
                String fecha_fin = new SimpleDateFormat("yyyy-MM-dd").format(JCCFechaFinAccsSist.getDate());
                String hora_inic = null;
                String hora_fin = null;
                String filtro = JTFBuscarAccesosSist.getText();

                if (!JCBTodoDiaAccsSist.isSelected())
                {
                    hora_inic = (String) JCBHoraInicAccsSist.getSelectedItem();
                    hora_fin = (String) JCBHoraFinAccsSist.getSelectedItem();
                }

                // Refrescamos la tabla de accesos realizados al sistema
                UGUI.rellenarTabla(JTHistorialAccesosSist,as.obtenerAccesosSistema(fecha_inic,fecha_fin,hora_inic,hora_fin,filtro.equals("Buscar ...") ? "" : filtro));
                // Cerramos la conexion
                as.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar los accesos realizados en el sistema.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* comprobarFechasHorasAccsSist: Metodo que comprueba que las fechas y horas de busqueda de accesos al sistema
         * sean correctos, realizando los cambios oportunos en caso contrario
         *
         * @param   void
         * @return  void
         */
        private void comprobarFechasHorasAccsSistema()
        {
            try
            {
                // Variable que denota si hay que avisar al usuario sobre el cambio de fecha-hora
                boolean aviso = false;

                // Comprobamos las fechas
                // Si la fecha de inicio es posterior a la de fin, avisamos del error y establecemos
                // la fecha de fin igual a la de inicio
                Calendar JCC = JCCFechaInicAccsSist.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),0,0,0);
                Calendar ci = Calendar.getInstance();
                ci.setTime(JCC.getTime());
                JCC = JCCFechaFinAccsSist.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),23,59,59);
                Calendar cf = Calendar.getInstance();
                cf.setTime(JCC.getTime());

                if (!cf.after(ci))
                {
                    JCCFechaFinAccsSist.setDate(JCCFechaInicAccsSist.getDate());
                    JCCFechaFinAccsSist.getModel().setSelectedItem(new SimpleDateFormat("EEEE dd 'de' MMMMM 'de' yyyy").format(JCCFechaInicAccsSist.getDate()));
                    JCCFechaFinAccsSist.setVisible(false);
                    JCCFechaFinAccsSist.setVisible(true);
                    aviso = true;
                }

                // Comprobacion de horas, en caso necesario
                JCC = JCCFechaInicAccsSist.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),23,59,59);
                cf = Calendar.getInstance();
                ci.setTime(JCC.getTime());
                JCC = JCCFechaFinAccsSist.getCalendar();
                JCC.set(JCC.get(JCC.YEAR),JCC.get(JCC.MONTH),JCC.get(JCC.DAY_OF_MONTH),0,0,0);
                cf = Calendar.getInstance();
                cf.setTime(JCC.getTime());
                if (!JCBTodoDiaAccsSist.isSelected() && (!cf.after(ci)))
                {
                    DateFormat dt = new SimpleDateFormat ("HH:mm");
                    if (dt.parse((String) JCBHoraInicAccsSist.getSelectedItem()).after(dt.parse((String) JCBHoraFinAccsSist.getSelectedItem())))
                    {
                        JCBHoraFinAccsSist.getModel().setSelectedItem((String) JCBHoraInicAccsSist.getSelectedItem());
                        JCBHoraInicAccsSist.getModel().setSelectedItem((String) JCBHoraFinAccsSist.getSelectedItem());
                        aviso = true;
                    }
                }

                // Mostramos el aviso del error encontrado y la respuesta realizada
                if (aviso)  JOptionPane.showMessageDialog(this,"La fecha-hora de fin debe ser posterior a la de inicio.\nSe han modificado la fecha y/o hora de fin.","AVISO",JOptionPane.WARNING_MESSAGE);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible realizar la comprobación de fechas y horas.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // *******************************************************************************************



        // *******************************************************************************************
        // ****************************************** NOTAS ******************************************
        // *******************************************************************************************
        /* iniciarNotas: Metodo que inicializa todos los elementos referentes al tab 'Notas'
         *
         * @param   void
         * @return  void
         */
        private void iniciarNotas()
        {
            // Rellenamos la JTable que muestra la lista de usuarios a los que poder dirigir notas
            refrescarContactos();
            // Rellenamos las JTables de los mensajes existentes en la bandeja de entrada y en la bandeja de salida
            refrescarBandejas();
            // Escondemos el formulario de muestra de notas
            JPContenidoNota.setVisible(false);
        }

        /* refrescarContactos: Metodo que rellena el JTable que muestra la lista de usuarios a los que poder dirigir notas
         *
         * @param   void
         * @return  void
         */
        private void refrescarContactos()
        {
            try
            {
                // Creamos un objeto de tipo Nota
                Nota nota = new Nota();

                // Obtenemos el filtro
                String Filtro = JTFBuscarContac.getText();

                // Rellenamos la tabla de destinatarios (contactos)
                UGUI.rellenarTabla(JTContactosNotas,nota.obtenerDestinatarios(Filtro.equals("Buscar ...") ? "" : Filtro));

                // Cerramos la conexion
                nota.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la lista de contactos.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarBandejas: Metodo que rellena las JTables de los mensajes existentes en la bandeja de entrada y en la bandeja de salida
         *
         * @param   void
         * @return  void
         */
        private void refrescarBandejas()
        {
            // Refrescamos la bandeja de entrada
            refrescarBandejaEntrada();
            // Refrescamos la bandeja de salida
            refrescarBandejaSalida();
        }

        /* refrescarBandejaEntrada: Metodo que refrescar los mensajes (notas) existentes en la bandeja de entrada
         *
         * @param   void
         * @return  void
         */
        private void refrescarBandejaEntrada()
        {
            try
            {
                // Creamos un objeto de tipo Nota
                Nota nota = new Nota();

                // Obtenemos las notas dirigidas al usuario
                ResultSet rs = nota.obtenerBandejaEntrada(this.DNI_Usu);

                // Rellenamos la bandeja de entrada
                UGUI.rellenarTabla(JTEntrada,rs);

                // Cerramos la conexion
                nota.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la bandeja de entrada.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* refrescarBandejaSalisa: Metodo que refrescar los mensajes (notas) existentes en la bandeja de salida (enviados)
         *
         * @param   void
         * @return  void
         */
        private void refrescarBandejaSalida()
        {
            try
            {
                // Creamos un objeto de tipo Nota
                Nota nota = new Nota();

                // Obtenemos las notas redactadas por el usuario
                ResultSet rs = nota.obtenerBandejaSalida(this.DNI_Usu);

                // Rellenamos la bandeja de salida
                UGUI.rellenarTabla(JTSalida,rs);

                // Cerramos la conexion
                nota.UBD.cerrarConexion();
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible mostrar la bandeja de salida.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }

        /* mostrarNota: Metodo que recibe una Nota y la muestra por pantalla
         *
         * @param   Nota    Nota    Nota a mostrar por pantalla
         * @return  void
         */
        private void mostrarNota(Nota nota)
        {
            // Establecemos los campos de la nota
            JTFAsuntoNota.setText(nota.getTema());
            JTFRedactorNota.setText(nota.getRedactor().getApellido1() + " " + nota.getRedactor().getApellido2() + ", " + nota.getRedactor().getNombre());
            JTFEnviado.setText(UCAL.calendarToString(nota.getFechaI()));
            JTATextoNota.setText(nota.getTexto());

            // Obtenemos los destinatarios (creamos la String de destinatarios)
            String Destinatario = "";
            String Destinatarios = "";
            int longitud = 0;
            Iterator it = nota.getDestinatarios().entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry VKDest = (Map.Entry) it.next();
                Destinatario dest = (Destinatario) VKDest.getValue();

                if (longitud + Destinatario.length() > 90)
                {
                    Destinatarios = Destinatarios + "\n";
                    longitud = 0;
                }

                Destinatario = dest.getReceptor().getApellido1() + " " + dest.getReceptor().getApellido2() + ", " + dest.getReceptor().getNombre();
                // Si se trata de una nota de entrada, no mostramos quienes la han leido
                if (JPEntrada.isShowing())      Destinatarios = Destinatarios + Destinatario + "; ";
                // En cambio, si se trata de una nota enviada, mostramos quienes la han leido
                else                            Destinatarios = Destinatarios + Destinatario + (dest.getLeida() ? " (Leída)" : "") + "; ";
                longitud = longitud + Destinatario.length() + 2;
            }
            Destinatarios = Destinatarios.substring(0,Destinatarios.length()-2);

            // Establecemos el campo de destinatarios
            JTAPara.setText(Destinatarios);
        }

        /* borrarFormNotaLect: Metodo que borra todos los campos destinados a mostrar los datos de las notas
         *
         * @param   void
         * @return  void
         */
        private void borrarFormNotaLect()
        {
            // Borramos el asunto
            JTFAsuntoNota.setText("");
            // Borramos el autor
            JTFRedactorNota.setText("");
            // Borramos la fecha
            JTFEnviado.setText("");
            // Borramos los destinatarios
            JTAPara.setText("");
            // Borramos el texto
            JTATextoNota.setText("");
        }

        /* borrarFormNotaEscr: Metodo que borra todos los campos destinados a redactar notas
         *
         * @param   void
         * @return  void
         */
        private void borrarFormNotaEscr()
        {
            // Borramos el asunto
            JTFAsunto.setText("");
            // Borramos el texto
            JTATexto.setText("");
        }

        /* enviarNota: Metodo que recoge todos los datos necesarios de la interfaz y envia la nota
         *
         * @param   void
         * @return  void
         */
        private void enviarNota()
        {
            try
            {
                // Obtenemos los destinatarios de la nota
                HashMap<String,Destinatario> hm = new HashMap();

                int[] destSelecs = JTContactosNotas.getSelectedRows();
                Personal pers = new Personal();
                for (int i=0; i<destSelecs.length; i++)
                {
                    // Obtenemos el DNI del contacto
                    pers.setApellido1((String) JTContactosNotas.getModel().getValueAt(destSelecs[i],0));
                    pers.setApellido2((String) JTContactosNotas.getModel().getValueAt(destSelecs[i],1));
                    pers.setNombre((String) JTContactosNotas.getModel().getValueAt(destSelecs[i],2));

                    // Insertamos el destinatario en el HashMap de destinatarios
                    hm.put(pers.obtenerDNI(), new Destinatario(new Personal(pers.obtenerDNI())));
                }

                // Creamos la nota
                Nota nota = new Nota(Calendar.getInstance(),new Personal(DNI_Usu),JTFAsunto.getText(),JTATexto.getText(),hm);

                // Insertamos la nota en la BD
                nota.insertNotaBD();

                // Borramos el formulario de redaccion de la nota
                borrarFormNotaEscr();

                // Des-seleccionamos los contactos
                JTContactosNotas.clearSelection();

                // Refrescamos las bandejas de entrada y salida
                refrescarBandejas();

                // Advertimos de que la nota ha sido enviada correctamente
                JOptionPane.showMessageDialog(this,"La nota ha sido enviada correctamente.","AVISO",JOptionPane.INFORMATION_MESSAGE);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible enviar la nota.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // *******************************************************************************************



        // *******************************************************************************************
        // ************************************* MANTENER ACCESO *************************************
        // *******************************************************************************************
        /* mantenerAcceso: Metodo que mantiene la tabla Acceso, asignando salida a todos los
         * accesos del dia anterior que no la tuvieran
         *
         * @param   void
         * @return  void
         */
        private void mantenerAcceso()
        {
            try
            {
                // Obtenemos los accesos que no disponen de salida
                Acceso acc = new Acceso();
                ArrayList<Acceso> al = acc.obtenerAccesosSinSalida();

                // Recorremos el ArrayList de accesos dotando a cada uno de fecha de salida
                Iterator it = al.iterator();
                while (it.hasNext())
                {
                    // Obtenemos el acceso
                    acc = (Acceso) it.next();
                    // Lo dotamos de salida
                    acc.darSalidaAutomatica();
                }
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible dar salida a los accesos realizados ayer\ny quecarecen de ella.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // *******************************************************************************************


        
        // *******************************************************************************************
        // ***************************************** CORREOS *****************************************
        // *******************************************************************************************
        /* enviarEmails: Metodo que envia todos los emails de peticion de devolucion de prestamos y
         * de actualizacion de personal correspondientes al dia de hoy
         *
         * @param   void
         * @return  void
         */
        private void enviarEmails()
        {
            try
            {
                // Enviamos los E-Mails de actualizacion del personal
                Cartero.enviarCorreosRenovacionPersonal(30);
                Cartero.enviarCorreosRenovacionPersonal(15);
                Cartero.enviarCorreosRenovacionPersonal(7);

                // Enviamos los E-Mails de peticion de devolucion de prestamos
                Cartero.enviarCorreosPrestamosPendientes(30);
                Cartero.enviarCorreosPrestamosPendientes(15);
                Cartero.enviarCorreosPrestamosPendientes(7);
            }
            catch (Exception e)
            {
                JOptionPane.showMessageDialog(this,"No ha sido posible enviar los E-Mails diarios.","ERROR",JOptionPane.ERROR_MESSAGE);
            }
        }
        // *******************************************************************************************



        // *******************************************************************************************
        // *********************************** ELIMINACION DE DATOS **********************************
        // *******************************************************************************************
        private void eliminarDatosAntiguos(int Dias) throws DataException, SQLException, RemoteException
        {
            // Eliminamos el personal ocasional cuya fecha de caducidad haya expirado
            Ocasional ocas = new Ocasional();
            ocas.eliminarOcasional();
            ocas.UBD.cerrarConexion();

            // Eliminamos los departamentos que no hayan actualizado su personal segun lo estipulado
            Departamento dep = new Departamento();
            dep.eliminarDepartamentos();
            dep.UBD.cerrarConexion();

            // Eliminamos los datos personales de todas aquella personas que no consten en ninguna tabla especializada
            Personal pers = new Personal();
            pers.eliminarPersonal();
            pers.UBD.cerrarConexion();

            // Eliminamos los accesos, llaveros, llaves, niveles, colores y tarjetas antiguos (dados de baja, en el caso de los cinco ultimos)
            Acceso acc = new Acceso();
            acc.eliminarAccesos(Dias);
            acc.UBD.cerrarConexion();
            Llavero ll = new Llavero();
            ll.eliminarLlaveros(Dias);
            ll.UBD.cerrarConexion();
            Llave l = new Llave();
            l.eliminarLlaves(Dias);
            l.UBD.cerrarConexion();
            Nivel niv = new Nivel();
            niv.eliminarNiveles(Dias);
            niv.UBD.cerrarConexion();
            API.Tablas.Color col = new API.Tablas.Color();
            col.eliminarColores(Dias);
            col.UBD.cerrarConexion();
            Tarjeta tarj = new Tarjeta();
            tarj.eliminarTarjetas(Dias);
            tarj.UBD.cerrarConexion();
        }
        // *******************************************************************************************




	// MAIN
        /** main: Metodo principal ejecutable de la clase, encargado de desplegar el JFrame
	 *
	 * @param   String args[]
	 * @return  void
	 */
        public static void main(String args[])
        {
            java.awt.EventQueue.invokeLater(new Runnable() { public void run() { new Aplicacion(Usuario,Tipo_Usuario,DNI_Usuario).setVisible(true); } });
        }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup BGEntradaSalida;
    private javax.swing.JButton JBAnadirAcceso;
    private javax.swing.JButton JBAnadirAlmLl;
    private javax.swing.JButton JBAnadirEmail;
    private javax.swing.JButton JBAnadirLPrestEmpDep;
    private javax.swing.JButton JBAnadirLPrestOcas;
    private javax.swing.JButton JBAnadirLsLl;
    private javax.swing.JButton JBAnadirParte;
    private javax.swing.JButton JBAnadirPersona;
    private javax.swing.JButton JBAnadirPrestDep;
    private javax.swing.JButton JBAnadirPrestDep1;
    private javax.swing.JButton JBAnadirPrestamo;
    private javax.swing.JButton JBAnadirTfno;
    private javax.swing.JButton JBBajaColor;
    private javax.swing.JButton JBBajaNivel;
    private javax.swing.JButton JBBajaTarj;
    private javax.swing.JButton JBColor;
    private javax.swing.JButton JBDarBajaL;
    private javax.swing.JButton JBDarBajaLl;
    private javax.swing.JButton JBDarSalidaConDev;
    private javax.swing.JButton JBDescartar;
    private javax.swing.JButton JBDeseleccionar;
    private javax.swing.JButton JBDevolver;
    private javax.swing.JButton JBDevolverPrestamo;
    private javax.swing.JButton JBEliminarAcceso;
    private javax.swing.JButton JBEliminarAlm;
    private javax.swing.JButton JBEliminarAlmL;
    private javax.swing.JButton JBEliminarAlmLl;
    private javax.swing.JButton JBEliminarCargo;
    private javax.swing.JButton JBEliminarColor;
    private javax.swing.JButton JBEliminarCuenta;
    private javax.swing.JButton JBEliminarDepartamento;
    private javax.swing.JButton JBEliminarEmpresa;
    private javax.swing.JButton JBEliminarL;
    private javax.swing.JButton JBEliminarLPrestEmpDep;
    private javax.swing.JButton JBEliminarLPrestOcas;
    private javax.swing.JButton JBEliminarLl;
    private javax.swing.JButton JBEliminarLsLl;
    private javax.swing.JButton JBEliminarNivel;
    private javax.swing.JButton JBEliminarParte;
    private javax.swing.JButton JBEliminarPerContac;
    private javax.swing.JButton JBEliminarPrestDep;
    private javax.swing.JButton JBEliminarPrestDep1;
    private javax.swing.JButton JBEliminarPrestamo;
    private javax.swing.JButton JBEliminarTAcceso;
    private javax.swing.JButton JBEliminarTUsuario;
    private javax.swing.JButton JBEliminarTarj;
    private javax.swing.JButton JBEliminarTfno;
    private javax.swing.JButton JBEliminarZona;
    private javax.swing.JButton JBEnviar;
    private javax.swing.JButton JBGrabarAlm;
    private javax.swing.JButton JBGrabarAlmL;
    private javax.swing.JButton JBGrabarAlmLl;
    private javax.swing.JButton JBGrabarCargo;
    private javax.swing.JButton JBGrabarColor;
    private javax.swing.JButton JBGrabarCuenta;
    private javax.swing.JButton JBGrabarL;
    private javax.swing.JButton JBGrabarLl;
    private javax.swing.JButton JBGrabarNivel;
    private javax.swing.JButton JBGrabarNivel1;
    private javax.swing.JButton JBGrabarPerContac;
    private javax.swing.JButton JBGrabarTUsuario;
    private javax.swing.JButton JBGrabarTarjeta;
    private javax.swing.JButton JBGrabarZona;
    private javax.swing.JButton JBModificarEmpresa;
    private javax.swing.JButton JBModificarEmpresa1;
    private javax.swing.JButton JBModificarParte;
    private javax.swing.JButton JBMostrarTodo;
    private javax.swing.JButton JBPDFAccs;
    private javax.swing.JButton JBPDFAccsSist;
    private javax.swing.JButton JBPDFCuentas;
    private javax.swing.JButton JBPDFParte;
    private javax.swing.JButton JBPrestsPDF;
    private javax.swing.JButton JBPrestsPDF1;
    private javax.swing.JButton JBPrestsPDF2;
    private javax.swing.JButton JBRecordPrest;
    private javax.swing.JButton JBSalida;
    private javax.swing.JComboBox JCBActividad;
    private javax.swing.JComboBox JCBActividad1;
    private javax.swing.JCheckBox JCBAutTelTAccesos;
    private javax.swing.JComboBox JCBCargos;
    private javax.swing.JComboBox JCBCodVig;
    private javax.swing.JComboBox JCBDNICuenta;
    private javax.swing.JComboBox JCBDNIDep;
    private javax.swing.JComboBox JCBEmpresas;
    private javax.swing.JCheckBox JCBFechasPartes;
    private javax.swing.JComboBox JCBHFTAccesos;
    private javax.swing.JComboBox JCBHITAccesos;
    private javax.swing.JComboBox JCBHoraFin;
    private javax.swing.JComboBox JCBHoraFinAccsSist;
    private javax.swing.JComboBox JCBHoraFinParte;
    private javax.swing.JComboBox JCBHoraInic;
    private javax.swing.JComboBox JCBHoraInicAccsSist;
    private javax.swing.JComboBox JCBHoraInicParte;
    private javax.swing.JComboBox JCBLlaveros;
    private javax.swing.JComboBox JCBNivel;
    private javax.swing.JComboBox JCBNivelTAcceso;
    private javax.swing.JComboBox JCBNivelZona;
    private javax.swing.JComboBox JCBOcasional;
    private javax.swing.JCheckBox JCBPDFAlms1;
    private javax.swing.JCheckBox JCBPDFAlms2;
    private javax.swing.JCheckBox JCBPDFLls;
    private javax.swing.JCheckBox JCBPDFLls1;
    private javax.swing.JCheckBox JCBPDFLls2;
    private javax.swing.JCheckBox JCBPDFLs;
    private javax.swing.JCheckBox JCBPDFLs1;
    private javax.swing.JCheckBox JCBPDFLs2;
    private javax.swing.JCheckBox JCBPDFTarjs;
    private javax.swing.JCheckBox JCBPDFTodas;
    private javax.swing.JCheckBox JCBPDFTodas1;
    private javax.swing.JCheckBox JCBPDFTodas2;
    private javax.swing.JComboBox JCBParte;
    private javax.swing.JCheckBox JCBTarjeta;
    private javax.swing.JComboBox JCBTipoCuenta;
    private javax.swing.JCheckBox JCBTodoDia;
    private javax.swing.JCheckBox JCBTodoDiaAccsSist;
    private javax.swing.JCheckBox JCBTodoDiaParte;
    private org.freixas.jcalendar.JCalendarCombo JCCFCaducidad;
    private org.freixas.jcalendar.JCalendarCombo JCCFUltActu;
    private org.freixas.jcalendar.JCalendarCombo JCCFechaFin;
    private org.freixas.jcalendar.JCalendarCombo JCCFechaFinAccsSist;
    private org.freixas.jcalendar.JCalendarCombo JCCFechaFinParte;
    private org.freixas.jcalendar.JCalendarCombo JCCFechaInic;
    private org.freixas.jcalendar.JCalendarCombo JCCFechaInicAccsSist;
    private org.freixas.jcalendar.JCalendarCombo JCCFechaInicParte;
    private org.freixas.jcalendar.JCalendarCombo JCCFechaParte;
    private javax.swing.JButton JEliminarCargo;
    private javax.swing.JButton JEliminarEmail;
    private javax.swing.JLabel JLActiv;
    private javax.swing.JLabel JLActivTAcceso;
    private javax.swing.JLabel JLApellido1;
    private javax.swing.JLabel JLApellido1DP;
    private javax.swing.JLabel JLApellido2;
    private javax.swing.JLabel JLApellido2DP;
    private javax.swing.JLabel JLAsunto;
    private javax.swing.JLabel JLCIFEmp;
    private javax.swing.JLabel JLCodAcc;
    private javax.swing.JLabel JLCodAlm;
    private javax.swing.JLabel JLCodDep;
    private javax.swing.JLabel JLCodEmp;
    private javax.swing.JLabel JLCodL;
    private javax.swing.JLabel JLCodLl;
    private javax.swing.JLabel JLCodNiv;
    private javax.swing.JLabel JLCodNiv1;
    private javax.swing.JLabel JLCodTAcceso;
    private javax.swing.JLabel JLCodV;
    private javax.swing.JLabel JLCodVig;
    private javax.swing.JLabel JLCodVigParte;
    private javax.swing.JLabel JLCodZona;
    private javax.swing.JLabel JLDNI;
    private javax.swing.JLabel JLDNICuenta;
    private javax.swing.JLabel JLDNIDP;
    private javax.swing.JLabel JLDNIOcas;
    private javax.swing.JLabel JLDescripAlm;
    private javax.swing.JLabel JLDescripL;
    private javax.swing.JLabel JLDescripLl;
    private javax.swing.JLabel JLDescripNiv;
    private javax.swing.JLabel JLDescripNiv1;
    private javax.swing.JLabel JLDescripZona;
    private javax.swing.JLabel JLDescripcionTUsuario;
    private javax.swing.JLabel JLDiasTAcceso;
    private javax.swing.JLabel JLDirEmp;
    private javax.swing.JLabel JLEmpresaPrest;
    private javax.swing.JLabel JLEmpresas;
    private javax.swing.JLabel JLEnviado;
    private javax.swing.JLabel JLFCaducidad;
    private javax.swing.JLabel JLFUltActu;
    private javax.swing.JLabel JLFecha;
    private javax.swing.JLabel JLHorasTAcceso;
    private javax.swing.JLabel JLIncidencias;
    private javax.swing.JLabel JLMotivo;
    private javax.swing.JLabel JLNivTAcceso;
    private javax.swing.JLabel JLNivel;
    private javax.swing.JLabel JLNivelZona;
    private javax.swing.JLabel JLNomDep;
    private javax.swing.JLabel JLNomEmp;
    private javax.swing.JLabel JLNombre;
    private javax.swing.JLabel JLNombreDP;
    private javax.swing.JLabel JLNombrePrest;
    private javax.swing.JLabel JLNotas;
    private javax.swing.JLabel JLNotasL;
    private javax.swing.JLabel JLOtrosDatos;
    private javax.swing.JLabel JLPara;
    private javax.swing.JLabel JLPartesLogo;
    private javax.swing.JLabel JLPassCuenta;
    private javax.swing.JLabel JLPeriodicidad;
    private javax.swing.JLabel JLPrestTablas;
    private javax.swing.JLabel JLPrestTablas1;
    private javax.swing.JLabel JLPrestTablas2;
    private javax.swing.JLabel JLTfnoDP;
    private javax.swing.JLabel JLTipoCuenta;
    private javax.swing.JLabel JLTipoPrest;
    private javax.swing.JLabel JLTipoTUsuario;
    private javax.swing.JLabel JLUsuarioCuenta;
    private javax.swing.JMenu JMAccessManager;
    private javax.swing.JMenu JMIBD;
    private javax.swing.JMenuItem JMICambiarDatos;
    private javax.swing.JMenuItem JMIExportar;
    private javax.swing.JMenuItem JMIImportar;
    private javax.swing.JMenuItem JMISalir;
    private javax.swing.JPanel JPAccesos;
    private javax.swing.JPanel JPAccesosB1;
    private javax.swing.JPanel JPAccesosB2;
    private javax.swing.JPanel JPAccsSistB;
    private javax.swing.JPanel JPAlmD;
    private javax.swing.JPanel JPAlmacenes;
    private javax.swing.JPanel JPAlmsB;
    private javax.swing.JPanel JPAvisoTfno;
    private javax.swing.JPanel JPBandejas;
    private javax.swing.JPanel JPCargoD;
    private javax.swing.JPanel JPCargos;
    private javax.swing.JPanel JPCargoss;
    private javax.swing.JPanel JPColorD;
    private javax.swing.JPanel JPColorTarjeta;
    private javax.swing.JPanel JPColores;
    private javax.swing.JPanel JPContactos;
    private javax.swing.JPanel JPContactosNotas;
    private javax.swing.JPanel JPContenidoNota;
    private javax.swing.JPanel JPCuentas;
    private javax.swing.JPanel JPCuentasB;
    private javax.swing.JPanel JPDNIAuto;
    private javax.swing.JPanel JPDP;
    private javax.swing.JPanel JPDPPrest;
    private javax.swing.JPanel JPDatosCuenta;
    private javax.swing.JPanel JPDatosPersonales;
    private javax.swing.JPanel JPDatosPrest;
    private javax.swing.JPanel JPDatosTUsuario;
    private javax.swing.JPanel JPDepartamentos;
    private javax.swing.JPanel JPDepartamentosB;
    private javax.swing.JPanel JPDepartamentosD;
    private javax.swing.JPanel JPElimC;
    private javax.swing.JPanel JPElimTE;
    private javax.swing.JPanel JPEmpleados;
    private javax.swing.JPanel JPEmpresas;
    private javax.swing.JPanel JPEmpresasB;
    private javax.swing.JPanel JPEmpresasD;
    private javax.swing.JPanel JPEmpresasPersonal;
    private javax.swing.JPanel JPEntrada;
    private javax.swing.JPanel JPHistorialAccesosSist;
    private javax.swing.JPanel JPInfoCuentas;
    private javax.swing.JPanel JPJCBDNIDep;
    private javax.swing.JPanel JPJTFNumCopsL;
    private javax.swing.JPanel JPJTFNumCopsLl;
    private javax.swing.JPanel JPLlaveB;
    private javax.swing.JPanel JPLlaveD;
    private javax.swing.JPanel JPLlaveroD;
    private javax.swing.JPanel JPLls;
    private javax.swing.JPanel JPLlsLLls;
    private javax.swing.JPanel JPLsLlsPrest;
    private javax.swing.JPanel JPLsPrestB;
    private javax.swing.JPanel JPNivelD;
    private javax.swing.JPanel JPNiveles;
    private javax.swing.JPanel JPNivelesB;
    private javax.swing.JPanel JPNotas;
    private javax.swing.JPanel JPOcasional;
    private javax.swing.JPanel JPPDFB;
    private javax.swing.JPanel JPParte;
    private javax.swing.JPanel JPPartes;
    private javax.swing.JPanel JPPartesB;
    private javax.swing.JPanel JPPartesBuscarB;
    private javax.swing.JPanel JPPersonal;
    private javax.swing.JPanel JPPersonalB;
    private javax.swing.JPanel JPPrestamos;
    private javax.swing.JPanel JPPrestsPDF;
    private javax.swing.JPanel JPPrestsPDF1;
    private javax.swing.JPanel JPPrestsPDF2;
    private javax.swing.JPanel JPRedactar;
    private javax.swing.JPanel JPResidente;
    private javax.swing.JPanel JPSalida;
    private javax.swing.JPanel JPTACs;
    private javax.swing.JPanel JPTAccesoB;
    private javax.swing.JPanel JPTAccesoD;
    private javax.swing.JPanel JPTAccesos;
    private javax.swing.JPanel JPTUsuarios1;
    private javax.swing.JPanel JPTUsuariosB;
    private javax.swing.JPanel JPTarjeta;
    private javax.swing.JPanel JPTarjetaD;
    private javax.swing.JPanel JPTarjetas;
    private javax.swing.JPanel JPTarjetasB;
    private javax.swing.JPanel JPTarjetasD;
    private javax.swing.JPanel JPTarjsPrest;
    private javax.swing.JPanel JPVigilante;
    private javax.swing.JPanel JPZona;
    private javax.swing.JPanel JPZonaB;
    private javax.swing.JPanel JPZonaD;
    private javax.swing.JPanel JPZonas;
    private javax.swing.JPanel JPZsLsLls;
    private javax.swing.JRadioButton JRBEntrada;
    private javax.swing.JRadioButton JRSalida;
    private javax.swing.JScrollPane JSPAccesos;
    private javax.swing.JScrollPane JSPDepartamentos;
    private javax.swing.JScrollPane JSPEmpleados;
    private javax.swing.JScrollPane JSPEmpresas;
    private javax.swing.JScrollPane JSPLLs;
    private javax.swing.JScrollPane JSPMotivo;
    private javax.swing.JScrollPane JSPOtrosDatos;
    private javax.swing.JTextArea JTADirEmp;
    private javax.swing.JTextArea JTAIncidencias;
    private javax.swing.JTextArea JTAMotivo;
    private javax.swing.JTextArea JTANotas;
    private javax.swing.JTextArea JTAOtrosDatos;
    private javax.swing.JTextArea JTAPara;
    private javax.swing.JTextArea JTATexto;
    private javax.swing.JTextArea JTATextoNota;
    private javax.swing.JTable JTAccesos;
    private javax.swing.JTable JTAlms;
    private javax.swing.JToggleButton JTBActivarPanelPDF;
    private javax.swing.JToggleButton JTBActivarPanelPDF1;
    private javax.swing.JToggleButton JTBActivarPanelPDF2;
    private javax.swing.JToggleButton JTBAnadirAlm;
    private javax.swing.JToggleButton JTBAnadirAlmL;
    private javax.swing.JToggleButton JTBAnadirAlmLlB;
    private javax.swing.JToggleButton JTBAnadirCargo;
    private javax.swing.JToggleButton JTBAnadirColor;
    private javax.swing.JToggleButton JTBAnadirCuenta;
    private javax.swing.JToggleButton JTBAnadirDepartamento;
    private javax.swing.JToggleButton JTBAnadirEmpresa;
    private javax.swing.JToggleButton JTBAnadirL;
    private javax.swing.JToggleButton JTBAnadirLl;
    private javax.swing.JToggleButton JTBAnadirNivel;
    private javax.swing.JToggleButton JTBAnadirPerContac;
    private javax.swing.JToggleButton JTBAnadirTAcceso;
    private javax.swing.JToggleButton JTBAnadirTUsuario;
    private javax.swing.JToggleButton JTBAnadirTarjeta;
    private javax.swing.JToggleButton JTBAnadirZona;
    private javax.swing.JToggleButton JTBModificarAlm;
    private javax.swing.JToggleButton JTBModificarCargo;
    private javax.swing.JToggleButton JTBModificarColor;
    private javax.swing.JToggleButton JTBModificarCuenta;
    private javax.swing.JToggleButton JTBModificarDepartamento;
    private javax.swing.JToggleButton JTBModificarEmpresa;
    private javax.swing.JToggleButton JTBModificarLl;
    private javax.swing.JToggleButton JTBModificarLlave;
    private javax.swing.JToggleButton JTBModificarNivel;
    private javax.swing.JToggleButton JTBModificarPerContac;
    private javax.swing.JToggleButton JTBModificarTAcceso;
    private javax.swing.JToggleButton JTBModificarTUsuario;
    private javax.swing.JToggleButton JTBModificarTarj;
    private javax.swing.JToggleButton JTBModificarZona;
    private javax.swing.JToggleButton JTBNumCopsL;
    private javax.swing.JToggleButton JTBNumCopsLl;
    private javax.swing.JTable JTCargos;
    private javax.swing.JTable JTCargoss;
    private javax.swing.JTable JTColores;
    private javax.swing.JTable JTContactos;
    private javax.swing.JTable JTContactosNotas;
    private javax.swing.JTable JTCuentas;
    private javax.swing.JTable JTDepartamentos;
    private javax.swing.JTable JTEmails;
    private javax.swing.JTable JTEmpleados;
    private javax.swing.JTable JTEmpresas;
    private javax.swing.JTable JTEntrada;
    private javax.swing.JTextField JTFActivTAcceso;
    private javax.swing.JTextField JTFActividad;
    private javax.swing.JTextField JTFApellido1;
    private javax.swing.JTextField JTFApellido1DP;
    private javax.swing.JTextField JTFApellido2;
    private javax.swing.JTextField JTFApellido2DP;
    private javax.swing.JTextField JTFAsunto;
    private javax.swing.JTextField JTFAsuntoNota;
    private javax.swing.JTextField JTFAvisoTfno;
    private javax.swing.JTextField JTFBuscar;
    private javax.swing.JTextField JTFBuscarAccesosSist;
    private javax.swing.JTextField JTFBuscarContac;
    private javax.swing.JTextField JTFBuscarCuenta;
    private javax.swing.JTextField JTFBuscarLLl;
    private javax.swing.JTextField JTFCIF;
    private javax.swing.JTextField JTFCIFEmp;
    private javax.swing.JTextField JTFCodAlm;
    private javax.swing.JTextField JTFCodCargo;
    private javax.swing.JTextField JTFCodDep;
    private javax.swing.JTextField JTFCodEmp;
    private javax.swing.JTextField JTFCodL;
    private javax.swing.JTextField JTFCodLl;
    private javax.swing.JTextField JTFCodNiv;
    private javax.swing.JTextField JTFCodTAcceso;
    private javax.swing.JTextField JTFCodTarj;
    private javax.swing.JTextField JTFCodTarjD;
    private javax.swing.JTextField JTFCodV;
    private javax.swing.JTextField JTFCodVig;
    private javax.swing.JTextField JTFCodZona;
    private javax.swing.JTextField JTFColorNiv;
    private javax.swing.JTextField JTFDNI;
    private javax.swing.JTextField JTFDNIDP;
    private javax.swing.JTextField JTFDescripAlm;
    private javax.swing.JTextField JTFDescripCargo;
    private javax.swing.JTextField JTFDescripL;
    private javax.swing.JTextField JTFDescripLl;
    private javax.swing.JTextField JTFDescripNiv;
    private javax.swing.JTextField JTFDescripTUsuario;
    private javax.swing.JTextField JTFDescripZona;
    private javax.swing.JTextField JTFDiasTAcceso;
    private javax.swing.JTextField JTFEmpresaPrest;
    private javax.swing.JTextField JTFEnviado;
    private javax.swing.JTextField JTFFiltro;
    private javax.swing.JTextField JTFNomDep;
    private javax.swing.JTextField JTFNomEmp;
    private javax.swing.JTextField JTFNombre;
    private javax.swing.JTextField JTFNombreDP;
    private javax.swing.JTextField JTFNombreDep;
    private javax.swing.JTextField JTFNombreEmp;
    private javax.swing.JTextField JTFNombrePrest;
    private javax.swing.JTextField JTFNotasL;
    private javax.swing.JTextField JTFNumCopsL;
    private javax.swing.JTextField JTFNumCopsLl;
    private javax.swing.JTextField JTFPassCuenta;
    private javax.swing.JTextField JTFPeriodicidad;
    private javax.swing.JTextField JTFRedactorNota;
    private javax.swing.JTextField JTFTUsuario;
    private javax.swing.JTextField JTFTfnoEmailDP;
    private javax.swing.JTextField JTFTipoPrest;
    private javax.swing.JTextField JTFUsuarioCuenta;
    private javax.swing.JTable JTHistorialAccesosSist;
    private javax.swing.JTable JTLls;
    private javax.swing.JTable JTLlsPrest;
    private javax.swing.JTable JTLs;
    private javax.swing.JTable JTLsLls;
    private javax.swing.JTable JTLsLlsPers;
    private javax.swing.JTable JTLsLlsPers1;
    private javax.swing.JTable JTLsPrest;
    private javax.swing.JTable JTLsPrestEmpDep;
    private javax.swing.JTable JTLsPrestEmpDepResto;
    private javax.swing.JTable JTLsPrestOcas;
    private javax.swing.JTable JTLsPrestOcasResto;
    private javax.swing.JTable JTLsZona;
    private javax.swing.JTable JTNiveles;
    private javax.swing.JTabbedPane JTPDepartamentos;
    private javax.swing.JTabbedPane JTPES;
    private javax.swing.JTabbedPane JTPFuncionalidades;
    private javax.swing.JTabbedPane JTPNotas;
    private javax.swing.JTable JTPartes;
    private javax.swing.JTable JTSalida;
    private javax.swing.JTable JTTAccesos;
    private javax.swing.JTable JTTUsuarios;
    private javax.swing.JTable JTTarjetas;
    private javax.swing.JTable JTTarjsPrest;
    private javax.swing.JTable JTTelefonos;
    private javax.swing.JTable JTTfns;
    private javax.swing.JTable JTTfnsPrest;
    private javax.swing.JTable JTZonas;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane10;
    private javax.swing.JScrollPane jScrollPane11;
    private javax.swing.JScrollPane jScrollPane12;
    private javax.swing.JScrollPane jScrollPane13;
    private javax.swing.JScrollPane jScrollPane14;
    private javax.swing.JScrollPane jScrollPane15;
    private javax.swing.JScrollPane jScrollPane16;
    private javax.swing.JScrollPane jScrollPane17;
    private javax.swing.JScrollPane jScrollPane18;
    private javax.swing.JScrollPane jScrollPane19;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane20;
    private javax.swing.JScrollPane jScrollPane21;
    private javax.swing.JScrollPane jScrollPane22;
    private javax.swing.JScrollPane jScrollPane23;
    private javax.swing.JScrollPane jScrollPane24;
    private javax.swing.JScrollPane jScrollPane25;
    private javax.swing.JScrollPane jScrollPane26;
    private javax.swing.JScrollPane jScrollPane27;
    private javax.swing.JScrollPane jScrollPane28;
    private javax.swing.JScrollPane jScrollPane29;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane30;
    private javax.swing.JScrollPane jScrollPane31;
    private javax.swing.JScrollPane jScrollPane32;
    private javax.swing.JScrollPane jScrollPane33;
    private javax.swing.JScrollPane jScrollPane34;
    private javax.swing.JScrollPane jScrollPane35;
    private javax.swing.JScrollPane jScrollPane36;
    private javax.swing.JScrollPane jScrollPane37;
    private javax.swing.JScrollPane jScrollPane38;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JScrollPane jScrollPane6;
    private javax.swing.JScrollPane jScrollPane7;
    private javax.swing.JScrollPane jScrollPane8;
    private javax.swing.JScrollPane jScrollPane9;
    private javax.swing.JSeparator jSeparator1;
    // End of variables declaration//GEN-END:variables
}
